@@ -1,7 +1,7 @@
 /*                         G 2 A S C . C
  * BRL-CAD
  *
- * Copyright (c) 1985-2013 United States Government as represented by
+ * Copyright (c) 1985-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This program is free software; you can redistribute it and/or
@@ -30,14 +30,18 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#include "bnetwork.h"
 #include "bio.h"
-#include "bin.h"
 
+#include "bu/app.h"
+#include "bu/debug.h"
+#include "bu/opt.h"
+#include "bu/units.h"
 #include "vmath.h"
-#include "db.h"
+#include "rt/db4.h"
 #include "raytrace.h"
 #include "wdb.h"
-#include "rtgeom.h"
+#include "rt/geom.h"
 #include "tcl.h"
 
 const mat_t id_mat = MAT_INIT_IDN; /* identity matrix for pipes */
@@ -89,7 +93,7 @@ tclify_name(const char *name)
 
     if (max_len > tclified_name_buffer_len) {
 	tclified_name_buffer_len = max_len;
-	tclified_name = bu_realloc(tclified_name, tclified_name_buffer_len, "tclified_name buffer");
+	tclified_name = (char *)bu_realloc(tclified_name, tclified_name_buffer_len, "tclified_name buffer");
     }
 
     dest = tclified_name;
@@ -109,16 +113,39 @@ int
 main(int argc, char **argv)
 {
     unsigned i;
+    int need_help = 0;
 
-    if (argc != 3) {
+    bu_setprogname(argv[0]);
+    Tcl_FindExecutable(argv[0]);
+
+    struct bu_opt_desc d[3];
+    BU_OPT(d[0], "h", "help",        "",         NULL,        &need_help, "Print help   and exit");
+    BU_OPT(d[1], "?", "",            "",         NULL,        &need_help, "");
+    BU_OPT_NULL(d[2]);
+
+    /* Skip first arg */
+    argv++; argc--;
+    struct bu_vls optparse_msg = BU_VLS_INIT_ZERO;
+    int uac = bu_opt_parse(&optparse_msg, argc, (const char **)argv, d);
+
+    if (uac == -1) {
+	bu_exit(EXIT_FAILURE, "%s", bu_vls_addr(&optparse_msg));
+    }
+    bu_vls_free(&optparse_msg);
+
+    argc = uac;
+
+    if (need_help) {
+	bu_exit(EXIT_SUCCESS, "%s", usage);
+    }
+
+    if (argc != 2) {
 	bu_exit(1, "%s", usage);
     }
 
-#if defined(_WIN32) && !defined(__CYGWIN__)
     setmode(fileno(stdin), O_BINARY);
     setmode(fileno(stdout), O_BINARY);
     setmode(fileno(stderr), O_BINARY);
-#endif
 
     iname = "-";
     ifp = stdin;
@@ -126,32 +153,37 @@ main(int argc, char **argv)
 
     bu_debug = BU_DEBUG_COREDUMP;
 
-    if (argc >= 3) {
-	iname = argv[1];
+    if (argc >= 2) {
+
+	iname = argv[0];
+
 	if (BU_STR_EQUAL(iname, "-")) {
 	    ifp = stdin;
 	} else {
 	    ifp = fopen(iname, "rb");
 	}
-	if (!ifp)  perror(iname);
-	if (BU_STR_EQUAL(argv[2], "-")) {
+
+	if (!ifp)
+	    perror(iname);
+
+	if (BU_STR_EQUAL(argv[1], "-")) {
 	    ofp = stdout;
 	} else {
-	    ofp = fopen(argv[2], "wb");
+	    ofp = fopen(argv[1], "wb");
 	}
-	if (!ofp)  perror(argv[2]);
+
+	if (!ofp)
+	    perror(argv[1]);
+
 	if (ifp == NULL || ofp == NULL) {
 	    bu_exit(1, "g2asc: can't open files.");
 	}
     }
+
     if (isatty(fileno(ifp))) {
 	bu_exit(1, "%s", usage);
     }
 
-    Tcl_FindExecutable(argv[0]);
-
-    rt_init_resource(&rt_uniresource, 0, NULL);
-
     /* First, determine what version database this is */
     if (fread((char *)&record, sizeof record, 1, ifp) != 1) {
 	bu_exit(2, "g2asc(%s) ERROR, file too short to be BRL-CAD database\n",
@@ -165,12 +197,12 @@ main(int argc, char **argv)
 	const char *u;
 
 	if (ifp == stdin || ofp == stdout) {
-	    bu_log("Cannot use stdin or stdout for Release 6 or later databases\n");
+	    bu_log("Unsupported: cannot use stdin or stdout for v5 or later geometry databases\n");
 	    bu_exit(1, "Please use the \"g2asc input.g output.g\" form\n");
 	}
 
-	bu_log("Exporting Release 6 database\n");
-	bu_log("  Note that the Release 6 binary format is machine independent.\n");
+	bu_log("Exporting v5 format geometry database\n");
+	bu_log("  Note that the v5 binary format is machine independent.\n");
 	bu_log("  Converting to ASCII to move database to a different\n");
 	bu_log("  computer architecture is no longer necessary.\n");
 	interp = Tcl_CreateInterp();
@@ -326,15 +358,15 @@ main(int argc, char **argv)
 	    Tcl_ResetResult(interp);
 	    rt_db_free_internal(&intern);
 	} FOR_ALL_DIRECTORY_END;
+
+	/* processing a v5, we're done */
 	return 0;
-    } else {
-	/* A record is already in the input buffer */
-	goto top;
     }
 
-    /* Read database file */
- top:
+    /* processing a v4 */
+top:
     do {
+	/* A v4 record is already in the input buffer */
 	/* Check record type and skip deleted records */
 	switch (record.u_id) {
 	    case ID_FREE:
@@ -343,7 +375,8 @@ main(int argc, char **argv)
 		soldump();
 		continue;
 	    case ID_COMB:
-		if (combdump() > 0)  goto top;
+		if (combdump() > 0)
+		    goto top;
 		continue;
 	    case ID_MEMB:
 		fprintf(stderr, "g2asc: stray MEMB record, skipped\n");
@@ -405,13 +438,12 @@ main(int argc, char **argv)
     }  while (fread((char *)&record, sizeof record, 1, ifp) == 1  &&
 	       !feof(ifp));
 
+    /* done with v4 */
     return 0;
 }
 
 
 /*
- *			N A M E
- *
  *  Take a database name and null-terminate it,
  *  converting unprintable characters to something printable.
  *  Here we deal with names not being null-terminated.
@@ -425,7 +457,7 @@ char *encode_name(char *str)
 
     while (op < &buf[NAMESIZE]) {
 	if (*ip == '\0')  break;
-	if (isascii((int)*ip) && isprint((int)*ip) && !isspace((int)*ip)) {
+	if (isprint((int)*ip) && !isspace((int)*ip)) {
 	    *op++ = *ip++;
 	}  else  {
 	    *op++ = '@';
@@ -450,8 +482,6 @@ char *encode_name(char *str)
 
 
 /*
- *			G E T _ E X T
- *
  *  Take "ngran" granules, and put them in memory.
  *  The first granule comes from the global extern "record",
  *  the remainder are read from ifp.
@@ -464,7 +494,7 @@ get_ext(struct bu_external *ep, size_t ngran)
     BU_EXTERNAL_INIT(ep);
 
     ep->ext_nbytes = ngran * sizeof(union record);
-    ep->ext_buf = (genptr_t)bu_malloc(ep->ext_nbytes, "get_ext ext_buf");
+    ep->ext_buf = (uint8_t *)bu_malloc(ep->ext_nbytes, "get_ext ext_buf");
 
     /* Copy the freebie (first) record into the array of records.  */
     memcpy((char *)ep->ext_buf, (char *)&record, sizeof(union record));
@@ -752,13 +782,13 @@ void
 dump_pipe_segs(char *name, struct bu_list *headp)
 {
 
-    struct wdb_pipept *sp;
+    struct wdb_pipe_pnt *sp;
 
     fprintf(ofp, "%c %.16s\n", DBID_PIPE, name);
 
     /* print parameters for each point: one point per line */
 
-    for (BU_LIST_FOR(sp, wdb_pipept, headp)) {
+    for (BU_LIST_FOR(sp, wdb_pipe_pnt, headp)) {
 	fprintf(ofp,  "%26.20e %26.20e %26.20e %26.20e %26.20e %26.20e\n",
 		sp->pp_id, sp->pp_od, sp->pp_bendradius, V3ARGS(sp->pp_coord));
     }
@@ -816,8 +846,7 @@ particle_dump(void)
 }
 
 
-/*			A R B N _ D U M P
- *
+/*
  *  Print out arbn information.
  *
  */
@@ -859,8 +888,6 @@ arbn_dump(void)
 
 
 /*
- *			C O M B D U M P
- *
  *  Note that for compatibility with programs such as FRED that
  *  (inappropriately) read .asc files, the member count has to be
  *  recalculated here.
@@ -938,7 +965,7 @@ combdump(void)	/* Print out Combination record information */
 		  record.c.c_rgb[1],
 		  record.c.c_rgb[2]);
     m1 = m2 = 0;
-    if (isascii((int)record.c.c_matname[0]) && isprint((int)record.c.c_matname[0])) {
+    if (isprint((int)record.c.c_matname[0])) {
 	m1 = 1;
 	if (record.c.c_matparm[0])
 	    m2 = 1;
@@ -978,8 +1005,6 @@ combdump(void)	/* Print out Combination record information */
 }
 
 /*
- *			M E M B D U M P
- *
  *  Print out Member record information.
  *  Intended to be called by combdump only.
  */
@@ -1066,7 +1091,6 @@ bspldump(void)	/* Print out B-spline solid description record information */
 		  record.B.B_id,		/* b */
 		  encode_name(record.B.B_name),	/* unique name */
 		  record.B.B_nsurf);	/* # of surfaces in this solid */
-		/*record.B.B_unused);	UNUSED (was resolution of flatness) */
 }
 
 void
@@ -1141,8 +1165,6 @@ bsurfdump(void)	/* Print d-spline surface description record information */
 }
 
 /*
- *			S T R C H O P
- *
  *  Take a string and a length, and null terminate,
  *  converting unprintable characters to something printable.
  */
@@ -1154,13 +1176,12 @@ char *strchop(char *str, size_t len)
     int warn = 0;
     char *ep;
 
-    if (len > sizeof(buf)-2)
-	len=sizeof(buf)-2;
+    CLAMP(len, 1, sizeof(buf)-2);
 
     ep = &buf[len-1];		/* Leave room for null */
     while (op < ep) {
 	if (*ip == '\0')  break;
-	if ((int)isascii((int)*ip) && ((int)isprint((int)*ip) || isspace((int)*ip))) {
+	if ((int)isprint((int)*ip) || isspace((int)*ip)) {
 	    *op++ = *ip++;
 	}  else  {
 	    *op++ = '@';
