@@ -1,7 +1,7 @@
 /*                       F A S T 4 - G . C
  * BRL-CAD
  *
- * Copyright (c) 1994-2013 United States Government as represented by
+ * Copyright (c) 1994-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This program is free software; you can redistribute it and/or
@@ -28,7 +28,6 @@
 
 /* system headers */
 #include <stdlib.h>
-#include <stdio.h>
 #include <math.h>
 #include <string.h>
 #include <ctype.h>
@@ -36,88 +35,98 @@
 #include "bio.h"
 
 /* interface headers */
-#include "db.h"
+#include "bu/app.h"
+#include "bu/debug.h"
+#include "bu/getopt.h"
+#include "bu/path.h"
+#include "rt/db4.h"
 #include "vmath.h"
 #include "nmg.h"
-#include "rtgeom.h"
+#include "rt/geom.h"
 #include "raytrace.h"
 #include "wdb.h"
-#include "plot3.h"
+#include "bn/plot3.h"
+
+
+/* NOTE: there should be no space after comma */
+#define COMMA ','
 
 
 /* convenient macro for building regions */
-#define	MK_REGION(fp, headp, name, r_id, rgb) {\
-    if (mode == 1) {\
-	if (!quiet)\
-	    bu_log("Making region: %s (PLATE)\n", name); \
+#define MK_REGION(fp, headp, name, r_id, rgb) {\
+	if (mode == 1) {\
+	    if (!quiet)\
+		bu_log("Making region: %s (PLATE)\n", name); \
 	    mk_comb(fp, name, &((headp)->l), 'P', (char *)NULL, (char *)NULL, rgb, r_id, 0, 1, 100, 0, 0, 0); \
-    } else if (mode == 2) {\
-	if (!quiet) \
-	    bu_log("Making region: %s (VOLUME)\n", name); \
+	} else if (mode == 2) {\
+	    if (!quiet) \
+		bu_log("Making region: %s (VOLUME)\n", name); \
 	    mk_comb(fp, name, &((headp)->l), 'V', (char *)NULL, (char *)NULL, rgb, r_id, 0, 1, 100, 0, 0, 0); \
-    } else {\
-	bu_log("Illegal mode (%d), while trying to make region (%s)\n", mode, name);\
-	bu_log("\tRegion not made!\n");\
-    }\
-}
+	} else {\
+	    bu_log("Illegal mode (%d), while trying to make region (%s)\n", mode, name);\
+	    bu_log("\tRegion not made!\n");\
+	}\
+    }
 
-#define	PUSH(ptr)	bu_ptbl_ins(&stack, (long *)ptr)
+
+#define PUSH(ptr) bu_ptbl_ins(&stack, (long *)ptr)
 #define POP(structure, ptr) { \
-    if (BU_PTBL_END(&stack) == 0) \
-	ptr = (struct structure *)NULL; \
-    else { \
-	ptr = (struct structure *)BU_PTBL_GET(&stack, BU_PTBL_END(&stack)-1); \
-	bu_ptbl_rm(&stack, (long *)ptr); \
-    } \
-}
-#define	PUSH2(ptr)	bu_ptbl_ins(&stack2, (long *)ptr)
-#define POP2(structure, ptr)	{ \
-    if (BU_PTBL_END(&stack2) == 0) \
-	ptr = (struct structure *)NULL; \
-    else { \
-	ptr = (struct structure *)BU_PTBL_GET(&stack2, BU_PTBL_END(&stack2)-1); \
-	bu_ptbl_rm(&stack2, (long *)ptr); \
-    } \
-}
+	if (BU_PTBL_LEN(&stack) == 0) \
+	    ptr = (struct structure *)NULL; \
+	else { \
+	    ptr = (struct structure *)BU_PTBL_GET(&stack, BU_PTBL_LEN(&stack)-1); \
+	    bu_ptbl_rm(&stack, (long *)ptr); \
+	} \
+    }
+#define PUSH2(ptr) bu_ptbl_ins(&stack2, (long *)ptr)
+#define POP2(structure, ptr) { \
+	if (BU_PTBL_LEN(&stack2) == 0) \
+	    ptr = (struct structure *)NULL; \
+	else { \
+	    ptr = (struct structure *)BU_PTBL_GET(&stack2, BU_PTBL_LEN(&stack2)-1); \
+	    bu_ptbl_rm(&stack2, (long *)ptr); \
+	} \
+    }
 
-#define	NAME_TREE_MAGIC	0x55555555
+
+#define NAME_TREE_MAGIC 0x55555555
 #define CK_TREE_MAGIC(ptr) {\
-    if (!ptr)\
-	bu_log("ERROR: Null name_tree pointer, file=%s, line=%d\n", __FILE__, __LINE__);\
-    else if (ptr->magic != NAME_TREE_MAGIC)\
-	bu_log("ERROR: bad name_tree pointer (%p), file=%s, line=%d\n", (void *)ptr, __FILE__, __LINE__);\
-}
+	if (!ptr)\
+	    bu_log("ERROR: Null name_tree pointer, file=%s, line=%d\n", __FILE__, __LINE__);\
+	else if (ptr->magic != NAME_TREE_MAGIC)\
+	    bu_log("ERROR: bad name_tree pointer (%p), file=%s, line=%d\n", (void *)ptr, __FILE__, __LINE__);\
+    }
 
 
-#define	PLATE_MODE	1
-#define	VOLUME_MODE	2
+#define PLATE_MODE	1
+#define VOLUME_MODE	2
 
-#define	POS_CENTER	1	/* face positions for facets */
-#define	POS_FRONT	2
+#define POS_CENTER	1	/* face positions for facets */
+#define POS_FRONT	2
 
-#define	END_OPEN	1	/* End closure codes for cones */
-#define	END_CLOSED	2
+#define END_OPEN	1	/* End closure codes for cones */
+#define END_CLOSED	2
 
-#define	GRID_BLOCK	256	/* allocate space for grid points in blocks of 256 points */
+#define GRID_BLOCK	256	/* allocate space for grid points in blocks of 256 points */
 
-#define	CLINE		'l'
-#define	CHEX1		'p'
-#define	CHEX2		'b'
-#define	CTRI		't'
-#define	CQUAD		'q'
-#define	CCONE1		'c'
-#define	CCONE2		'd'
-#define	CCONE3		'e'
-#define	CSPHERE		's'
-#define	NMG		'n'
-#define	BOT		't'
-#define	COMPSPLT	'h'
+#define CLINE		'l'
+#define CHEX1		'p'
+#define CHEX2		'b'
+#define CTRI		't'
+#define CQUAD		'q'
+#define CCONE1		'c'
+#define CCONE2		'd'
+#define CCONE3		'e'
+#define CSPHERE		's'
+#define NMG		'n'
+#define BOT		't'
+#define COMPSPLT	'h'
 
 #define HOLE 1
 #define WALL 2
 #define INT_LIST_BLOCK		256	/* Number of int_list array slots to allocate */
-#define	MAX_LINE_SIZE			128	/* Length of char array for input line */
-#define	REGION_LIST_BLOCK	256	/* initial length of array of region ids to process */
+#define MAX_LINE_SIZE			128	/* Length of char array for input line */
+#define REGION_LIST_BLOCK	256	/* initial length of array of region ids to process */
 
 
 struct fast4_color {
@@ -127,6 +136,7 @@ struct fast4_color {
     unsigned char rgb[3];
 };
 
+
 struct cline {
     int pt1, pt2;
     int element_id;
@@ -159,6 +169,7 @@ struct hole_list {
     struct hole_list *next;
 };
 
+
 struct holes {
     int group;
     int component;
@@ -183,50 +194,48 @@ static int hex_faces[12][3]={
     { 0, 2, 3 }  /* 12 */
 };
 
+
 static struct fast4_color HeadColor;
 
-static char	line[MAX_LINE_SIZE+1];		/* Space for input line */
-static FILE	*fpin;			/* Input FASTGEN4 file pointer */
+static char line[MAX_LINE_SIZE+1];		/* Space for input line */
+static FILE *fpin;			/* Input FASTGEN4 file pointer */
 static struct rt_wdb *fpout;		/* Output BRL-CAD file pointer */
-static FILE	*fp_plot=NULL;		/* file for plot output */
-static FILE	*fp_muves=NULL;		/* file for MUVES data, output CHGCOMP and CBACKING data */
-static int	grid_size;		/* Number of points that will fit in current grid_pts array */
-static int	max_grid_no=0;		/* Maximum grid number used */
-static int	mode=0;			/* Plate mode (1) or volume mode (2), of current component */
-static int	group_id=(-1);		/* Group identification number from SECTION card */
-static int	comp_id=(-1);		/* Component identification number from SECTION card */
-static int	region_id=0;		/* Region id number (group id no X 1000 + component id no) */
-static int	region_id_max=0;
-static char	field[9];		/* Space for storing one field from an input line */
-static char	vehicle[17];		/* Title for BRL-CAD model from VEHICLE card */
-static int	name_count;		/* Count of number of times this name_name has been used */
-static int	pass;			/* Pass number (0 -> only make names, 1-> do geometry) */
-static int	bot=0;			/* Flag: >0 -> There are BOT's in current component */
-static int	warnings=0;		/* Flag: >0 -> Print warning messages */
-static int	debug=0;		/* Debug flag */
-static int	rt_debug=0;		/* RT_G_DEBUG */
-static int	quiet=0;		/* flag to not blather */
-static int	comp_count=0;		/* Count of components in FASTGEN4 file */
-static int	f4_do_skips=0;		/* flag indicating that not all components will be processed */
-static int	*region_list;		/* array of region_ids to be processed */
-static int	region_list_len=0;	/* actual length of the malloc'd region_list array */
-static int	f4_do_plot=0;		/* flag indicating plot file should be created */
+static FILE *fp_plot=NULL;		/* file for plot output */
+static FILE *fp_muves=NULL;		/* file for MUVES data, output CHGCOMP and CBACKING data */
+static int grid_size;		/* Number of points that will fit in current grid_pts array */
+static int max_grid_no=0;		/* Maximum grid number used */
+static int mode=0;			/* Plate mode (1) or volume mode (2), of current component */
+static int group_id=(-1);		/* Group identification number from SECTION card */
+static int comp_id=(-1);		/* Component identification number from SECTION card */
+static int region_id=0;		/* Region id number (group id no X 1000 + component id no) */
+static int region_id_max=0;
+static char field[9];		/* Space for storing one field from an input line */
+static char vehicle[17];		/* Title for BRL-CAD model from VEHICLE card */
+static int name_count;		/* Count of number of times this name_name has been used */
+static int pass;			/* Pass number (0 -> only make names, 1-> do geometry) */
+static int bot=0;			/* Flag: >0 -> There are BOT's in current component */
+static int warnings=0;		/* Flag: >0 -> Print warning messages */
+static int debug=0;		/* Debug flag */
+static int frt_debug=0;		/* RT_G_DEBUG */
+static int quiet=0;		/* flag to not blather */
+static int comp_count=0;		/* Count of components in FASTGEN4 file */
+static int f4_do_skips=0;		/* flag indicating that not all components will be processed */
+static int *region_list;		/* array of region_ids to be processed */
+static int region_list_len=0;	/* actual length of the malloc'd region_list array */
+static int f4_do_plot=0;		/* flag indicating plot file should be created */
 static struct wmember *group_head = (struct wmember *)NULL; /* Lists of regions for groups */
 static ssize_t group_head_cnt=0;
-static struct wmember  hole_head;	/* List of regions used as holes (not solid parts of model) */
+static struct wmember hole_head;	/* List of regions used as holes (not solid parts of model) */
 static struct bu_ptbl stack;		/* Stack for traversing name_tree */
 static struct bu_ptbl stack2;		/* Stack for traversing name_tree */
-static fastf_t	min_radius;		/* minimum radius for TGC solids */
+static fastf_t min_radius;		/* minimum radius for TGC solids */
 
-static int	*faces=NULL;	/* one triplet per face indexing three grid points */
-static fastf_t	*thickness;	/* thickness of each face */
-static char	*facemode;	/* mode for each face */
-static int	face_size=0;	/* actual length of above arrays */
-static int	face_count=0;	/* number of faces in above arrays */
+static int *faces=NULL;	/* one triplet per face indexing three grid points */
+static fastf_t *thickness;	/* thickness of each face */
+static char *facemode;	/* mode for each face */
+static int face_size=0;	/* actual length of above arrays */
+static int face_count=0;	/* number of faces in above arrays */
 
-/*static int	*int_list;*/		/* Array of integers */
-/*static int	int_list_count=0;*/	/* Number of ints in above array */
-/*static int	int_list_length=0;*/	/* Length of int_list array */
 
 static point_t *grid_points = NULL;
 
@@ -258,7 +267,7 @@ get_line(void)
 	if (len < 0) goto out; /* eof or error */
 	if (len == 0) continue;
 	bu_vls_trimspace(&buffer);
-	len = bu_vls_strlen(&buffer);
+	len = (int)bu_vls_strlen(&buffer);
 	if (len == 0) continue;
 	done = 1;
     }
@@ -331,6 +340,7 @@ plot_tri(int pt1, int pt2, int pt3)
     pdv_3cont(fp_plot, grid_points[pt1]);
 }
 
+
 static void
 Check_names(void)
 {
@@ -377,7 +387,7 @@ Check_names(void)
 
 
 static struct name_tree *
-Search_names(struct name_tree *root, char *name, int *found)
+Search_names(struct name_tree *root, const char *name, int *found)
 {
     struct name_tree *ptr;
 
@@ -423,7 +433,7 @@ Search_ident(struct name_tree *root, int reg_id, int *found)
     while (1) {
 	int diff;
 
-	diff = reg_id -  ptr->region_id;
+	diff = reg_id - ptr->region_id;
 
 	if (diff == 0) {
 	    *found = 1;
@@ -486,7 +496,7 @@ List_names(void)
 
 
 static void
-Insert_region_name(char *name, int reg_id)
+Insert_region_name(const char *name, int reg_id)
 {
     struct name_tree *nptr_model, *rptr_model;
     struct name_tree *new_ptr;
@@ -521,9 +531,7 @@ Insert_region_name(char *name, int reg_id)
     new_ptr->name = bu_strdup(name);
     new_ptr->magic = NAME_TREE_MAGIC;
 
-    if (reg_id > region_id_max) {
-	region_id_max = reg_id;
-    }
+    V_MAX(region_id_max, reg_id);
 
     if (!name_root) {
 	name_root = new_ptr;
@@ -564,8 +572,6 @@ Insert_region_name(char *name, int reg_id)
 	}
     }
     Check_names();
-    if (RT_G_DEBUG&DEBUG_MEM_FULL &&  bu_mem_barriercheck())
-	bu_log("ERROR: bu_mem_barriercheck failed in Insert_region_name\n");
 }
 
 
@@ -590,27 +596,27 @@ find_region_name(int g_id, int c_id)
 }
 
 
-static char *
-make_unique_name(char *name)
+static void
+make_unique_name(struct bu_vls *name)
 {
     struct bu_vls vls = BU_VLS_INIT_ZERO;
     int found;
 
     /* make a unique name from what we got off the $NAME card */
 
-    (void)Search_names(name_root, name, &found);
+    (void)Search_names(name_root, bu_vls_cstr(name), &found);
     if (!found)
-	return bu_strdup(name);
+	return;
 
     while (found) {
 	bu_vls_trunc(&vls, 0);
-	bu_vls_printf(&vls, "%s_%d", name, name_count);
+	bu_vls_printf(&vls, "%s_%d", bu_vls_cstr(name), name_count);
 	(void)Search_names(name_root, bu_vls_addr(&vls), &found);
+	name_count++;
     }
-    if (RT_G_DEBUG&DEBUG_MEM_FULL &&  bu_mem_barriercheck())
-	bu_log("ERROR: bu_mem_barriercheck failed in make_unique_name\n");
 
-    return bu_vls_strgrab(&vls);
+    bu_vls_sprintf(name, "%s", bu_vls_cstr(&vls));
+    bu_vls_free(&vls);
 }
 
 
@@ -619,7 +625,7 @@ make_region_name(int g_id, int c_id)
 {
     int r_id;
     const char *tmp_name;
-    char *name;
+    struct bu_vls name = BU_VLS_INIT_ZERO;
 
     r_id = g_id * 1000 + c_id;
 
@@ -631,14 +637,15 @@ make_region_name(int g_id, int c_id)
 	return;
 
     /* create a new name */
-    name = (char *)bu_malloc(MAX_LINE_SIZE, "make_region_name");
-    snprintf(name, MAX_LINE_SIZE, "comp_%04d.r", r_id);
+    bu_vls_sprintf(&name, "comp_%04d.r", r_id);
 
-    make_unique_name(name);
+    make_unique_name(&name);
 
-    Insert_region_name(name, r_id);
+    Insert_region_name(bu_vls_cstr(&name), r_id);
+    bu_vls_free(&name);
 }
 
+
 static char *
 get_solid_name(char type, int element_id, int c_id, int g_id, int inner)
 {
@@ -699,8 +706,6 @@ Insert_name(struct name_tree **root, char *name, int inner)
 	}
 	ptr->nleft = new_ptr;
     }
-    if (RT_G_DEBUG&DEBUG_MEM_FULL &&  bu_mem_barriercheck())
-	bu_log("ERROR: bu_mem_barriercheck failed in Insert_name\n");
 }
 
 
@@ -717,97 +722,6 @@ make_solid_name(char type, int element_id, int c_id, int g_id, int inner)
 }
 
 
-/*
-  static void
-  insert_int(int in)
-  {
-  int i;
-
-  for (i=0; i<int_list_count; i++) {
-  if (int_list[i] == in)
-  return;
-  }
-
-  if (int_list_count == int_list_length) {
-  if (int_list_length == 0)
-  int_list = (int *)bu_malloc(INT_LIST_BLOCK*sizeof(int), "insert_id: int_list");
-  else
-  int_list = (int *)bu_realloc((char *)int_list, (int_list_length + INT_LIST_BLOCK)*sizeof(int), "insert_id: int_list");
-  int_list_length += INT_LIST_BLOCK;
-  }
-
-  int_list[int_list_count] = in;
-  int_list_count++;
-
-  if (RT_G_DEBUG&DEBUG_MEM_FULL &&  bu_mem_barriercheck())
-  bu_log("ERROR: bu_mem_barriercheck failed in insert_int\n");
-  }
-*/
-
-
-/*
-  static void
-  Subtract_holes(struct wmember *head, int comp_id, int group_id)
-  {
-  struct holes *hole_ptr;
-  struct hole_list *list_ptr;
-
-  if (debug)
-  bu_log("Subtract_holes(comp_id=%d, group_id=%d)\n", comp_id, group_id);
-
-  hole_ptr = hole_root;
-  while (hole_ptr) {
-  if (hole_ptr->group == group_id && hole_ptr->component == comp_id) {
-  list_ptr = hole_ptr->holes;
-  while (list_ptr) {
-  struct name_tree *ptr;
-  int reg_id;
-
-  reg_id = list_ptr->group * 1000 + list_ptr->component;
-  ptr = name_root;
-  while (ptr && ptr->region_id != reg_id) {
-  int diff;
-
-  diff = reg_id - ptr->region_id;
-  if (diff > 0)
-  ptr = ptr->rright;
-  else if (diff < 0)
-  ptr = ptr->rleft;
-  }
-
-  bu_ptbl_reset(&stack);
-
-  while (ptr && ptr->region_id == reg_id) {
-
-  while (ptr && ptr->region_id == reg_id) {
-  PUSH(ptr);
-  ptr = ptr->rleft;
-  }
-  POP(name_tree, ptr);
-  if (!ptr ||  ptr->region_id != reg_id)
-  break;
-
-  if (debug)
-  bu_log("\tSubtracting %s\n", ptr->name);
-
-  if (mk_addmember(ptr->name, &(head->l), NULL, WMOP_SUBTRACT) == (struct wmember *)NULL)
-  bu_exit(1, "Subtract_holes: mk_addmember failed\n");
-
-  ptr = ptr->rright;
-  }
-
-  list_ptr = list_ptr->next;
-  }
-  break;
-  }
-  hole_ptr = hole_ptr->next;
-  }
-  if (RT_G_DEBUG&DEBUG_MEM_FULL &&  bu_mem_barriercheck())
-  bu_log("ERROR: bu_mem_barriercheck failed in subtract_holes\n");
-  }
-*/
-
-
 static void
 f4_do_compsplt(void)
 {
@@ -887,6 +801,10 @@ Add_stragglers_to_groups(void)
 	/* visit node */
 	CK_TREE_MAGIC(ptr);
 
+	/* FIXME: should not be manually recreating the wmember list
+	 * when extending it.  just use a null-terminated list and
+	 * realloc as needed...
+	 */
 	if (!ptr->in_comp_group && ptr->region_id > 0 && !is_a_hole(ptr->region_id)) {
 	    /* add this component to a series */
 
@@ -895,7 +813,7 @@ Add_stragglers_to_groups(void)
 		ssize_t new_cnt, i;
 		struct bu_list *list_first;
 
-		new_cnt = (ssize_t)ceil(region_id_max/1000.0);
+		new_cnt = lrint(ceil(region_id_max/1000.0));
 		new_head = (struct wmember *)bu_calloc(new_cnt, sizeof(struct wmember), "group_head list");
 		bu_log("ptr->region_id=%d region_id_max=%d new_cnt=%ld\n", ptr->region_id, region_id_max, new_cnt);
 
@@ -962,11 +880,10 @@ f4_do_groups(void)
 static void
 f4_do_name(void)
 {
-    int i, j;
+    int foundr = 0;
     int g_id;
     int c_id;
-    char comp_name[MAX_LINE_SIZE] = {0}; /* should only use 25 chars */
-    char tmp_name[MAX_LINE_SIZE] = {0}; /* should only use 25 chars */
+    struct bu_vls comp_name = BU_VLS_INIT_ZERO;
 
     if (pass)
 	return;
@@ -992,43 +909,30 @@ f4_do_name(void)
 	return;
     }
 
-    /* skip leading blanks */
-    i = 56;
-    while ((size_t)i < sizeof(comp_name) && isspace((int)line[i]))
-	i++;
-
-    if (i == sizeof(comp_name))
+    /* Eliminate leading and trailing blanks */
+    bu_vls_sprintf(&comp_name, "%s", &line[56]);
+    bu_vls_trimspace(&comp_name);
+    if (!bu_vls_strlen(&comp_name)) {
+	bu_vls_free(&comp_name);
 	return;
-
-    bu_strlcpy(comp_name, &line[i], sizeof(comp_name) - i);
-
-    /* eliminate trailing blanks */
-    i = sizeof(comp_name) - i;
-    while ( --i >= 0 && isspace((int)comp_name[i]))
-	comp_name[i] = '\0';
-
-    /* copy comp_name to tmp_name while replacing white space with "_" */
-    i = (-1);
-    j = (-1);
-
-    /* copy */
-    while (comp_name[++i] != '\0') {
-	if (isspace((int)comp_name[i]) || comp_name[i] == '/') {
-	    if (j == (-1) || tmp_name[j] != '_')
-		tmp_name[++j] = '_';
-	} else {
-	    tmp_name[++j] = comp_name[i];
-	}
     }
-    tmp_name[++j] = '\0';
+
+    /* Simplify */
+    bu_vls_simplify(&comp_name, NULL, NULL, NULL);
 
     /* reserve this name for group name */
-    make_unique_name(tmp_name);
-    Insert_region_name(tmp_name, region_id);
+    Search_ident(name_root, region_id, &foundr);
+    if (!foundr) {
+	make_unique_name(&comp_name);
+    } else {
+	bu_log("Already encountered region id %d - reusing name\n", region_id);
+    }
+
+    Insert_region_name(bu_vls_cstr(&comp_name), region_id);
 
     name_count = 0;
-    if (RT_G_DEBUG&DEBUG_MEM_FULL &&  bu_mem_barriercheck())
-	bu_log("ERROR: bu_mem_barriercheck failed in f4_do_name\n");
+
+    bu_vls_free(&comp_name);
 }
 
 
@@ -1041,9 +945,6 @@ f4_do_grid(void)
     if (!pass)	/* not doing geometry yet */
 	return;
 
-    if (RT_G_DEBUG&DEBUG_MEM_FULL &&  bu_mem_barriercheck())
-	bu_log("ERROR: bu_mem_barriercheck failed at start of f4_do_grid\n");
-
     bu_strlcpy(field, &line[8], sizeof(field));
     grid_no = atoi(field);
 
@@ -1066,11 +967,7 @@ f4_do_grid(void)
     }
 
     VSET(grid_points[grid_no], x*25.4, y*25.4, z*25.4);
-
-    if (grid_no > max_grid_no)
-	max_grid_no = grid_no;
-    if (RT_G_DEBUG&DEBUG_MEM_FULL &&  bu_mem_barriercheck())
-	bu_log("ERROR: bu_mem_barriercheck failed at end of f4_do_grid\n");
+    V_MAX(max_grid_no, grid_no);
 }
 
 
@@ -1302,13 +1199,15 @@ f4_do_ccone1(void)
 
     if (mode == PLATE_MODE) {
 	if (thick <= 0.0) {
-	    bu_log("ERROR: Plate mode CCONE1 has illegal thickness (%f)\n", thick/25.4);
+	    bu_log("WARNING: Plate mode CCONE1 has illegal thickness (%f)\n", thick/25.4);
 	    bu_log("\tgroup_id = %d, comp_id = %d, element_id = %d\n",
 		   group_id, comp_id, element_id);
-	    bu_log("\tCCONE1 solid ignored\n");
-	    return;
+	    bu_log("\tCCONE1 solid plate mode overridden, now being treated as volume mode\n");
+	    mode = VOLUME_MODE;
 	}
+    }
 
+    if (mode == PLATE_MODE) {
 	if (r1-thick < min_radius && r2-thick < min_radius) {
 	    bu_log("ERROR: Plate mode CCONE1 has too large thickness (%f)\n", thick/25.4);
 	    bu_log("\tgroup_id = %d, comp_id = %d, element_id = %d\n",
@@ -1327,10 +1226,8 @@ f4_do_ccone1(void)
     }
 
     /* BRL_CAD doesn't allow zero radius, so use a very small radius */
-    if (r1 < min_radius)
-	r1 = min_radius;
-    if (r2 < min_radius)
-	r2 = min_radius;
+    V_MAX(r1, min_radius);
+    V_MAX(r2, min_radius);
 
     VSUB2(height, grid_points[pt2], grid_points[pt1]);
 
@@ -1382,8 +1279,8 @@ f4_do_ccone1(void)
 	    dist_to_new_base = inner_r1 * length/(r1 - r2);
 	    inner_r1 = min_radius;
 	    VJOIN1(base, base, dist_to_new_base, height_dir);
-	} else if (inner_r1 < min_radius) {
-	    inner_r1 = min_radius;
+	} else {
+	    V_MAX(inner_r1, min_radius);
 	}
 
 	if (end2 == END_OPEN) {
@@ -1402,8 +1299,8 @@ f4_do_ccone1(void)
 	    dist_to_new_top = inner_r2 * length/(r2 - r1);
 	    inner_r2 = min_radius;
 	    VJOIN1(top, top, -dist_to_new_top, height_dir);
-	} else if (inner_r2 < min_radius) {
-	    inner_r2 = min_radius;
+	} else {
+	    V_MAX(inner_r2, min_radius);
 	}
 
 	VSUB2(inner_height, top, base);
@@ -1507,11 +1404,8 @@ f4_do_ccone2(void)
 	return;
     }
 
-    if (ro1 < min_radius)
-	ro1 = min_radius;
-
-    if (ro2 < min_radius)
-	ro2 = min_radius;
+    V_MAX(ro1, min_radius);
+    V_MAX(ro2, min_radius);
 
     BU_LIST_INIT(&r_head.l);
 
@@ -1529,11 +1423,8 @@ f4_do_ccone2(void)
 	    bu_exit(1, "mk_addmember failed!\n");
 	bu_free(name, "solid_name");
 
-	if (ri1 < min_radius)
-	    ri1 = min_radius;
-
-	if (ri2 < min_radius)
-	    ri2 = min_radius;
+	V_MAX(ri1, min_radius);
+	V_MAX(ri2, min_radius);
 
 	name = make_solid_name(CCONE2, element_id, comp_id, group_id, 2);
 	mk_trc_h(fpout, name, grid_points[pt1], height, ri1, ri2);
@@ -1677,10 +1568,8 @@ f4_do_ccone3(void)
     }
 
     for (i=0; i<4; i++) {
-	if (ro[i] < min_radius)
-	    ro[i] = min_radius;
-	if (ri[i] < min_radius)
-	    ri[i] = min_radius;
+	V_MAX(ro[i], min_radius);
+	V_MAX(ri[i], min_radius);
     }
 
     BU_LIST_INIT(&r_head.l);
@@ -1794,18 +1683,12 @@ Add_holes(int type, int gr, int comp, struct hole_list *ptr)
 	if (!skip_region(gr*1000 + comp)) {
 	    /* add holes for this region to the list of regions to process */
 	    hptr = ptr;
-	    if (RT_G_DEBUG&DEBUG_MEM_FULL &&  bu_mem_barriercheck())
-		bu_log("ERROR: bu_mem_barriercheck failed in Add_hole\n");
 	    while (hptr) {
 		if (f4_do_skips == region_list_len) {
 		    region_list_len += REGION_LIST_BLOCK;
 		    region_list = (int *)bu_realloc((char *)region_list, region_list_len*sizeof(int), "region_list");
-		    if (RT_G_DEBUG&DEBUG_MEM_FULL &&  bu_mem_barriercheck())
-			bu_log("ERROR: bu_mem_barriercheck failed in Add_hole (after realloc)\n");
 		}
 		region_list[f4_do_skips++] = 1000*hptr->group + hptr->component;
-		if (RT_G_DEBUG&DEBUG_MEM_FULL &&  bu_mem_barriercheck())
-		    bu_log("ERROR: bu_mem_barriercheck failed in Add_hole (after adding %d\n)\n", 1000*hptr->group + hptr->component);
 		hptr = hptr->next;
 	    }
 	}
@@ -1881,8 +1764,7 @@ f4_do_hole_wall(int type)
 	line[s_len] = '\0';
 
     s_len = strlen(line);
-    if (s_len > 80)
-	s_len = 80;
+    V_MIN(s_len, 80);
 
     bu_strlcpy(field, &line[8], sizeof(field));
     group = atoi(field);
@@ -1951,13 +1833,9 @@ f4_Add_bot_face(int pt1, int pt2, int pt3, fastf_t thick, int pos)
 
     if (face_count >= face_size) {
 	face_size += GRID_BLOCK;
-	if (bu_debug&BU_DEBUG_MEM_CHECK &&  bu_mem_barriercheck())
-	    bu_log("memory corrupted before realloc of faces, thickness, and facemode\n");
 	faces = (int *)bu_realloc((void *)faces,  face_size*3*sizeof(int), "faces");
 	thickness = (fastf_t *)bu_realloc((void *)thickness, face_size*sizeof(fastf_t), "thickness");
 	facemode = (char *)bu_realloc((void *)facemode, face_size*sizeof(char), "facemode");
-	if (bu_debug&BU_DEBUG_MEM_CHECK &&  bu_mem_barriercheck())
-	    bu_log("memory corrupted after realloc of faces, thickness, and facemode\n");
     }
 
     faces[face_count*3] = pt1;
@@ -1968,14 +1846,11 @@ f4_Add_bot_face(int pt1, int pt2, int pt3, fastf_t thick, int pos)
 	thickness[face_count] = thick;
 	facemode[face_count] = pos;
     } else {
-	thickness[face_count] = 0, 0;
+	thickness[face_count] = 0.0;
 	facemode[face_count] = 0;
     }
 
     face_count++;
-
-    if (bu_debug&BU_DEBUG_MEM_CHECK &&  bu_mem_barriercheck())
-	bu_log("memory corrupted at end of f4_Add_bot_face()\n");
 }
 
 
@@ -2000,15 +1875,11 @@ f4_do_tri(void)
 	return;
 
     if (faces == NULL) {
-	if (bu_debug&BU_DEBUG_MEM_CHECK &&  bu_mem_barriercheck())
-	    bu_log("memory corrupted before malloc of faces\n");
 	faces = (int *)bu_malloc(GRID_BLOCK*3*sizeof(int), "faces");
 	thickness = (fastf_t *)bu_malloc(GRID_BLOCK*sizeof(fastf_t), "thickness");
 	facemode = (char *)bu_malloc(GRID_BLOCK*sizeof(char), "facemode");
 	face_size = GRID_BLOCK;
 	face_count = 0;
-	if (bu_debug&BU_DEBUG_MEM_CHECK &&  bu_mem_barriercheck())
-	    bu_log("memory corrupted after malloc of faces, thickness, and facemode\n");
     }
 
     bu_strlcpy(field, &line[24], sizeof(field));
@@ -2040,13 +1911,7 @@ f4_do_tri(void)
     if (f4_do_plot)
 	plot_tri(pt1, pt2, pt3);
 
-    if (bu_debug&BU_DEBUG_MEM_CHECK &&  bu_mem_barriercheck())
-	bu_log("memory corrupted before call to f4_Add_bot_face()\n");
-
     f4_Add_bot_face(pt1, pt2, pt3, thick, pos);
-
-    if (bu_debug&BU_DEBUG_MEM_CHECK &&  bu_mem_barriercheck())
-	bu_log("memory corrupted after call to f4_Add_bot_face()\n");
 }
 
 
@@ -2150,7 +2015,7 @@ make_bot_object(void)
     for (i=0; i<face_count*3; i++)
 	faces[i] -= min_pt;
     bot_ip.num_faces = face_count;
-    bot_ip.faces = bu_calloc(face_count*3, sizeof(int), "BOT faces");
+    bot_ip.faces = (int *)bu_calloc(face_count*3, sizeof(int), "BOT faces");
     for (i=0; i<face_count*3; i++)
 	bot_ip.faces[i] = faces[i];
 
@@ -2198,7 +2063,7 @@ make_bot_object(void)
 static void
 skip_section(void)
 {
-    off_t section_start;
+    b_off_t section_start;
 
     /* skip to start of next section */
     section_start = bu_ftell(fpin);
@@ -2225,8 +2090,8 @@ skip_section(void)
 }
 
 
-/*	cleanup from previous component and start a new one.
- *	This is called with final == 1 when ENDDATA is found
+/* cleanup from previous component and start a new one.
+ * This is called with final == 1 when ENDDATA is found
  */
 static void
 f4_do_section(int final)
@@ -2456,8 +2321,6 @@ Process_hole_wall(void)
 {
     if (debug)
 	bu_log("Process_hole_wall\n");
-    if (bu_debug & BU_DEBUG_MEM_CHECK)
-	bu_prmem("At start of Process_hole_wall:");
 
     rewind(fpin);
     while (1) {
@@ -2547,8 +2410,6 @@ Process_input(int pass_number)
 
     if (debug)
 	bu_log("\n\nProcess_input(pass = %d)\n", pass_number);
-    if (bu_debug & BU_DEBUG_MEM_CHECK)
-	bu_prmem("At start of Process_input:");
 
     if (pass_number != 0 && pass_number != 1) {
 	bu_exit(1, "Process_input: illegal pass number %d\n", pass_number);
@@ -2626,7 +2487,7 @@ make_region_list(char *str)
     region_list_len = REGION_LIST_BLOCK;
     f4_do_skips = 0;
 
-    ptr = strtok(str, ",");
+    ptr = strtok(str, CPP_XSTR(COMMA));
     while (ptr) {
 	if ((ptr2=strchr(ptr, '-'))) {
 	    int i, start, stop;
@@ -2635,18 +2496,12 @@ make_region_list(char *str)
 	    ptr2++;
 	    start = atoi(ptr);
 	    stop = atoi(ptr2);
-	    if (bu_debug&BU_DEBUG_MEM_CHECK &&  bu_mem_barriercheck())
-		bu_log("ERROR: bu_mem_barriercheck failed in make_region_list\n");
 	    for (i=start; i<=stop; i++) {
 		if (f4_do_skips == region_list_len) {
 		    region_list_len += REGION_LIST_BLOCK;
 		    region_list = (int *)bu_realloc((char *)region_list, region_list_len*sizeof(int), "region_list");
-		    if (bu_debug&BU_DEBUG_MEM_CHECK &&  bu_mem_barriercheck())
-			bu_log("ERROR: bu_mem_barriercheck failed in make_region_list (after realloc)\n");
 		}
 		region_list[f4_do_skips++] = i;
-		if (bu_debug&BU_DEBUG_MEM_CHECK &&  bu_mem_barriercheck())
-		    bu_log("ERROR: bu_mem_barriercheck failed in make_region_list (after adding %d)\n", i);
 	    }
 	} else {
 	    if (f4_do_skips == region_list_len) {
@@ -2655,7 +2510,7 @@ make_region_list(char *str)
 	    }
 	    region_list[f4_do_skips++] = atoi(ptr);
 	}
-	ptr = strtok((char *)NULL, ",");
+	ptr = strtok((char *)NULL, ", ");
     }
 }
 
@@ -2899,8 +2754,8 @@ main(int argc, char **argv)
 		warnings = 1;
 		break;
 	    case 'x':
-		sscanf(bu_optarg, "%x", (unsigned int *)&rt_debug);
-		bu_debug = rt_debug;
+		sscanf(bu_optarg, "%x", (unsigned int *)&frt_debug);
+		bu_debug = frt_debug;
 		break;
 	    case 'b':
 		sscanf(bu_optarg, "%x", (unsigned int *)&bu_debug);
@@ -2914,15 +2769,10 @@ main(int argc, char **argv)
 	}
     }
 
-    if (bu_debug & BU_DEBUG_MEM_CHECK)
-	bu_log("doing memory checking\n");
-
     if (argc-bu_optind != 2) {
 	usage();
     }
 
-    rt_init_resource(&rt_uniresource, 0, NULL);
-
     if ((fpin=fopen(argv[bu_optind], "rb")) == (FILE *)NULL) {
 	perror("fast4-g");
 	bu_exit(1, "Cannot open FASTGEN4 file (%s)\n", argv[bu_optind]);
@@ -2941,11 +2791,11 @@ main(int argc, char **argv)
     }
 
     if (bu_debug) {
-	bu_printb("librtbu_debug", bu_debug, DEBUG_FORMAT);
+	bu_printb("librtbu_debug", bu_debug, RT_DEBUG_FORMAT);
 	bu_log("\n");
     }
-    if (RTG.NMG_debug) {
-	bu_printb("librt RTG.NMG_debug", RTG.NMG_debug, NMG_DEBUG_FORMAT);
+    if (nmg_debug) {
+	bu_printb("librt nmg_debug", nmg_debug, NMG_DEBUG_FORMAT);
 	bu_log("\n");
     }
 
@@ -2990,8 +2840,12 @@ main(int argc, char **argv)
     rewind(fpin);
 
     /* Make an ID record if no vehicle card was found */
-    if (!vehicle[0])
-	mk_id_units(fpout, argv[bu_optind], "in");
+    if (!vehicle[0]) {
+	struct bu_vls fname = BU_VLS_INIT_ZERO;
+	bu_path_component(&fname, argv[bu_optind], BU_PATH_BASENAME);
+	mk_id_units(fpout, bu_vls_cstr(&fname), "in");
+	bu_vls_free(&fname);
+    }
 
     if (!quiet)
 	bu_log("Building components....\n");
@@ -3020,6 +2874,7 @@ main(int argc, char **argv)
     return 0;
 }
 
+
 /*
  * Local Variables:
  * mode: C
