@@ -1,7 +1,7 @@
 /*                        G - I G E S . C
  * BRL-CAD
  *
- * Copyright (c) 2004-2013 United States Government as represented by
+ * Copyright (c) 2004-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This program is free software; you can redistribute it and/or
@@ -44,18 +44,21 @@
 #include "vmath.h"
 #include "nmg.h"
 #include "raytrace.h"
-#include "rtgeom.h"
+#include "rt/geom.h"
 
+#include "bu/app.h"
+#include "bu/parallel.h"
+#include "bu/getopt.h"
 /* private */
 #include "./iges.h"
 #include "brlcad_ident.h"
 #include "brlcad_version.h"
 
 
-#define CP_BUF_SIZE 4096	/* size of buffer for file copy */
+#define CP_BUF_SIZE BU_PAGE_SIZE /* size of buffer for file copy */
 #define SUFFIX_LEN 10 /* max size of suffix for 'part' files (-m option) */
 
-extern union tree *do_nmg_region_end(struct db_tree_state *tsp, const struct db_full_path *pathp, union tree *curtree, genptr_t client_data);
+extern union tree *do_nmg_region_end(struct db_tree_state *tsp, const struct db_full_path *pathp, union tree *curtree, void *client_data);
 void w_start_global(
     FILE *fp_dir,
     FILE *fp_param,
@@ -70,14 +73,14 @@ extern void write_vertex_list(struct nmgregion *r, struct bu_ptbl *vtab, FILE *f
 extern void nmg_region_edge_list(struct bu_ptbl *tab, struct nmgregion *r);
 extern int nmgregion_to_iges(char *name, struct nmgregion *r, int dependent, FILE *fp_dir, FILE *fp_param);
 extern int write_shell_face_loop(struct nmgregion *r, int edge_de, struct bu_ptbl *etab, int vert_de, struct bu_ptbl *vtab, FILE *fp_dir, FILE *fp_param);
-extern void csg_comb_func(struct db_i *dbip, struct directory *dp, genptr_t ptr);
-extern void csg_leaf_func(struct db_i *dbip, struct directory *dp, genptr_t ptr);
-extern void set_iges_tolerances(struct bn_tol *set_tol, struct rt_tess_tol *set_ttol);
-extern void count_refs(struct db_i *dbip, struct directory *dp, genptr_t ptr);
+extern void csg_comb_func(struct db_i *dbip, struct directory *dp, void *ptr);
+extern void csg_leaf_func(struct db_i *dbip, struct directory *dp, void *ptr);
+extern void set_iges_tolerances(struct bn_tol *set_tol, struct bg_tess_tol *set_ttol);
+extern void count_refs(struct db_i *dbip, struct directory *dp, void *ptr);
 extern int nmgregion_to_tsurf(char *name, struct nmgregion *r, FILE *fp_dir, FILE *fp_param);
 extern int write_solid_instance(int orig_de, mat_t mat, FILE *fp_dir, FILE *fp_param);
 extern void get_props(struct iges_properties *props, struct rt_comb_internal *comb);
-extern int comb_to_iges(struct rt_comb_internal *comb, int length, int dependent, struct iges_properties *props, int de_pointers[], FILE *fp_dir, FILE *fp_param);
+extern int comb_to_iges(struct rt_comb_internal *comb, size_t length, int dependent, struct iges_properties *props, int de_pointers[], FILE *fp_dir, FILE *fp_param);
 
 static void
 usage(const char *argv0)
@@ -116,7 +119,7 @@ static int ncpu = 1;	/* Number of processors */
 static char *output_file = NULL;	/* output filename */
 static FILE *fp_dir = NULL;	/* IGES start, global, and directory sections */
 static FILE *fp_param = NULL;	/* IGES parameter section */
-static struct rt_tess_tol ttol;
+static struct bg_tess_tol ttol;
 static struct bn_tol tol;
 static struct model *the_model;
 struct db_i *DBIP;
@@ -132,7 +135,7 @@ extern int tor_to_iges(struct rt_db_internal *, char *, FILE *, FILE *);
 extern int tgc_to_iges(struct rt_db_internal *, char *, FILE *, FILE *);
 extern int nmg_to_iges(struct rt_db_internal *, char *, FILE *, FILE *);
 extern int sketch_to_iges(struct rt_db_internal *, char *, FILE *, FILE *);
-extern void iges_init(struct bn_tol *, struct rt_tess_tol *, int, struct db_i *);
+extern void iges_init(struct bn_tol *, struct bg_tess_tol *, int, struct db_i *);
 extern void Print_stats(FILE *);
 
 struct iges_functab
@@ -185,9 +188,6 @@ char **independent;
 size_t no_of_indeps = 0;
 int do_nurbs = 0;
 
-/*
- * M A I N
- */
 int
 main(int argc, char *argv[])
 {
@@ -209,7 +209,7 @@ main(int argc, char *argv[])
     tree_state.ts_ttol = &ttol;
     tree_state.ts_m = &the_model;
 
-    ttol.magic = RT_TESS_TOL_MAGIC;
+    ttol.magic = BG_TESS_TOL_MAGIC;
     /* Defaults, updated by command line options. */
     ttol.abs = 0.0;
     ttol.rel = 0.01;
@@ -225,8 +225,6 @@ main(int argc, char *argv[])
     the_model = nmg_mm();
     BU_LIST_INIT(&RTG.rtg_vlfree);	/* for vlist macros */
 
-    rt_init_resource(&rt_uniresource, 0, NULL);
-
     prog_name = argv[0];
 
     /* Get command line arguments. */
@@ -264,11 +262,11 @@ main(int argc, char *argv[])
 		tol.dist_sq = tol.dist * tol.dist;
 		break;
 	    case 'x':
-		sscanf(bu_optarg, "%x", (unsigned int *)&RTG.debug);
+		sscanf(bu_optarg, "%x", (unsigned int *)&rt_debug);
 		break;
 	    case 'X':
-		sscanf(bu_optarg, "%x", (unsigned int *)&RTG.NMG_debug);
-		NMG_debug = RTG.NMG_debug;
+		sscanf(bu_optarg, "%x", (unsigned int *)&nmg_debug);
+		NMG_debug = nmg_debug;
 		break;
 	    case 'o':		/* Output file name. */
 		output_file = bu_optarg;
@@ -348,7 +346,7 @@ main(int argc, char *argv[])
 			   0,			/* take all regions */
 			   do_nmg_region_end,
 			   nmg_booltree_leaf_tess,
-			   (genptr_t)NULL);	/* in librt/nmg_bool.c */
+			   (void *)NULL);	/* in librt/nmg_bool.c */
 
 	if (ret)
 	    bu_exit(1, "g-iges: Could not facetize anything!");
@@ -395,7 +393,7 @@ main(int argc, char *argv[])
 			   0,			/* take all regions */
 			   do_nmg_region_end,
 			   nmg_booltree_leaf_tess,
-			   (genptr_t)NULL);	/* in librt/nmg_bool.c */
+			   (void *)NULL);	/* in librt/nmg_bool.c */
 
 	if (ret)
 	    bu_exit(1, "g-iges: Could not facetize anything!");
@@ -420,7 +418,7 @@ main(int argc, char *argv[])
     }
 
     /* Print some statistics */
-    Print_stats(stdout);
+    Print_stats(stderr);
 
     /* report on the success rate for facetizing regions */
     if (mode == FACET_MODE || mode == TRIMMED_SURF_MODE) {
@@ -447,14 +445,14 @@ main(int argc, char *argv[])
 static union tree *
 process_boolean(struct db_tree_state *tsp, union tree *curtree, const struct db_full_path *pathp)
 {
-    union tree *result = NULL;
+    static union tree *result = NULL;
 
     /* Begin bomb protection */
     if (!BU_SETJUMP) {
 	/* try */
 
-	(void)nmg_model_fuse(*tsp->ts_m, tsp->ts_tol);
-	result = nmg_booltree_evaluate(curtree, tsp->ts_tol, &rt_uniresource);
+	(void)nmg_model_fuse(*tsp->ts_m, &RTG.rtg_vlfree, tsp->ts_tol);
+	result = nmg_booltree_evaluate(curtree, &RTG.rtg_vlfree, tsp->ts_tol, &rt_uniresource);
 
     } else {
 	/* catch */
@@ -470,7 +468,7 @@ process_boolean(struct db_tree_state *tsp, union tree *curtree, const struct db_
 	/* Sometimes the NMG library adds debugging bits when
 	 * it detects an internal error, before bombing out.
 	 */
-	RTG.NMG_debug = NMG_debug;	/* restore mode */
+	nmg_debug = NMG_debug;	/* restore mode */
 
 	/* Release the tree memory & input regions */
 	db_free_tree(curtree, &rt_uniresource);		/* Does an nmg_kr() */
@@ -488,14 +486,12 @@ process_boolean(struct db_tree_state *tsp, union tree *curtree, const struct db_
 
 
 /*
- * D O _ N M G _ R E G I O N _ E N D
- *
  * Called from db_walk_tree().
  *
  * This routine must be prepared to run in parallel.
  */
 union tree *
-do_nmg_region_end(struct db_tree_state *tsp, const struct db_full_path *pathp, union tree *curtree, genptr_t UNUSED(client_data))
+do_nmg_region_end(struct db_tree_state *tsp, const struct db_full_path *pathp, union tree *curtree, void *UNUSED(client_data))
 {
     union tree *result;
     struct nmgregion *r;
@@ -504,13 +500,13 @@ do_nmg_region_end(struct db_tree_state *tsp, const struct db_full_path *pathp, u
     int dependent;
     size_t i;
 
-    RT_CK_TESS_TOL(tsp->ts_ttol);
+    BG_CK_TESS_TOL(tsp->ts_ttol);
     BN_CK_TOL(tsp->ts_tol);
     NMG_CK_MODEL(*tsp->ts_m);
 
     BU_LIST_INIT(&vhead);
 
-    if (RT_G_DEBUG&DEBUG_TREEWALK || verbose) {
+    if (RT_G_DEBUG&RT_DEBUG_TREEWALK || verbose) {
 	char *sofar = db_path_to_string(pathp);
 	bu_log("\ndo_nmg_region_end(%d %d%%) %s\n",
 	       regions_tried,
@@ -555,7 +551,7 @@ do_nmg_region_end(struct db_tree_state *tsp, const struct db_full_path *pathp, u
 
 		/* construct a unique file name */
 		len = strlen(output_file) + strlen(dp->d_namep) + 6 + SUFFIX_LEN;
-		multi_name = bu_malloc(sizeof(char)*len, "multi_name");
+		multi_name = (char *)bu_malloc(sizeof(char)*len, "multi_name");
 		snprintf(multi_name, len, "%s/%s.igs", output_file, dp->d_namep);
 		bu_strlcpy(suffix, "a", sizeof(suffix));
 		suffix[0]--;
@@ -567,7 +563,7 @@ do_nmg_region_end(struct db_tree_state *tsp, const struct db_full_path *pathp, u
 
 		    /* not unique, try adding a suffix */
 		    len = strlen(suffix);
-		    i = len - 1;;
+		    i = len - 1;
 		    suffix[i]++;
 		    while (suffix[i] > 'z' && i > 0) {
 			suffix[i] = 'a';
@@ -582,7 +578,7 @@ do_nmg_region_end(struct db_tree_state *tsp, const struct db_full_path *pathp, u
 			bu_log("too many files with the same name (%s)\n", dp->d_namep);
 			bu_exit(1, "Cannot create a unique filename, \n");
 		    }
-		    snprintf(multi_name, len, "%s/%s%s.igs", output_file, dp->d_namep, suffix);
+		    snprintf(multi_name, len, "%s/%s%.6s.igs", output_file, dp->d_namep, suffix);
 		}
 		if ((fp_dir = fopen(multi_name, "wb")) == NULL) {
 		    perror("g-iges");
@@ -616,9 +612,6 @@ do_nmg_region_end(struct db_tree_state *tsp, const struct db_full_path *pathp, u
 	} else if (mode == TRIMMED_SURF_MODE)
 	    dp->d_uses = (-nmgregion_to_tsurf(dp->d_namep, r, fp_dir, fp_param));
 
-	/* NMG region is no longer necessary */
-	nmg_kr(r);
-
 	if (multi_file) {
 	    char copy_buffer[CP_BUF_SIZE] = {0};
 
@@ -659,11 +652,8 @@ do_nmg_region_end(struct db_tree_state *tsp, const struct db_full_path *pathp, u
 static int de_pointer_number;
 
 int
-get_de_pointers(tp, dp, de_len, de_pointers)
-    union tree *tp;
-    struct directory *dp;
-    int de_len;
-    int *de_pointers;
+get_de_pointers(union tree *tp, struct directory *dp, int de_len,
+		int *de_pointers)
 {
     RT_CK_TREE(tp);
     RT_CK_DIR(dp);
@@ -715,12 +705,12 @@ get_de_pointers(tp, dp, de_len, de_pointers)
 
 
 void
-csg_comb_func(struct db_i *dbip, struct directory *dp, genptr_t UNUSED(ptr))
+csg_comb_func(struct db_i *dbip, struct directory *dp, void *UNUSED(ptr))
 {
     struct rt_db_internal intern;
     struct rt_comb_internal *comb;
     struct iges_properties props;
-    int comb_len;
+    size_t comb_len;
     size_t i;
     int dependent = 1;
     int *de_pointers;
@@ -766,7 +756,7 @@ csg_comb_func(struct db_i *dbip, struct directory *dp, genptr_t UNUSED(ptr))
     comb_form = 0;
 
     de_pointer_number = 0;
-    if (get_de_pointers(comb->tree, dp, comb_len, de_pointers)) {
+    if (get_de_pointers(comb->tree, dp, (int)comb_len, de_pointers)) {
 	bu_log("Error in combination %s\n", dp->d_namep);
 	bu_free((char *)de_pointers, "csg_comb_func de_pointers");
 	rt_db_free_internal(&intern);
@@ -800,7 +790,7 @@ csg_comb_func(struct db_i *dbip, struct directory *dp, genptr_t UNUSED(ptr))
 
 
 void
-csg_leaf_func(struct db_i *dbip, struct directory *dp, genptr_t UNUSED(ptr))
+csg_leaf_func(struct db_i *dbip, struct directory *dp, void *UNUSED(ptr))
 {
     struct rt_db_internal ip;
 
@@ -832,7 +822,7 @@ csg_leaf_func(struct db_i *dbip, struct directory *dp, genptr_t UNUSED(ptr))
 
 
 void
-incr_refs(struct db_i *dbip, struct rt_comb_internal *comb, union tree *tp, genptr_t UNUSED(user_ptr1), genptr_t UNUSED(user_ptr2), genptr_t UNUSED(user_ptr3), genptr_t UNUSED(user_ptr4))
+incr_refs(struct db_i *dbip, struct rt_comb_internal *comb, union tree *tp, void *UNUSED(user_ptr1), void *UNUSED(user_ptr2), void *UNUSED(user_ptr3), void *UNUSED(user_ptr4))
 {
     struct directory *dp;
 
@@ -848,7 +838,7 @@ incr_refs(struct db_i *dbip, struct rt_comb_internal *comb, union tree *tp, genp
 
 
 void
-count_refs(struct db_i *dbip, struct directory *dp, genptr_t UNUSED(ptr))
+count_refs(struct db_i *dbip, struct directory *dp, void *UNUSED(ptr))
 {
     struct rt_db_internal intern;
     struct rt_comb_internal *comb;
@@ -875,7 +865,7 @@ count_refs(struct db_i *dbip, struct directory *dp, genptr_t UNUSED(ptr))
     comb_form = 0;
 
     db_tree_funcleaf(dbip, comb, comb->tree, incr_refs,
-		     (genptr_t)NULL, (genptr_t)NULL, (genptr_t)NULL, (genptr_t)NULL);
+		     (void *)NULL, (void *)NULL, (void *)NULL, (void *)NULL);
 
 }
 
