@@ -1,7 +1,7 @@
 /*                       S P L I N E F . C
  * BRL-CAD
  *
- * Copyright (c) 1990-2013 United States Government as represented by
+ * Copyright (c) 1990-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This program is free software; you can redistribute it and/or
@@ -22,9 +22,9 @@
 
 #include <stdio.h>
 
-#include "bu.h"
 #include "vmath.h"
 
+#include "bu/defines.h"
 
 /* spline functionality */
 fastf_t
