@@ -1,7 +1,7 @@
 /*                      T R I M S U R F . C
  * BRL-CAD
  *
- * Copyright (c) 1994-2013 United States Government as represented by
+ * Copyright (c) 1994-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This program is free software; you can redistribute it and/or
@@ -25,6 +25,7 @@
  *
  */
 
+#include "bu/debug.h"
 #include "./iges_struct.h"
 #include "./iges_extern.h"
 
@@ -48,20 +49,18 @@ struct snurb_hit
 
 
 struct face_g_snurb *
-Get_nurb_surf(entityno, m)
-    int entityno;
-    struct model *m;
+Get_nurb_surf(int entityno, struct model *m)
 {
     struct face_g_snurb *srf;
     point_t pt;
-    int entity_type;
+    int entity_type = 0;
     int i;
     int rational;
     int ncoords;
     int n_rows, n_cols, u_order, v_order;
     int n_u, n_v;
     int pt_type;
-    double a;
+    double a = 0.0;
 
     /* Acquiring Data */
 
@@ -98,7 +97,7 @@ Get_nurb_surf(entityno, m)
     n_u = n_cols+u_order;
     n_v = n_rows+v_order;
     if (!m) {
-	srf = rt_nurb_new_snurb(u_order, v_order, n_u, n_v, n_rows, n_cols, pt_type, (struct resource *)NULL);
+	srf = nmg_nurb_new_snurb(u_order, v_order, n_u, n_v, n_rows, n_cols, pt_type);
     } else {
 	int pnum;
 
@@ -193,9 +192,7 @@ Get_nurb_surf(entityno, m)
 
 
 void
-Assign_cnurb_to_eu(eu, crv)
-    struct edgeuse *eu;
-    struct edge_g_cnurb *crv;
+Assign_cnurb_to_eu(struct edgeuse *eu, struct edge_g_cnurb *crv)
 {
     fastf_t *ctl_points;
     fastf_t *knots;
@@ -226,21 +223,23 @@ Assign_cnurb_to_eu(eu, crv)
 
 
 struct edge_g_cnurb *
-Get_cnurb(entity_no)
-    int entity_no;
+Get_cnurb(int entity_no)
 {
     struct edge_g_cnurb *crv;
-    point_t pt, pt2;
-    int entity_type;
-    int num_pts;
-    int degree;
-    int i;
-    int planar;
-    int rational;
-    int pt_type;
-    int ncoords;
-    double a;
-    double x, y, z;
+    double a = 0.0;
+    double x = 0.0;
+    double y = 0.0;
+    double z = 0.0;
+    int degree = 0;
+    int entity_type = 0;
+    int i = 0;
+    int ncoords = 0;
+    int num_pts = 0;
+    int planar = 0;
+    int pt_type = 0;
+    int rational = 0;
+    point_t pt = VINIT_ZERO;
+    point_t pt2 = VINIT_ZERO;
 
     if (dir[entity_no]->param <= pstart) {
 	bu_log("Get_cnurb: Illegal parameter pointer for entity D%07d (%s)\n" ,
@@ -275,7 +274,7 @@ Get_cnurb(entity_no)
 	pt_type = RT_NURB_MAKE_PT_TYPE(ncoords, RT_NURB_PT_UV, RT_NURB_PT_NONRAT);
     }
 
-    crv = rt_nurb_new_cnurb(degree+1, num_pts+degree+1, num_pts, pt_type);
+    crv = nmg_nurb_new_cnurb(degree+1, num_pts+degree+1, num_pts, pt_type);
     /* knot vector */
     for (i = 0; i < num_pts+degree+1; i++) {
 	Readdbl(&a, "");
@@ -314,21 +313,16 @@ Get_cnurb(entity_no)
 
 
 void
-Assign_vu_geom(vu, u, v, srf)
-    struct vertexuse *vu;
-    fastf_t u, v;
-    struct face_g_snurb *srf;
+Assign_vu_geom(struct vertexuse *vu, fastf_t u, fastf_t v, struct face_g_snurb *srf)
 {
     point_t uvw;
-    hpoint_t pt_on_srf;
+    hpoint_t pt_on_srf = HINIT_ZERO;
     struct vertexuse *vu1;
     int moved = 0;
 
     NMG_CK_VERTEXUSE(vu);
     NMG_CK_SNURB(srf);
 
-    VSETALLN(pt_on_srf, 0.0, 4);
-
     if (u < srf->u.knots[0] || v < srf->v.knots[0] ||
 	u > srf->u.knots[srf->u.k_size-1] || v > srf->v.knots[srf->v.k_size-1]) {
 	bu_log("WARNING: UV point outside of domain of surface!:\n");
@@ -337,19 +331,13 @@ Assign_vu_geom(vu, u, v, srf)
 	       srf->u.knots[0], srf->v.knots[0],
 	       srf->u.knots[srf->u.k_size-1], srf->v.knots[srf->v.k_size-1]);
 
-	if (u < srf->u.knots[0])
-	    u = srf->u.knots[0];
-	if (v < srf->v.knots[0])
-	    v = srf->v.knots[0];
-	if (u > srf->u.knots[srf->u.k_size-1])
-	    u = srf->u.knots[srf->u.k_size-1];
-	if (v > srf->v.knots[srf->v.k_size-1])
-	    v = srf->v.knots[srf->v.k_size-1];
+	CLAMP(u, srf->u.knots[0], srf->u.knots[srf->u.k_size-1]);
+	CLAMP(v, srf->v.knots[0], srf->v.knots[srf->v.k_size-1]);
 
 	moved = 1;
     }
 
-    rt_nurb_s_eval(srf, u, v, pt_on_srf);
+    nmg_nurb_s_eval(srf, u, v, pt_on_srf);
     if (RT_NURB_IS_PT_RATIONAL(srf->pt_type)) {
 	fastf_t sca;
 
@@ -371,10 +359,7 @@ Assign_vu_geom(vu, u, v, srf)
 
 
 void
-Add_trim_curve(entity_no, lu, srf)
-    int entity_no;
-    struct loopuse *lu;
-    struct face_g_snurb *srf;
+Add_trim_curve(int entity_no, struct loopuse *lu, struct face_g_snurb *srf)
 {
     int entity_type;
     struct edge_g_cnurb *crv;
@@ -469,7 +454,7 @@ Add_trim_curve(entity_no, lu, srf)
 	    VSET(end, y+u_translation, x+v_translation, z);
 
 		/* build edge_g_cnurb arc */
-	    crv = rt_arc2d_to_cnurb(center, start, end, RT_NURB_PT_UV, &tol);
+	    crv = nmg_arc2d_to_cnurb(center, start, end, RT_NURB_PT_UV, &tol);
 
 	    /* apply transformation to control points */
 	    for (i = 0; i < crv->c_size; i++) {
@@ -498,7 +483,7 @@ Add_trim_curve(entity_no, lu, srf)
 	    /* Assign edge geometry */
 	    Assign_cnurb_to_eu(eu, crv);
 
-	    rt_nurb_free_cnurb(crv);
+	    nmg_nurb_free_cnurb(crv);
 	}
 	    break;
 
@@ -530,7 +515,7 @@ Add_trim_curve(entity_no, lu, srf)
 	    /* Assign edge geometry */
 	    Assign_cnurb_to_eu(eu, crv);
 
-	    rt_nurb_free_cnurb(crv);
+	    nmg_nurb_free_cnurb(crv);
 	    break;
 	default:
 	    bu_log("Curves of type %d are not yet handled for trimmed surfaces\n", entity_type);
@@ -540,20 +525,19 @@ Add_trim_curve(entity_no, lu, srf)
 
 
 struct loopuse *
-Make_trim_loop(entity_no, orientation, srf, fu)
-    int entity_no;
-    int orientation;
-    struct face_g_snurb *srf;
-    struct faceuse *fu;
+Make_trim_loop(int entity_no, int orientation, struct face_g_snurb *srf, struct faceuse *fu)
 {
     struct loopuse *lu;
     struct edgeuse *eu;
     struct edgeuse *new_eu;
     struct vertex *vp;
-    int entity_type;
-    int ncoords;
-    double x, y, z;
-    fastf_t u, v;
+    int entity_type = 0;
+    int ncoords = 0;
+    double x = 0.0;
+    double y = 0.0;
+    double z = 0.0;
+    fastf_t u = 0.0;
+    fastf_t v = 0.0;
     int i;
 
     NMG_CK_SNURB(srf);
@@ -590,7 +574,7 @@ Make_trim_loop(entity_no, orientation, srf, fu)
 
 	    /* if last EU is zero length, kill it */
 	    eu = BU_LIST_LAST(edgeuse, &lu->down_hd);
-	    if (bn_dist_pt3_pt3(eu->vu_p->v_p->vg_p->coord, eu->eumate_p->vu_p->v_p->vg_p->coord) < tol.dist)
+	    if (bn_dist_pnt3_pnt3(eu->vu_p->v_p->vg_p->coord, eu->eumate_p->vu_p->v_p->vg_p->coord) < tol.dist)
 		nmg_keu(eu);
 	    else {
 		bu_log("ERROR: composite trimming curve is not closed!\n");
@@ -622,11 +606,11 @@ Make_trim_loop(entity_no, orientation, srf, fu)
 	    VSET(end, x+u_translation, y+v_translation, z);
 
 	    /* build edge_g_cnurb circle */
-	    crv = rt_arc2d_to_cnurb(center, start, end, RT_NURB_PT_UV, &tol);
+	    crv = nmg_arc2d_to_cnurb(center, start, end, RT_NURB_PT_UV, &tol);
 
 	    /* split circle into two pieces */
 	    BU_LIST_INIT(&curv_hd);
-	    rt_nurb_c_split(&curv_hd, crv);
+	    nmg_nurb_c_split(&curv_hd, crv);
 	    crv1 = BU_LIST_FIRST(edge_g_cnurb, &curv_hd);
 	    crv2 = BU_LIST_LAST(edge_g_cnurb, &curv_hd);
 
@@ -654,9 +638,9 @@ Make_trim_loop(entity_no, orientation, srf, fu)
 	    Assign_cnurb_to_eu(eu, crv1);
 	    Assign_cnurb_to_eu(new_eu, crv2);
 
-	    rt_nurb_free_cnurb(crv);
-	    rt_nurb_free_cnurb(crv1);
-	    rt_nurb_free_cnurb(crv2);
+	    nmg_nurb_free_cnurb(crv);
+	    nmg_nurb_free_cnurb(crv1);
+	    nmg_nurb_free_cnurb(crv2);
 	}
 	    break;
 
@@ -677,7 +661,7 @@ Make_trim_loop(entity_no, orientation, srf, fu)
 
 	    /* split circle into two pieces */
 	    BU_LIST_INIT(&curv_hd);
-	    rt_nurb_c_split(&curv_hd, crv);
+	    nmg_nurb_c_split(&curv_hd, crv);
 	    crv1 = BU_LIST_FIRST(edge_g_cnurb, &curv_hd);
 	    crv2 = BU_LIST_LAST(edge_g_cnurb, &curv_hd);
 	    /* Split last edge in loop */
@@ -715,9 +699,9 @@ Make_trim_loop(entity_no, orientation, srf, fu)
 	    Assign_cnurb_to_eu(eu, crv1);
 	    Assign_cnurb_to_eu(new_eu, crv2);
 
-	    rt_nurb_free_cnurb(crv);
-	    rt_nurb_free_cnurb(crv1);
-	    rt_nurb_free_cnurb(crv2);
+	    nmg_nurb_free_cnurb(crv);
+	    nmg_nurb_free_cnurb(crv1);
+	    nmg_nurb_free_cnurb(crv2);
 	}
 	    break;
 	default:
@@ -729,17 +713,14 @@ Make_trim_loop(entity_no, orientation, srf, fu)
 
 
 struct loopuse *
-Make_loop(entity_no, orientation, on_surf_de, srf, fu)
-    int entity_no;
-    int orientation;
-    int on_surf_de;
-    struct face_g_snurb *srf;
-    struct faceuse *fu;
+Make_loop(int entity_no, int orientation, int on_surf_de, struct face_g_snurb *srf, struct faceuse *fu)
 {
     struct loopuse *lu;
-    int entity_type;
-    int surf_de, param_curve_de, model_curve_de;
-    int i;
+    int entity_type = 0;
+    int surf_de = 0;
+    int param_curve_de = 0;
+    int model_curve_de = 0;
+    int i = 0;
 
     NMG_CK_SNURB(srf);
     NMG_CK_FACEUSE(fu);
@@ -772,9 +753,7 @@ Make_loop(entity_no, orientation, on_surf_de, srf, fu)
 
 
 struct loopuse *
-Make_default_loop(srf, fu)
-    struct face_g_snurb *srf;
-    struct faceuse *fu;
+Make_default_loop(struct face_g_snurb *srf, struct faceuse *fu)
 {
     struct loopuse *lu;
     struct edgeuse *eu;
@@ -893,9 +872,7 @@ Make_default_loop(srf, fu)
 
 
 void
-Assign_surface_to_fu(fu, srf)
-    struct faceuse *fu;
-    struct face_g_snurb *srf;
+Assign_surface_to_fu(struct faceuse *fu, struct face_g_snurb *srf)
 {
     struct face *f;
 
@@ -918,9 +895,7 @@ Assign_surface_to_fu(fu, srf)
 
 
 struct faceuse *
-trim_surf(entityno, s)
-    int entityno;
-    struct shell *s;
+trim_surf(int entityno, struct shell *s)
 {
     struct model *m;
     struct face_g_snurb *srf;
@@ -928,8 +903,8 @@ trim_surf(entityno, s)
     struct loopuse *lu;
     struct loopuse *kill_lu;
     struct vertex *verts[3];
-    int entity_type;
-    int surf_de;
+    int entity_type = 0;
+    int surf_de = 0;
     int has_outer_boundary, inner_loop_count, outer_loop;
     int *inner_loop = NULL;
     int i;
@@ -940,11 +915,6 @@ trim_surf(entityno, s)
     m = nmg_find_model(&s->l.magic);
     NMG_CK_MODEL(m);
 
-    if (bu_debug & BU_DEBUG_MEM_CHECK) {
-	bu_log("barriercheck at start of trim_surf():\n");
-	bu_mem_barriercheck();
-    }
-
     /* Acquiring Data */
     if (dir[entityno]->param <= pstart) {
 	bu_log("Illegal parameter pointer for entity D%07d (%s)\n" ,
@@ -981,52 +951,22 @@ trim_surf(entityno, s)
     for (i = 0; i < 3; i++)
 	verts[i] = (struct vertex *)NULL;
 
-    if (bu_debug & BU_DEBUG_MEM_CHECK) {
-	bu_log("barriercheck before making face:\n");
-	bu_mem_barriercheck();
-    }
-
     fu = nmg_cface(s, verts, 3);
     Assign_surface_to_fu(fu, srf);
 
     kill_lu = BU_LIST_FIRST(loopuse, &fu->lu_hd);
 
     if (!has_outer_boundary) {
-	if (bu_debug & BU_DEBUG_MEM_CHECK) {
-	    bu_log("barriercheck before making default loop():\n");
-	    bu_mem_barriercheck();
-	}
-
 	lu = Make_default_loop(srf, fu);
     } else {
-	if (bu_debug & BU_DEBUG_MEM_CHECK) {
-	    bu_log("barriercheck before Make_loop():\n");
-	    bu_mem_barriercheck();
-	}
-
 	lu = Make_loop((outer_loop-1)/2, OT_SAME, surf_de, srf, fu);
     }
 
-    if (bu_debug & BU_DEBUG_MEM_CHECK) {
-	bu_log("barriercheck after making loop:\n");
-	bu_mem_barriercheck();
-    }
-
-
     (void)nmg_klu(kill_lu);
 
     /* first loop is an outer loop, orientation must be OT_SAME */
-    if (bu_debug & BU_DEBUG_MEM_CHECK) {
-	bu_log("barriercheck before nmg_snurb_calc_lu_uv_orient(():\n");
-	bu_mem_barriercheck();
-	bu_log("check complete!\n");
-    }
 
     lu_uv_orient = nmg_snurb_calc_lu_uv_orient(lu);
-    if (bu_debug & BU_DEBUG_MEM_CHECK) {
-	bu_log("barriercheck after nmg_snurb_calc_lu_uv_orient(():\n");
-	bu_mem_barriercheck();
-    }
     if (lu_uv_orient == OT_SAME)
 	nmg_set_lu_orientation(lu, 0);
     else {
@@ -1062,9 +1002,7 @@ trim_surf(entityno, s)
 
 
 int
-uv_in_fu(u, v, fu)
-    fastf_t u, v;
-    struct faceuse *fu;
+uv_in_fu(fastf_t u, fastf_t v, struct faceuse *fu)
 {
     int ot_sames, ot_opps;
     struct loopuse *lu;
@@ -1102,14 +1040,10 @@ uv_in_fu(u, v, fu)
  * in direction "ray_dir". Place hits on "hit_list"
  */
 void
-find_intersections(fu, mid_pt, ray_dir, hit_list)
-    struct faceuse *fu;
-    point_t mid_pt;
-    vect_t ray_dir;
-    struct bu_list *hit_list;
+find_intersections(struct faceuse *fu, point_t mid_pt, vect_t ray_dir, struct bu_list *hit_list)
 {
     plane_t pl1, pl2;
-    struct bu_list bezier;
+    struct bu_list bezier_l;
     struct face *f;
     struct face_g_snurb *fg;
 
@@ -1127,34 +1061,34 @@ find_intersections(fu, mid_pt, ray_dir, hit_list)
     pl1[W] = VDOT(mid_pt, pl1);
     pl2[W] = VDOT(mid_pt, pl2);
 
-    BU_LIST_INIT(&bezier);
+    BU_LIST_INIT(&bezier_l);
 
-    rt_nurb_bezier(&bezier, fg, (struct resource *)NULL);
-    while (BU_LIST_NON_EMPTY(&bezier)) {
+    nmg_nurb_bezier(&bezier_l, fg);
+    while (BU_LIST_NON_EMPTY(&bezier_l)) {
 	struct face_g_snurb *srf;
-	struct rt_nurb_uv_hit *hp;
+	struct nmg_nurb_uv_hit *hp;
 
-	srf = BU_LIST_FIRST(face_g_snurb,  &bezier);
+	srf = BU_LIST_FIRST(face_g_snurb,  &bezier_l);
 	BU_LIST_DEQUEUE(&srf->l);
 
-	hp = rt_nurb_intersect(srf, pl1, pl2, UV_TOL, (struct resource *)NULL, NULL);
+	hp = nmg_nurb_intersect(srf, pl1, pl2, UV_TOL, NULL);
 	/* process each hit point */
-	while (hp != (struct rt_nurb_uv_hit *)NULL) {
-	    struct rt_nurb_uv_hit *next;
+	while (hp != (struct nmg_nurb_uv_hit *)NULL) {
+	    struct nmg_nurb_uv_hit *next;
 	    struct snurb_hit *myhit;
 	    vect_t to_hit;
 	    fastf_t homo_hit[4];
 
 	    next = hp->next;
 
-	    if (RTG.NMG_debug & DEBUG_RT_ISECT)
+	    if (nmg_debug & NMG_DEBUG_RT_ISECT)
 		bu_log("\tintersect snurb surface at uv=(%g %g)\n", hp->u, hp->v);
 
 	    /* check if point is in face (trimming curves) */
 	    if (!uv_in_fu(hp->u, hp->v, fu)) {
 		/* not a hit */
 
-		if (RTG.NMG_debug & DEBUG_RT_ISECT)
+		if (nmg_debug & NMG_DEBUG_RT_ISECT)
 		    bu_log("\tNot a hit\n");
 
 		bu_free((char *)hp, "nurb_uv_hit");
@@ -1167,7 +1101,7 @@ find_intersections(fu, mid_pt, ray_dir, hit_list)
 	    myhit->f = f;
 
 	    /* calculate actual hit point (x y z) */
-	    rt_nurb_s_eval(srf, hp->u, hp->v, homo_hit);
+	    nmg_nurb_s_eval(srf, hp->u, hp->v, homo_hit);
 	    if (RT_NURB_IS_PT_RATIONAL(srf->pt_type)) {
 		fastf_t inv_homo;
 
@@ -1181,7 +1115,7 @@ find_intersections(fu, mid_pt, ray_dir, hit_list)
 	    myhit->dist = VDOT(to_hit, ray_dir);
 
 	    /* get surface normal */
-	    rt_nurb_s_norm(srf, hp->u, hp->v, myhit->norm);
+	    nmg_nurb_s_norm(srf, hp->u, hp->v, myhit->norm);
 
 	    /* may need to reverse it */
 	    if (f->flip) {
@@ -1208,16 +1142,14 @@ find_intersections(fu, mid_pt, ray_dir, hit_list)
 	    bu_free((char *)hp, "nurb_uv_hit");
 	    hp = next;
 	}
-	rt_nurb_free_snurb(srf, (struct resource *)NULL);
+	nmg_nurb_free_snurb(srf);
     }
 }
 
 
 /* adjust flip flag on faces using hit list data */
 void
-adjust_flips(hit_list, ray_dir)
-    struct bu_list *hit_list;
-    vect_t ray_dir;
+adjust_flips(struct bu_list *hit_list, vect_t ray_dir)
 {
     struct snurb_hit *hit;
     int enter = 0;
@@ -1250,9 +1182,7 @@ adjust_flips(hit_list, ray_dir)
 
 /* Find a uv point that is actually in the face */
 int
-Find_uv_in_fu(u_in, v_in, fu)
-    fastf_t *u_in, *v_in;
-    struct faceuse *fu;
+Find_uv_in_fu(fastf_t *u_in, fastf_t *v_in, struct faceuse *fu)
 {
     struct face *f;
     struct face_g_snurb *fg;
@@ -1368,7 +1298,7 @@ Find_pt_in_fu(struct faceuse *fu, point_t pt, vect_t norm)
 	return 1;
 
     /* calculate actual hit point (x y z) */
-    rt_nurb_s_eval(fg, u, v, homo_hit);
+    nmg_nurb_s_eval(fg, u, v, homo_hit);
     if (RT_NURB_IS_PT_RATIONAL(fg->pt_type)) {
 	fastf_t inv_homo;
 
@@ -1379,7 +1309,7 @@ Find_pt_in_fu(struct faceuse *fu, point_t pt, vect_t norm)
     }
 
     /* get surface normal */
-    rt_nurb_s_norm(fg, u, v, norm);
+    nmg_nurb_s_norm(fg, u, v, norm);
 
     /* may need to reverse it */
     if (f->flip) {
@@ -1394,7 +1324,9 @@ void
 Convtrimsurfs()
 {
 
-    int i, convsurf = 0, totsurfs = 0;
+    size_t i;
+    size_t convsurf = 0;
+    size_t totsurfs = 0;
     struct model *m;
     struct nmgregion *r;
     struct shell *s;
@@ -1403,49 +1335,37 @@ Convtrimsurfs()
 
     bu_log("\n\nConverting Trimmed Surface entities:\n");
 
-    if (RT_G_DEBUG & DEBUG_MEM_FULL)
-	bu_mem_barriercheck();
-
     m = nmg_mm();
     r = nmg_mrsv(m);
     s = BU_LIST_FIRST(shell, &r->s_hd);
 
     for (i = 0; i < totentities; i++) {
 	if (dir[i]->type == 144) {
-	    if (RT_G_DEBUG & DEBUG_MEM_FULL)
-		bu_mem_barriercheck();
-
 	    totsurfs++;
 	    fu = trim_surf(i, s);
 	    if (fu) {
 		nmg_face_bb(fu->f_p, &tol);
 		convsurf++;
 	    }
-	    if (RT_G_DEBUG & DEBUG_MEM_FULL)
-		bu_mem_barriercheck();
-
 	}
     }
 
     nmg_rebound(m, &tol);
 
-    bu_log("\n\t%d surfaces converted, adjusting surface normals....\n", convsurf);
+    bu_log("\n\t%zu surfaces converted, adjusting surface normals....\n", convsurf);
 
     /* do some raytracing to get face orientations correct */
     for (BU_LIST_FOR(fu, faceuse, &s->fu_hd)) {
 	struct faceuse *fu2;
-	point_t mid_pt;
+	point_t mid_pt = VINIT_ZERO;
 	vect_t ray_dir;
 
-	/* initialize for compiler */
-	VSETALL(mid_pt, 0.0);
-
 	if (fu->orientation != OT_SAME)
 	    continue;
 
 	BU_LIST_INIT(&hit_list);
 	if (Find_pt_in_fu(fu, mid_pt, ray_dir)) {
-	    bu_log("Convtrimsurfs: Cannot find a point in fu (%p)\n", fu);
+	    bu_log("Convtrimsurfs: Cannot find a point in fu (%p)\n", (void *)fu);
 	    nmg_pr_fu(fu, " ");
 	    bu_exit(1, "Convtrimsurfs: Cannot find a point in fu\n");
 	}
@@ -1474,29 +1394,20 @@ Convtrimsurfs()
 
     }
 
-    bu_log("Converted %d Trimmed Surfaces successfully out of %d total Trimmed Surfaces\n", convsurf, totsurfs);
-
-    if (RT_G_DEBUG & DEBUG_MEM_FULL)
-	bu_mem_barriercheck();
+    bu_log("Converted %zu Trimmed Surfaces successfully out of %zu total Trimmed Surfaces\n", convsurf, totsurfs);
 
     if (convsurf) {
-	(void)nmg_vertex_fuse(&m->magic, &tol);
+	(void)nmg_vertex_fuse(&m->magic, &RTG.rtg_vlfree, &tol);
 
 	if (!BU_STR_EMPTY(curr_file->obj_name))
 	    mk_nmg(fdout, curr_file->obj_name, m);
 	else
 	    mk_nmg(fdout, "Trimmed_surf", m);
     }
-    if (RT_G_DEBUG & DEBUG_MEM_FULL)
-	bu_mem_barriercheck();
 
     if (!convsurf) {
 	nmg_km(m);
     }
-
-    if (RT_G_DEBUG & DEBUG_MEM_FULL)
-	bu_mem_barriercheck();
-
 }
 
 
