@@ -1,7 +1,7 @@
 /*                         S T L - G . C
  * BRL-CAD
  *
- * Copyright (c) 2002-2013 United States Government as represented by
+ * Copyright (c) 2002-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This program is free software; you can redistribute it and/or
@@ -34,16 +34,20 @@
 #include <string.h>
 #include <ctype.h>
 #include <errno.h>
+#include "bnetwork.h"
 #include "bio.h"
-#include "bin.h"
 
+#include "bu/app.h"
+#include "bu/cv.h"
+#include "bu/getopt.h"
+#include "bu/units.h"
 #include "vmath.h"
 #include "nmg.h"
-#include "rtgeom.h"
+#include "rt/geom.h"
 #include "raytrace.h"
 #include "wdb.h"
 
-static struct vert_root *tree_root;
+static struct bn_vert_tree *tree;
 static struct wmember all_head;
 static char *input_file;	/* name of the input file */
 static char *brlcad_file;	/* name of output file */
@@ -59,7 +63,7 @@ static FILE *fd_in;		/* input file */
 static struct rt_wdb *fd_out;	/* Resulting BRL-CAD file */
 static float conv_factor=1.0;	/* conversion factor from model units to mm */
 static unsigned int obj_count=0; /* Count of parts converted for "stl-g" conversions */
-static int *bot_faces=NULL;	 /* array of ints (indices into tree_root->the_array array) three per face */
+static int *bot_faces=NULL;	 /* array of ints (indices into tree->the_array array) three per face */
 static int bot_fsize=0;		/* current size of the bot_faces array */
 static int bot_fcurr=0;		/* current bot face */
 
@@ -108,7 +112,7 @@ mk_unique_brlcad_name(struct bu_vls *name)
 {
     char *c;
     int count=0;
-    int len;
+    size_t len;
 
     c = bu_vls_addr(name);
 
@@ -147,16 +151,6 @@ Convert_part_ascii(char line[MAX_LINE_SIZE])
     vect_t normal={0, 0, 0};
     int solid_in_region=0;
 
-    if (RT_G_DEBUG & DEBUG_MEM_FULL)
-	bu_prmem("At start of Convert_part_ascii():\n");
-
-    if (RT_G_DEBUG & DEBUG_MEM_FULL) {
-	bu_log("Barrier check at start of Convert_part_ascii:\n");
-	if (bu_mem_barriercheck())
-	    bu_exit(EXIT_FAILURE, "Barrier check failed!\n");
-    }
-
-
     bot_fcurr = 0;
     BU_LIST_INIT(&head.l);
 
@@ -222,9 +216,6 @@ Convert_part_ascii(char line[MAX_LINE_SIZE])
 
     bu_log("\tUsing solid name: %s\n", bu_vls_addr(&solid_name));
 
-    if (RT_G_DEBUG & DEBUG_MEM || RT_G_DEBUG & DEBUG_MEM_FULL)
-	bu_prmem("At start of Convert_part_ascii()");
-
     while (bu_fgets(line1, MAX_LINE_SIZE, fd_in) != NULL) {
 	start = (-1);
 	while (isspace((int)line1[++start]));
@@ -279,14 +270,14 @@ Convert_part_ascii(char line[MAX_LINE_SIZE])
 
 			bu_log("Non-triangular loop:\n");
 			for (n=0; n<3; n++)
-			    bu_log("\t(%g %g %g)\n", V3ARGS(&tree_root->the_array[tmp_face[n]]));
+			    bu_log("\t(%g %g %g)\n", V3ARGS(&tree->the_array[tmp_face[n]]));
 
 			bu_log("\t(%g %g %g)\n", x, y, z);
 		    }
 		    x *= conv_factor;
 		    y *= conv_factor;
 		    z *= conv_factor;
-		    tmp_face[vert_no++] = Add_vert(x, y, z, tree_root, tol.dist_sq);
+		    tmp_face[vert_no++] = bn_vert_tree_add(tree, x, y, z, tol.dist_sq);
 		} else {
 		    bu_log("Unrecognized line: %s\n", line1);
 		}
@@ -313,7 +304,7 @@ Convert_part_ascii(char line[MAX_LINE_SIZE])
 
 		bu_log("Making Face:\n");
 		for (n=0; n<3; n++)
-		    bu_log("\tvertex #%d: (%g %g %g)\n", tmp_face[n], V3ARGS(&tree_root->the_array[3*tmp_face[n]]));
+		    bu_log("\tvertex #%d: (%g %g %g)\n", tmp_face[n], V3ARGS(&tree->the_array[3*tmp_face[n]]));
 		VPRINT(" normal", normal);
 	    }
 
@@ -336,9 +327,9 @@ Convert_part_ascii(char line[MAX_LINE_SIZE])
 	    bu_log("\t%d faces were degenerate\n", degenerate_count);
     }
 
-    mk_bot(fd_out, bu_vls_addr(&solid_name), RT_BOT_SOLID, RT_BOT_UNORIENTED, 0, tree_root->curr_vert, bot_fcurr,
-	   tree_root->the_array, bot_faces, NULL, NULL);
-    clean_vert_tree(tree_root);
+    mk_bot(fd_out, bu_vls_addr(&solid_name), RT_BOT_SOLID, RT_BOT_UNORIENTED, 0, tree->curr_vert, bot_fcurr,
+	   tree->the_array, bot_faces, NULL, NULL);
+    bn_vert_tree_clean(tree);
 
     if (face_count && !solid_in_region) {
 	(void)mk_addmember(bu_vls_addr(&solid_name), &head.l, NULL, WMOP_UNION);
@@ -350,24 +341,15 @@ Convert_part_ascii(char line[MAX_LINE_SIZE])
 	mk_lrcomb(fd_out, bu_vls_addr(&region_name), &head, 1, (char *)NULL,
 		  (char *)NULL, color, const_id, 0, mat_code, 100, 0);
 	if (face_count) {
-	    (void)mk_addmember(bu_vls_addr(&region_name), &all_head.l,
-			       NULL, WMOP_UNION);
+	    (void)mk_addmember(bu_vls_addr(&region_name), &all_head.l, NULL, WMOP_UNION);
 	}
     } else {
-	mk_lrcomb(fd_out, bu_vls_addr(&region_name), &head, 1, (char *)NULL,
-		  (char *)NULL, color, id_no, 0, mat_code, 100, 0);
+	mk_lrcomb(fd_out, bu_vls_addr(&region_name), &head, 1, (char *)NULL, (char *)NULL, color, id_no, 0, mat_code, 100, 0);
 	if (face_count)
-	    (void)mk_addmember(bu_vls_addr(&region_name), &all_head.l,
-			       NULL, WMOP_UNION);
+	    (void)mk_addmember(bu_vls_addr(&region_name), &all_head.l, NULL, WMOP_UNION);
 	id_no++;
     }
 
-    if (RT_G_DEBUG & DEBUG_MEM_FULL) {
-	bu_log("Barrier check at end of Convert_part_ascii:\n");
-	if (bu_mem_barriercheck())
-	    bu_exit(EXIT_FAILURE, "Barrier check failed!\n");
-    }
-
     bu_vls_free(&region_name);
     bu_vls_free(&solid_name);
 
@@ -441,11 +423,11 @@ Convert_part_binary()
 
 	VMOVE(normal, flts);
 	VSCALE(pt, &flts[3], conv_factor);
-	tmp_face[0] = Add_vert(V3ARGS(pt), tree_root, tol.dist_sq);
+	tmp_face[0] = bn_vert_tree_add(tree, V3ARGS(pt), tol.dist_sq);
 	VSCALE(pt, &flts[6], conv_factor);
-	tmp_face[1] = Add_vert(V3ARGS(pt), tree_root, tol.dist_sq);
+	tmp_face[1] = bn_vert_tree_add(tree, V3ARGS(pt), tol.dist_sq);
 	VSCALE(pt, &flts[9], conv_factor);
-	tmp_face[2] = Add_vert(V3ARGS(pt), tree_root, tol.dist_sq);
+	tmp_face[2] = bn_vert_tree_add(tree, V3ARGS(pt), tol.dist_sq);
 
 	/* check for degenerate faces */
 	if (tmp_face[0] == tmp_face[1]) {
@@ -468,7 +450,7 @@ Convert_part_binary()
 
 	    bu_log("Making Face:\n");
 	    for (n=0; n<3; n++)
-		bu_log("\tvertex #%d: (%g %g %g)\n", tmp_face[n], V3ARGS(&tree_root->the_array[3*tmp_face[n]]));
+		bu_log("\tvertex #%d: (%g %g %g)\n", tmp_face[n], V3ARGS(&tree->the_array[3*tmp_face[n]]));
 	    VPRINT(" normal", normal);
 	}
 
@@ -488,8 +470,8 @@ Convert_part_binary()
     }
 
     mk_bot(fd_out, bu_vls_addr(&solid_name), RT_BOT_SOLID, RT_BOT_UNORIENTED, 0,
-	   tree_root->curr_vert, bot_fcurr, tree_root->the_array, bot_faces, NULL, NULL);
-    clean_vert_tree(tree_root);
+	   tree->curr_vert, bot_fcurr, tree->the_array, bot_faces, NULL, NULL);
+    bn_vert_tree_clean(tree);
 
     BU_LIST_INIT(&head.l);
     if (face_count) {
@@ -501,24 +483,15 @@ Convert_part_binary()
 	mk_lrcomb(fd_out, bu_vls_addr(&region_name), &head, 1, (char *)NULL,
 		  (char *)NULL, NULL, const_id, 0, mat_code, 100, 0);
 	if (face_count) {
-	    (void)mk_addmember(bu_vls_addr(&region_name), &all_head.l,
-			       NULL, WMOP_UNION);
+	    (void)mk_addmember(bu_vls_addr(&region_name), &all_head.l, NULL, WMOP_UNION);
 	}
     } else {
-	mk_lrcomb(fd_out, bu_vls_addr(&region_name), &head, 1, (char *)NULL,
-		  (char *)NULL, NULL, id_no, 0, mat_code, 100, 0);
+	mk_lrcomb(fd_out, bu_vls_addr(&region_name), &head, 1, (char *)NULL, (char *)NULL, NULL, id_no, 0, mat_code, 100, 0);
 	if (face_count)
-	    (void)mk_addmember(bu_vls_addr(&region_name), &all_head.l,
-			       NULL, WMOP_UNION);
+	    (void)mk_addmember(bu_vls_addr(&region_name), &all_head.l, NULL, WMOP_UNION);
 	id_no++;
     }
 
-    if (RT_G_DEBUG & DEBUG_MEM_FULL) {
-	bu_log("Barrier check at end of Convert_part_ascii:\n");
-	if (bu_mem_barriercheck())
-	    bu_exit(EXIT_FAILURE, "Barrier check failed!\n");
-    }
-
     return;
 }
 
@@ -556,14 +529,13 @@ Convert_input()
 }
 
 
-/*
- *			M A I N
- */
 int
 main(int argc, char *argv[])
 {
     int c;
 
+    bu_setprogname(argv[0]);
+
     tol.magic = BN_TOL_MAGIC;
 
     /* this value selected as a reasonable compromise between eliminating
@@ -632,8 +604,8 @@ main(int argc, char *argv[])
 		debug = 1;
 		break;
 	    case 'x':
-		sscanf(bu_optarg, "%x", (unsigned int *)&RTG.debug);
-		bu_printb("librt RT_G_DEBUG", RT_G_DEBUG, DEBUG_FORMAT);
+		sscanf(bu_optarg, "%x", (unsigned int *)&rt_debug);
+		bu_printb("librt RT_G_DEBUG", RT_G_DEBUG, RT_DEBUG_FORMAT);
 		bu_log("\n");
 		break;
 	    default:
@@ -643,8 +615,6 @@ main(int argc, char *argv[])
 	}
     }
 
-    rt_init_resource(&rt_uniresource, 0, NULL);
-
     input_file = argv[bu_optind];
     if ((fd_in=fopen(input_file, "rb")) == NULL) {
 	bu_log("Cannot open input file (%s)\n", input_file);
@@ -664,7 +634,7 @@ main(int argc, char *argv[])
     BU_LIST_INIT(&all_head.l);
 
     /* create a tree structure to hold the input vertices */
-    tree_root = create_vert_tree();
+    tree = bn_vert_tree_create();
 
     Convert_input();
 
