@@ -1,7 +1,7 @@
 /*                          C O N V . H
  * BRL-CAD
  *
- * Copyright (c) 2004-2013 United States Government as represented by
+ * Copyright (c) 2004-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This program is free software; you can redistribute it and/or
@@ -21,6 +21,9 @@
  *
  */
 
+#ifndef EXTERNAL_UNIGRAPHICS_CONV_H
+#define EXTERNAL_UNIGRAPHICS_CONV_H
+
 /* default indentation for a single level in the DAG */
 #define LEVEL_INDENT 6
 #define FLAG_SUPPRESS		0
@@ -42,7 +45,9 @@ typedef struct ug_tol {
 extern ug_tol ugtol;
 
 extern int debug;
-#define dprintf if (debug) printf
+#define ugdprintf if (debug) printf
+
+#endif /* EXTERNAL_UNIGRAPHICS_CONV_H */
 
 /*
  * Local Variables:
