@@ -1,7 +1,7 @@
 /*                           L O G . H
  * BRL-CAD
  *
- * Copyright (c) 2004-2013 United States Government as represented by
+ * Copyright (c) 2004-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This program is free software; you can redistribute it and/or
@@ -18,14 +18,19 @@
  * information.
  */
 
+#ifndef EXTERNAL_UNIGRAPHICS_LOG_H
+#define EXTERNAL_UNIGRAPHICS_LOG_H
+
 #include "common.h"
 
 #include <stdarg.h>
 
-extern void log(const char *fmt, ...);
+/*extern void log(const char *fmt, ...);*/
 extern void dialog(const char *fmt, ...);
 extern void log_init(void);
 
+#endif /* EXTERNAL_UNIGRAPHICS_LOG_H */
+
 /*
  * Local Variables:
  * mode: C
