@@ -1,7 +1,7 @@
 /*                        F B C M A P . C
  * BRL-CAD
  *
- * Copyright (c) 1986-2013 United States Government as represented by
+ * Copyright (c) 1986-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This program is free software; you can redistribute it and/or
@@ -29,10 +29,14 @@
 #include "common.h"
 
 #include <stdlib.h>
+
 #include "bio.h"
 
-#include "bu.h"
-#include "fb.h"
+#include "bu/app.h"
+#include "bu/getopt.h"
+#include "bu/log.h"
+#include "bu/snooze.h"
+#include "dm.h"
 #include "pkg.h"
 
 
@@ -584,8 +588,6 @@ unsigned char utah9[256*3] = {
 };
 
 
-/* p a r s _ A r g v ()
- */
 static int
 pars_Argv(int argc, char **argv)
 {
@@ -653,7 +655,12 @@ main(int argc, char **argv)
     int fudge;
     ColorMap cmap;
     ColorMap *cp = &cmap;
-    FBIO *fbp;
+    struct fb *fbp;
+
+    bu_setprogname(argv[0]);
+
+    bu_log("DEPRECATION WARNING:  This command is scheduled for removal.  Please contact the developers if you use this command.\n\n");
+    bu_snooze(BU_SEC2USEC(1));
 
     if (argc == 1 && isatty(fileno(stdin)) && isatty(fileno(stdout)))
 	usage();
