@@ -1,7 +1,7 @@
 /*                     F B S T R E T C H . C
  * BRL-CAD
  *
- * Copyright (c) 2004-2013 United States Government as represented by
+ * Copyright (c) 2004-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This program is free software; you can redistribute it and/or
@@ -76,10 +76,16 @@
 #include <signal.h>
 #include <string.h>
 #include <stdarg.h>
+
 #include "bio.h"
 
-#include "bu.h"
-#include "fb.h"			/* BRL-CAD package libfb.a interface */
+#include "bu/app.h"
+#include "bu/color.h"
+#include "bu/getopt.h"
+#include "bu/getopt.h"
+#include "bu/log.h"
+#include "bu/str.h"
+#include "dm.h"			/* BRL-CAD package libfb.a interface */
 
 
 #define USAGE1 "fbstretch [ -s size ] [ -w width ] [ -n height ]"
@@ -94,12 +100,12 @@ static bool_t sample = 0;		/* set: sampling; clear: averaging */
 static bool_t verbose = 0;		/* set for size info printout */
 static float x_scale = -1.0;		/* horizontal scaling factor */
 static float y_scale = -1.0;		/* vertical scaling factor */
-static bool_t x_compress;		/* set iff compressing horizontally */
-static bool_t y_compress;		/* set iff compressing vertically */
+static bool_t x_compress;		/* set if compressing horizontally */
+static bool_t y_compress;		/* set if compressing vertically */
 static char *src_file = NULL;		/* source frame buffer name */
-static FBIO *src_fbp = FBIO_NULL;	/* source frame buffer handle */
+static struct fb *src_fbp = FB_NULL;	/* source frame buffer handle */
 static char *dst_file = NULL;		/* destination frame buffer name */
-static FBIO *dst_fbp = FBIO_NULL;	/* destination frame buffer handle */
+static struct fb *dst_fbp = FB_NULL;	/* destination frame buffer handle */
 static int src_width = 512;
 static int src_height = 512;		/* source image size */
 static int dst_width = 0;
@@ -109,23 +115,23 @@ static unsigned char *dst_buf;		/* calloc()ed output scan line buffer */
 
 /* in ioutil.c */
 extern void Message(const char *format, ...);
-extern void Fatal(FBIO *fbiop, const char *format, ...);
+extern void Fatal(struct fb *fbiop, const char *format, ...);
 
 
 static void
 Stretch_Fatal(const char *str)
 {
-    if (src_fbp != FBIO_NULL && fb_close(src_fbp) == -1) {
+    if (src_fbp != FB_NULL && fb_close(src_fbp) == -1) {
 	Message("Error closing input frame buffer");
-	src_fbp = FBIO_NULL;
+	src_fbp = FB_NULL;
     }
 
-    if (dst_fbp != FBIO_NULL && fb_close(dst_fbp) == -1) {
+    if (dst_fbp != FB_NULL && fb_close(dst_fbp) == -1) {
 	Message("Error closing output frame buffer");
-	src_fbp = FBIO_NULL;
+	src_fbp = FB_NULL;
     }
 
-    Fatal(FBIO_NULL, "%s", str);
+    Fatal(FB_NULL, "%s", str);
     /* NOT REACHED */
 }
 
@@ -142,6 +148,8 @@ Sig_Catcher(int sig)
 int
 main(int argc, char **argv)
 {
+    bu_setprogname(argv[0]);
+
     /* Plant signal catcher. */
     {
 	static int getsigs[] = {
@@ -185,11 +193,11 @@ main(int argc, char **argv)
 		    sample = 1;
 		    break;
 
-		case 'f':	/* -f in_fb */
+		case 'f':	/* -f in_struct fb */
 		    src_file = bu_optarg;
 		    break;
 
-		case 'F':	/* -F out_fb */
+		case 'F':	/* -F out_struct fb */
 		    dst_file = bu_optarg;
 		    break;
 
@@ -298,7 +306,7 @@ main(int argc, char **argv)
     if ((src_fbp = fb_open(src_file == NULL ? dst_file : src_file,
 			   src_width, src_height
 	     )
-	    ) == FBIO_NULL
+	    ) == FB_NULL
 	)
 	Stretch_Fatal("Couldn't open input image");
     else {
@@ -331,7 +339,7 @@ main(int argc, char **argv)
 	    )
 	    dst_fbp = src_fbp;	/* No No No Not a Second Time */
 	else if ((dst_fbp = fb_open(dst_file, dst_width, dst_height))
-		 == FBIO_NULL
+		 == FB_NULL
 	    )
 	    Stretch_Fatal("Couldn't open output frame buffer");
 
@@ -791,7 +799,7 @@ main(int argc, char **argv)
 done:
     /* Close the frame buffers. */
 
-    assert(src_fbp != FBIO_NULL && dst_fbp != FBIO_NULL);
+    assert(src_fbp != FB_NULL && dst_fbp != FB_NULL);
 
     if (fb_close(src_fbp) == -1)
 	Message("Error closing input frame buffer");
