@@ -1,7 +1,7 @@
 /*                        S E R V E R . C
  * BRL-CAD
  *
- * Copyright (c) 1998-2013 United States Government as represented by
+ * Copyright (c) 1998-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This library is free software; you can redistribute it and/or
@@ -26,7 +26,6 @@
 #include "common.h"
 
 #include <stdlib.h>
-#include <stdio.h>
 #include <string.h>
 #if defined(HAVE_SYS_TYPES_H)
 #  include <sys/types.h>
@@ -34,11 +33,12 @@
 #if defined(HAVE_SYS_TIME_H)
 #  include <sys/time.h>
 #endif
-#include "bselect.h"
-#include "bio.h"
+#include "bsocket.h"
 
-#include "fb.h"
-#include "fbmsg.h"
+#include "bu/color.h"
+#include "bu/log.h"
+#include "dm.h"
+#include "vmath.h"
 #include "pkg.h"
 
 
@@ -48,7 +48,7 @@
  * These are the only symbols intended for export to LIBFB users.
  */
 
-FBIO *fb_server_fbp = FBIO_NULL;
+struct fb *fb_server_fbp = FB_NULL;
 fd_set *fb_server_select_list;
 int *fb_server_max_fd = (int *)NULL;
 int fb_server_got_fb_free = 0;	/* !0 => we have received an fb_free */
@@ -57,8 +57,6 @@ int fb_server_retain_on_close = 0;	/* !0 => we are holding a reusable FB open */
 
 
 /*
- * F B _ S E R V E R _ G O T _ U N K N O W N
- *
  * This is where we go for message types we don't understand.
  *
  * The fb_log() might go to the ordinary LIBFB routine, or
@@ -66,17 +64,15 @@ int fb_server_retain_on_close = 0;	/* !0 => we are holding a reusable FB open */
  * which might send the message back across the wire.
  */
 static void
-fb_server_got_unknown(struct pkg_conn *pcp, char *buf)
+fb_server_fb_unknown(struct pkg_conn *pcp, char *buf)
 {
-    fb_log("fb_server_got_unknown: message type %d not part of remote LIBFB protocol, ignored.\n",
+    fb_log("fb_server_fb_unknown: message type %d not part of remote LIBFB protocol, ignored.\n",
 	   pcp->pkc_type);
     (void)free(buf);
 }
 
 
 /*
- * F B _ S E R V E R _ F B _ O P E N
- *
  * There can only be one framebuffer (fbp) open at any one time
  * (although that can be a stacker driving many actual windows),
  * but framebuffers can be opened and closed and opened again in the
@@ -89,13 +85,15 @@ fb_server_fb_open(struct pkg_conn *pcp, char *buf)
     char rbuf[5*NET_LONG_LEN+1];
     int want;
 
-    if (buf == NULL) return;
-    if (pcp == PKC_NULL) return;
+    if (buf == NULL)
+       	return;
+    if (pcp == PKC_NULL)
+       	return;
 
     width = pkg_glong(&buf[0*NET_LONG_LEN]);
     height = pkg_glong(&buf[1*NET_LONG_LEN]);
 
-    if (fb_server_fbp == FBIO_NULL) {
+    if (fb_server_fbp == FB_NULL) {
 	/* Attempt to open new framebuffer */
 	if (strlen(&buf[8]) == 0)
 	    fb_server_fbp = fb_open(NULL, width, height);
@@ -105,37 +103,32 @@ fb_server_fb_open(struct pkg_conn *pcp, char *buf)
 	/* Use existing framebuffer */
     }
 
-    if (fb_server_fbp == FBIO_NULL) {
+    if (fb_server_fbp == FB_NULL) {
 	(void)pkg_plong(&rbuf[0*NET_LONG_LEN], -1);	/* ret */
 	(void)pkg_plong(&rbuf[1*NET_LONG_LEN], 0);
 	(void)pkg_plong(&rbuf[2*NET_LONG_LEN], 0);
 	(void)pkg_plong(&rbuf[3*NET_LONG_LEN], 0);
 	(void)pkg_plong(&rbuf[4*NET_LONG_LEN], 0);
     } else {
+	int selfd = 0;
 	(void)pkg_plong(&rbuf[0*NET_LONG_LEN], 0);	/* ret */
-	(void)pkg_plong(&rbuf[1*NET_LONG_LEN], fb_server_fbp->if_max_width);
-	(void)pkg_plong(&rbuf[2*NET_LONG_LEN], fb_server_fbp->if_max_height);
-	(void)pkg_plong(&rbuf[3*NET_LONG_LEN], fb_server_fbp->if_width);
-	(void)pkg_plong(&rbuf[4*NET_LONG_LEN], fb_server_fbp->if_height);
-	if (fb_server_fbp->if_selfd > 0 && fb_server_select_list) {
-	    FD_SET(fb_server_fbp->if_selfd, fb_server_select_list);
-	    if (fb_server_max_fd != NULL &&
-		fb_server_fbp->if_selfd > *fb_server_max_fd)
-		*fb_server_max_fd = fb_server_fbp->if_selfd;
-	}
+	(void)pkg_plong(&rbuf[1*NET_LONG_LEN], fb_get_max_width(fb_server_fbp));
+	(void)pkg_plong(&rbuf[2*NET_LONG_LEN], fb_get_max_height(fb_server_fbp));
+	(void)pkg_plong(&rbuf[3*NET_LONG_LEN], fb_getwidth(fb_server_fbp));
+	(void)pkg_plong(&rbuf[4*NET_LONG_LEN], fb_getheight(fb_server_fbp));
+	selfd = fb_set_fd(fb_server_fbp, fb_server_select_list);
+	if (fb_server_max_fd != NULL && selfd > *fb_server_max_fd)
+	    *fb_server_max_fd = selfd;
     }
 
     want = 5*NET_LONG_LEN;
     if (pkg_send(MSG_RETURN, rbuf, want, pcp) != want)
 	fprintf(stderr, "pkg_send fb_open reply\n");
-    if (buf) (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
 
-/*
- * F B _ S E R V E R _ F B _ C L O S E
- *
- */
 static void
 fb_server_fb_close(struct pkg_conn *pcp, char *buf)
 {
@@ -149,24 +142,21 @@ fb_server_fb_close(struct pkg_conn *pcp, char *buf)
 	(void)fb_flush(fb_server_fbp);
 	(void)pkg_plong(&rbuf[0], 0);		/* return success */
     } else {
-	if (fb_server_fbp->if_selfd > 0 && fb_server_select_list) {
-	    FD_CLR(fb_server_fbp->if_selfd, fb_server_select_list);
-	}
+	(void)fb_clear_fd(fb_server_fbp, fb_server_select_list);
 	(void)pkg_plong(&rbuf[0], fb_close(fb_server_fbp));
-	fb_server_fbp = FBIO_NULL;
+	fb_server_fbp = FB_NULL;
     }
     /* Don't check for errors, SGI linger mode or other events
      * may have already closed down all the file descriptors.
      * If communication has broken, other end will know we are gone.
      */
     (void)pkg_send(MSG_RETURN, rbuf, NET_LONG_LEN, pcp);
-    if (buf) (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
 
 /*
- * F B _ S E R V E R _ F B _ F R E E
- *
  * The fb_free() call is more potent than fb_close(), which it
  * must precede; it causes shared memory to be returned & stuff like that.
  * The current FBSERV program exits after getting one.
@@ -185,29 +175,28 @@ fb_server_fb_free(struct pkg_conn *pcp, char *buf)
 	(void)pkg_plong(&rbuf[0], -1);
     } else {
 	(void)pkg_plong(&rbuf[0], fb_free(fb_server_fbp));
-	fb_server_fbp = FBIO_NULL;
+	fb_server_fbp = FB_NULL;
     }
 
     if (pkg_send(MSG_RETURN, rbuf, NET_LONG_LEN, pcp) != NET_LONG_LEN)
 	fprintf(stderr, "pkg_send fb_free reply\n");
-    if (buf) (void)free(buf);
+    if (buf)
+	(void)free(buf);
 
     fb_server_got_fb_free = 1;
 }
 
 
-/*
- * F B _ S E R V E R _ F B _ C L E A R
- *
- */
 static void
 fb_server_fb_clear(struct pkg_conn *pcp, char *buf)
 {
     RGBpixel bg;
     char rbuf[NET_LONG_LEN+1];
 
-    if (buf == NULL) return;
-    if (pcp == PKC_NULL) return;
+    if (buf == NULL)
+	return;
+    if (pcp == PKC_NULL)
+	return;
 
     bg[RED] = buf[0];
     bg[GRN] = buf[1];
@@ -216,14 +205,11 @@ fb_server_fb_clear(struct pkg_conn *pcp, char *buf)
     (void)pkg_plong(rbuf, fb_clear(fb_server_fbp, bg));
     pkg_send(MSG_RETURN, rbuf, NET_LONG_LEN, pcp);
 
-    if (buf) (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
 
-/*
- * F B _ S E R V E R _ F B _ R E A D
- *
- */
 static void
 fb_server_fb_read(struct pkg_conn *pcp, char *buf)
 {
@@ -233,8 +219,10 @@ fb_server_fb_read(struct pkg_conn *pcp, char *buf)
     static unsigned char *scanbuf = NULL;
     static size_t buflen = 0;
 
-    if (buf == NULL) return;
-    if (pcp == PKC_NULL) return;
+    if (buf == NULL)
+	return;
+    if (pcp == PKC_NULL)
+	return;
 
     x = pkg_glong(&buf[0*NET_LONG_LEN]);
     y = pkg_glong(&buf[1*NET_LONG_LEN]);
@@ -244,27 +232,28 @@ fb_server_fb_read(struct pkg_conn *pcp, char *buf)
 	if (scanbuf != NULL)
 	    free((char *)scanbuf);
 	buflen = num*sizeof(RGBpixel);
-	if (buflen < 1024*sizeof(RGBpixel))
-	    buflen = 1024*sizeof(RGBpixel);
+	V_MAX(buflen, 1024*sizeof(RGBpixel));
+
 	if ((scanbuf = (unsigned char *)malloc(buflen)) == NULL) {
 	    fb_log("fb_read: malloc failed!");
-	    if (buf) (void)free(buf);
+	    if (buf)
+		(void)free(buf);
 	    buflen = 0;
 	    return;
 	}
     }
 
     ret = fb_read(fb_server_fbp, x, y, scanbuf, num);
-    if (ret < 0) ret = 0;		/* map error indications */
+    V_MAX(ret, 0);		/* map error indications */
+
     /* sending a 0-length package indicates error */
     pkg_send(MSG_RETURN, (char *)scanbuf, ret*sizeof(RGBpixel), pcp);
-    if (buf) (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
 
 /*
- * F B _ S E R V E R _ F B _ W R I T E
- *
  * Client can ask for PKG-level acknowledgements (with error code) or not,
  * based upon whether type is MSG_FBWRITE or MSG_FBWRITE+MSG_NORETURN.
  */
@@ -276,8 +265,10 @@ fb_server_fb_write(struct pkg_conn *pcp, char *buf)
     int ret;
     int type;
 
-    if (buf == NULL) return;
-    if (pcp == PKC_NULL) return;
+    if (buf == NULL)
+	return;
+    if (pcp == PKC_NULL)
+	return;
 
     x = pkg_glong(&buf[0*NET_LONG_LEN]);
     y = pkg_glong(&buf[1*NET_LONG_LEN]);
@@ -289,13 +280,11 @@ fb_server_fb_write(struct pkg_conn *pcp, char *buf)
 	(void)pkg_plong(&rbuf[0*NET_LONG_LEN], ret);
 	pkg_send(MSG_RETURN, rbuf, NET_LONG_LEN, pcp);
     }
-    if (buf) (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
 
-/*
- * F B _ S E R V E R _ F B _ R E A D R E C T
- */
 static void
 fb_server_fb_readrect(struct pkg_conn *pcp, char *buf)
 {
@@ -306,8 +295,10 @@ fb_server_fb_readrect(struct pkg_conn *pcp, char *buf)
     static unsigned char *scanbuf = NULL;
     static size_t buflen = 0;
 
-    if (buf == NULL) return;
-    if (pcp == PKC_NULL) return;
+    if (buf == NULL)
+	return;
+    if (pcp == PKC_NULL)
+	return;
 
     xmin = pkg_glong(&buf[0*NET_LONG_LEN]);
     ymin = pkg_glong(&buf[1*NET_LONG_LEN]);
@@ -319,27 +310,28 @@ fb_server_fb_readrect(struct pkg_conn *pcp, char *buf)
 	if (scanbuf != NULL)
 	    free((char *)scanbuf);
 	buflen = num*sizeof(RGBpixel);
-	if (buflen < 1024*sizeof(RGBpixel))
-	    buflen = 1024*sizeof(RGBpixel);
+	V_MAX(buflen, 1024*sizeof(RGBpixel));
+
 	if ((scanbuf = (unsigned char *)malloc(buflen)) == NULL) {
 	    fb_log("fb_read: malloc failed!");
-	    if (buf) (void)free(buf);
+	    if (buf)
+		(void)free(buf);
 	    buflen = 0;
 	    return;
 	}
     }
 
     ret = fb_readrect(fb_server_fbp, xmin, ymin, width, height, scanbuf);
-    if (ret < 0) ret = 0;		/* map error indications */
+    V_MAX(ret, 0);		/* map error indications */
+
     /* sending a 0-length package indicates error */
     pkg_send(MSG_RETURN, (char *)scanbuf, ret*sizeof(RGBpixel), pcp);
-    if (buf) (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
 
 /*
- * F B _ S E R V E R _ F B _ W R I T E R E C T
- *
  * A whole rectangle of pixels at once, probably large.
  */
 static void
@@ -351,8 +343,10 @@ fb_server_fb_writerect(struct pkg_conn *pcp, char *buf)
     int ret;
     int type;
 
-    if (buf == NULL) return;
-    if (pcp == PKC_NULL) return;
+    if (buf == NULL)
+	return;
+    if (pcp == PKC_NULL)
+	return;
 
     x = pkg_glong(&buf[0*NET_LONG_LEN]);
     y = pkg_glong(&buf[1*NET_LONG_LEN]);
@@ -367,13 +361,11 @@ fb_server_fb_writerect(struct pkg_conn *pcp, char *buf)
 	(void)pkg_plong(&rbuf[0*NET_LONG_LEN], ret);
 	pkg_send(MSG_RETURN, rbuf, NET_LONG_LEN, pcp);
     }
-    if (buf) (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
 
-/*
- * F B _ S E R V E R _ F B _ B W R E A D R E C T
- */
 static void
 fb_server_fb_bwreadrect(struct pkg_conn *pcp, char *buf)
 {
@@ -384,8 +376,10 @@ fb_server_fb_bwreadrect(struct pkg_conn *pcp, char *buf)
     static unsigned char *scanbuf = NULL;
     static size_t buflen = 0;
 
-    if (buf == NULL) return;
-    if (pcp == PKC_NULL) return;
+    if (buf == NULL)
+	return;
+    if (pcp == PKC_NULL)
+       	return;
 
     xmin = pkg_glong(&buf[0*NET_LONG_LEN]);
     ymin = pkg_glong(&buf[1*NET_LONG_LEN]);
@@ -397,27 +391,28 @@ fb_server_fb_bwreadrect(struct pkg_conn *pcp, char *buf)
 	if (scanbuf != NULL)
 	    free((char *)scanbuf);
 	buflen = num;
-	if (buflen < 1024)
-	    buflen = 1024;
+	V_MAX(buflen, 1024);
+
 	if ((scanbuf = (unsigned char *)malloc(buflen)) == NULL) {
 	    fb_log("fb_bwreadrect: malloc failed!");
-	    if (buf) (void)free(buf);
+	    if (buf)
+		(void)free(buf);
 	    buflen = 0;
 	    return;
 	}
     }
 
     ret = fb_bwreadrect(fb_server_fbp, xmin, ymin, width, height, scanbuf);
-    if (ret < 0) ret = 0;		/* map error indications */
+    V_MAX(ret, 0);		/* map error indications */
+
     /* sending a 0-length package indicates error */
     pkg_send(MSG_RETURN, (char *)scanbuf, ret, pcp);
-    if (buf) (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
 
 /*
- * F B _ S E R V E R _ F B _ B W W R I T E R E C T
- *
  * A whole rectangle of monochrome pixels at once, probably large.
  */
 static void
@@ -429,8 +424,10 @@ fb_server_fb_bwwriterect(struct pkg_conn *pcp, char *buf)
     int ret;
     int type;
 
-    if (buf == NULL) return;
-    if (pcp == PKC_NULL) return;
+    if (buf == NULL)
+       	return;
+    if (pcp == PKC_NULL)
+       	return;
 
     x = pkg_glong(&buf[0*NET_LONG_LEN]);
     y = pkg_glong(&buf[1*NET_LONG_LEN]);
@@ -447,24 +444,23 @@ fb_server_fb_bwwriterect(struct pkg_conn *pcp, char *buf)
     } else {
 	/* No formal return code.  Note errors locally */
 	if (ret < 0) fb_log("fb_server_fb_bwwriterect(%d, %d, %d, %d) error %d\n",
-			    x, y, width, height, ret);
+				x, y, width, height, ret);
     }
-    if (buf) (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
 
-/*
- * F B _ S E R V E R _ F B _ C U R S O R
- *
- */
 static void
 fb_server_fb_cursor(struct pkg_conn *pcp, char *buf)
 {
     int mode, x, y;
     char rbuf[NET_LONG_LEN+1];
 
-    if (buf == NULL) return;
-    if (pcp == PKC_NULL) return;
+    if (buf == NULL)
+       	return;
+    if (pcp == PKC_NULL)
+       	return;
 
     mode = pkg_glong(&buf[0*NET_LONG_LEN]);
     x = pkg_glong(&buf[1*NET_LONG_LEN]);
@@ -472,14 +468,11 @@ fb_server_fb_cursor(struct pkg_conn *pcp, char *buf)
 
     (void)pkg_plong(&rbuf[0], fb_cursor(fb_server_fbp, mode, x, y));
     pkg_send(MSG_RETURN, rbuf, NET_LONG_LEN, pcp);
-    if (buf) (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
 
-/*
- * F B _ S E R V E R _ F B _ G E T _ C U R S O R
- *
- */
 static void
 fb_server_fb_getcursor(struct pkg_conn *pcp, char *buf)
 {
@@ -495,14 +488,11 @@ fb_server_fb_getcursor(struct pkg_conn *pcp, char *buf)
     (void)pkg_plong(&rbuf[2*NET_LONG_LEN], x);
     (void)pkg_plong(&rbuf[3*NET_LONG_LEN], y);
     pkg_send(MSG_RETURN, rbuf, 4*NET_LONG_LEN, pcp);
-    if (buf) (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
 
-/*
- * F B _ S E R V E R _ F B _ S E T C U R S O R
- *
- */
 static void
 fb_server_fb_setcursor(struct pkg_conn *pcp, char *buf)
 {
@@ -511,7 +501,8 @@ fb_server_fb_setcursor(struct pkg_conn *pcp, char *buf)
     int xbits, ybits;
     int xorig, yorig;
 
-    if (buf == NULL) return;
+    if (buf == NULL)
+	return;
     if (pcp == PKC_NULL) return;
 
     xbits = pkg_glong(&buf[0*NET_LONG_LEN]);
@@ -526,13 +517,12 @@ fb_server_fb_setcursor(struct pkg_conn *pcp, char *buf)
 	(void)pkg_plong(&rbuf[0*NET_LONG_LEN], ret);
 	pkg_send(MSG_RETURN, rbuf, NET_LONG_LEN, pcp);
     }
-    if (buf) (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
 
 /*
- * F B _ S E R V E R _ F B _ S C U R S O R
- *
  * An OLD interface.  Retained so old clients can still be served.
  */
 static void
@@ -541,8 +531,10 @@ fb_server_fb_scursor(struct pkg_conn *pcp, char *buf)
     int mode, x, y;
     char rbuf[NET_LONG_LEN+1];
 
-    if (buf == NULL) return;
-    if (pcp == PKC_NULL) return;
+    if (buf == NULL)
+	return;
+    if (pcp == PKC_NULL)
+	return;
 
     mode = pkg_glong(&buf[0*NET_LONG_LEN]);
     x = pkg_glong(&buf[1*NET_LONG_LEN]);
@@ -550,13 +542,12 @@ fb_server_fb_scursor(struct pkg_conn *pcp, char *buf)
 
     (void)pkg_plong(&rbuf[0], fb_scursor(fb_server_fbp, mode, x, y));
     pkg_send(MSG_RETURN, rbuf, NET_LONG_LEN, pcp);
-    if (buf) (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
 
 /*
- * F B _ S E R V E R _ F B _ W I N D O W
- *
  * An OLD interface.  Retained so old clients can still be served.
  */
 static void
@@ -565,21 +556,22 @@ fb_server_fb_window(struct pkg_conn *pcp, char *buf)
     int x, y;
     char rbuf[NET_LONG_LEN+1];
 
-    if (buf == NULL) return;
-    if (pcp == PKC_NULL) return;
+    if (buf == NULL)
+	return;
+    if (pcp == PKC_NULL)
+	return;
 
     x = pkg_glong(&buf[0*NET_LONG_LEN]);
     y = pkg_glong(&buf[1*NET_LONG_LEN]);
 
     (void)pkg_plong(&rbuf[0], fb_window(fb_server_fbp, x, y));
     pkg_send(MSG_RETURN, rbuf, NET_LONG_LEN, pcp);
-    if (buf) (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
 
 /*
- * F B _ S E R V E R _ F B _ Z O O M
- *
  * An OLD interface.  Retained so old clients can still be served.
  */
 static void
@@ -588,22 +580,21 @@ fb_server_fb_zoom(struct pkg_conn *pcp, char *buf)
     int x, y;
     char rbuf[NET_LONG_LEN+1];
 
-    if (buf == NULL) return;
-    if (pcp == PKC_NULL) return;
+    if (buf == NULL)
+	return;
+    if (pcp == PKC_NULL)
+	return;
 
     x = pkg_glong(&buf[0*NET_LONG_LEN]);
     y = pkg_glong(&buf[1*NET_LONG_LEN]);
 
     (void)pkg_plong(&rbuf[0], fb_zoom(fb_server_fbp, x, y));
     pkg_send(MSG_RETURN, rbuf, NET_LONG_LEN, pcp);
-    if (buf) (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
 
-/*
- * F B _ S E R V E R _ F B _ V I E W
- *
- */
 static void
 fb_server_fb_view(struct pkg_conn *pcp, char *buf)
 {
@@ -611,8 +602,10 @@ fb_server_fb_view(struct pkg_conn *pcp, char *buf)
     int xcenter, ycenter, xzoom, yzoom;
     char rbuf[NET_LONG_LEN+1];
 
-    if (buf == NULL) return;
-    if (pcp == PKC_NULL) return;
+    if (buf == NULL)
+	return;
+    if (pcp == PKC_NULL)
+	return;
 
     xcenter = pkg_glong(&buf[0*NET_LONG_LEN]);
     ycenter = pkg_glong(&buf[1*NET_LONG_LEN]);
@@ -622,14 +615,11 @@ fb_server_fb_view(struct pkg_conn *pcp, char *buf)
     ret = fb_view(fb_server_fbp, xcenter, ycenter, xzoom, yzoom);
     (void)pkg_plong(&rbuf[0], ret);
     pkg_send(MSG_RETURN, rbuf, NET_LONG_LEN, pcp);
-    if (buf) (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
 
-/*
- * F B _ S E R V E R _ F B _ G E T V I E W
- *
- */
 static void
 fb_server_fb_getview(struct pkg_conn *pcp, char *buf)
 {
@@ -646,14 +636,11 @@ fb_server_fb_getview(struct pkg_conn *pcp, char *buf)
     (void)pkg_plong(&rbuf[3*NET_LONG_LEN], xzoom);
     (void)pkg_plong(&rbuf[4*NET_LONG_LEN], yzoom);
     pkg_send(MSG_RETURN, rbuf, 5*NET_LONG_LEN, pcp);
-    if (buf) (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
 
-/*
- * F B _ S E R V E R _ F B _ R M A P
- *
- */
 static void
 fb_server_fb_rmap(struct pkg_conn *pcp, char *buf)
 {
@@ -672,13 +659,12 @@ fb_server_fb_rmap(struct pkg_conn *pcp, char *buf)
     }
     pkg_send(MSG_DATA, (char *)cm, sizeof(cm), pcp);
     pkg_send(MSG_RETURN, rbuf, NET_LONG_LEN, pcp);
-    if (buf) (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
 
 /*
- * F B _ S E R V E R _ F B _ W M A P
- *
  * Accept a color map sent by the client, and write it to the framebuffer.
  * Network format is to send each entry as a network (IBM) order 2-byte
  * short, 256 red shorts, followed by 256 green and 256 blue, for a total
@@ -692,8 +678,10 @@ fb_server_fb_wmap(struct pkg_conn *pcp, char *buf)
     long ret;
     ColorMap map;
 
-    if (buf == NULL) return;
-    if (pcp == PKC_NULL) return;
+    if (buf == NULL)
+       	return;
+    if (pcp == PKC_NULL)
+	return;
 
     if (pcp->pkc_len == 0)
 	ret = fb_wmap(fb_server_fbp, COLORMAP_NULL);
@@ -707,14 +695,11 @@ fb_server_fb_wmap(struct pkg_conn *pcp, char *buf)
     }
     (void)pkg_plong(&rbuf[0], ret);
     pkg_send(MSG_RETURN, rbuf, NET_LONG_LEN, pcp);
-    if (buf) (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
 
-/*
- * F B _ S E R V E R _ F B _ F L U S H
- *
- */
 static void
 fb_server_fb_flush(struct pkg_conn *pcp, char *buf)
 {
@@ -730,27 +715,23 @@ fb_server_fb_flush(struct pkg_conn *pcp, char *buf)
 	pkg_send(MSG_RETURN, rbuf, NET_LONG_LEN, pcp);
     }
 
-    if (buf) (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
 
-/*
- * F B _ S E R V E R _ F B _ P O L L
- *
- */
 static void
 fb_server_fb_poll(struct pkg_conn *pcp, char *buf)
 {
     if (pcp == PKC_NULL) return;
 
     (void)fb_poll(fb_server_fbp);
-    if (buf) (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
 
 /*
- * F B _ S E R V E R _ F B _ H E L P
- *
  * At one time at least we couldn't send a zero length PKG
  * message back and forth, so we receive a dummy long here.
  */
@@ -760,50 +741,52 @@ fb_server_fb_help(struct pkg_conn *pcp, char *buf)
     long ret;
     char rbuf[NET_LONG_LEN+1];
 
-    if (pcp == PKC_NULL) return;
-    if (buf == NULL) return;
+    if (pcp == PKC_NULL)
+       	return;
+    if (buf == NULL)
+       	return;
 
     (void)pkg_glong(&buf[0*NET_LONG_LEN]);
 
     ret = fb_help(fb_server_fbp);
     (void)pkg_plong(&rbuf[0], ret);
     pkg_send(MSG_RETURN, rbuf, NET_LONG_LEN, pcp);
-    if (buf) (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
-
-const struct pkg_switch fb_server_pkg_switch[] = {
-    { MSG_FBOPEN,		fb_server_fb_open,	"Open Framebuffer", NULL },
-    { MSG_FBCLOSE,		fb_server_fb_close,	"Close Framebuffer", NULL },
-    { MSG_FBCLEAR,		fb_server_fb_clear,	"Clear Framebuffer", NULL },
-    { MSG_FBREAD,		fb_server_fb_read,	"Read Pixels", NULL },
-    { MSG_FBWRITE,		fb_server_fb_write,	"Write Pixels", NULL },
-    { MSG_FBWRITE+MSG_NORETURN,	fb_server_fb_write,	"Asynch write", NULL },
-    { MSG_FBCURSOR,		fb_server_fb_cursor,	"Cursor", NULL },
-    { MSG_FBGETCURSOR,		fb_server_fb_getcursor,	"Get Cursor", NULL },	   /*NEW*/
-    { MSG_FBSCURSOR,		fb_server_fb_scursor,	"Screen Cursor", NULL }, /*OLD*/
-    { MSG_FBWINDOW,		fb_server_fb_window,	"Window", NULL },	   /*OLD*/
-    { MSG_FBZOOM,		fb_server_fb_zoom,	"Zoom", NULL },	   /*OLD*/
-    { MSG_FBVIEW,		fb_server_fb_view,	"View", NULL },	   /*NEW*/
-    { MSG_FBGETVIEW,		fb_server_fb_getview,	"Get View", NULL },	   /*NEW*/
-    { MSG_FBRMAP,		fb_server_fb_rmap,	"R Map", NULL },
-    { MSG_FBWMAP,		fb_server_fb_wmap,	"W Map", NULL },
-    { MSG_FBHELP,		fb_server_fb_help,	"Help Request", NULL },
-    { MSG_ERROR,		fb_server_got_unknown,	"Error Message", NULL },
-    { MSG_CLOSE,		fb_server_got_unknown,	"Close Connection", NULL },
-    { MSG_FBREADRECT, 		fb_server_fb_readrect,	"Read Rectangle", NULL },
-    { MSG_FBWRITERECT,		fb_server_fb_writerect,	"Write Rectangle", NULL },
-    { MSG_FBWRITERECT+MSG_NORETURN, fb_server_fb_writerect,	"Write Rectangle", NULL },
-    { MSG_FBBWREADRECT, 	fb_server_fb_bwreadrect,	"Read BW Rectangle", NULL },
-    { MSG_FBBWWRITERECT,	fb_server_fb_bwwriterect,	"Write BW Rectangle", NULL },
-    { MSG_FBBWWRITERECT+MSG_NORETURN, fb_server_fb_bwwriterect, "Write BW Rectangle", NULL },
-    { MSG_FBFLUSH,		fb_server_fb_flush,		"Flush Output", NULL },
-    { MSG_FBFLUSH + MSG_NORETURN, fb_server_fb_flush,		"Flush Output", NULL },
-    { MSG_FBFREE,		fb_server_fb_free,		"Free Resources", NULL },
-    { MSG_FBPOLL,		fb_server_fb_poll,		"Handle Events", NULL },
-    { MSG_FBSETCURSOR,		fb_server_fb_setcursor,		"Set Cursor Shape", NULL },
-    { MSG_FBSETCURSOR + MSG_NORETURN, fb_server_fb_setcursor,	"Set Cursor Shape", NULL },
-    { 0, NULL, NULL, NULL }
+const struct pkg_switch pkg_switch[] = {
+    { MSG_FBOPEN,                       fb_server_fb_open,        "Open Framebuffer", NULL },
+    { MSG_FBCLOSE,                      fb_server_fb_close,       "Close Framebuffer", NULL },
+    { MSG_FBCLEAR,                      fb_server_fb_clear,       "Clear Framebuffer", NULL },
+    { MSG_FBREAD,                       fb_server_fb_read,        "Read Pixels", NULL },
+    { MSG_FBWRITE,                      fb_server_fb_write,       "Write Pixels", NULL },
+    { MSG_FBWRITE + MSG_NORETURN,       fb_server_fb_write,       "Asynch write", NULL },
+    { MSG_FBCURSOR,                     fb_server_fb_cursor,      "Cursor", NULL },
+    { MSG_FBGETCURSOR,                  fb_server_fb_getcursor,   "Get Cursor", NULL },      /*NEW*/
+    { MSG_FBSCURSOR,                    fb_server_fb_scursor,     "Screen Cursor", NULL }, /*OLD*/
+    { MSG_FBWINDOW,                     fb_server_fb_window,      "Window", NULL },          /*OLD*/
+    { MSG_FBZOOM,                       fb_server_fb_zoom,        "Zoom", NULL },    /*OLD*/
+    { MSG_FBVIEW,                       fb_server_fb_view,        "View", NULL },    /*NEW*/
+    { MSG_FBGETVIEW,                    fb_server_fb_getview,     "Get View", NULL },        /*NEW*/
+    { MSG_FBRMAP,                       fb_server_fb_rmap,        "R Map", NULL },
+    { MSG_FBWMAP,                       fb_server_fb_wmap,        "W Map", NULL },
+    { MSG_FBHELP,                       fb_server_fb_help,        "Help Request", NULL },
+    { MSG_ERROR,                        fb_server_fb_unknown,     "Error Message", NULL },
+    { MSG_CLOSE,                        fb_server_fb_unknown,     "Close Connection", NULL },
+    { MSG_FBREADRECT,                   fb_server_fb_readrect,    "Read Rectangle", NULL },
+    { MSG_FBWRITERECT,                  fb_server_fb_writerect,   "Write Rectangle", NULL },
+    { MSG_FBWRITERECT + MSG_NORETURN,   fb_server_fb_writerect,   "Write Rectangle", NULL },
+    { MSG_FBBWREADRECT,                 fb_server_fb_bwreadrect,  "Read BW Rectangle", NULL },
+    { MSG_FBBWWRITERECT,                fb_server_fb_bwwriterect, "Write BW Rectangle", NULL },
+    { MSG_FBBWWRITERECT + MSG_NORETURN, fb_server_fb_bwwriterect, "Write BW Rectangle", NULL },
+    { MSG_FBFLUSH,                      fb_server_fb_flush,       "Flush Output", NULL },
+    { MSG_FBFLUSH + MSG_NORETURN,       fb_server_fb_flush,       "Flush Output", NULL },
+    { MSG_FBFREE,                       fb_server_fb_free,        "Free Resources", NULL },
+    { MSG_FBPOLL,                       fb_server_fb_poll,        "Handle Events", NULL },
+    { MSG_FBSETCURSOR,                  fb_server_fb_setcursor,   "Set Cursor Shape", NULL },
+    { MSG_FBSETCURSOR + MSG_NORETURN,   fb_server_fb_setcursor,   "Set Cursor Shape", NULL },
+    { 0,                                NULL,           NULL, NULL }
 };
 
 
