@@ -1,7 +1,7 @@
 /*                         N O I S E . C
  * BRL-CAD
  *
- * Copyright (c) 2004-2013 United States Government as represented by
+ * Copyright (c) 2004-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This library is free software; you can redistribute it and/or
@@ -26,7 +26,7 @@
  * lattice points.  The functions should be evaluated at non-integer
  * locations for their nature to be realized.
  *
- * Conatins contributed code from:
+ * Contains contributed code from:
  * F. Kenton Musgrave
  * Robert Skinner
  *
@@ -37,9 +37,13 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
-#include "bu.h"
+
+#include "bu/malloc.h"
+#include "bu/parallel.h"
+#include "bu/exit.h"
 #include "vmath.h"
-#include "bn.h"
+#include "bn/noise.h"
+#include "bn/rand.h"
 
 
 /**
@@ -59,7 +63,7 @@
  *
  *@n x, y, z are set to the noise space location for the source point.
  *@n ix, iy, iz are the integer lattice point (integer portion of x, y, z)
- *@n fx, fy, fz are the fractional lattice distance above ix, iy, iz
+ *@n fx, fy, fz are the fractional lattice distances above ix, iy, iz
  *
  * The noise function has a finite domain, which can be exceeded when
  * using fractal textures with very high frequencies.  This routine is
@@ -76,9 +80,7 @@ filter_args(fastf_t *src, fastf_t *p, fastf_t *f, int *ip)
 
     for (i=0; i < 3; i++) {
 	/* assure values are positive */
-	if (src[i] < 0) dst[i] = -src[i];
-	else dst[i] = src[i];
-
+	dst[i] = fabs(src[i]);
 
 	/* fold space */
 	while (dst[i] > max || dst[i]<0) {
@@ -127,14 +129,13 @@ static double RTable[MAXSIZE];
 struct str_ht {
     uint32_t magic;
     char hashTableValid;
-    uint32_t *hashTableMagic1;
     uint32_t *hashTable;
-    uint32_t *hashTableMagic2;
     uint32_t magic_end;
 };
 
 static struct str_ht ht;
 
+/* must not vary in size cross-platform for repeatibility */
 #define TABLE_SIZE 4096
 
 #define MAGIC_STRHT1 1771561
@@ -144,12 +145,6 @@ static struct str_ht ht;
 #define CK_HT() {							\
 	BU_CKMAG(&ht.magic, MAGIC_STRHT1, "struct str_ht ht 1");	\
 	BU_CKMAG(&ht.magic_end, MAGIC_STRHT2, "struct str_ht ht 2");	\
-	BU_CKMAG(ht.hashTableMagic1, MAGIC_TAB1, "hashTable Magic 1");	\
-	BU_CKMAG(ht.hashTableMagic2, MAGIC_TAB2, "hashTable Magic 2");	\
-	if (ht.hashTable != &ht.hashTableMagic1[1])			\
-	    bu_bomb("ht.hashTable changed rel ht.hashTableMagic1");	\
-	if (ht.hashTableMagic2 != &ht.hashTable[TABLE_SIZE])		\
-	    bu_bomb("ht.hashTable changed rel ht.hashTableMagic2");	\
     }
 
 
@@ -168,29 +163,28 @@ static struct str_ht ht;
 	]
 
 
+static int sem_noise = 0;
+
+
 void
 bn_noise_init(void)
 {
     uint32_t i, j, k, temp;
     uint32_t rndtabi = BN_RAND_TABSIZE - 1;
 
-    bu_semaphore_acquire(BU_SEM_BN_NOISE);
+    if (!sem_noise)
+	sem_noise = bu_semaphore_register("SEM_NOISE");
+
+    bu_semaphore_acquire(sem_noise);
 
     if (ht.hashTableValid) {
-	bu_semaphore_release(BU_SEM_BN_NOISE);
+	bu_semaphore_release(sem_noise);
 	return;
     }
 
     BN_RANDSEED(rndtabi, (BN_RAND_TABSIZE-1));
 
-    /* alloc table size plus two magic numbers */
-    ht.hashTableMagic1 = (uint32_t *) bu_calloc(1, 2*sizeof(uint32_t) + TABLE_SIZE*sizeof(uint32_t), "noise hashTable");
-
-    ht.hashTable = &ht.hashTableMagic1[1];
-    ht.hashTableMagic2 = &ht.hashTable[TABLE_SIZE];
-
-    *ht.hashTableMagic1 = (uint32_t)MAGIC_TAB1;
-    *ht.hashTableMagic2 = (uint32_t)MAGIC_TAB2;
+    ht.hashTable = (uint32_t *)bu_calloc(1, (TABLE_SIZE*sizeof(uint32_t)), "noise hashTable");
 
     ht.magic_end = (uint32_t)MAGIC_STRHT2;
     ht.magic = (uint32_t)MAGIC_STRHT1;
@@ -214,7 +208,7 @@ bn_noise_init(void)
 
     ht.hashTableValid = 1;
 
-    bu_semaphore_release(BU_SEM_BN_NOISE);
+    bu_semaphore_release(sem_noise);
 
     CK_HT();
 }
@@ -479,7 +473,7 @@ build_spec_tbl(double h_val, double lacunarity, double octaves)
 /**
  * The first order of business is to see if we have pre-computed the
  * spectral weights table for these parameters in a previous
- * invocation.  If not, the we compute them and save them for possible
+ * invocation.  If not, then we compute them and save them for possible
  * future use
  */
 struct fbm_spec *
@@ -494,7 +488,7 @@ find_spec_wgt(double h, double l, double o)
 	    bu_bomb("find_spec_wgt");
 	if (EQUAL(ep->lacunarity, l)
 	    && EQUAL(ep->h_val, h)
-	    && ep->octaves > (o - SMALL_FASTF))
+	    && (ep->octaves > o || EQUAL(ep->octaves, o)))
 	{
 	    return ep;
 	}
@@ -504,7 +498,7 @@ find_spec_wgt(double h, double l, double o)
      * the list to wait our turn to add what we want to the table.
      */
 
-    bu_semaphore_acquire(BU_SEM_BN_NOISE);
+    bu_semaphore_acquire(sem_noise);
 
     /* We search the list one more time in case the last process to
      * hold the semaphore just created the table we were about to add
@@ -515,14 +509,16 @@ find_spec_wgt(double h, double l, double o)
 	    bu_bomb("find_spec_wgt");
 	if (EQUAL(ep->lacunarity, l)
 	    && EQUAL(ep->h_val, h)
-	    && ep->octaves > (o - SMALL_FASTF))
+	    && (ep->octaves > o || EQUAL(ep->octaves, o)))
+	{
 	    break;
+	}
     }
 
     if (i >= etbl_next)
 	ep = build_spec_tbl(h, l, o);
 
-    bu_semaphore_release(BU_SEM_BN_NOISE);
+    bu_semaphore_release(sem_noise);
 
     return ep;
 }
@@ -538,7 +534,7 @@ bn_noise_fbm(fastf_t *point, double h_val, double lacunarity, double octaves)
 
     /* The first order of business is to see if we have pre-computed
      * the spectral weights table for these parameters in a previous
-     * invocation.  If not, the we compute them and save them for
+     * invocation.  If not, then we compute them and save them for
      * possible future use
      */
 
@@ -569,7 +565,7 @@ bn_noise_fbm(fastf_t *point, double h_val, double lacunarity, double octaves)
 
     return value;
 
-} /* bn_noise_fbm() */
+}
 
 
 double
@@ -583,18 +579,16 @@ bn_noise_turb(fastf_t *point, double h_val, double lacunarity, double octaves)
 
     /* The first order of business is to see if we have pre-computed
      * the spectral weights table for these parameters in a previous
-     * invocation.  If not, the we compute them and save them for
+     * invocation.  If not, then we compute them and save them for
      * possible future use
      */
 
-#define CACHE_SPECTRAL_WGTS 1
-#ifdef CACHE_SPECTRAL_WGTS
-
     ep = find_spec_wgt(h_val, lacunarity, octaves);
 
     /* now we're ready to compute the fBm value */
 
     value = 0.0;            /* initialize vars to proper values */
+    /* not cached: frequency = 1.0; */
 
     /* copy the point so we don't corrupt the caller's copy of the
      * variable
@@ -605,6 +599,10 @@ bn_noise_turb(fastf_t *point, double h_val, double lacunarity, double octaves)
     /* inner loop of spectral construction */
     oct=(int)octaves; /* save repeating double->int cast */
     for (i=0; i < oct; i++) {
+	/* not cached:
+	 * value += fabs(bn_noise_perlin(pt)) * pow(frequency, -h_val);
+	 * frequency *= lacunarity;
+	 */
 	value += fabs(bn_noise_perlin(pt)) * spec_wgts[i];
 	PSCALE(pt, lacunarity);
     }
@@ -615,31 +613,12 @@ bn_noise_turb(fastf_t *point, double h_val, double lacunarity, double octaves)
 	 * preset in loop above
 	 */
 	value += noise_remainder * bn_noise_perlin(pt) * spec_wgts[i];
-    }
-#else
-    PCOPY(pt, point);
-
-    value = 0.0;      /* initialize vars to proper values */
-    frequency = 1.0;
-
-    oct=(int)octaves; /* save repeating double->int cast */
-    for (i=0; i < oct; i++) {
-	value += fabs(bn_noise_perlin(pt)) * pow(frequency, -h_val);
-	frequency *= lacunarity;
-	PSCALE(pt, lacunarity);
+	/* not cached: value += noise_remainder * bn_noise_perlin(pt) * pow(frequency, -h_val); */
     }
 
-    noise_remainder = octaves - (int)octaves;
-    if (noise_remainder) {
-	/* add in ``octaves'' noise_remainder ``i'' and spatial freq. are
-	 * preset in loop above
-	 */
-	value += noise_remainder * bn_noise_perlin(pt) * pow(frequency, -h_val);
-    }
-#endif
     return value;
 
-} /* bn_noise_turb() */
+}
 
 
 double
@@ -652,7 +631,7 @@ bn_noise_ridged(fastf_t *point, double h_val, double lacunarity, double octaves,
 
     /* The first order of business is to see if we have pre-computed
      * the spectral weights table for these parameters in a previous
-     * invocation.  If not, the we compute them and save them for
+     * invocation.  If not, then we compute them and save them for
      * possible future use
      */
 
@@ -671,7 +650,7 @@ bn_noise_ridged(fastf_t *point, double h_val, double lacunarity, double octaves,
     /* get absolute value of noise_signal (this creates the ridges) */
     if (noise_signal < 0.0) noise_signal = -noise_signal;
 
-    /* invert and translate (note that "offset shoudl be ~= 1.0 */
+    /* invert and translate (note that "offset should be ~= 1.0 */
     noise_signal = offset - noise_signal;
 
     /* square the noise_signal, to increase "sharpness" of ridges */
@@ -700,15 +679,13 @@ bn_noise_ridged(fastf_t *point, double h_val, double lacunarity, double octaves,
 double
 bn_noise_mf(fastf_t *point, double h_val, double lacunarity, double octaves, double offset)
 {
-    double frequency = 1.0;
     struct fbm_spec *ep;
-    double result, weight, noise_signal, *spec_wgts;
+    double result, *spec_wgts;
     point_t pt;
-    int i;
 
     /* The first order of business is to see if we have pre-computed
      * the spectral weights table for these parameters in a previous
-     * invocation.  If not, the we compute them and save them for
+     * invocation.  If not, then we compute them and save them for
      * possible future use
      */
 
@@ -722,19 +699,29 @@ bn_noise_mf(fastf_t *point, double h_val, double lacunarity, double octaves, dou
     offset = 1.0;
 
     result = (bn_noise_perlin(pt) + offset) * spec_wgts[0];
-    weight = result;
 
-    for (i=1; i < octaves; i++) {
-	PSCALE(pt, lacunarity);
+#if 0
+    /* TODO - the code below doesn't contribute to the result - what
+     * should it be doing? */
+    {
+	double weight = result;
+	double noise_signal;
+	double frequency = 1.0;
+	int i;
 
-	if (weight > 1.0) weight = 1.0;
+	for (i=1; i < octaves; i++) {
+	    PSCALE(pt, lacunarity);
 
-	noise_signal = (bn_noise_perlin(pt) + offset) * spec_wgts[i];
+	    V_MIN(weight, 1.0);
 
-	noise_signal += fabs(bn_noise_perlin(pt)) * pow(frequency, -h_val);
-	frequency *= lacunarity;
-	PSCALE(pt, lacunarity);
+	    noise_signal = (bn_noise_perlin(pt) + offset) * spec_wgts[i];
+
+	    noise_signal += fabs(bn_noise_perlin(pt)) * pow(frequency, -h_val);
+	    frequency *= lacunarity;
+	    PSCALE(pt, lacunarity);
+	}
     }
+#endif
     return result;
 }
 
