@@ -1,7 +1,7 @@
 /*                       L I N E B U F . C
  * BRL-CAD
  *
- * Copyright (c) 2004-2013 United States Government as represented by
+ * Copyright (c) 2004-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This library is free software; you can redistribute it and/or
@@ -22,7 +22,7 @@
 
 #include <stdio.h>
 
-#include "bu.h"
+#include "bu/log.h"
 
 #ifndef BUFSIZE
 #  define BUFSIZE 2048
