@@ -1,7 +1,7 @@
 /*                          M R E A D . C
  * BRL-CAD
  *
- * Copyright (c) 1992-2013 United States Government as represented by
+ * Copyright (c) 1992-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This program is free software; you can redistribute it and/or
@@ -25,8 +25,7 @@
 #endif
 #include "bio.h"
 
-#include "bu.h"
-
+#include "bu/log.h"
 
 long int
 bu_mread(int fd, void *bufp, long int n)
@@ -36,7 +35,7 @@ bu_mread(int fd, void *bufp, long int n)
     char *cbufp = (char *)bufp;
 
     while (count < n) {
-	nread = read(fd, cbufp, (size_t)n-count);
+	nread = read(fd, cbufp, (size_t)(n-count));
 	if (UNLIKELY(nread < 0)) {
 	    return nread;
 	}
