@@ -1,7 +1,7 @@
 /*                      P A R A L L E L . C
  * BRL-CAD
  *
- * Copyright (c) 2004-2013 United States Government as represented by
+ * Copyright (c) 2004-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This library is free software; you can redistribute it and/or
@@ -26,52 +26,35 @@
 #include <string.h>
 #include <signal.h>
 
+#ifdef HAVE_SYS_TYPES_H
+#  include <sys/types.h>
+#endif
+
 #ifdef HAVE_SYS_TIME_H
 #  include <sys/time.h>
 #endif
 
-#ifdef HAVE_SYS_RESOURCE_H
-#  include <sys/resource.h>
-#endif
-
 #ifdef linux
-#  include <sys/types.h>
-#  ifdef HAVE_SYS_WAIT_H
-#    include <sys/wait.h>
-#  endif
 #  include <sys/stat.h>
-#  include <sys/sysinfo.h>
 #endif
 
 #if defined(__FreeBSD__) || defined(__OpenBSD__)
-#  include <sys/types.h>
 #  include <sys/param.h>
 #  include <sys/sysctl.h>
-#  ifdef HAVE_SYS_WAIT_H
-#    include <sys/wait.h>
-#  endif
 #  include <sys/stat.h>
 #endif
 
 #ifdef __APPLE__
-#  include <sys/types.h>
-#  ifdef HAVE_SYS_WAIT_H
-#    include <sys/wait.h>
-#  endif
 #  include <sys/stat.h>
 #  include <sys/param.h>
 #  include <sys/sysctl.h>
 #endif
 
 #ifdef __sp3__
-#  include <sys/types.h>
 #  include <sys/sysconfig.h>
 #  include <sys/var.h>
 #endif
 
-#ifdef HAVE_SYS_TYPES_H
-#  include <sys/types.h>
-#endif
 #ifdef HAVE_ULOCKS_H
 #  include <ulocks.h>
 #endif
@@ -79,10 +62,6 @@
 #  include <sys/sysmp.h> /* for sysmp() */
 #endif
 
-#ifdef HAVE_SYS_WAIT_H
-#  include <sys/wait.h>
-#endif
-
 #ifdef HAVE_SCHED_H
 #  include <sched.h>
 #else
@@ -91,6 +70,8 @@
 #  endif
 #endif
 
+#include "bresource.h"
+
 /*
  * multithreading support for SunOS 5.X / Solaris 2.x
  */
@@ -109,39 +90,52 @@
 #  define rt_thread_t pthread_t
 #endif
 
+#ifdef _WIN32
+#  define rt_thread_t HANDLE
+#endif
+
 #include "bio.h"
 
-#include "bu.h"
+#include "bu/debug.h"
+#include "bu/log.h"
+#include "bu/malloc.h"
+#include "bu/parallel.h"
+#include "bu/snooze.h"
+#include "bu/str.h"
 
 #include "./parallel.h"
 
+/* #define CPP11THREAD */
 
-struct thread_data {
-    void (*user_func)(int, genptr_t);
-    genptr_t user_arg;
-    int cpu_id;
-    int counted;
-    int affinity;
+#if defined(CPP11THREAD)
+void parallel_cpp11thread(void (*func)(int, void *), size_t ncpu, void *arg);
+#endif /* CPP11THREAD */
+
+int BU_SEM_THREAD;
+
+
+typedef enum {
+    PARALLEL_GET = 0,
+    PARALLEL_PUT = 1
+} parallel_action_t;
+
+
+struct parallel_info {
+    int id; /* cpu+1 */
+    int parent;
+    size_t lim;
+    size_t started;
+    size_t finished;
 };
 
 
-/**
- * process id of the initiating thread. used to shutdown bu_parallel
- * threads/procs.
- */
-static int pid_of_initiating_thread = 0;
-
-/* # threads started */
-static int parallel_nthreads_started = 0;
-
-/* # threads properly finished */
-static int parallel_nthreads_finished = 0;
-
-/* User's arg to his threads */
-static genptr_t parallel_arg;
-
-/* user function to run in parallel */
-static void (*parallel_func)(int, genptr_t);
+struct thread_data {
+    void (*user_func)(int, void *);
+    void *user_arg;
+    int cpu_id;
+    int affinity;
+    struct parallel_info *parent;
+};
 
 
 int
@@ -154,27 +148,11 @@ bu_parallel_id(void)
 int
 bu_is_parallel(void)
 {
-    if (pid_of_initiating_thread != 0)
-	return 1;
+    /* this routine is deprecated, do not use. */
     return 0;
 }
 
 
-void
-bu_kill_parallel(void)
-{
-    if (pid_of_initiating_thread == 0)
-	return;
-
-    if (pid_of_initiating_thread == bu_process_id())
-	return;
-
-    bu_terminate(pid_of_initiating_thread);
-
-    return;
-}
-
-
 void
 bu_nice_set(int newnice)
 {
@@ -201,7 +179,7 @@ bu_nice_set(int newnice)
 }
 
 
-int
+size_t
 bu_avail_cpus(void)
 {
     int ncpu = -1;
@@ -262,6 +240,9 @@ bu_avail_cpus(void)
 #  endif
 
 
+    /*
+     * multithreading support for SunOS 5.X / Solaris 2.x
+     */
 #  if defined(_SC_NPROCESSORS_ONLN)
     /* SUNOS and linux (and now Mac 10.6+) */
     if (ncpu < 0) {
@@ -272,6 +253,7 @@ bu_avail_cpus(void)
     }
 #endif
 
+
 #if defined(_SC_NPROC_ONLN)
     if (ncpu < 0) {
 	ncpu = sysconf(_SC_NPROC_ONLN);
@@ -281,6 +263,7 @@ bu_avail_cpus(void)
     }
 #endif
 
+
 #  if defined(linux)
     if (ncpu < 0) {
 	/* old linux method */
@@ -321,6 +304,7 @@ bu_avail_cpus(void)
     }
 #  endif
 
+
 #endif /* PARALLEL */
 
     if (UNLIKELY(bu_debug & BU_DEBUG_PARALLEL)) {
@@ -342,185 +326,301 @@ bu_avail_cpus(void)
 
 #ifdef PARALLEL
 
-HIDDEN void
-parallel_interface_arg(struct thread_data *user_thread_data)
+#if !defined(HAVE_THREAD_LOCAL)  || !defined(CPP11THREAD)
+/* this function provides book-keeping so that we give out unique
+ * thread identifiers and for tracking a thread's parent context.
+ *
+ * it must be able to keep track of a maximum of MAX_PSW*MAX_PSW
+ * threads, where each primary thread kicks off at least as many
+ * secondary threads.
+ */
+static struct parallel_info *
+parallel_mapping(parallel_action_t action, int id, size_t max)
 {
+    /* container for keeping track of recursive invocation data, limits, current values */
+    static struct parallel_info mapping[MAX_PSW*MAX_PSW] = {{0,0,0,0,0}};
+    int got_cpu;
+
+    switch (action) {
+	case PARALLEL_GET:
+	    if (id < 0) {
+		bu_semaphore_acquire(BU_SEM_THREAD);
+		for (got_cpu = 1; got_cpu < MAX_PSW*MAX_PSW; got_cpu++) {
+		    if (mapping[got_cpu].id == 0) {
+			mapping[got_cpu].id = got_cpu;
+			break;
+		    }
+		}
+		bu_semaphore_release(BU_SEM_THREAD);
+
+		if (got_cpu >= MAX_PSW*MAX_PSW) {
+		    bu_log("Compile-time parallelism limit reached (%d >= %d).\n", got_cpu, MAX_PSW*MAX_PSW);
+		    bu_bomb("Unable to track threading.\n");
+		}
+
+		mapping[got_cpu].started = mapping[got_cpu].finished = 0;
+		mapping[got_cpu].parent = bu_parallel_id();
+
+	    } else {
+		got_cpu = id;
+		if (mapping[got_cpu].id != got_cpu) {
+		    /* presumably id == 0 */
+		    mapping[got_cpu].id = got_cpu;
+		}
+	    }
+
+	    if (mapping[got_cpu].lim == 0 && max > 0)
+		mapping[got_cpu].lim = max;
+
+	    return &mapping[got_cpu];
+
+	case PARALLEL_PUT:
+	    mapping[id].started = mapping[id].finished = mapping[id].lim = mapping[id].parent = 0;
+	    mapping[id].id = 0; /* separate to avoid race */
+    }
+
+    return NULL;
+}
+
+
+static void
+parallel_wait_for_slot(int throttle, struct parallel_info *parent, size_t max_threads)
+{
+    size_t threads = max_threads;
+
+    while (1) {
+	if (parent->started < parent->finished) {
+	    /*bu_log("Warning - parent->started (%d) is less than parent->finished (%d)\n", parent->started, parent->finished);*/
+	    return;
+	}
+	threads = parent->started - parent->finished;
+
+	/*bu_log("threads=%d (start %d - done %d)\n", threads, parent->started, parent->finished);
+	  bu_log("max_threads=%d, throttle: %d\n", max_threads, throttle);*/
+
+	if (threads < max_threads || !throttle) {
+	    return;
+	}
+	bu_snooze(BU_SEC2USEC(1));
+    }
+}
+
+
+HIDDEN void *
+parallel_interface_arg(void *utd)
+{
+    struct thread_data *user_thread_data = (struct thread_data *)utd;
+
     /* keep track of our parallel ID number */
     thread_set_cpu(user_thread_data->cpu_id);
 
     if (user_thread_data->affinity) {
+	int ret;
 	/* lock us onto a core corresponding to our parallel ID number */
-	parallel_set_affinity(user_thread_data->cpu_id);
+	ret = parallel_set_affinity(user_thread_data->cpu_id);
+	if (ret) {
+	    bu_log("WARNING: encountered unexpected problem setting CPU affinity\n");
+	}
     }
 
-    if (!user_thread_data->counted) {
-	bu_semaphore_acquire(BU_SEM_SYSCALL);
-	parallel_nthreads_started++;
-	bu_semaphore_release(BU_SEM_SYSCALL);
-    }
+    bu_semaphore_acquire(BU_SEM_THREAD);
+    user_thread_data->parent->started++;
+    bu_semaphore_release(BU_SEM_THREAD);
 
     (*(user_thread_data->user_func))(user_thread_data->cpu_id, user_thread_data->user_arg);
 
-    if (!user_thread_data->counted) {
-	bu_semaphore_acquire(BU_SEM_SYSCALL);
-	parallel_nthreads_finished++;
-	bu_semaphore_release(BU_SEM_SYSCALL);
-    }
+    bu_semaphore_acquire(BU_SEM_THREAD);
+    user_thread_data->parent->finished++;
+    bu_semaphore_release(BU_SEM_THREAD);
+
+    parallel_mapping(PARALLEL_PUT, user_thread_data->cpu_id, 0);
+
+    return NULL;
 }
 
 
+#if defined(_WIN32)
 /**
- * Interface layer between bu_parallel and the user's function.
- * Necessary so that we can provide unique thread numbers as a
- * parameter to the user's function, and to decrement the global
- * counter when the user's function returns to us (as opposed to
- * dumping core or longjmp'ing too far).
- *
- * Note that not all architectures can pass an argument (e.g. the
- * pointer to the user's function), so we depend on using a global
- * variable to communicate this.  This is no problem, since only one
- * copy of bu_parallel() may be active at any one time.
+ * A separate stub to call parallel_interface_arg that avoids a
+ *  potential crash* on 64-bit windows and calls ExitThread to
+ *  cleanly stop the thread.
+ *  *See ThreadProc MSDN documentation.
  */
-HIDDEN void
-parallel_interface(void)
+HIDDEN DWORD
+parallel_interface_arg_stub(struct thread_data *user_thread_data)
 {
-    struct thread_data user_thread_data_pi;
-    char *libbu_affinity = NULL;
-
-    /* OFF by default until linux issue is debugged */
-    int affinity = 0;
-
-    user_thread_data_pi.user_func = parallel_func;
-    user_thread_data_pi.user_arg  = parallel_arg;
-
-    bu_semaphore_acquire(BU_SEM_SYSCALL);
-    user_thread_data_pi.cpu_id = parallel_nthreads_started++;
-    bu_semaphore_release(BU_SEM_SYSCALL);
-
-    libbu_affinity = getenv("LIBBU_AFFINITY");
-    if (libbu_affinity)
-	affinity = (int)strtol(libbu_affinity, NULL, 0x10);
-
-    user_thread_data_pi.counted = 1;
-    user_thread_data_pi.affinity = affinity;
-
-    parallel_interface_arg(&user_thread_data_pi);
-
-    bu_semaphore_acquire(BU_SEM_SYSCALL);
-    parallel_nthreads_finished++;
-    bu_semaphore_release(BU_SEM_SYSCALL);
+    parallel_interface_arg(user_thread_data);
+    ExitThread(0);
+    return 0; /* Extraneous */
 }
+#endif
 
-
+#endif /* !HAVE_THREAD_LOCAL || !CPP11THREAD */
 #endif /* PARALLEL */
 
 
 void
-bu_parallel(void (*func)(int, genptr_t), int ncpu, genptr_t arg)
+bu_parallel(void (*func)(int, void *), size_t ncpu, void *arg)
 {
-    /* avoid using the 'register' keyword in here "just in case" */
-
 #ifndef PARALLEL
 
-    bu_log("bu_parallel(%d., %p):  Not compiled for PARALLEL machine, running single-threaded\n", ncpu, arg);
+    if (!func)
+	return; /* nothing to do */
+
+    bu_log("bu_parallel(%zu., %p):  Not compiled for PARALLEL machine, running single-threaded\n", ncpu, arg);
     /* do the work anyways */
     (*func)(0, arg);
 
+#elif defined(HAVE_THREAD_LOCAL) && defined(CPP11THREAD)
+
+
+    if (!func)
+	return; /* nothing to do */
+
+    if (ncpu == 1) {
+	func(ncpu, arg);
+	return;
+    } else if (ncpu > MAX_PSW) {
+	bu_log("WARNING: bu_parallel() ncpu(%zd) > MAX_PSW(%d), adjusting ncpu\n", ncpu, MAX_PSW);
+	ncpu = MAX_PSW;
+    }
+
+    parallel_cpp11thread(func, ncpu, arg);
+
 #else
 
-    struct thread_data *user_thread_data_bu;
-    int avail_cpus = 1;
-    int x;
+    struct thread_data *thread_context;
+    rt_thread_t thread_tbl[MAX_PSW];
+    size_t avail_cpus = 1;
+    size_t x;
+    size_t i;
+
+    /* number of threads created/ended */
+    size_t nthreadc;
+    size_t nthreade;
 
     char *libbu_affinity = NULL;
 
-    /* OFF by default until linux issue is debugged */
+    /* OFF by default as modern schedulers are smarter than this. */
     int affinity = 0;
 
-    /*
-     * multithreading support for SunOS 5.X / Solaris 2.x
-     */
-#  if defined(SUNOS) && SUNOS >= 52
-    static int concurrency = 0; /* Max concurrency we have set */
-#  endif
-#  if (defined(SUNOS) && SUNOS >= 52) || defined(HAVE_PTHREAD_H)
-    int nthreadc;
-    int nthreade;
+    /* ncpu == 0 means throttle our thread creation as slots become available */
+    int throttle = 0;
+
+    struct parallel_info *parent;
+
     rt_thread_t thread;
-    rt_thread_t thread_tbl[MAX_PSW];
-    int i;
-#  endif /* SUNOS */
+
+    if (!func)
+	return; /* nothing to do */
 
     if (UNLIKELY(bu_debug & BU_DEBUG_PARALLEL))
-	bu_log("bu_parallel(%d, %p)\n", ncpu, arg);
-
-    if (UNLIKELY(pid_of_initiating_thread))
-	bu_bomb("bu_parallel() called from within parallel section\n");
-
-    pid_of_initiating_thread = bu_process_id();
+	bu_log("bu_parallel(%zu, %p)\n", ncpu, arg);
 
     if (ncpu > MAX_PSW) {
-	bu_log("WARNING: bu_parallel() ncpu(%d) > MAX_PSW(%d), adjusting ncpu\n", ncpu, MAX_PSW);
+	bu_log("WARNING: bu_parallel() ncpu(%zd) > MAX_PSW(%d), adjusting ncpu\n", ncpu, MAX_PSW);
 	ncpu = MAX_PSW;
     }
-    parallel_nthreads_started = 0;
-    parallel_nthreads_finished = 0;
-    parallel_func = func;
-    parallel_arg = arg;
 
     libbu_affinity = getenv("LIBBU_AFFINITY");
     if (libbu_affinity)
 	affinity = (int)strtol(libbu_affinity, NULL, 0x10);
-
-    user_thread_data_bu = (struct thread_data *)bu_calloc(ncpu, sizeof(*user_thread_data_bu), "struct thread_data *user_thread_data_bu");
-
-    /* Fill in the data of user_thread_data_bu structures of all threads */
-    for(x = 0; x < ncpu; x++) {
-	user_thread_data_bu[x].user_func = func;
-	user_thread_data_bu[x].user_arg  = arg;
-	user_thread_data_bu[x].cpu_id    = x;
-	user_thread_data_bu[x].counted   = 0;
-	user_thread_data_bu[x].affinity  = affinity;
+    if (UNLIKELY(bu_debug & BU_DEBUG_PARALLEL)) {
+	if (affinity)
+	    bu_log("CPU affinity enabled. (LIBBU_AFFINITY=%d)\n", affinity);
+	else
+	    bu_log("CPU affinity disabled.\n");
     }
 
     /* if we're in debug mode, allow additional cpus */
     if (!(bu_debug & BU_DEBUG_PARALLEL)) {
+	/* otherwise, limit ourselves to what is actually available */
 	avail_cpus = bu_avail_cpus();
 	if (ncpu > avail_cpus) {
-	    bu_log("%d cpus requested, but only %d available\n", ncpu, avail_cpus);
+	    bu_log("%zd cpus requested, but only %zu available\n", ncpu, avail_cpus);
 	    ncpu = avail_cpus;
 	}
     }
 
+    parent = parallel_mapping(PARALLEL_GET, bu_parallel_id(), ncpu);
+
+    if (ncpu < 1) {
+	/* want to maximize threading potential, but have to throttle
+	 * thread creation.  what is our parallelization limit?
+	 */
+	throttle = 1;
+
+	/* any "zero" limit scopes propagate upward */
+	while (parent->lim == 0 && parent->id > 0) {
+	    parent = parallel_mapping(PARALLEL_GET, parent->parent, ncpu);
+	}
+
+	/* if the top-most parent is unspecified, use all available cpus */
+	if (parent->lim == 0) {
+	    ncpu = bu_avail_cpus();
+	} else {
+	    ncpu = parent->lim;
+	}
+
+	/* starting a "zero" bu_parallel means we get one worker
+	 * thread back (for this thread)
+	 */
+	bu_semaphore_acquire(BU_SEM_THREAD);
+	if (parent->started > 0)
+	    parent->started--;
+	bu_semaphore_release(BU_SEM_THREAD);
+    } else if (ncpu == 1) {
+	/* single cpu case bypasses nearly everything, just invoke */
+	(*func)(0, arg);
+
+	parallel_mapping(PARALLEL_PUT, bu_parallel_id(), 0); /* correct? we never got. */
+	return;
+    }
+
+    thread_context = (struct thread_data *)bu_calloc(ncpu, sizeof(*thread_context), "struct thread_data *thread_context");
+
+    /* Fill in the data of thread_context structures of all threads */
+    for (x = 0; x < ncpu; x++) {
+	struct parallel_info *next = parallel_mapping(PARALLEL_GET, -1, ncpu);
+
+	thread_context[x].user_func = func;
+	thread_context[x].user_arg  = arg;
+	thread_context[x].cpu_id    = next->id;
+	thread_context[x].affinity  = affinity;
+	thread_context[x].parent    = parent;
+    }
+
     /*
      * multithreading support for SunOS 5.X / Solaris 2.x
      */
 #  if defined(SUNOS) && SUNOS >= 52
 
-    thread = 0;
     nthreadc = 0;
 
     /* Give the thread system a hint... */
-    if (ncpu > concurrency) {
-	if (thr_setconcurrency(ncpu)) {
-	    bu_log("ERROR parallel.c/bu_parallel(): thr_setconcurrency(%d) failed\n",
-		   ncpu);
-	    /* Not much to do, lump it */
-	} else {
-	    concurrency = ncpu;
+    {
+	static size_t concurrency = 0; /* Max concurrency we have set */
+	if (ncpu > concurrency) {
+	    if (thr_setconcurrency((int)ncpu)) {
+		bu_log("ERROR parallel.c/bu_parallel(): thr_setconcurrency(%zd) failed\n", ncpu);
+		/* Not much to do, lump it */
+	    } else {
+		concurrency = ncpu;
+	    }
 	}
     }
 
     /* Create the threads */
     for (x = 0; x < ncpu; x++) {
+	parallel_wait_for_slot(throttle, parent, ncpu);
 
-	if (thr_create(0, 0, (void *(*)(void *))parallel_interface_arg, &user_thread_data_bu[x], 0, &thread)) {
+	if (thr_create(0, 0, parallel_interface_arg, &thread_context[x], 0, &thread)) {
 	    bu_log("ERROR: bu_parallel: thr_create(0x0, 0x0, 0x%x, 0x0, 0, 0x%x) failed for processor thread # %d\n",
 		   parallel_interface_arg, &thread, x);
 	    /* Not much to do, lump it */
 	} else {
 	    if (UNLIKELY(bu_debug & BU_DEBUG_PARALLEL))
-		bu_log("bu_parallel(): created thread: (thread: 0x%x) (loop:%d) (nthreadc:%d)\n",
+		bu_log("bu_parallel(): created thread: (thread: 0x%x) (loop:%d) (nthreadc:%zu)\n",
 		       thread, x, nthreadc);
 
 	    thread_tbl[nthreadc] = thread;
@@ -530,19 +630,17 @@ bu_parallel(void (*func)(int, genptr_t), int ncpu, genptr_t arg)
 
     if (UNLIKELY(bu_debug & BU_DEBUG_PARALLEL))
 	for (i = 0; i < nthreadc; i++)
-	    bu_log("bu_parallel(): thread_tbl[%d] = 0x%x\n",
-		   i, thread_tbl[i]);
+	    bu_log("bu_parallel(): thread_tbl[%d] = 0x%x\n", i, thread_tbl[i]);
 
     /*
      * Wait for completion of all threads.  We don't wait for threads
      * in order.  We wait for any old thread but we keep track of how
      * many have returned and whether it is one that we started
      */
-    thread = 0;
     nthreade = 0;
     for (x = 0; x < nthreadc; x++) {
 	if (UNLIKELY(bu_debug & BU_DEBUG_PARALLEL))
-	    bu_log("bu_parallel(): waiting for thread to complete:\t(loop:%d) (nthreadc:%d) (nthreade:%d)\n",
+	    bu_log("bu_parallel(): waiting for thread to complete:\t(loop:%d) (nthreadc:%zu) (nthreade:%zu)\n",
 		   x, nthreadc, nthreade);
 
 	if (thr_join((rt_thread_t)0, &thread, NULL)) {
@@ -566,166 +664,135 @@ bu_parallel(void (*func)(int, genptr_t), int ncpu, genptr_t arg)
 	}
 
 	if (UNLIKELY(bu_debug & BU_DEBUG_PARALLEL))
-	    bu_log("bu_parallel(): thread completed: (thread: %d)\t(loop:%d) (nthreadc:%d) (nthreade:%d)\n",
+	    bu_log("bu_parallel(): thread completed: (thread: %d)\t(loop:%d) (nthreadc:%zu) (nthreade:%zu)\n",
 		   thread, x, nthreadc, nthreade);
     }
 
     if (UNLIKELY(bu_debug & BU_DEBUG_PARALLEL))
-	bu_log("bu_parallel(): %d threads created.  %d threads exited.\n",
-	       nthreadc, nthreade);
+	bu_log("bu_parallel(): %zu threads created.  %zud threads exited.\n", nthreadc, nthreade);
 #  endif	/* SUNOS */
 
 #  if defined(HAVE_PTHREAD_H)
 
-    thread = 0;
-    nthreadc = 0;
-
     /* Create the posix threads.
      *
      * Start at 1 so we can treat the parent as thread 0.
      */
+    nthreadc = 0;
     for (x = 0; x < ncpu; x++) {
 	pthread_attr_t attrs;
 	pthread_attr_init(&attrs);
 	pthread_attr_setstacksize(&attrs, 10*1024*1024);
 
-	if (pthread_create(&thread, &attrs, (void *(*)(void *))parallel_interface_arg, &user_thread_data_bu[x])) {
-	    bu_log("ERROR: bu_parallel: pthread_create(0x0, 0x0, 0x%lx, 0x0, 0, %p) failed for processor thread # %d\n",
+	parallel_wait_for_slot(throttle, parent, ncpu);
+
+	if (pthread_create(&thread, &attrs, parallel_interface_arg, &thread_context[x])) {
+	    bu_log("ERROR: bu_parallel: pthread_create(0x0, 0x0, 0x%lx, 0x0, 0, %p) failed for processor thread # %zu\n",
 		   (unsigned long int)parallel_interface_arg, (void *)&thread, x);
-	    /* Not much to do, lump it */
+
 	} else {
 	    if (UNLIKELY(bu_debug & BU_DEBUG_PARALLEL)) {
-		bu_log("bu_parallel(): created thread: (thread: %p) (loop: %d) (nthreadc: %d)\n",
+		bu_log("bu_parallel(): created thread: (thread: %p) (loop: %zu) (nthreadc: %zu)\n",
 		       (void*)thread, x, nthreadc);
 	    }
-
 	    thread_tbl[nthreadc] = thread;
 	    nthreadc++;
 	}
+
 	/* done with the attributes after create */
 	pthread_attr_destroy(&attrs);
     }
 
-
     if (UNLIKELY(bu_debug & BU_DEBUG_PARALLEL)) {
 	for (i = 0; i < nthreadc; i++) {
-	    bu_log("bu_parallel(): thread_tbl[%d] = %p\n",
-		   i, (void *)thread_tbl[i]);
+	    bu_log("bu_parallel(): thread_tbl[%zu] = %p\n", i, (void *)thread_tbl[i]);
 	}
-#    ifdef SIGINFO
-	/* may be BSD-only (calls _thread_dump_info()) */
-	raise(SIGINFO);
-#    endif
     }
 
     /*
      * Wait for completion of all threads.
      * Wait for them in order.
      */
-    thread = 0;
     nthreade = 0;
     for (x = 0; x < nthreadc; x++) {
 	int ret;
 
 	if (UNLIKELY(bu_debug & BU_DEBUG_PARALLEL))
-	    bu_log("bu_parallel(): waiting for thread %p to complete:\t(loop:%d) (nthreadc:%d) (nthreade:%d)\n",
+	    bu_log("bu_parallel(): waiting for thread %p to complete:\t(loop:%zu) (nthreadc:%zu) (nthreade:%zu)\n",
 		   (void *)thread_tbl[x], x, nthreadc, nthreade);
 
 	if ((ret = pthread_join(thread_tbl[x], NULL)) != 0) {
 	    /* badness happened */
-	    bu_log("pthread_join(thread_tbl[%d]=%p) ret=%d\n", x, (void *)thread_tbl[x], ret);
+	    bu_log("pthread_join(thread_tbl[%zu]=%p) ret=%d\n", x, (void *)thread_tbl[x], ret);
 	}
 
 	nthreade++;
+	thread = thread_tbl[x];
 	thread_tbl[x] = (rt_thread_t)-1;
 
 	if (UNLIKELY(bu_debug & BU_DEBUG_PARALLEL))
-	    bu_log("bu_parallel(): thread completed: (thread: %p)\t(loop:%d) (nthreadc:%d) (nthreade:%d)\n",
+	    bu_log("bu_parallel(): thread completed: (thread: %p)\t(loop:%zu) (nthreadc:%zu) (nthreade:%zu)\n",
 		   (void *)thread, x, nthreadc, nthreade);
+
     }
 
     if (UNLIKELY(bu_debug & BU_DEBUG_PARALLEL))
-	bu_log("bu_parallel(): %d threads created.  %d threads exited.\n",
-	       nthreadc, nthreade);
+	bu_log("bu_parallel(): %zu threads created.  %zu threads exited.\n", nthreadc, nthreade);
 
 #  endif /* end if posix threads */
 
 
-#  ifdef WIN32
-    /* Win32 Threads */
-
-    thread = 0;
-    nthreadc = 0;
-    DWORD dwThreadIdArray[ncpu];
-    HANDLE hThreadArray[ncpu] = {0};
-
+#  ifdef _WIN32
     /* Create the Win32 threads */
+    nthreadc = 0;
+    for (i = 0; i < ncpu; i++) {
+	parallel_wait_for_slot(throttle, parent, ncpu);
 
-    for(int i = 0; i < ncpu; i++) {
-
-	hThreadArray[i] = CreateThread(
+	thread = CreateThread(
 	    NULL,
 	    0,
-	    (LPVOID)parallel_interface,
-	    &user_thread_data_bu[i],
+	    (LPTHREAD_START_ROUTINE)parallel_interface_arg_stub,
+	    &thread_context[i],
 	    0,
-	    &dwThreadIdArray[i]);
-
-	if (hThreadArray[i] == NULL) {
-	    bu_log("bu_parallel(): Error in CreateThread");
-	    bu_exit();
-	}
+	    NULL);
 
+	thread_tbl[i] = thread;
 	nthreadc++;
-    }
-    /* Wait for other threads in the array */
 
-    WaitForMultipleObjects(nthreadc, hThreadArray, TRUE, INFINITE);
+	/* Ensure that all successfully created threads are in sequential order.*/
+	if (thread_tbl[i] == NULL) {
+	    bu_log("bu_parallel(): Error in CreateThread, Win32 error code %d.\n", GetLastError());
+	    --nthreadc;
+	}
+    }
+
+
+    {
+	/* Wait for other threads in the array */
+	DWORD returnCode;
+	returnCode = WaitForMultipleObjects((DWORD)nthreadc, thread_tbl, TRUE, INFINITE);
+	if (returnCode == WAIT_FAILED) {
+	    bu_log("bu_parallel(): Error in WaitForMultipleObjects, Win32 error code %d.\n", GetLastError());
+	}
+    }
+
+    nthreade = 0;
     for (x = 0; x < nthreadc; x++) {
 	int ret;
-	if ((ret = CloseHandle(hThreadArray[x]) != 0)) {
-	    /* The thread not closing properly */
-	    bu_log("bu_parallel(): Error closing threads");
-	    x--;
+	if ((ret = CloseHandle(thread_tbl[x]) == 0)) {
+	    /* Thread didn't close properly if return value is zero; don't retry and potentially loop forever.  */
+	    bu_log("bu_parallel(): Error closing thread %zu of %zu, Win32 error code %d.\n", x, nthreadc, GetLastError());
 	}
+
 	nthreade++;
+	thread_tbl[x] = (rt_thread_t)-1;
     }
 #  endif /* end if Win32 threads */
 
-    /*
-     * Ensure that all the threads are REALLY finished.  On some
-     * systems, if threads core dump, the rest of the gang keeps
-     * going, so this can actually happen (sigh).
-     */
-    if (UNLIKELY(parallel_nthreads_finished != parallel_nthreads_started)) {
-	bu_log("*** ERROR bu_parallel(%d): %d workers did not finish!\n\n",
-	       ncpu, ncpu - parallel_nthreads_finished);
-    }
-    if (UNLIKELY(parallel_nthreads_started != ncpu)) {
-	bu_log("bu_parallel() NOTICE:  only %d workers started, expected %d\n",
-	       parallel_nthreads_started, ncpu);
-    }
-
     if (UNLIKELY(bu_debug & BU_DEBUG_PARALLEL))
-	bu_log("bu_parallel(%d) complete, now serial\n", ncpu);
+	bu_log("bu_parallel(%zd) complete\n", ncpu);
 
-#  if defined(unix) || defined(__unix)
-    /* Cray is known to wander among various pids, perhaps others.
-     *
-     * At this point, all multi-tasking activity should have ceased,
-     * and we should be just a single UNIX process with our original
-     * PID and open file table (kernel struct u).  If not, then any
-     * output may be written into the wrong file.
-     */
-    x = bu_process_id();
-    if (UNLIKELY(pid_of_initiating_thread != x)) {
-	bu_log("WARNING: bu_parallel():  PID of initiating thread changed from %d to %d, open file table may be botched!\n",
-	       pid_of_initiating_thread, x);
-    }
-#  endif
-    pid_of_initiating_thread = 0;	/* No threads any more */
-
-    bu_free(user_thread_data_bu, "struct thread_data *user_thread_data_bu");
+    bu_free(thread_context, "struct thread_data *thread_context");
 
 #endif /* PARALLEL */
 
