@@ -1,6 +1,9 @@
 /*
  * uce-dirent.h - operating system independent dirent implementation
  *
+ * TODO - see if https://github.com/tronkko/dirent has any logic for
+ * newer Windows systems we should be using here...
+ *
  * Copyright (C) 1998-2002  Toni Ronkko
  *
  * Permission is hereby granted, free of charge, to any person obtaining
@@ -25,9 +28,9 @@
  *
  * May 28 1998, Toni Ronkko <tronkko@messi.uku.fi>
  *
- * $Id: uce-dirent.h,v 1.7 2002/05/13 10:48:35 tr Exp $
+ * $Id$
  *
- * $Log: uce-dirent.h,v $
+ * $Log$
  * Revision 1.7  2002/05/13 10:48:35  tr
  * embedded some source code directly to the header so that no source
  * modules need to be included in the MS Visual C project using the
@@ -78,8 +81,8 @@
  * HAVE_DIRECT_H, HAVE_DIR_H, HAVE_NDIR_H, HAVE_SYS_DIR_H and
  * HAVE_SYS_NDIR_H according to the files found.
  */
-#ifndef DIRENT_H
-#define DIRENT_H
+#ifndef LIBBU_UCE_DIRENT_H
+#define LIBBU_UCE_DIRENT_H
 #define DIRENT_H_INCLUDED
 
 /* find out platform */
@@ -149,7 +152,7 @@
 # include <sys/ndir.h>
 # define NAMLEN(dp) ((int)((dp)->d_namlen))
 
-#elif defined(HAVE_DIRECT_H)
+#elif defined(HAVE_DIRECT_H) && !defined(_MSC_VER)
 # include <direct.h>
 # define NAMLEN(dp) ((int)((dp)->d_namlen))
 
@@ -168,6 +171,9 @@
 #elif defined(MSDOS) || defined(WIN32)
 
   /* figure out type of underlying directory interface to be used */
+# if defined(HAVE_DIRECT_H)
+# include <direct.h>
+# endif
 # if defined(WIN32)
 #   define DIRENT_WIN32_INTERFACE
 # elif defined(MSDOS)
@@ -672,4 +678,4 @@ _setdirname (struct DIR *dirp) {
 #endif
 
 
-#endif /*DIRENT_H*/
+#endif /*LIBBU_UCE_DIRENT_H*/
