@@ -1,7 +1,7 @@
 /*                           V L S . C
  * BRL-CAD
  *
- * Copyright (c) 2004-2013 United States Government as represented by
+ * Copyright (c) 2004-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This library is free software; you can redistribute it and/or
@@ -20,21 +20,26 @@
 
 #include "common.h"
 
-#include <stdlib.h>
-#include <ctype.h>
-#include <string.h>
-#include <stdarg.h>
 #include <assert.h>
+#include <ctype.h>
+#include <errno.h> /* for errno */
+#include <limits.h> /* for LONG_MAX */
+#include <stdarg.h>
+#include <stdlib.h> /* for strtol */
+#include <string.h>
 
-#ifdef HAVE_STDINT_H
-#   include <stdint.h>
-#endif
 
 #include "bio.h"
 
-#include "bu.h"
+#include "vmath.h"
+#include "bu/log.h"
+#include "bu/malloc.h"
+#include "bu/parallel.h"
+#include "bu/str.h"
+#include "bu/uuid.h"
+#include "bu/vls.h"
 
-#include "./vls_internals.h"
+#include "./vls_vprintf.h"
 
 /* non-published globals */
 extern const char bu_vls_message[];
@@ -43,10 +48,13 @@ extern const char bu_strdup_message[];
 /* private constants */
 
 /* minimum initial allocation size */
-static const unsigned int _VLS_ALLOC_MIN = 32;
+static const unsigned int VLS_ALLOC_MIN = 32;
+
+/* minimum vls allocation increment size */
+static const size_t VLS_ALLOC_STEP = 128;
 
 /* minimum vls buffer allocation size */
-static const unsigned int _VLS_ALLOC_READ = 4096;
+static const unsigned int VLS_ALLOC_READ = BU_PAGE_SIZE;
 
 void
 bu_vls_init(struct bu_vls *vp)
@@ -95,6 +103,14 @@ bu_vls_vlsinit(void)
 }
 
 
+const char *
+bu_vls_cstr(const struct bu_vls *vp)
+{
+    /* a wrapper for bu_vls_addr, but name is intended to be more intuitive to use */
+    return bu_vls_addr(vp);
+}
+
+
 char *
 bu_vls_addr(const struct bu_vls *vp)
 {
@@ -121,13 +137,13 @@ bu_vls_addr(const struct bu_vls *vp)
 
 
 void
-bu_vls_extend(struct bu_vls *vp, unsigned int extra)
+bu_vls_extend(struct bu_vls *vp, size_t extra)
 {
     BU_CK_VLS(vp);
 
     /* allocate at least 32 bytes (8 or 4 words) extra */
-    if (extra < _VLS_ALLOC_MIN)
-	extra = _VLS_ALLOC_MIN;
+    if (extra < VLS_ALLOC_MIN)
+	extra = VLS_ALLOC_MIN;
 
     /* first time allocation.
      *
@@ -138,7 +154,7 @@ bu_vls_extend(struct bu_vls *vp, unsigned int extra)
      * implementation changes.
      */
     if (vp->vls_max <= 0 || vp->vls_str == (char *)0) {
-	vp->vls_str = (char *)bu_malloc((size_t)extra, bu_vls_message);
+	vp->vls_str = (char *)bu_malloc(extra, bu_vls_message);
 	vp->vls_max = extra;
 	vp->vls_len = 0;
 	vp->vls_offset = 0;
@@ -147,10 +163,10 @@ bu_vls_extend(struct bu_vls *vp, unsigned int extra)
     }
 
     /* make sure to increase in step sized increments */
-    if ((size_t)extra < _VLS_ALLOC_STEP)
-	extra = (unsigned int)_VLS_ALLOC_STEP;
-    else if ((size_t)extra % _VLS_ALLOC_STEP != 0)
-	extra += (unsigned int)(_VLS_ALLOC_STEP - (extra % _VLS_ALLOC_STEP));
+    if (extra < VLS_ALLOC_STEP)
+	extra = VLS_ALLOC_STEP;
+    else if (extra % VLS_ALLOC_STEP != 0)
+	extra += VLS_ALLOC_STEP - (extra % VLS_ALLOC_STEP);
 
     /* need more space? */
     if (vp->vls_offset + vp->vls_len + extra >= vp->vls_max) {
@@ -189,50 +205,48 @@ bu_vls_trunc(struct bu_vls *vp, int len)
 
     if (len < 0) {
 	/* now an absolute length */
-	len = vp->vls_len + len;
+	len = (int)vp->vls_len + len;
     }
     if (vp->vls_len <= (size_t)len)
 	return;
     if (len == 0)
 	vp->vls_offset = 0;
 
-    vp->vls_str[len+vp->vls_offset] = '\0'; /* force null termination */
+    if (vp->vls_str)
+	vp->vls_str[len+vp->vls_offset] = '\0'; /* force null termination */
+
     vp->vls_len = len;
 }
 
 
 void
-bu_vls_trunc2(struct bu_vls *vp, int len)
+bu_vls_nibble(struct bu_vls *vp, b_off_t len)
 {
     BU_CK_VLS(vp);
 
-    if (vp->vls_len <= (size_t)len)
-	return;
-
-    if (len < 0)
-	len = 0;
-    if (len == 0)
-	vp->vls_offset = 0;
-
-    vp->vls_str[len+vp->vls_offset] = '\0'; /* force null termination */
-    vp->vls_len = len;
-}
-
-
-void
-bu_vls_nibble(struct bu_vls *vp, int len)
-{
-    BU_CK_VLS(vp);
-
-    if (len < 0 && (-len) > (ssize_t)vp->vls_offset)
-	len = -vp->vls_offset;
-    if ((size_t)len >= vp->vls_len) {
+    /* if the caller asks to nibble everything, truncate the string */
+    if (len > 0 && (size_t)len >= vp->vls_len) {
 	bu_vls_trunc(vp, 0);
 	return;
     }
 
-    vp->vls_len -= len;
-    vp->vls_offset += len;
+    /* if the caller asks to un-nibble more than we can, we just
+     * unroll to the original beginning of the string
+     */
+    if (len < 0 && (size_t)(-len) > vp->vls_offset) {
+	vp->vls_len += vp->vls_offset;
+	vp->vls_offset = 0;
+	return;
+    }
+
+    /* could be collapsed due to prior, but appease dumb compilers */
+    if (len < 0) {
+	vp->vls_len += (size_t)(-len);
+	vp->vls_offset -= (size_t)(-len);
+    } else {
+	vp->vls_len -= (size_t)len;
+	vp->vls_offset += (size_t)len;
+    }
 }
 
 
@@ -271,7 +285,7 @@ bu_vls_strdup(const struct bu_vls *vp)
     BU_CK_VLS(vp);
 
     len = bu_vls_strlen(vp);
-    str = bu_malloc(len+1, bu_strdup_message);
+    str = (char *)bu_malloc(len+1, bu_strdup_message);
     bu_strlcpy(str, bu_vls_addr(vp), len+1);
 
     return str;
@@ -320,7 +334,7 @@ bu_vls_strcpy(struct bu_vls *vp, const char *s)
     /* cancel offset before extending */
     vp->vls_offset = 0;
     if (len+1 >= vp->vls_max)
-	bu_vls_extend(vp, (unsigned int)len+1);
+	bu_vls_extend(vp, len+1);
 
     memcpy(vp->vls_str, s, len+1); /* include null */
     vp->vls_len = len;
@@ -348,11 +362,11 @@ bu_vls_strncpy(struct bu_vls *vp, const char *s, size_t n)
     /* cancel offset before extending */
     vp->vls_offset = 0;
     if (len+1 >= vp->vls_max)
-	bu_vls_extend(vp, (unsigned int)len+1);
+	bu_vls_extend(vp, len+1);
 
     memcpy(vp->vls_str, s, len);
     vp->vls_str[len] = '\0'; /* force null termination */
-    vp->vls_len = (int)len;
+    vp->vls_len = len;
 }
 
 
@@ -371,10 +385,10 @@ bu_vls_strcat(struct bu_vls *vp, const char *s)
 	return;
 
     if (vp->vls_offset + vp->vls_len + len+1 >= vp->vls_max)
-	bu_vls_extend(vp, (unsigned int)len+1);
+	bu_vls_extend(vp, len+1);
 
     memcpy(vp->vls_str +vp->vls_offset + vp->vls_len, s, len+1); /* include null */
-    vp->vls_len += (int)len;
+    vp->vls_len += len;
 }
 
 
@@ -395,10 +409,10 @@ bu_vls_strncat(struct bu_vls *vp, const char *s, size_t n)
 	return;
 
     if (vp->vls_offset + vp->vls_len + len+1 >= vp->vls_max)
-	bu_vls_extend(vp, (unsigned int)len+1);
+	bu_vls_extend(vp, len+1);
 
     memcpy(vp->vls_str + vp->vls_offset + vp->vls_len, s, len);
-    vp->vls_len += (int)len;
+    vp->vls_len += len;
     vp->vls_str[vp->vls_offset + vp->vls_len] = '\0'; /* force null termination */
 }
 
@@ -413,7 +427,7 @@ bu_vls_vlscat(struct bu_vls *dest, const struct bu_vls *src)
 	return;
 
     if (dest->vls_offset + dest->vls_len + src->vls_len+1 >= dest->vls_max)
-	bu_vls_extend(dest, (unsigned int)src->vls_len+1);
+	bu_vls_extend(dest, src->vls_len+1);
 
     /* copy source string, including null */
     memcpy(dest->vls_str +dest->vls_offset + dest->vls_len, src->vls_str+src->vls_offset, src->vls_len+1);
@@ -435,6 +449,41 @@ bu_vls_vlscatzap(struct bu_vls *dest, struct bu_vls *src)
 }
 
 
+void
+bu_vls_substr(struct bu_vls *dest, const struct bu_vls *src, size_t begin, size_t nchars)
+{
+    size_t len;
+    size_t i, end;
+
+    BU_CK_VLS(src);
+    BU_CK_VLS(dest);
+
+    len = src->vls_len;
+    if (UNLIKELY(len == 0))
+	return;
+    if (UNLIKELY(begin > len))
+	return;
+    if (UNLIKELY(nchars == 0))
+	return;
+
+    if (nchars > len)
+	nchars = len;
+
+    bu_vls_trunc(dest, 0);
+    bu_vls_extend(dest, nchars + 1);
+
+    end = begin + nchars;
+    if (end > len)
+	end = len;
+
+    for (i = begin; i < end; ++i)
+	bu_vls_putc(dest, bu_vls_cstr(src)[i]);
+
+    /* ensure we have an end */
+    bu_vls_putc(dest, '\0');
+}
+
+
 int
 bu_vls_strcmp(struct bu_vls *s1, struct bu_vls *s2)
 {
@@ -529,7 +578,7 @@ bu_vls_write(int fd, const struct bu_vls *vp)
 	return;
 
     bu_semaphore_acquire(BU_SEM_SYSCALL);
-    status = write(fd, vp->vls_str + vp->vls_offset, vp->vls_len);
+    status = write(fd, vp->vls_str + vp->vls_offset, (size_t)vp->vls_len);
     bu_semaphore_release(BU_SEM_SYSCALL);
 
     if (UNLIKELY(status < 0 || (size_t)status != vp->vls_len)) {
@@ -549,7 +598,7 @@ bu_vls_read(struct bu_vls *vp, int fd)
     BU_CK_VLS(vp);
 
     for (;;) {
-	bu_vls_extend(vp, _VLS_ALLOC_READ);
+	bu_vls_extend(vp, VLS_ALLOC_READ);
 	todo = vp->vls_max - vp->vls_len - vp->vls_offset - 1;
 
 	bu_semaphore_acquire(BU_SEM_SYSCALL);
@@ -577,8 +626,8 @@ bu_vls_read(struct bu_vls *vp, int fd)
 int
 bu_vls_gets(struct bu_vls *vp, FILE *fp)
 {
-    int startlen;
-    int endlen;
+    size_t startlen;
+    size_t endlen;
     int done;
     char *bufp;
     char buffer[BUFSIZ+1] = {0};
@@ -588,25 +637,33 @@ bu_vls_gets(struct bu_vls *vp, FILE *fp)
     startlen = bu_vls_strlen(vp);
 
     do {
+	size_t buflen;
+
 	bufp = bu_fgets(buffer, BUFSIZ+1, fp);
 
 	if (!bufp)
 	    return -1;
 
+	/* an empty parse means we're done */
+	buflen = strlen(bufp);
+	if (buflen == 0) {
+	    break;
+	}
+
 	/* keep reading if we just filled the buffer */
-	if ((strlen(bufp) == BUFSIZ) && (bufp[BUFSIZ-1] != '\n') && (bufp[BUFSIZ-1] != '\r')) {
+	if ((buflen == BUFSIZ) && (bufp[BUFSIZ-1] != '\n') && (bufp[BUFSIZ-1] != '\r')) {
 	    done = 0;
 	} else {
 	    done = 1;
 	}
 
 	/* strip the trailing EOL (or at least part of it) */
-	if ((bufp[strlen(bufp)-1] == '\n') || (bufp[strlen(bufp)-1] == '\r'))
-	    bufp[strlen(bufp)-1] = '\0';
+	if ((bufp[buflen-1] == '\n') || (bufp[buflen-1] == '\r'))
+	    bufp[buflen-1] = '\0';
 
 	/* handle \r\n lines */
-	if (bufp[strlen(bufp)-1] == '\r')
-	    bufp[strlen(bufp)-1] = '\0';
+	if (bufp[buflen-1] == '\r')
+	    bufp[buflen-1] = '\0';
 
 	bu_vls_printf(vp, "%s", bufp);
     } while (!done);
@@ -616,7 +673,7 @@ bu_vls_gets(struct bu_vls *vp, FILE *fp)
     if (endlen < startlen)
 	return -1;
 
-    return endlen;
+    return (int)endlen;
 }
 
 void
@@ -625,7 +682,7 @@ bu_vls_putc(struct bu_vls *vp, int c)
     BU_CK_VLS(vp);
 
     if (vp->vls_offset + vp->vls_len+1 >= vp->vls_max)
-	bu_vls_extend(vp, (unsigned int)_VLS_ALLOC_STEP);
+	bu_vls_extend(vp, VLS_ALLOC_STEP);
 
     vp->vls_str[vp->vls_offset + vp->vls_len++] = (char)c;
     vp->vls_str[vp->vls_offset + vp->vls_len] = '\0'; /* force null termination */
@@ -651,10 +708,10 @@ void
 bu_vls_printf(struct bu_vls *vls, const char *fmt, ...)
 {
     va_list ap;
-    va_start(ap, fmt);
 
     BU_CK_VLS(vls);
 
+    va_start(ap, fmt);
     bu_vls_vprintf(vls, fmt, ap);
     va_end(ap);
 }
@@ -664,36 +721,37 @@ void
 bu_vls_sprintf(struct bu_vls *vls, const char *fmt, ...)
 {
     va_list ap;
-    va_start(ap, fmt);
 
     BU_CK_VLS(vls);
 
     bu_vls_trunc(vls, 0); /* poof */
+
+    va_start(ap, fmt);
     bu_vls_vprintf(vls, fmt, ap);
     va_end(ap);
 }
 
 
 void
-bu_vls_spaces(struct bu_vls *vp, int cnt)
+bu_vls_spaces(struct bu_vls *vp, size_t cnt)
 {
     BU_CK_VLS(vp);
 
-    if (UNLIKELY(cnt <= 0))
+    if (UNLIKELY(cnt == 0))
 	return;
     if (vp->vls_offset + vp->vls_len + cnt+1 >= vp->vls_max)
-	bu_vls_extend(vp, (unsigned)cnt);
+	bu_vls_extend(vp, cnt);
 
-    memset(vp->vls_str + vp->vls_offset + vp->vls_len, ' ', (size_t)cnt);
+    memset(vp->vls_str + vp->vls_offset + vp->vls_len, ' ', cnt);
     vp->vls_len += cnt;
 }
 
 
-int
+size_t
 bu_vls_print_positions_used(const struct bu_vls *vp)
 {
     char *start;
-    int used;
+    size_t used;
 
     BU_CK_VLS(vp);
 
@@ -720,18 +778,18 @@ bu_vls_detab(struct bu_vls *vp)
 {
     struct bu_vls src = BU_VLS_INIT_ZERO;
     char *cp;
-    int used;
+    size_t used;
 
     BU_CK_VLS(vp);
 
     bu_vls_vlscatzap(&src, vp);	/* make temporary copy of src */
-    bu_vls_extend(vp, (unsigned int)bu_vls_strlen(&src) + (unsigned int)_VLS_ALLOC_STEP);
+    bu_vls_extend(vp, bu_vls_strlen(&src) + VLS_ALLOC_STEP);
 
     cp = bu_vls_addr(&src);
     used = 0;
     while (*cp != '\0') {
 	if (*cp == '\t') {
-	    int todo;
+	    size_t todo;
 	    todo = 8 - (used % 8);
 	    bu_vls_spaces(vp, todo);
 	    used += todo;
@@ -750,19 +808,102 @@ bu_vls_detab(struct bu_vls *vp)
 
 
 void
-bu_vls_prepend(struct bu_vls *vp, char *str)
+bu_vls_prepend(struct bu_vls *vp, const char *str)
 {
     size_t len = strlen(str);
 
-    bu_vls_extend(vp, (unsigned int)len);
+    bu_vls_extend(vp, len);
 
     /* memmove is supposed to be safe even if strings overlap */
-    memmove(vp->vls_str+vp->vls_offset+len, vp->vls_str+vp->vls_offset, vp->vls_len);
+    memmove(vp->vls_str+vp->vls_offset+len, vp->vls_str+vp->vls_offset, vp->vls_len+1);
 
     /* insert the data at the head of the string */
     memcpy(vp->vls_str+vp->vls_offset, str, len);
 
-    vp->vls_len += (int)len;
+    vp->vls_len += len;
+}
+
+HIDDEN int
+vls_char_in_set(const char *c, const char *str)
+{
+    unsigned int i = 0;
+    if (!c || !str) return 0;
+    for (i = 0; i < strlen(str); i++) {
+	if ((unsigned char)(*c) == (unsigned char )(str[i])) return 1;
+    }
+    return 0;
+}
+
+int
+bu_vls_simplify(struct bu_vls *vp, const char *keep, const char *de_dup, const char *trim)
+{
+    struct bu_vls tmpstr = BU_VLS_INIT_ZERO;
+    const char *tr;
+    unsigned char *c = NULL;
+    int ret = 0;
+
+    if (!vp || bu_vls_strlen(vp) == 0) return 0;
+
+    /* Map for replacing diacritic characters(code >= 192) from the extended
+     * ASCII set with a specific mapping from the standard ASCII set. See
+     * http://stackoverflow.com/questions/14094621/ for more info */
+
+    /*   "ÀÁÂÃÄÅÆÇÈÉÊËÌÍÎÏÐÑÒÓÔÕÖ×ØÙÚÛÜÝÞßàáâãäåæçèéêëìíîïðñòóôõö÷øùúûüýþÿ" */
+    tr = "AAAAAAECEEEEIIIIDNOOOOOx0UUUUYPsaaaaaaeceeeeiiiiOnooooo/0uuuuypy";
+
+    c = (unsigned char *)bu_vls_addr(vp);
+    while (*c) {
+	unsigned char ch = (*c);
+	if (vls_char_in_set((const char *)&ch, keep)) {
+	    bu_vls_putc(&tmpstr, ch);
+	} else {
+	    if (isalnum(ch)) {
+		bu_vls_putc(&tmpstr, ch);
+	    } else {
+		if (ch >= 192 && ch < strlen(tr) + 192) {
+		    bu_vls_putc(&tmpstr, tr[ch-192]);
+		} else {
+		    bu_vls_putc(&tmpstr, '_');
+		}
+	    }
+	}
+	c++;
+    }
+
+    if (de_dup) {
+	struct bu_vls dd_str = BU_VLS_INIT_ZERO;
+	unsigned char ch, currh;
+	c = (unsigned char *)bu_vls_addr(&tmpstr);
+	if (*c) bu_vls_putc(&dd_str, (unsigned char)(*c));
+	while (*c) {
+	    ch = (unsigned char)(*c);
+	    c++;
+	    currh = (unsigned char)(*c);
+	    if (!(ch == currh && vls_char_in_set((const char *)&currh, de_dup))) {
+		bu_vls_putc(&dd_str, currh);
+	    }
+	}
+	bu_vls_sprintf(&tmpstr, "%s", bu_vls_addr(&dd_str));
+	bu_vls_free(&dd_str);
+    }
+
+
+    if (trim) {
+	int ccnt = 0;
+	c = (unsigned char *)bu_vls_addr(&tmpstr);
+	while (*c && vls_char_in_set((const char *)c, trim)) {ccnt++; c++;}
+	if (ccnt) bu_vls_nibble(&tmpstr, ccnt);
+	ccnt = 0;
+	c = (unsigned char *)&(bu_vls_addr(&tmpstr)[(strlen(bu_vls_addr(&tmpstr)) - 1)]);
+	while (*c && vls_char_in_set((const char *)c, trim)) {ccnt++; c--;}
+	if (ccnt)
+	    bu_vls_trunc(&tmpstr, (int)(bu_vls_strlen(&tmpstr) - ccnt));
+    }
+
+    ret = (!bu_vls_strcmp(&tmpstr, vp)) ? 0 : 1;
+    bu_vls_sprintf(vp, "%s", bu_vls_addr(&tmpstr));
+    bu_vls_free(&tmpstr);
+    return ret;
 }
 
 /*
