@@ -1,7 +1,7 @@
 /*                 T R I _ I N T E R S E C T . H
  * BRL-CAD
  *
- * Copyright (c) 2011-2013 United States Government as represented by
+ * Copyright (c) 2011-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This library is free software; you can redistribute it and/or
@@ -23,15 +23,18 @@
  * Intersect 2 triangles using a modified Möller routine.
  */
 
+#ifndef LIBGCV_TRI_INTERSECT_H
+#define LIBGCV_TRI_INTERSECT_H
 
-int gcv_tri_tri_intersect_with_isectline(
-		struct soup_s *UNUSED(left),
-		struct soup_s *UNUSED(right),
-		struct face_s *lf,
-		struct face_s *rf,
-		int *coplanar,
-		point_t *isectpt,
-		const struct bn_tol *tol);
+__BEGIN_DECLS
+
+/* FIXME: need not be public API */
+GCV_EXPORT int
+gcv_tri_tri_intersect_with_isectline(struct soup_s *UNUSED(left), struct soup_s *UNUSED(right), struct face_s *lf, struct face_s *rf, int *coplanar, point_t *isectpt, const struct bn_tol *tol);
+
+__END_DECLS
+
+#endif /* LIBGCV_TRI_INTERSECT_H */
 
 /*
  * Local Variables:
