@@ -2,7 +2,7 @@
 /*                        S H _ O S L . C
  * BRL-CAD
  *
- * Copyright (c) 2004-2013 United States Government as represented by
+ * Copyright (c) 2004-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This library is free software; you can redistribute it and/or
@@ -20,7 +20,7 @@
  */
 /** @file liboptical/sh_osl.c
  *
- * This shader is an interface for OSL shadying system. More information
+ * This shader is an interface for OSL shading system. More information
  * when more features are implemented.
  *
  */
@@ -38,7 +38,7 @@
 #include "vmath.h"
 #include "raytrace.h"
 #include "optical.h"
-#include "light.h"
+#include "optical/light.h"
 
 #define OSL_MAGIC 0x1837    /* make this a unique number for each shader */
 #define CK_OSL_SP(_p) BU_CKMAG(_p, OSL_MAGIC, "osl_specific")
@@ -85,13 +85,13 @@ struct bu_structparse osl_print_tab[] = {
 extern "C" {
 
     HIDDEN int osl_setup(register struct region *rp, struct bu_vls *matparm,
-			 genptr_t *dpp, const struct mfuncs *mfp,
+			 void **dpp, const struct mfuncs *mfp,
 			 struct rt_i *rtip);
 
     HIDDEN int osl_render(struct application *ap, const struct partition *pp,
-			  struct shadework *swp, genptr_t dp);
-    HIDDEN void osl_print(register struct region *rp, genptr_t dp);
-    HIDDEN void osl_free(genptr_t cp);
+			  struct shadework *swp, void *dp);
+    HIDDEN void osl_print(register struct region *rp, void *dp);
+    HIDDEN void osl_free(void *cp);
 
 }
 
@@ -117,28 +117,28 @@ osl_parse_edge(char *edge, ShaderEdge &sh_edge)
     ShaderParam sh_param1, sh_param2;
 
     /* Name of the first shader */
-    if((item = strtok(edge, "#")) == NULL){
+    if ((item = strtok(edge, "#")) == NULL) {
 	fprintf(stderr, "[Error] Expecting the first shader name, found NULL.\n");
 	return -1;
     }
     sh_param1.layername = item;
 
     /* Parameter of the first shader */
-    if((item = strtok(NULL, "#")) == NULL){
+    if ((item = strtok(NULL, "#")) == NULL) {
 	fprintf(stderr, "[Error] Expecting the parameter of the first shader, found NULL.\n");
 	return -1;
     }
     sh_param1.paramname = item;
 
     /* Name of the first shader */
-    if((item = strtok(NULL, "#")) == NULL){
+    if ((item = strtok(NULL, "#")) == NULL) {
 	fprintf(stderr, "[Error] Expecting the second shader name, found NULL.\n");
 	return -1;
     }
     sh_param2.layername = item;
 
     /* Name of the first shader */
-    if((item = strtok(NULL, "#")) == NULL){
+    if ((item = strtok(NULL, "#")) == NULL) {
 	fprintf(stderr, "[Error] Expecting the parameter of the second shader, found NULL.\n");
 	return -1;
     }
@@ -158,14 +158,14 @@ osl_parse_shader(char *shadername, ShaderInfo &sh_info)
     sh_info.shadername = "../shaders/" + std::string(item);
 
     /* Check for parameters */
-    while((item = strtok(NULL, "#")) != NULL){
+    while ((item = strtok(NULL, "#")) != NULL) {
 
 	/* Setting layer name, in case we're doing a shader group */
 	if (BU_STR_EQUAL(item, "layername")) {
 
 	    /* Get the name of the layer being set */
 	    item = strtok(NULL, "#");
-	    if(item == NULL){
+	    if (item == NULL) {
 		fprintf(stderr, "[Error] Missing layer name\n");
 		return -1;
 	    }
@@ -177,13 +177,13 @@ osl_parse_shader(char *shadername, ShaderInfo &sh_info)
 
 	    /* Get the type of parameter being set */
 	    item = strtok(NULL, "#");
-	    if(item == NULL){
+	    if (item == NULL) {
 		fprintf(stderr, "[Error] Missing parameter type\n");
 		return -1;
 	    }
 	    else if (BU_STR_EQUAL(item, "int")) {
 		item = strtok(NULL, "#");
-		if(item == NULL){
+		if (item == NULL) {
 		    fprintf(stderr, "[Error] Missing float value\n");
 		    return -1;
 		}
@@ -192,7 +192,7 @@ osl_parse_shader(char *shadername, ShaderInfo &sh_info)
 	    }
 	    else if (BU_STR_EQUAL(item, "float")) {
 		item = strtok(NULL, "#");
-		if(item == NULL){
+		if (item == NULL) {
 		    fprintf(stderr, "[Error] Missing float value\n");
 		    return -1;
 		}
@@ -201,9 +201,9 @@ osl_parse_shader(char *shadername, ShaderInfo &sh_info)
 	    }
 	    else if (BU_STR_EQUAL(item, "color")) {
 		Color3 color_value;
-		for(int i=0; i<3; i++){
+		for (int i=0; i<3; i++) {
 		    item = strtok(NULL, "#");
-		    if(item == NULL){
+		    if (item == NULL) {
 			fprintf(stderr, "[Error] Missing %d-th component of color value\n", i);
 			return -1;
 		    }
@@ -220,9 +220,9 @@ osl_parse_shader(char *shadername, ShaderInfo &sh_info)
 		else if (BU_STR_EQUAL(item, "vector")) type = TypeDesc::TypeVector;
 
 		Vec3 vec_value;
-		for(int i=0; i<3; i++){
+		for (int i=0; i<3; i++) {
 		    item = strtok(NULL, "#");
-		    if(item == NULL){
+		    if (item == NULL) {
 			fprintf(stderr, "[Error] Missing %d-th component of %s value\n", i, type_name.c_str());
 			return -1;
 		    }
@@ -231,13 +231,13 @@ osl_parse_shader(char *shadername, ShaderInfo &sh_info)
 		ShaderInfo::TypeVec type_vec(type, vec_value);
 		sh_info.vparam.push_back(std::make_pair(param_name, type_vec));
 	    }
-	    else if (BU_STR_EQUAL(item, "matrix")){
+	    else if (BU_STR_EQUAL(item, "matrix")) {
 		fprintf(stderr, "matrix\n");
 		Matrix44 mat_value;
-		for(int i=0; i<4; i++)
-		    for(int j=0; j<4; j++){
+		for (int i=0; i<4; i++)
+		    for (int j=0; j<4; j++) {
 			item = strtok(NULL, "#");
-			if(item == NULL){
+			if (item == NULL) {
 			    fprintf(stderr, "[Error] Missing %d-th component of matrix value\n", i*4 + j);
 			    return -1;
 			}
@@ -249,7 +249,7 @@ osl_parse_shader(char *shadername, ShaderInfo &sh_info)
 	    }
 	    else if (BU_STR_EQUAL(item, "string")) {
 		item = strtok(NULL, "#");
-		if(item == NULL){
+		if (item == NULL) {
 		    fprintf(stderr, "[Error] Missing string\n");
 		    return -1;
 		}
@@ -354,8 +354,7 @@ osl_parse(const struct bu_vls *in_vls, ShaderGroupInfo &group_info)
     return 0;
 }
 
-/* O S L _ S E T U P
- *
+/*
  * This routine is called (at prep time)
  * once for each region which uses this shader.
  * Any shader-specific initialization should be done here.
@@ -369,7 +368,7 @@ osl_parse(const struct bu_vls *in_vls, ShaderGroupInfo &group_info)
 extern "C" {
 
 HIDDEN int osl_setup(register struct region *rp, struct bu_vls *matparm,
-		     genptr_t *dpp, const struct mfuncs *mfp,
+		     void **dpp, const struct mfuncs *mfp,
 		     struct rt_i *rtip)
 {
     register struct osl_specific *osl_sp;
@@ -379,7 +378,7 @@ HIDDEN int osl_setup(register struct region *rp, struct bu_vls *matparm,
     BU_CK_VLS(matparm);
     RT_CK_REGION(rp);
 
-    if (rdebug&RDEBUG_SHADE)
+    if (optical_debug&OPTICAL_DEBUG_SHADE)
 	bu_log("osl_setup(%s)\n", rp->reg_name);
 
     /* Get memory for the shader parameters and shader-specific data */
@@ -395,7 +394,7 @@ HIDDEN int osl_setup(register struct region *rp, struct bu_vls *matparm,
 
     /* Parse the user's arguments to fill osl specifics */
     ShaderGroupInfo group_info;
-    if (osl_parse(matparm, group_info) < 0){
+    if (osl_parse(matparm, group_info) < 0) {
 	return -1;
     }
 
@@ -404,32 +403,26 @@ HIDDEN int osl_setup(register struct region *rp, struct bu_vls *matparm,
      * -----------------------------------
      */
     /* If OSL system was not initialized yet, do it */
-    if (oslr == NULL){
+    if (oslr == NULL) {
 	oslr = new OSLRenderer();
     }
     /* Add this shader to OSL system */
     osl_sp->shader_ref = oslr->AddShader(group_info);
 
-    if (rdebug&RDEBUG_SHADE) {
+    if (optical_debug&OPTICAL_DEBUG_SHADE) {
 	bu_struct_print(" Parameters:", osl_print_tab, (char *)osl_sp);
     }
 
     return 1;
 }
 
-/*
- * O S L _ P R I N T
- */
-HIDDEN void osl_print(register struct region *rp, genptr_t dp)
+HIDDEN void osl_print(register struct region *rp, void *dp)
 {
     bu_struct_print(rp->reg_name, osl_print_tab, (char *)dp);
 }
 
 
-/*
- * O S L _ F R E E
- */
-HIDDEN void osl_free(genptr_t cp)
+HIDDEN void osl_free(void *cp)
 {
     register struct osl_specific *osl_sp =
 	(struct osl_specific *)cp;
@@ -448,9 +441,6 @@ osl_refraction_hit(struct application *ap, struct partition *PartHeadp, struct s
      */
     struct partition *pp;
 
-    /* will serve as a pointer for the entry and exit hitpoints */
-    struct hit *hitp;
-
     /* will serve as a pointer to the solid primitive we hit */
     struct soltab *stp;
 
@@ -509,14 +499,12 @@ osl_refraction_hit(struct application *ap, struct partition *PartHeadp, struct s
 
 
 /*
- * O S L _ R E N D E R
- *
  * This is called (from viewshade() in shade.c) once for each hit point
  * to be shaded.  The purpose here is to fill in values in the shadework
  * structure.
  */
 HIDDEN int osl_render(struct application *ap, const struct partition *pp,
-		      struct shadework *swp, genptr_t dp)
+		      struct shadework *swp, void *dp)
 /* defined in ../h/shadework.h */
 /* ptr to the shader-specific struct */
 {
@@ -532,7 +520,7 @@ HIDDEN int osl_render(struct application *ap, const struct partition *pp,
     RT_CHECK_PT(pp);
     CK_OSL_SP(osl_sp);
 
-    if (rdebug&RDEBUG_SHADE)
+    if (optical_debug&OPTICAL_DEBUG_SHADE)
 	bu_struct_print("osl_render Parameters:", osl_print_tab,
 			(char *)osl_sp);
 
@@ -540,21 +528,21 @@ HIDDEN int osl_render(struct application *ap, const struct partition *pp,
 
     /* Check if it is the first time this thread is calling this function */
     bool visited = false;
-    for(size_t i = 0; i < visited_addrs.size(); i++){
-	if(ap->a_resource == visited_addrs[i]){
+    for (size_t i = 0; i < visited_addrs.size(); i++) {
+	if (ap->a_resource == visited_addrs[i]) {
 	    visited = true;
 	    thread_info = thread_infos[i];
 	    break;
 	}
     }
-    if(!visited){
+    if (!visited) {
 	visited_addrs.push_back(ap->a_resource);
 	/* Get thread specific information from OSLRender system */
 	thread_info = oslr->CreateThreadInfo();
 	thread_infos.push_back(thread_info);
     }
 
-    if(ap->a_level == 0){
+    if (ap->a_level == 0) {
 	default_a_hit = ap->a_hit; /* save the default hit callback (colorview @ rt) */
 	default_a_miss = ap->a_miss;
     }
@@ -605,7 +593,7 @@ HIDDEN int osl_render(struct application *ap, const struct partition *pp,
     Color3 weight = oslr->QueryColor(&info);
 
     /* Fire another ray */
-    if((info.out_ray_type & RAY_REFLECT) || (info.out_ray_type & RAY_TRANSMIT)){
+    if ((info.out_ray_type & RAY_REFLECT) || (info.out_ray_type & RAY_TRANSMIT)) {
 
 	struct application new_ap;
 	RT_APPLICATION_INIT(&new_ap);
@@ -620,7 +608,7 @@ HIDDEN int osl_render(struct application *ap, const struct partition *pp,
 	VMOVE(new_ap.a_ray.r_pt, info.out_ray.origin);
 
 	/* This next ray represents refraction */
-	if (info.out_ray_type & RAY_TRANSMIT){
+	if (info.out_ray_type & RAY_TRANSMIT) {
 
 	    /* Displace the hit point a little bit in the direction
 	       of the next ray */
