@@ -1,7 +1,7 @@
 /*                       S H _ T E X T . C
  * BRL-CAD
  *
- * Copyright (c) 1998-2013 United States Government as represented by
+ * Copyright (c) 1998-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This library is free software; you can redistribute it and/or
@@ -55,10 +55,10 @@ struct txt_specific {
 #define TX_NULL ((struct txt_specific *)0)
 #define TX_O(m) bu_offsetof(struct txt_specific, m)
 
+/* local sp_hook functions */
+HIDDEN void txt_transp_hook(const struct bu_structparse *, const char *, void *, const char *, void *);
+HIDDEN void txt_source_hook(const struct bu_structparse *, const char *, void *, const char *, void *);
 
-HIDDEN void txt_transp_hook(struct bu_structparse *ptab, char *name, char *cp, char *value);
-HIDDEN void txt_source_hook(const struct bu_structparse *ip, const char *sp_name, genptr_t base, char *p);
-HIDDEN int txt_load_datasource(struct txt_specific *texture, struct db_i *dbInstance, const long unsigned int size);
 
 
 struct bu_structparse txt_parse[] = {
@@ -66,7 +66,7 @@ struct bu_structparse txt_parse[] = {
     {"%V",	1, "file", TX_O(tx_name),		txt_source_hook, NULL, NULL },
     {"%V",	1, "obj", TX_O(tx_name),		txt_source_hook, NULL, NULL },
     {"%V",	1, "object", TX_O(tx_name),		txt_source_hook, NULL, NULL },
-    {"%V",	1, "texture", TX_O(tx_name),	 BU_STRUCTPARSE_FUNC_NULL, NULL, NULL },
+    {"%V",	1, "texture", TX_O(tx_name),	        BU_STRUCTPARSE_FUNC_NULL, NULL, NULL },
     {"%d",	1, "w",		TX_O(tx_w),		BU_STRUCTPARSE_FUNC_NULL, NULL, NULL },
     {"%d",	1, "n",		TX_O(tx_n),		BU_STRUCTPARSE_FUNC_NULL, NULL, NULL },
     {"%d",	1, "l",		TX_O(tx_n),		BU_STRUCTPARSE_FUNC_NULL, NULL, NULL },
@@ -85,31 +85,37 @@ struct bu_structparse txt_parse[] = {
  * fail.
  */
 HIDDEN void
-txt_source_hook(const struct bu_structparse *UNUSED(ip), const char *sp_name, genptr_t base, char *UNUSED(p))
+txt_source_hook(const struct bu_structparse *UNUSED(sdp),
+		const char *name,
+		void *base,
+		const char *UNUSED(value),
+		void *UNUSED(data))
 {
     struct txt_specific *textureSpecific = (struct txt_specific *)base;
-    if (bu_strncmp(sp_name, "file", 4)==0) {
+    if (bu_strncmp(name, "file", 4) == 0) {
 	textureSpecific->tx_datasrc=TXT_SRC_FILE;
-    } else if (bu_strncmp(sp_name, "obj", 3)==0) {
-	textureSpecific->tx_datasrc=TXT_SRC_OBJECT;
+    } else if (bu_strncmp(name, "obj", 3) == 0) {
+	textureSpecific->tx_datasrc = TXT_SRC_OBJECT;
     } else {
-	textureSpecific->tx_datasrc=TXT_SRC_AUTO;
+	textureSpecific->tx_datasrc = TXT_SRC_AUTO;
     }
 }
 
 
 /*
- * T X T _ T R A N S P _ H O O K
- *
  * Hooked function, called by bu_structparse
  */
 HIDDEN void
-txt_transp_hook(struct bu_structparse *ptab, char *name, char *cp, char *UNUSED(value))
+txt_transp_hook(const struct bu_structparse *sdp,
+		const char *name,
+		void *base,
+		const char *UNUSED(value),
+		void *UNUSED(data))
 {
     register struct txt_specific *tp =
-	(struct txt_specific *)cp;
+	(struct txt_specific *)base;
 
-    if (BU_STR_EQUAL(name, txt_parse[0].sp_name) && ptab == txt_parse) {
+    if (BU_STR_EQUAL(name, txt_parse[0].sp_name) && sdp == txt_parse) {
 	tp->tx_trans_valid = 1;
     } else {
 	bu_log("file:%s, line:%d txt_transp_hook name:(%s) instead of (%s)\n",
@@ -119,8 +125,6 @@ txt_transp_hook(struct bu_structparse *ptab, char *name, char *cp, char *UNUSED(
 
 
 /*
- * t x t _ l o a d _ d a t a s o u r c e
- *
  * This is a helper routine used in txt_setup() to load a texture either from
  * a file or from a db object.  The resources are released in txt_free()
  * (there is no specific unload_datasource function).
@@ -136,7 +140,7 @@ txt_load_datasource(struct txt_specific *texture, struct db_i *dbInstance, const
 	bu_bomb("ERROR: txt_load_datasource() received NULL arg (struct txt_specific *)\n");
     }
 
-    bu_log("Loading texture %s [%V]...", texture->tx_datasrc==TXT_SRC_AUTO?"from auto-determined datasource":texture->tx_datasrc==TXT_SRC_OBJECT?"from a database object":texture->tx_datasrc==TXT_SRC_FILE?"from a file":"from an unknown source (ERROR)", &texture->tx_name);
+    bu_log("Loading texture %s [%s]...", texture->tx_datasrc==TXT_SRC_AUTO?"from auto-determined datasource":texture->tx_datasrc==TXT_SRC_OBJECT?"from a database object":texture->tx_datasrc==TXT_SRC_FILE?"from a file":"from an unknown source (ERROR)", bu_vls_addr(&texture->tx_name));
 
     /* if the source is auto or object, we try to load the object */
     if ((texture->tx_datasrc==TXT_SRC_AUTO) || (texture->tx_datasrc==TXT_SRC_OBJECT)) {
@@ -175,9 +179,12 @@ txt_load_datasource(struct txt_specific *texture, struct db_i *dbInstance, const
 
 	    /* check size of object */
 	    if (texture->tx_binunifp->count < size) {
-		bu_log("\nWARNING: %V needs %d bytes, binary object only has %lu\n", texture->tx_name, size, texture->tx_binunifp->count);
+		bu_log("\nWARNING: %s needs %zu bytes, binary object only has %zu\n", bu_vls_addr(&texture->tx_name), size, texture->tx_binunifp->count);
 	    } else if (texture->tx_binunifp->count > size) {
-		bu_log("\nWARNING: Binary object is larger than specified texture size\n\tBinary Object: %zu pixels\n\tSpecified Texture Size: %zu pixels\n...continuing to load using image subsection...", texture->tx_binunifp->count);
+		bu_log("\nWARNING: Binary object is larger than specified texture size\n"
+		       "\tBinary Object: %zu pixels\n\tSpecified Texture Size: %zu pixels\n"
+		       "...continuing to load using image subsection...",
+		       texture->tx_binunifp->count, size);
 	    }
 	}
     }
@@ -193,9 +200,12 @@ txt_load_datasource(struct txt_specific *texture, struct db_i *dbInstance, const
 	    return -1;				/* FAIL */
 
 	if (texture->tx_mp->buflen < size) {
-	    bu_log("\nWARNING: %V needs %d bytes, file only has %lu\n", &texture->tx_name, size, texture->tx_mp->buflen);
+	    bu_log("\nWARNING: %s needs %zu bytes, file only has %zu\n", bu_vls_addr(&texture->tx_name), size, texture->tx_mp->buflen);
 	} else if (texture->tx_mp->buflen > size) {
-	    bu_log("\nWARNING: Texture file size is larger than specified texture size\n\tInput File: %zu pixels\n\tSpecified Texture Size: %lu pixels\n...continuing to load using image subsection...", texture->tx_mp->buflen, size);
+	    bu_log("\nWARNING: Texture file size is larger than specified texture size\n"
+		   "\tInput File: %zu pixels\n\tSpecified Texture Size: %zu pixels\n"
+		   "...continuing to load using image subsection...",
+		   texture->tx_mp->buflen, size);
 	}
 
     }
@@ -207,8 +217,6 @@ txt_load_datasource(struct txt_specific *texture, struct db_i *dbInstance, const
 
 
 /*
- * T X T _ R E N D E R
- *
  * Given a u, v coordinate within the texture (0 <= u, v <= 1.0),
  * return a pointer to the relevant pixel.
  *
@@ -216,7 +224,7 @@ txt_load_datasource(struct txt_specific *texture, struct db_i *dbInstance, const
  * which works out very naturally for the indexing scheme.
  */
 HIDDEN int
-txt_render(struct application *ap, const struct partition *pp, struct shadework *swp, genptr_t dp)
+txt_render(struct application *ap, const struct partition *pp, struct shadework *swp, void *dp)
 {
     register struct txt_specific *tp =
 	(struct txt_specific *)dp;
@@ -225,13 +233,14 @@ txt_render(struct application *ap, const struct partition *pp, struct shadework
     register fastf_t r, g, b;
     struct uvcoord uvc;
     long tmp;
+    char color_warn = 0;
 
     RT_CK_AP(ap);
     RT_CHECK_PT(pp);
 
     uvc = swp->sw_uv;
 
-    if (rdebug & RDEBUG_SHADE)
+    if (optical_debug & OPTICAL_DEBUG_SHADE)
 	bu_log("in txt_render(): du=%g, dv=%g\n",
 	       uvc.uv_du, uvc.uv_dv);
 
@@ -258,8 +267,8 @@ txt_render(struct application *ap, const struct partition *pp, struct shadework
      * texture isn't and can't be read, give debug colors
      */
 
-    if ((bu_vls_strlen(&tp->tx_name)<=0) || (!tp->tx_mp && !tp->tx_binunifp)) {
-	bu_log("WARNING: texture [%V] could not be read\n", &tp->tx_name);
+    if ((bu_vls_strlen(&tp->tx_name) <= 0) || (!tp->tx_mp && !tp->tx_binunifp)) {
+	bu_log("WARNING: texture [%s] could not be read\n", bu_vls_addr(&tp->tx_name));
 	VSET(swp->sw_color, uvc.uv_u, 0, uvc.uv_v);
 	if (swp->sw_reflect > 0 || swp->sw_transmit > 0)
 	    (void)rr_render(ap, pp, swp);
@@ -287,7 +296,7 @@ txt_render(struct application *ap, const struct partition *pp, struct shadework
     if (xmax > 1) xmax = 1;
     if (ymax > 1) ymax = 1;
 
-    if (rdebug & RDEBUG_SHADE)
+    if (optical_debug & OPTICAL_DEBUG_SHADE)
 	bu_log("footprint in texture space is (%g %g) <-> (%g %g)\n",
 	       xmin * (tp->tx_w-1), ymin * (tp->tx_n-1),
 	       xmax * (tp->tx_w-1), ymax * (tp->tx_n-1));
@@ -295,22 +304,28 @@ txt_render(struct application *ap, const struct partition *pp, struct shadework
     dx = (int)(xmax * (tp->tx_w-1)) - (int)(xmin * (tp->tx_w-1));
     dy = (int)(ymax * (tp->tx_n-1)) - (int)(ymin * (tp->tx_n-1));
 
-    if (rdebug & RDEBUG_SHADE)
+    if (optical_debug & OPTICAL_DEBUG_SHADE)
 	bu_log("\tdx = %d, dy = %d\n", dx, dy);
 
     if (dx == 0 && dy == 0) {
 	/* No averaging necessary */
 
 	register unsigned char *cp=NULL;
+	unsigned int offset;
 
+	offset = (int)(ymin * (tp->tx_n-1)) * tp->tx_w * 3 + (int)(xmin * (tp->tx_w-1)) * 3;
 	if (tp->tx_mp) {
-	    cp = ((unsigned char *)(tp->tx_mp->buf)) +
-		(int)(ymin * (tp->tx_n-1)) * tp->tx_w * 3 +
-		(int)(xmin * (tp->tx_w-1)) * 3;
+	    if (offset >= tp->tx_mp->buflen) {
+		offset %= tp->tx_mp->buflen;
+		color_warn = 1;
+	    }
+	    cp = ((unsigned char *)(tp->tx_mp->buf)) + offset;
 	} else if (tp->tx_binunifp) {
-	    cp = ((unsigned char *)(tp->tx_binunifp->u.uint8)) +
-		(int)(ymin * (tp->tx_n-1)) * tp->tx_w * 3 +
-		(int)(xmin * (tp->tx_w-1)) * 3;
+	    if (offset >= tp->tx_binunifp->count) {
+		offset %= tp->tx_binunifp->count;
+		color_warn = 1;
+	    }
+	    cp = ((unsigned char *)(tp->tx_binunifp->u.uint8)) + offset;
 	} else {
 	    bu_bomb("sh_text.c -- No texture data found\n");
 	}
@@ -320,11 +335,12 @@ txt_render(struct application *ap, const struct partition *pp, struct shadework
     } else {
 	/* Calculate weighted average of cells in footprint */
 
-	fastf_t tot_area=0.0;
+	fastf_t tot_area = 0.0;
 	fastf_t cell_area;
 	int start_line, stop_line, line;
 	int start_col, stop_col, col;
 	fastf_t xstart, xstop, ystart, ystop;
+	unsigned int max_offset;
 
 	xstart = xmin * (tp->tx_w-1);
 	xstop = xmax * (tp->tx_w-1);
@@ -338,16 +354,28 @@ txt_render(struct application *ap, const struct partition *pp, struct shadework
 
 	r = g = b = 0.0;
 
-	if (rdebug & RDEBUG_SHADE) {
+	if (optical_debug & OPTICAL_DEBUG_SHADE) {
 	    bu_log("\thit in texture space = (%g %g)\n", uvc.uv_u * (tp->tx_w-1), uvc.uv_v * (tp->tx_n-1));
 	    bu_log("\t averaging from  (%g %g) to (%g %g)\n", xstart, ystart, xstop, ystop);
 	    bu_log("\tcontributions to average:\n");
 	}
 
+	max_offset = stop_line * tp->tx_w * 3 + (int)(xstart) * 3 + (dx + 1) * 3;
+	if (tp->tx_mp) {
+	    if (max_offset > tp->tx_mp->buflen) {
+		color_warn = 1;
+	    }
+	} else if (tp->tx_binunifp) {
+	    if (max_offset > tp->tx_binunifp->count) {
+		color_warn = 1;
+	    }
+	}
+
 	for (line = start_line; line <= stop_line; line++) {
 	    register unsigned char *cp=NULL;
 	    fastf_t line_factor;
 	    fastf_t line_upper, line_lower;
+	    unsigned int offset;
 
 	    line_upper = line + 1.0;
 	    if (line_upper > ystop)
@@ -357,12 +385,17 @@ txt_render(struct application *ap, const struct partition *pp, struct shadework
 		line_lower = ystart;
 	    line_factor = line_upper - line_lower;
 
+	    offset = line * tp->tx_w * 3 + (int)(xstart) * 3;
 	    if (tp->tx_mp) {
-		cp = ((unsigned char *)(tp->tx_mp->buf)) +
-		    line * tp->tx_w * 3 + (int)(xstart) * 3;
+		if (offset >= tp->tx_mp->buflen) {
+		    offset %= tp->tx_mp->buflen;
+		}
+		cp = ((unsigned char *)(tp->tx_mp->buf)) + offset;
 	    } else if (tp->tx_binunifp) {
-		cp = ((unsigned char *)(tp->tx_binunifp->u.uint8)) +
-		    line * tp->tx_w * 3 + (int)(xstart) * 3;
+		if (offset >= tp->tx_binunifp->count) {
+		    offset %= tp->tx_binunifp->count;
+                }
+		cp = ((unsigned char *)(tp->tx_binunifp->u.uint8)) + offset;
 	    } else {
 		/* not reachable */
 		bu_bomb("sh_text.c -- Unable to read datasource\n");
@@ -379,7 +412,7 @@ txt_render(struct application *ap, const struct partition *pp, struct shadework
 		cell_area = line_factor * (col_upper - col_lower);
 		tot_area += cell_area;
 
-		if (rdebug & RDEBUG_SHADE)
+		if (optical_debug & OPTICAL_DEBUG_SHADE)
 		    bu_log("\t %d %d %d weight=%g (from col=%d line=%d)\n", *cp, *(cp+1), *(cp+2), cell_area, col, line);
 
 		r += (*cp++) * cell_area;
@@ -393,15 +426,25 @@ txt_render(struct application *ap, const struct partition *pp, struct shadework
 	b /= tot_area;
     }
 
-    if (rdebug & RDEBUG_SHADE)
+    /*
+     * If the actual image file size is less than the provided size,
+     * warn the user by displaying a color closer to red.
+     */
+    if (color_warn == 1) {
+	r = (r + 255.0) / 2;
+	g /= 2;
+	b /= 2;
+    }
+
+    if (optical_debug & OPTICAL_DEBUG_SHADE)
 	bu_log(" average: %g %g %g\n", r, g, b);
 
     if (!tp->tx_trans_valid) {
     opaque:
 	VSET(swp->sw_color,
-	     r * bn_inv255,
-	     g * bn_inv255,
-	     b * bn_inv255);
+	     r / 255.0,
+	     g / 255.0,
+	     b / 255.0);
 
 	if (swp->sw_reflect > 0 || swp->sw_transmit > 0)
 	    (void)rr_render(ap, pp, swp);
@@ -430,8 +473,6 @@ txt_render(struct application *ap, const struct partition *pp, struct shadework
 
 
 /*
- * B W T X T _ R E N D E R
- *
  * Given a u, v coordinate within the texture (0 <= u, v <= 1.0),
  * return the filtered intensity.
  *
@@ -439,7 +480,7 @@ txt_render(struct application *ap, const struct partition *pp, struct shadework
  * which works out very naturally for the indexing scheme.
  */
 HIDDEN int
-bwtxt_render(struct application *ap, const struct partition *pp, struct shadework *swp, genptr_t dp)
+bwtxt_render(struct application *ap, const struct partition *pp, struct shadework *swp, void *dp)
 {
     register struct txt_specific *tp =
 	(struct txt_specific *)dp;
@@ -457,7 +498,7 @@ bwtxt_render(struct application *ap, const struct partition *pp, struct shadewor
      * If no texture file present, or if
      * texture isn't and can't be read, give debug colors
      */
-    if ((bu_vls_strlen(&tp->tx_name)<=0) || (!tp->tx_mp && !tp->tx_binunifp)) {
+    if ((bu_vls_strlen(&tp->tx_name) <= 0) || (!tp->tx_mp && !tp->tx_binunifp)) {
 	VSET(swp->sw_color, uvc.uv_u, 0, uvc.uv_v);
 	if (swp->sw_reflect > 0 || swp->sw_transmit > 0)
 	    (void)rr_render(ap, pp, swp);
@@ -509,7 +550,7 @@ bwtxt_render(struct application *ap, const struct partition *pp, struct shadewor
     if (dx < 1) dx = 1;
     if (dy < 1) dy = 1;
     bw = 0;
-    for (line=0; line<dy; line++) {
+    for (line = 0; line < dy; line++) {
 	register unsigned char *cp=NULL;
 	register unsigned char *ep;
 
@@ -533,7 +574,7 @@ bwtxt_render(struct application *ap, const struct partition *pp, struct shadewor
     if (!tp->tx_trans_valid) {
     opaque:
 	VSETALL(swp->sw_color,
-		bw * bn_inv255 / (dx*dy));
+		(bw / 255.0) / (dx*dy));
 	if (swp->sw_reflect > 0 || swp->sw_transmit > 0)
 	    (void)rr_render(ap, pp, swp);
 	return 1;
@@ -555,11 +596,8 @@ bwtxt_render(struct application *ap, const struct partition *pp, struct shadewor
 }
 
 
-/*
- * T X T _ S E T U P
- */
 HIDDEN int
-txt_setup(register struct region *rp, struct bu_vls *matparm, genptr_t *dpp, const struct mfuncs *mfp, struct rt_i *rtip)
+txt_setup(register struct region *rp, struct bu_vls *matparm, void **dpp, const struct mfuncs *mfp, struct rt_i *rtip)
 {
     register struct txt_specific *tp;
     int pixelbytes = 3;
@@ -581,7 +619,7 @@ txt_setup(register struct region *rp, struct bu_vls *matparm, genptr_t *dpp, con
     tp->tx_mp = NULL;
 
     /* load given values */
-    if (bu_struct_parse(matparm, txt_parse, (char *)tp) < 0) {
+    if (bu_struct_parse(matparm, txt_parse, (char *)tp, NULL) < 0) {
 	BU_PUT(tp, struct txt_specific);
 	return -1;
     }
@@ -591,7 +629,7 @@ txt_setup(register struct region *rp, struct bu_vls *matparm, genptr_t *dpp, con
     if (tp->tx_n < 0) tp->tx_n = tp->tx_w;
     if (tp->tx_trans_valid) rp->reg_transmit = 1;
     BU_CK_VLS(&tp->tx_name);
-    if (bu_vls_strlen(&tp->tx_name)<=0) return -1;
+    if (bu_vls_strlen(&tp->tx_name) <= 0) return -1;
     /* !?! if (tp->tx_name[0] == '\0') return -1;	*/ /* FAIL, no file */
 
     if (BU_STR_EQUAL(mfp->mf_name, "bwtexture")) pixelbytes = 1;
@@ -603,7 +641,7 @@ txt_setup(register struct region *rp, struct bu_vls *matparm, genptr_t *dpp, con
     }
 
 
-    if (rdebug & RDEBUG_SHADE) {
+    if (optical_debug & OPTICAL_DEBUG_SHADE) {
 	bu_log("txt_setup: texture loaded!  type=%s name=%s\n", tp->tx_datasrc==TXT_SRC_AUTO?"auto":tp->tx_datasrc==TXT_SRC_OBJECT?"object":tp->tx_datasrc==TXT_SRC_FILE?"file":"unknown", bu_vls_addr(&tp->tx_name));
 	bu_struct_print("texture", txt_parse, (char *)tp);
     }
@@ -612,29 +650,23 @@ txt_setup(register struct region *rp, struct bu_vls *matparm, genptr_t *dpp, con
 }
 
 
-/*
- * T X T _ P R I N T
- */
 HIDDEN void
-txt_print(register struct region *rp, genptr_t UNUSED(dp))
+txt_print(register struct region *rp, void *UNUSED(dp))
 {
     bu_struct_print(rp->reg_name, txt_parse, (char *)rp->reg_udata);
 }
 
 
-/*
- * T X T _ F R E E
- */
 HIDDEN void
-txt_free(genptr_t cp)
+txt_free(void *cp)
 {
     struct txt_specific *tp =	(struct txt_specific *)cp;
 
     bu_vls_free(&tp->tx_name);
     if (tp->tx_binunifp) rt_binunif_free(tp->tx_binunifp);
-    if (tp->tx_mp) bu_close_mapped_file(tp->tx_mp);
-    tp->tx_binunifp = GENPTR_NULL; /* sanity */
-    tp->tx_mp = GENPTR_NULL; /* sanity */
+    bu_close_mapped_file(tp->tx_mp);
+    tp->tx_binunifp = (struct rt_binunif_internal *)NULL; /* sanity */
+    tp->tx_mp = (struct bu_mapped_file *)NULL; /* sanity */
     BU_PUT(cp, struct txt_specific);
 }
 
@@ -655,11 +687,8 @@ struct bu_structparse ckr_parse[] = {
 };
 
 
-/*
- * C K R _ R E N D E R
- */
 HIDDEN int
-ckr_render(struct application *ap, const struct partition *pp, register struct shadework *swp, genptr_t dp)
+ckr_render(struct application *ap, const struct partition *pp, register struct shadework *swp, void *dp)
 {
     register struct ckr_specific *ckp =
 	(struct ckr_specific *)dp;
@@ -676,9 +705,9 @@ ckr_render(struct application *ap, const struct partition *pp, register struct s
     }
 
     VSET(swp->sw_color,
-	 (unsigned char)cp[0] * bn_inv255,
-	 (unsigned char)cp[1] * bn_inv255,
-	 (unsigned char)cp[2] * bn_inv255);
+	 (unsigned char)cp[0] / 255.0,
+	 (unsigned char)cp[1] / 255.0,
+	 (unsigned char)cp[2] / 255.0);
 
     if (swp->sw_reflect > 0 || swp->sw_transmit > 0)
 	(void)rr_render(ap, pp, swp);
@@ -687,13 +716,8 @@ ckr_render(struct application *ap, const struct partition *pp, register struct s
 }
 
 
-/*
- * C K R _ S E T U P
- */
 HIDDEN int
-ckr_setup(register struct region *UNUSED(rp), struct bu_vls *matparm, genptr_t *dpp, const struct mfuncs *UNUSED(mfp), struct rt_i *UNUSED(rtip))
-
-
+ckr_setup(register struct region *UNUSED(rp), struct bu_vls *matparm, void **dpp, const struct mfuncs *UNUSED(mfp), struct rt_i *UNUSED(rtip))
 /* New since 4.4 release */
 {
     register struct ckr_specific *ckp;
@@ -704,7 +728,7 @@ ckr_setup(register struct region *UNUSED(rp), struct bu_vls *matparm, genptr_t *
     ckp->ckr_a[0] = ckp->ckr_a[1] = ckp->ckr_a[2] = 255;
     ckp->ckr_b[0] = ckp->ckr_b[1] = ckp->ckr_b[2] = 0;
     ckp->ckr_scale = 2.0;
-    if (bu_struct_parse(matparm, ckr_parse, (char *)ckp) < 0) {
+    if (bu_struct_parse(matparm, ckr_parse, (char *)ckp, NULL) < 0) {
 	BU_PUT(ckp, struct ckr_specific);
 	return -1;
     }
@@ -718,34 +742,26 @@ ckr_setup(register struct region *UNUSED(rp), struct bu_vls *matparm, genptr_t *
 }
 
 
-/*
- * C K R _ P R I N T
- */
 HIDDEN void
-ckr_print(register struct region *rp, genptr_t UNUSED(dp))
+ckr_print(register struct region *rp, void *UNUSED(dp))
 {
-    bu_struct_print(rp->reg_name, ckr_parse, rp->reg_udata);
+    bu_struct_print(rp->reg_name, ckr_parse, (const char *)rp->reg_udata);
 }
 
 
-/*
- * C K R _ F R E E
- */
 HIDDEN void
-ckr_free(genptr_t cp)
+ckr_free(void *cp)
 {
     BU_PUT(cp, struct ckr_specific);
 }
 
 
 /*
- * T S T M _ R E N D E R
- *
  * Render a map which varies red with U and blue with V values.
  * Mostly useful for debugging ft_uv() routines.
  */
 HIDDEN int
-tstm_render(struct application *ap, const struct partition *pp, register struct shadework *swp, genptr_t UNUSED(dp))
+tstm_render(struct application *ap, const struct partition *pp, register struct shadework *swp, void *UNUSED(dp))
 {
     VSET(swp->sw_color, swp->sw_uv.uv_u, 0, swp->sw_uv.uv_v);
 
@@ -769,11 +785,8 @@ static vect_t star_colors[] = {
 };
 
 
-/*
- * S T A R _ R E N D E R
- */
 HIDDEN int
-star_render(register struct application *ap, const struct partition *pp, struct shadework *swp, genptr_t UNUSED(dp))
+star_render(register struct application *ap, const struct partition *pp, struct shadework *swp, void *UNUSED(dp))
 {
     /* Probably want to diddle parameters based on what part of sky */
     if (bn_rand0to1(ap->a_resource->re_randptr) >= 0.98) {
@@ -799,8 +812,6 @@ star_render(register struct application *ap, const struct partition *pp, struct
 
 
 /*
- * B M P _ R E N D E R
- *
  * Given a u, v coordinate within the texture (0 <= u, v <= 1.0),
  * compute a new surface normal.
  * For now we come up with a local coordinate system, and
@@ -811,7 +822,7 @@ star_render(register struct application *ap, const struct partition *pp, struct
  * which works out very naturally for the indexing scheme.
  */
 HIDDEN int
-bmp_render(struct application *ap, const struct partition *pp, struct shadework *swp, genptr_t dp)
+bmp_render(struct application *ap, const struct partition *pp, struct shadework *swp, void *dp)
 {
     register struct txt_specific *tp =
 	(struct txt_specific *)dp;
@@ -825,7 +836,7 @@ bmp_render(struct application *ap, const struct partition *pp, struct shadework
      * If no texture file present, or if
      * texture isn't and can't be read, give debug color.
      */
-    if ((bu_vls_strlen(&tp->tx_name)<=0) || (!tp->tx_mp && !tp->tx_binunifp)) {
+    if ((bu_vls_strlen(&tp->tx_name) <= 0) || (!tp->tx_mp && !tp->tx_binunifp)) {
 	VSET(swp->sw_color, swp->sw_uv.uv_u, 0, swp->sw_uv.uv_v);
 	if (swp->sw_reflect > 0 || swp->sw_transmit > 0)
 	    (void)rr_render(ap, pp, swp);
@@ -866,7 +877,7 @@ bmp_render(struct application *ap, const struct partition *pp, struct shadework
     pertU = ((fastf_t)(*cp) - 128.0) / 128.0;
     pertV = ((fastf_t)(*(cp+2)) - 128.0) / 128.0;
 
-    if (rdebug&RDEBUG_LIGHT) {
+    if (optical_debug&OPTICAL_DEBUG_LIGHT) {
 	VPRINT("normal", swp->sw_hit.hit_normal);
 	VPRINT("u", u);
 	VPRINT("v", v);
@@ -875,7 +886,7 @@ bmp_render(struct application *ap, const struct partition *pp, struct shadework
     }
     VJOIN2(swp->sw_hit.hit_normal, swp->sw_hit.hit_normal, pertU, u, pertV, v);
     VUNITIZE(swp->sw_hit.hit_normal);
-    if (rdebug&RDEBUG_LIGHT) {
+    if (optical_debug&OPTICAL_DEBUG_LIGHT) {
 	VPRINT("after", swp->sw_hit.hit_normal);
     }
 
@@ -886,11 +897,8 @@ bmp_render(struct application *ap, const struct partition *pp, struct shadework
 }
 
 
-/*
- * E N V M A P _ S E T U P
- */
 HIDDEN int
-envmap_setup(register struct region *rp, struct bu_vls *matparm, genptr_t *UNUSED(dpp), const struct mfuncs *UNUSED(mfp), struct rt_i *rtip)
+envmap_setup(register struct region *rp, struct bu_vls *matparm, void **UNUSED(dpp), const struct mfuncs *UNUSED(mfp), struct rt_i *rtip)
 {
     struct mfuncs *shaders = MF_NULL;
 
@@ -916,7 +924,8 @@ envmap_setup(register struct region *rp, struct bu_vls *matparm, genptr_t *UNUSE
 	optical_shader_init(&shaders);
     }
     if (mlib_setup(&shaders, &env_region, rtip) < 0)
-	bu_log("envmap_setup() material '%s' failed\n", env_region.reg_mater);
+	bu_log("envmap_setup() material '%s' failed\n",
+	       env_region.reg_mater.ma_shader);
 
 
     return 0;		/* This region should be dropped */
@@ -924,44 +933,37 @@ envmap_setup(register struct region *rp, struct bu_vls *matparm, genptr_t *UNUSE
 
 
 /*
- * M L I B _ Z E R O
- *
  * Regardless of arguments, always return zero.
  * Useful mostly as a stub print, and/or free routine.
  */
 /* VARARGS */
 int
-mlib_zero(struct application *UNUSED(a), const struct partition *UNUSED(b), struct shadework *UNUSED(c), genptr_t UNUSED(d))
+mlib_zero(struct application *UNUSED(a), const struct partition *UNUSED(b), struct shadework *UNUSED(c), void *UNUSED(d))
 {
     return 0;
 }
 
 
 /*
- * M L I B _ O N E
- *
  * Regardless of arguments, always return one.
  * Useful mostly as a stub setup routine.
  */
 /* VARARGS */
 int
-mlib_one(struct region *UNUSED(a), struct bu_vls *UNUSED(b), genptr_t *UNUSED(c), const struct mfuncs *UNUSED(d), struct rt_i *UNUSED(e))
+mlib_one(struct region *UNUSED(a), struct bu_vls *UNUSED(b), void **UNUSED(c), const struct mfuncs *UNUSED(d), struct rt_i *UNUSED(e))
 {
     return 1;
 }
 
 
-/*
- * M L I B _ V O I D
- */
 /* VARARGS */
 void
-mlib_void(struct region *UNUSED(a), genptr_t UNUSED(b))
+mlib_void(struct region *UNUSED(a), void *UNUSED(b))
 {
 }
 
 static void
-mlib_void2(genptr_t UNUSED(b))
+mlib_void2(void *UNUSED(b))
 {
 }
 
