@@ -1,7 +1,7 @@
 /*                           P K G . C
  * BRL-CAD
  *
- * Copyright (c) 2004-2013 United States Government as represented by
+ * Copyright (c) 2004-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This library is free software; you can redistribute it and/or
@@ -27,11 +27,11 @@
 #include "common.h"
 
 #include <stdlib.h>
-#include <stdio.h>
 #include <ctype.h>		/* used by inet_addr() routine, below */
 #include <time.h>
 #include <string.h>
 
+
 #ifdef HAVE_SYS_PARAM_H
 #  include <sys/param.h>
 #endif
@@ -58,15 +58,6 @@
 #  undef LITTLE_ENDIAN		/* defined in netinet/{ip.h, tcp.h} */
 #endif
 
-#ifdef HAVE_UNISTD_H
-#  include <unistd.h>
-#else
-#  include <windows.h>
-#  include <io.h>
-#  include <fcntl.h>
-#endif
-
-
 /* Not all systems with "BSD Networking" include UNIX Domain sockets */
 #ifdef HAVE_SYS_UN_H
 #  include <sys/un.h>		/* UNIX Domain sockets */
@@ -81,13 +72,19 @@
 #  include <sys/netinet/tcp.h>
 #endif
 
-#ifdef HAVE_WRITEV
+#ifdef HAVE_SYS_UIO_H
 #  include <sys/uio.h>		/* for struct iovec (writev) */
 #endif
 
 #include <errno.h>
+
+#include "bio.h"
+
 #include "pkg.h"
 
+#if defined(HAVE_GETHOSTBYNAME) && !defined(HAVE_DECL_GETHOSTBYNAME) && !defined(_WINSOCKAPI_)
+extern struct hostent *gethostbyname(const char *);
+#endif
 
 /* compatibility for pedantic bug/limitation in gcc 4.6.2, need to
  * mark macros as extensions else they may emit "ISO C forbids
@@ -135,14 +132,10 @@
 #  define DMSG(s) /**/
 #endif
 
-#if !defined(_WIN32) || defined(__CYGWIN__)
-extern int errno;
-#endif
-
 int pkg_nochecking = 0;	/* set to disable extra checking for input */
 int pkg_permport = 0;	/* TCP port that pkg_permserver() is listening on XXX */
 
-#define MAX_PKG_ERRBUF_SIZE 80
+#define MAX_PKG_ERRBUF_SIZE 128
 static char _pkg_errbuf[MAX_PKG_ERRBUF_SIZE] = {0};
 static FILE *_pkg_debug = (FILE*)NULL;
 
@@ -198,8 +191,6 @@ pkg_plong(char *buf, unsigned long l)
 
 
 /**
- * _ P K G _ T I M E S T A M P
- *
  * Output a timestamp to the log, suitable for starting each line
  * with.
  *
@@ -231,14 +222,12 @@ _pkg_timestamp(void)
 
 
 /**
- * _ P K G _ E R R L O G
- *
  * Default error logger.  Writes to stderr.
  *
  * This is a private implementation function.
  */
 static void
-_pkg_errlog(char *s)
+_pkg_errlog(const char *s)
 {
     if (_pkg_debug) {
 	_pkg_timestamp();
@@ -250,14 +239,12 @@ _pkg_errlog(char *s)
 
 
 /**
- * _ P K G _ P E R R O R
- *
  * Produce a perror on the error logging output.
  *
  * This is a private implementation function.
  */
 static void
-_pkg_perror(void (*errlog) (char *msg), char *s)
+_pkg_perror(void (*errlog)(const char *msg), const char *s)
 {
     snprintf(_pkg_errbuf, MAX_PKG_ERRBUF_SIZE, "%s: ", s);
 
@@ -273,8 +260,6 @@ _pkg_perror(void (*errlog) (char *msg), char *s)
 
 
 /**
- * _ P K G _ M A K E C O N N
- *
  * Malloc and initialize a pkg_conn structure.  We have already
  * connected to a client or server on the given file descriptor.
  *
@@ -285,7 +270,7 @@ _pkg_perror(void (*errlog) (char *msg), char *s)
  */
 static
 struct pkg_conn *
-_pkg_makeconn(int fd, const struct pkg_switch *switchp, void (*errlog) (char *msg))
+_pkg_makeconn(int fd, const struct pkg_switch *switchp, void (*errlog)(const char *msg))
 {
     struct pkg_conn *pc;
 
@@ -320,8 +305,6 @@ _pkg_makeconn(int fd, const struct pkg_switch *switchp, void (*errlog) (char *ms
 
 
 /**
- * _ P K G _ C K _ D E B U G
- *
  * This is a private implementation function.
  */
 static void
@@ -350,7 +333,7 @@ _pkg_ck_debug(void)
 
 
 struct pkg_conn *
-pkg_open(const char *host, const char *service, const char *protocol, const char *uname, const char *UNUSED(passwd), const struct pkg_switch *switchp, void (*errlog) (char *msg))
+pkg_open(const char *host, const char *service, const char *protocol, const char *uname, const char *UNUSED(passwd), const struct pkg_switch *switchp, void (*errlog)(const char *msg))
 {
 #ifdef HAVE_WINSOCK_H
     LPHOSTENT lpHostEntry;
@@ -468,7 +451,7 @@ pkg_open(const char *host, const char *service, const char *protocol, const char
 	    return PKC_ERROR;
 	}
 	sinhim.sin_family = hp->h_addrtype;
-	memcpy((char *)&sinhim.sin_addr, hp->h_addr, (size_t)hp->h_length);
+	memcpy((char *)&sinhim.sin_addr, hp->h_addr_list[0], (size_t)hp->h_length);
     }
     addr = (struct sockaddr *) &sinhim;
     addrlen = sizeof(struct sockaddr_in);
@@ -508,7 +491,7 @@ pkg_open(const char *host, const char *service, const char *protocol, const char
 
 
 struct pkg_conn *
-pkg_transerver(const struct pkg_switch *switchp, void (*errlog)(char *))
+pkg_transerver(const struct pkg_switch *switchp, void (*errlog)(const char *))
 {
     struct pkg_conn *conn;
 
@@ -534,7 +517,7 @@ pkg_transerver(const struct pkg_switch *switchp, void (*errlog)(char *))
  * This is a private implementation function.
  */
 static int
-_pkg_permserver_impl(struct in_addr iface, const char *service, const char *protocol, int backlog, void (*errlog)(char *msg))
+_pkg_permserver_impl(struct in_addr iface, const char *service, const char *protocol, int backlog, void (*errlog)(const char *msg))
 {
     struct servent *sp;
     int pkg_listenfd;
@@ -700,7 +683,7 @@ _pkg_permserver_impl(struct in_addr iface, const char *service, const char *prot
 
 
 int
-pkg_permserver(const char *service, const char *protocol, int backlog, void (*errlog) (char *msg))
+pkg_permserver(const char *service, const char *protocol, int backlog, void (*errlog)(const char *msg))
 {
     struct in_addr iface;
     iface.s_addr = INADDR_ANY;
@@ -709,7 +692,7 @@ pkg_permserver(const char *service, const char *protocol, int backlog, void (*er
 
 
 int
-pkg_permserver_ip(const char *ipOrHostname, const char *service, const char *protocol, int backlog, void (*errlog)(char *msg))
+pkg_permserver_ip(const char *ipOrHostname, const char *service, const char *protocol, int backlog, void (*errlog)(const char *msg))
 {
     struct hostent* host;
     struct in_addr iface;
@@ -720,7 +703,7 @@ pkg_permserver_ip(const char *ipOrHostname, const char *service, const char *pro
 	} else {
 	    /* XXX gethostbyname is deprecated on Windows */
 	    host = gethostbyname(ipOrHostname);
-	    iface = *(struct in_addr*)host->h_addr;
+	    iface = *(struct in_addr*)host->h_addr_list[0];
 	}
 	return _pkg_permserver_impl(iface, service, protocol, backlog, errlog);
     } else {
@@ -731,7 +714,7 @@ pkg_permserver_ip(const char *ipOrHostname, const char *service, const char *pro
 
 
 struct pkg_conn *
-pkg_getclient(int fd, const struct pkg_switch *switchp, void (*errlog) (char *msg), int nodelay)
+pkg_getclient(int fd, const struct pkg_switch *switchp, void (*errlog)(const char *msg), int nodelay)
 {
     int s2;
 #ifdef HAVE_WINSOCK_H
@@ -772,6 +755,7 @@ pkg_getclient(int fd, const struct pkg_switch *switchp, void (*errlog) (char *ms
 #endif
 
     do  {
+	errno = 0;
 #ifdef HAVE_WINSOCK_H
 	s2 = accept(fd, (struct sockaddr *)NULL, NULL);
 #else
@@ -781,6 +765,7 @@ pkg_getclient(int fd, const struct pkg_switch *switchp, void (*errlog) (char *ms
 	    if (errno == EINTR)
 		continue;
 #ifdef HAVE_WINSOCK_H
+		errno = WSAGetLastError();
 	    if (errno == WSAEWOULDBLOCK)
 		return PKC_NULL;
 #else
@@ -788,6 +773,11 @@ pkg_getclient(int fd, const struct pkg_switch *switchp, void (*errlog) (char *ms
 		return PKC_NULL;
 #endif
 	    _pkg_perror(errlog, "pkg_getclient: accept");
+#ifdef HAVE_WINSOCK_H
+		char msgbuf[256] = { '\0' };
+		FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS, NULL, errno, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), msgbuf, sizeof(msgbuf), NULL);
+		_pkg_perror(errlog, (char *)msgbuf);
+#endif
 	    return PKC_ERROR;
 	}
     }  while (s2 < 0);
@@ -853,8 +843,6 @@ pkg_close(struct pkg_conn *pc)
 
 
 /**
- * P K G _ I N G E T
- *
  * A functional replacement for bu_mread() through the first level
  * input buffer.
  *
@@ -888,8 +876,6 @@ _pkg_inget(struct pkg_conn *pc, char *buf, size_t count)
 
 
 /**
- * _ P K G _ C H E C K I N
- *
  * This routine is called whenever it is necessary to see if there is
  * more input that can be read.  If input is available, it is read
  * into pkc_inbuf[].  If nodelay is set, poll without waiting.
@@ -911,6 +897,7 @@ _pkg_checkin(struct pkg_conn *pc, int nodelay)
     else
 	tv.tv_usec = 20000;	/* 20 ms */
 
+    errno = 0;
     FD_ZERO(&bits);
     FD_SET(pc->pkc_fd, &bits);
     i = select(pc->pkc_fd+1, &bits, (fd_set *)0, (fd_set *)0, &tv);
@@ -1024,6 +1011,7 @@ pkg_send(int type, const char *buf, size_t len, struct pkg_conn *pc)
 	if (len > 0)
 	    memcpy(tbuf+sizeof(hdr), buf, len);
 
+	errno = 0;
 	i = PKG_SEND(pc->pkc_fd, tbuf, len+sizeof(hdr));
 	if ((size_t)i != len+sizeof(hdr)) {
 	    if (i < 0) {
@@ -1040,6 +1028,7 @@ pkg_send(int type, const char *buf, size_t len, struct pkg_conn *pc)
 	return (int)len;
     }
     /* Send them separately */
+    errno = 0;
     if ((i = PKG_SEND(pc->pkc_fd, (char *)&hdr, sizeof(hdr))) != sizeof(hdr)) {
 	if (i < 0) {
 	    if (errno == EBADF)
@@ -1054,6 +1043,7 @@ pkg_send(int type, const char *buf, size_t len, struct pkg_conn *pc)
     }
     if (len <= 0)
 	return 0;
+    errno = 0;
     i = PKG_SEND(pc->pkc_fd, buf, len);
     if ((size_t)i != len) {
 	if (i < 0) {
@@ -1148,6 +1138,7 @@ pkg_2send(int type, const char *buf1, size_t len1, const char *buf2, size_t len2
 	    memcpy(tbuf+sizeof(hdr), buf1, len1);
 	if (len2 > 0)
 	    memcpy(tbuf+sizeof(hdr)+len1, buf2, len2);
+	errno = 0;
 	i = PKG_SEND(pc->pkc_fd, tbuf, len1+len2+sizeof(hdr));
 	if ((size_t)i != len1+len2+sizeof(hdr)) {
 	    if (i < 0) {
@@ -1164,7 +1155,9 @@ pkg_2send(int type, const char *buf1, size_t len1, const char *buf2, size_t len2
 	return (int)(len1+len2);
     }
     /* Send it in three pieces */
-    if ((i = (ssize_t)PKG_SEND(pc->pkc_fd, (char *)&hdr, sizeof(hdr))) != sizeof(hdr)) {
+    errno = 0;
+    i = (ssize_t)PKG_SEND(pc->pkc_fd, (char *)&hdr, sizeof(hdr));
+    if (i != sizeof(hdr)) {
 	if (i < 0) {
 	    if (errno == EBADF)
 		return -1;
@@ -1180,6 +1173,7 @@ pkg_2send(int type, const char *buf1, size_t len1, const char *buf2, size_t len2
 	return -1;		/* amount of user data sent */
     }
 
+    errno = 0;
     i = PKG_SEND(pc->pkc_fd, buf1, len1);
     if ((size_t)i != len1) {
 	if (i < 0) {
@@ -1199,6 +1193,7 @@ pkg_2send(int type, const char *buf1, size_t len1, const char *buf2, size_t len2
     if (len2 <= (size_t)0)
 	return i;
 
+    errno = 0;
     i = PKG_SEND(pc->pkc_fd, buf2, len2);
     if (i != (ssize_t)len2) {
 	if (i < 0) {
@@ -1271,6 +1266,7 @@ pkg_flush(struct pkg_conn *pc)
 	return 0;
     }
 
+    errno = 0;
     i = write(pc->pkc_fd, pc->pkc_stream, (size_t)pc->pkc_strpos);
     if (i != pc->pkc_strpos) {
 	if (i < 0) {
@@ -1293,8 +1289,6 @@ pkg_flush(struct pkg_conn *pc)
 
 
 /**
- * _ P K G _ G E T H D R
- *
  * Get header from a new message.
  *
  * Returns 1 when there is some message to go look at and -1 on fatal
@@ -1512,8 +1506,6 @@ pkg_bwaitfor (int type, struct pkg_conn *pc)
 
 
 /**
- * _ P K G _ D I S P A T C H
- *
  * Given that a whole message has arrived, send it to the appropriate
  * User Handler, or else grouse.  Returns -1 on fatal error, 0 on no
  * handler, 1 if all's well.
