@@ -1,7 +1,7 @@
 /*                        D B 5 _ I O . C
  * BRL-CAD
  *
- * Copyright (c) 2004-2013 United States Government as represented by
+ * Copyright (c) 2004-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This library is free software; you can redistribute it and/or
@@ -30,14 +30,22 @@
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
-#include "bin.h"
+#include "bnetwork.h"
 
-#include "bu.h"
+#include "bu/endian.h"
+#include "bu/parse.h"
+#include "bu/cv.h"
 #include "vmath.h"
 #include "bn.h"
-#include "db5.h"
+#include "rt/db5.h"
 #include "raytrace.h"
-#include "mater.h"
+#include "librt_private.h"
+
+
+/**
+ * Number of bytes used for each value of DB5HDR_WIDTHCODE_*
+ */
+#define ENCODE_LEN(len) (1 << len)
 
 
 int
@@ -45,44 +53,58 @@ db5_header_is_valid(const unsigned char *hp)
 {
     const struct db5_ondisk_header *odp = (const struct db5_ondisk_header *)hp;
 
-    if (odp->db5h_magic1 != DB5HDR_MAGIC1) return 0;
-    if (hp[7] != DB5HDR_MAGIC2) return 0;
+    if (odp->db5h_magic1 != DB5HDR_MAGIC1)
+	return 0;
+    if (hp[7] != DB5HDR_MAGIC2)
+	return 0;
 
     /* hflags */
     if ((odp->db5h_hflags & DB5HDR_HFLAGS_DLI_MASK) != DB5HDR_HFLAGS_DLI_HEADER_OBJECT)
 	return 0;
-    if ((odp->db5h_hflags & DB5HDR_HFLAGS_NAME_PRESENT)) return 0;
-    if (((odp->db5h_hflags & DB5HDR_HFLAGS_OBJECT_WIDTH_MASK) >> DB5HDR_HFLAGS_OBJECT_WIDTH_SHIFT)
-	!= DB5HDR_WIDTHCODE_8BIT) return 0;
+    if ((odp->db5h_hflags & DB5HDR_HFLAGS_NAME_PRESENT))
+	return 0;
+    if (((odp->db5h_hflags & DB5HDR_HFLAGS_OBJECT_WIDTH_MASK) >> DB5HDR_HFLAGS_OBJECT_WIDTH_SHIFT) != DB5HDR_WIDTHCODE_8BIT)
+	return 0;
 
     /* aflags */
-    if ((odp->db5h_aflags & DB5HDR_AFLAGS_ZZZ_MASK) != DB5_ZZZ_UNCOMPRESSED) return 0;
-    if (odp->db5h_aflags & DB5HDR_AFLAGS_PRESENT) return 0;
-    if (((odp->db5h_aflags & DB5HDR_AFLAGS_WIDTH_MASK) >> DB5HDR_AFLAGS_WIDTH_SHIFT)
-	!= DB5HDR_WIDTHCODE_8BIT) return 0;
+    if ((odp->db5h_aflags & DB5HDR_AFLAGS_ZZZ_MASK) != DB5_ZZZ_UNCOMPRESSED)
+	return 0;
+    if (odp->db5h_aflags & DB5HDR_AFLAGS_PRESENT)
+	return 0;
+    if (((odp->db5h_aflags & DB5HDR_AFLAGS_WIDTH_MASK) >> DB5HDR_AFLAGS_WIDTH_SHIFT) != DB5HDR_WIDTHCODE_8BIT)
+	return 0;
 
     /* bflags */
-    if ((odp->db5h_bflags & DB5HDR_BFLAGS_ZZZ_MASK) != DB5_ZZZ_UNCOMPRESSED) return 0;
-    if (odp->db5h_bflags & DB5HDR_BFLAGS_PRESENT) return 0;
-    if (((odp->db5h_bflags & DB5HDR_BFLAGS_WIDTH_MASK) >> DB5HDR_BFLAGS_WIDTH_SHIFT)
-	!= DB5HDR_WIDTHCODE_8BIT) return 0;
+    if ((odp->db5h_bflags & DB5HDR_BFLAGS_ZZZ_MASK) != DB5_ZZZ_UNCOMPRESSED)
+	return 0;
+    if (odp->db5h_bflags & DB5HDR_BFLAGS_PRESENT)
+	return 0;
+    if (((odp->db5h_bflags & DB5HDR_BFLAGS_WIDTH_MASK) >> DB5HDR_BFLAGS_WIDTH_SHIFT) != DB5HDR_WIDTHCODE_8BIT)
+	return 0;
 
     /* major and minor type */
-    if (odp->db5h_major_type != DB5_MAJORTYPE_RESERVED) return 0;
-    if (odp->db5h_minor_type != 0) return 0;
+    if (odp->db5h_major_type != DB5_MAJORTYPE_RESERVED)
+	return 0;
+    if (odp->db5h_minor_type != 0)
+	return 0;
 
     /* Check length, known to be 8-bit.  Header len=1 8-byte chunk. */
-    if (hp[6] != 1) return 0;
+    if (hp[6] != 1)
+	return 0;
 
     return 1;		/* valid */
 }
 
+
 int
 db5_select_length_encoding(size_t len)
 {
-    if (len <= 255) return DB5HDR_WIDTHCODE_8BIT;
-    if (len <= 65535) return DB5HDR_WIDTHCODE_16BIT;
-    if (len < 0x7ffffffe) return DB5HDR_WIDTHCODE_32BIT;
+    if (len <= 255)
+	return DB5HDR_WIDTHCODE_8BIT;
+    if (len <= 65535)
+	return DB5HDR_WIDTHCODE_16BIT;
+    if (len < 0x7ffffffe)
+	return DB5HDR_WIDTHCODE_32BIT;
     return DB5HDR_WIDTHCODE_64BIT;
 }
 
@@ -112,15 +134,18 @@ db5_decode_length(size_t *lenp, const unsigned char *cp, int format)
     return 0;
 }
 
-int
+
+size_t
 db5_decode_signed(size_t *lenp, const unsigned char *cp, int format)
 {
     switch (format) {
 	case DB5HDR_WIDTHCODE_8BIT:
-	    if ((*lenp = (*cp)) & 0x80) *lenp |= ((size_t)-1 ^ 0xFF);
+	    if ((*lenp = (*cp)) & 0x80)
+		*lenp |= ((size_t)-1 ^ 0xFF);
 	    return 1;
 	case DB5HDR_WIDTHCODE_16BIT:
-	    if ((*lenp = BU_GSHORT(cp)) & 0x8000)  *lenp |= ((size_t)-1 ^ 0xFFFF);
+	    if ((*lenp = BU_GSHORT(cp)) & 0x8000)
+		*lenp |= ((size_t)-1 ^ 0xFFFF);
 	    return 2;
 	case DB5HDR_WIDTHCODE_32BIT:
 	    if ((*lenp = BU_GLONG(cp)) & 0x80000000)
@@ -138,15 +163,6 @@ db5_decode_signed(size_t *lenp, const unsigned char *cp, int format)
 }
 
 
-/**
- * D B 5 _ E N C O D E _ L E N G T H
- *
- * Given a value and a variable-width format spec, store it in network
- * order.
- *
- * Returns -
- * pointer to next available byte.
- */
 unsigned char *
 db5_encode_length(
     unsigned char *cp,
@@ -173,18 +189,15 @@ db5_encode_length(
 
 
 /**
- * D B 5 _ C R A C K _ D I S K _ H E A D E R
- *
  * Returns -
  * 0 on success
  * -1 on error
  */
-int
-db5_crack_disk_header(struct db5_raw_internal *rip, const unsigned char *cp)
+HIDDEN int
+crack_disk_header(struct db5_raw_internal *rip, const unsigned char *cp)
 {
     if (cp[0] != DB5HDR_MAGIC1) {
-	bu_log("db5_crack_disk_header() bad magic1 -- database has become corrupted\n expected x%x, got x%x\n",
-	       DB5HDR_MAGIC1, cp[0]);
+	bu_log("crack_disk_header() bad magic1: expected x%x, got x%x\n", DB5HDR_MAGIC1, cp[0]);
 	if (cp[0] == 'I') {
 	    bu_log ("Concatenation of different database versions detected.\n");
 	    bu_log ("Run 'dbupgrade' on all databases before concatenation (cat command).\n");
@@ -216,55 +229,40 @@ db5_crack_disk_header(struct db5_raw_internal *rip, const unsigned char *cp)
     rip->major_type = cp[4];
     rip->minor_type = cp[5];
 
-    if (RT_G_DEBUG&DEBUG_DB) bu_log("db5_crack_disk_header()\n\
-	h_dli=%d, h_object_width=%d, h_name_present=%d, h_name_width=%d, \n\
-	a_width=%d, a_present=%d, a_zzz=%d, \n\
-	b_width=%d, b_present=%d, b_zzz=%d, major=%d, minor=%d\n",
-				    rip->h_dli,
-				    rip->h_object_width,
-				    rip->h_name_present,
-				    rip->h_name_width,
-				    rip->a_width,
-				    rip->a_present,
-				    rip->a_zzz,
-				    rip->b_width,
-				    rip->b_present,
-				    rip->b_zzz,
-				    rip->major_type,
-				    rip->minor_type);
+    if (RT_G_DEBUG&RT_DEBUG_DB) {
+	bu_log("crack_disk_header()\n"
+	       "\th_dli=%d, h_object_width=%d, h_name_present=%d, h_name_width=%d,\n"
+	       "\ta_width=%d, a_present=%d, a_zzz=%d,\n"
+	       "\tb_width=%d, b_present=%d, b_zzz=%d, major=%d, minor=%d\n",
+	       rip->h_dli, rip->h_object_width, rip->h_name_present, rip->h_name_width,
+	       rip->a_width, rip->a_present, rip->a_zzz,
+	       rip->b_width, rip->b_present, rip->b_zzz, rip->major_type, rip->minor_type);
+    }
 
     return 0;
 }
 
 
-/**
- * D B 5 _ G E T _ R A W _ I N T E R N A L _ P T R
- *
- * Returns -
- * on success, pointer to first unused byte
- * NULL, on error
- */
 const unsigned char *
 db5_get_raw_internal_ptr(struct db5_raw_internal *rip, const unsigned char *ip)
 {
     const unsigned char *cp = ip;
 
-    if (db5_crack_disk_header(rip, cp) < 0) return NULL;
+    if (crack_disk_header(rip, cp) < 0)
+	return NULL;
     cp += sizeof(struct db5_ondisk_header);
 
     cp += db5_decode_length(&rip->object_length, cp, rip->h_object_width);
     rip->object_length <<= 3;	/* cvt 8-byte chunks to byte count */
 
     if ((size_t)rip->object_length < sizeof(struct db5_ondisk_header)) {
-	bu_log("db5_get_raw_internal_ptr(): object_length=%ld is too short, database is corrupted\n",
-	       rip->object_length);
+	bu_log("db5_get_raw_internal_ptr(): object_length=%ld is too short, database possibly corrupted\n", rip->object_length);
 	return NULL;
     }
 
     /* Verify trailing magic number */
     if (ip[rip->object_length-1] != DB5HDR_MAGIC2) {
-	bu_log("db5_get_raw_internal_ptr() bad magic2 -- database has become corrupted.\n expected x%x, got x%x\n",
-	       DB5HDR_MAGIC2, ip[rip->object_length-1]);
+	bu_log("db5_get_raw_internal_ptr() bad magic2: expected x%x, got x%x\n", DB5HDR_MAGIC2, ip[rip->object_length-1]);
 	return NULL;
     }
 
@@ -274,25 +272,25 @@ db5_get_raw_internal_ptr(struct db5_raw_internal *rip, const unsigned char *ip)
 
     /* Grab name, if present */
     if (rip->h_name_present) {
-	cp += db5_decode_length(&rip->name.ext_nbytes,
-				cp, rip->h_name_width);
-	rip->name.ext_buf = (genptr_t)cp;	/* discard const */
+	cp += db5_decode_length(&rip->name.ext_nbytes, cp, rip->h_name_width);
+	rip->name.ext_buf = (uint8_t *)cp;	/* discard const */
 	cp += rip->name.ext_nbytes;
     }
 
     /* Point to attributes, if present */
     if (rip->a_present) {
-	cp += db5_decode_length(&rip->attributes.ext_nbytes,
-				cp, rip->a_width);
-	rip->attributes.ext_buf = (genptr_t)cp;	/* discard const */
+	cp += db5_decode_length(&rip->attributes.ext_nbytes, cp, rip->a_width);
+	rip->attributes.ext_buf = (uint8_t *)cp;	/* discard const */
+#if defined(USE_BINARY_ATTRIBUTES)
+	rip->attributes.widcode = rip->a_width;
+#endif
 	cp += rip->attributes.ext_nbytes;
     }
 
     /* Point to body, if present */
     if (rip->b_present) {
-	cp += db5_decode_length(&rip->body.ext_nbytes,
-				cp, rip->b_width);
-	rip->body.ext_buf = (genptr_t)cp;	/* discard const */
+	cp += db5_decode_length(&rip->body.ext_nbytes, cp, rip->b_width);
+	rip->body.ext_buf = (uint8_t *)cp;	/* discard const */
 	cp += rip->body.ext_nbytes;
     }
 
@@ -301,23 +299,25 @@ db5_get_raw_internal_ptr(struct db5_raw_internal *rip, const unsigned char *ip)
     return ip + rip->object_length;
 }
 
+
 int
 db5_get_raw_internal_fp(struct db5_raw_internal *rip, FILE *fp)
 {
     struct db5_ondisk_header header;
     unsigned char lenbuf[8];
-    int count = 0;
+    size_t count = 0;
     size_t used;
     size_t want, got;
     size_t dlen;
     unsigned char *cp;
 
     if (fread((unsigned char *)&header, sizeof header, 1, fp) != 1) {
-	if (feof(fp)) return -1;
+	if (feof(fp))
+	    return -1;
 	bu_log("db5_get_raw_internal_fp(): fread header error\n");
 	return -2;
     }
-    if (db5_crack_disk_header(rip, (unsigned char *)&header) < 0)
+    if (crack_disk_header(rip, (unsigned char *)&header) < 0)
 	return -2;
     used = sizeof(header);
 
@@ -348,14 +348,14 @@ db5_get_raw_internal_fp(struct db5_raw_internal *rip, FILE *fp)
     used += dlen;
 
     /* verify the length won't overflow before we <<=3 it */
-    if ( rip->object_length > UINTPTR_MAX>>3 ) {
+    if (rip->object_length > UINTPTR_MAX>>3) {
 	bu_log("db5_get_raw_internal_fp():  bad length read\n");
 	return -1;
     }
     rip->object_length <<= 3;	/* cvt 8-byte chunks to byte count */
 
     if (rip->object_length < sizeof(struct db5_ondisk_header) || rip->object_length < used) {
-	bu_log("db5_get_raw_internal_fp(): object_length=%ld is too short, database is corrupted\n",
+	bu_log("db5_get_raw_internal_fp(): object_length=%ld is too short, database possibly corrupted\n",
 	       rip->object_length);
 	return -1;
     }
@@ -370,15 +370,13 @@ db5_get_raw_internal_fp(struct db5_raw_internal *rip, FILE *fp)
     want = rip->object_length - used;
 
     if ((got = fread(cp, 1, want, fp)) != want) {
-	bu_log("db5_get_raw_internal_fp(): Database is too short, want=%ld, got=%ld\n",
-	       want, got);
+	bu_log("db5_get_raw_internal_fp(): database is too short, want=%ld, got=%ld\n", want, got);
 	return -2;
     }
 
     /* Verify trailing magic number */
     if (rip->buf[rip->object_length-1] != DB5HDR_MAGIC2) {
-	bu_log("db5_get_raw_internal_fp(): bad magic2 -- database has become corrupted.\n expected x%x, got x%x\n",
-	       DB5HDR_MAGIC2, rip->buf[rip->object_length-1]);
+	bu_log("db5_get_raw_internal_fp() bad magic2: expected x%x, got x%x\n", DB5HDR_MAGIC2, rip->buf[rip->object_length-1]);
 	return -2;
     }
 
@@ -388,31 +386,32 @@ db5_get_raw_internal_fp(struct db5_raw_internal *rip, FILE *fp)
 
     /* Grab name, if present */
     if (rip->h_name_present) {
-	cp += db5_decode_length(&rip->name.ext_nbytes,
-				cp, rip->h_name_width);
-	rip->name.ext_buf = (genptr_t)cp;	/* discard const */
+	cp += db5_decode_length(&rip->name.ext_nbytes, cp, rip->h_name_width);
+	rip->name.ext_buf = (uint8_t *)cp;	/* discard const */
 	cp += rip->name.ext_nbytes;
     }
 
     /* Point to attributes, if present */
     if (rip->a_present) {
-	cp += db5_decode_length(&rip->attributes.ext_nbytes,
-				cp, rip->a_width);
-	rip->attributes.ext_buf = (genptr_t)cp;	/* discard const */
+	cp += db5_decode_length(&rip->attributes.ext_nbytes, cp, rip->a_width);
+	rip->attributes.ext_buf = (uint8_t *)cp;	/* discard const */
+#if defined(USE_BINARY_ATTRIBUTES)
+	rip->attributes.widcode = rip->a_width;
+#endif
 	cp += rip->attributes.ext_nbytes;
     }
 
     /* Point to body, if present */
     if (rip->b_present) {
-	cp += db5_decode_length(&rip->body.ext_nbytes,
-				cp, rip->b_width);
-	rip->body.ext_buf = (genptr_t)cp;	/* discard const */
+	cp += db5_decode_length(&rip->body.ext_nbytes, cp, rip->b_width);
+	rip->body.ext_buf = (uint8_t *)cp;	/* discard const */
 	cp += rip->body.ext_nbytes;
     }
 
     return 0;		/* success */
 }
 
+
 void
 db5_export_object3(
     struct bu_external *out,
@@ -443,7 +442,7 @@ db5_export_object3(
 	namelen = (long)strlen(name) + 1;	/* includes null */
 	if (namelen > 1) {
 	    n_width = db5_select_length_encoding(namelen);
-	    need += namelen + db5_enc_len[n_width];
+	    need += namelen + ENCODE_LEN(n_width);
 	} else {
 	    name = NULL;
 	    namelen = 0;
@@ -456,7 +455,7 @@ db5_export_object3(
 	BU_CK_EXTERNAL(attrib);
 	if (attrib->ext_nbytes > 0) {
 	    a_width = db5_select_length_encoding(attrib->ext_nbytes);
-	    need += attrib->ext_nbytes + db5_enc_len[a_width];
+	    need += attrib->ext_nbytes + ENCODE_LEN(a_width);
 	} else {
 	    attrib = NULL;
 	    a_width = 0;
@@ -468,7 +467,7 @@ db5_export_object3(
 	BU_CK_EXTERNAL(body);
 	if (body->ext_nbytes > 0) {
 	    b_width = db5_select_length_encoding(body->ext_nbytes);
-	    need += body->ext_nbytes + db5_enc_len[b_width];
+	    need += body->ext_nbytes + ENCODE_LEN(b_width);
 	} else {
 	    body = NULL;
 	    b_width = 0;
@@ -480,7 +479,7 @@ db5_export_object3(
 
     /* Allocate the buffer for the combined external representation */
     BU_EXTERNAL_INIT(out);
-    out->ext_buf = bu_malloc(need, "external object3");
+    out->ext_buf = (uint8_t *)bu_malloc(need, "external object3");
     out->ext_nbytes = need;		/* will be trimmed, below */
 
     /* Determine encoding for the header length field */
@@ -533,7 +532,7 @@ db5_export_object3(
 	 * followed by no bytes (for an empty value), followed by a NULL value termination,
 	 * followed by a NULL attribute-value termination. Minimum is 4 bytes
 	 */
-	BU_ASSERT_PTR(attrib->ext_nbytes, >=, 4);
+	BU_ASSERT(attrib->ext_nbytes >= 4);
 	cp = db5_encode_length(cp, attrib->ext_nbytes, a_width);
 	memcpy(cp, attrib->ext_buf, attrib->ext_nbytes);
 	cp += attrib->ext_nbytes;
@@ -555,16 +554,17 @@ db5_export_object3(
 
     /* Verify multiple of 8 */
     togo = cp - ((unsigned char *)out->ext_buf);
-    BU_ASSERT_LONG(togo&7, ==, 0);
+    BU_ASSERT((togo&7) == 0);
 
     /* Finally, go back to the header and write the actual object length */
     cp = ((unsigned char *)out->ext_buf) + sizeof(struct db5_ondisk_header);
     (void)db5_encode_length(cp, togo>>3, h_width);
 
     out->ext_nbytes = togo;
-    BU_ASSERT_LONG(out->ext_nbytes, >=, 8);
+    BU_ASSERT(out->ext_nbytes >= 8);
 }
 
+
 void
 db5_make_free_object_hdr(struct bu_external *ep, size_t length)
 {
@@ -574,12 +574,12 @@ db5_make_free_object_hdr(struct bu_external *ep, size_t length)
 
     BU_CK_EXTERNAL(ep);
 
-    BU_ASSERT_SIZE_T(length, >=, 8);
-    BU_ASSERT_SIZE_T(length&7, ==, 0);
+    BU_ASSERT(length >= 8);
+    BU_ASSERT((length&7) == 0);
 
     /* Reserve enough space to hold any free header, even w/64-bit len */
     ep->ext_nbytes = 8+8;
-    ep->ext_buf = bu_calloc(1, ep->ext_nbytes, "db5_make_free_object_hdr");
+    ep->ext_buf = (uint8_t *)bu_calloc(1, ep->ext_nbytes, "db5_make_free_object_hdr");
 
     /* Determine encoding for the header length field */
     h_width = db5_select_length_encoding(length>>3);
@@ -594,6 +594,7 @@ db5_make_free_object_hdr(struct bu_external *ep, size_t length)
     db5_encode_length(cp, length>>3, h_width);
 }
 
+
 void
 db5_make_free_object(struct bu_external *ep, size_t length)
 {
@@ -603,10 +604,10 @@ db5_make_free_object(struct bu_external *ep, size_t length)
 
     BU_CK_EXTERNAL(ep);
 
-    BU_ASSERT_SIZE_T(length, >=, 8);
-    BU_ASSERT_SIZE_T(length&7, ==, 0);
+    BU_ASSERT(length >= 8);
+    BU_ASSERT((length&7) == 0);
 
-    ep->ext_buf = bu_calloc(1, length, "db5_make_free_object");
+    ep->ext_buf = (uint8_t *)bu_calloc(1, length, "db5_make_free_object");
     ep->ext_nbytes = length;
 
     /* Determine encoding for the header length field */
@@ -625,6 +626,7 @@ db5_make_free_object(struct bu_external *ep, size_t length)
     *cp = DB5HDR_MAGIC2;
 }
 
+
 int
 rt_db_cvt_to_external5(
     struct bu_external *ext,
@@ -695,6 +697,7 @@ rt_db_cvt_to_external5(
     return 0;		/* OK */
 }
 
+
 int
 db_wrap_v5_external(struct bu_external *ep, const char *name)
 {
@@ -709,7 +712,7 @@ db_wrap_v5_external(struct bu_external *ep, const char *name)
 	       name);
 	return -1;
     }
-    BU_ASSERT_LONG(raw.h_dli, ==, DB5HDR_HFLAGS_DLI_APPLICATION_DATA_OBJECT);
+    BU_ASSERT(raw.h_dli == DB5HDR_HFLAGS_DLI_APPLICATION_DATA_OBJECT);
 
     /* See if name needs to be changed */
     if (raw.name.ext_buf == NULL || !BU_STR_EQUAL(name, (const char *)raw.name.ext_buf)) {
@@ -737,6 +740,7 @@ db_wrap_v5_external(struct bu_external *ep, const char *name)
     return 0;
 }
 
+
 int
 db_put_external5(struct bu_external *ep, struct directory *dp, struct db_i *dbip)
 {
@@ -744,8 +748,8 @@ db_put_external5(struct bu_external *ep, struct directory *dp, struct db_i *dbip
     RT_CK_DIR(dp);
     BU_CK_EXTERNAL(ep);
 
-    if (RT_G_DEBUG&DEBUG_DB) bu_log("db_put_external5(%s) ep=x%x, dbip=x%x, dp=x%x\n",
-				    dp->d_namep, ep, dbip, dp);
+    if (RT_G_DEBUG&RT_DEBUG_DB) bu_log("db_put_external5(%s) ep=%p, dbip=%p, dp=%p\n",
+				       dp->d_namep, (void *)ep, (void *)dbip, (void *)dp);
 
     if (dbip->dbi_read_only) {
 	bu_log("db_put_external5(%s):  READ-ONLY file\n",
@@ -753,7 +757,7 @@ db_put_external5(struct bu_external *ep, struct directory *dp, struct db_i *dbip
 	return -1;
     }
 
-    BU_ASSERT_LONG(dbip->dbi_version, ==, 5);
+    BU_ASSERT(dbip->dbi_version == 5);
 
     /* First, change the name. */
     if (db_wrap_v5_external(ep, dp->d_namep) < 0) {
@@ -769,7 +773,7 @@ db_put_external5(struct bu_external *ep, struct directory *dp, struct db_i *dbip
 	    return -5;
 	}
     }
-    BU_ASSERT_LONG(ep->ext_nbytes, ==, dp->d_len);
+    BU_ASSERT(ep->ext_nbytes == dp->d_len);
 
     if (dp->d_flags & RT_DIR_INMEM) {
 	memcpy(dp->d_un.ptr, (char *)ep->ext_buf, ep->ext_nbytes);
@@ -782,6 +786,7 @@ db_put_external5(struct bu_external *ep, struct directory *dp, struct db_i *dbip
     return 0;
 }
 
+
 int
 rt_db_put_internal5(
     struct directory *dp,
@@ -795,7 +800,7 @@ rt_db_put_internal5(
     RT_CK_DIR(dp);
     RT_CK_DBI(dbip);
     RT_CK_DB_INTERNAL(ip);
-    BU_ASSERT_LONG(dbip->dbi_version, ==, 5);
+    BU_ASSERT(dbip->dbi_version == 5);
 
     if (resp)
 	RT_CK_RESOURCE(resp);
@@ -814,7 +819,7 @@ rt_db_put_internal5(
 	    goto fail;
 	}
     }
-    BU_ASSERT_LONG(ext.ext_nbytes, ==, dp->d_len);
+    BU_ASSERT(ext.ext_nbytes == dp->d_len);
 
     if (dp->d_flags & RT_DIR_INMEM) {
 	memcpy(dp->d_un.ptr, ext.ext_buf, ext.ext_nbytes);
@@ -836,12 +841,7 @@ fail:
 }
 
 
-/* FIXME: should have gone away with v6.  needed now to pass the minor_type down during read */
-extern int rt_binunif_import5_minor_type(struct rt_db_internal *, const struct bu_external *, const mat_t, const struct db_i *, struct resource *, int);
-
 /**
- * R T _ D B _ E X T E R N A L 5 _ T O _ I N T E R N A L 5
- *
  * Given an object in external form, convert it to internal form.  The
  * caller is responsible for freeing the external form.
  *
@@ -873,7 +873,7 @@ rt_db_external5_to_internal5(
 	resp = &rt_uniresource;
     }
 
-    BU_ASSERT_LONG(dbip->dbi_version, ==, 5);
+    BU_ASSERT(dbip->dbi_version == 5);
 
     if (db5_get_raw_internal_ptr(&raw, ep->ext_buf) == NULL) {
 	bu_log("rt_db_external5_to_internal5(%s):  import failure\n",
@@ -884,7 +884,8 @@ rt_db_external5_to_internal5(
     if ((raw.major_type == DB5_MAJORTYPE_BRLCAD)
 	||(raw.major_type == DB5_MAJORTYPE_BINARY_UNIF)) {
 	/* As a convenience to older ft_import routines */
-	if (mat == NULL) mat = bn_mat_identity;
+	if (mat == NULL)
+	    mat = bn_mat_identity;
     } else {
 	bu_log("rt_db_external5_to_internal5(%s):  unable to import non-BRL-CAD object, major=%d minor=%d\n",
 	       name, raw.major_type, raw.minor_type);
@@ -929,7 +930,7 @@ rt_db_external5_to_internal5(
 	    return -1;
     }
 
-    /* ip has already been initialized, and should not be re-inited */
+    /* ip has already been initialized, and should not be re-initialized */
     ret = -1;
     if (id == ID_BINUNIF) {
 	/* FIXME: binunif export needs to write out minor_type so
@@ -956,6 +957,7 @@ rt_db_external5_to_internal5(
     return id;			/* OK */
 }
 
+
 int
 rt_db_get_internal5(
     struct rt_db_internal *ip,
@@ -972,7 +974,7 @@ rt_db_get_internal5(
 	RT_CK_RESOURCE(resp);
     }
 
-    BU_ASSERT_LONG(dbip->dbi_version, ==, 5);
+    BU_ASSERT(dbip->dbi_version == 5);
 
     if (db_get_external(&ext, dp, dbip) < 0)
 	return -2;		/* FAIL */
@@ -983,9 +985,6 @@ rt_db_get_internal5(
 }
 
 
-/**
- *
- */
 void
 db5_export_color_table(struct bu_vls *ostr, struct db_i *dbip)
 {
@@ -994,6 +993,7 @@ db5_export_color_table(struct bu_vls *ostr, struct db_i *dbip)
     rt_vls_color_map(ostr);
 }
 
+
 void
 db5_import_color_table(char *cp)
 {
@@ -1007,6 +1007,7 @@ db5_import_color_table(char *cp)
     }
 }
 
+
 int
 db5_put_color_table(struct db_i *dbip)
 {
@@ -1014,7 +1015,7 @@ db5_put_color_table(struct db_i *dbip)
     int ret;
 
     RT_CK_DBI(dbip);
-    BU_ASSERT_LONG(dbip->dbi_version, ==, 5);
+    BU_ASSERT(dbip->dbi_version == 5);
 
     db5_export_color_table(&str, dbip);
 
@@ -1025,6 +1026,7 @@ db5_put_color_table(struct db_i *dbip)
     return ret;
 }
 
+
 int
 db5_get_attributes(const struct db_i *dbip, struct bu_attribute_value_set *avs, const struct directory *dp)
 {
