@@ -1,7 +1,7 @@
 /*                          A R B 8 . C
  * BRL-CAD
  *
- * Copyright (c) 1985-2013 United States Government as represented by
+ * Copyright (c) 1985-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This library is free software; you can redistribute it and/or
@@ -21,8 +21,8 @@
 /** @{ */
 /** @file primitives/arb8/arb8.c
  *
- * Intersect a ray with an Arbitrary Regular Polyhedron with as many
- * as 8 vertices.
+ * Intersect a ray with an Arbitrary Polyhedron with as many as 8
+ * vertices.
  *
  * An ARB is a convex volume bounded by 4 (pyramid), 5 (wedge), or 6
  * (box) planes.  This analysis depends on the properties of objects
@@ -48,19 +48,20 @@
 #include "common.h"
 
 #include <stddef.h>
-#include <stdio.h>
 #include <math.h>
 #include <string.h>
 #include "bio.h"
 
-#include "bu.h"
+
+#include "bu/parallel.h"
+#include "bu/cv.h"
 #include "vmath.h"
 #include "bn.h"
 #include "nmg.h"
-#include "db.h"
-#include "rtgeom.h"
+#include "rt/db4.h"
+#include "rt/geom.h"
+#include "rt/arb_edit.h"
 #include "raytrace.h"
-#include "nurb.h"
 
 #include "../../librt_private.h"
 
@@ -155,7 +156,7 @@ const struct bu_structparse rt_arb_parse[] = {
 };
 
 
-short local_arb6_edge_vertex_mapping[10][2] = {
+const short local_arb6_edge_vertex_mapping[10][2] = {
     {0, 1},	/* edge 12 */
     {1, 2},	/* edge 23 */
     {2, 3},	/* edge 34 */
@@ -169,7 +170,7 @@ short local_arb6_edge_vertex_mapping[10][2] = {
 };
 
 
-short local_arb4_edge_vertex_mapping[6][2] = {
+const short local_arb4_edge_vertex_mapping[6][2] = {
     {0, 1},	/* edge 12 */
     {1, 2},	/* edge 23 */
     {2, 0},	/* edge 31 */
@@ -179,13 +180,40 @@ short local_arb4_edge_vertex_mapping[6][2] = {
 };
 
 
+#ifdef USE_OPENCL
+/* largest data members first */
+struct clt_arb_specific {
+    cl_double arb_peqns[4*6];
+    cl_int arb_nmfaces;
+};
+
+size_t
+clt_arb_pack(struct bu_pool *pool, struct soltab *stp)
+{
+    struct arb_specific *arb =
+        (struct arb_specific *)stp->st_specific;
+    struct clt_arb_specific *args;
+
+    const struct aface *afp;
+    cl_int j;
+
+    const size_t size = sizeof(*args);
+    args = (struct clt_arb_specific*)bu_pool_alloc(pool, 1, size);
+
+    for (afp = &arb->arb_face[j=0]; j < 6; j++, afp++) {
+        HMOVE(args->arb_peqns+4*j, afp->peqn);
+    }
+    args->arb_nmfaces = arb->arb_nmfaces;
+    return size;
+}
+#endif /* USE_OPENCL */
+
+
 /* rt_arb_get_cgtype(), rt_arb_std_type(), and rt_arb_centroid()
  * stolen from mged/arbs.c */
 
 /**
- * R T _ A R B _ G E T _ C G T Y P E
- *
- * C G A R B S :   determines COMGEOM arb types from GED general arbs
+ * determines COMGEOM arb types from GED general arbs
  *
  * Inputs -
  *
@@ -204,8 +232,8 @@ rt_arb_get_cgtype(
     int *cgtype,
     struct rt_arb_internal *arb,
     const struct bn_tol *tol,
-    register int *uvec, /* array of indexes to unique points in arb->pt[] */
-    register int *svec) /* array of indexes to like points in arb->pt[] */
+    int *uvec, /* array of indexes to unique points in arb->pt[] */
+    int *svec) /* array of indexes to like points in arb->pt[] */
 {
     register int i, j;
     int numuvec, unique;
@@ -298,8 +326,6 @@ rt_arb_get_cgtype(
 
 
 /**
- * R T _ A R B _ S T D _ T Y P E
- *
  * Given an ARB in internal form, return its specific ARB type.
  *
  * Set tol.dist = 0.0001 to obtain past behavior.
@@ -340,8 +366,6 @@ rt_arb_std_type(const struct rt_db_internal *ip, const struct bn_tol *tol)
 
 
 /**
- * R T _ A R B _ C E N T R O I D
- *
  * Find the center point for the arb in the rt_db_internal structure,
  * and return it as a point_t.
  */
@@ -390,8 +414,6 @@ rt_arb_centroid(point_t *cent, const struct rt_db_internal *ip)
 
 
 /**
- * R T _ A R B _ A D D _ P T
- *
  * Add another point to a struct arb_specific, checking for unique
  * pts.  The first two points are easy.  The third one triggers most
  * of the plane calculations, and forth and subsequent ones are merely
@@ -402,7 +424,7 @@ rt_arb_centroid(point_t *cent, const struct rt_db_internal *ip)
  * -1 point was rejected
  */
 HIDDEN int
-rt_arb_add_pt(register pointp_t point, const char *title, struct prep_arb *pap, int ptno, const char *name)
+rt_arb_add_pnt(register pointp_t point, const char *title, struct prep_arb *pap, int ptno, const char *name)
 
 
 /* current point # on face */
@@ -535,8 +557,6 @@ rt_arb_add_pt(register pointp_t point, const char *title, struct prep_arb *pap,
 
 
 /**
- * R T _ A R B _ M K _ P L A N E S
- *
  * Given an rt_arb_internal structure with 8 points in it, compute the
  * face information.
  *
@@ -556,14 +576,14 @@ rt_arb_mk_planes(register struct prep_arb *pap, struct rt_arb_internal *aip, con
     /*
      * Determine a point which is guaranteed to be within the solid.
      * This is done by averaging all the vertices.  This center is
-     * needed for rt_arb_add_pt, which demands a point inside the
+     * needed for rt_arb_add_pnt, which demands a point inside the
      * solid.  The center of the enclosing RPP strategy used for the
      * bounding sphere can be tricked by thin plates which are
      * non-axis aligned, so this dual-strategy is required.  (What a
      * bug hunt!).
      */
     VSETALL(sum, 0);
-    for (i=0; i<8; i++) {
+    for (i = 0; i < 8; i++) {
 	VADD2(sum, sum, aip->pt[i]);
     }
     VSCALE(pap->pa_center, sum, 0.125);	/* sum/8 */
@@ -575,7 +595,7 @@ rt_arb_mk_planes(register struct prep_arb *pap, struct rt_arb_internal *aip, con
      * its own vertex number, if non-equivalent).
      */
     equiv_pts[0] = 0;
-    for (i=1; i<8; i++) {
+    for (i = 1; i < 8; i++) {
 	for (j = i-1; j >= 0; j--) {
 	    /* Compare vertices I and J */
 	    vect_t work;
@@ -590,7 +610,7 @@ rt_arb_mk_planes(register struct prep_arb *pap, struct rt_arb_internal *aip, con
 	equiv_pts[i] = i;
     next_point: ;
     }
-    if (RT_G_DEBUG & DEBUG_ARB8) {
+    if (RT_G_DEBUG & RT_DEBUG_ARB8) {
 	bu_log("arb(%s) equiv_pts[] = %d %d %d %d %d %d %d %d\n",
 	       name,
 	       equiv_pts[0], equiv_pts[1], equiv_pts[2], equiv_pts[3],
@@ -598,15 +618,15 @@ rt_arb_mk_planes(register struct prep_arb *pap, struct rt_arb_internal *aip, con
     }
 
     pap->pa_faces = 0;
-    for (i=0; i<6; i++) {
+    for (i = 0; i < 6; i++) {
 	int npts;
 
 	npts = 0;
-	for (j=0; j<4; j++) {
+	for (j = 0; j < 4; j++) {
 	    int pt_index;
 
 	    pt_index = rt_arb_info[i].ai_sub[j];
-	    if (RT_G_DEBUG & DEBUG_ARB8) {
+	    if (RT_G_DEBUG & RT_DEBUG_ARB8) {
 		bu_log("face %d, j=%d, npts=%d, orig_vert=%d, vert=%d\n",
 		       i, j, npts,
 		       pt_index, equiv_pts[pt_index]);
@@ -622,7 +642,7 @@ rt_arb_mk_planes(register struct prep_arb *pap, struct rt_arb_internal *aip, con
 		    goto skip_pt;
 		}
 	    }
-	    if (rt_arb_add_pt(aip->pt[pt_index],
+	    if (rt_arb_add_pnt(aip->pt[pt_index],
 			      rt_arb_info[i].ai_title, pap, npts, name) == 0) {
 		/* Point was accepted */
 		pap->pa_pindex[npts][pap->pa_faces] = pt_index;
@@ -663,8 +683,6 @@ rt_arb_mk_planes(register struct prep_arb *pap, struct rt_arb_internal *aip, con
 
 
 /**
- * R T _ A R B _ B B O X
- *
  * Find the bounding RPP of an arb
  */
 int
@@ -687,8 +705,6 @@ rt_arb_bbox(struct rt_db_internal *ip, point_t *min, point_t *max, const struct
 
 
 /**
- * R T _ A R B _ S E T U P
- *
  * This is packaged as a separate function, so that it can also be
  * called "on the fly" from the UV mapper.
  *
@@ -724,7 +740,7 @@ rt_arb_setup(struct soltab *stp, struct rt_arb_internal *aip, struct rt_i *rtip,
 		sizeof(struct arb_specific) +
 		sizeof(struct aface) * (pa.pa_faces - 4),
 		"arb_specific");
-	    stp->st_specific = (genptr_t)arbp;
+	    stp->st_specific = (void *)arbp;
 	}
 	arbp->arb_nmfaces = pa.pa_faces;
 	memcpy((char *)arbp->arb_face, (char *)pa.pa_face,
@@ -775,8 +791,6 @@ rt_arb_setup(struct soltab *stp, struct rt_arb_internal *aip, struct rt_i *rtip,
 
 
 /**
- * R T _ A R B _ P R E P
- *
  * This is the actual LIBRT "prep" interface.
  *
  * Returns -
@@ -795,9 +809,6 @@ rt_arb_prep(struct soltab *stp, struct rt_db_internal *ip, struct rt_i *rtip)
 }
 
 
-/**
- * R T _ A R B _ P R I N T
- */
 void
 rt_arb_print(register const struct soltab *stp)
 {
@@ -811,7 +822,7 @@ rt_arb_print(register const struct soltab *stp)
 	return;
     }
     bu_log("%d faces:\n", arbp->arb_nmfaces);
-    for (i=0; i < arbp->arb_nmfaces; i++) {
+    for (i = 0; i < arbp->arb_nmfaces; i++) {
 	afp = &(arbp->arb_face[i]);
 	VPRINT("A", afp->A);
 	HPRINT("Peqn", afp->peqn);
@@ -829,8 +840,6 @@ rt_arb_print(register const struct soltab *stp)
 
 
 /**
- * R T _ A R B _ S H O T
- *
  * Function -
  * Shoot a ray at an ARB8.
  *
@@ -856,7 +865,7 @@ rt_arb_shot(struct soltab *stp, register struct xray *rp, struct application *ap
     out = INFINITY;
     iplane = oplane = -1;
 
-    if (RT_G_DEBUG & DEBUG_ARB8) {
+    if (RT_G_DEBUG & RT_DEBUG_ARB8) {
 	bu_log("\n\n------------\n arb: ray point %g %g %g -> %g %g %g\n",
 	       V3ARGS(rp->r_pt),
 	       V3ARGS(rp->r_dir));
@@ -873,7 +882,7 @@ rt_arb_shot(struct soltab *stp, register struct xray *rp, struct application *ap
 	dxbdn = VDOT(afp->peqn, rp->r_pt) - afp->peqn[W];
 	dn = -VDOT(afp->peqn, rp->r_dir);
 
-	if (RT_G_DEBUG & DEBUG_ARB8) {
+	if (RT_G_DEBUG & RT_DEBUG_ARB8) {
 	    HPRINT("arb: Plane Equation", afp->peqn);
 	    bu_log("arb: dn=%g dxbdn=%g s=%g\n", dn, dxbdn, dxbdn/dn);
 	}
@@ -929,8 +938,6 @@ rt_arb_shot(struct soltab *stp, register struct xray *rp, struct application *ap
 
 #define RT_ARB8_SEG_MISS(SEG)	(SEG).seg_stp=RT_SOLTAB_NULL
 /**
- * R T _ A R B _ V S H O T
- *
  * This is the Becker vector version
  */
 void
@@ -1018,8 +1025,6 @@ rt_arb_vshot(struct soltab **stp, struct xray **rp, struct seg *segp, int n, str
 
 
 /**
- * R T _ A R B _ N O R M
- *
  * Given ONE ray distance, return the normal and entry/exit point.
  */
 void
@@ -1036,8 +1041,6 @@ rt_arb_norm(register struct hit *hitp, struct soltab *stp, register struct xray
 
 
 /**
- * R T _ A R B _ C U R V E
- *
  * Return the "curvature" of the ARB face.  Pick a principle direction
  * orthogonal to normal, and indicate no curvature.
  */
@@ -1052,8 +1055,6 @@ rt_arb_curve(register struct curvature *cvp, register struct hit *hitp, struct s
 
 
 /**
- * R T _ A R B _ U V
- *
  * For a hit on a face of an ARB, return the (u, v) coordinates of the
  * hit point.  0 <= u, v <= 1.
  *
@@ -1102,12 +1103,13 @@ rt_arb_uv(struct application *ap, struct soltab *stp, register struct hit *hitp,
 	rt_db_free_internal(&intern);
 
 	if (ret != 0 || arbp->arb_opt == (struct oface *)0) {
-	    bu_log("rt_arb_uv(%s) dynamic setup failure st_specific=x%x, optp=x%x\n",
+	    bu_log("rt_arb_uv(%s) dynamic setup failure st_specific=%p, optp=%p\n",
 		   stp->st_name,
-		   stp->st_specific, arbp->arb_opt);
+		   stp->st_specific, (void *)arbp->arb_opt);
 	    return;
 	}
-	if (RT_G_DEBUG&DEBUG_SOLIDS) rt_pr_soltab(stp);
+	if (RT_G_DEBUG&RT_DEBUG_SOLIDS)
+	    rt_pr_soltab(stp);
     }
 
     ofp = &arbp->arb_opt[hitp->hit_surfno];
@@ -1143,9 +1145,6 @@ rt_arb_uv(struct application *ap, struct soltab *stp, register struct hit *hitp,
 }
 
 
-/**
- * R T _ A R B _ F R E E
- */
 void
 rt_arb_free(register struct soltab *stp)
 {
@@ -1165,8 +1164,6 @@ rt_arb_free(register struct soltab *stp)
     RT_ADD_VLIST(vlist_head, arb_pts[d], BN_VLIST_LINE_DRAW);
 
 /**
- * R T _ A R B _ P L O T
- *
  * Plot an ARB by tracing out four "U" shaped contours This draws each
  * edge only once.
  *
@@ -1174,7 +1171,7 @@ rt_arb_free(register struct soltab *stp)
  * be.
  */
 int
-rt_arb_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct rt_tess_tol *UNUSED(ttol), const struct bn_tol *UNUSED(tol), const struct rt_view_info *UNUSED(info))
+rt_arb_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct bg_tess_tol *UNUSED(ttol), const struct bn_tol *UNUSED(tol), const struct rt_view_info *UNUSED(info))
 {
     point_t *pts;
     struct rt_arb_internal *aip;
@@ -1193,15 +1190,10 @@ rt_arb_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct rt_te
     return 0;
 }
 
-
-/**
- * R T _ A R B _ C L A S S
- */
 int
 rt_arb_class(const struct soltab *stp, const fastf_t *min, const fastf_t *max, const struct bn_tol *tol)
 {
-    register struct arb_specific *arbp =
-	(struct arb_specific *)stp->st_specific;
+    register struct arb_specific *arbp = (struct arb_specific *)stp->st_specific;
     register int i;
 
     if (arbp == (struct arb_specific *)0) {
@@ -1209,9 +1201,8 @@ rt_arb_class(const struct soltab *stp, const fastf_t *min, const fastf_t *max, c
 	return BN_CLASSIFY_UNIMPLEMENTED;
     }
 
-    for (i=0; i<arbp->arb_nmfaces; i++) {
-	if (bn_hlf_class(arbp->arb_face[i].peqn, min, max, tol) ==
-	    BN_CLASSIFY_OUTSIDE)
+    for (i = 0; i < arbp->arb_nmfaces; i++) {
+	if (bn_hlf_class(arbp->arb_face[i].peqn, min, max, tol) == BN_CLASSIFY_OUTSIDE)
 	    return BN_CLASSIFY_OUTSIDE;
     }
 
@@ -1219,10 +1210,7 @@ rt_arb_class(const struct soltab *stp, const fastf_t *min, const fastf_t *max, c
     return BN_CLASSIFY_UNIMPLEMENTED; /* let the caller assume the worst */
 }
 
-
 /**
- * R T _ A R B _ I M P O R T
- *
  * Import an ARB8 from the database format to the internal format.
  * There are two parts to this: First, the database is presently
  * single precision binary floating point.  Second, the ARB in the
@@ -1270,7 +1258,7 @@ rt_arb_import4(struct rt_db_internal *ip, const struct bu_external *ep, register
     if (mat == NULL) mat = bn_mat_identity;
     MAT4X3PNT(aip->pt[0], mat, &vec[0]);
 
-    for (i=1; i<8; i++) {
+    for (i = 1; i < 8; i++) {
 	VADD2(work, &vec[0*3], &vec[i*3]);
 	MAT4X3PNT(aip->pt[i], mat, work);
     }
@@ -1278,9 +1266,6 @@ rt_arb_import4(struct rt_db_internal *ip, const struct bu_external *ep, register
 }
 
 
-/**
- * R T _ A R B _ E X P O R T
- */
 int
 rt_arb_export4(struct bu_external *ep, const struct rt_db_internal *ip, double local2mm, const struct db_i *dbip)
 {
@@ -1297,7 +1282,7 @@ rt_arb_export4(struct bu_external *ep, const struct rt_db_internal *ip, double l
 
     BU_CK_EXTERNAL(ep);
     ep->ext_nbytes = sizeof(union record);
-    ep->ext_buf = (genptr_t)bu_malloc(ep->ext_nbytes, "arb external");
+    ep->ext_buf = (uint8_t *)bu_malloc(ep->ext_nbytes, "arb external");
     rec = (union record *)ep->ext_buf;
 
     rec->s.s_id = ID_SOLID;
@@ -1305,7 +1290,7 @@ rt_arb_export4(struct bu_external *ep, const struct rt_db_internal *ip, double l
 
     /* NOTE: This also converts to dbfloat_t */
     VSCALE(&rec->s.s_values[3*0], aip->pt[0], local2mm);
-    for (i=1; i < 8; i++) {
+    for (i = 1; i < 8; i++) {
 	VSUB2SCALE(&rec->s.s_values[3*i],
 		   aip->pt[i], aip->pt[0], local2mm);
     }
@@ -1314,8 +1299,6 @@ rt_arb_export4(struct bu_external *ep, const struct rt_db_internal *ip, double l
 
 
 /**
- * R T _ A R B _ I M P O R T 5
- *
  * Import an arb from the db5 format and convert to the internal
  * structure.  Code duplicated from rt_arb_import4() with db5 help from
  * g_ell.c
@@ -1333,7 +1316,7 @@ rt_arb_import5(struct rt_db_internal *ip, const struct bu_external *ep, register
     BU_CK_EXTERNAL(ep);
     if (dbip) RT_CK_DBI(dbip);
 
-    BU_ASSERT_LONG(ep->ext_nbytes, ==, SIZEOF_NETWORK_DOUBLE * 3*8);
+    BU_ASSERT(ep->ext_nbytes == SIZEOF_NETWORK_DOUBLE * 3*8);
 
     ip->idb_major_type = DB5_MAJORTYPE_BRLCAD;
     ip->idb_type = ID_ARB8;
@@ -1346,16 +1329,13 @@ rt_arb_import5(struct rt_db_internal *ip, const struct bu_external *ep, register
     /* Convert from database (network) to internal (host) format */
     bu_cv_ntohd((unsigned char *)vec, ep->ext_buf, 8*3);
     if (mat == NULL) mat = bn_mat_identity;
-    for (i=0; i<8; i++) {
+    for (i = 0; i < 8; i++) {
 	MAT4X3PNT(aip->pt[i], mat, &vec[i*3]);
     }
     return 0;	/* OK */
 }
 
 
-/**
- * R T _ A R B _ E X P O R T 5
- */
 int
 rt_arb_export5(struct bu_external *ep, const struct rt_db_internal *ip, double local2mm, const struct db_i *dbip)
 {
@@ -1374,8 +1354,8 @@ rt_arb_export5(struct bu_external *ep, const struct rt_db_internal *ip, double l
 
     BU_CK_EXTERNAL(ep);
     ep->ext_nbytes = SIZEOF_NETWORK_DOUBLE * 8 * ELEMENTS_PER_VECT;
-    ep->ext_buf = (genptr_t)bu_malloc(ep->ext_nbytes, "arb external");
-    for (i=0; i<8; i++) {
+    ep->ext_buf = (uint8_t *)bu_malloc(ep->ext_nbytes, "arb external");
+    for (i = 0; i < 8; i++) {
 	VSCALE(&vec[i*ELEMENTS_PER_VECT], aip->pt[i], local2mm);
     }
     bu_cv_htond(ep->ext_buf, (unsigned char *)vec, 8*ELEMENTS_PER_VECT);
@@ -1384,8 +1364,6 @@ rt_arb_export5(struct bu_external *ep, const struct rt_db_internal *ip, double l
 
 
 /**
- * R T _ A R B _ D E S C R I B E
- *
  * Make human-readable formatted presentation of this solid.  First
  * line describes type of solid.  Additional lines are indented one
  * tab, and give parameter values.
@@ -1425,7 +1403,7 @@ rt_arb_describe(struct bu_vls *str, const struct rt_db_internal *ip, int verbose
 
 	if (!verbose) return 0;
 
-	for (i=1; i < 8; i++) {
+	for (i = 1; i < 8; i++) {
 	    sprintf(buf, "\t%d (%g, %g, %g)\n", i+1,
 		    INTCLAMP(aip->pt[i][X] * mm2local),
 		    INTCLAMP(aip->pt[i][Y] * mm2local),
@@ -1437,7 +1415,7 @@ rt_arb_describe(struct bu_vls *str, const struct rt_db_internal *ip, int verbose
 	bu_vls_strcat(str, buf);
 	switch (arb_type) {
 	    case ARB8:
-		for (i=0; i<8; i++) {
+		for (i = 0; i < 8; i++) {
 		    sprintf(buf, "\t%d (%g, %g, %g)\n", i+1,
 			    INTCLAMP(aip->pt[i][X] * mm2local),
 			    INTCLAMP(aip->pt[i][Y] * mm2local),
@@ -1446,7 +1424,7 @@ rt_arb_describe(struct bu_vls *str, const struct rt_db_internal *ip, int verbose
 		}
 		break;
 	    case ARB7:
-		for (i=0; i<7; i++) {
+		for (i = 0; i < 7; i++) {
 		    sprintf(buf, "\t%d (%g, %g, %g)\n", i+1,
 			    INTCLAMP(aip->pt[i][X] * mm2local),
 			    INTCLAMP(aip->pt[i][Y] * mm2local),
@@ -1455,7 +1433,7 @@ rt_arb_describe(struct bu_vls *str, const struct rt_db_internal *ip, int verbose
 		}
 		break;
 	    case ARB6:
-		for (i=0; i<5; i++) {
+		for (i = 0; i < 5; i++) {
 		    sprintf(buf, "\t%d (%g, %g, %g)\n", i+1,
 			    INTCLAMP(aip->pt[i][X] * mm2local),
 			    INTCLAMP(aip->pt[i][Y] * mm2local),
@@ -1469,7 +1447,7 @@ rt_arb_describe(struct bu_vls *str, const struct rt_db_internal *ip, int verbose
 		bu_vls_strcat(str, buf);
 		break;
 	    case ARB5:
-		for (i=0; i<5; i++) {
+		for (i = 0; i < 5; i++) {
 		    sprintf(buf, "\t%d (%g, %g, %g)\n", i+1,
 			    INTCLAMP(aip->pt[i][X] * mm2local),
 			    INTCLAMP(aip->pt[i][Y] * mm2local),
@@ -1478,7 +1456,7 @@ rt_arb_describe(struct bu_vls *str, const struct rt_db_internal *ip, int verbose
 		}
 		break;
 	    case ARB4:
-		for (i=0; i<3; i++) {
+		for (i = 0; i < 3; i++) {
 		    sprintf(buf, "\t%d (%g, %g, %g)\n", i+1,
 			    INTCLAMP(aip->pt[i][X] * mm2local),
 			    INTCLAMP(aip->pt[i][Y] * mm2local),
@@ -1498,8 +1476,6 @@ rt_arb_describe(struct bu_vls *str, const struct rt_db_internal *ip, int verbose
 
 
 /**
- * R T _ A R B _ I F R E E
- *
  * Free the storage associated with the rt_db_internal version of this
  * solid.
  */
@@ -1508,13 +1484,11 @@ rt_arb_ifree(struct rt_db_internal *ip)
 {
     RT_CK_DB_INTERNAL(ip);
     bu_free(ip->idb_ptr, "arb ifree");
-    ip->idb_ptr = (genptr_t)NULL;
+    ip->idb_ptr = (void *)NULL;
 }
 
 
 /**
- * R T _ A R B _ T E S S
- *
  * "Tessellate" an ARB into an NMG data structure.  Purely a
  * mechanical transformation of one faceted object into another.
  *
@@ -1523,7 +1497,7 @@ rt_arb_ifree(struct rt_db_internal *ip)
  * 0 OK.  *r points to nmgregion that holds this tessellation.
  */
 int
-rt_arb_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, const struct rt_tess_tol *UNUSED(ttol), const struct bn_tol *tol)
+rt_arb_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, const struct bg_tess_tol *UNUSED(ttol), const struct bn_tol *tol)
 {
     struct rt_arb_internal *aip;
     struct shell *s;
@@ -1542,13 +1516,14 @@ rt_arb_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
     pa.pa_tol_sq = tol->dist_sq;
     if (rt_arb_mk_planes(&pa, aip, "(tess)") < 0) return -2;
 
-    for (i=0; i<8; i++) verts[i] = (struct vertex *)0;
+    for (i = 0; i < 8; i++)
+	verts[i] = (struct vertex *)0;
 
     *r = nmg_mrsv(m);	/* Make region, empty shell, vertex */
     s = BU_LIST_FIRST(shell, &(*r)->s_hd);
 
     /* Process each face */
-    for (i=0; i < pa.pa_faces; i++) {
+    for (i = 0; i < pa.pa_faces; i++) {
 	if (pa.pa_clockwise[i] != 0) {
 	    /* Counter-Clockwise orientation (CCW) */
 	    vertp[0] = &verts[pa.pa_pindex[0][i]];
@@ -1567,7 +1542,7 @@ rt_arb_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
 	    *vertpp++ = &verts[pa.pa_pindex[1][i]];
 	    *vertpp++ = &verts[pa.pa_pindex[0][i]];
 	}
-	if (RT_G_DEBUG & DEBUG_ARB8) {
+	if (RT_G_DEBUG & RT_DEBUG_ARB8) {
 	    bu_log("face %d, npts=%d, verts %d %d %d %d\n",
 		   i, pa.pa_npts[i],
 		   pa.pa_pindex[0][i], pa.pa_pindex[1][i],
@@ -1580,23 +1555,23 @@ rt_arb_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
     }
 
     /* Associate vertex geometry */
-    for (i=0; i<8; i++)
+    for (i = 0; i < 8; i++)
 	if (verts[i]) nmg_vertex_gv(verts[i], aip->pt[i]);
 
     /* Associate face geometry */
-    for (i=0; i < pa.pa_faces; i++) {
+    for (i = 0; i < pa.pa_faces; i++) {
 	/* We already know the plane equations, this is fast */
 	nmg_face_g(fu[i], pa.pa_face[i].peqn);
     }
 
     /* Mark edges as real */
-    (void)nmg_mark_edges_real(&s->l.magic);
+    (void)nmg_mark_edges_real(&s->l.magic, &RTG.rtg_vlfree);
 
     /* Compute "geometry" for region and shell */
     nmg_region_a(*r, tol);
 
     /* Some arbs may not be within tolerance, so triangulate faces where needed */
-    nmg_make_faces_within_tol(s, tol);
+    nmg_make_faces_within_tol(s, &RTG.rtg_vlfree, tol);
 
     return 0;
 }
@@ -1615,8 +1590,6 @@ static const int rt_arb_vert_index_scramble[4] = { 0, 1, 3, 2 };
 
 
 /**
- * R T _ A R B _ T N U R B
- *
  * "Tessellate" an ARB into a trimmed-NURB-NMG data structure.  Purely
  * a mechanical transformation of one faceted object into another.
  *
@@ -1650,13 +1623,14 @@ rt_arb_tnurb(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, c
     pa.pa_tol_sq = tol->dist_sq;
     if (rt_arb_mk_planes(&pa, aip, "(tnurb)") < 0) return -2;
 
-    for (i=0; i<8; i++) verts[i] = (struct vertex *)0;
+    for (i = 0; i < 8; i++)
+	verts[i] = (struct vertex *)0;
 
     *r = nmg_mrsv(m);	/* Make region, empty shell, vertex */
     s = BU_LIST_FIRST(shell, &(*r)->s_hd);
 
     /* Process each face */
-    for (i=0; i < pa.pa_faces; i++) {
+    for (i = 0; i < pa.pa_faces; i++) {
 	if (pa.pa_clockwise[i] != 0) {
 	    /* Counter-Clockwise orientation (CCW) */
 	    vertp[0] = &verts[pa.pa_pindex[0][i]];
@@ -1675,7 +1649,7 @@ rt_arb_tnurb(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, c
 	    *vertpp++ = &verts[pa.pa_pindex[1][i]];
 	    *vertpp++ = &verts[pa.pa_pindex[0][i]];
 	}
-	if (RT_G_DEBUG & DEBUG_ARB8) {
+	if (RT_G_DEBUG & RT_DEBUG_ARB8) {
 	    bu_log("face %d, npts=%d, verts %d %d %d %d\n",
 		   i, pa.pa_npts[i],
 		   pa.pa_pindex[0][i], pa.pa_pindex[1][i],
@@ -1714,11 +1688,11 @@ rt_arb_tnurb(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, c
     }
 
     /* Associate vertex geometry */
-    for (i=0; i<8; i++)
+    for (i = 0; i < 8; i++)
 	if (verts[i]) nmg_vertex_gv(verts[i], aip->pt[i]);
 
     /* Associate face geometry */
-    for (i=0; i < pa.pa_faces; i++) {
+    for (i = 0; i < pa.pa_faces; i++) {
 	struct face_g_snurb *fg;
 	int j;
 
@@ -1747,7 +1721,7 @@ rt_arb_tnurb(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, c
 	NMG_CK_EDGEUSE(eu);
 
 	/* For ctl_points, need 4 verts in order 0, 1, 3, 2 */
-	for (j=0; j < pa.pa_npts[i]; j++) {
+	for (j = 0; j < pa.pa_npts[i]; j++) {
 	    VMOVE(&fg->ctl_points[rt_arb_vert_index_scramble[j]*3],
 		  eu->vu_p->v_p->vg_p->coord);
 
@@ -1772,7 +1746,7 @@ rt_arb_tnurb(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, c
 
 
     /* Mark edges as real */
-    (void)nmg_mark_edges_real(&s->l.magic);
+    (void)nmg_mark_edges_real(&s->l.magic, &RTG.rtg_vlfree);
 
     /* Compute "geometry" for region and shell */
     nmg_region_a(*r, tol);
@@ -1783,8 +1757,6 @@ rt_arb_tnurb(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, c
 /* --- General ARB8 utility routines --- */
 
 /**
- * R T _ A R B _ C A L C _ P O I N T S
- *
  * Takes the planes[] array and intersects the planes to find the
  * vertices of a GENARB8.  The vertices are stored into arb->pt[].
  * This is an analog of rt_arb_calc_planes().
@@ -1798,7 +1770,7 @@ rt_arb_calc_points(struct rt_arb_internal *arb, int cgtype, const plane_t planes
     RT_ARB_CK_MAGIC(arb);
 
     /* find new points for entire solid */
-    for (i=0; i<8; i++) {
+    for (i = 0; i < 8; i++) {
 	if (rt_arb_3face_intersect(pt[i], planes, cgtype, i*3) < 0) {
 	    bu_log("rt_arb_calc_points: Intersection of planes fails %d\n", i);
 	    return -1;			/* FAIL */
@@ -1806,7 +1778,7 @@ rt_arb_calc_points(struct rt_arb_internal *arb, int cgtype, const plane_t planes
     }
 
     /* Move new points to arb tol->dist))*/
-    for (i=0; i<8; i++) {
+    for (i = 0; i < 8; i++) {
 	VMOVE(arb->pt[i], pt[i]);
     }
 
@@ -1821,26 +1793,29 @@ int
 rt_arb_check_points(struct rt_arb_internal *arb, int cgtype, const struct bn_tol *tol)
 {
     register int i;
+    const short arb8_evm[12][2] = arb8_edge_vertex_mapping;
+    const short arb7_evm[12][2] = arb7_edge_vertex_mapping;
+    const short arb5_evm[9][2] = arb5_edge_vertex_mapping;
 
     switch (cgtype) {
 	case ARB8:
-	    for (i=0; i<12; ++i) {
-		if (VNEAR_EQUAL(arb->pt[arb8_edge_vertex_mapping[i][0]],
-				arb->pt[arb8_edge_vertex_mapping[i][1]],
+	    for (i = 0; i < 12; ++i) {
+		if (VNEAR_EQUAL(arb->pt[arb8_evm[i][0]],
+				arb->pt[arb8_evm[i][1]],
 				tol->dist))
 		    return -1;
 	    }
 	    break;
 	case ARB7:
-	    for (i=0; i<11; ++i) {
-		if (VNEAR_EQUAL(arb->pt[arb7_edge_vertex_mapping[i][0]],
-				arb->pt[arb7_edge_vertex_mapping[i][1]],
+	    for (i = 0; i < 11; ++i) {
+		if (VNEAR_EQUAL(arb->pt[arb7_evm[i][0]],
+				arb->pt[arb7_evm[i][1]],
 				tol->dist))
 		    return -1;
 	    }
 	    break;
 	case ARB6:
-	    for (i=0; i<8; ++i) {
+	    for (i = 0; i < 8; ++i) {
 		if (VNEAR_EQUAL(arb->pt[local_arb6_edge_vertex_mapping[i][0]],
 				arb->pt[local_arb6_edge_vertex_mapping[i][1]],
 				tol->dist))
@@ -1848,15 +1823,15 @@ rt_arb_check_points(struct rt_arb_internal *arb, int cgtype, const struct bn_tol
 	    }
 	    break;
 	case ARB5:
-	    for (i=0; i<8; ++i) {
-		if (VNEAR_EQUAL(arb->pt[arb5_edge_vertex_mapping[i][0]],
-				arb->pt[arb5_edge_vertex_mapping[i][1]],
+	    for (i = 0; i < 8; ++i) {
+		if (VNEAR_EQUAL(arb->pt[arb5_evm[i][0]],
+				arb->pt[arb5_evm[i][1]],
 				tol->dist))
 		    return -1;
 	    }
 	    break;
 	case ARB4:
-	    for (i=0; i<6; ++i) {
+	    for (i = 0; i < 6; ++i) {
 		if (VNEAR_EQUAL(arb->pt[local_arb4_edge_vertex_mapping[i][0]],
 				arb->pt[local_arb4_edge_vertex_mapping[i][1]],
 				tol->dist))
@@ -1880,8 +1855,6 @@ static const int rt_arb_planes[5][24] = {
 
 
 /**
- * R T _ A R B _ 3 F A C E _ I N T E R S E C T
- *
  * Finds the intersection point of three faces of an ARB.
  *
  * Returns -
@@ -1904,13 +1877,11 @@ rt_arb_3face_intersect(
     i2 = rt_arb_planes[j][loc+1];
     i3 = rt_arb_planes[j][loc+2];
 
-    return bn_mkpoint_3planes(point, planes[i1], planes[i2], planes[i3]);
+    return bn_make_pnt_3planes(point, planes[i1], planes[i2], planes[i3]);
 }
 
 
 /**
- * R T _ A R B _ C A L C _ P L A N E S
- *
  * Calculate the plane (face) equations for an arb output previously
  * went to es_peqn[i].
  *
@@ -1930,25 +1901,26 @@ rt_arb_calc_planes(struct bu_vls *error_msg_ret,
 {
     register int i, p1, p2, p3;
     int type = cgtype - ARB4; /* ARB4 at location 0, ARB5 at 1, etc. */
+    const int arb_faces[5][24] = rt_arb_faces;
 
     RT_ARB_CK_MAGIC(arb);
     BN_CK_TOL(tol);
 
-    for (i=0; i<6; i++) {
-	if (rt_arb_faces[type][i*4] == -1)
+    for (i = 0; i < 6; i++) {
+	if (arb_faces[type][i*4] == -1)
 	    break;	/* faces are done */
 
-	p1 = rt_arb_faces[type][i*4];
-	p2 = rt_arb_faces[type][i*4+1];
-	p3 = rt_arb_faces[type][i*4+2];
+	p1 = arb_faces[type][i*4];
+	p2 = arb_faces[type][i*4+1];
+	p3 = arb_faces[type][i*4+2];
 
-	if (bn_mk_plane_3pts(planes[i],
+	if (bn_make_plane_3pnts(planes[i],
 			     arb->pt[p1],
 			     arb->pt[p2],
 			     arb->pt[p3],
 			     tol) < 0) {
 	    bu_vls_printf(error_msg_ret, "%d %d%d%d%d (bad face)\n",
-			  i+1, p1+1, p2+1, p3+1, rt_arb_faces[type][i*4+3]+1);
+			  i+1, p1+1, p2+1, p3+1, arb_faces[type][i*4+3]+1);
 	    return -1;
 	}
     }
@@ -1958,8 +1930,6 @@ rt_arb_calc_planes(struct bu_vls *error_msg_ret,
 
 
 /**
- * R T _ A R B _ M O V E _ E D G E
- *
  * Moves an arb edge (end1, end2) with bounding planes bp1 and bp2
  * through point "thru".  The edge has (non-unit) slope "dir".  Note
  * that the fact that the normals here point in rather than out makes
@@ -1997,8 +1967,6 @@ rt_arb_move_edge(struct bu_vls *error_msg_ret,
 
 
 /**
- * E D I T A R B
- *
  * An ARB edge is moved by finding the direction of the line
  * containing the edge and the 2 "bounding" planes.  The new edge is
  * found by intersecting the new line location with the bounding
@@ -2054,11 +2022,16 @@ rt_arb_edit(struct bu_vls *error_msg_ret,
 	    const struct bn_tol *tol)
 {
     int pt1 = 0, pt2 = 0, bp1, bp2, newp, p1, p2, p3;
-    short *edptr;		/* pointer to arb edit array */
-    short *final;		/* location of points to redo */
+    const short *edptr;		/* pointer to arb edit array */
+    const short *final;		/* location of points to redo */
     int i;
     const int *iptr;
     int edit_class = RT_ARB_EDIT_EDGE;
+    const short earb8[12][18] = earb8_edit_array;
+    const short earb7[12][18] = earb7_edit_array;
+    const short earb6[10][18] = earb6_edit_array;
+    const short earb5[9][18] = earb5_edit_array;
+    const short earb4[5][18] = earb4_edit_array;
 
     RT_ARB_CK_MAGIC(arb);
 
@@ -2173,13 +2146,13 @@ rt_arb_edit(struct bu_vls *error_msg_ret,
 	    goto err;
 
     if (newp >= 0 && newp < 6) {
-	for (i=0; i<3; i++) {
+	for (i = 0; i < 3; i++) {
 	    /* redo this plane (newp), use points p1, p2, p3 */
 	    p1 = *edptr++;
 	    p2 = *edptr++;
 	    p3 = *edptr++;
 
-	    if (bn_mk_plane_3pts(planes[newp], arb->pt[p1], arb->pt[p2],
+	    if (bn_make_plane_3pnts(planes[newp], arb->pt[p1], arb->pt[p2],
 				 arb->pt[p3], tol))
 		goto err;
 
@@ -2191,16 +2164,17 @@ rt_arb_edit(struct bu_vls *error_msg_ret,
 
     if (newp == 8) {
 	/* special...redo next planes using pts defined in faces */
-	for (i=0; i<3; i++) {
+	const int arb_faces[5][24] = rt_arb_faces;
+	for (i = 0; i < 3; i++) {
 	    if ((newp = *edptr++) == -1)
 		break;
 
-	    iptr = &rt_arb_faces[arb_type-4][4*newp];
+	    iptr = &arb_faces[arb_type-4][4*newp];
 	    p1 = *iptr++;
 	    p2 = *iptr++;
 	    p3 = *iptr++;
 
-	    if (bn_mk_plane_3pts(planes[newp], arb->pt[p1], arb->pt[p2],
+	    if (bn_make_plane_3pnts(planes[newp], arb->pt[p1], arb->pt[p2],
 				 arb->pt[p3], tol))
 		goto err;
 	}
@@ -2210,7 +2184,7 @@ rt_arb_edit(struct bu_vls *error_msg_ret,
      * push necessary points back into the planes
      */
     edptr = final;	/* point to the correct location */
-    for (i=0; i<2; i++) {
+    for (i = 0; i < 2; i++) {
 	const plane_t *c_planes = (const plane_t *)planes;
 
 	if ((p1 = *edptr++) == -1)
@@ -2226,7 +2200,7 @@ rt_arb_edit(struct bu_vls *error_msg_ret,
      * recalculate plane 2 = 456
      */
     if (arb_type == ARB7 && edit_class == RT_ARB_EDIT_POINT) {
-	if (bn_mk_plane_3pts(planes[2], arb->pt[4], arb->pt[5], arb->pt[6], tol))
+	if (bn_make_plane_3pnts(planes[2], arb->pt[4], arb->pt[5], arb->pt[6], tol))
 	    goto err;
     }
 
@@ -2264,10 +2238,6 @@ err:
 }
 
 
-/**
- * R T _ A R B _ P A R A M S
- *
- */
 int
 rt_arb_params(struct pc_pc_set * UNUSED(ps), const struct rt_db_internal *ip)
 {
@@ -2278,8 +2248,61 @@ rt_arb_params(struct pc_pc_set * UNUSED(ps), const struct rt_db_internal *ip)
 
 
 /**
- * R T _ A R B _ V O L U M E
- *
+ * compute surface area of an arb8 by dividing it into
+ * it's component faces and summing the face areas.
+ */
+void
+rt_arb_surf_area(fastf_t *area, const struct rt_db_internal *ip)
+{
+    const int arb_faces[5][24] = rt_arb_faces;
+    int i, a, b, c, type;
+    vect_t b_a, c_a, area_;
+    plane_t plane;
+    struct bn_tol tmp_tol, tol;
+    struct rt_arb_internal *aip = (struct rt_arb_internal *)ip->idb_ptr;
+    RT_ARB_CK_MAGIC(aip);
+
+    /* set up tolerance for rt_arb_std_type */
+    tol.magic = BN_TOL_MAGIC;
+    tol.dist = 0.0001; /* to get old behavior of rt_arb_std_type() */
+    tol.dist_sq = tol.dist * tol.dist;
+    tol.perp = 1e-5;
+    tol.para = 1 - tol.perp;
+
+    type = rt_arb_std_type(ip, &tol) - 4;
+
+    /* tol struct needed for bn_make_plane_3pnts,
+     * can't be passed to the function since it
+     * must fit into the rt_functab interface */
+    tmp_tol.magic = BN_TOL_MAGIC;
+    tmp_tol.dist = RT_LEN_TOL;
+    tmp_tol.dist_sq = tmp_tol.dist * tmp_tol.dist;
+
+    for (i = 0; i < 6; i++) {
+	if (arb_faces[type][i*4] == -1)
+	    break;	/* faces are done */
+
+	/* a, b, c = face of the GENARB8 */
+	a = arb_faces[type][i*4];
+	b = arb_faces[type][i*4+1];
+	c = arb_faces[type][i*4+2];
+
+	/* create a plane from a, b, c */
+	if (bn_make_plane_3pnts(plane, aip->pt[a], aip->pt[b], aip->pt[c], &tmp_tol) < 0) {
+	    continue;
+	}
+
+	/* calculate area of the face */
+	VSUB2(b_a, aip->pt[b], aip->pt[a]);
+	VSUB2(c_a, aip->pt[c], aip->pt[a]);
+	VCROSS(area_, b_a, c_a);
+
+	*area += MAGNITUDE(area_);
+    }
+}
+
+
+/**
  * compute volume of an arb8 by dividing it into
  * 6 arb4 and summing the volumes.
  */
@@ -2294,7 +2317,7 @@ rt_arb_volume(fastf_t *vol, const struct rt_db_internal *ip)
     struct rt_arb_internal *aip = (struct rt_arb_internal *)ip->idb_ptr;
     RT_ARB_CK_MAGIC(aip);
 
-    /* tol struct needed for bn_mk_plane_3pts,
+    /* tol struct needed for bn_make_plane_3pnts,
      * can't be passed to the function since it
      * must fit into the rt_functab interface */
     tmp_tol.magic = BN_TOL_MAGIC;
@@ -2310,13 +2333,13 @@ rt_arb_volume(fastf_t *vol, const struct rt_db_internal *ip)
 	d = farb4[i][3];
 
 	/* create a plane from a, b, c */
-	if (bn_mk_plane_3pts(plane, aip->pt[a], aip->pt[b], aip->pt[c], &tmp_tol) < 0) {
+	if (bn_make_plane_3pnts(plane, aip->pt[a], aip->pt[b], aip->pt[c], &tmp_tol) < 0) {
 	    continue;
 	}
 
 	/* height of arb4 is distance from the plane created using the
 	 * points of the base, and the top point 'd' */
-	arb4_height = fabs(DIST_PT_PLANE(aip->pt[d], plane));
+	arb4_height = fabs(DIST_PNT_PLANE(aip->pt[d], plane));
 
 	/* calculate area of arb4 base */
 	VSUB2(b_a, aip->pt[b], aip->pt[a]);
@@ -2330,12 +2353,16 @@ rt_arb_volume(fastf_t *vol, const struct rt_db_internal *ip)
 
 
 int
-rt_arb_get_edge_list(const struct rt_db_internal *ip, short (*edge_list[])[2])
+rt_arb_get_edge_list(const struct rt_db_internal *ip, const short (*edge_list[])[2])
 {
-    size_t edge_count=0;
+    size_t edge_count = 0;
     int arb_type;
     struct bn_tol tmp_tol;
     struct rt_arb_internal *aip = (struct rt_arb_internal *)ip->idb_ptr;
+    const short arb8_evm[12][2] = arb8_edge_vertex_mapping;
+    const short arb7_evm[12][2] = arb7_edge_vertex_mapping;
+    const short arb5_evm[9][2] = arb5_edge_vertex_mapping;
+    const short arb4_evm[5][2] = arb4_edge_vertex_mapping;
 
     RT_ARB_CK_MAGIC(aip);
 
@@ -2352,12 +2379,12 @@ rt_arb_get_edge_list(const struct rt_db_internal *ip, short (*edge_list[])[2])
     switch (arb_type) {
 	case ARB8:
 	    edge_count = 12;
-	    (*edge_list) = arb8_edge_vertex_mapping;
+	    (*edge_list) = arb8_evm;
 
 	    break;
 	case ARB7:
 	    edge_count = 12;
-	    (*edge_list) = arb7_edge_vertex_mapping;
+	    (*edge_list) = arb7_evm;
 
 	    break;
 	case ARB6:
@@ -2367,12 +2394,12 @@ rt_arb_get_edge_list(const struct rt_db_internal *ip, short (*edge_list[])[2])
 	    break;
 	case ARB5:
 	    edge_count = 9;
-	    (*edge_list) = arb5_edge_vertex_mapping;
+	    (*edge_list) = arb5_evm;
 
 	    break;
 	case ARB4:
 	    edge_count = 5;
-	    (*edge_list) = arb4_edge_vertex_mapping;
+	    (*edge_list) = arb4_evm;
 
 	    break;
 	default:
@@ -2390,12 +2417,12 @@ rt_arb_find_e_nearest_pt2(int *edge,
 			  const struct rt_db_internal *ip,
 			  const point_t pt2,
 			  const mat_t mat,
-			  const fastf_t ptol)
+			  fastf_t ptol)
 {
     int i;
     fastf_t dist=MAX_FASTF, tmp_dist;
-    short (*edge_list)[2] = {0};
-    int edge_count=0;
+    const short (*edge_list)[2] = {0};
+    int edge_count = 0;
     struct bn_tol tol;
     struct rt_arb_internal *aip = (struct rt_arb_internal *)ip->idb_ptr;
 
@@ -2423,7 +2450,7 @@ rt_arb_find_e_nearest_pt2(int *edge,
 	p1[Z] = 0.0;
 
 	if (edge_list[i][0] == edge_list[i][1]) {
-	    tmp_dist = bn_dist_pt3_pt3(pt2, p1);
+	    tmp_dist = bn_dist_pnt3_pnt3(pt2, p1);
 
 	    if (tmp_dist < ptol) {
 		*vert1 = edge_list[i][0] + 1;
@@ -2437,7 +2464,7 @@ rt_arb_find_e_nearest_pt2(int *edge,
 	} else {
 	    MAT4X3PNT(p2, mat, aip->pt[edge_list[i][1]]);
 	    p2[Z] = 0.0;
-	    ret = bn_dist_pt2_lseg2(&tmp_dist, pca, p1, p2, pt2, &tol);
+	    ret = bn_dist_pnt2_lseg2(&tmp_dist, pca, p1, p2, pt2, &tol);
 	}
 
 	if (ret < 3 || tmp_dist < dist) {
