@@ -1,7 +1,7 @@
 /*                           E H Y . C
  * BRL-CAD
  *
- * Copyright (c) 1990-2013 United States Government as represented by
+ * Copyright (c) 1990-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This library is free software; you can redistribute it and/or
@@ -150,15 +150,15 @@
 #include "common.h"
 
 #include <stddef.h>
-#include <stdio.h>
 #include <string.h>
 #include <math.h>
 #include "bio.h"
 
+#include "bu/cv.h"
 #include "vmath.h"
-#include "db.h"
+#include "rt/db4.h"
 #include "nmg.h"
-#include "rtgeom.h"
+#include "rt/geom.h"
 #include "raytrace.h"
 
 #include "../../librt_private.h"
@@ -174,6 +174,7 @@ struct ehy_specific {
     fastf_t ehy_cprime;	/* c / |H| */
 };
 
+
 const struct bu_structparse rt_ehy_parse[] = {
     { "%f", 3, "V",   bu_offsetofarray(struct rt_ehy_internal, ehy_V, fastf_t, X),  BU_STRUCTPARSE_FUNC_NULL, NULL, NULL },
     { "%f", 3, "H",   bu_offsetofarray(struct rt_ehy_internal, ehy_H, fastf_t, X),  BU_STRUCTPARSE_FUNC_NULL, NULL, NULL },
@@ -187,9 +188,38 @@ const struct bu_structparse rt_ehy_parse[] = {
 
 static int ehy_is_valid(struct rt_ehy_internal *ehy);
 
+#ifdef USE_OPENCL
+/* largest data members first */
+struct clt_ehy_specific {
+    cl_double ehy_V[3];		/* vector to ehy origin */
+    cl_double ehy_Hunit[3];	/* unit H vector */
+    cl_double ehy_SoR[16];	/* Scale(Rot(vect)) */
+    cl_double ehy_invRoS[16];	/* invRot(Scale(vect)) */
+    cl_double ehy_cprime;	/* c / |H| */
+};
+
+size_t
+clt_ehy_pack(struct bu_pool *pool, struct soltab *stp)
+{
+    struct ehy_specific *ehy =
+        (struct ehy_specific *)stp->st_specific;
+    struct clt_ehy_specific *args;
+
+    const size_t size = sizeof(*args);
+    args = (struct clt_ehy_specific*)bu_pool_alloc(pool, 1, size);
+
+    VMOVE(args->ehy_V, ehy->ehy_V);
+    VMOVE(args->ehy_Hunit, ehy->ehy_Hunit);
+    MAT_COPY(args->ehy_SoR, ehy->ehy_SoR);
+    MAT_COPY(args->ehy_invRoS, ehy->ehy_invRoS);
+    args->ehy_cprime = ehy->ehy_cprime;
+    return size;
+}
+
+#endif /* USE_OPENCL */
+
+
 /**
- * R T _ E H Y _ B B O X
- *
  * Create a bounding RPP for an ehy
  */
 int
@@ -242,8 +272,6 @@ rt_ehy_bbox(struct rt_db_internal *ip, point_t *min, point_t *max, const struct
 
 
 /**
- * R T _ E H Y _ P R E P
- *
  * Given a pointer to a GED database record, and a transformation
  * matrix, determine if this is a valid EHY, and if so, precompute
  * various terms of the formula.
@@ -290,7 +318,7 @@ rt_ehy_prep(struct soltab *stp, struct rt_db_internal *ip, struct rt_i *rtip)
     stp->st_meth = &OBJ[ID_EHY];
 
     BU_GET(ehy, struct ehy_specific);
-    stp->st_specific = (genptr_t)ehy;
+    stp->st_specific = (void *)ehy;
 
     /* make unit vectors in A, H, and BxH directions */
     VMOVE(ehy->ehy_Hunit, xip->ehy_H);
@@ -332,9 +360,6 @@ rt_ehy_prep(struct soltab *stp, struct rt_db_internal *ip, struct rt_i *rtip)
 }
 
 
-/**
- * R T _ E H Y _ P R I N T
- */
 void
 rt_ehy_print(const struct soltab *stp)
 {
@@ -357,8 +382,6 @@ rt_ehy_print(const struct soltab *stp)
 
 
 /**
- * R T _ E H Y _ S H O T
- *
  * Intersect a ray with a ehy.  If an intersection occurs, a struct
  * seg will be acquired and filled in.
  *
@@ -447,7 +470,7 @@ rt_ehy_shot(struct soltab *stp, struct xray *rp, struct application *ap, struct
     /*
      * Check for hitting the top plate.
      */
- check_plates:
+check_plates:
     /* check top plate */
     if (hitp == &hits[1]  &&  !ZERO(dp[Z])) {
 	/* 1 hit so far, this is worthwhile */
@@ -490,8 +513,6 @@ rt_ehy_shot(struct soltab *stp, struct xray *rp, struct application *ap, struct
 
 
 /**
- * R T _ E H Y _ N O R M
- *
  * Given ONE ray distance, return the normal and entry/exit point.
  */
 void
@@ -528,8 +549,6 @@ rt_ehy_norm(struct hit *hitp, struct soltab *stp, struct xray *rp)
 
 
 /**
- * R T _ E H Y _ C U R V E
- *
  * Return the curvature of the ehy.
  */
 void
@@ -585,8 +604,6 @@ rt_ehy_curve(struct curvature *cvp, struct hit *hitp, struct soltab *stp)
 
 
 /**
- * R T _ E H Y _ U V
- *
  * For a hit on the surface of an ehy, return the (u, v) coordinates
  * of the hit point, 0 <= u, v <= 1.
  *
@@ -619,14 +636,14 @@ rt_ehy_uv(struct application *ap, struct soltab *stp, struct hit *hitp, struct u
 		uvp->uv_u = 0;
 	    } else {
 		len = sqrt(pprime[X]*pprime[X] + pprime[Y]*pprime[Y]);
-		uvp->uv_u = acos(pprime[X]/len) * bn_inv2pi;
+		uvp->uv_u = acos(pprime[X]/len) * M_1_2PI;
 	    }
 	    uvp->uv_v = -pprime[Z];
 	    break;
 	case EHY_NORM_TOP:
 	    /* top plate, polar coords */
 	    len = sqrt(pprime[X]*pprime[X] + pprime[Y]*pprime[Y]);
-	    uvp->uv_u = acos(pprime[X]/len) * bn_inv2pi;
+	    uvp->uv_u = acos(pprime[X]/len) * M_1_2PI;
 	    uvp->uv_v = 1.0 - len;
 	    break;
     }
@@ -640,9 +657,6 @@ rt_ehy_uv(struct application *ap, struct soltab *stp, struct hit *hitp, struct u
 }
 
 
-/**
- * R T _ E H Y _ F R E E
- */
 void
 rt_ehy_free(struct soltab *stp)
 {
@@ -653,15 +667,6 @@ rt_ehy_free(struct soltab *stp)
 }
 
 
-/**
- * R T _ E H Y _ C L A S S
- */
-int
-rt_ehy_class(void)
-{
-    return 0;
-}
-
 /* Our canonical hyperbola in the Y-Z plane has equation
  * z = +- (a/b) * sqrt(b^2 + y^2), and opens toward +Z and -Z with asymptote
  * origin at the origin.
@@ -690,6 +695,7 @@ ehy_hyperbola_b(fastf_t mag_h, fastf_t c, fastf_t r)
     return (c * r) / sqrt(mag_h * (mag_h + 2.0 * c));
 }
 
+
 /* The contour of an ehy in the plane H-R (where R is one of the ehy axes A or
  * B) is the positive half of a hyperbola with asymptote origin at
  * ((|H| + c)Hu), opening toward -H. We can transform this hyperbola to get an
@@ -705,14 +711,14 @@ ehy_hyperbola_b(fastf_t mag_h, fastf_t c, fastf_t r)
  * asymptote origin at (0, -|H| + c), multiplying the z values by -1 gives
  * corresponding distances along the ehy height vector H.
  */
-static struct rt_pt_node *
+static struct rt_pnt_node *
 ehy_hyperbolic_curve(fastf_t mag_h, fastf_t c, fastf_t r, int num_points)
 {
     int count;
-    struct rt_pt_node *curve;
+    struct rt_pnt_node *curve;
 
-    BU_ALLOC(curve, struct rt_pt_node);
-    BU_ALLOC(curve->next, struct rt_pt_node);
+    BU_ALLOC(curve, struct rt_pnt_node);
+    BU_ALLOC(curve->next, struct rt_pnt_node);
 
     curve->next->next = NULL;
     VSET(curve->p,       0, 0, -mag_h);
@@ -727,6 +733,7 @@ ehy_hyperbolic_curve(fastf_t mag_h, fastf_t c, fastf_t r, int num_points)
     return curve;
 }
 
+
 /* The contour of an ehy in the plane H-R (where R is one of the ehy axes A or
  * B) is the positive half of a hyperbola with asymptote origin at
  * ((|H| + c)Hu), opening toward -H. We can transform this hyperbola to get an
@@ -743,7 +750,7 @@ ehy_hyperbolic_curve(fastf_t mag_h, fastf_t c, fastf_t r, int num_points)
  * Substituting b = (ar) / sqrt(|H| (|H| + 2a)) (see above comment):
  *
  *   y = (r / sqrt(|H| (|H| + 2a))) * sqrt((|H| + a - z)^2 - a^2)
- *     = r * sqrt( ((|H| + a - z)^2 - a^2) / (|H| (|H| + 2a))) )
+ *     = r * sqrt(((|H| + a - z)^2 - a^2) / (|H| (|H| + 2a))))
  */
 static fastf_t
 ehy_hyperbola_y(fastf_t mag_H, fastf_t c, fastf_t r, fastf_t z)
@@ -756,15 +763,16 @@ ehy_hyperbola_y(fastf_t mag_H, fastf_t c, fastf_t r, fastf_t z)
     return r * sqrt(n / d);
 }
 
+
 /* Plot the elliptical cross section of the given ehy at distance h along the
  * ehy height vector (h >= 0, h <= |H|) consisting of num_points points.
  */
 static void
 ehy_plot_ellipse(
-	struct bu_list *vhead,
-	struct rt_ehy_internal *ehy,
-	fastf_t h,
-	fastf_t num_points)
+    struct bu_list *vhead,
+    struct rt_ehy_internal *ehy,
+    fastf_t h,
+    fastf_t num_points)
 {
     fastf_t mag_H;
     vect_t V, Hu, Au, Bu, A, B, cross_section_plane;
@@ -787,18 +795,19 @@ ehy_plot_ellipse(
     plot_ellipse(vhead, cross_section_plane, A, B, num_points);
 }
 
+
 static void
 ehy_plot_hyperbola(
-	struct bu_list *vhead,
-	struct rt_ehy_internal *ehy,
-	struct rt_pt_node *pts,
-	vect_t Ru,
-	fastf_t r)
+    struct bu_list *vhead,
+    struct rt_ehy_internal *ehy,
+    struct rt_pnt_node *pts,
+    vect_t Ru,
+    fastf_t r)
 {
     point_t p;
     vect_t ehy_V, Hu;
     fastf_t mag_H, c, z;
-    struct rt_pt_node *node;
+    struct rt_pnt_node *node;
 
     VMOVE(ehy_V, ehy->ehy_V);
     mag_H = MAGNITUDE(ehy->ehy_H);
@@ -820,6 +829,7 @@ ehy_plot_hyperbola(
     }
 }
 
+
 static int
 ehy_curve_points(
     const struct rt_ehy_internal *ehy,
@@ -833,11 +843,12 @@ ehy_curve_points(
     VADD2(p1, ehy->ehy_V, ehy->ehy_H);
     VJOIN1(p2, ehy->ehy_V, avg_r, ehy->ehy_Au);
 
-    approx_curve_len = 2.0 * DIST_PT_PT(p1, p2);
+    approx_curve_len = 2.0 * DIST_PNT_PNT(p1, p2);
 
     return approx_curve_len / info->point_spacing;
 }
 
+
 static int
 ehy_ellipse_points(
     const struct rt_ehy_internal *ehy,
@@ -846,11 +857,12 @@ ehy_ellipse_points(
     fastf_t avg_radius, avg_circumference;
 
     avg_radius = (ehy->ehy_r1 + ehy->ehy_r2) / 2.0;
-    avg_circumference = bn_twopi * avg_radius;
+    avg_circumference = M_2PI * avg_radius;
 
     return avg_circumference / info->point_spacing;
 }
 
+
 int
 rt_ehy_adaptive_plot(struct rt_db_internal *ip, const struct rt_view_info *info)
 {
@@ -858,7 +870,7 @@ rt_ehy_adaptive_plot(struct rt_db_internal *ip, const struct rt_view_info *info)
     fastf_t mag_H, z, z_step, c, r1, r2;
     int i, num_curve_points, num_ellipse_points, num_curves;
     struct rt_ehy_internal *ehy;
-    struct rt_pt_node *pts_r1, *pts_r2, *node, *node1, *node2;
+    struct rt_pnt_node *pts_r1, *pts_r2, *node, *node1, *node2;
 
     BU_CK_LIST_HEAD(info->vhead);
     RT_CK_DB_INTERNAL(ip);
@@ -918,30 +930,29 @@ rt_ehy_adaptive_plot(struct rt_db_internal *ip, const struct rt_view_info *info)
     for (i = 0; i < num_curve_points; ++i) {
 	node = node1;
 	node1 = node1->next;
-	bu_free(node, "rt_pt_node");
+	bu_free(node, "rt_pnt_node");
 
 	node = node2;
 	node2 = node2->next;
-	bu_free(node, "rt_pt_node");
+	bu_free(node, "rt_pnt_node");
     }
 
     return 0;
 }
 
-/**
- * R T _ E H Y _ P L O T
- */
+
 int
-rt_ehy_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct rt_tess_tol *ttol, const struct bn_tol *UNUSED(tol), const struct rt_view_info *UNUSED(info))
+rt_ehy_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct bg_tess_tol *ttol, const struct bn_tol *UNUSED(tol), const struct rt_view_info *UNUSED(info))
 {
     fastf_t c, dtol, mag_h, ntol, r1, r2;
     fastf_t **ellipses, theta_prev, theta_new;
-    int *pts_dbl, i, j, nseg;
-    int jj, na, nb, nell, recalc_b;
+    int *pts_dbl;
+    size_t i, j, nseg, nell;
+    int jj, na, nb, recalc_b;
     mat_t R;
     mat_t invR;
     point_t p1;
-    struct rt_pt_node *pos_a, *pos_b, *pts_a, *pts_b;
+    struct rt_pnt_node *pos_a, *pos_b, *pts_a, *pts_b;
     struct rt_ehy_internal *xip;
     vect_t A, Au, B, Bu, Hu, V, Work;
 
@@ -984,8 +995,8 @@ rt_ehy_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct rt_te
      */
 
     /* approximate positive half of hyperbola along semi-minor axis */
-    BU_ALLOC(pts_b, struct rt_pt_node);
-    BU_ALLOC(pts_b->next, struct rt_pt_node);
+    BU_ALLOC(pts_b, struct rt_pnt_node);
+    BU_ALLOC(pts_b->next, struct rt_pnt_node);
 
     pts_b->next->next = NULL;
     VSET(pts_b->p,       0, 0, -mag_h);
@@ -999,14 +1010,14 @@ rt_ehy_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct rt_te
     /* construct positive half of hyperbola along semi-major axis of
      * ehy using same z coords as hyperbola along semi-minor axis
      */
-    BU_ALLOC(pts_a, struct rt_pt_node);
+    BU_ALLOC(pts_a, struct rt_pnt_node);
     VMOVE(pts_a->p, pts_b->p);	/* 1st pt is the apex */
     pts_a->next = NULL;
     pos_b = pts_b->next;
     pos_a = pts_a;
     while (pos_b) {
 	/* copy node from b_hyperbola to a_hyperbola */
-	BU_ALLOC(pos_a->next, struct rt_pt_node);
+	BU_ALLOC(pos_a->next, struct rt_pnt_node);
 	pos_a = pos_a->next;
 	pos_a->p[Z] = pos_b->p[Z];
 	/* at given z, find y on hyperbola */
@@ -1037,24 +1048,24 @@ rt_ehy_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct rt_te
 	/* free mem for old approximation of hyperbola */
 	pos_b = pts_b;
 	while (pos_b) {
-	    struct rt_pt_node *next;
+	    struct rt_pnt_node *next;
 
 	    /* get next node before freeing */
 	    next = pos_b->next;
-	    bu_free((char *)pos_b, "rt_pt_node");
+	    bu_free((char *)pos_b, "rt_pnt_node");
 	    pos_b = next;
 	}
 	/* construct hyperbola along semi-major axis of ehy using same
 	 * z coords as parab along semi-minor axis
 	 */
-	BU_ALLOC(pts_b, struct rt_pt_node);
+	BU_ALLOC(pts_b, struct rt_pnt_node);
 	pts_b->p[Z] = pts_a->p[Z];
 	pts_b->next = NULL;
 	pos_a = pts_a->next;
 	pos_b = pts_b;
 	while (pos_a) {
 	    /* copy node from a_hyperbola to b_hyperbola */
-	    BU_ALLOC(pos_b->next, struct rt_pt_node);
+	    BU_ALLOC(pos_b->next, struct rt_pnt_node);
 	    pos_b = pos_b->next;
 	    pos_b->p[Z] = pos_a->p[Z];
 	    /* at given z, find y on hyperbola */
@@ -1078,7 +1089,7 @@ rt_ehy_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct rt_te
     /* make ellipses at each z level */
     i = 0;
     nseg = 0;
-    theta_prev = bn_twopi;
+    theta_prev = M_2PI;
     pos_a = pts_a->next;	/* skip over apex of ehy */
     pos_b = pts_b->next;
     while (pos_a) {
@@ -1089,7 +1100,7 @@ rt_ehy_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct rt_te
 	VSET(p1, 0., pos_b->p[Y], 0.);
 	theta_new = ell_angle(p1, pos_a->p[Y], pos_b->p[Y], dtol, ntol);
 	if (nseg == 0) {
-	    nseg = (int)(bn_twopi / theta_new) + 1;
+	    nseg = (int)(M_2PI / theta_new) + 1;
 	    pts_dbl[i] = 0;
 	} else if (theta_new < theta_prev) {
 	    nseg *= 2;
@@ -1118,12 +1129,12 @@ rt_ehy_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct rt_te
     }
 
     /* connect ellipses */
-    for (i = nell-2; i >= 0; i--) {
+    for (i = nell-1; i > 0; i--) {
 	/* skip top ellipse */
 	int bottom, top;
 
-	top = i + 1;
-	bottom = i;
+	top = i;
+	bottom = i - 1;
 	if (pts_dbl[top])
 	    nseg /= 2;	/* # segs in 'bottom' ellipse */
 
@@ -1173,19 +1184,19 @@ rt_ehy_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct rt_te
 
 
 /**
- * R T _ E H Y _ T E S S
- *
  * Returns -
  * -1 failure
  * 0 OK.  *r points to nmgregion that holds this tessellation.
  */
 int
-rt_ehy_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, const struct rt_tess_tol *ttol, const struct bn_tol *tol)
+rt_ehy_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, const struct bg_tess_tol *ttol, const struct bn_tol *tol)
 {
     fastf_t c, dtol, mag_h, ntol, r1, r2, cprime;
     fastf_t **ellipses, theta_prev, theta_new;
-    int *pts_dbl, face, i, j, nseg;
-    int jj, na, nb, nell, recalc_b;
+    int *pts_dbl;
+    int idx;
+    size_t face, i, j, nseg, nell;
+    int jj, na, nb, recalc_b;
     mat_t R;
     mat_t invR;
     mat_t invRoS;
@@ -1193,7 +1204,7 @@ rt_ehy_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
     mat_t SoR;
     struct rt_ehy_internal *xip;
     point_t p1;
-    struct rt_pt_node *pos_a, *pos_b, *pts_a, *pts_b;
+    struct rt_pnt_node *pos_a, *pos_b, *pts_a, *pts_b;
     struct shell *s;
     struct faceuse **outfaceuses = NULL;
     struct faceuse *fu_top;
@@ -1253,8 +1264,8 @@ rt_ehy_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
      */
 
     /* approximate positive half of hyperbola along semi-minor axis */
-    BU_ALLOC(pts_b, struct rt_pt_node);
-    BU_ALLOC(pts_b->next, struct rt_pt_node);
+    BU_ALLOC(pts_b, struct rt_pnt_node);
+    BU_ALLOC(pts_b->next, struct rt_pnt_node);
 
     pts_b->next->next = NULL;
     VSET(pts_b->p,       0, 0, -mag_h);
@@ -1268,14 +1279,14 @@ rt_ehy_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
     /* construct positive half of hyperbola along semi-major axis of
      * ehy using same z coords as parab along semi-minor axis
      */
-    BU_ALLOC(pts_a, struct rt_pt_node);
+    BU_ALLOC(pts_a, struct rt_pnt_node);
     VMOVE(pts_a->p, pts_b->p);	/* 1st pt is the apex */
     pts_a->next = NULL;
     pos_b = pts_b->next;
     pos_a = pts_a;
     while (pos_b) {
 	/* copy node from b_hyperbola to a_hyperbola */
-	BU_ALLOC(pos_a->next, struct rt_pt_node);
+	BU_ALLOC(pos_a->next, struct rt_pnt_node);
 	pos_a = pos_a->next;
 	pos_a->p[Z] = pos_b->p[Z];
 	/* at given z, find y on hyperbola */
@@ -1306,23 +1317,23 @@ rt_ehy_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
 	/* free mem for old approximation of hyperbola */
 	pos_b = pts_b;
 	while (pos_b) {
-	    struct rt_pt_node *tmp;
+	    struct rt_pnt_node *tmp;
 
 	    tmp = pos_b->next;
-	    bu_free((char *)pos_b, "rt_pt_node");
+	    bu_free((char *)pos_b, "rt_pnt_node");
 	    pos_b = tmp;
 	}
 	/* construct hyperbola along semi-major axis of ehy using same
 	 * z coords as parab along semi-minor axis
 	 */
-	BU_ALLOC(pts_b, struct rt_pt_node);
+	BU_ALLOC(pts_b, struct rt_pnt_node);
 	pts_b->p[Z] = pts_a->p[Z];
 	pts_b->next = NULL;
 	pos_a = pts_a->next;
 	pos_b = pts_b;
 	while (pos_a) {
 	    /* copy node from a_hyperbola to b_hyperbola */
-	    BU_ALLOC(pos_b->next, struct rt_pt_node);
+	    BU_ALLOC(pos_b->next, struct rt_pnt_node);
 	    pos_b = pos_b->next;
 	    pos_b->p[Z] = pos_a->p[Z];
 	    /* at given z, find y on hyperbola */
@@ -1347,7 +1358,7 @@ rt_ehy_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
     /* make ellipses at each z level */
     i = 0;
     nseg = 0;
-    theta_prev = bn_twopi;
+    theta_prev = M_2PI;
     pos_a = pts_a->next;	/* skip over apex of ehy */
     pos_b = pts_b->next;
     while (pos_a) {
@@ -1358,7 +1369,7 @@ rt_ehy_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
 	VSET(p1, 0., pos_b->p[Y], 0.);
 	theta_new = ell_angle(p1, pos_a->p[Y], pos_b->p[Y], dtol, ntol);
 	if (nseg == 0) {
-	    nseg = (int)(bn_twopi / theta_new) + 1;
+	    nseg = (size_t)(M_2PI / theta_new) + 1;
 	    pts_dbl[i] = 0;
 	    /* maximum number of faces needed for ehy */
 	    face = nseg*(1 + 3*((1 << (nell-1)) - 1));
@@ -1393,18 +1404,17 @@ rt_ehy_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
     vells = (struct vertex ***)
 	bu_malloc(nell*sizeof(struct vertex **), "vertex [][]");
     j = nseg;
-    for (i = nell-1; i >= 0; i--) {
-	vells[i] = (struct vertex **)
-	    bu_malloc(j*sizeof(struct vertex *), "vertex []");
+    for (i = 0; i < nell; i++) {
+	vells[i] = (struct vertex **)bu_malloc(j*sizeof(struct vertex *), "vertex []");
 	if (i && pts_dbl[i])
-	    j /= 2;
+	    j /=2;
     }
 
     /* top face of ehy */
     for (i = 0; i < nseg; i++)
 	vells[nell-1][i] = (struct vertex *)NULL;
     face = 0;
-    BU_ASSERT_PTR(outfaceuses, !=, NULL);
+    BU_ASSERT(outfaceuses != NULL);
     if ((outfaceuses[face++] = nmg_cface(s, vells[nell-1], nseg)) == 0) {
 	bu_log("rt_ehy_tess() failure, top face\n");
 	goto fail;
@@ -1433,12 +1443,12 @@ rt_ehy_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
     }
 
     /* connect ellipses with triangles */
-    for (i = nell-2; i >= 0; i--) {
+    for (idx = nell-2; idx >= 0; idx--) {
 	/* skip top ellipse */
 	int bottom, top;
 
-	top = i + 1;
-	bottom = i;
+	top = idx + 1;
+	bottom = idx;
 	if (pts_dbl[top])
 	    nseg /= 2;	/* # segs in 'bottom' ellipse */
 	vertp[0] = (struct vertex *)0;
@@ -1544,19 +1554,19 @@ rt_ehy_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
     }
 
     /* Associate the face geometry */
-    for (i=0; i < face; i++) {
+    for (i = 0; i < face; i++) {
 	if (nmg_fu_planeeqn(outfaceuses[i], tol) < 0)
 	    goto fail;
     }
 
     /* Glue the edges of different outward pointing face uses together */
-    nmg_gluefaces(outfaceuses, face, tol);
+    nmg_gluefaces(outfaceuses, face, &RTG.rtg_vlfree, tol);
 
     /* Compute "geometry" for region and shell */
     nmg_region_a(*r, tol);
 
     /* XXX just for testing, to make up for loads of triangles ... */
-    nmg_shell_coplanar_face_merge(s, tol, 1);
+    nmg_shell_coplanar_face_merge(s, tol, 1, &RTG.rtg_vlfree);
 
     /* free mem */
     bu_free((char *)outfaceuses, "faceuse []");
@@ -1568,8 +1578,8 @@ rt_ehy_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
     bu_free((char *)vells, "vertex [][]");
 
     /* Assign vertexuse normals */
-    nmg_vertex_tabulate(&vert_tab, &s->l.magic);
-    for (i=0; i<BU_PTBL_END(&vert_tab); i++) {
+    nmg_vertex_tabulate(&vert_tab, &s->l.magic, &RTG.rtg_vlfree);
+    for (i = 0; i < BU_PTBL_LEN(&vert_tab); i++) {
 	point_t pt_prime, tmp_pt;
 	vect_t norm, rev_norm, tmp_vect;
 	struct vertex_g *vg;
@@ -1609,7 +1619,7 @@ rt_ehy_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
     bu_ptbl_free(&vert_tab);
     return 0;
 
- fail:
+fail:
     /* free mem */
     bu_free((char *)outfaceuses, "faceuse []");
     for (i = 0; i < nell; i++) {
@@ -1624,8 +1634,6 @@ rt_ehy_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
 
 
 /**
- * R T _ E H Y _ I M P O R T
- *
  * Import an EHY from the database format to the internal format.
  * Apply modeling transformations as well.
  */
@@ -1699,8 +1707,6 @@ rt_ehy_import4(struct rt_db_internal *ip, const struct bu_external *ep, const fa
 
 
 /**
- * R T _ E H Y _ E X P O R T
- *
  * The name is added by the caller, in the usual place.
  */
 int
@@ -1718,7 +1724,7 @@ rt_ehy_export4(struct bu_external *ep, const struct rt_db_internal *ip, double l
 
     BU_CK_EXTERNAL(ep);
     ep->ext_nbytes = sizeof(union record);
-    ep->ext_buf = (genptr_t)bu_calloc(1, ep->ext_nbytes, "ehy external");
+    ep->ext_buf = (uint8_t *)bu_calloc(1, ep->ext_nbytes, "ehy external");
     ehy = (union record *)ep->ext_buf;
 
     ehy->s.s_id = ID_SOLID;
@@ -1761,8 +1767,6 @@ rt_ehy_export4(struct bu_external *ep, const struct rt_db_internal *ip, double l
 
 
 /**
- * R T _ E H Y _ I M P O R T 5
- *
  * Import an EHY from the database format to the internal format.
  * Apply modeling transformations as well.
  */
@@ -1777,7 +1781,7 @@ rt_ehy_import5(struct rt_db_internal *ip, const struct bu_external *ep, const fa
     BU_CK_EXTERNAL(ep);
     if (dbip) RT_CK_DBI(dbip);
 
-    BU_ASSERT_LONG(ep->ext_nbytes, ==, SIZEOF_NETWORK_DOUBLE * 3*4);
+    BU_ASSERT(ep->ext_nbytes == SIZEOF_NETWORK_DOUBLE * 3*4);
 
     RT_CK_DB_INTERNAL(ip);
     ip->idb_major_type = DB5_MAJORTYPE_BRLCAD;
@@ -1812,8 +1816,6 @@ rt_ehy_import5(struct rt_db_internal *ip, const struct bu_external *ep, const fa
 
 
 /**
- * R T _ E H Y _ E X P O R T 5
- *
  * The name is added by the caller, in the usual place.
  */
 int
@@ -1833,7 +1835,7 @@ rt_ehy_export5(struct bu_external *ep, const struct rt_db_internal *ip, double l
 
     BU_CK_EXTERNAL(ep);
     ep->ext_nbytes = SIZEOF_NETWORK_DOUBLE * 3*4;
-    ep->ext_buf = (genptr_t)bu_malloc(ep->ext_nbytes, "ehy external");
+    ep->ext_buf = (uint8_t *)bu_malloc(ep->ext_nbytes, "ehy external");
 
     if (!NEAR_EQUAL(MAGNITUDE(xip->ehy_Au), 1.0, RT_LEN_TOL)) {
 	bu_log("rt_ehy_export4: Au not a unit vector!\n");
@@ -1875,8 +1877,6 @@ rt_ehy_export5(struct bu_external *ep, const struct rt_db_internal *ip, double l
 
 
 /**
- * R T _ E H Y _ D E S C R I B E
- *
  * Make human-readable formatted presentation of this solid.  First
  * line describes type of solid.  Additional lines are indented one
  * tab, and give parameter values.
@@ -1920,8 +1920,6 @@ rt_ehy_describe(struct bu_vls *str, const struct rt_db_internal *ip, int verbose
 
 
 /**
- * R T _ E H Y _ I F R E E
- *
  * Free the storage associated with the rt_db_internal version of this
  * solid.
  */
@@ -1937,14 +1935,10 @@ rt_ehy_ifree(struct rt_db_internal *ip)
     xip->ehy_magic = 0;		/* sanity */
 
     bu_free((char *)xip, "ehy ifree");
-    ip->idb_ptr = GENPTR_NULL;	/* sanity */
+    ip->idb_ptr = ((void *)0);	/* sanity */
 }
 
 
-/**
- * R T _ E H Y _ P A R A M S
- *
- */
 int
 rt_ehy_params(struct pc_pc_set *ps, const struct rt_db_internal *ip)
 {
@@ -1954,6 +1948,7 @@ rt_ehy_params(struct pc_pc_set *ps, const struct rt_db_internal *ip)
     return 0;			/* OK */
 }
 
+
 static int
 ehy_is_valid(struct rt_ehy_internal *ehy)
 {
@@ -1991,6 +1986,66 @@ ehy_is_valid(struct rt_ehy_internal *ehy)
     return 1;
 }
 
+
+void
+rt_ehy_surf_area(fastf_t *area, const struct rt_db_internal *ip)
+{
+    struct rt_ehy_internal *eip;
+    fastf_t a, b, h, integralArea, sqrt_rb;
+    RT_CK_DB_INTERNAL(ip);
+    eip = (struct rt_ehy_internal *)ip->idb_ptr;
+    RT_EHY_CK_MAGIC(eip);
+
+    a = eip->ehy_c;
+    h = MAGNITUDE(eip->ehy_H);
+    b = (eip->ehy_r1 * a) / sqrt(h * (h - 2 * a));
+
+    /** Formula taken from : https://docs.google.com/file/d/0BydeQ6BPlVejRWt6NlJLVDl0d28/edit
+     * Area can be calculated by subtracting integral of hyperbola from the area of the bounding rectangle
+     */
+    sqrt_rb = sqrt(eip->ehy_r1 * eip->ehy_r1 + b * b);
+    integralArea = (a / b) * ((eip->ehy_r1 * sqrt_rb) + ((b * b / 2) * (log(sqrt_rb + eip->ehy_r1) - log(sqrt_rb - eip->ehy_r1))));
+    *area = 2 * eip->ehy_r1 * (a + h) - integralArea;
+}
+
+
+/**
+ * The centroid lies along ehy_H due to symmetry.
+ * Initially the distance of the centroid from the apex is found. The
+ * coordinates of the points at this distance along the unit vector
+ * gives the centroid of the elliptical hyperboloid of two sheets.
+ * Formula taken from: https://docs.google.com/file/d/0BydeQ6BPlVejRWt6NlJLVDl0d28/edit
+ */
+void
+rt_ehy_centroid(point_t *cent, const struct rt_db_internal *ip)
+{
+    struct rt_ehy_internal *eip;
+    fastf_t a, b, h, area, dist, pwr, dist_C;
+    vect_t h_vec, unit_vec;
+    point_t apex;
+    RT_CK_DB_INTERNAL(ip);
+    eip =  (struct rt_ehy_internal *)ip->idb_ptr;
+    RT_EHY_CK_MAGIC(eip);
+
+    a = eip->ehy_c;
+    h = MAGNITUDE(eip->ehy_H);
+    b = (eip->ehy_r1 * a) / sqrt(h * h - 2 * a * h);
+
+    VMOVE(h_vec, eip->ehy_H);
+    VMOVE(apex, eip->ehy_V);
+    VSCALE(unit_vec, h_vec, (1 / h));
+
+    rt_ehy_surf_area( &area, ip);
+    pwr = pow(h * h + 2 * a * h, 3);
+    if(pwr < 0)
+	bu_log("invalid parameters.\n");
+    dist = ((2 * b) * sqrt(pwr)) / (3 * area * a);
+    dist_C = dist - a;
+
+    VJOIN1(*cent, apex, dist_C, unit_vec);
+}
+
+
 /** @} */
 /*
  * Local Variables:
