@@ -1,7 +1,7 @@
 /*                           E P A . C
  * BRL-CAD
  *
- * Copyright (c) 1990-2013 United States Government as represented by
+ * Copyright (c) 1990-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This library is free software; you can redistribute it and/or
@@ -147,15 +147,15 @@
 #include "common.h"
 
 #include <stddef.h>
-#include <stdio.h>
 #include <string.h>
 #include <math.h>
 #include "bio.h"
 
+#include "bu/cv.h"
 #include "vmath.h"
-#include "db.h"
+#include "rt/db4.h"
 #include "nmg.h"
-#include "rtgeom.h"
+#include "rt/geom.h"
 #include "raytrace.h"
 
 #include "../../librt_private.h"
@@ -184,9 +184,37 @@ const struct bu_structparse rt_epa_parse[] = {
     { {'\0', '\0', '\0', '\0'}, 0, (char *)NULL, 0, BU_STRUCTPARSE_FUNC_NULL, NULL, NULL }
 };
 
+
+#ifdef USE_OPENCL
+/* largest data members first */
+struct clt_epa_specific {
+    cl_double epa_V[3];     /* vector to epa origin */
+    cl_double epa_Hunit[3]; /* unit H vector */
+    cl_double epa_SoR[16];  /* Scale(Rot(vect)) */
+    cl_double epa_invRoS[16];   /* invRot(Scale(vect)) */
+};
+
+size_t
+clt_epa_pack(struct bu_pool *pool, struct soltab *stp)
+{
+    struct epa_specific *epa =
+        (struct epa_specific *)stp->st_specific;
+    struct clt_epa_specific *args;
+
+    const size_t size = sizeof(*args);
+    args = (struct clt_epa_specific*)bu_pool_alloc(pool, 1, size);
+
+    VMOVE(args->epa_V, epa->epa_V);
+    VMOVE(args->epa_Hunit, epa->epa_Hunit);
+    MAT_COPY(args->epa_SoR, epa->epa_SoR);
+    MAT_COPY(args->epa_invRoS, epa->epa_invRoS);
+    return size;
+}
+
+#endif /* USE_OPENCL */
+
+
 /**
- * R T _ E P A _ B B O X
- *
  * Create a bounding RPP for an epa
  */
 int
@@ -239,8 +267,6 @@ rt_epa_bbox(struct rt_db_internal *ip, point_t *min, point_t *max, const struct
 
 
 /**
- * R T _ E P A _ P R E P
- *
  * Given a pointer to a GED database record, and a transformation
  * matrix, determine if this is a valid EPA, and if so, precompute
  * various terms of the formula.
@@ -287,7 +313,7 @@ rt_epa_prep(struct soltab *stp, struct rt_db_internal *ip, struct rt_i *rtip)
     stp->st_meth = &OBJ[ID_EPA];
 
     BU_GET(epa, struct epa_specific);
-    stp->st_specific = (genptr_t)epa;
+    stp->st_specific = (void *)epa;
 
     epa->epa_h = mag_h;
     epa->epa_inv_r1sq = 1 / (r1 * r1);
@@ -333,9 +359,6 @@ rt_epa_prep(struct soltab *stp, struct rt_db_internal *ip, struct rt_i *rtip)
 }
 
 
-/**
- * R T _ E P A _ P R I N T
- */
 void
 rt_epa_print(const struct soltab *stp)
 {
@@ -357,8 +380,6 @@ rt_epa_print(const struct soltab *stp)
 
 
 /**
- * R T _ E P A _ S H O T
- *
  * Intersect a ray with a epa.  If an intersection occurs, a struct
  * seg will be acquired and filled in.
  *
@@ -481,8 +502,6 @@ rt_epa_shot(struct soltab *stp, struct xray *rp, struct application *ap, struct
 
 
 /**
- * R T _ E P A _ N O R M
- *
  * Given ONE ray distance, return the normal and entry/exit point.
  */
 void
@@ -518,8 +537,6 @@ rt_epa_norm(struct hit *hitp, struct soltab *stp, struct xray *rp)
 
 
 /**
- * R T _ E P A _ C U R V E
- *
  * Return the curvature of the epa.
  */
 void
@@ -571,8 +588,6 @@ rt_epa_curve(struct curvature *cvp, struct hit *hitp, struct soltab *stp)
 
 
 /**
- * R T _ E P A _ U V
- *
  * For a hit on the surface of an epa, return the (u, v) coordinates
  * of the hit point, 0 <= u, v <= 1.
  *
@@ -605,14 +620,14 @@ rt_epa_uv(struct application *ap, struct soltab *stp, struct hit *hitp, struct u
 		uvp->uv_u = 0;
 	    } else {
 		len = sqrt(pprime[X]*pprime[X] + pprime[Y]*pprime[Y]);
-		uvp->uv_u = acos(pprime[X]/len) * bn_inv2pi;
+		uvp->uv_u = acos(pprime[X]/len) * M_1_2PI;
 	    }
 	    uvp->uv_v = -pprime[Z];
 	    break;
 	case EPA_NORM_TOP:
 	    /* top plate, polar coords */
 	    len = sqrt(pprime[X]*pprime[X] + pprime[Y]*pprime[Y]);
-	    uvp->uv_u = acos(pprime[X]/len) * bn_inv2pi;
+	    uvp->uv_u = acos(pprime[X]/len) * M_1_2PI;
 	    uvp->uv_v = 1.0 - len;
 	    break;
     }
@@ -625,9 +640,6 @@ rt_epa_uv(struct application *ap, struct soltab *stp, struct hit *hitp, struct u
 }
 
 
-/**
- * R T _ E P A _ F R E E
- */
 void
 rt_epa_free(struct soltab *stp)
 {
@@ -638,16 +650,6 @@ rt_epa_free(struct soltab *stp)
 }
 
 
-/**
- * R T _ E P A _ C L A S S
- */
-int
-rt_epa_class(void)
-{
-    return 0;
-}
-
-
 /* A canonical parabola in the Y-Z plane has equation z = y^2 / 4p, and opens
  * toward positive z with vertex at the origin.
  *
@@ -682,14 +684,14 @@ epa_parabola_p(fastf_t r, fastf_t mag_h)
  * vertex at (0, -|H|), multiplying the z values by -1 gives corresponding
  * distances along the epa height vector H.
  */
-static struct rt_pt_node *
+static struct rt_pnt_node *
 epa_parabolic_curve(fastf_t mag_h, fastf_t r, int num_points)
 {
     int count;
-    struct rt_pt_node *curve;
+    struct rt_pnt_node *curve;
 
-    BU_ALLOC(curve, struct rt_pt_node);
-    BU_ALLOC(curve->next, struct rt_pt_node);
+    BU_ALLOC(curve, struct rt_pnt_node);
+    BU_ALLOC(curve->next, struct rt_pnt_node);
 
     curve->next->next = NULL;
     VSET(curve->p,       0, 0, -mag_h);
@@ -762,14 +764,14 @@ static void
 epa_plot_parabola(
 	struct bu_list *vhead,
 	struct rt_epa_internal *epa,
-	struct rt_pt_node *pts,
+	struct rt_pnt_node *pts,
 	vect_t Ru,
 	fastf_t r)
 {
     point_t p;
     vect_t epa_V, Hu;
     fastf_t mag_H, z;
-    struct rt_pt_node *node;
+    struct rt_pnt_node *node;
 
     VMOVE(epa_V, epa->epa_V);
     mag_H = MAGNITUDE(epa->epa_H);
@@ -803,7 +805,7 @@ epa_curve_points(
     VADD2(p1, epa->epa_V, epa->epa_H);
     VJOIN1(p2, epa->epa_V, avg_r, epa->epa_Au);
 
-    approx_curve_len = 2.0 * DIST_PT_PT(p1, p2);
+    approx_curve_len = 2.0 * DIST_PNT_PNT(p1, p2);
 
     return approx_curve_len / info->point_spacing;
 }
@@ -816,7 +818,7 @@ epa_ellipse_points(
     fastf_t avg_radius, avg_circumference;
 
     avg_radius = (epa->epa_r1 + epa->epa_r2) / 2.0;
-    avg_circumference = bn_twopi * avg_radius;
+    avg_circumference = M_2PI * avg_radius;
 
     return avg_circumference / info->point_spacing;
 }
@@ -828,7 +830,7 @@ rt_epa_adaptive_plot(struct rt_db_internal *ip, const struct rt_view_info *info)
     fastf_t mag_H, z, z_step, r1, r2;
     int i, num_curve_points, num_ellipse_points, num_curves;
     struct rt_epa_internal *epa;
-    struct rt_pt_node *pts_r1, *pts_r2, *node, *node1, *node2;
+    struct rt_pnt_node *pts_r1, *pts_r2, *node, *node1, *node2;
 
     BU_CK_LIST_HEAD(info->vhead);
     RT_CK_DB_INTERNAL(ip);
@@ -892,21 +894,18 @@ rt_epa_adaptive_plot(struct rt_db_internal *ip, const struct rt_view_info *info)
     for (i = 0; i < num_curve_points; ++i) {
 	node = node1;
 	node1 = node1->next;
-	bu_free(node, "rt_pt_node");
+	bu_free(node, "rt_pnt_node");
 
 	node = node2;
 	node2 = node2->next;
-	bu_free(node, "rt_pt_node");
+	bu_free(node, "rt_pnt_node");
     }
 
     return 0;
 }
 
-/**
- * R T _ E P A _ P L O T
- */
 int
-rt_epa_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct rt_tess_tol *ttol, const struct bn_tol *UNUSED(tol), const struct rt_view_info *UNUSED(info))
+rt_epa_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct bg_tess_tol *ttol, const struct bn_tol *UNUSED(tol), const struct rt_view_info *UNUSED(info))
 {
     fastf_t dtol, mag_h, ntol, r1, r2;
     fastf_t **ellipses, theta_new, theta_prev;
@@ -916,7 +915,7 @@ rt_epa_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct rt_te
     mat_t invR;
     struct rt_epa_internal *xip;
     point_t p1;
-    struct rt_pt_node *pos_a, *pos_b, *pts_a, *pts_b;
+    struct rt_pnt_node *pos_a, *pos_b, *pts_a, *pts_b;
     vect_t A, Au, B, Bu, Hu, V, Work;
 
     BU_CK_LIST_HEAD(vhead);
@@ -952,15 +951,15 @@ rt_epa_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct rt_te
 	ntol = ttol->norm;
     else
 	/* tolerate everything */
-	ntol = bn_pi;
+	ntol = M_PI;
 
     /*
      * build epa from 2 parabolas
      */
 
     /* approximate positive half of parabola along semi-minor axis */
-    BU_ALLOC(pts_b, struct rt_pt_node);
-    BU_ALLOC(pts_b->next, struct rt_pt_node);
+    BU_ALLOC(pts_b, struct rt_pnt_node);
+    BU_ALLOC(pts_b->next, struct rt_pnt_node);
 
     pts_b->next->next = NULL;
     VSET(pts_b->p,       0, 0, -mag_h);
@@ -975,14 +974,14 @@ rt_epa_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct rt_te
     /* construct positive half of parabola along semi-major axis of
      * epa using same z coords as parab along semi-minor axis
      */
-    BU_ALLOC(pts_a, struct rt_pt_node);
+    BU_ALLOC(pts_a, struct rt_pnt_node);
     VMOVE(pts_a->p, pts_b->p);	/* 1st pt is the apex */
     pts_a->next = NULL;
     pos_b = pts_b->next;
     pos_a = pts_a;
     while (pos_b) {
 	/* copy node from b_parabola to a_parabola */
-	BU_ALLOC(pos_a->next, struct rt_pt_node);
+	BU_ALLOC(pos_a->next, struct rt_pnt_node);
 	pos_a = pos_a->next;
 	pos_a->p[Z] = pos_b->p[Z];
 	/* at given z, find y on parabola */
@@ -1009,24 +1008,24 @@ rt_epa_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct rt_te
 	/* free mem for old approximation of parabola */
 	pos_b = pts_b;
 	while (pos_b) {
-	    struct rt_pt_node *next;
+	    struct rt_pnt_node *next;
 
 	    /* get next node before freeing */
 	    next = pos_b->next;
-	    bu_free((char *)pos_b, "rt_pt_node");
+	    bu_free((char *)pos_b, "rt_pnt_node");
 	    pos_b = next;
 	}
 	/* construct parabola along semi-major axis of epa using same
 	 * z coords as parab along semi-minor axis
 	 */
-	BU_ALLOC(pts_b, struct rt_pt_node);
+	BU_ALLOC(pts_b, struct rt_pnt_node);
 	pts_b->p[Z] = pts_a->p[Z];
 	pts_b->next = NULL;
 	pos_a = pts_a->next;
 	pos_b = pts_b;
 	while (pos_a) {
 	    /* copy node from a_parabola to b_parabola */
-	    BU_ALLOC(pos_b->next, struct rt_pt_node);
+	    BU_ALLOC(pos_b->next, struct rt_pnt_node);
 	    pos_b = pos_b->next;
 	    pos_b->p[Z] = pos_a->p[Z];
 	    /* at given z, find y on parabola */
@@ -1045,7 +1044,7 @@ rt_epa_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct rt_te
     /* make ellipses at each z level */
     i = 0;
     nseg = 0;
-    theta_prev = bn_twopi;
+    theta_prev = M_2PI;
     pos_a = pts_a->next;	/* skip over apex of epa */
     pos_b = pts_b->next;
     while (pos_a) {
@@ -1056,7 +1055,7 @@ rt_epa_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct rt_te
 	VSET(p1, 0., pos_b->p[Y], 0.);
 	theta_new = ell_angle(p1, pos_a->p[Y], pos_b->p[Y], dtol, ntol);
 	if (nseg == 0) {
-	    nseg = (int)(bn_twopi / theta_new) + 1;
+	    nseg = (int)(M_2PI / theta_new) + 1;
 	    pts_dbl[i] = 0;
 	} else if (theta_new < theta_prev) {
 	    nseg *= 2;
@@ -1150,7 +1149,7 @@ rt_ell_norms(fastf_t *ov, fastf_t *A, fastf_t *B, fastf_t *h_vec, fastf_t t, int
     sqrt_1mt = sqrt(1.0 - t);
     if (sqrt_1mt <= SMALL_FASTF)
 	bu_bomb("rt_epa_tess: rt_ell_norms: sqrt(1.0 -t) is zero\n");
-    theta = 2 * bn_pi / sides;
+    theta = M_2PI / sides;
     ang = 0.;
 
     for (n = 1; n <= sides; n++, ang += theta) {
@@ -1166,8 +1165,6 @@ rt_ell_norms(fastf_t *ov, fastf_t *A, fastf_t *B, fastf_t *h_vec, fastf_t t, int
 
 
 /**
- * R T _ E L L
- *
  * Generate an ellipsoid with the specified number of sides approximating it.
  */
 void
@@ -1176,7 +1173,7 @@ rt_ell(fastf_t *ov, const fastf_t *V, const fastf_t *A, const fastf_t *B, int si
     fastf_t ang, theta, x, y;
     int n;
 
-    theta = 2 * bn_pi / sides;
+    theta = M_2PI / sides;
     ang = 0.;
     /* make ellipse regardless of whether it meets req's */
     for (n = 1; n <= sides; n++, ang += theta) {
@@ -1189,14 +1186,12 @@ rt_ell(fastf_t *ov, const fastf_t *V, const fastf_t *A, const fastf_t *B, int si
 
 
 /**
- * R T _ E P A _ T E S S
- *
  * Returns -
  * -1 failure
  * 0 OK.  *r points to nmgregion that holds this tessellation.
  */
 int
-rt_epa_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, const struct rt_tess_tol *ttol, const struct bn_tol *tol)
+rt_epa_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, const struct bg_tess_tol *ttol, const struct bn_tol *tol)
 {
     fastf_t dtol, mag_h, ntol, r1, r2;
     fastf_t **ellipses, **normals, theta_new, theta_prev;
@@ -1207,7 +1202,7 @@ rt_epa_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
     mat_t invR;
     struct rt_epa_internal *xip;
     point_t p1;
-    struct rt_pt_node *pos_a, *pos_b, *pts_a, *pts_b;
+    struct rt_pnt_node *pos_a, *pos_b, *pts_a, *pts_b;
     struct shell *s;
     struct faceuse **outfaceuses = NULL;
     struct vertex *vertp[3];
@@ -1255,15 +1250,15 @@ rt_epa_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
 	ntol = ttol->norm;
     else
 	/* tolerate everything */
-	ntol = bn_pi;
+	ntol = M_PI;
 
     /*
      * build epa from 2 parabolas
      */
 
     /* approximate positive half of parabola along semi-minor axis */
-    BU_ALLOC(pts_b, struct rt_pt_node);
-    BU_ALLOC(pts_b->next, struct rt_pt_node);
+    BU_ALLOC(pts_b, struct rt_pnt_node);
+    BU_ALLOC(pts_b->next, struct rt_pnt_node);
 
     pts_b->next->next = NULL;
     VSET(pts_b->p,       0, 0, -mag_h);
@@ -1277,14 +1272,14 @@ rt_epa_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
     /* construct positive half of parabola along semi-major axis of
      * epa using same z coords as parab along semi-minor axis
      */
-    BU_ALLOC(pts_a, struct rt_pt_node);
+    BU_ALLOC(pts_a, struct rt_pnt_node);
     VMOVE(pts_a->p, pts_b->p);	/* 1st pt is the apex */
     pts_a->next = NULL;
     pos_b = pts_b->next;
     pos_a = pts_a;
     while (pos_b) {
 	/* copy node from b_parabola to a_parabola */
-	BU_ALLOC(pos_a->next, struct rt_pt_node);
+	BU_ALLOC(pos_a->next, struct rt_pnt_node);
 	pos_a = pos_a->next;
 	pos_a->p[Z] = pos_b->p[Z];
 	/* at given z, find y on parabola */
@@ -1311,23 +1306,23 @@ rt_epa_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
 	/* free mem for old approximation of parabola */
 	pos_b = pts_b;
 	while (pos_b) {
-	    struct rt_pt_node *tmp;
+	    struct rt_pnt_node *tmp;
 
 	    tmp = pos_b->next;
-	    bu_free((char *)pos_b, "rt_pt_node");
+	    bu_free((char *)pos_b, "rt_pnt_node");
 	    pos_b = tmp;
 	}
 	/* construct parabola along semi-major axis of epa
 	 * using same z coords as parab along semi-minor axis
 	 */
-	BU_ALLOC(pts_b, struct rt_pt_node);
+	BU_ALLOC(pts_b, struct rt_pnt_node);
 	pts_b->p[Z] = pts_a->p[Z];
 	pts_b->next = NULL;
 	pos_a = pts_a->next;
 	pos_b = pts_b;
 	while (pos_a) {
 	    /* copy node from a_parabola to b_parabola */
-	    BU_ALLOC(pos_b->next, struct rt_pt_node);
+	    BU_ALLOC(pos_b->next, struct rt_pnt_node);
 	    pos_b = pos_b->next;
 	    pos_b->p[Z] = pos_a->p[Z];
 	    /* at given z, find y on parabola */
@@ -1353,7 +1348,7 @@ rt_epa_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
     /* make ellipses at each z level */
     i = 0;
     nseg = 0;
-    theta_prev = bn_twopi;
+    theta_prev = M_2PI;
     pos_a = pts_a->next;	/* skip over apex of epa */
     pos_b = pts_b->next;
     while (pos_a) {
@@ -1367,7 +1362,7 @@ rt_epa_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
 	VSET(p1, 0., pos_b->p[Y], 0.);
 	theta_new = ell_angle(p1, pos_a->p[Y], pos_b->p[Y], dtol, ntol);
 	if (nseg == 0) {
-	    nseg = (int)(bn_twopi / theta_new) + 1;
+	    nseg = (int)(M_2PI / theta_new) + 1;
 	    pts_dbl[i] = 0;
 	    /* maximum number of faces needed for epa */
 	    face = nseg*(1 + 3*((1 << (nell-1)) - 1));
@@ -1426,7 +1421,7 @@ rt_epa_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
     }
 
     /* Mark the edges of this face as real, this is the only real edge */
-    (void)nmg_mark_edges_real(&outfaceuses[0]->l.magic);
+    (void)nmg_mark_edges_real(&outfaceuses[0]->l.magic, &RTG.rtg_vlfree);
 
     /* connect ellipses with triangles */
     for (i = nell-2; i >= 0; i--) {
@@ -1541,14 +1536,14 @@ rt_epa_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
     }
 
     /* Associate the face geometry */
-    for (i=0; i < face; i++) {
+    for (i = 0; i < face; i++) {
 	if (nmg_fu_planeeqn(outfaceuses[i], tol) < 0)
 	    goto fail;
     }
 
     /* Associate vertexuse normals */
-    for (i=0; i<nell; i++) {
-	for (j=0; j<segs_per_ell[i]; j++) {
+    for (i = 0; i < nell; i++) {
+	for (j = 0; j < segs_per_ell[i]; j++) {
 	    VREVERSE(rev_norm, &normals[i][j*3]);
 	    for (BU_LIST_FOR(vu, vertexuse, &vells[i][j]->vu_hd)) {
 
@@ -1580,13 +1575,13 @@ rt_epa_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
     }
 
     /* Glue the edges of different outward pointing face uses together */
-    nmg_gluefaces(outfaceuses, face, tol);
+    nmg_gluefaces(outfaceuses, face, &RTG.rtg_vlfree, tol);
 
     /* Compute "geometry" for region and shell */
     nmg_region_a(*r, tol);
 
     /* XXX just for testing, to make up for loads of triangles ... */
-    nmg_shell_coplanar_face_merge(s, tol, 1);
+    nmg_shell_coplanar_face_merge(s, tol, 1, &RTG.rtg_vlfree);
 
     /* free mem */
     bu_free((char *)outfaceuses, "faceuse []");
@@ -1616,8 +1611,6 @@ rt_epa_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
 
 
 /**
- * R T _ E P A _ I M P O R T
- *
  * Import an EPA from the database format to the internal format.
  * Apply modeling transformations as well.
  */
@@ -1688,8 +1681,6 @@ rt_epa_import4(struct rt_db_internal *ip, const struct bu_external *ep, const fa
 
 
 /**
- * R T _ E P A _ E X P O R T
- *
  * The name is added by the caller, in the usual place.
  */
 int
@@ -1708,7 +1699,7 @@ rt_epa_export4(struct bu_external *ep, const struct rt_db_internal *ip, double l
 
     BU_CK_EXTERNAL(ep);
     ep->ext_nbytes = sizeof(union record);
-    ep->ext_buf = (genptr_t)bu_calloc(1, ep->ext_nbytes, "epa external");
+    ep->ext_buf = (uint8_t *)bu_calloc(1, ep->ext_nbytes, "epa external");
     epa = (union record *)ep->ext_buf;
 
     epa->s.s_id = ID_SOLID;
@@ -1750,8 +1741,6 @@ rt_epa_export4(struct bu_external *ep, const struct rt_db_internal *ip, double l
 
 
 /**
- * R T _ E P A _ I M P O R T 5
- *
  * Import an EPA from the database format to the internal format.
  * Apply modeling transformations as well.
  */
@@ -1766,7 +1755,7 @@ rt_epa_import5(struct rt_db_internal *ip, const struct bu_external *ep, const fa
     if (dbip) RT_CK_DBI(dbip);
     BU_CK_EXTERNAL(ep);
 
-    BU_ASSERT_LONG(ep->ext_nbytes, ==, SIZEOF_NETWORK_DOUBLE * 11);
+    BU_ASSERT(ep->ext_nbytes == SIZEOF_NETWORK_DOUBLE * 11);
 
     RT_CK_DB_INTERNAL(ip);
     ip->idb_major_type = DB5_MAJORTYPE_BRLCAD;
@@ -1800,8 +1789,6 @@ rt_epa_import5(struct rt_db_internal *ip, const struct bu_external *ep, const fa
 
 
 /**
- * R T _ E P A _ E X P O R T 5
- *
  * The name is added by the caller, in the usual place.
  */
 int
@@ -1822,7 +1809,7 @@ rt_epa_export5(struct bu_external *ep, const struct rt_db_internal *ip, double l
 
     BU_CK_EXTERNAL(ep);
     ep->ext_nbytes = SIZEOF_NETWORK_DOUBLE * 11;
-    ep->ext_buf = (genptr_t)bu_malloc(ep->ext_nbytes, "epa external");
+    ep->ext_buf = (uint8_t *)bu_malloc(ep->ext_nbytes, "epa external");
 
     if (!NEAR_EQUAL(MAGNITUDE(xip->epa_Au), 1.0, RT_LEN_TOL)) {
 	bu_log("rt_epa_export4: Au not a unit vector!\n");
@@ -1863,8 +1850,6 @@ rt_epa_export5(struct bu_external *ep, const struct rt_db_internal *ip, double l
 
 
 /**
- * R T _ E P A _ D E S C R I B E
- *
  * Make human-readable formatted presentation of this solid.  First
  * line describes type of solid.  Additional lines are indented one
  * tab, and give parameter values.
@@ -1905,8 +1890,6 @@ rt_epa_describe(struct bu_vls *str, const struct rt_db_internal *ip, int verbose
 
 
 /**
- * R T _ E P A _ I F R E E
- *
  * Free the storage associated with the rt_db_internal version of this
  * solid.
  */
@@ -1922,14 +1905,10 @@ rt_epa_ifree(struct rt_db_internal *ip)
     xip->epa_magic = 0;		/* sanity */
 
     bu_free((char *)xip, "epa ifree");
-    ip->idb_ptr = GENPTR_NULL;	/* sanity */
+    ip->idb_ptr = ((void *)0);	/* sanity */
 }
 
 
-/**
- * R T _ E P A _ P A R A M S
- *
- */
 int
 rt_epa_params(struct pc_pc_set *ps, const struct rt_db_internal *ip)
 {
@@ -1940,9 +1919,6 @@ rt_epa_params(struct pc_pc_set *ps, const struct rt_db_internal *ip)
 }
 
 
-/**
- * R T _ E P A _ V O L U M E
- */
 void
 rt_epa_volume(fastf_t *vol, const struct rt_db_internal *ip)
 {
@@ -1951,13 +1927,10 @@ rt_epa_volume(fastf_t *vol, const struct rt_db_internal *ip)
     RT_EPA_CK_MAGIC(xip);
 
     mag_h = MAGNITUDE(xip->epa_H);
-    *vol = 0.5 * M_PI * xip->epa_r1 * xip->epa_r2 * mag_h;
+    *vol = M_PI_2 * xip->epa_r1 * xip->epa_r2 * mag_h;
 }
 
 
-/**
- * R T _ E P A _ C E N T R O I D
- */
 void
 rt_epa_centroid(point_t *cent, const struct rt_db_internal *ip)
 {
@@ -1967,9 +1940,6 @@ rt_epa_centroid(point_t *cent, const struct rt_db_internal *ip)
 }
 
 
-/**
- * R T _ E P A _ S U R F _ A R E A
- */
 void
 rt_epa_surf_area(fastf_t *area, const struct rt_db_internal *ip)
 {
