@@ -1,7 +1,7 @@
 /*                          H A L F . C
  * BRL-CAD
  *
- * Copyright (c) 1985-2013 United States Government as represented by
+ * Copyright (c) 1985-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This library is free software; you can redistribute it and/or
@@ -40,15 +40,15 @@
 #include "common.h"
 
 #include <stddef.h>
-#include <stdio.h>
 #include <math.h>
 #include "bio.h"
 
+#include "bu/cv.h"
 #include "vmath.h"
-#include "rtgeom.h"
+#include "rt/geom.h"
 #include "raytrace.h"
 #include "nmg.h"
-#include "db.h"
+#include "rt/db4.h"
 
 #include "../../librt_private.h"
 
@@ -67,9 +67,6 @@ const struct bu_structparse rt_hlf_parse[] = {
 };
 
 
-/**
- * R T _ H L F _ P R E P
- */
 int
 rt_hlf_prep(struct soltab *stp, struct rt_db_internal *ip, struct rt_i *rtip)
 {
@@ -85,7 +82,7 @@ rt_hlf_prep(struct soltab *stp, struct rt_db_internal *ip, struct rt_i *rtip)
      * and a distance.
      */
     BU_GET(halfp, struct half_specific);
-    stp->st_specific = (genptr_t)halfp;
+    stp->st_specific = (void *)halfp;
 
     VMOVE(halfp->half_eqn, hip->eqn);
     halfp->half_eqn[W] = hip->eqn[W];
@@ -109,9 +106,6 @@ rt_hlf_prep(struct soltab *stp, struct rt_db_internal *ip, struct rt_i *rtip)
 }
 
 
-/**
- * R T _ H L F _ P R I N T
- */
 void
 rt_hlf_print(register const struct soltab *stp)
 {
@@ -130,8 +124,6 @@ rt_hlf_print(register const struct soltab *stp)
 
 
 /**
- * R T _ H L F _ S H O T
- *
  * Function -
  * Shoot a ray at a HALFSPACE
  *
@@ -179,7 +171,7 @@ rt_hlf_shot(struct soltab *stp, register struct xray *rp, struct application *ap
 		return 0;	/* MISS */
 	}
     }
-    if (RT_G_DEBUG & DEBUG_ARB8)
+    if (RT_G_DEBUG & RT_DEBUG_ARB8)
 	bu_log("half: in=%f, out=%f\n", in, out);
 
     {
@@ -200,8 +192,6 @@ rt_hlf_shot(struct soltab *stp, register struct xray *rp, struct application *ap
 #define RT_HALF_SEG_MISS(SEG)	(SEG).seg_stp=RT_SOLTAB_NULL
 
 /**
- * R T _ H L F _ V S H O T
- *
  * This is the Becker vector version
  */
 void
@@ -259,8 +249,6 @@ rt_hlf_vshot(struct soltab **stp, struct xray **rp, struct seg *segp, int n, str
 
 
 /**
- * R T _ H L F _ N O R M
- *
  * Given ONE ray distance, return the normal and entry/exit point.
  * The normal is already filled in.
  */
@@ -294,8 +282,6 @@ rt_hlf_norm(register struct hit *hitp, struct soltab *stp, register struct xray
 
 
 /**
- * R T _ H L F _ C U R V E
- *
  * Return the "curvature" of the halfspace.  Pick a principle
  * direction orthogonal to normal, and indicate no curvature.
  */
@@ -312,8 +298,6 @@ rt_hlf_curve(struct curvature *cvp, struct hit *hitp, struct soltab *stp)
 
 
 /**
- * R T _ H L F _ U V
- *
  * For a hit on a face of an HALF, return the (u, v) coordinates of
  * the hit point.  0 <= u, v <= 1.  u extends along the Xbase
  * direction.  v extends along the "Ybase" direction.  Note that a
@@ -327,7 +311,7 @@ rt_hlf_uv(struct application *ap, struct soltab *stp, register struct hit *hitp,
 
     vect_t P_A;
     fastf_t f;
-    auto double ival;
+    double ival;
 
     f = hitp->hit_dist;
     if (f <= -INFINITY || f >= INFINITY) {
@@ -386,9 +370,6 @@ rt_hlf_uv(struct application *ap, struct soltab *stp, register struct hit *hitp,
 }
 
 
-/**
- * R T _ H L F _ F R E E
- */
 void
 rt_hlf_free(struct soltab *stp)
 {
@@ -400,27 +381,6 @@ rt_hlf_free(struct soltab *stp)
 
 
 /**
- * R T _ H L F _ C L A S S
- *
- * Classify this halfspace against a bounding RPP.  Since this is an
- * infinite solid, it is very important that this function properly.
- *
- */
-int
-rt_hlf_class(const struct soltab *stp, const fastf_t *min, const fastf_t *max, const struct bn_tol *tol)
-{
-    if (stp) RT_CK_SOLTAB(stp);
-    if (tol) BN_CK_TOL(tol);
-    if (!min) return 0;
-    if (!max) return 0;
-
-    return 0;
-}
-
-
-/**
- * R T _ H L F _ P L O T
- *
  * The representation of a halfspace is an OUTWARD pointing normal
  * vector, and the distance of the plane from the origin.
  *
@@ -429,7 +389,7 @@ rt_hlf_class(const struct soltab *stp, const fastf_t *min, const fastf_t *max, c
  * the plane, with the outward normal drawn shorter.
  */
 int
-rt_hlf_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct rt_tess_tol *UNUSED(ttol), const struct bn_tol *UNUSED(tol), const struct rt_view_info *UNUSED(info))
+rt_hlf_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct bg_tess_tol *UNUSED(ttol), const struct bn_tol *UNUSED(tol), const struct rt_view_info *UNUSED(info))
 {
     struct rt_half_internal *hip;
     vect_t cent;		/* some point on the plane */
@@ -479,8 +439,6 @@ rt_hlf_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct rt_te
 
 
 /**
- * H A L F _ X F O R M
- *
  * Returns -
  * -1 failure
  * 0 success
@@ -491,8 +449,7 @@ rt_hlf_xform(
     const mat_t mat,
     struct rt_db_internal *ip,
     int release,
-    struct db_i *dbip,
-    struct resource *resp)
+    struct db_i *dbip)
 {
     struct rt_half_internal *hip, *hop;
     point_t orig_pt, pt;
@@ -501,7 +458,6 @@ rt_hlf_xform(
     if (dbip) RT_CK_DBI(dbip);
 
     RT_CK_DB_INTERNAL(ip);
-    RT_CK_RESOURCE(resp);
     hip = (struct rt_half_internal *)ip->idb_ptr;
     RT_HALF_CK_MAGIC(hip);
     RT_CK_DB_INTERNAL(op);
@@ -510,7 +466,7 @@ rt_hlf_xform(
 	RT_DB_INTERNAL_INIT(op);
 	BU_ALLOC(hop, struct rt_half_internal);
 	hop->magic = RT_HALF_INTERNAL_MAGIC;
-	op->idb_ptr = (genptr_t)hop;
+	op->idb_ptr = (void *)hop;
 	op->idb_meth = &OBJ[ID_HALF];
 	op->idb_major_type = DB5_MAJORTYPE_BRLCAD;
 	op->idb_type = ID_HALF;
@@ -560,8 +516,6 @@ rt_hlf_xform(
 
 
 /**
- * H A L F _ I M P O R T
- *
  * Returns -
  * -1 failure
  * 0 success
@@ -626,9 +580,6 @@ rt_hlf_import4(struct rt_db_internal *ip, const struct bu_external *ep, const fa
 }
 
 
-/**
- * R T _ H L F _ E X P O R T
- */
 int
 rt_hlf_export4(struct bu_external *ep, const struct rt_db_internal *ip, double local2mm, const struct db_i *dbip)
 {
@@ -644,7 +595,7 @@ rt_hlf_export4(struct bu_external *ep, const struct rt_db_internal *ip, double l
 
     BU_CK_EXTERNAL(ep);
     ep->ext_nbytes = sizeof(union record);
-    ep->ext_buf = (genptr_t)bu_calloc(1, ep->ext_nbytes, "half external");
+    ep->ext_buf = (uint8_t *)bu_calloc(1, ep->ext_nbytes, "half external");
     rec = (union record *)ep->ext_buf;
 
     rec->s.s_id = ID_SOLID;
@@ -656,9 +607,6 @@ rt_hlf_export4(struct bu_external *ep, const struct rt_db_internal *ip, double l
 }
 
 
-/**
- * R T _ H L F _ I M P O R T 5
- */
 int
 rt_hlf_import5(struct rt_db_internal *ip, const struct bu_external *ep, register const fastf_t *mat, const struct db_i *dbip)
 {
@@ -673,7 +621,7 @@ rt_hlf_import5(struct rt_db_internal *ip, const struct bu_external *ep, register
 
     BU_CK_EXTERNAL(ep);
 
-    BU_ASSERT_LONG(ep->ext_nbytes, ==, SIZEOF_NETWORK_DOUBLE * ELEMENTS_PER_PLANE);
+    BU_ASSERT(ep->ext_nbytes == SIZEOF_NETWORK_DOUBLE * ELEMENTS_PER_PLANE);
 
     RT_CK_DB_INTERNAL(ip);
     ip->idb_major_type = DB5_MAJORTYPE_BRLCAD;
@@ -718,9 +666,6 @@ rt_hlf_import5(struct rt_db_internal *ip, const struct bu_external *ep, register
 }
 
 
-/**
- * R T _ H A L F _ E X P O R T 5
- */
 int
 rt_hlf_export5(struct bu_external *ep, const struct rt_db_internal *ip, double local2mm, const struct db_i *dbip)
 {
@@ -739,7 +684,7 @@ rt_hlf_export5(struct bu_external *ep, const struct rt_db_internal *ip, double l
 
     BU_CK_EXTERNAL(ep);
     ep->ext_nbytes = SIZEOF_NETWORK_DOUBLE * 4;
-    ep->ext_buf = (genptr_t)bu_malloc(ep->ext_nbytes, "half external");
+    ep->ext_buf = (uint8_t *)bu_malloc(ep->ext_nbytes, "half external");
 
     /* only the distance needs to be scaled */
     scaled_dist = hip->eqn[W] * local2mm;
@@ -759,8 +704,6 @@ rt_hlf_export5(struct bu_external *ep, const struct rt_db_internal *ip, double l
 
 
 /**
- * R T _ H L F _ D E S C R I B E
- *
  * Make human-readable formatted presentation of this solid.  First
  * line describes type of solid.  Additional lines are indented one
  * tab, and give parameter values.
@@ -788,8 +731,6 @@ rt_hlf_describe(struct bu_vls *str, const struct rt_db_internal *ip, int verbose
 
 
 /**
- * R T _ H L F _ I F R E E
- *
  * Free the storage associated with the rt_db_internal version of this
  * solid.
  */
@@ -799,15 +740,12 @@ rt_hlf_ifree(struct rt_db_internal *ip)
     RT_CK_DB_INTERNAL(ip);
 
     bu_free(ip->idb_ptr, "hlf ifree");
-    ip->idb_ptr = GENPTR_NULL;
+    ip->idb_ptr = ((void *)0);
 }
 
 
-/**
- * R T _ H L F _ T E S S
- */
 int
-rt_hlf_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, const struct rt_tess_tol *UNUSED(ttol), const struct bn_tol *UNUSED(tol))
+rt_hlf_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, const struct bg_tess_tol *UNUSED(ttol), const struct bn_tol *UNUSED(tol))
 {
     struct rt_half_internal *vip;
 
@@ -823,10 +761,6 @@ rt_hlf_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
 }
 
 
-/**
- * R T _ H L F _ P A R A M S
- *
- */
 int
 rt_hlf_params(struct pc_pc_set *UNUSED(ps), const struct rt_db_internal *ip)
 {
