@@ -1,7 +1,7 @@
 /*                           H Y P . C
  * BRL-CAD
  *
- * Copyright (c) 1990-2013 United States Government as represented by
+ * Copyright (c) 1990-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This library is free software; you can redistribute it and/or
@@ -38,11 +38,12 @@
 #include <math.h>
 
 /* interface headers */
+#include "bu/cv.h"
 #include "vmath.h"
-#include "db.h"
+#include "rt/db4.h"
 #include "nmg.h"
 #include "raytrace.h"
-#include "rtgeom.h"
+#include "rt/geom.h"
 
 #include "../../librt_private.h"
 
@@ -114,8 +115,6 @@ const struct bu_structparse rt_hyp_parse[] = {
 };
 
 /**
- * R T _ H Y P _ B B O X
- *
  * Create a bounding RPP for an hyp
  */
 int
@@ -168,8 +167,6 @@ rt_hyp_bbox(struct rt_db_internal *ip, point_t *min, point_t *max, const struct
 }
 
 /**
- * R T _ H Y P _ P R E P
- *
  * Given a pointer to a GED database record, and a transformation
  * matrix, determine if this is a valid HYP, and if so, precompute
  * various terms of the formula.
@@ -200,7 +197,7 @@ rt_hyp_prep(struct soltab *stp, struct rt_db_internal *ip, struct rt_i *rtip)
     stp->st_meth = &OBJ[ID_HYP];
 
     hyp =  hyp_internal_to_specific(hyp_ip);
-    stp->st_specific = (genptr_t)hyp;
+    stp->st_specific = (void *)hyp;
 
     /* calculate bounding sphere */
     VMOVE(stp->st_center, hyp->hyp_V);
@@ -214,9 +211,6 @@ rt_hyp_prep(struct soltab *stp, struct rt_db_internal *ip, struct rt_i *rtip)
 }
 
 
-/**
- * R T _ H Y P _ P R I N T
- */
 void
 rt_hyp_print(const struct soltab *stp)
 {
@@ -238,8 +232,6 @@ rt_hyp_print(const struct soltab *stp)
 
 
 /**
- * R T _ H Y P _ S H O T
- *
  * Intersect a ray with a hyp.  If an intersection occurs, a struct
  * seg will be acquired and filled in.
  *
@@ -414,8 +406,6 @@ rt_hyp_shot(struct soltab *stp, struct xray *rp, struct application *ap, struct
 
 
 /**
- * R T _ H Y P _ N O R M
- *
  * Given ONE ray distance, return the normal and entry/exit point.
  */
 void
@@ -463,8 +453,6 @@ rt_hyp_norm(struct hit *hitp, struct soltab *stp, struct xray *rp)
 
 
 /**
- * R T _ H Y P _ C U R V E
- *
  * Return the curvature of the hyp.
  */
 void
@@ -537,8 +525,6 @@ rt_hyp_curve(struct curvature *cvp, struct hit *hitp, struct soltab *stp)
 
 
 /**
- * R T _ H Y P _ U V
- *
  * For a hit on the surface of an hyp, return the (u, v) coordinates
  * of the hit point, 0 <= u, v <= 1.
  *
@@ -553,11 +539,11 @@ rt_hyp_uv(struct application *ap, struct soltab *stp, struct hit *hitp, struct u
     if (ap) RT_CK_APPLICATION(ap);
 
     /* u = (angle from semi-major axis on basic hyperboloid) / (2*pi) */
-    uvp->uv_u = M_1_PI * 0.5
+    uvp->uv_u = M_1_2PI
 	* (atan2(-hitp->hit_vpriv[X] * hyp->hyp_r2, hitp->hit_vpriv[Y] * hyp->hyp_r1) + M_PI);
 
     /* v ranges (0, 1) on each plate */
-    switch(hitp->hit_surfno) {
+    switch (hitp->hit_surfno) {
 	case HYP_NORM_BODY:
 	    /* v = (z + Hmag) / (2*Hmag) */
 	    uvp->uv_v = (hitp->hit_vpriv[Z] + hyp->hyp_Hmag) / (2.0 * hyp->hyp_Hmag);
@@ -591,9 +577,6 @@ rt_hyp_uv(struct application *ap, struct soltab *stp, struct hit *hitp, struct u
 }
 
 
-/**
- * R T _ H Y P _ F R E E
- */
 void
 rt_hyp_free(struct soltab *stp)
 {
@@ -604,11 +587,8 @@ rt_hyp_free(struct soltab *stp)
 }
 
 
-/**
- * R T _ H Y P _ P L O T
- */
 int
-rt_hyp_plot(struct bu_list *vhead, struct rt_db_internal *incoming, const struct rt_tess_tol *UNUSED(ttol), const struct bn_tol *UNUSED(tol), const struct rt_view_info *UNUSED(info))
+rt_hyp_plot(struct bu_list *vhead, struct rt_db_internal *incoming, const struct bg_tess_tol *UNUSED(ttol), const struct bn_tol *UNUSED(tol), const struct rt_view_info *UNUSED(info))
 {
     int i, j;		/* loop indices */
     struct rt_hyp_internal *hyp_in;
@@ -684,20 +664,20 @@ rt_hyp_plot(struct bu_list *vhead, struct rt_db_internal *incoming, const struct
 
 	/* draw ellipse */
 	RT_ADD_VLIST(vhead, ell[15], BN_VLIST_LINE_MOVE);
-	for (j=0; j<16; j++) {
+	for (j = 0; j < 16; j++) {
 	    RT_ADD_VLIST(vhead, ell[j], BN_VLIST_LINE_DRAW);
 	}
 
 	/* add ellipse's points to ribs */
-	for (j=0; j<16; j++) {
+	for (j = 0; j < 16; j++) {
 	    VMOVE(ribs[j][i], ell[j]);
 	}
     }
 
     /* draw ribs */
-    for (i=0; i<16; i++) {
+    for (i = 0; i < 16; i++) {
 	RT_ADD_VLIST(vhead, ribs[i][0], BN_VLIST_LINE_MOVE);
-	for (j=1; j<7; j++) {
+	for (j = 1; j < 7; j++) {
 	    RT_ADD_VLIST(vhead, ribs[i][j], BN_VLIST_LINE_DRAW);
 	}
 
@@ -710,25 +690,25 @@ rt_hyp_plot(struct bu_list *vhead, struct rt_db_internal *incoming, const struct
 
 
 /**
- * R T _ H Y P _ T E S S
- *
  * Returns -
  * -1 failure
  * 0 OK.  *r points to nmgregion that holds this tessellation.
  */
 int
-rt_hyp_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, const struct rt_tess_tol *ttol, const struct bn_tol *tol)
+rt_hyp_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, const struct bg_tess_tol *ttol, const struct bn_tol *tol)
 {
     fastf_t c, dtol, f, mag_a, mag_h, ntol, r1, r2, r3, cprime;
     fastf_t **ellipses = NULL;
     fastf_t theta_new;
-    int *pts_dbl, face, i, j, nseg;
-    int jj, nell;
-    mat_t invRoS;
-    mat_t SoR;
+    int *pts_dbl;
+    int idx;
+    size_t face, i, j, nseg;
+    size_t jj, nell;
+    mat_t invRoS = MAT_INIT_IDN;
+    mat_t SoR = MAT_INIT_IDN;
     struct rt_hyp_internal *iip;
     struct hyp_specific *xip;
-    struct rt_pt_node *pos_a, *pos_b, *pts_a, *pts_b;
+    struct rt_pnt_node *pos_a, *pos_b, *pts_a, *pts_b;
     struct shell *s;
     struct faceuse **outfaceuses = NULL;
     struct faceuse *fu_top;
@@ -739,9 +719,6 @@ rt_hyp_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
     vect_t A, Au, B, Bu, Hu, V;
     struct bu_ptbl vert_tab;
 
-    MAT_ZERO(invRoS);
-    MAT_ZERO(SoR);
-
     RT_CK_DB_INTERNAL(ip);
     iip = (struct rt_hyp_internal *)ip->idb_ptr;
     RT_HYP_CK_MAGIC(iip);
@@ -786,22 +763,22 @@ rt_hyp_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
 	ntol = ttol->norm;
     else
 	/* tolerate everything */
-	ntol = bn_pi;
+	ntol = M_PI;
 
     /*
      * build hyp from 2 hyperbolas
      */
 
     /* calculate major axis hyperbola */
-    BU_ALLOC(pts_a, struct rt_pt_node);
+    BU_ALLOC(pts_a, struct rt_pnt_node);
 
     /* set base, center, and top points */
     pos_a = pts_a;
     VSET(pos_a->p, sqrt((mag_h*mag_h) * (c*c) + (r1*r1)), 0, -mag_h);
-    BU_ALLOC(pos_a->next, struct rt_pt_node);
+    BU_ALLOC(pos_a->next, struct rt_pnt_node);
     pos_a = pos_a->next;
     VSET(pos_a->p, r1, 0, 0);
-    BU_ALLOC(pos_a->next, struct rt_pt_node);
+    BU_ALLOC(pos_a->next, struct rt_pnt_node);
     pos_a = pos_a->next;
     VSET(pos_a->p, sqrt((mag_h*mag_h) * (c*c) + (r1*r1)), 0, mag_h);
     pos_a->next = NULL;
@@ -813,7 +790,7 @@ rt_hyp_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
 	fastf_t mm, len, dist, ang0, ang2;
 	vect_t v01, v02; /* vectors from p0->p1 and p0->p2 */
 	vect_t nLine, nHyp;
-	struct rt_pt_node *add;
+	struct rt_pnt_node *add;
 
 	while (i) {
 	    pos_a = pts_a;
@@ -853,7 +830,7 @@ rt_hyp_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
 
 		if (dist > dtol || ang0 > ntol || ang2 > ntol) {
 		    /* split segment */
-		    BU_ALLOC(add, struct rt_pt_node);
+		    BU_ALLOC(add, struct rt_pnt_node);
 		    VMOVE(add->p, p1);
 		    add->next = pos_a->next;
 		    pos_a->next = add;
@@ -867,7 +844,7 @@ rt_hyp_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
     }
 
     /* calculate minor axis hyperbola */
-    BU_ALLOC(pts_b, struct rt_pt_node);
+    BU_ALLOC(pts_b, struct rt_pnt_node);
 
     pos_a = pts_a;
     pos_b = pts_b;
@@ -878,7 +855,7 @@ rt_hyp_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
 	pos_b->p[Y] = r2 * sqrt(pos_b->p[Z] * pos_b->p[Z]/(r3*r3) + 1.0);
 	pos_a = pos_a->next;
 	if (pos_a) {
-	    BU_ALLOC(pos_b->next, struct rt_pt_node);
+	    BU_ALLOC(pos_b->next, struct rt_pnt_node);
 	    pos_b = pos_b->next;
 	} else {
 	    pos_b->next = NULL;
@@ -909,7 +886,7 @@ rt_hyp_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
 	VSET(p1, 0., pos_b->p[Y], 0.);
 	theta_new = ell_angle(p1, pos_a->p[X], pos_b->p[Y], dtol, ntol);
 	if (nseg == 0) {
-	    nseg = (int)(bn_twopi / theta_new) + 1;
+	    nseg = (int)(M_2PI / theta_new) + 1;
 	    pts_dbl[i] = 0;
 	    /* maximum number of faces needed for hyp */
 	    face = 2*nseg*nell - 4;	/*nseg*(1 + 3*((1 << (nell-1)) - 1));*/
@@ -939,18 +916,17 @@ rt_hyp_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
     vells = (struct vertex ***)
 	bu_malloc(nell*sizeof(struct vertex **), "vertex [][]");
     j = nseg;
-    for (i = nell-1; i >= 0; i--) {
-	vells[i] = (struct vertex **)
-	    bu_malloc(j*sizeof(struct vertex *), "vertex []");
+    for (i = 0; i < nell; i++) {
+	vells[i] = (struct vertex **)bu_malloc(j*sizeof(struct vertex *), "vertex []");
 	if (i && pts_dbl[i])
-	    j /= 2;
+	    j /=2;
     }
 
     /* top face of hyp */
     for (i = 0; i < nseg; i++)
 	vells[nell-1][i] = (struct vertex *)NULL;
     face = 0;
-    BU_ASSERT_PTR(outfaceuses, !=, NULL);
+    BU_ASSERT(outfaceuses != NULL);
     if ((outfaceuses[face++] = nmg_cface(s, vells[nell-1], nseg)) == 0) {
 	bu_log("rt_hyp_tess() failure, top face\n");
 	goto fail;
@@ -980,12 +956,12 @@ rt_hyp_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
 
     /* connect ellipses with triangles */
 
-    for (i = nell-2; i >= 0; i--) {
+    for (idx = nell-2; idx >= 0; idx--) {
 	/* skip top ellipse */
 	int bottom, top;
 
-	top = i + 1;
-	bottom = i;
+	top = idx + 1;
+	bottom = idx;
 	if (pts_dbl[top])
 	    nseg /= 2;	/* # segs in 'bottom' ellipse */
 	vertp[0] = (struct vertex *)0;
@@ -1069,7 +1045,7 @@ rt_hyp_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
     for (i = 0; i < nseg; i++)
 	vells[0][i] = (struct vertex *)NULL;
 
-    BU_ASSERT_PTR(outfaceuses, !=, NULL);
+    BU_ASSERT(outfaceuses != NULL);
     if ((outfaceuses[face++] = nmg_cface(s, vells[0], nseg)) == 0) {
 	bu_log("rt_hyp_tess() failure, top face\n");
 	goto fail;
@@ -1098,9 +1074,9 @@ rt_hyp_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
     }
 
     /* Associate the face geometry */
-    for (i=0; i < face; i++) {
+    for (i = 0; i < face; i++) {
 	if (nmg_fu_planeeqn(outfaceuses[i], tol) < 0) {
-	    bu_log("planeeqn fail:\n\ti:\t%d\n\toutfaceuses:\n\tmin:\t%f\t%f\t%f\n\tmax:\t%f\t%f\t%f\n",
+	    bu_log("planeeqn fail:\n\ti:\t%zu\n\toutfaceuses:\n\tmin:\t%f\t%f\t%f\n\tmax:\t%f\t%f\t%f\n",
 		   i, outfaceuses[i]->f_p->min_pt[0],
 		   outfaceuses[i]->f_p->min_pt[1],
 		   outfaceuses[i]->f_p->min_pt[2],
@@ -1112,13 +1088,13 @@ rt_hyp_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
     }
 
     /* Glue the edges of different outward pointing face uses together */
-    nmg_gluefaces(outfaceuses, face, tol);
+    nmg_gluefaces(outfaceuses, face, &RTG.rtg_vlfree, tol);
 
     /* Compute "geometry" for region and shell */
     nmg_region_a(*r, tol);
 
     /* XXX just for testing, to make up for loads of triangles ... */
-    nmg_shell_coplanar_face_merge(s, tol, 1);
+    nmg_shell_coplanar_face_merge(s, tol, 1, &RTG.rtg_vlfree);
 
     /* free mem */
     if (outfaceuses)
@@ -1135,8 +1111,8 @@ rt_hyp_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
 	bu_free((char *)vells, "vertex [][]");
 
     /* Assign vertexuse normals */
-    nmg_vertex_tabulate(&vert_tab, &s->l.magic);
-    for (i=0; i<BU_PTBL_END(&vert_tab); i++) {
+    nmg_vertex_tabulate(&vert_tab, &s->l.magic, &RTG.rtg_vlfree);
+    for (i = 0; i < BU_PTBL_LEN(&vert_tab); i++) {
 	point_t pt_prime, tmp_pt;
 	vect_t norm, rev_norm, tmp_vect;
 	struct vertex_g *vg;
@@ -1199,8 +1175,6 @@ rt_hyp_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
 
 
 /**
- * R T _ H Y P _ I M P O R T 5
- *
  * Import an HYP from the database format to the internal format.
  * Note that the data read will be in network order.  This means
  * Big-Endian integers and IEEE doubles for floating point.
@@ -1219,7 +1193,7 @@ rt_hyp_import5(struct rt_db_internal *ip, const struct bu_external *ep, const ma
     BU_CK_EXTERNAL(ep);
     if (dbip) RT_CK_DBI(dbip);
 
-    BU_ASSERT_LONG(ep->ext_nbytes, ==, SIZEOF_NETWORK_DOUBLE * 3 * 4);
+    BU_ASSERT(ep->ext_nbytes == SIZEOF_NETWORK_DOUBLE * 3 * 4);
 
     /* set up the internal structure */
     ip->idb_major_type = DB5_MAJORTYPE_BRLCAD;
@@ -1253,8 +1227,6 @@ rt_hyp_import5(struct rt_db_internal *ip, const struct bu_external *ep, const ma
 
 
 /**
- * R T _ H Y P _ E X P O R T 5
- *
  * Export an HYP from internal form to external format.  Note that
  * this means converting all integers to Big-Endian format and
  * floating point data to IEEE double.
@@ -1278,7 +1250,7 @@ rt_hyp_export5(struct bu_external *ep, const struct rt_db_internal *ip, double l
 
     BU_CK_EXTERNAL(ep);
     ep->ext_nbytes = SIZEOF_NETWORK_DOUBLE * ELEMENTS_PER_VECT * 4;
-    ep->ext_buf = (genptr_t)bu_calloc(1, ep->ext_nbytes, "hyp external");
+    ep->ext_buf = (uint8_t *)bu_calloc(1, ep->ext_nbytes, "hyp external");
 
 
     /* Since libwdb users may want to operate in units other than mm,
@@ -1299,8 +1271,6 @@ rt_hyp_export5(struct bu_external *ep, const struct rt_db_internal *ip, double l
 
 
 /**
- * R T _ H Y P _ D E S C R I B E
- *
  * Make human-readable formatted presentation of this solid.  First
  * line describes type of solid.  Additional lines are indented one
  * tab, and give parameter values.
@@ -1348,8 +1318,6 @@ rt_hyp_describe(struct bu_vls *str, const struct rt_db_internal *ip, int verbose
 
 
 /**
- * R T _ H Y P _ I F R E E
- *
  * Free the storage associated with the rt_db_internal version of this
  * solid.
  */
@@ -1365,13 +1333,10 @@ rt_hyp_ifree(struct rt_db_internal *ip)
     hyp_ip->hyp_magic = 0;			/* sanity */
 
     bu_free((char *)hyp_ip, "hyp ifree");
-    ip->idb_ptr = GENPTR_NULL;	/* sanity */
+    ip->idb_ptr = ((void *)0);	/* sanity */
 }
 
 
-/**
- * R T _ H Y P _ P A R A M S
- */
 int
 rt_hyp_params(struct pc_pc_set * UNUSED(ps), const struct rt_db_internal *ip)
 {
@@ -1381,9 +1346,6 @@ rt_hyp_params(struct pc_pc_set * UNUSED(ps), const struct rt_db_internal *ip)
 }
 
 
-/**
- * R T _ H Y P _ C E N T R O I D
- */
 void
 rt_hyp_centroid(point_t *cent, const struct rt_db_internal *ip)
 {
@@ -1401,8 +1363,6 @@ rt_hyp_centroid(point_t *cent, const struct rt_db_internal *ip)
 
 
 /**
- * R T _ H Y P _ S U R F _ A R E A
- *
  * only the stub to make analyze happy
  * TODO: needs an implementation
  */
@@ -1410,9 +1370,6 @@ void
 rt_hyp_surf_area(fastf_t *UNUSED(area), const struct rt_db_internal *UNUSED(ip)) {}
 
 
-/**
- * R T _ H Y P _ V O L U M E
- */
 void
 rt_hyp_volume(fastf_t *volume, const struct rt_db_internal *ip)
 {
@@ -1425,7 +1382,7 @@ rt_hyp_volume(fastf_t *volume, const struct rt_db_internal *ip)
 	RT_HYP_CK_MAGIC(hip);
 
 	hyp = hyp_internal_to_specific(hip);
-	*volume = M_PI * hyp->hyp_r1 * hyp->hyp_r2 * hyp->hyp_Hmag * 2 *
+	*volume = M_2PI * hyp->hyp_r1 * hyp->hyp_r2 * hyp->hyp_Hmag *
 	    (1 + hyp->hyp_Hmag * hyp->hyp_Hmag * hyp->hyp_c * hyp->hyp_c / (12 * hyp->hyp_r1 * hyp->hyp_r1));
 	bu_free(hyp, "hyp volume");
     }
