@@ -1,7 +1,7 @@
 /*                   O B J _ X F O R M . C
  * BRL-CAD
  *
- * Copyright (c) 2010-2013 United States Government as represented by
+ * Copyright (c) 2010-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This library is free software; you can redistribute it and/or
@@ -20,11 +20,10 @@
 
 #include "common.h"
 
-#include "raytrace.h"
-
+#include "rt/func.h"
 
 int
-rt_obj_xform(struct rt_db_internal *op, const mat_t mat, struct rt_db_internal *ip, int release, struct db_i *dbip, struct resource *resp)
+rt_obj_xform(struct rt_db_internal *op, const mat_t mat, struct rt_db_internal *ip, int release, struct db_i *dbip)
 {
     int id;
     const struct rt_functab *ft;
@@ -35,7 +34,6 @@ rt_obj_xform(struct rt_db_internal *op, const mat_t mat, struct rt_db_internal *
     RT_CK_DB_INTERNAL(op);
     RT_CK_DB_INTERNAL(ip);
     if (dbip) RT_CK_DBI(dbip);
-    if (resp) RT_CK_RESOURCE(resp);
 
     id = ip->idb_minor_type;
     if (id < 0)
@@ -47,7 +45,7 @@ rt_obj_xform(struct rt_db_internal *op, const mat_t mat, struct rt_db_internal *
     if (!ft->ft_xform)
 	return -4;
 
-    return ft->ft_xform(op, mat, ip, release, dbip, resp);
+    return ft->ft_xform(op, mat, ip, release, dbip);
 }
 
 
