@@ -1,7 +1,7 @@
 /*                           R E V O L V E . C
  * BRL-CAD
  *
- * Copyright (c) 1990-2013 United States Government as represented by
+ * Copyright (c) 1990-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This library is free software; you can redistribute it and/or
@@ -33,9 +33,11 @@
 #include <math.h>
 
 #include "vmath.h"
-#include "db.h"
+#include "bu/cv.h"
+#include "bu/debug.h"
+#include "rt/db4.h"
 #include "nmg.h"
-#include "rtgeom.h"
+#include "rt/geom.h"
 #include "raytrace.h"
 
 /* local interface header */
@@ -71,7 +73,7 @@ rt_revolve_make(const struct rt_functab *ftp, struct rt_db_internal *intern)
     intern->idb_meth = ftp;
     BU_ALLOC(rev, struct rt_revolve_internal);
 
-    intern->idb_ptr = (genptr_t)rev;
+    intern->idb_ptr = (void *)rev;
     rev->magic = RT_REVOLVE_INTERNAL_MAGIC;
 
     BU_VLS_INIT(&rev->sketch_name);
@@ -80,8 +82,6 @@ rt_revolve_make(const struct rt_functab *ftp, struct rt_db_internal *intern)
 
 
 /**
- * R T _ R E V O L V E _ B B O X
- *
  * Calculate a bounding RPP around a sketch
  */
 int
@@ -192,8 +192,6 @@ rt_revolve_bbox(struct rt_db_internal *ip, point_t *min, point_t *max, const str
 }
 
 /**
- * R T _ R E V O L V E _ P R E P
- *
  * Given a pointer to a GED database record, and a transformation
  * matrix, determine if this is a valid REVOLVE, and if so, precompute
  * various terms of the formula.
@@ -232,7 +230,7 @@ rt_revolve_prep(struct soltab *stp, struct rt_db_internal *ip, struct rt_i *rtip
     stp->st_meth = &OBJ[ID_REVOLVE];
 
     BU_GET(rev, struct revolve_specific);
-    stp->st_specific = (genptr_t)rev;
+    stp->st_specific = (void *)rev;
 
     VMOVE(rev->v3d, rip->v3d);
     VMOVE(rev->zUnit, rip->axis3d);
@@ -341,9 +339,6 @@ rt_revolve_prep(struct soltab *stp, struct rt_db_internal *ip, struct rt_i *rtip
 }
 
 
-/**
- * R T _ R E V O L V E _ P R I N T
- */
 void
 rt_revolve_print(const struct soltab *stp)
 {
@@ -360,8 +355,6 @@ rt_revolve_print(const struct soltab *stp)
 
 
 /**
- * R T _ R E V O L V E _ S H O T
- *
  * Intersect a ray with a revolve.  If an intersection occurs, a struct
  * seg will be acquired and filled in.
  *
@@ -412,7 +405,7 @@ rt_revolve_shot(struct soltab *stp, struct xray *rp, struct application *ap, str
     VMOVE(ur, vr);
     VUNITIZE(ur);
 
-    if (rev->ang < 2*M_PI) {
+    if (rev->ang < M_2PI) {
 	VREVERSE(normS, rev->yUnit);	/* start normal */
 	start = (VDOT(normS, rev->v3d) - VDOT(normS, rp->r_pt)) / VDOT(normS, rp->r_dir);
 
@@ -530,7 +523,7 @@ rt_revolve_shot(struct soltab *stp, struct xray *rp, struct application *ap, str
 		if (pt1[X] < 0) {
 		    angle += M_PI;
 		} else if (angle < 0) {
-		    angle += 2*M_PI;
+		    angle += M_2PI;
 		}
 		hit2d[X] = -hit2d[X];
 		if (angle > rev->ang) {
@@ -587,7 +580,7 @@ rt_revolve_shot(struct soltab *stp, struct xray *rp, struct application *ap, str
 			hit2d[X] = -aa;		/* use neg to check for overlap in contains() */
 			hit2d[Y] = hit1[Z];
 			if (angle < 0) {
-			    angle += 2*M_PI;
+			    angle += M_2PI;
 			}
 			if (angle < rev->ang &&
 			    !((angle + M_PI < rev->ang || angle - M_PI > 0)
@@ -655,7 +648,7 @@ rt_revolve_shot(struct soltab *stp, struct xray *rp, struct application *ap, str
 			    if (hit2d[X] < 0) {
 				angle += M_PI;
 			    } else if (angle < 0) {
-				angle += 2*M_PI;
+				angle += M_2PI;
 			    }
 			    hit2d[X] = -hit2d[X];
 			    if (angle < rev->ang &&
@@ -682,7 +675,7 @@ rt_revolve_shot(struct soltab *stp, struct xray *rp, struct application *ap, str
 			    if (-hit2d[X] < 0) {
 				angle += M_PI;
 			    } else if (angle < 0) {
-				angle += 2*M_PI;
+				angle += M_2PI;
 			    }
 			    if (angle < rev->ang &&
 				!((angle + M_PI < rev->ang || angle - M_PI > 0)
@@ -726,7 +719,7 @@ rt_revolve_shot(struct soltab *stp, struct xray *rp, struct application *ap, str
 				if (hit2d[X] < 0) {
 				    angle += M_PI;
 				} else if (angle < 0) {
-				    angle += 2*M_PI;
+				    angle += M_2PI;
 				}
 				hit2d[X] = -hit2d[X];
 				if (angle < rev->ang) {
@@ -758,7 +751,7 @@ rt_revolve_shot(struct soltab *stp, struct xray *rp, struct application *ap, str
 				if (hit2d[X] < 0) {
 				    angle += M_PI;
 				} else if (angle < 0) {
-				    angle += 2*M_PI;
+				    angle += M_2PI;
 				}
 				hit2d[X] = -hit2d[X];
 				if (angle < rev->ang) {
@@ -795,7 +788,7 @@ rt_revolve_shot(struct soltab *stp, struct xray *rp, struct application *ap, str
 			    if (hit2d[X] < 0) {
 				angle += M_PI;
 			    } else if (angle < 0) {
-				angle += 2*M_PI;
+				angle += M_2PI;
 			    }
 			    hit2d[X] = -hit2d[X];
 			    if (angle < rev->ang) {
@@ -811,7 +804,7 @@ rt_revolve_shot(struct soltab *stp, struct xray *rp, struct application *ap, str
 				    if (ZERO(m)) {
 					hitp->hit_vpriv[Z] = 0.0;
 				    } else {
-					hitp->hit_vpriv[Z] = (hit2d[X]>0) ? 1.0/m : -1.0/m;;
+					hitp->hit_vpriv[Z] = (hit2d[X]>0) ? 1.0/m : -1.0/m;
 				    }
 				    hitp->hit_magic = RT_HIT_MAGIC;
 				    hitp->hit_dist = k1;
@@ -1012,8 +1005,6 @@ rt_revolve_shot(struct soltab *stp, struct xray *rp, struct application *ap, str
 
 
 /**
- * R T _ R E V O L V E _ N O R M
- *
  * Given ONE ray distance, return the normal and entry/exit point.
  */
 void
@@ -1085,8 +1076,6 @@ rt_revolve_norm(struct hit *hitp, struct soltab *stp, struct xray *rp)
 
 
 /**
- * R T _ R E V O L V E _ C U R V E
- *
  * Return the curvature of the revolve.
  */
 void
@@ -1105,8 +1094,6 @@ rt_revolve_curve(struct curvature *cvp, struct hit *hitp, struct soltab *stp)
 
 
 /**
- * R T _ R E V O L V E _ U V
- *
  * For a hit on the surface of an revolve, return the (u, v) coordinates
  * of the hit point, 0 <= u, v <= 1.
 
@@ -1153,7 +1140,7 @@ rt_revolve_uv(struct application *ap, struct soltab *stp, struct hit *hitp, stru
 	    break;
 	default:
 	    angle = atan2(hitp->hit_vpriv[Y], hitp->hit_vpriv[X]);
-	    if (angle < 0) angle += 2*M_PI;
+	    if (angle < 0) angle += M_2PI;
 	    uvp->uv_u = angle / rev->ang;
 
 	    lng = (uint32_t *)rev->skt->curve.segment[hitp->hit_surfno];
@@ -1204,9 +1191,6 @@ rt_revolve_uv(struct application *ap, struct soltab *stp, struct hit *hitp, stru
 }
 
 
-/**
- * R T _ R E V O L V E _ F R E E
- */
 void
 rt_revolve_free(struct soltab *stp)
 {
@@ -1217,21 +1201,8 @@ rt_revolve_free(struct soltab *stp)
 }
 
 
-/**
- * R T _ R E V O L V E _ C L A S S
- */
 int
-rt_revolve_class()
-{
-    return 0;
-}
-
-
-/**
- * R T _ R E V O L V E _ P L O T
- */
-int
-rt_revolve_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct rt_tess_tol *ttol, const struct bn_tol *UNUSED(tol), const struct rt_view_info *UNUSED(info))
+rt_revolve_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct bg_tess_tol *ttol, const struct bn_tol *UNUSED(tol), const struct rt_view_info *UNUSED(info))
 {
     struct rt_revolve_internal *rip;
 
@@ -1242,7 +1213,7 @@ rt_revolve_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct r
     vect_t ell[16], cir[16], ucir[16], height, xdir, ydir, ux, uy, uz, rEnd, xEnd, yEnd;
     fastf_t cos22_5 = 0.9238795325112867385;
     fastf_t cos67_5 = 0.3826834323650898373;
-    int *endcount = NULL;
+    int *endcount = NULL, ang_sign;
     point_t add, add2, add3;
 
     BU_CK_LIST_HEAD(vhead);
@@ -1254,8 +1225,8 @@ rt_revolve_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct r
     verts = rip->skt->verts;
     crv = &rip->skt->curve;
 
-    if (rip->ang < 2*M_PI) {
-	narc = ceil(rip->ang * 8 * M_1_PI);
+    if (rip->ang < M_2PI) {
+	narc = ceil(fabs(rip->ang * 8 * M_1_PI));
     } else {
 	narc = 16;
     }
@@ -1273,11 +1244,12 @@ rt_revolve_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct r
     VUNITIZE(uz);
 
     /* setup unit circle to be scaled */
+    ang_sign = rip->ang < 0 ? -1 : 1;
     VMOVE(ucir[ 0], ux);
     VREVERSE(ucir[ 8], ucir[0]);
 
     VSCALE(xdir, ux, cos22_5);
-    VSCALE(ydir, uy, cos67_5);
+    VSCALE(ydir, uy, ang_sign * cos67_5);
     VADD2(ucir[ 1], xdir, ydir);
     VREVERSE(ucir[ 9], ucir[1]);
     VREVERSE(xdir, xdir);
@@ -1285,7 +1257,7 @@ rt_revolve_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct r
     VREVERSE(ucir[15], ucir[7]);
 
     VSCALE(xdir, ux, M_SQRT1_2);
-    VSCALE(ydir, uy, M_SQRT1_2);
+    VSCALE(ydir, uy, ang_sign * M_SQRT1_2);
     VADD2(ucir[ 2], xdir, ydir);
     VREVERSE(ucir[10], ucir[2]);
     VREVERSE(xdir, xdir);
@@ -1294,14 +1266,15 @@ rt_revolve_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct r
 
 
     VSCALE(xdir, ux, cos67_5);
-    VSCALE(ydir, uy, cos22_5);
+    VSCALE(ydir, uy, ang_sign * cos22_5);
     VADD2(ucir[ 3], xdir, ydir);
     VREVERSE(ucir[11], ucir[3]);
     VREVERSE(xdir, xdir);
     VADD2(ucir[ 5], xdir, ydir);
     VREVERSE(ucir[13], ucir[5]);
 
-    VMOVE(ucir[ 4], uy);
+    VSCALE(ydir, uy, ang_sign);
+    VMOVE(ucir[ 4], ydir);
     VREVERSE(ucir[12], ucir[4]);
 
     /* find open endpoints, and determine which points are used */
@@ -1446,14 +1419,12 @@ rt_revolve_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct r
 
 
 /**
- * R T _ R E V O L V E _ T E S S
- *
  * Returns -
  * -1 failure
  * 0 OK.  *r points to nmgregion that holds this tessellation.
  */
 int
-rt_revolve_tess(struct nmgregion **UNUSED(r), struct model *UNUSED(m), struct rt_db_internal *ip, const struct rt_tess_tol *UNUSED(ttol), const struct bn_tol *UNUSED(tol))
+rt_revolve_tess(struct nmgregion **UNUSED(r), struct model *UNUSED(m), struct rt_db_internal *ip, const struct bg_tess_tol *UNUSED(ttol), const struct bn_tol *UNUSED(tol))
 {
     struct rt_revolve_internal *rip = NULL;
     struct rt_sketch_internal *sketch_ip = NULL;
@@ -1485,8 +1456,6 @@ rt_revolve_tess(struct nmgregion **UNUSED(r), struct model *UNUSED(m), struct rt
 
 
 /**
- * R T _ R E V O L V E _ I M P O R T 5
- *
  * Import an REVOLVE from the database format to the internal format.
  * Note that the data read will be in network order.  This means
  * Big-Endian integers and IEEE doubles for floating point.
@@ -1494,7 +1463,7 @@ rt_revolve_tess(struct nmgregion **UNUSED(r), struct model *UNUSED(m), struct rt
  * Apply modeling transformations as well.
  */
 int
-rt_revolve_import5(struct rt_db_internal *ip, const struct bu_external *ep, const mat_t mat, const struct db_i *dbip, struct resource *resp)
+rt_revolve_import5(struct rt_db_internal *ip, const struct bu_external *ep, const mat_t mat, const struct db_i *dbip)
 {
     struct rt_revolve_internal *rip;
 
@@ -1534,7 +1503,11 @@ rt_revolve_import5(struct rt_db_internal *ip, const struct bu_external *ep, cons
 	       sketch_name);
 	rip->skt = (struct rt_sketch_internal *)NULL;
     } else {
-	if (rt_db_get_internal(&tmp_ip, dp, dbip, bn_mat_identity, resp) != ID_SKETCH) {
+	/* initialize before our first use */
+	if (rt_uniresource.re_magic != RESOURCE_MAGIC)
+	    rt_init_resource(&rt_uniresource, 0, NULL);
+
+	if (rt_db_get_internal(&tmp_ip, dp, dbip, bn_mat_identity, &rt_uniresource) != ID_SKETCH) {
 	    bu_log("ERROR: Cannot import sketch (%s) for extrusion\n",
 		   sketch_name);
 	    bu_free(ip->idb_ptr, "extrusion");
@@ -1561,8 +1534,6 @@ rt_revolve_import5(struct rt_db_internal *ip, const struct bu_external *ep, cons
 
 
 /**
- * R T _ R E V O L V E _ X F O R M
- *
  * Apply a transformation matrix to the specified 'ip' input revolve
  * object, storing the results in the specified 'op' out pointer or
  * creating a copy if NULL.
@@ -1573,30 +1544,23 @@ rt_revolve_xform(
     const mat_t mat,
     struct rt_db_internal *ip,
     int release,
-    struct db_i *dbip,
-    struct resource *resp)
+    struct db_i *dbip)
 {
     struct rt_revolve_internal *rip, *rop;
     point_t tmp_vec;
 
     if (dbip) RT_CK_DBI(dbip);
     RT_CK_DB_INTERNAL(ip);
-    RT_CK_RESOURCE(resp);
     rip = (struct rt_revolve_internal *)ip->idb_ptr;
     RT_REVOLVE_CK_MAGIC(rip);
 
-    if (bu_debug&BU_DEBUG_MEM_CHECK) {
-	bu_log("Barrier check at start of revolve_xform():\n");
-	bu_mem_barriercheck();
-    }
-
     if (op != ip) {
 	RT_DB_INTERNAL_INIT(op);
 	BU_ALLOC(rop, struct rt_revolve_internal);
 	rop->magic = RT_REVOLVE_INTERNAL_MAGIC;
 	bu_vls_init(&rop->sketch_name);
 	bu_vls_vlscat(&rop->sketch_name, &rip->sketch_name);
-	op->idb_ptr = (genptr_t)rop;
+	op->idb_ptr = (void *)rop;
 	op->idb_meth = &OBJ[ID_REVOLVE];
 	op->idb_major_type = DB5_MAJORTYPE_BRLCAD;
 	op->idb_type = ID_REVOLVE;
@@ -1624,18 +1588,11 @@ rt_revolve_xform(
 	rop->skt = (struct rt_sketch_internal *)NULL;
     }
 
-    if (bu_debug&BU_DEBUG_MEM_CHECK) {
-	bu_log("Barrier check at end of revolve_xform():\n");
-	bu_mem_barriercheck();
-    }
-
     return 0;
 }
 
 
 /**
- * R T _ R E V O L V E _ E X P O R T 5
- *
  * Export an REVOLVE from internal form to external format.  Note that
  * this means converting all integers to Big-Endian format and
  * floating point data to IEEE double.
@@ -1661,7 +1618,7 @@ rt_revolve_export5(struct bu_external *ep, const struct rt_db_internal *ip, doub
 
     BU_CK_EXTERNAL(ep);
     ep->ext_nbytes = SIZEOF_NETWORK_DOUBLE * (ELEMENTS_PER_VECT*3 + 1) + bu_vls_strlen(&rip->sketch_name) + 1;
-    ep->ext_buf = (genptr_t)bu_calloc(1, ep->ext_nbytes, "revolve external");
+    ep->ext_buf = (uint8_t *)bu_calloc(1, ep->ext_nbytes, "revolve external");
 
     ptr = (unsigned char *)ep->ext_buf;
 
@@ -1684,8 +1641,6 @@ rt_revolve_export5(struct bu_external *ep, const struct rt_db_internal *ip, doub
 
 
 /**
- * R T _ R E V O L V E _ D E S C R I B E
- *
  * Make human-readable formatted presentation of this solid.  First
  * line describes type of solid.  Additional lines are indented one
  * tab, and give parameter values.
@@ -1733,8 +1688,6 @@ rt_revolve_describe(struct bu_vls *str, const struct rt_db_internal *ip, int ver
 
 
 /**
- * R T _ R E V O L V E _ I F R E E
- *
  * Free the storage associated with the rt_db_internal version of this
  * solid.
  */
@@ -1755,7 +1708,7 @@ rt_revolve_ifree(struct rt_db_internal *ip)
 	bu_log("Freeing bogus revolve, VLS string not initialized\n");
 
     bu_free((char *)revolve_ip, "revolve ifree");
-    ip->idb_ptr = GENPTR_NULL;	/* sanity */
+    ip->idb_ptr = ((void *)0);	/* sanity */
 }
 
 
