@@ -1,7 +1,7 @@
 /*                        S K E T C H . C
  * BRL-CAD
  *
- * Copyright (c) 1990-2013 United States Government as represented by
+ * Copyright (c) 1990-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This library is free software; you can redistribute it and/or
@@ -33,19 +33,21 @@
 #include <string.h>
 #include <math.h>
 #include <ctype.h>
-#include "bin.h"
+#include "bnetwork.h"
 
-#include "tcl.h"
 #include "vmath.h"
-#include "db.h"
+#include "bu/debug.h"
+#include "bu/cv.h"
+#include "bu/opt.h"
+#include "rt/db4.h"
 #include "nmg.h"
-#include "rtgeom.h"
+#include "rt/geom.h"
 #include "raytrace.h"
-#include "nurb.h"
+
+#include "../../librt_private.h"
 
 
-fastf_t rt_cnurb_par_edge(const struct edge_g_cnurb *crv, fastf_t epsilon);
-extern void get_indices(genptr_t seg, int *start, int *end);	/* from g_extrude.c */
+extern void get_indices(void *seg, int *start, int *end);	/* from g_extrude.c */
 
 
 int
@@ -116,8 +118,6 @@ rt_check_curve(const struct rt_curve *crv, const struct rt_sketch_internal *skt,
 
 
 /**
- * R T _ S K E T C H _ P R E P
- *
  * Given a pointer to a GED database record, and a transformation
  * matrix, determine if this is a valid SKETCH, and if so, precompute
  * various terms of the formula.
@@ -139,14 +139,11 @@ rt_sketch_prep(struct soltab *stp, struct rt_db_internal *ip, struct rt_i *rtip)
     if (ip) RT_CK_DB_INTERNAL(ip);
     if (rtip) RT_CK_RTI(rtip);
 
-    stp->st_specific = (genptr_t)NULL;
+    stp->st_specific = (void *)NULL;
     return 0;
 }
 
 
-/**
- * R T _ S K E T C H _ P R I N T
- */
 void
 rt_sketch_print(const struct soltab *stp)
 {
@@ -155,8 +152,6 @@ rt_sketch_print(const struct soltab *stp)
 
 
 /**
- * R T _ S K E T C H _ S H O T
- *
  * Intersect a ray with a sketch.  If an intersection occurs, a struct
  * seg will be acquired and filled in.
  *
@@ -183,8 +178,6 @@ rt_sketch_shot(struct soltab *stp, struct xray *rp, struct application *ap, stru
 
 
 /**
- * R T _ S K E T C H _ N O R M
- *
  * Given ONE ray distance, return the normal and entry/exit point.
  */
 void
@@ -202,8 +195,6 @@ rt_sketch_norm(struct hit *hitp, struct soltab *stp, struct xray *rp)
 
 
 /**
- * R T _ S K E T C H _ C U R V E
- *
  * Return the curvature of the sketch.
  */
 void
@@ -223,8 +214,6 @@ rt_sketch_curve(struct curvature *cvp, struct hit *hitp, struct soltab *stp)
 
 
 /**
- * R T _ S K E T C H _ U V
- *
  * For a hit on the surface of an sketch, return the (u, v) coordinates
  * of the hit point, 0 <= u, v <= 1.
  *
@@ -241,9 +230,6 @@ rt_sketch_uv(struct application *ap, struct soltab *stp, struct hit *hitp, struc
 }
 
 
-/**
- * R T _ S K E T C H _ F R E E
- */
 void
 rt_sketch_free(struct soltab *stp)
 {
@@ -251,16 +237,6 @@ rt_sketch_free(struct soltab *stp)
 }
 
 
-/**
- * R T _ S K E T C H _ C L A S S
- */
-int
-rt_sketch_class(void)
-{
-    return 0;
-}
-
-
 /*
   determine whether the 2D point 'pt' is within the bounds of the sketch
   by counting the number of intersections between the curve and a line
@@ -353,8 +329,8 @@ rt_sketch_contains(struct rt_sketch_internal *sk, point2d_t pt)
 			}
 			angMax -= angMin;
 			angle -= angMin;
-			if (angMax < 0) angMax += 2*M_PI;
-			if (angle < 0) angle += 2*M_PI;
+			if (angMax < 0) angMax += M_2PI;
+			if (angle < 0) angle += M_2PI;
 			if (angle < angMax) hits++;
 		    }
 		}
@@ -488,7 +464,7 @@ rt_sketch_degree(struct rt_sketch_internal *sk)
 
 
 int
-seg_to_vlist(struct bu_list *vhead, const struct rt_tess_tol *ttol, fastf_t *V, fastf_t *u_vec, fastf_t *v_vec, struct rt_sketch_internal *sketch_ip, genptr_t seg)
+seg_to_vlist(struct bu_list *vhead, const struct bg_tess_tol *ttol, fastf_t *V, fastf_t *u_vec, fastf_t *v_vec, struct rt_sketch_internal *sketch_ip, void *seg)
 {
     int ret=0;
     int i;
@@ -519,8 +495,8 @@ seg_to_vlist(struct bu_list *vhead, const struct rt_tess_tol *ttol, fastf_t *V,
 		break;
 	    }
 	    VJOIN2(pt, V, sketch_ip->verts[lsg->start][0], u_vec, sketch_ip->verts[lsg->start][1], v_vec);
-	    RT_ADD_VLIST(vhead, pt, BN_VLIST_LINE_MOVE)
-		VJOIN2(pt, V, sketch_ip->verts[lsg->end][0], u_vec, sketch_ip->verts[lsg->end][1], v_vec);
+	    RT_ADD_VLIST(vhead, pt, BN_VLIST_LINE_MOVE);
+	    VJOIN2(pt, V, sketch_ip->verts[lsg->end][0], u_vec, sketch_ip->verts[lsg->end][1], v_vec);
 	    RT_ADD_VLIST(vhead, pt, BN_VLIST_LINE_DRAW);
 	    break;
 	case CURVE_CARC_MAGIC:
@@ -580,8 +556,8 @@ seg_to_vlist(struct bu_list *vhead, const struct rt_tess_tol *ttol, fastf_t *V,
 		}
 		if (csg->radius <= 0.0) {
 		    /* this is a full circle */
-		    nsegs = ceil(2.0 * M_PI / delta);
-		    delta = 2.0 * M_PI / (double)nsegs;
+		    nsegs = ceil(M_2PI / delta);
+		    delta = M_2PI / (double)nsegs;
 		    cosdel = cos(delta);
 		    sindel = sin(delta);
 		    oldu = 1.0;
@@ -634,11 +610,11 @@ seg_to_vlist(struct bu_list *vhead, const struct rt_tess_tol *ttol, fastf_t *V,
 		if (csg->orientation) {
 		    /* clock-wise */
 		    while (end_ang > start_ang)
-			end_ang -= 2.0 * M_PI;
+			end_ang -= M_2PI;
 		} else {
 		    /* counter-clock-wise */
 		    while (end_ang < start_ang)
-			end_ang += 2.0 * M_PI;
+			end_ang += M_2PI;
 		}
 		tot_ang = end_ang - start_ang;
 		nsegs = ceil(tot_ang / delta);
@@ -759,15 +735,15 @@ seg_to_vlist(struct bu_list *vhead, const struct rt_tess_tol *ttol, fastf_t *V,
 		    int j;
 
 		    t = nsg->k.knots[0] + i*param_delta;
-		    rt_nurb_c_eval(&eg, t, pt);
+		    nmg_nurb_c_eval(&eg, t, pt);
 		    if (RT_NURB_IS_PT_RATIONAL(nsg->pt_type)) {
 			for (j=0; j<coords-1; j++)
 			    pt[j] /= pt[coords-1];
 		    }
 		    if (i == 0)
-			RT_ADD_VLIST(vhead, pt, BN_VLIST_LINE_MOVE)
-			    else
-				RT_ADD_VLIST(vhead, pt, BN_VLIST_LINE_DRAW);
+			RT_ADD_VLIST(vhead, pt, BN_VLIST_LINE_MOVE);
+		    else
+			RT_ADD_VLIST(vhead, pt, BN_VLIST_LINE_DRAW);
 		}
 		bu_free((char *)eg.ctl_points, "eg.ctl_points");
 		break;
@@ -850,7 +826,7 @@ seg_to_vlist(struct bu_list *vhead, const struct rt_tess_tol *ttol, fastf_t *V,
 	    }
 
 	    /* now do subdivision as necessary */
-	    bezier_hd = subdivide_bezier(bezier_hd, bsg->degree, epsilon, 0);
+	    bezier_hd = bezier_subdivide(bezier_hd, bsg->degree, epsilon, 0);
 
 	    /* plot the results */
 	    bz = BU_LIST_FIRST(bezier_2d_list, &bezier_hd->l);
@@ -879,40 +855,24 @@ seg_to_vlist(struct bu_list *vhead, const struct rt_tess_tol *ttol, fastf_t *V,
 }
 
 
-/**
- * C U R V E _ T O _ V L I S T
- */
 int
-curve_to_vlist(struct bu_list *vhead, const struct rt_tess_tol *ttol, fastf_t *V, fastf_t *u_vec, fastf_t *v_vec, struct rt_sketch_internal *sketch_ip, struct rt_curve *crv)
+curve_to_vlist(struct bu_list *vhead, const struct bg_tess_tol *ttol, fastf_t *V, fastf_t *u_vec, fastf_t *v_vec, struct rt_sketch_internal *sketch_ip, struct rt_curve *crv)
 {
     size_t seg_no;
     int ret=0;
 
     BU_CK_LIST_HEAD(vhead);
 
-    if (bu_debug&BU_DEBUG_MEM_CHECK) {
-	bu_log("Barrier check at start of curve_to_vlist():\n");
-	bu_mem_barriercheck();
-    }
-
     for (seg_no=0; seg_no < crv->count; seg_no++) {
 	ret += seg_to_vlist(vhead, ttol, V, u_vec, v_vec, sketch_ip, crv->segment[seg_no]);
     }
 
-    if (bu_debug&BU_DEBUG_MEM_CHECK) {
-	bu_log("Barrier check at end of curve_to_vlist():\n");
-	bu_mem_barriercheck();
-    }
-
     return ret;
 }
 
 
-/**
- * R T _ S K E T C H _ P L O T
- */
 int
-rt_sketch_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct rt_tess_tol *ttol, const struct bn_tol *UNUSED(tol), const struct rt_view_info *UNUSED(info))
+rt_sketch_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct bg_tess_tol *ttol, const struct bn_tol *UNUSED(tol), const struct rt_view_info *UNUSED(info))
 {
     struct rt_sketch_internal *sketch_ip;
     int ret;
@@ -933,16 +893,189 @@ rt_sketch_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct rt
     return myret;
 }
 
+/**
+ * Given a sketch, bounds, and precision for rt_sketch_centroid, check
+ * if the precision is large enough (if the sample size is small
+ * enough) to intersect every curve of the given sketch.
+ *
+ * Returns -
+ * 0 precision is sufficiently large
+ * !0 precision is too small
+ */
+static int
+sketch_centroid_check_precision(const struct rt_sketch_internal *skt,
+				   fastf_t precision,
+				   fastf_t bounds[4])
+{
+    fastf_t x_inc, y_inc, min_inc;
+    fastf_t min_size = INFINITY;
+    unsigned int i;
+    const struct rt_curve *crv = &skt->curve;
+    x_inc = (bounds[1]-bounds[0])/precision;
+    y_inc = (bounds[3]-bounds[2])/precision;
+    min_inc = x_inc < y_inc ? x_inc : y_inc;
+    for (i = 0; i < crv->count; i++) {
+	point2d_t size;
+	const struct line_seg *lsg;
+	const struct carc_seg *csg;
+	const struct nurb_seg *nsg;
+	const struct bezier_seg *bsg;
+	const uint32_t *lng;
+	int j;
+	point2d_t min_pt, max_pt;
+
+	V2SETALL(min_pt, INFINITY);
+	V2SETALL(max_pt, -INFINITY);
+
+	lng = (uint32_t *)crv->segment[i];
+
+	switch (*lng) {
+	    case CURVE_LSEG_MAGIC:
+		lsg = (struct line_seg *)lng;
+		V2SUB2(size, skt->verts[lsg->end], skt->verts[lsg->start]);
+		V_MIN(min_size, MAGNITUDE2(size));
+		break;
+	    case CURVE_CARC_MAGIC:
+		csg = (struct carc_seg *)lng;
+		V2SUB2(size, skt->verts[csg->end], skt->verts[csg->start]);
+		V_MIN(min_size, MAGNITUDE2(size));
+		break;
+ 	    case CURVE_NURB_MAGIC:
+		nsg = (struct nurb_seg *)lng;
+		for (j = 0; j < nsg->c_size; j++) {
+		    V2MINMAX(min_pt, max_pt, skt->verts[nsg->ctl_points[j]]);
+		}
+		V2SUB2(size, max_pt, min_pt);
+		V_MIN(min_size, MAGNITUDE2(size));
+		break;
+     	    case CURVE_BEZIER_MAGIC:
+		bsg = (struct bezier_seg *)lng;
+		for (j = 0; j <= bsg->degree; j++) {
+		    V2MINMAX(min_pt, max_pt, skt->verts[bsg->ctl_points[j]]);
+		}
+		V2SUB2(size, max_pt, min_pt);
+		V_MIN(min_size, MAGNITUDE2(size));
+		break;
+	}
+    }
+    return (min_size < min_inc);
+}
+
+static long
+sketch_update_centroid_with_coords(point_t *cent,
+				   struct rt_sketch_internal *skt,
+				   long n,
+				   fastf_t i,
+				   fastf_t j)
+{
+    point2d_t pt;
+    point_t pt3d;
+    V2SET(pt, i, j);
+    VSET(pt3d, i, j, 0);
+    if (rt_sketch_contains(skt, pt)) {
+	VADD2(*cent, *cent, pt3d);
+	n++;
+    }
+    return n;
+}
+
+
+static long
+sketch_centroid_with_precision(point_t *cent,
+			       struct rt_sketch_internal *skt,
+			       fastf_t precision,
+			       long n,
+			       fastf_t bounds[4])
+{
+    fastf_t x_inc, y_inc;
+    fastf_t x_skip, y_skip;
+    fastf_t i, j;
+
+    x_inc = (bounds[1] - bounds[0])/precision;
+    y_inc = (bounds[3] - bounds[2])/precision;
+
+    x_skip = 2 * x_inc;
+    y_skip = 2 * y_inc;
+
+    if (n > 0) {
+	VSCALE(*cent, *cent, n);
+	for (i = bounds[0] + x_inc; i < bounds[1]; i += x_skip) {
+	    for (j = bounds[2]; j < bounds[3]; j += y_inc) {
+		n = sketch_update_centroid_with_coords(cent, skt, n, i, j);
+	    }
+	}
+	for (i = bounds[0]; i < bounds[1]; i += x_skip) {
+	    for (j = bounds[2] + y_inc; j < bounds[3]; j += y_skip) {
+		n = sketch_update_centroid_with_coords(cent, skt, n, i, j);
+	    }
+	}
+    } else {
+	for (i = bounds[0]; i < bounds[1]; i += x_inc) {
+	    for (j = bounds[2]; j < bounds[3]; j += y_inc) {
+		n = sketch_update_centroid_with_coords(cent, skt, n, i, j);
+	    }
+	}
+    }
+    VSCALE(*cent, *cent, 1.0 / (fastf_t)n);
+    return n;
+}
+
+void
+rt_sketch_centroid(point_t *cent, const struct rt_db_internal *ip)
+{
+    /* With BN_TOL_DIST instead, even relatively simple sketches can
+     * bounce back and forth between points < 0.005 apart but > 0.0005
+     * for a long time.  Since calculating is very computationally
+     * intensive, using this TOLERANCE instead makes the computation
+     * finish significantly faster without a huge loss of accuracy
+     */
+    const fastf_t TOLERANCE = 0.005;
+    /* This is the initial precision that will be used as the
+     * precision for the first iteration, assuming that the sketch
+     * does not have any features that are small enough to escape
+     * detection with this precision.  1024 was chosen because
+     * smaller values tend to be very inaccurate, and oscillate enough
+     * that convergence is not achieved; 1024 is a good balance
+     * between (for the majority of sketches) being too large (and
+     * missing a smaller convergence) and being too small (and doing
+     * extra iterations for no reason).
+     */
+    fastf_t precision = 1024;
+    fastf_t bounds[4];
+    struct rt_sketch_internal *sketch_ip = (struct rt_sketch_internal *)ip->idb_ptr;
+    point_t current_cents[2];
+    point_t difference = {INFINITY, INFINITY, INFINITY};
+    long n[2] = {0, 0};
+    int i = 1;
+    RT_SKETCH_CK_MAGIC(sketch_ip);
+
+    VSETALL(*cent, 0.0);
+    VSETALL(current_cents[0], 0.0);
+    VSETALL(current_cents[1], 0.0);
+
+    rt_sketch_bounds(sketch_ip, bounds);
+    while (sketch_centroid_check_precision(sketch_ip, precision, bounds)) {
+	precision *= 2;
+    }
+
+    while (!NEAR_ZERO(MAGNITUDE(difference), TOLERANCE)) {
+	n[i] = sketch_centroid_with_precision(&current_cents[i], sketch_ip, precision, n[i], bounds);
+	precision *= M_SQRT2;
+	VSUB2(difference, current_cents[i], current_cents[!i]);
+	i = !i;
+    }
+
+    VMOVE(*cent, current_cents[!i]);
+}
+
 
 /**
- * R T _ S K E T C H _ T E S S
- *
  * Returns -
  * -1 failure
  * 0 OK.  *r points to nmgregion that holds this tessellation.
  */
 int
-rt_sketch_tess(struct nmgregion **UNUSED(r), struct model *UNUSED(m), struct rt_db_internal *ip, const struct rt_tess_tol *UNUSED(ttol), const struct bn_tol *UNUSED(tol))
+rt_sketch_tess(struct nmgregion **UNUSED(r), struct model *UNUSED(m), struct rt_db_internal *ip, const struct bg_tess_tol *UNUSED(ttol), const struct bn_tol *UNUSED(tol))
 {
     if (ip) RT_CK_DB_INTERNAL(ip);
 
@@ -951,8 +1084,6 @@ rt_sketch_tess(struct nmgregion **UNUSED(r), struct model *UNUSED(m), struct rt_
 
 
 /**
- * R T _ S K E T C H _ I M P O R T
- *
  * Import an SKETCH from the database format to the internal format.
  * Apply modeling transformations as well.
  */
@@ -980,11 +1111,6 @@ rt_sketch_import4(struct rt_db_internal *ip, const struct bu_external *ep, const
 	return -1;
     }
 
-    if (bu_debug&BU_DEBUG_MEM_CHECK) {
-	bu_log("Barrier check at start of sketch_import4():\n");
-	bu_mem_barriercheck();
-    }
-
     RT_CK_DB_INTERNAL(ip);
     ip->idb_major_type = DB5_MAJORTYPE_BRLCAD;
     ip->idb_type = ID_SKETCH;
@@ -1022,9 +1148,9 @@ rt_sketch_import4(struct rt_db_internal *ip, const struct bu_external *ep, const
     }
 
     if (sketch_ip->curve.count)
-	sketch_ip->curve.segment = (genptr_t *)bu_calloc(sketch_ip->curve.count, sizeof(genptr_t), "segs");
+	sketch_ip->curve.segment = (void **)bu_calloc(sketch_ip->curve.count, sizeof(void *), "segs");
     else
-	sketch_ip->curve.segment = (genptr_t *)NULL;
+	sketch_ip->curve.segment = (void **)NULL;
     for (seg_no=0; seg_no < sketch_ip->curve.count; seg_no++) {
 	uint32_t magic;
 	struct line_seg *lsg;
@@ -1046,7 +1172,7 @@ rt_sketch_import4(struct rt_db_internal *ip, const struct bu_external *ep, const
 		ptr += SIZEOF_NETWORK_LONG;
 		lsg->end = ntohl(*(uint32_t *)ptr);
 		ptr += SIZEOF_NETWORK_LONG;
-		sketch_ip->curve.segment[seg_no] = (genptr_t)lsg;
+		sketch_ip->curve.segment[seg_no] = (void *)lsg;
 		break;
 	    case CURVE_CARC_MAGIC:
 		BU_ALLOC(csg, struct carc_seg);
@@ -1062,7 +1188,7 @@ rt_sketch_import4(struct rt_db_internal *ip, const struct bu_external *ep, const
 		bu_cv_ntohd((unsigned char *)&scan, ptr, 1);
 		csg->radius = scan; /* convert double to fastf_t */
 		ptr += SIZEOF_NETWORK_DOUBLE;
-		sketch_ip->curve.segment[seg_no] = (genptr_t)csg;
+		sketch_ip->curve.segment[seg_no] = (void *)csg;
 		break;
 	    case CURVE_NURB_MAGIC:
 		BU_ALLOC(nsg, struct nurb_seg);
@@ -1107,7 +1233,7 @@ rt_sketch_import4(struct rt_db_internal *ip, const struct bu_external *ep, const
 		} else {
 		    nsg->weights = (fastf_t *)NULL;
 		}
-		sketch_ip->curve.segment[seg_no] = (genptr_t)nsg;
+		sketch_ip->curve.segment[seg_no] = (void *)nsg;
 		break;
 	    case CURVE_BEZIER_MAGIC:
 		BU_ALLOC(bsg, struct bezier_seg);
@@ -1119,7 +1245,7 @@ rt_sketch_import4(struct rt_db_internal *ip, const struct bu_external *ep, const
 		    bsg->ctl_points[i] = ntohl(*(uint32_t *)ptr);
 		    ptr += SIZEOF_NETWORK_LONG;
 		}
-		sketch_ip->curve.segment[seg_no] = (genptr_t)bsg;
+		sketch_ip->curve.segment[seg_no] = (void *)bsg;
 		break;
 	    default:
 		bu_bomb("rt_sketch_import4: ERROR: unrecognized segment type!\n");
@@ -1136,18 +1262,11 @@ rt_sketch_import4(struct rt_db_internal *ip, const struct bu_external *ep, const
 	ptr += SIZEOF_NETWORK_LONG;
     }
 
-    if (bu_debug&BU_DEBUG_MEM_CHECK) {
-	bu_log("Barrier check at end of sketch_import4():\n");
-	bu_mem_barriercheck();
-    }
-
     return 0;			/* OK */
 }
 
 
 /**
- * R T _ S K E T C H _ E X P O R T
- *
  * The name is added by the caller, in the usual place.
  */
 int
@@ -1168,11 +1287,6 @@ rt_sketch_export4(struct bu_external *ep, const struct rt_db_internal *ip, doubl
     sketch_ip = (struct rt_sketch_internal *)ip->idb_ptr;
     RT_SKETCH_CK_MAGIC(sketch_ip);
 
-    if (bu_debug&BU_DEBUG_MEM_CHECK) {
-	bu_log("Barrier check at start of sketch_export4():\n");
-	bu_mem_barriercheck();
-    }
-
     BU_CK_EXTERNAL(ep);
 
     nbytes = sizeof(union record);		/* base record */
@@ -1209,7 +1323,7 @@ rt_sketch_export4(struct bu_external *ep, const struct rt_db_internal *ip, doubl
 
     ngran = ceil((double)(nbytes + sizeof(union record)) / sizeof(union record));
     ep->ext_nbytes = ngran * sizeof(union record);
-    ep->ext_buf = (genptr_t)bu_calloc(1, ep->ext_nbytes, "sketch external");
+    ep->ext_buf = (uint8_t *)bu_calloc(1, ep->ext_nbytes, "sketch external");
 
     rec = (union record *)ep->ext_buf;
 
@@ -1339,18 +1453,12 @@ rt_sketch_export4(struct bu_external *ep, const struct rt_db_internal *ip, doubl
 	*(uint32_t *)ptr = htonl(sketch_ip->curve.reverse[seg_no]);
 	ptr += SIZEOF_NETWORK_LONG;
     }
-    if (bu_debug&BU_DEBUG_MEM_CHECK) {
-	bu_log("Barrier check at end of sketch_export4():\n");
-	bu_mem_barriercheck();
-    }
 
     return 0;
 }
 
 
 /**
- * R T _ S K E T C H _ I M P O R T 5
- *
  * Import an SKETCH from the database format to the internal format.
  * Apply modeling transformations as well.
  */
@@ -1367,11 +1475,6 @@ rt_sketch_import5(struct rt_db_internal *ip, const struct bu_external *ep, const
     double v[ELEMENTS_PER_VECT];
     double *vp;
 
-    if (bu_debug&BU_DEBUG_MEM_CHECK) {
-	bu_log("Barrier check at start of sketch_import5():\n");
-	bu_mem_barriercheck();
-    }
-
     if (dbip) RT_CK_DBI(dbip);
     BU_CK_EXTERNAL(ep);
 
@@ -1416,9 +1519,9 @@ rt_sketch_import5(struct rt_db_internal *ip, const struct bu_external *ep, const
     }
 
     if (sketch_ip->curve.count)
-	sketch_ip->curve.segment = (genptr_t *)bu_calloc(sketch_ip->curve.count, sizeof(genptr_t), "segs");
+	sketch_ip->curve.segment = (void **)bu_calloc(sketch_ip->curve.count, sizeof(void *), "segs");
     else
-	sketch_ip->curve.segment = (genptr_t *)NULL;
+	sketch_ip->curve.segment = (void **)NULL;
     for (seg_no=0; seg_no < sketch_ip->curve.count; seg_no++) {
 	uint32_t magic;
 	struct line_seg *lsg;
@@ -1440,7 +1543,7 @@ rt_sketch_import5(struct rt_db_internal *ip, const struct bu_external *ep, const
 		ptr += SIZEOF_NETWORK_LONG;
 		lsg->end = ntohl(*(uint32_t *)ptr);
 		ptr += SIZEOF_NETWORK_LONG;
-		sketch_ip->curve.segment[seg_no] = (genptr_t)lsg;
+		sketch_ip->curve.segment[seg_no] = (void *)lsg;
 		break;
 	    case CURVE_CARC_MAGIC:
 		BU_ALLOC(csg, struct carc_seg);
@@ -1456,7 +1559,7 @@ rt_sketch_import5(struct rt_db_internal *ip, const struct bu_external *ep, const
 		bu_cv_ntohd((unsigned char *)&scan, ptr, 1);
 		csg->radius = scan; /* double to fastf_t */
 		ptr += SIZEOF_NETWORK_DOUBLE;
-		sketch_ip->curve.segment[seg_no] = (genptr_t)csg;
+		sketch_ip->curve.segment[seg_no] = (void *)csg;
 		break;
 	    case CURVE_NURB_MAGIC:
 		BU_ALLOC(nsg, struct nurb_seg);
@@ -1500,7 +1603,7 @@ rt_sketch_import5(struct rt_db_internal *ip, const struct bu_external *ep, const
 		    ptr += SIZEOF_NETWORK_DOUBLE * nsg->c_size;
 		} else
 		    nsg->weights = (fastf_t *)NULL;
-		sketch_ip->curve.segment[seg_no] = (genptr_t)nsg;
+		sketch_ip->curve.segment[seg_no] = (void *)nsg;
 		break;
 	    case CURVE_BEZIER_MAGIC:
 		BU_ALLOC(bsg, struct bezier_seg);
@@ -1512,7 +1615,7 @@ rt_sketch_import5(struct rt_db_internal *ip, const struct bu_external *ep, const
 		    bsg->ctl_points[i] = ntohl(*(uint32_t *)ptr);
 		    ptr += SIZEOF_NETWORK_LONG;
 		}
-		sketch_ip->curve.segment[seg_no] = (genptr_t)bsg;
+		sketch_ip->curve.segment[seg_no] = (void *)bsg;
 		break;
 	    default:
 		bu_bomb("rt_sketch_import4: ERROR: unrecognized segment type!\n");
@@ -1531,18 +1634,11 @@ rt_sketch_import5(struct rt_db_internal *ip, const struct bu_external *ep, const
 	ptr += SIZEOF_NETWORK_LONG;
     }
 
-    if (bu_debug&BU_DEBUG_MEM_CHECK) {
-	bu_log("Barrier check at end of sketch_import5():\n");
-	bu_mem_barriercheck();
-    }
-
     return 0;			/* OK */
 }
 
 
 /**
- * R T _ S K E T C H _ E X P O R T 5
- *
  * The name is added by the caller, in the usual place.
  */
 int
@@ -1556,11 +1652,6 @@ rt_sketch_export5(struct bu_external *ep, const struct rt_db_internal *ip, doubl
     /* must be double for import and export */
     double tmp_vec[ELEMENTS_PER_VECT];
 
-    if (bu_debug&BU_DEBUG_MEM_CHECK) {
-	bu_log("Barrier check at start of sketch_export5():\n");
-	bu_mem_barriercheck();
-    }
-
     if (dbip) RT_CK_DBI(dbip);
 
     RT_CK_DB_INTERNAL(ip);
@@ -1615,7 +1706,7 @@ rt_sketch_export5(struct bu_external *ep, const struct rt_db_internal *ip, doubl
 		bu_bomb("rt_sketch_export5: unsupported segment type\n");
 	}
     }
-    ep->ext_buf = (genptr_t)bu_malloc(ep->ext_nbytes, "sketch external");
+    ep->ext_buf = (uint8_t *)bu_malloc(ep->ext_nbytes, "sketch external");
 
     cp = (unsigned char *)ep->ext_buf;
 
@@ -1744,18 +1835,11 @@ rt_sketch_export5(struct bu_external *ep, const struct rt_db_internal *ip, doubl
 	cp += SIZEOF_NETWORK_LONG;
     }
 
-    if (bu_debug&BU_DEBUG_MEM_CHECK) {
-	bu_log("Barrier check at end of sketch_export5():\n");
-	bu_mem_barriercheck();
-    }
-
     return 0;
 }
 
 
 /**
- * R T _ S K E T C H _ D E S C R I B E
- *
  * Make human-readable formatted presentation of this solid.  First
  * line describes type of solid.  Additional lines are indented one
  * tab, and give parameter values.
@@ -1995,13 +2079,11 @@ rt_curve_free(struct rt_curve *crv)
 
     crv->count = 0;
     crv->reverse = (int *)NULL;
-    crv->segment = (genptr_t)NULL;
+    crv->segment = (void **)NULL;
 }
 
 
 /**
- * R T _ S K E T C H _ I F R E E
- *
  * Free the storage associated with the rt_db_internal version of this
  * solid.
  */
@@ -2017,11 +2099,6 @@ rt_sketch_ifree(struct rt_db_internal *ip)
     RT_SKETCH_CK_MAGIC(sketch_ip);
     sketch_ip->magic = 0;			/* sanity */
 
-    if (bu_debug&BU_DEBUG_MEM_CHECK) {
-	bu_log("Barrier check at start of sketch_ifree():\n");
-	bu_mem_barriercheck();
-    }
-
     if (sketch_ip->verts)
 	bu_free((char *)sketch_ip->verts, "sketch_ip->verts");
 
@@ -2030,12 +2107,7 @@ rt_sketch_ifree(struct rt_db_internal *ip)
     rt_curve_free(crv);
 
     bu_free((char *)sketch_ip, "sketch ifree");
-    ip->idb_ptr = GENPTR_NULL;	/* sanity */
-
-    if (bu_debug&BU_DEBUG_MEM_CHECK) {
-	bu_log("Barrier check at end of sketch_ifree():\n");
-	bu_mem_barriercheck();
-    }
+    ip->idb_ptr = ((void *)0);	/* sanity */
 }
 
 
@@ -2047,7 +2119,7 @@ rt_copy_curve(struct rt_curve *crv_out, const struct rt_curve *crv_in)
     crv_out->count = crv_in->count;
     if (crv_out->count) {
 	crv_out->reverse = (int *)bu_calloc(crv_out->count, sizeof(int), "crv->reverse");
-	crv_out->segment = (genptr_t *)bu_calloc(crv_out->count, sizeof(genptr_t), "crv->segments");
+	crv_out->segment = (void **)bu_calloc(crv_out->count, sizeof(void *), "crv->segments");
     }
 
     for (j=0; j<crv_out->count; j++) {
@@ -2063,19 +2135,19 @@ rt_copy_curve(struct rt_curve *crv_out, const struct rt_curve *crv_in)
 	    case CURVE_LSEG_MAGIC:
 		lsg_in = (struct line_seg *)lng;
 		BU_ALLOC(lsg_out, struct line_seg);
-		crv_out->segment[j] = (genptr_t)lsg_out;
+		crv_out->segment[j] = (void *)lsg_out;
 		*lsg_out = *lsg_in;
 		break;
 	    case CURVE_CARC_MAGIC:
 		csg_in = (struct carc_seg *)lng;
 		BU_ALLOC(csg_out, struct carc_seg);
-		crv_out->segment[j] = (genptr_t)csg_out;
+		crv_out->segment[j] = (void *)csg_out;
 		*csg_out = *csg_in;
 		break;
 	    case CURVE_NURB_MAGIC:
 		nsg_in = (struct nurb_seg *)lng;
 		BU_ALLOC(nsg_out, struct nurb_seg);
-		crv_out->segment[j] = (genptr_t)nsg_out;
+		crv_out->segment[j] = (void *)nsg_out;
 		*nsg_out = *nsg_in;
 		nsg_out->ctl_points = (int *)bu_calloc(nsg_in->c_size, sizeof(int), "nsg_out->ctl_points");
 		for (i=0; i<(size_t)nsg_out->c_size; i++)
@@ -2086,14 +2158,14 @@ rt_copy_curve(struct rt_curve *crv_out, const struct rt_curve *crv_in)
 			nsg_out->weights[i] = nsg_in->weights[i];
 		} else
 		    nsg_out->weights = (fastf_t *)NULL;
-		nsg_out->k.knots = bu_malloc(nsg_in->k.k_size * sizeof(fastf_t), "nsg_out->k.knots");
+		nsg_out->k.knots = (fastf_t *)bu_malloc(nsg_in->k.k_size * sizeof(fastf_t), "nsg_out->k.knots");
 		for (i=0; i<(size_t)nsg_in->k.k_size; i++)
 		    nsg_out->k.knots[i] = nsg_in->k.knots[i];
 		break;
 	    case CURVE_BEZIER_MAGIC:
 		bsg_in = (struct bezier_seg *)lng;
 		BU_ALLOC(bsg_out, struct bezier_seg);
-		crv_out->segment[j] = (genptr_t)bsg_out;
+		crv_out->segment[j] = (void *)bsg_out;
 		*bsg_out = *bsg_in;
 		bsg_out->ctl_points = (int *)bu_calloc(bsg_out->degree + 1,
 						       sizeof(int), "bsg_out->ctl_points");
@@ -2118,11 +2190,6 @@ rt_copy_sketch(const struct rt_sketch_internal *sketch_ip)
 
     RT_SKETCH_CK_MAGIC(sketch_ip);
 
-    if (bu_debug&BU_DEBUG_MEM_CHECK) {
-	bu_log("Barrier check at start of rt_copy_sketch():\n");
-	bu_mem_barriercheck();
-    }
-
     BU_ALLOC(out, struct rt_sketch_internal);
     *out = *sketch_ip;	/* struct copy */
 
@@ -2137,11 +2204,6 @@ rt_copy_sketch(const struct rt_sketch_internal *sketch_ip)
     if (crv_out)
 	rt_copy_curve(crv_out, &sketch_ip->curve);
 
-    if (bu_debug&BU_DEBUG_MEM_CHECK) {
-	bu_log("Barrier check at end of rt_copy_sketch():\n");
-	bu_mem_barriercheck();
-    }
-
     return out;
 }
 
@@ -2271,185 +2333,148 @@ rt_sketch_get(struct bu_vls *logstr, const struct rt_db_internal *intern, const
 
 
 int
-get_tcl_curve(Tcl_Interp *interp, struct rt_curve *crv, Tcl_Obj *seg_list)
+get_tcl_curve(struct bu_vls *logstr, struct rt_curve *crv, const char *argv1)
 {
     int count;
-    int ret, j;
+    int j;
+    const char **seg_list = NULL;
 
-    /* get number of segments */
-    count = 0;
-    ret=Tcl_ListObjLength(interp, seg_list, &count);
-    if (ret) {
-	return ret;
+    /* split initial list */
+    if (bu_argv_from_tcl_list(argv1, &count, (const char ***)&seg_list) != 0) {
+	return -1;
     }
 
     if (count) {
 	crv->count = count;
 	crv->reverse = (int *)bu_calloc(count, sizeof(int), "crv->reverse");
-	crv->segment = (genptr_t *)bu_calloc(count, sizeof(genptr_t), "crv->segment");
+	crv->segment = (void **)bu_calloc(count, sizeof(void *), "crv->segment");
     }
 
     /* loop through all the segments */
-    for (j=0; j<count; j++) {
-	Tcl_Obj *seg, *seg_type, *seg_elem, *seg_val;
-	char *type, *elem;
-	int k, seg_len;
-
+    for (j = 0; j < count; j++) {
+	int seg_argc;
+	const char **seg_argv = NULL;
+	const char *elem, *sval;
+	int k;
 
 	/* get the next segment */
-	ret=Tcl_ListObjIndex(interp, seg_list, j, &seg);
-	if (ret)
-	    return ret;
+	if (bu_argv_from_tcl_list(seg_list[j], &seg_argc, (const char ***)&seg_argv) != 0) {
+	    return -1;
+	}
 
-	ret=Tcl_ListObjLength(interp, seg, &seg_len);
-	if (ret)
-	    return ret;
+	if (seg_argc < 1) return 0;
 
-	/* get the segment type */
-	ret=Tcl_ListObjIndex(interp, seg, 0, &seg_type);
-	if (ret)
-	    return ret;
-	type = Tcl_GetString(seg_type);
-
-	if (BU_STR_EQUAL(type, "line")) {
+	/* get the next segment */
+	if (BU_STR_EQUAL(seg_argv[0], "line")) {
 	    struct line_seg *lsg;
 
 	    BU_ALLOC(lsg, struct line_seg);
-	    for (k=1; k<seg_len; k += 2) {
-		ret=Tcl_ListObjIndex(interp, seg, k, &seg_elem);
-		if (ret)
-		    return ret;
-
-		ret=Tcl_ListObjIndex(interp, seg, k+1, &seg_val);
-		if (ret)
-		    return ret;
-
-		elem = Tcl_GetString(seg_elem);
+	    for (k=1; k<seg_argc; k += 2) {
+		elem = seg_argv[k];
+		sval = seg_argv[k+1];
 		switch (*elem) {
 		    case 'S':
-			Tcl_GetIntFromObj(interp, seg_val, &lsg->start);
+			(void)bu_opt_int(NULL, 1, &sval, (void *)&lsg->start);
 			break;
 		    case 'E':
-			Tcl_GetIntFromObj(interp, seg_val, &lsg->end);
+			(void)bu_opt_int(NULL, 1, &sval, (void *)&lsg->end);
 			break;
 		}
 	    }
 	    lsg->magic = CURVE_LSEG_MAGIC;
-	    crv->segment[j] = (genptr_t)lsg;
-	} else if (BU_STR_EQUAL(type, "bezier")) {
+	    crv->segment[j] = (void *)lsg;
+	} else if (BU_STR_EQUAL(seg_argv[0], "bezier")) {
 	    struct bezier_seg *bsg;
 	    int num_points;
 
 	    BU_ALLOC(bsg, struct bezier_seg);
-	    for (k=1; k<seg_len; k+= 2) {
-
-		ret=Tcl_ListObjIndex(interp, seg, k, &seg_elem);
-		if (ret)
-		    return ret;
-
-		ret=Tcl_ListObjIndex(interp, seg, k+1, &seg_val);
-		if (ret)
-		    return ret;
-
-		elem = Tcl_GetString(seg_elem);
+	    for (k=1; k<seg_argc; k+= 2) {
+		elem = seg_argv[k];
+		sval = seg_argv[k+1];
 		switch (*elem) {
 		    case 'D': /* degree */
-			Tcl_GetIntFromObj(interp, seg_val,
-					  &bsg->degree);
+			(void)bu_opt_int(NULL, 1, &sval, (void *)&bsg->degree);
 			break;
 		    case 'P': /* list of control points */
 			num_points = 0;
-			(void)tcl_obj_to_int_array(interp,
-						   seg_val, &bsg->ctl_points,
-						   &num_points);
-
+			(void)_rt_tcl_list_to_int_array(sval, &bsg->ctl_points, &num_points);
 			if (num_points != bsg->degree + 1) {
-			    Tcl_SetResult(interp, "ERROR: degree and number of control points disagree for a Bezier segment\n", TCL_STATIC);
-			    return TCL_ERROR;
+			    bu_vls_printf(logstr, "ERROR: degree and number of control points disagree for a Bezier segment\n");
+			    bu_free((char *)seg_argv, "free seg argv");
+			    bu_free((char *)seg_list, "free seg argv");
+			    return 1;
 			}
 		}
 	    }
 	    bsg->magic = CURVE_BEZIER_MAGIC;
-	    crv->segment[j] = (genptr_t)bsg;
-	} else if (BU_STR_EQUAL(type, "carc")) {
+	    crv->segment[j] = (void *)bsg;
+	} else if (BU_STR_EQUAL(seg_argv[0], "carc")) {
 	    struct carc_seg *csg;
-	    double tmp;
 
 	    BU_ALLOC(csg, struct carc_seg);
-	    for (k=1; k<seg_len; k += 2) {
-		ret=Tcl_ListObjIndex(interp, seg, k, &seg_elem);
-		if (ret)
-		    return ret;
-
-		ret=Tcl_ListObjIndex(interp, seg, k+1, &seg_val);
-		if (ret)
-		    return ret;
-
-		elem = Tcl_GetString(seg_elem);
+	    for (k=1; k<seg_argc; k += 2) {
+		elem = seg_argv[k];
+		sval = seg_argv[k+1];
 		switch (*elem) {
 		    case 'S':
-			Tcl_GetIntFromObj(interp, seg_val, &csg->start);
+			(void)bu_opt_int(NULL, 1, &sval, (void *)&csg->start);
 			break;
 		    case 'E':
-			Tcl_GetIntFromObj(interp, seg_val, &csg->end);
+			(void)bu_opt_int(NULL, 1, &sval, (void *)&csg->end);
 			break;
 		    case 'R':
-			Tcl_GetDoubleFromObj(interp, seg_val, &tmp);
-			csg->radius = tmp;
+			(void)bu_opt_fastf_t(NULL, 1, &sval, (void *)&csg->radius);
 			break;
 		    case 'L' :
-			Tcl_GetBooleanFromObj(interp, seg_val, &csg->center_is_left);
+			(void)bu_opt_bool(NULL, 1, &sval, (void *)&csg->center_is_left);
 			break;
 		    case 'O':
-			Tcl_GetBooleanFromObj(interp, seg_val, &csg->orientation);
+			(void)bu_opt_bool(NULL, 1, &sval, (void *)&csg->orientation);
 			break;
 		}
 	    }
 	    csg->magic = CURVE_CARC_MAGIC;
-	    crv->segment[j] = (genptr_t)csg;
-	} else if (BU_STR_EQUAL(type, "nurb")) {
+	    crv->segment[j] = (void *)csg;
+	} else if (BU_STR_EQUAL(seg_argv[0], "nurb")) {
 	    struct nurb_seg *nsg;
 
 	    BU_ALLOC(nsg, struct nurb_seg);
-	    for (k=1; k<seg_len; k += 2) {
-		ret=Tcl_ListObjIndex(interp, seg, k, &seg_elem);
-		if (ret)
-		    return ret;
-
-		ret=Tcl_ListObjIndex(interp, seg, k+1, &seg_val);
-		if (ret)
-		    return ret;
-
-		elem = Tcl_GetString(seg_elem);
+	    for (k=1; k<seg_argc; k += 2) {
+		elem = seg_argv[k];
+		sval = seg_argv[k+1];
 		switch (*elem) {
 		    case 'O':
-			Tcl_GetIntFromObj(interp, seg_val, &nsg->order);
+			(void)bu_opt_int(NULL, 1, &sval, (void *)&nsg->order);
 			break;
 		    case 'T':
-			Tcl_GetIntFromObj(interp, seg_val, &nsg->pt_type);
+			(void)bu_opt_int(NULL, 1, &sval, (void *)&nsg->pt_type);
 			break;
 		    case 'K':
-			tcl_obj_to_fastf_array(interp, seg_val, &nsg->k.knots, &nsg->k.k_size);
+			(void)_rt_tcl_list_to_fastf_array(sval, &nsg->k.knots, &nsg->k.k_size);
 			break;
 		    case 'P' :
-			tcl_obj_to_int_array(interp, seg_val, &nsg->ctl_points, &nsg->c_size);
+			(void)_rt_tcl_list_to_int_array(sval, &nsg->ctl_points, &nsg->c_size);
 			break;
 		    case 'W':
-			tcl_obj_to_fastf_array(interp, seg_val, &nsg->weights, &nsg->c_size);
+			(void)_rt_tcl_list_to_fastf_array(sval, &nsg->weights, &nsg->c_size);
 			break;
 		}
 	    }
 	    nsg->magic = CURVE_NURB_MAGIC;
-	    crv->segment[j] = (genptr_t)nsg;
+	    crv->segment[j] = (void *)nsg;
 	} else {
-	    Tcl_ResetResult(interp);
-	    Tcl_AppendResult(interp, "ERROR: Unrecognized segment type: ",
-			     Tcl_GetString(seg), (char *)NULL);
-	    return TCL_ERROR;
+	    bu_vls_sprintf(logstr, "ERROR: Unrecognized segment type: %s\n", seg_argv[0]);
+	    bu_free((char *)seg_argv, "free seg argv");
+	    bu_free((char *)seg_list, "free seg argv");
+	    return 1;
 	}
+
+	bu_free((char *)seg_argv, "free seg argv");
     }
 
-    return TCL_OK;
+    bu_free((char *)seg_list, "free seg argv");
+
+    return 0;
 }
 
 
@@ -2468,7 +2493,7 @@ rt_sketch_adjust(struct bu_vls *logstr, struct rt_db_internal *intern, int argc,
 	if (BU_STR_EQUAL(argv[0], "V")) {
 	    newval = skt->V;
 	    array_len = 3;
-	    if (tcl_list_to_fastf_array(brlcad_interp, argv[1], &newval, &array_len) !=
+	    if (_rt_tcl_list_to_fastf_array(argv[1], &newval, &array_len) !=
 		array_len) {
 		bu_vls_printf(logstr, "ERROR: Incorrect number of coordinates for vertex\n");
 		return BRLCAD_ERROR;
@@ -2476,7 +2501,7 @@ rt_sketch_adjust(struct bu_vls *logstr, struct rt_db_internal *intern, int argc,
 	} else if (BU_STR_EQUAL(argv[0], "A")) {
 	    newval = skt->u_vec;
 	    array_len = 3;
-	    if (tcl_list_to_fastf_array(brlcad_interp, argv[1], &newval, &array_len) !=
+	    if (_rt_tcl_list_to_fastf_array(argv[1], &newval, &array_len) !=
 		array_len) {
 		bu_vls_printf(logstr, "ERROR: Incorrect number of coordinates for vertex\n");
 		return BRLCAD_ERROR;
@@ -2484,7 +2509,7 @@ rt_sketch_adjust(struct bu_vls *logstr, struct rt_db_internal *intern, int argc,
 	} else if (BU_STR_EQUAL(argv[0], "B")) {
 	    newval = skt->v_vec;
 	    array_len = 3;
-	    if (tcl_list_to_fastf_array(brlcad_interp, argv[1], &newval, &array_len) !=
+	    if (_rt_tcl_list_to_fastf_array(argv[1], &newval, &array_len) !=
 		array_len) {
 		bu_vls_printf(logstr, "ERROR: Incorrect number of coordinates for vertex\n");
 		return BRLCAD_ERROR;
@@ -2509,7 +2534,7 @@ rt_sketch_adjust(struct bu_vls *logstr, struct rt_db_internal *intern, int argc,
 	    }
 
 	    len = 0;
-	    (void)tcl_list_to_fastf_array(brlcad_interp, dupstr, &new_verts, &len);
+	    (void)_rt_tcl_list_to_fastf_array(dupstr, &new_verts, &len);
 	    bu_free(dupstr, "sketch adjust strdup");
 
 	    if (len%2) {
@@ -2523,18 +2548,14 @@ rt_sketch_adjust(struct bu_vls *logstr, struct rt_db_internal *intern, int argc,
 	    skt->vert_count = len / 2;
 	} else if (BU_STR_EQUAL(argv[0], "SL")) {
 	    /* the entire segment list */
-	    Tcl_Obj *tmp;
 	    struct rt_curve *crv;
 
-	    /* create a Tcl object */
-	    tmp = Tcl_NewStringObj(argv[1], -1);
-
 	    crv = &skt->curve;
 	    crv->count = 0;
 	    crv->reverse = (int *)NULL;
-	    crv->segment = (genptr_t)NULL;
+	    crv->segment = (void **)NULL;
 
-	    if ((ret=get_tcl_curve(brlcad_interp, crv, tmp)) != TCL_OK)
+	    if ((ret=get_tcl_curve(logstr, crv, argv[1])) != 0)
 		return ret;
 	} else if (*argv[0] == 'V' && isdigit((int)*(argv[0]+1))) {
 	    /* changing a specific vertex */
@@ -2548,7 +2569,7 @@ rt_sketch_adjust(struct bu_vls *logstr, struct rt_db_internal *intern, int argc,
 		return BRLCAD_ERROR;
 	    }
 	    array_len = 2;
-	    if (tcl_list_to_fastf_array(brlcad_interp, argv[1], &new_vert, &array_len) != array_len) {
+	    if (_rt_tcl_list_to_fastf_array(argv[1], &new_vert, &array_len) != array_len) {
 		bu_vls_printf(logstr, "ERROR: Incorrect number of coordinates for vertex\n");
 		return BRLCAD_ERROR;
 	    }
@@ -2562,10 +2583,6 @@ rt_sketch_adjust(struct bu_vls *logstr, struct rt_db_internal *intern, int argc,
 }
 
 
-/**
- * R T _ S K E T C H _ P A R A M S
- *
- */
 int
 rt_sketch_params(struct pc_pc_set *UNUSED(ps), const struct rt_db_internal *ip)
 {
@@ -2620,7 +2637,12 @@ rt_curve_order_segments(struct rt_curve *crv)
 {
     int i, j, k;
     int count;
-    int start1, end1, start2, end2, start3, end3;
+    int start1 = 0;
+    int start2 = 0;
+    int start3 = 0;
+    int end1 = 0;
+    int end2 = 0;
+    int end3 = 0;
 
     count = crv->count;
     if (count < 2) {
@@ -2629,7 +2651,7 @@ rt_curve_order_segments(struct rt_curve *crv)
 
     for (j=1; j<count; j++) {
 	int tmp_reverse;
-	genptr_t tmp_seg;
+	void *tmp_seg;
 	int fixed=0;
 
 	i = j - 1;
@@ -2666,7 +2688,7 @@ rt_curve_order_segments(struct rt_curve *crv)
 	    if (end3 != end1)
 		continue;
 
-	    rt_curve_reverse_segment(crv->segment[k]);
+	    rt_curve_reverse_segment((uint32_t *)crv->segment[k]);
 
 	    if (k != j) {
 		/* exchange j and k segments */
