@@ -1,7 +1,7 @@
 /*                      S U B M O D E L . C
  * BRL-CAD
  *
- * Copyright (c) 2000-2013 United States Government as represented by
+ * Copyright (c) 2000-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This library is free software; you can redistribute it and/or
@@ -34,15 +34,15 @@
 #include "common.h"
 
 #include <stddef.h>
-#include <stdio.h>
 #include <math.h>
 #include <string.h>
 #include "bio.h"
 
+#include "bu/parallel.h"
 #include "vmath.h"
-#include "db.h"
+#include "rt/db4.h"
 #include "nmg.h"
-#include "rtgeom.h"
+#include "rt/geom.h"
 #include "raytrace.h"
 
 #define RT_SUBMODEL_O(m) bu_offsetof(struct rt_submodel_internal, m)
@@ -67,8 +67,6 @@ struct submodel_specific {
 
 
 /**
- * R T _ S U B M O D E L _ P R E P
- *
  * Given a pointer to a GED database record, and a transformation matrix,
  * determine if this is a valid SUBMODEL, and if so, precompute various
  * terms of the formula.
@@ -111,9 +109,6 @@ rt_submodel_prep(struct soltab *stp, struct rt_db_internal *ip, struct rt_i *rti
 	if ((sub_dbip = db_open(bu_vls_addr(&sip->file), DB_OPEN_READONLY)) == DBI_NULL)
 	    return -1;
 
-	/* Save the overhead of stat() calls on subsequent opens */
-	if (sub_dbip->dbi_mf) sub_dbip->dbi_mf->dont_restat = 1;
-
 	if (!db_is_directory_non_empty(sub_dbip)) {
 	    /* This is first open of db, build directory */
 	    if (db_dirbuild(sub_dbip) < 0) {
@@ -141,7 +136,7 @@ rt_submodel_prep(struct soltab *stp, struct rt_db_internal *ip, struct rt_i *rti
 
 	    bu_semaphore_release(RT_SEM_MODEL);
 
-	    if (RT_G_DEBUG & (DEBUG_DB|DEBUG_SOLIDS)) {
+	    if (RT_G_DEBUG & (RT_DEBUG_DB|RT_DEBUG_SOLIDS)) {
 		bu_log("rt_submodel_prep(%s): Re-used already prepped database %s, rtip=%p\n",
 		       stp->st_dp->d_namep,
 		       sub_dbip->dbi_filename,
@@ -159,7 +154,7 @@ rt_submodel_prep(struct soltab *stp, struct rt_db_internal *ip, struct rt_i *rti
 
     bu_semaphore_release(RT_SEM_MODEL);
 
-    if (RT_G_DEBUG & (DEBUG_DB|DEBUG_SOLIDS)) {
+    if (RT_G_DEBUG & (RT_DEBUG_DB|RT_DEBUG_SOLIDS)) {
 	bu_log("rt_submodel_prep(%s): Opened database %s\n",
 	       stp->st_dp->d_namep, sub_dbip->dbi_filename);
     }
@@ -212,16 +207,14 @@ rt_submodel_prep(struct soltab *stp, struct rt_db_internal *ip, struct rt_i *rti
     rt_prep_parallel(sub_rtip, 1);
 
     /* Ensure bu_ptbl rti_resources is full size.  Ptrs will be null */
-    if (BU_PTBL_LEN(&sub_rtip->rti_resources) < sub_rtip->rti_resources.blen) {
-	BU_PTBL_END(&sub_rtip->rti_resources) = sub_rtip->rti_resources.blen;
-    }
+    bu_ptbl_trunc(&sub_rtip->rti_resources, sub_rtip->rti_resources.blen);
 
     if (RT_G_DEBUG) rt_pr_cut_info(sub_rtip, stp->st_name);
 
  done:
     BU_GET(submodel, struct submodel_specific);
     submodel->magic = RT_SUBMODEL_SPECIFIC_MAGIC;
-    stp->st_specific = (genptr_t)submodel;
+    stp->st_specific = (void *)submodel;
 
     MAT_COPY(submodel->subm2m, sip->root2leaf);
     bn_mat_inv(submodel->m2subm, sip->root2leaf);
@@ -237,7 +230,7 @@ rt_submodel_prep(struct soltab *stp, struct rt_db_internal *ip, struct rt_i *rti
     VSCALE(radvec, diam, 0.5);
     stp->st_aradius = stp->st_bradius = MAGNITUDE(radvec);
 
-    if (RT_G_DEBUG & (DEBUG_DB|DEBUG_SOLIDS)) {
+    if (RT_G_DEBUG & (RT_DEBUG_DB|RT_DEBUG_SOLIDS)) {
 	bu_log("rt_submodel_prep(%s): finished loading database %s\n",
 	       stp->st_dp->d_namep, sub_dbip->dbi_filename);
     }
@@ -246,9 +239,6 @@ rt_submodel_prep(struct soltab *stp, struct rt_db_internal *ip, struct rt_i *rti
 }
 
 
-/**
- * R T _ S U B M O D E L _ P R I N T
- */
 void
 rt_submodel_print(const struct soltab *stp)
 {
@@ -261,19 +251,16 @@ rt_submodel_print(const struct soltab *stp)
     bn_mat_print("m2subm", submodel->m2subm);
 
     bu_log_indent_delta(4);
-    bu_log("submodel->rtip=x%x\n", submodel->rtip);
+    bu_log("submodel->rtip=%p\n", (void *)submodel->rtip);
 
     /* Loop through submodel's solid table printing them too. */
     RT_VISIT_ALL_SOLTABS_START(stp, submodel->rtip) {
 	rt_pr_soltab(stp);
-    } RT_VISIT_ALL_SOLTABS_END
-	  bu_log_indent_delta(-4);
+    } RT_VISIT_ALL_SOLTABS_END;
+    bu_log_indent_delta(-4);
 }
 
 
-/**
- * R T _ S U B M O D E L _ A _ M I S S
- */
 int
 rt_submodel_a_miss(struct application *ap)
 {
@@ -291,9 +278,6 @@ struct submodel_gobetween {
 };
 
 
-/**
- * R T _ S U B M O D E L _ A _ H I T
- */
 int
 rt_submodel_a_hit(struct application *ap, struct partition *PartHeadp, struct seg *UNUSED(segHeadp))
 {
@@ -357,7 +341,7 @@ rt_submodel_a_hit(struct application *ap, struct partition *PartHeadp, struct se
 	MAT4XSCALOR(up_segp->seg_out.hit_dist, submodel->subm2m, outseg->seg_out.hit_dist);
 	up_segp->seg_out.hit_dist -= gp->delta;
 
-	BU_ASSERT_DOUBLE(up_segp->seg_in.hit_dist, <=, up_segp->seg_out.hit_dist);
+	BU_ASSERT(up_segp->seg_in.hit_dist <= up_segp->seg_out.hit_dist);
 
 	/* Link to ray in upper model, not submodel */
 	up_segp->seg_in.hit_rayp = &up_ap->a_ray;
@@ -441,8 +425,6 @@ rt_submodel_a_hit(struct application *ap, struct partition *PartHeadp, struct se
 
 
 /**
- * R T _ S U B M O D E L _ S H O T
- *
  * Intersect a ray with a submodel.
  * If an intersection occurs, a struct seg will be acquired
  * and filled in.
@@ -462,7 +444,7 @@ rt_submodel_shot(struct soltab *stp, struct xray *rp, struct application *ap, st
     int code;
     struct bu_ptbl *restbl;
     struct resource *resp;
-    int cpu;
+    size_t cpu;
 
     RT_CK_SOLTAB(stp);
     RT_CK_RTI(ap->a_rt_i);
@@ -476,7 +458,7 @@ rt_submodel_shot(struct soltab *stp, struct xray *rp, struct application *ap, st
     sub_ap.a_rt_i = submodel->rtip;
     sub_ap.a_hit = rt_submodel_a_hit;
     sub_ap.a_miss = rt_submodel_a_miss;
-    sub_ap.a_uptr = (genptr_t)&gb;
+    sub_ap.a_uptr = (void *)&gb;
     sub_ap.a_purpose = "rt_submodel_shot";
 
     /* Ensure even # of accurate hits, for building good partitions */
@@ -492,7 +474,7 @@ rt_submodel_shot(struct soltab *stp, struct xray *rp, struct application *ap, st
      */
     restbl = &submodel->rtip->rti_resources;	/* a ptbl */
     cpu = ap->a_resource->re_cpu;
-    BU_ASSERT_LONG(cpu, <, BU_PTBL_END(restbl));
+    BU_ASSERT(cpu < BU_PTBL_LEN(restbl));
     if ((resp = (struct resource *)BU_PTBL_GET(restbl, cpu)) == NULL) {
 	/* First ray for this cpu for this submodel, alloc up */
 	BU_ALLOC(resp, struct resource);
@@ -526,8 +508,6 @@ rt_submodel_shot(struct soltab *stp, struct xray *rp, struct application *ap, st
 
 
 /**
- * R T _ S U B M O D E L _ N O R M
- *
  * Given ONE ray distance, return the normal and entry/exit point.
  */
 void
@@ -551,8 +531,6 @@ rt_submodel_norm(struct hit *hitp, struct soltab *stp, struct xray *rp)
 
 
 /**
- * R T _ S U B M O D E L _ C U R V E
- *
  * Return the curvature of the submodel.
  */
 void
@@ -574,8 +552,6 @@ rt_submodel_curve(struct curvature *cvp, struct hit *hitp, struct soltab *stp)
 
 
 /**
- * R T _ S U B M O D E L _ U V
- *
  * For a hit on the surface of an submodel, return the (u, v) coordinates
  * of the hit point, 0 <= u, v <= 1.
  * u = azimuth
@@ -596,9 +572,6 @@ rt_submodel_uv(struct application *ap, struct soltab *stp, struct hit *hitp, str
 }
 
 
-/**
- * R T _ S U B M O D E L _ F R E E
- */
 void
 rt_submodel_free(struct soltab *stp)
 {
@@ -631,21 +604,6 @@ rt_submodel_free(struct soltab *stp)
 }
 
 
-/**
- * R T _ S U B M O D E L _ C L A S S
- */
-int
-rt_submodel_class(const struct soltab *stp, const fastf_t *min, const fastf_t *max, const struct bn_tol *tol)
-{
-    if (stp) RT_CK_SOLTAB(stp);
-    if (tol) BN_CK_TOL(tol);
-    if (!min) return 0;
-    if (!max) return 0;
-
-    return 0;
-}
-
-
 struct goodies {
     struct db_i *dbip;
     struct bu_list *vheadp;
@@ -653,35 +611,31 @@ struct goodies {
 
 
 /**
- * R T _ S U B M O D E L _ W I R E F R A M E _ L E A F
- *
  * This routine must be prepared to run in parallel.
  * This routine should be generally exported for other uses.
  */
 HIDDEN union tree *
-rt_submodel_wireframe_leaf(struct db_tree_state *tsp, const struct db_full_path *pathp, struct rt_db_internal *ip, genptr_t UNUSED(client_data))
+rt_submodel_wireframe_leaf(struct db_tree_state *tsp, const struct db_full_path *pathp, struct rt_db_internal *ip, void *UNUSED(client_data))
 {
     union tree *curtree;
     struct goodies *gp;
     int ret;
 
-    RT_CK_TESS_TOL(tsp->ts_ttol);
+    BG_CK_TESS_TOL(tsp->ts_ttol);
     BN_CK_TOL(tsp->ts_tol);
     RT_CK_DB_INTERNAL(ip);
     RT_CK_RESOURCE(tsp->ts_resp);
 
     gp = (struct goodies *)tsp->ts_m;	/* hack */
-    if (gp) RT_CK_DBI(gp->dbip);
+    if (gp)
+	RT_CK_DBI(gp->dbip);
 
-    /* NON-PARALLEL access to vlist pointed to by vheadp is not semaphored */
-    if (bu_is_parallel()) bu_bomb("rt_submodel_wireframe_leaf() non-parallel code\n");
-
-    if (RT_G_DEBUG&DEBUG_TREEWALK) {
+    if (RT_G_DEBUG&RT_DEBUG_TREEWALK) {
 	char *sofar = db_path_to_string(pathp);
 
 	bu_log("rt_submodel_wireframe_leaf(%s) path=%s\n",
 	       ip->idb_meth->ft_name, sofar);
-	bu_free((genptr_t)sofar, "path string");
+	bu_free((void *)sofar, "path string");
     }
 
     ret = -1;
@@ -696,7 +650,8 @@ rt_submodel_wireframe_leaf(struct db_tree_state *tsp, const struct db_full_path
     }
 
     /* Indicate success by returning something other than TREE_NULL */
-    RT_GET_TREE(curtree, tsp->ts_resp);
+    BU_GET(curtree, union tree);
+    RT_TREE_INIT(curtree);
     curtree->tr_op = OP_NOP;
 
     return curtree;
@@ -704,8 +659,6 @@ rt_submodel_wireframe_leaf(struct db_tree_state *tsp, const struct db_full_path
 
 
 /**
- * R T _ S U B M O D E L _ P L O T
- *
  * Not unlike mged/dodraw.c drawtrees()
  *
  * Note:  The submodel will be drawn entirely in one color
@@ -714,7 +667,7 @@ rt_submodel_wireframe_leaf(struct db_tree_state *tsp, const struct db_full_path
  * which by definition, is all one color.
  */
 int
-rt_submodel_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct rt_tess_tol *ttol, const struct bn_tol *tol, const struct rt_view_info *UNUSED(info))
+rt_submodel_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct bg_tess_tol *ttol, const struct bn_tol *tol, const struct rt_view_info *UNUSED(info))
 {
     struct rt_submodel_internal *sip;
     struct db_tree_state state;
@@ -767,7 +720,7 @@ rt_submodel_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct
 		       0,			/* take all regions */
 		       NULL,			/* rt_submodel_wireframe_region_end */
 		       rt_submodel_wireframe_leaf,
-		       (genptr_t)NULL);
+		       (void *)NULL);
 
     if (ret < 0) bu_log("rt_submodel_plot() db_walk_tree(%s) failure\n", bu_vls_addr(&sip->treetop));
     if (bu_vls_strlen(&sip->file) != 0)
@@ -777,14 +730,12 @@ rt_submodel_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct
 
 
 /**
- * R T _ S U B M O D E L _ T E S S
- *
  * Returns -
  * -1 failure
  * 0 OK.  *r points to nmgregion that holds this tessellation.
  */
 int
-rt_submodel_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, const struct rt_tess_tol *UNUSED(ttol), const struct bn_tol *UNUSED(tol))
+rt_submodel_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, const struct bg_tess_tol *UNUSED(ttol), const struct bn_tol *UNUSED(tol))
 {
     struct rt_submodel_internal *sip;
 
@@ -800,8 +751,6 @@ rt_submodel_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *i
 
 
 /**
- * R T _ S U B M O D E L _ I M P O R T
- *
  * Import an SUBMODEL from the database format to the internal format.
  * Apply modeling transformations as well.
  */
@@ -837,12 +786,12 @@ rt_submodel_import4(struct rt_db_internal *ip, const struct bu_external *ep, con
 
     bu_vls_strcpy(&str, rp->ss.ss_args);
 
-    if (bu_struct_parse(&str, rt_submodel_parse, (char *)sip) < 0) {
+    if (bu_struct_parse(&str, rt_submodel_parse, (char *)sip, NULL) < 0) {
 	bu_vls_free(&str);
     fail:
 	bu_free((char *)sip, "rt_submodel_import4: sip");
 	ip->idb_type = ID_NULL;
-	ip->idb_ptr = (genptr_t)NULL;
+	ip->idb_ptr = (void *)NULL;
 	return -2;
     }
     bu_vls_free(&str);
@@ -858,8 +807,6 @@ rt_submodel_import4(struct rt_db_internal *ip, const struct bu_external *ep, con
 
 
 /**
- * R T _ S U B M O D E L _ E X P O R T
- *
  * The name is added by the caller, in the usual place.
  */
 int
@@ -878,7 +825,7 @@ rt_submodel_export4(struct bu_external *ep, const struct rt_db_internal *ip, dou
 
     BU_CK_EXTERNAL(ep);
     ep->ext_nbytes = sizeof(union record)*DB_SS_NGRAN;
-    ep->ext_buf = bu_calloc(1, ep->ext_nbytes, "submodel external");
+    ep->ext_buf = (uint8_t *)bu_calloc(1, ep->ext_nbytes, "submodel external");
     rec = (union record *)ep->ext_buf;
 
     bu_vls_struct_print(&str, rt_submodel_parse, (char *)sip);
@@ -893,8 +840,6 @@ rt_submodel_export4(struct bu_external *ep, const struct rt_db_internal *ip, dou
 
 
 /**
- * R T _ S U B M O D E L _ I M P O R T 5
- *
  * Import an SUBMODEL from the database format to the internal format.
  * Apply modeling transformations as well.
  */
@@ -917,19 +862,21 @@ rt_submodel_import5(struct rt_db_internal *ip, const struct bu_external *ep, con
 
     sip = (struct rt_submodel_internal *)ip->idb_ptr;
     sip->magic = RT_SUBMODEL_INTERNAL_MAGIC;
-    sip->dbip = dbip;
-
+    BU_VLS_INIT(&sip->file);
+    BU_VLS_INIT(&sip->treetop);
+    sip->meth = 0;
     if (mat == NULL) mat = bn_mat_identity;
     MAT_COPY(sip->root2leaf, mat);
+    sip->dbip = dbip;
 
     bu_vls_strncpy(&str, (const char *)ep->ext_buf, ep->ext_nbytes);
 
-    if (bu_struct_parse(&str, rt_submodel_parse, (char *)sip) < 0) {
+    if (bu_struct_parse(&str, rt_submodel_parse, (char *)sip, NULL) < 0) {
 	bu_vls_free(&str);
     fail:
 	bu_free((char *)sip, "rt_submodel_import4: sip");
 	ip->idb_type = ID_NULL;
-	ip->idb_ptr = (genptr_t)NULL;
+	ip->idb_ptr = (void *)NULL;
 	return -2;
     }
     bu_vls_free(&str);
@@ -945,8 +892,6 @@ rt_submodel_import5(struct rt_db_internal *ip, const struct bu_external *ep, con
 
 
 /**
- * R T _ S U B M O D E L _ E X P O R T 5
- *
  * The name is added by the caller, in the usual place.
  */
 int
@@ -965,7 +910,7 @@ rt_submodel_export5(struct bu_external *ep, const struct rt_db_internal *ip, dou
     BU_CK_EXTERNAL(ep);
     bu_vls_struct_print(&str, rt_submodel_parse, (char *)sip);
     ep->ext_nbytes = bu_vls_strlen(&str);
-    ep->ext_buf = bu_calloc(1, ep->ext_nbytes, "submodel external");
+    ep->ext_buf = (uint8_t *)bu_calloc(1, ep->ext_nbytes, "submodel external");
 
     bu_strlcpy((char *)ep->ext_buf, bu_vls_addr(&str), ep->ext_nbytes);
     bu_vls_free(&str);
@@ -975,8 +920,6 @@ rt_submodel_export5(struct bu_external *ep, const struct rt_db_internal *ip, dou
 
 
 /**
- * R T _ S U B M O D E L _ D E S C R I B E
- *
  * Make human-readable formatted presentation of this solid.
  * First line describes type of solid.
  * Additional lines are indented one tab, and give parameter values.
@@ -1002,8 +945,6 @@ rt_submodel_describe(struct bu_vls *str, const struct rt_db_internal *ip, int ve
 
 
 /**
- * R T _ S U B M O D E L _ I F R E E
- *
  * Free the storage associated with the rt_db_internal version of this solid.
  */
 void
@@ -1016,16 +957,14 @@ rt_submodel_ifree(struct rt_db_internal *ip)
     sip = (struct rt_submodel_internal *)ip->idb_ptr;
     RT_SUBMODEL_CK_MAGIC(sip);
     sip->magic = 0;			/* sanity */
-
-    bu_free((genptr_t)sip, "submodel ifree");
-    ip->idb_ptr = GENPTR_NULL;	/* sanity */
+    bu_vls_free(&sip->file);
+    bu_vls_free(&sip->treetop);
+    sip->dbip = DBI_NULL;
+    bu_free((void *)sip, "submodel ifree");
+    ip->idb_ptr = ((void *)0);	/* sanity */
 }
 
 
-/**
- * R T _ S U B M O D E L _ P A R A M S
- *
- */
 int
 rt_submodel_params(struct pc_pc_set *UNUSED(ps), const struct rt_db_internal *ip)
 {
