@@ -1,7 +1,7 @@
 /*                         T O R . C
  * BRL-CAD
  *
- * Copyright (c) 1985-2013 United States Government as represented by
+ * Copyright (c) 1985-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This library is free software; you can redistribute it and/or
@@ -111,15 +111,15 @@
 #include "common.h"
 
 #include <stddef.h>
-#include <stdio.h>
 #include <string.h>
 #include <math.h>
 #include "bio.h"
 
+#include "bu/cv.h"
 #include "vmath.h"
-#include "db.h"
+#include "rt/db4.h"
 #include "nmg.h"
-#include "rtgeom.h"
+#include "rt/geom.h"
 #include "raytrace.h"
 
 #include "../../librt_private.h"
@@ -154,9 +154,38 @@ struct tor_specific {
     mat_t tor_invR;	/* invRot(vect') */
 };
 
+#ifdef USE_OPENCL
+/* largest data members first */
+struct clt_tor_specific {
+    cl_double tor_alpha;	/* 0 < (R2/R1) <= 1 */
+    cl_double tor_r1;		/* for inverse scaling of k values. */
+    cl_double tor_V[3];		/* Vector to center of torus */
+    cl_double tor_SoR[16];	/* Scale(Rot(vect)) */
+    cl_double tor_invR[16];	/* invRot(vect') */
+};
+
+size_t
+clt_tor_pack(struct bu_pool *pool, struct soltab *stp)
+{
+    struct tor_specific *tor =
+        (struct tor_specific *)stp->st_specific;
+    struct clt_tor_specific *args;
+
+    const size_t size = sizeof(*args);
+    args = (struct clt_tor_specific*)bu_pool_alloc(pool, 1, size);
+
+    args->tor_alpha = tor->tor_alpha;
+    args->tor_r1 = tor->tor_r1;
+    VMOVE(args->tor_V, tor->tor_V);
+    MAT_COPY(args->tor_SoR, tor->tor_SoR);
+    MAT_COPY(args->tor_invR, tor->tor_invR);
+    return size;
+}
+
+#endif /* USE_OPENCL */
+
+
 /**
- * R T _ T O R _ B B O X
- *
  * Compute the bounding RPP for a circular torus.
  */
 int
@@ -206,8 +235,6 @@ rt_tor_bbox(struct rt_db_internal *ip, point_t *min, point_t *max, const struct
 
 
 /**
- * R T _ T O R _ P R E P
- *
  * Given a pointer to a GED database record, and a transformation
  * matrix, determine if this is a valid torus, and if so, precompute
  * various terms of the formula.
@@ -269,7 +296,7 @@ rt_tor_prep(struct soltab *stp, struct rt_db_internal *ip, struct rt_i *rtip)
 
     /* Solid is OK, compute constant terms now */
     BU_GET(tor, struct tor_specific);
-    stp->st_specific = (genptr_t)tor;
+    stp->st_specific = (void *)tor;
 
     tor->tor_r1 = tip->r_a;
     tor->tor_r2 = tip->r_h;
@@ -301,9 +328,6 @@ rt_tor_prep(struct soltab *stp, struct rt_db_internal *ip, struct rt_i *rtip)
 }
 
 
-/**
- * R T _ T O R _ P R I N T
- */
 void
 rt_tor_print(register const struct soltab *stp)
 {
@@ -321,8 +345,6 @@ rt_tor_print(register const struct soltab *stp)
 
 
 /**
- * R T _ T O R _ S H O T
- *
  * Intersect a ray with an torus, where all constant terms have been
  * precomputed by rt_tor_prep().  If an intersection occurs, one or
  * two struct seg(s) will be acquired and filled in.
@@ -496,7 +518,7 @@ rt_tor_shot(struct soltab *stp, register struct xray *rp, struct application *ap
 		register short n;
 		register short lim;
 
-		/* Inline rt_pt_sort().  Sorts k[] into descending order. */
+		/* Inline rt_pnt_sort().  Sorts k[] into descending order. */
 		for (lim = i-1; lim > 0; lim--) {
 		    for (n = 0; n < lim; n++) {
 			fastf_t u;
@@ -542,8 +564,6 @@ rt_tor_shot(struct soltab *stp, register struct xray *rp, struct application *ap
 
 #define RT_TOR_SEG_MISS(SEG)		(SEG).seg_stp=(struct soltab *) 0;
 /**
- * R T _ T O R _ V S H O T
- *
  * This is the Becker vector version
  */
 void
@@ -654,7 +674,7 @@ rt_tor_vshot(struct soltab **stp, struct xray **rp, struct seg *segp, int n, str
 	X2_Y2.cf[2] *= 4.0;
 
 	/* Inline expansion of (void) bn_poly_sub(&C, &Asqr, &X2_Y2) */
-	/* offset is know to be 2 */
+	/* offset is known to be 2 */
 	C[i].dgr	= 4;
 	C[i].cf[0] = Asqr.cf[0];
 	C[i].cf[1] = Asqr.cf[1];
@@ -664,7 +684,7 @@ rt_tor_vshot(struct soltab **stp, struct xray **rp, struct seg *segp, int n, str
     }
 
     /* Unfortunately finding the 4th order roots are too ugly to
-     * inline.
+     * expand the root solving manually.
      */
     for (i = 0; i < n; i++) {
 	if (segp[i].seg_stp == 0) continue;	/* Skip */
@@ -767,7 +787,7 @@ rt_tor_vshot(struct soltab **stp, struct xray **rp, struct seg *segp, int n, str
 	tor = (struct tor_specific *)stp[i]->st_specific;
 
 	/* Sort most distant to least distant. */
-	rt_pt_sort(C[i].cf, 4);
+	rt_pnt_sort(C[i].cf, 4);
 	/* Now, t[0] > t[npts-1] */
 
 	/* segp[i].seg_in.hit_normal holds dprime */
@@ -791,8 +811,6 @@ rt_tor_vshot(struct soltab **stp, struct xray **rp, struct seg *segp, int n, str
 
 
 /**
- * R T _ T O R _ N O R M
- *
  * Compute the normal to the torus, given a point on the UNIT TORUS
  * centered at the origin on the X-Y plane.  The gradient of the torus
  * at that point is in fact the normal vector, which will have to be
@@ -842,8 +860,6 @@ rt_tor_norm(register struct hit *hitp, struct soltab *stp, register struct xray
 
 
 /**
- * R T _ T O R _ C U R V E
- *
  * Return the curvature of the torus.
  */
 void
@@ -889,9 +905,6 @@ rt_tor_curve(register struct curvature *cvp, register struct hit *hitp, struct s
 }
 
 
-/**
- * R T _ T O R _ U V
- */
 void
 rt_tor_uv(struct application *ap, struct soltab *stp, struct hit *hitp, struct uvcoord *uvp)
 {
@@ -912,20 +925,17 @@ rt_tor_uv(struct application *ap, struct soltab *stp, struct hit *hitp, struct u
     /*
      * -pi/2 <= atan2(x, y) <= pi/2
      */
-    uvp->uv_u = atan2(pprime[Y], pprime[X]) * bn_inv2pi + 0.5;
+    uvp->uv_u = atan2(pprime[Y], pprime[X]) * M_1_2PI + 0.5;
 
     VSET(work, pprime[X], pprime[Y], 0.0);
     VUNITIZE(work);
     VSUB2(pprime2, pprime, work);
     VUNITIZE(pprime2);
     costheta = VDOT(pprime2, work);
-    uvp->uv_v = atan2(pprime2[Z], costheta) * bn_inv2pi + 0.5;
+    uvp->uv_v = atan2(pprime2[Z], costheta) * M_1_2PI + 0.5;
 }
 
 
-/**
- * R T _ T O R _ F R E E
- */
 void
 rt_tor_free(struct soltab *stp)
 {
@@ -936,16 +946,7 @@ rt_tor_free(struct soltab *stp)
 }
 
 
-int
-rt_tor_class(void)
-{
-    return 0;
-}
-
-
 /**
- * R T _ N U M _ C I R C U L A R _ S E G M E N T S
- *
  * Given a circle with a specified radius, determine the minimum
  * number of straight line segments that the circle can be
  * approximated with, while still meeting the given maximum
@@ -996,7 +997,7 @@ rt_num_circular_segments(double maxerr, double radius)
 	 */
 	return 360*10;
     }
-    n = (bn_pi / half_theta) + 0.99;
+    n = (M_PI / half_theta) + 0.99;
 
     /* Impose the limits again */
     if (n <= 6) return 6;
@@ -1013,7 +1014,7 @@ tor_ellipse_points(
     fastf_t avg_radius, circumference;
 
     avg_radius = (MAGNITUDE(ellipse_A) + MAGNITUDE(ellipse_B)) / 2.0;
-    circumference = bn_twopi * avg_radius;
+    circumference = M_2PI * avg_radius;
 
     return circumference / info->point_spacing;
 }
@@ -1087,7 +1088,7 @@ rt_tor_adaptive_plot(struct rt_db_internal *ip, const struct rt_view_info *info)
 	num_ellipses = 3;
     }
 
-    radian_step = bn_twopi / num_ellipses;
+    radian_step = M_2PI / num_ellipses;
     radian = 0;
     for (i = 0; i < num_ellipses; ++i) {
 	ellipse_point_at_radian(center, tor->v, tor_a, tor_b, radian);
@@ -1105,15 +1106,13 @@ rt_tor_adaptive_plot(struct rt_db_internal *ip, const struct rt_view_info *info)
 }
 
 /**
- * R T _ T O R _ P L O T
- *
  * The TORUS has the following input fields:
  * ti.v V from origin to center
  * ti.h Radius Vector, Normal to plane of torus
  * ti.a, ti.b perpendicular, to CENTER of torus (for top, bottom)
  */
 int
-rt_tor_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct rt_tess_tol *ttol, const struct bn_tol *UNUSED(tol), const struct rt_view_info *UNUSED(info))
+rt_tor_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct bg_tess_tol *ttol, const struct bn_tol *UNUSED(tol), const struct rt_view_info *UNUSED(info))
 {
     fastf_t alpha;
     fastf_t beta;
@@ -1176,7 +1175,7 @@ rt_tor_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct rt_te
      */
     if (ttol->norm > 0.0) {
 	register int nseg;
-	nseg = (bn_pi / ttol->norm) + 0.99;
+	nseg = (M_PI / ttol->norm) + 0.99;
 	if (nseg > nlen) nlen = nseg;
 	if (nseg > nw) nw = nseg;
     }
@@ -1191,7 +1190,7 @@ rt_tor_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct rt_te
 #define TOR_NORM_A(ww, ll)	(&norms[TOR_PT(ww, ll)*3])
 
     for (len = 0; len < nlen; len++) {
-	beta = bn_twopi * len / nlen;
+	beta = M_2PI * len / nlen;
 	cos_beta = cos(beta);
 	sin_beta = sin(beta);
 	/* G always points out to rim, along radius vector */
@@ -1199,7 +1198,7 @@ rt_tor_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct rt_te
 	/* We assume that |radius| = |A|.  Circular */
 	VSCALE(G, radius, dist_to_rim);
 	for (w = 0; w < nw; w++) {
-	    alpha = bn_twopi * w / nw;
+	    alpha = M_2PI * w / nw;
 	    cos_alpha = cos(alpha);
 	    sin_alpha = sin(alpha);
 	    VCOMB2(edge, cos_alpha, G, sin_alpha*tip->r_h, tip->h);
@@ -1229,11 +1228,8 @@ rt_tor_plot(struct bu_list *vhead, struct rt_db_internal *ip, const struct rt_te
 }
 
 
-/**
- * R T _ T O R _ T E S S
- */
 int
-rt_tor_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, const struct rt_tess_tol *ttol, const struct bn_tol *tol)
+rt_tor_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, const struct bg_tess_tol *ttol, const struct bn_tol *tol)
 {
     fastf_t alpha;
     fastf_t beta;
@@ -1300,7 +1296,7 @@ rt_tor_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
      */
     if (ttol->norm > 0.0) {
 	register int nseg;
-	nseg = (bn_pi / ttol->norm) + 0.99;
+	nseg = (M_PI / ttol->norm) + 0.99;
 	if (nseg > nlen) nlen = nseg;
 	if (nseg > nw) nw = nseg;
     }
@@ -1312,7 +1308,7 @@ rt_tor_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
     norms = (fastf_t *)bu_malloc(nw * nlen * sizeof(vect_t), "rt_tor_tess: norms[]");
 
     for (len = 0; len < nlen; len++) {
-	beta = bn_twopi * len / nlen;
+	beta = M_2PI * len / nlen;
 	cos_beta = cos(beta);
 	sin_beta = sin(beta);
 	/* G always points out to rim, along radius vector */
@@ -1320,7 +1316,7 @@ rt_tor_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
 	/* We assume that |radius| = |A|.  Circular */
 	VSCALE(G, radius, dist_to_rim);
 	for (w = 0; w < nw; w++) {
-	    alpha = bn_twopi * w / nw;
+	    alpha = M_2PI * w / nw;
 	    cos_alpha = cos(alpha);
 	    sin_alpha = sin(alpha);
 	    VCOMB2(edge, cos_alpha, G, sin_alpha*tip->r_h, tip->h);
@@ -1408,8 +1404,6 @@ rt_tor_tess(struct nmgregion **r, struct model *m, struct rt_db_internal *ip, co
 
 
 /**
- * R T _ T O R _ I M P O R T
- *
  * Import a torus from the database format to the internal format.
  * Apply modeling transformations at the same time.
  */
@@ -1474,9 +1468,6 @@ rt_tor_import4(struct rt_db_internal *ip, const struct bu_external *ep, register
 }
 
 
-/**
- * R T _ T O R _ E X P O R T 5
- */
 int
 rt_tor_export5(struct bu_external *ep, const struct rt_db_internal *ip, double local2mm, const struct db_i *dbip)
 {
@@ -1494,7 +1485,7 @@ rt_tor_export5(struct bu_external *ep, const struct rt_db_internal *ip, double l
 
     BU_CK_EXTERNAL(ep);
     ep->ext_nbytes = SIZEOF_NETWORK_DOUBLE * (2*3+2);
-    ep->ext_buf = (genptr_t)bu_malloc(ep->ext_nbytes, "tor external");
+    ep->ext_buf = (uint8_t *)bu_malloc(ep->ext_nbytes, "tor external");
 
     /* scale values into local buffer */
     VSCALE(&vec[0*3], tip->v, local2mm);
@@ -1510,8 +1501,6 @@ rt_tor_export5(struct bu_external *ep, const struct rt_db_internal *ip, double l
 
 
 /**
- * R T _ T O R _ E X P O R T
- *
  * The name will be added by the caller.
  */
 int
@@ -1534,7 +1523,7 @@ rt_tor_export4(struct bu_external *ep, const struct rt_db_internal *ip, double l
 
     BU_CK_EXTERNAL(ep);
     ep->ext_nbytes = sizeof(union record);
-    ep->ext_buf = (genptr_t)bu_calloc(1, ep->ext_nbytes, "tor external");
+    ep->ext_buf = (uint8_t *)bu_calloc(1, ep->ext_nbytes, "tor external");
     rec = (union record *)ep->ext_buf;
 
     rec->s.s_id = ID_SOLID;
@@ -1598,8 +1587,6 @@ rt_tor_export4(struct bu_external *ep, const struct rt_db_internal *ip, double l
 
 
 /**
- * R T _ T O R _ I M P O R T 5
- *
  * Taken from the database record:
  * v vertex (point) of center of torus.
  * h unit vector in the normal direction of the torus
@@ -1627,7 +1614,7 @@ rt_tor_import5(struct rt_db_internal *ip, const struct bu_external *ep, register
     if (dbip) RT_CK_DBI(dbip);
 
     BU_CK_EXTERNAL(ep);
-    BU_ASSERT_LONG(ep->ext_nbytes, ==, SIZEOF_NETWORK_DOUBLE * (2*3+2));
+    BU_ASSERT(ep->ext_nbytes == SIZEOF_NETWORK_DOUBLE * (2*3+2));
 
     RT_CK_DB_INTERNAL(ip);
 
@@ -1668,8 +1655,6 @@ rt_tor_import5(struct rt_db_internal *ip, const struct bu_external *ep, register
 
 
 /**
- * R T _ T O R _ D E S C R I B E
- *
  * Make human-readable formatted presentation of this solid.  First
  * line describes type of solid.  Additional lines are indented one
  * tab, and give parameter values.
@@ -1725,8 +1710,6 @@ rt_tor_describe(struct bu_vls *str, const struct rt_db_internal *ip, int verbose
 
 
 /**
- * R T _ T O R _ I F R E E
- *
  * Free the storage associated with the rt_db_internal version of this
  * solid.
  */
@@ -1741,14 +1724,10 @@ rt_tor_ifree(struct rt_db_internal *ip)
     RT_TOR_CK_MAGIC(tip);
 
     bu_free((char *)tip, "rt_tor_internal");
-    ip->idb_ptr = GENPTR_NULL;	/* sanity */
+    ip->idb_ptr = ((void *)0);	/* sanity */
 }
 
 
-/**
- * R T _ T O R _ P A R A M S
- *
- */
 int
 rt_tor_params(struct pc_pc_set *UNUSED(ps), const struct rt_db_internal *ip)
 {
@@ -1758,10 +1737,6 @@ rt_tor_params(struct pc_pc_set *UNUSED(ps), const struct rt_db_internal *ip)
 }
 
 
-/**
- * R T _ T O R _ S U R F A C E _ A R E A
- *
- */
 void
 rt_tor_surf_area(fastf_t *area, const struct rt_db_internal *ip)
 {
@@ -1774,10 +1749,6 @@ rt_tor_surf_area(fastf_t *area, const struct rt_db_internal *ip)
 }
 
 
-/**
- * R T _ T O R _ V O L U M E
- *
- */
 void
 rt_tor_volume(fastf_t *vol, const struct rt_db_internal *ip)
 {
@@ -1787,10 +1758,6 @@ rt_tor_volume(fastf_t *vol, const struct rt_db_internal *ip)
 }
 
 
-/**
- * R T _ T O R _ C E N T R O I D
- *
- */
 void
 rt_tor_centroid(point_t *cent, const struct rt_db_internal *ip)
 {
@@ -1799,7 +1766,6 @@ rt_tor_centroid(point_t *cent, const struct rt_db_internal *ip)
     VMOVE(*cent,tip->v);
 }
 
-
 /*
  * Local Variables:
  * mode: C
