@@ -1,7 +1,7 @@
 /*                         S H O O T . C
  * BRL-CAD
  *
- * Copyright (c) 2000-2013 United States Government as represented by
+ * Copyright (c) 2000-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This library is free software; you can redistribute it and/or
@@ -21,15 +21,14 @@
 
 #include "common.h"
 
-#include <stdio.h>
 #include <math.h>
 #include <string.h>
 #include "bio.h"
 
 #include "vmath.h"
-#include "bu.h"
+
 #include "raytrace.h"
-#include "plot3.h"
+#include "bn/plot3.h"
 
 
 #define V3PT_DEPARTING_RPP(_step, _lo, _hi, _pt)			\
@@ -53,7 +52,7 @@ rt_res_pieces_init(struct resource *resp, struct rt_i *rtip)
     RT_CK_RESOURCE(resp);
     RT_CK_RTI(rtip);
 
-    psptab = bu_calloc(rtip->rti_nsolids_with_pieces, sizeof(struct rt_piecestate), "re_pieces[]");
+    psptab = (struct rt_piecestate *)bu_calloc(rtip->rti_nsolids_with_pieces, sizeof(struct rt_piecestate), "re_pieces[]");
     resp->re_pieces = psptab;
 
     RT_VISIT_ALL_SOLTABS_START(stp, rtip) {
@@ -127,48 +126,6 @@ rt_res_pieces_clean(struct resource *resp, struct rt_i *rtip)
 }
 
 
-/**
- * Along the given axis, find which NUgrid cell this value lies in.
- * Use method of binary subdivision.
- */
-int
-rt_find_nugrid(const struct nugridnode *nugnp, int axis, fastf_t val)
-{
-    int min;
-    int max;
-    int lim = nugnp->nu_cells_per_axis[axis]-1;
-    int cur;
-
-    if (val < nugnp->nu_axis[axis][0].nu_spos ||
-	val > nugnp->nu_axis[axis][lim].nu_epos)
-	return -1;
-
-    min = 0;
-    max = lim;
-again:
-    cur = (min + max) / 2;
-
-    if (cur <= 0) return 0;
-    if (cur >= lim) return lim;
-
-    if (val < nugnp->nu_axis[axis][cur].nu_spos) {
-	max = cur;
-	goto again;
-    }
-    if (val > nugnp->nu_axis[axis][cur+1].nu_epos) {
-	min = cur+1;
-	goto again;
-    }
-    if (val < nugnp->nu_axis[axis][cur].nu_epos)
-	return cur;
-    else
-	return cur+1;
-}
-
-
-/**
- *
- */
 const union cutter *
 rt_advance_to_next_cell(register struct rt_shootray_status *ssp)
 {
@@ -226,148 +183,6 @@ rt_advance_to_next_cell(register struct rt_shootray_status *ssp)
 
     top:
 	switch (curcut->cut_type) {
-	    case CUT_NUGRIDNODE: {
-		/*********************************************************
-		 * NOTE: This portion implements Gigante's non-uniform 3-D
-		 * space grid/mesh discretization.
-		 *********************************************************/
-		register int out_axis;
-		register const struct nu_axis **nu_axis =
-		    (const struct nu_axis **)&curcut->nugn.nu_axis[0];
-		register const int *nu_stepsize =
-		    &curcut->nugn.nu_stepsize[0];
-		register const int *nu_cells_per_axis =
-		    &curcut->nugn.nu_cells_per_axis[0];
-		register const union cutter *nu_grid =
-		    curcut->nugn.nu_grid;
-
-		if (ssp->lastcell == CUTTER_NULL || ssp->lastcell->cut_type != CUT_NUGRIDNODE) {
-		    /* We have just started into this NUgrid.  We must
-		     * find our location and set up the NUgrid traversal
-		     * state variables.
-		     */
-		    register int x, y, z;
-
-		    px = ap->a_ray.r_pt[X] + t0*ap->a_ray.r_dir[X];
-		    py = ap->a_ray.r_pt[Y] + t0*ap->a_ray.r_dir[Y];
-		    pz = ap->a_ray.r_pt[Z] + t0*ap->a_ray.r_dir[Z];
-
-		    /* Must find cell that contains newray.r_pt.  We do
-		     * this by binary subdivision.  If any are out of
-		     * bounds, we have left the NUgrid and will pop a
-		     * level off the stack in the outer loop (if
-		     * applicable).
-		     */
-		    x = rt_find_nugrid(&curcut->nugn, X, px);
-		    if (x<0) break;
-		    y = rt_find_nugrid(&curcut->nugn, Y, py);
-		    if (y<0) break;
-		    z = rt_find_nugrid(&curcut->nugn, Z, pz);
-		    if (z<0) break;
-
-		    cutp = &nu_grid[z*nu_stepsize[Z] +
-				    y*nu_stepsize[Y] +
-				    x*nu_stepsize[X]];
-
-		    ssp->igrid[X] = x;
-		    ssp->igrid[Y] = y;
-		    ssp->igrid[Z] = z;
-
-		    NUGRID_T_SETUP(X, px, x);
-		    NUGRID_T_SETUP(Y, py, y);
-		    NUGRID_T_SETUP(Z, pz, z);
-		} else {
-		    /* Advance from previous cell to next cell.  Take next
-		     * step, finding ray entry distance.
-		     */
-		    cutp = ssp->lastcell;
-		    out_axis = ssp->out_axis;
-
-		    /* We may be simply advancing to the next box in the
-		     * *same* NUgrid cell (if, for instance, the NUgrid
-		     * cell is a cutnode with boxnode leaves).  So if t0
-		     * hasn't advanced past the end of the box, advance a
-		     * tiny bit (less than rt_ct_optim makes boxnodes) and
-		     * be handled by tree-traversing code below.
-		     */
-
-		    if (cutp->cut_type == CUT_CUTNODE &&
-			t0 + ssp->ap->a_rt_i->rti_tol.dist < ssp->tv[out_axis]) {
-			t0 += ssp->ap->a_rt_i->rti_tol.dist;
-			break;
-		    }
-
-		    /* Advance to the next cell as appropriate, bailing
-		     * out with cutp=CUTTER_NULL if we run past the end of
-		     * the NUgrid array.
-		     */
-
-		nugrid_again:
-		    t0 = ssp->tv[out_axis];
-		    if (ssp->rstep[out_axis] > 0) {
-			if (++(ssp->igrid[out_axis]) >=
-			    nu_cells_per_axis[out_axis]) {
-			    cutp = CUTTER_NULL;
-			    break;
-			}
-
-			if (cutp != CUTTER_NULL && cutp != ssp->curcut) {
-			    cutp += nu_stepsize[out_axis];
-			}
-		    } else {
-			if (--(ssp->igrid[out_axis]) < 0) {
-			    cutp = CUTTER_NULL;
-			    break;
-			}
-			if (cutp != CUTTER_NULL && cutp != ssp->curcut) {
-			    cutp -= nu_stepsize[out_axis];
-			}
-		    }
-
-		    /* Update t advancement value for this axis */
-		    NUGRID_T_ADV(out_axis, ssp->igrid[out_axis]);
-		}
-
-		/* find minimum exit t value */
-		if (ssp->tv[X] < ssp->tv[Y]) {
-		    if (ssp->tv[Z] < ssp->tv[X]) {
-			out_axis = Z;
-		    } else {
-			out_axis = X;
-		    }
-		} else {
-		    if (ssp->tv[Z] < ssp->tv[Y]) {
-			out_axis = Z;
-		    } else {
-			out_axis = Y;
-		    }
-		}
-
-		/* Zip through empty cells. */
-		if (cutp->cut_type == CUT_BOXNODE &&
-		    cutp->bn.bn_len <= 0 &&
-		    cutp->bn.bn_piecelen <= 0) {
-		    ++ssp->resp->re_nempty_cells;
-		    goto nugrid_again;
-		}
-
-		ssp->out_axis = out_axis;
-		ssp->lastcell = cutp;
-
-		if (RT_G_DEBUG&DEBUG_ADVANCE)
-		    bu_log("t0=%g found in cell (%d, %d, %d), out_axis=%c at %g; cell is %s\n",
-			   t0,
-			   ssp->igrid[X], ssp->igrid[Y],
-			   ssp->igrid[Z],
-			   "XYZ*"[ssp->out_axis],
-			   ssp->tv[out_axis],
-			   cutp->cut_type==CUT_CUTNODE?"cut":
-			   cutp->cut_type==CUT_BOXNODE?"box":
-			   cutp->cut_type==CUT_NUGRIDNODE?"nu":
-			   "?");
-		break;
-	    } /* CUT_NUGRIDNODE */
-
 	    case CUT_CUTNODE:
 		/* fall through */
 
@@ -409,26 +224,13 @@ rt_advance_to_next_cell(register struct rt_shootray_status *ssp)
 	py = ap->a_ray.r_pt[Y] + t0*ap->a_ray.r_dir[Y];
 	pz = ap->a_ray.r_pt[Z] + t0*ap->a_ray.r_dir[Z];
 
-	/* Optimization: when it's a boxnode in a nugrid, just
-	 * return.
-	 */
-	if (cutp->cut_type == CUT_BOXNODE &&
-	    curcut->cut_type == CUT_NUGRIDNODE) {
-	    ssp->newray.r_pt[X] = px;
-	    ssp->newray.r_pt[Y] = py;
-	    ssp->newray.r_pt[Z] = pz;
-	    ssp->newray.r_min = 0;
-	    ssp->newray.r_max = ssp->tv[ssp->out_axis] - t0;
-	    goto done_return_cutp;
-	}
-
 	/* Given direction of travel, see if point is outside bound.
 	 * This will be the model RPP for NUBSP.
 	 */
 	if (PT_DEPARTING_RPP(ssp->rstep, ssp->curmin, ssp->curmax, px, py, pz))
 	    goto pop_space_stack;
 
-	if (RT_G_DEBUG&DEBUG_ADVANCE) {
+	if (RT_G_DEBUG&RT_DEBUG_ADVANCE) {
 	    bu_log(
 		"rt_advance_to_next_cell() dist_corr=%g, pt=(%g, %g, %g)\n",
 		t0 /*ssp->dist_corr*/, px, py, pz);
@@ -465,7 +267,7 @@ rt_advance_to_next_cell(register struct rt_shootray_status *ssp)
 
 	switch (cutp->cut_type) {
 	    case CUT_BOXNODE:
-		if (RT_G_DEBUG&DEBUG_ADVANCE &&
+		if (RT_G_DEBUG&RT_DEBUG_ADVANCE &&
 		    PT_DEPARTING_RPP(ssp->rstep, ssp->curmin, ssp->curmax, px, py, pz)
 		    ) {
 		    /* This cell is old news. */
@@ -489,7 +291,7 @@ rt_advance_to_next_cell(register struct rt_shootray_status *ssp)
 		if (cutp==ssp->lastcut) {
 		    fastf_t delta;
 		push_to_next_box:				;
-		    if (RT_G_DEBUG & DEBUG_ADVANCE) {
+		    if (RT_G_DEBUG & RT_DEBUG_ADVANCE) {
 			bu_log(
 			    "%d, %d box push odist_corr=%.20e n=%.20e model_end=%.20e\n",
 			    ap->a_x, ap->a_y,
@@ -515,14 +317,14 @@ rt_advance_to_next_cell(register struct rt_shootray_status *ssp)
 		    fraction = frexp(ssp->box_end,
 				     &exponent);
 
-		    if (RT_G_DEBUG & DEBUG_ADVANCE) {
+		    if (RT_G_DEBUG & RT_DEBUG_ADVANCE) {
 			bu_log(
 			    "exp=%d, fraction=%.20e\n",
 			    exponent, fraction);
 		    }
 		    fraction += SQRT_SMALL_FASTF;
 		    delta = ldexp(fraction, exponent);
-		    if (RT_G_DEBUG & DEBUG_ADVANCE) {
+		    if (RT_G_DEBUG & RT_DEBUG_ADVANCE) {
 			bu_log(
 			    "ldexp: delta=%g, fract=%g, exp=%d\n",
 			    delta,
@@ -535,7 +337,7 @@ rt_advance_to_next_cell(register struct rt_shootray_status *ssp)
 		    ssp->box_start = ssp->box_end + delta;
 		    ssp->box_end = ssp->box_start + delta;
 
-		    if (RT_G_DEBUG & DEBUG_ADVANCE) {
+		    if (RT_G_DEBUG & RT_DEBUG_ADVANCE) {
 			bu_log(
 			    "push%d: was=%.20e, now=%.20e\n\n",
 			    push_flag,
@@ -556,7 +358,7 @@ rt_advance_to_next_cell(register struct rt_shootray_status *ssp)
 		}
 		if (push_flag) {
 		    push_flag = 0;
-		    if (RT_G_DEBUG & DEBUG_ADVANCE) {
+		    if (RT_G_DEBUG & RT_DEBUG_ADVANCE) {
 			bu_log(
 			    "%d, %d Escaped %d. dist_corr=%g, box_start=%g, box_end=%g\n",
 			    ap->a_x, ap->a_y,
@@ -566,10 +368,10 @@ rt_advance_to_next_cell(register struct rt_shootray_status *ssp)
 			    ssp->box_end);
 		    }
 		}
-		if (RT_G_DEBUG & DEBUG_ADVANCE) {
+		if (RT_G_DEBUG & RT_DEBUG_ADVANCE) {
 		    bu_log(
-			"rt_advance_to_next_cell()=x%x lastcut=x%x\n",
-			cutp, ssp->lastcut);
+			"rt_advance_to_next_cell()=%p lastcut=%p\n",
+			(void *)cutp, (void *)ssp->lastcut);
 		}
 
 		ssp->newray.r_pt[X] = px;
@@ -585,33 +387,16 @@ rt_advance_to_next_cell(register struct rt_shootray_status *ssp)
 		    goto push_to_next_box;
 		}
 
-	    done_return_cutp:	ssp->lastcut = cutp;
+		ssp->lastcut = cutp;
 		ssp->dist_corr = t0;
 		ssp->box_start = t0 + ssp->newray.r_min;
 		ssp->box_end = t0 + ssp->newray.r_max;
-		if (RT_G_DEBUG & DEBUG_ADVANCE) {
+		if (RT_G_DEBUG & RT_DEBUG_ADVANCE) {
 		    bu_log(
 			"rt_advance_to_next_cell() box=(%g, %g)\n",
 			ssp->box_start, ssp->box_end);
 		}
 		return cutp;
-	    case CUT_NUGRIDNODE: {
-		struct rt_shootray_status *old;
-
-		BU_ALLOC(old, struct rt_shootray_status);
-		*old = *ssp;	/* struct copy */
-
-		/* Descend into node */
-		ssp->old_status = old;
-		ssp->lastcut = ssp->lastcell = CUTTER_NULL;
-		curcut = ssp->curcut = cutp;
-		VSET(ssp->curmin, cutp->nugn.nu_axis[X][0].nu_spos,
-		     cutp->nugn.nu_axis[Y][0].nu_spos,
-		     cutp->nugn.nu_axis[Z][0].nu_spos);
-		VSET(ssp->curmax, ssp->curcut->nugn.nu_axis[X][ssp->curcut->nugn.nu_cells_per_axis[X]-1].nu_epos,
-		     ssp->curcut->nugn.nu_axis[Y][ssp->curcut->nugn.nu_cells_per_axis[Y]-1].nu_epos,
-		     ssp->curcut->nugn.nu_axis[Z][ssp->curcut->nugn.nu_cells_per_axis[Z]-1].nu_epos);
-		break; }
 	    case CUT_CUTNODE:
 		bu_bomb("rt_advance_to_next_cell: impossible: cutnote as leaf!");
 		break;
@@ -647,14 +432,13 @@ escaped_from_model:
  * bit vector (provided by the caller) gets a bit set for every
  * primitive that meets the above criteria.  No primitive
  * intersections are performed.
- *
- * XXX This routine does not work for NUGRID XXX
  */
 fastf_t
 rt_find_backing_dist(struct rt_shootray_status *ss, struct bu_bitv *backbits) {
-    fastf_t min_backing_dist=BACKING_DIST;
-    fastf_t cur_dist=0.0;
-    point_t cur_pt;
+    fastf_t min_backing_dist = BACKING_DIST;
+    fastf_t prev_dist = -1.0;
+    fastf_t curr_dist = 0.0;
+    point_t curr_pt;
     union cutter *cutp;
     struct bu_bitv *solidbits;
     struct xray ray;
@@ -672,19 +456,19 @@ rt_find_backing_dist(struct rt_shootray_status *ss, struct bu_bitv *backbits) {
 
     ray = ss->ap->a_ray;	/* struct copy, don't mess with the original */
 
-    /* cur_dist keeps track of where we are along the ray.  stop when
-     * cur_dist reaches far intersection of ray and model bounding box
+    VMOVE(curr_pt, ss->ap->a_ray.r_pt);
+
+    /* curr_dist keeps track of where we are along the ray.  stop when
+     * curr_dist reaches far intersection of ray and model bounding box
      */
-    while (cur_dist <= ss->ap->a_ray.r_max) {
-	/* calculate the current point along the ray */
-	VJOIN1(cur_pt, ss->ap->a_ray.r_pt, cur_dist, ss->ap->a_ray.r_dir);
+    while (curr_dist <= ss->ap->a_ray.r_max && !EQUAL(curr_dist, prev_dist)) {
 
 	/* descend into the space partitioning tree based on this
 	 * point.
 	 */
 	cutp = &ss->ap->a_rt_i->rti_CutHead;
 	while (cutp->cut_type == CUT_CUTNODE) {
-	    if (cur_pt[cutp->cn.cn_axis] >= cutp->cn.cn_point) {
+	    if (curr_pt[cutp->cn.cn_axis] >= cutp->cn.cn_point) {
 		cutp=cutp->cn.cn_r;
 	    } else {
 		cutp=cutp->cn.cn_l;
@@ -702,8 +486,9 @@ rt_find_backing_dist(struct rt_shootray_status *ss, struct bu_bitv *backbits) {
 	     */
 	    goto done;
 	} else {
-	    /* increment cur_dist into next cell for next execution of this loop */
-	    cur_dist = ray.r_max + ss->ap->a_rt_i->rti_tol.dist;
+	    /* increment curr_dist into next cell for next execution of this loop */
+	    prev_dist = curr_dist;
+	    curr_dist = ray.r_max + ss->ap->a_rt_i->rti_tol.dist;
 	}
 
 	/* process this box node (look at all the pieces) */
@@ -729,7 +514,12 @@ rt_find_backing_dist(struct rt_shootray_status *ss, struct bu_bitv *backbits) {
 		BU_BITSET(solidbits, plp->stp->st_bit);
 	    }
 	}
+
+	/* calculate the next point along the ray */
+	VJOIN1(curr_pt, ss->ap->a_ray.r_pt, curr_dist, ss->ap->a_ray.r_dir);
+
     }
+
 done:
     /* put our bit vector on the resource list */
     BU_CK_BITV(solidbits);
@@ -753,9 +543,6 @@ rt_3move_raydist(FILE *fp, struct xray *rayp, double dist)
 }
 
 
-/**
- *
- */
 void
 rt_3cont_raydist(FILE *fp, struct xray *rayp, double dist)
 {
@@ -766,9 +553,6 @@ rt_3cont_raydist(FILE *fp, struct xray *rayp, double dist)
 }
 
 
-/**
- *
- */
 void
 rt_plot_cell(const union cutter *cutp, const struct rt_shootray_status *ssp, struct bu_list *waiting_segs_hd, struct rt_i *rtip)
 {
@@ -859,13 +643,13 @@ rt_shootray(register struct application *ap)
 					   the ray start point */
     struct bu_ptbl *regionbits;	/* table of all involved regions */
     char *status;
-    auto struct partition InitialPart;	/* Head of Initial Partitions */
-    auto struct partition FinalPart;	/* Head of Final Partitions */
+    struct partition InitialPart;	/* Head of Initial Partitions */
+    struct partition FinalPart;	/* Head of Final Partitions */
     struct soltab **stpp;
     register const union cutter *cutp;
     struct resource *resp;
     struct rt_i *rtip;
-    const int debug_shoot = RT_G_DEBUG & DEBUG_SHOOT;
+    const int debug_shoot = RT_G_DEBUG & RT_DEBUG_SHOOT;
     fastf_t pending_hit = 0; /* dist of closest odd hit pending */
 
     RT_AP_CHECK(ap);
@@ -890,7 +674,8 @@ rt_shootray(register struct application *ap)
 #endif
     if (ap->a_resource == RESOURCE_NULL) {
 	ap->a_resource = &rt_uniresource;
-	if (RT_G_DEBUG)bu_log("rt_shootray:  defaulting a_resource to &rt_uniresource\n");
+	if (RT_G_DEBUG)
+	    bu_log("rt_shootray:  defaulting a_resource to &rt_uniresource\n");
 	if (rt_uniresource.re_magic == 0)
 	    rt_init_resource(&rt_uniresource, 0, ap->a_rt_i);
     }
@@ -903,7 +688,7 @@ rt_shootray(register struct application *ap)
 
     if (RT_G_DEBUG) {
 	/* only test extensively if something in run-time debug is enabled */
-	if (RT_G_DEBUG & (DEBUG_ALLRAYS|DEBUG_SHOOT|DEBUG_PARTITION|DEBUG_ALLHITS)) {
+	if (RT_G_DEBUG & (RT_DEBUG_ALLRAYS|RT_DEBUG_SHOOT|RT_DEBUG_PARTITION|RT_DEBUG_ALLHITS)) {
 	    bu_log_indent_delta(2);
 	    bu_log("\n**********shootray cpu=%d  %d, %d lvl=%d a_onehit=%d (%s)\n",
 		   resp->re_cpu,
@@ -933,7 +718,7 @@ rt_shootray(register struct application *ap)
 
     if (!BU_LIST_IS_INITIALIZED(&resp->re_parthead)) {
 	/* XXX This shouldn't happen any more */
-	bu_log("rt_shootray() resp=x%x uninitialized, fixing it\n", resp);
+	bu_log("rt_shootray() resp=%p uninitialized, fixing it\n", (void *)resp);
 	/*
 	 * We've been handed a mostly un-initialized resource struct,
 	 * with only a magic number and a cpu number filled in.  Init
@@ -945,7 +730,7 @@ rt_shootray(register struct application *ap)
     }
     /* Ensure that this CPU's resource structure is registered */
     if (resp != &rt_uniresource)
-	BU_ASSERT_PTR(BU_PTBL_GET(&rtip->rti_resources, resp->re_cpu), !=, NULL);
+	BU_ASSERT(BU_PTBL_GET(&rtip->rti_resources, resp->re_cpu) != NULL);
 
     solidbits = rt_get_solidbitv(rtip->nsolids, resp);
 
@@ -970,15 +755,14 @@ rt_shootray(register struct application *ap)
 
     /* Verify that direction vector has unit length */
     if (RT_G_DEBUG) {
-	register fastf_t f, diff;
-	/* Fancy version of BN_VEC_NON_UNIT_LEN() */
+	fastf_t f, diff;
+
 	f = MAGSQ(ap->a_ray.r_dir);
-	if (NEAR_ZERO(f, 0.0001)) {
+	if (NEAR_ZERO(f, ap->a_rt_i->rti_tol.dist)) {
 	    bu_bomb("rt_shootray:  zero length dir vector\n");
-	    return 0;
 	}
 	diff = f - 1;
-	if (!NEAR_ZERO(diff, 0.0001)) {
+	if (!NEAR_ZERO(diff, ap->a_rt_i->rti_tol.dist)) {
 	    bu_log("rt_shootray: non-unit dir vect (x%d y%d lvl%d)\n",
 		   ap->a_x, ap->a_y, ap->a_level);
 	    f = 1/f;
@@ -1144,16 +928,8 @@ rt_shootray(register struct application *ap)
     ss.lastcut = CUTTER_NULL;
     ss.old_status = (struct rt_shootray_status *)NULL;
     ss.curcut = &ap->a_rt_i->rti_CutHead;
-    if (ss.curcut->cut_type == CUT_NUGRIDNODE) {
-	ss.lastcell = CUTTER_NULL;
-	VSET(ss.curmin, ss.curcut->nugn.nu_axis[X][0].nu_spos,
-	     ss.curcut->nugn.nu_axis[Y][0].nu_spos,
-	     ss.curcut->nugn.nu_axis[Z][0].nu_spos);
-	VSET(ss.curmax, ss.curcut->nugn.nu_axis[X][ss.curcut->nugn.nu_cells_per_axis[X]-1].nu_epos,
-	     ss.curcut->nugn.nu_axis[Y][ss.curcut->nugn.nu_cells_per_axis[Y]-1].nu_epos,
-	     ss.curcut->nugn.nu_axis[Z][ss.curcut->nugn.nu_cells_per_axis[Z]-1].nu_epos);
-    } else if (ss.curcut->cut_type == CUT_CUTNODE ||
-	       ss.curcut->cut_type == CUT_BOXNODE) {
+
+    if (ss.curcut->cut_type == CUT_CUTNODE || ss.curcut->cut_type == CUT_BOXNODE) {
 	ss.lastcell = ss.curcut;
 	VMOVE(ss.curmin, rtip->mdl_min);
 	VMOVE(ss.curmax, rtip->mdl_max);
@@ -1349,7 +1125,7 @@ rt_shootray(register struct application *ap)
 		resp->re_shot_hit++;
 	    }
 	}
-	if (RT_G_DEBUG & DEBUG_ADVANCE)
+	if (RT_G_DEBUG & RT_DEBUG_ADVANCE)
 	    rt_plot_cell(cutp, &ss, &(waiting_segs.l), rtip);
 
 	/*
@@ -1392,7 +1168,7 @@ rt_shootray(register struct application *ap)
 			register fastf_t dist;
 
 			dist = (*psp)->mindist;
-			BU_ASSERT_DOUBLE(dist, <, INFINITY);
+			BU_ASSERT(dist < INFINITY);
 			if (dist < pending_hit) {
 			    pending_hit = dist;
 			    if (debug_shoot) bu_log("pending_hit lowered to %g by %s\n", pending_hit, (*psp)->stp->st_name);
@@ -1425,7 +1201,7 @@ rt_shootray(register struct application *ap)
      * segments into the partition list.
      */
 weave:
-    if (RT_G_DEBUG&DEBUG_ADVANCE)
+    if (RT_G_DEBUG&RT_DEBUG_ADVANCE)
 	bu_log("rt_shootray: ray has left known space\n");
 
     /* Process any pending hits into segs */
@@ -1476,33 +1252,23 @@ weave:
 	goto out;
     }
 
-    /*
-     * Ray/model intersections exist.  Pass the list to the user's
-     * a_hit() routine.  Note that only the hit_dist elements of
-     * pt_inhit and pt_outhit have been computed yet.  To compute both
-     * hit_point and hit_normal, use the
-     *
-     * RT_HIT_NORMAL(NULL, hitp, stp, rayp, 0);
-     *
-     * macro.  To compute just hit_point, use
-     *
-     * VJOIN1(hitp->hit_point, rp->r_pt, hitp->hit_dist, rp->r_dir);
-     */
 hitit:
+    /* Ray/model intersections exist */
+
     if (debug_shoot) rt_pr_partitions(rtip, &FinalPart, "a_hit()");
 
-    /*
-     * Before recursing, release storage for unused Initial
+    /* Before recursing, release storage for unused Initial
      * partitions.  finished_segs can not be released yet, because
      * FinalPart partitions will point to hits in those segments.
      */
     RT_FREE_PT_LIST(&InitialPart, resp);
 
-    /*
-     * finished_segs is only used by special hit routines which don't
+    /* finished_segs is only used by special hit routines which don't
      * follow the traditional solid modeling paradigm.
      */
-    if (RT_G_DEBUG&DEBUG_ALLHITS) rt_pr_partitions(rtip, &FinalPart, "Partition list passed to a_hit() routine");
+    if (RT_G_DEBUG&RT_DEBUG_ALLHITS) rt_pr_partitions(rtip, &FinalPart, "Partition list passed to a_hit() routine");
+
+    /* Invoke caller's a_hit callback with the list of partitions */
     if (ap->a_hit) {
 	ap->a_return = ap->a_hit(ap, &FinalPart, &finished_segs);
 	status = "HIT";
@@ -1539,7 +1305,7 @@ out:
     }
 
     /* Terminate any logging */
-    if (RT_G_DEBUG&(DEBUG_ALLRAYS|DEBUG_SHOOT|DEBUG_PARTITION|DEBUG_ALLHITS)) {
+    if (RT_G_DEBUG&(RT_DEBUG_ALLRAYS|RT_DEBUG_SHOOT|RT_DEBUG_PARTITION|RT_DEBUG_ALLHITS)) {
 	bu_log_indent_delta(-2);
 	bu_log("----------shootray cpu=%d  %d, %d lvl=%d (%s) %s ret=%d\n",
 	       resp->re_cpu,
@@ -1561,7 +1327,7 @@ rt_cell_n_on_ray(register struct application *ap, int n)
     register const union cutter *cutp;
     struct resource *resp;
     struct rt_i *rtip;
-    const int debug_shoot = RT_G_DEBUG & DEBUG_SHOOT;
+    const int debug_shoot = RT_G_DEBUG & RT_DEBUG_SHOOT;
 
     memset(&ss, 0, sizeof(struct rt_shootray_status));
 
@@ -1589,7 +1355,7 @@ rt_cell_n_on_ray(register struct application *ap, int n)
     RT_CK_RESOURCE(resp);
     ss.resp = resp;
 
-    if (RT_G_DEBUG&(DEBUG_ALLRAYS|DEBUG_SHOOT|DEBUG_PARTITION|DEBUG_ALLHITS)) {
+    if (RT_G_DEBUG&(RT_DEBUG_ALLRAYS|RT_DEBUG_SHOOT|RT_DEBUG_PARTITION|RT_DEBUG_ALLHITS)) {
 	bu_log_indent_delta(2);
 	bu_log("\n**********cell_n_on_ray cpu=%d  %d, %d lvl=%d (%s), n=%d\n",
 	       resp->re_cpu,
@@ -1607,7 +1373,7 @@ rt_cell_n_on_ray(register struct application *ap, int n)
 
     if (!BU_LIST_IS_INITIALIZED(&resp->re_parthead)) {
 	/* XXX This shouldn't happen any more */
-	bu_log("rt_cell_n_on_ray() resp=x%x uninitialized, fixing it\n", resp);
+	bu_log("rt_cell_n_on_ray() resp=%p uninitialized, fixing it\n", (void *)resp);
 	/*
 	 * We've been handed a mostly un-initialized resource struct,
 	 * with only a magic number and a cpu number filled in.
@@ -1618,18 +1384,18 @@ rt_cell_n_on_ray(register struct application *ap, int n)
 	rt_init_resource(resp, resp->re_cpu, rtip);
     }
     /* Ensure that this CPU's resource structure is registered */
-    BU_ASSERT_PTR(BU_PTBL_GET(&rtip->rti_resources, resp->re_cpu), !=, NULL);
+    BU_ASSERT(BU_PTBL_GET(&rtip->rti_resources, resp->re_cpu) != NULL);
 
     /* Verify that direction vector has unit length */
     if (RT_G_DEBUG) {
 	fastf_t f, diff;
-	/* Fancy version of BN_VEC_NON_UNIT_LEN() */
+
 	f = MAGSQ(ap->a_ray.r_dir);
-	if (NEAR_ZERO(f, 0.0001)) {
+	if (NEAR_ZERO(f, ap->a_rt_i->rti_tol.dist)) {
 	    bu_bomb("rt_cell_n_on_ray:  zero length dir vector\n");
 	}
 	diff = f - 1;
-	if (!NEAR_ZERO(diff, 0.0001)) {
+	if (!NEAR_ZERO(diff, ap->a_rt_i->rti_tol.dist)) {
 	    bu_log("rt_cell_n_on_ray: non-unit dir vect (x%d y%d lvl%d)\n",
 		   ap->a_x, ap->a_y, ap->a_level);
 	    f = 1/f;
@@ -1714,16 +1480,8 @@ rt_cell_n_on_ray(register struct application *ap, int n)
     ss.lastcut = CUTTER_NULL;
     ss.old_status = (struct rt_shootray_status *)NULL;
     ss.curcut = &ap->a_rt_i->rti_CutHead;
-    if (ss.curcut->cut_type == CUT_NUGRIDNODE) {
-	ss.lastcell = CUTTER_NULL;
-	VSET(ss.curmin, ss.curcut->nugn.nu_axis[X][0].nu_spos,
-	     ss.curcut->nugn.nu_axis[Y][0].nu_spos,
-	     ss.curcut->nugn.nu_axis[Z][0].nu_spos);
-	VSET(ss.curmax, ss.curcut->nugn.nu_axis[X][ss.curcut->nugn.nu_cells_per_axis[X]-1].nu_epos,
-	     ss.curcut->nugn.nu_axis[Y][ss.curcut->nugn.nu_cells_per_axis[Y]-1].nu_epos,
-	     ss.curcut->nugn.nu_axis[Z][ss.curcut->nugn.nu_cells_per_axis[Z]-1].nu_epos);
-    } else if (ss.curcut->cut_type == CUT_CUTNODE ||
-	       ss.curcut->cut_type == CUT_BOXNODE) {
+
+    if (ss.curcut->cut_type == CUT_CUTNODE || ss.curcut->cut_type == CUT_BOXNODE) {
 	ss.lastcell = ss.curcut;
 	VMOVE(ss.curmin, rtip->mdl_min);
 	VMOVE(ss.curmax, rtip->mdl_max);
@@ -1809,7 +1567,7 @@ rt_shootray_simple_hit(struct application *a, struct partition *PartHeadp, struc
     struct partition *p = NULL, *c = NULL, *pp;
 
     for (pp = PartHeadp->pt_forw; pp != PartHeadp; pp = pp->pt_forw) {
-	if(p) {
+	if (p) {
 	    BU_ALLOC(c->pt_forw, struct partition);
 	    c->pt_forw->pt_back = c;
 	    c = c->pt_forw;
