@@ -1,7 +1,7 @@
 /*                          T R E E . C
  * BRL-CAD
  *
- * Copyright (c) 1995-2013 United States Government as represented by
+ * Copyright (c) 1995-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This library is free software; you can redistribute it and/or
@@ -22,17 +22,17 @@
 #include "common.h"
 
 #include <stddef.h>
-#include <stdio.h>
 #include <math.h>
 #include <string.h>
 #include "bio.h"
 
-#include "bu.h"
+#include "bu/parallel.h"
 #include "vmath.h"
 #include "bn.h"
-#include "db.h"
+#include "rt/db4.h"
 #include "raytrace.h"
-#include "mater.h"
+
+#include "./cache.h"
 
 
 #define ACQUIRE_SEMAPHORE_TREE(_hash) switch ((_hash)&03) {	\
@@ -105,24 +105,31 @@ _rt_tree_region_assign(union tree *tp, const struct region *regionp)
  * This routine must be prepared to run in parallel.
  */
 HIDDEN int
-_rt_gettree_region_start(struct db_tree_state *tsp, const struct db_full_path *pathp, const struct rt_comb_internal *combp, genptr_t UNUSED(client_data))
+_rt_gettree_region_start(struct db_tree_state *tsp, const struct db_full_path *pathp, const struct rt_comb_internal *combp, void *UNUSED(client_data))
 {
-  if (tsp) {
-    RT_CK_RTI(tsp->ts_rtip);
-    RT_CK_RESOURCE(tsp->ts_resp);
-    if (pathp) RT_CK_FULL_PATH(pathp);
-    if (combp) RT_CHECK_COMB(combp);
+    if (tsp) {
+	RT_CK_RTI(tsp->ts_rtip);
+	RT_CK_RESOURCE(tsp->ts_resp);
+	if (pathp) RT_CK_FULL_PATH(pathp);
+	if (combp) RT_CHECK_COMB(combp);
 
-    /* Ignore "air" regions unless wanted */
-    if (tsp->ts_rtip->useair == 0 &&  tsp->ts_aircode != 0) {
-      tsp->ts_rtip->rti_air_discards++;
-      return -1;	/* drop this region */
+	/* Ignore "air" regions unless wanted */
+	if (tsp->ts_rtip->useair == 0 &&  tsp->ts_aircode != 0) {
+	    tsp->ts_rtip->rti_air_discards++;
+	    return -1;	/* drop this region */
+	}
     }
-  }
-  return 0;
+    return 0;
 }
 
 
+struct gettree_data
+{
+    struct bu_hash_tbl *tbl;
+    struct rt_cache *cache;
+};
+
+
 /**
  * This routine will be called by db_walk_tree() once all the solids
  * in this region have been visited.
@@ -135,14 +142,13 @@ _rt_gettree_region_start(struct db_tree_state *tsp, const struct db_full_path *p
  * into the serial section.  (_rt_tree_region_assign, rt_bound_tree)
  */
 HIDDEN union tree *
-_rt_gettree_region_end(struct db_tree_state *tsp, const struct db_full_path *pathp, union tree *curtree, genptr_t client_data)
+_rt_gettree_region_end(struct db_tree_state *tsp, const struct db_full_path *pathp, union tree *curtree, void *client_data)
 {
     struct region *rp;
     struct directory *dp = NULL;
     size_t shader_len=0;
     struct rt_i *rtip;
-    Tcl_HashTable *tbl = (Tcl_HashTable *)client_data;
-    Tcl_HashEntry *entry;
+    struct bu_hash_tbl *tbl = ((struct gettree_data *)client_data)->tbl;
     matp_t inv_mat;
     struct bu_attribute_value_set avs;
     struct bu_attribute_value_pair *avpp;
@@ -192,7 +198,7 @@ _rt_gettree_region_end(struct db_tree_state *tsp, const struct db_full_path *pat
     rp->reg_name = db_path_to_string(pathp);
 
 
-    if (RT_G_DEBUG&DEBUG_TREEWALK) {
+    if (RT_G_DEBUG&RT_DEBUG_TREEWALK) {
 	bu_log("_rt_gettree_region_end() %s\n", rp->reg_name);
 	rt_pr_tree(curtree, 0);
     }
@@ -224,9 +230,7 @@ _rt_gettree_region_end(struct db_tree_state *tsp, const struct db_full_path *pat
     bu_semaphore_release(RT_SEM_RESULTS);
 
     if (tbl && bu_avs_get(&tsp->ts_attrs, "ORCA_Comp")) {
-	int newentry;
-	long int reg_bit = rp->reg_bit;
-	const char *key = (char *)reg_bit;
+	uint32_t key = rp->reg_bit;
 
 	inv_mat = (matp_t)bu_calloc(16, sizeof(fastf_t), "inv_mat");
 	bn_mat_inv(inv_mat, tsp->ts_mat);
@@ -234,14 +238,13 @@ _rt_gettree_region_end(struct db_tree_state *tsp, const struct db_full_path *pat
 	/* enter critical section */
 	bu_semaphore_acquire(RT_SEM_RESULTS);
 
-	entry = Tcl_CreateHashEntry(tbl, key, &newentry);
-	Tcl_SetHashValue(entry, (ClientData)inv_mat);
+	(void)bu_hash_set(tbl, (const uint8_t *)&key, sizeof(key), (void *)inv_mat);
 
 	/* leave critical section */
 	bu_semaphore_release(RT_SEM_RESULTS);
     }
 
-    if (RT_G_DEBUG & DEBUG_REGIONS) {
+    if (RT_G_DEBUG & RT_DEBUG_REGIONS) {
 	bu_log("Add Region %s instnum %ld\n",
 	       rp->reg_name, rp->reg_instnum);
     }
@@ -365,7 +368,7 @@ _rt_find_identical_solid(const matp_t mat, struct directory *dp, struct rt_i *rt
 	    /* dp->d_uses is NOT incremented, because number of
 	     * soltab's using it has not gone up.
 	     */
-	    if (RT_G_DEBUG & DEBUG_SOLIDS) {
+	    if (RT_G_DEBUG & RT_DEBUG_SOLIDS) {
 		bu_log(mat ?
 		       "%s re-referenced %ld\n" :
 		       "%s re-referenced %ld (identity mat)\n",
@@ -417,9 +420,9 @@ _rt_find_identical_solid(const matp_t mat, struct directory *dp, struct rt_i *rt
     /* Enter an exclusive critical section to protect nsolids.
      * nsolids++ needs to be locked to a SINGLE thread
      */
-    bu_semaphore_acquire(RT_SEM_STATS);
+    bu_semaphore_acquire(BU_SEM_GENERAL);
     stp->st_bit = rtip->nsolids++;
-    bu_semaphore_release(RT_SEM_STATS);
+    bu_semaphore_release(BU_SEM_GENERAL);
 
     /*
      * Fill in the last little bit of the structure in full parallel
@@ -437,8 +440,9 @@ _rt_find_identical_solid(const matp_t mat, struct directory *dp, struct rt_i *rt
  * This routine must be prepared to run in parallel.
  */
 HIDDEN union tree *
-_rt_gettree_leaf(struct db_tree_state *tsp, const struct db_full_path *pathp, struct rt_db_internal *ip, genptr_t UNUSED(client_data))
+_rt_gettree_leaf(struct db_tree_state *tsp, const struct db_full_path *pathp, struct rt_db_internal *ip, void *client_data)
 {
+    struct gettree_data *data;
     struct soltab *stp;
     struct directory *dp;
     matp_t mat;
@@ -455,6 +459,12 @@ _rt_gettree_leaf(struct db_tree_state *tsp, const struct db_full_path *pathp, st
     RT_CK_RTI(rtip);
     RT_CK_RESOURCE(tsp->ts_resp);
     dp = DB_FULL_PATH_CUR_DIR(pathp);
+
+    data = (struct gettree_data *)client_data;
+
+    if (!data)
+	bu_bomb("missing argument");
+
     if (!dp)
 	return TREE_NULL;
 
@@ -507,15 +517,16 @@ _rt_gettree_leaf(struct db_tree_state *tsp, const struct db_full_path *pathp, st
     VSETALL(stp->st_min,  INFINITY);
 
     /*
-     * If the ft_prep routine wants to keep the internal structure,
-     * that is OK, as long as idb_ptr is set to null.  Note that the
-     * prep routine may have changed st_id.
+     * If prep wants to keep the internal structure, that is OK, as
+     * long as idb_ptr is set to null.  Note that the prep routine may
+     * have changed st_id.
      */
-    ret = -1;
-    if (stp->st_meth->ft_prep) {
-	ret = stp->st_meth->ft_prep(stp, ip, rtip);
+    if (rtip->rti_dbip->dbi_version > 4) {
+	ret = rt_cache_prep(data->cache, stp, ip);
+    } else {
+	ret = rt_obj_prep(stp, ip, stp->st_rtip);
     }
-    if (ret != 0) {
+    if (ret) {
 	int hash;
 	/* Error, solid no good */
 	bu_log("_rt_gettree_leaf(%s):  prep failure\n", dp->d_namep);
@@ -561,20 +572,20 @@ _rt_gettree_leaf(struct db_tree_state *tsp, const struct db_full_path *pathp, st
 	    db_dup_path_tail(&stp->st_path, pathp, i);
 	}
     }
-    if (RT_G_DEBUG&DEBUG_TREEWALK && stp->st_path.magic == DB_FULL_PATH_MAGIC) {
+    if (RT_G_DEBUG&RT_DEBUG_TREEWALK && stp->st_path.magic == DB_FULL_PATH_MAGIC) {
 	char *sofar = db_path_to_string(&stp->st_path);
 	bu_log("_rt_gettree_leaf() st_path=%s\n", sofar);
 	bu_free(sofar, "path string");
     }
 
-    if (RT_G_DEBUG&DEBUG_SOLIDS) {
+    if (RT_G_DEBUG&RT_DEBUG_SOLIDS) {
 	struct bu_vls str = BU_VLS_INIT_ZERO;
 	bu_log("\n---Primitive %ld: %s\n", stp->st_bit, dp->d_namep);
 
 	/* verbose=1, mm2local=1.0 */
 	ret = -1;
 	if (stp->st_meth->ft_describe) {
-	    ret = stp->st_meth->ft_describe(&str, ip, 1, 1.0, tsp->ts_resp, tsp->ts_dbip);
+	    ret = stp->st_meth->ft_describe(&str, ip, 1, 1.0);
 	}
 	if (ret < 0) {
 	    bu_log("_rt_gettree_leaf(%s):  solid describe failure\n",
@@ -585,13 +596,14 @@ _rt_gettree_leaf(struct db_tree_state *tsp, const struct db_full_path *pathp, st
     }
 
 found_it:
-    RT_GET_TREE(curtree, tsp->ts_resp);
+    BU_GET(curtree, union tree);
+    RT_TREE_INIT(curtree);
     curtree->tr_op = OP_SOLID;
     curtree->tr_a.tu_stp = stp;
     /* regionp will be filled in later by _rt_tree_region_assign() */
     curtree->tr_a.tu_regionp = (struct region *)0;
 
-    if (RT_G_DEBUG&DEBUG_TREEWALK) {
+    if (RT_G_DEBUG&RT_DEBUG_TREEWALK) {
 	char *sofar = db_path_to_string(pathp);
 	bu_log("_rt_gettree_leaf() %s\n", sofar);
 	bu_free(sofar, "path string");
@@ -662,8 +674,9 @@ _rt_tree_kill_dead_solid_refs(union tree *tp)
 		stp = tp->tr_a.tu_stp;
 		RT_CK_SOLTAB(stp);
 		if (stp->st_aradius <= 0) {
-		    if (RT_G_DEBUG&DEBUG_TREEWALK)bu_log("encountered dead solid '%s' stp=x%x, tp=x%x\n",
-							 stp->st_dp->d_namep, stp, tp);
+		    if (RT_G_DEBUG&RT_DEBUG_TREEWALK)
+			bu_log("encountered dead solid '%s' stp=%p, tp=%p\n",
+			       stp->st_dp->d_namep, (void *)stp, (void *)tp);
 		    rt_free_soltab(stp);
 		    tp->tr_a.tu_stp = SOLTAB_NULL;
 		    tp->tr_op = OP_NOP;	/* Convert to NOP */
@@ -672,8 +685,8 @@ _rt_tree_kill_dead_solid_refs(union tree *tp)
 	    }
 
 	default:
-	    bu_log("_rt_tree_kill_dead_solid_refs(x%x): unknown op=x%x\n",
-		   tp, tp->tr_op);
+	    bu_log("_rt_tree_kill_dead_solid_refs(%p): unknown op=%x\n",
+		   (void *)tp, tp->tr_op);
 	    return;
 
 	case OP_XOR:
@@ -703,7 +716,8 @@ rt_gettrees_muves(struct rt_i *rtip, const char **attrs, int argc, const char **
 {
     struct soltab *stp;
     struct region *regp;
-    Tcl_HashTable *tbl;
+    struct bu_hash_tbl *tbl;
+
     size_t prev_sol_count;
     int i;
     int num_attrs=0;
@@ -717,15 +731,16 @@ rt_gettrees_muves(struct rt_i *rtip, const char **attrs, int argc, const char **
 	return -1;		/* FAIL */
     }
 
-    if (argc <= 0) return -1;	/* FAIL */
+    if (argc <= 0)
+	return -1;	/* FAIL */
 
-    BU_ALLOC(tbl, Tcl_HashTable);
-    Tcl_InitHashTable(tbl, TCL_ONE_WORD_KEYS);
-    rtip->Orca_hash_tbl = (genptr_t)tbl;
+    tbl = bu_hash_create(64);
+    rtip->Orca_hash_tbl = (void *)tbl;
 
     prev_sol_count = rtip->nsolids;
 
     {
+	struct gettree_data data;
 	struct db_tree_state tree_state;
 
 	tree_state = rt_initial_tree_state;	/* struct copy */
@@ -733,7 +748,6 @@ rt_gettrees_muves(struct rt_i *rtip, const char **attrs, int argc, const char **
 	tree_state.ts_rtip = rtip;
 	tree_state.ts_resp = NULL;	/* sanity.  Needs to be updated */
 
-
 	if (attrs) {
 	    if (db_version(rtip->rti_dbip) < 5) {
 		bu_log("WARNING: requesting attributes from an old database version (ignored)\n");
@@ -761,12 +775,21 @@ rt_gettrees_muves(struct rt_i *rtip, const char **attrs, int argc, const char **
 	    bu_avs_init_empty(&tree_state.ts_attrs);
 	}
 
+	data.tbl = tbl;
+	if (rtip->rti_dbip->dbi_version > 4) {
+	    data.cache = rt_cache_open();
+	}
+
 	i = db_walk_tree(rtip->rti_dbip, argc, argv, ncpus,
 			 &tree_state,
 			 _rt_gettree_region_start,
 			 _rt_gettree_region_end,
-			 _rt_gettree_leaf, (genptr_t)tbl);
+			 _rt_gettree_leaf, (void *)&data);
 	bu_avs_free(&tree_state.ts_attrs);
+
+	if (rtip->rti_dbip->dbi_version > 4) {
+	    rt_cache_close(data.cache);
+	}
     }
 
     /* DEBUG:  Ensure that all region trees are valid */
@@ -811,7 +834,7 @@ again:
 	    VMINMAX(rtip->mdl_min, rtip->mdl_max, stp->st_max);
 	    stp->st_piecestate_num = rtip->rti_nsolids_with_pieces++;
 	}
-	if (RT_G_DEBUG&DEBUG_SOLIDS)
+	if (RT_G_DEBUG&RT_DEBUG_SOLIDS)
 	    rt_pr_soltab(stp);
     } RT_VISIT_ALL_SOLTABS_END;
 
@@ -849,7 +872,8 @@ again:
 	db_ck_tree(regp->reg_treetop);
     }
 
-    if (i < 0) return i;
+    if (i < 0)
+	return i;
 
     if (rtip->nsolids <= prev_sol_count)
 	bu_log("rt_gettrees(%s) warning:  no primitives found\n", argv[0]);
@@ -912,8 +936,8 @@ top:
 	    return 0;		/* Retain */
 
 	default:
-	    bu_log("rt_tree_elim_nops(x%x): unknown op=x%x\n",
-		   tp, tp->tr_op);
+	    bu_log("rt_tree_elim_nops(%p): unknown op=%x\n",
+		   (void *)tp, tp->tr_op);
 	    return -1;
 
 	case OP_XOR:
@@ -923,14 +947,14 @@ top:
 	    right = tp->tr_b.tb_right;
 	    if (rt_tree_elim_nops(left, resp) < 0) {
 		*tp = *right;	/* struct copy */
-		RT_FREE_TREE(left, resp);
-		RT_FREE_TREE(right, resp);
+		BU_PUT(left, union tree);
+		BU_PUT(right, union tree);
 		goto top;
 	    }
 	    if (rt_tree_elim_nops(right, resp) < 0) {
 		*tp = *left;	/* struct copy */
-		RT_FREE_TREE(left, resp);
-		RT_FREE_TREE(right, resp);
+		BU_PUT(left, union tree);
+		BU_PUT(right, union tree);
 		goto top;
 	    }
 	    break;
@@ -960,8 +984,8 @@ top:
 	    }
 	    if (rt_tree_elim_nops(right, resp) < 0) {
 		*tp = *left;	/* struct copy */
-		RT_FREE_TREE(left, resp);
-		RT_FREE_TREE(right, resp);
+		BU_PUT(left, union tree);
+		BU_PUT(right, union tree);
 		goto top;
 	    }
 	    break;
@@ -971,7 +995,7 @@ top:
 	    /* UNARY tree -- for completeness only, should never be seen */
 	    left = tp->tr_b.tb_left;
 	    if (rt_tree_elim_nops(left, resp) < 0) {
-		RT_FREE_TREE(left, resp);
+		BU_PUT(left, union tree);
 		tp->tr_op = OP_NOP;
 		return -1;	/* Kill ref to unary op, too */
 	    }
@@ -991,15 +1015,17 @@ rt_find_solid(const struct rt_i *rtip, const char *name)
     struct directory *dp;
 
     RT_CHECK_RTI(rtip);
-    if ((dp = db_lookup((struct db_i *)rtip->rti_dbip, (char *)name,
-			LOOKUP_QUIET)) == RT_DIR_NULL)
+    dp = db_lookup((struct db_i *)rtip->rti_dbip, (char *)name, LOOKUP_QUIET);
+    if (dp == RT_DIR_NULL)
 	return RT_SOLTAB_NULL;
 
     RT_VISIT_ALL_SOLTABS_START(stp, (struct rt_i *)rtip) {
-	if (stp->st_dp == dp)
+	if (stp->st_dp == dp) {
 	    return stp;
-    } RT_VISIT_ALL_SOLTABS_END
-	  return RT_SOLTAB_NULL;
+	}
+    } RT_VISIT_ALL_SOLTABS_END;
+
+    return RT_SOLTAB_NULL;
 }
 
 
@@ -1012,7 +1038,7 @@ rt_optim_tree(union tree *tp, struct resource *resp)
 
     RT_CK_TREE(tp);
     while ((sp = resp->re_boolstack) == (union tree **)0)
-	rt_grow_boolstack(resp);
+	rt_bool_growstack(resp);
     stackend = &(resp->re_boolstack[resp->re_boolslen-1]);
     *sp++ = TREE_NULL;
     *sp++ = tp;
@@ -1038,7 +1064,7 @@ rt_optim_tree(union tree *tp, struct resource *resp)
 		*sp++ = tp->tr_b.tb_left;
 		if (sp >= stackend) {
 		    int off = sp - resp->re_boolstack;
-		    rt_grow_boolstack(resp);
+		    rt_bool_growstack(resp);
 		    sp = &(resp->re_boolstack[off]);
 		    stackend = &(resp->re_boolstack[resp->re_boolslen-1]);
 		}
@@ -1052,7 +1078,7 @@ rt_optim_tree(union tree *tp, struct resource *resp)
 		*sp++ = tp->tr_b.tb_left;
 		if (sp >= stackend) {
 		    int off = sp - resp->re_boolstack;
-		    rt_grow_boolstack(resp);
+		    rt_bool_growstack(resp);
 		    sp = &(resp->re_boolstack[off]);
 		    stackend = &(resp->re_boolstack[resp->re_boolslen-1]);
 		}
@@ -1065,6 +1091,53 @@ rt_optim_tree(union tree *tp, struct resource *resp)
 }
 
 
+#ifdef USE_OPENCL
+void
+rt_bit_tree(struct bit_tree *btp, const union tree *tp, size_t *len)
+{
+    int idx;
+    unsigned st_bit, uop, rchild;
+
+    if (tp == TREE_NULL)
+	return;
+
+    idx = (*len)++;
+    switch (tp->tr_op) {
+	case OP_SOLID:
+	    /* Tree Leaf */
+	    st_bit = tp->tr_a.tu_stp->st_bit;
+	    if (btp)
+		btp[idx].val = (st_bit << 3) | UOP_SOLID;
+	    return;
+	case OP_SUBTRACT:
+	    uop = UOP_SUBTRACT;
+	    break;
+	case OP_UNION:
+	    uop = UOP_UNION;
+	    break;
+	case OP_INTERSECT:
+	    uop = UOP_INTERSECT;
+	    break;
+	case OP_XOR:
+	    uop = UOP_XOR;
+	    break;
+	default:
+	    bu_log("rt_bit_tree: bad op[%d]\n", tp->tr_op);
+	    exit(1);
+	    break;
+    }
+
+    rt_bit_tree(btp, tp->tr_b.tb_left, len);
+
+    rchild = *len;
+    if (btp)
+	btp[idx].val = (rchild << 3) | uop;
+
+    rt_bit_tree(btp, tp->tr_b.tb_right, len);
+}
+#endif
+
+
 /*
  * Local Variables:
  * mode: C
