@@ -1,7 +1,7 @@
 /*                           W D B . C
  * BRL-CAD
  *
- * Copyright (c) 1987-2013 United States Government as represented by
+ * Copyright (c) 1987-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This library is free software; you can redistribute it and/or
@@ -38,15 +38,13 @@
 
 #include "common.h"
 
-#include <stdio.h>
 #include <math.h>
 #include <string.h>
 #include "bio.h"
 
-#include "bu.h"
 #include "vmath.h"
 #include "bn.h"
-#include "rtgeom.h"
+#include "rt/geom.h"
 #include "raytrace.h"
 #include "wdb.h"
 
@@ -61,7 +59,7 @@ mk_half(struct rt_wdb *wdbp, const char *name, const fastf_t *norm, fastf_t d)
     VMOVE(half->eqn, norm);
     half->eqn[3] = d;
 
-    return wdb_export(wdbp, name, (genptr_t)half, ID_HALF, mk_conv2mm);
+    return wdb_export(wdbp, name, (void *)half, ID_HALF, mk_conv2mm);
 }
 
 
@@ -81,7 +79,7 @@ mk_grip(
     VMOVE(grip->normal, normal);
     grip->mag = magnitude;
 
-    return wdb_export(wdbp, name, (genptr_t)grip, ID_GRIP, mk_conv2mm);
+    return wdb_export(wdbp, name, (void *)grip, ID_GRIP, mk_conv2mm);
 }
 
 
@@ -111,7 +109,7 @@ mk_wedge(struct rt_wdb *wdbp, const char *name, const fastf_t *vert, const fastf
     vect_t xvec;	/* x_axis vector */
     vect_t txvec;	/* top x_axis vector */
     vect_t yvec;	/* y-axis vector */
-    vect_t zvec;	/* z-axix vector */
+    vect_t zvec;	/* z-axis vector */
     vect_t x_unitv;	/* x-axis unit vector*/
     vect_t z_unitv;	/* z-axis unit vector */
     vect_t y_unitv;
@@ -246,7 +244,7 @@ mk_arb8(struct rt_wdb *wdbp, const char *name, const fastf_t *pts)
 	VMOVE(arb->pt[i], &pts[i*3]);
     }
 
-    return wdb_export(wdbp, name, (genptr_t)arb, ID_ARB8, mk_conv2mm);
+    return wdb_export(wdbp, name, (void *)arb, ID_ARB8, mk_conv2mm);
 }
 
 
@@ -262,7 +260,7 @@ mk_sph(struct rt_wdb *wdbp, const char *name, const fastf_t *center, fastf_t rad
     VSET(ell->b, 0, radius, 0);
     VSET(ell->c, 0, 0, radius);
 
-    return wdb_export(wdbp, name, (genptr_t)ell, ID_ELL, mk_conv2mm);
+    return wdb_export(wdbp, name, (void *)ell, ID_ELL, mk_conv2mm);
 }
 
 
@@ -278,7 +276,7 @@ mk_ell(struct rt_wdb *wdbp, const char *name, const fastf_t *center, const fastf
     VMOVE(ell->b, b);
     VMOVE(ell->c, c);
 
-    return wdb_export(wdbp, name, (genptr_t)ell, ID_ELL, mk_conv2mm);
+    return wdb_export(wdbp, name, (void *)ell, ID_ELL, mk_conv2mm);
 }
 
 
@@ -328,7 +326,7 @@ mk_hyp(struct rt_wdb *wdbp, const char *name, const point_t vertex, const vect_t
 	hyp->hyp_b = minorLen;
     }
 
-    return wdb_export(wdbp, name, (genptr_t)hyp, ID_HYP, mk_conv2mm);
+    return wdb_export(wdbp, name, (void *)hyp, ID_HYP, mk_conv2mm);
 }
 
 
@@ -344,7 +342,7 @@ mk_tor(struct rt_wdb *wdbp, const char *name, const fastf_t *center, const fastf
     tor->r_a = r1;
     tor->r_h = r2;
 
-    return wdb_export(wdbp, name, (genptr_t)tor, ID_TOR, mk_conv2mm);
+    return wdb_export(wdbp, name, (void *)tor, ID_TOR, mk_conv2mm);
 }
 
 
@@ -383,12 +381,12 @@ mk_tgc(struct rt_wdb *wdbp, const char *name, const fastf_t *base, const fastf_t
     VMOVE(tgc->c, c);
     VMOVE(tgc->d, d);
 
-    return wdb_export(wdbp, name, (genptr_t)tgc, ID_TGC, mk_conv2mm);
+    return wdb_export(wdbp, name, (void *)tgc, ID_TGC, mk_conv2mm);
 }
 
 
 int
-mk_cone(struct rt_wdb *wdbp, const char *name, const fastf_t *base, const fastf_t *dirv, fastf_t height, fastf_t rad1, fastf_t rad2)
+mk_cone(struct rt_wdb *wdbp, const char *name, const fastf_t *base, const fastf_t *dirv, fastf_t height, fastf_t base_radius, fastf_t nose_radius)
 {
     vect_t a, avec;	/* one base radius vector */
     vect_t b, bvec;	/* another base radius vector */
@@ -409,10 +407,10 @@ mk_cone(struct rt_wdb *wdbp, const char *name, const fastf_t *base, const fastf_
     bn_vec_ortho(a, h_unitv);
     VUNITIZE(a);
     VCROSS(b, h_unitv, a);
-    VSCALE(avec, a, rad1);
-    VSCALE(bvec, b, rad1);
-    VSCALE(cvec, a, rad2);
-    VSCALE(dvec, b, rad2);
+    VSCALE(avec, a, base_radius);
+    VSCALE(bvec, b, base_radius);
+    VSCALE(cvec, a, nose_radius);
+    VSCALE(dvec, b, nose_radius);
 
     return mk_tgc(wdbp, name, base, hgtv, avec, bvec, cvec, dvec);
 }
@@ -471,7 +469,7 @@ mk_rpc(
     VMOVE(rpc->rpc_B, breadth);
     rpc->rpc_r = half_w;
 
-    return wdb_export(wdbp, name, (genptr_t)rpc, ID_RPC, mk_conv2mm);
+    return wdb_export(wdbp, name, (void *)rpc, ID_RPC, mk_conv2mm);
 }
 
 
@@ -496,7 +494,7 @@ mk_rhc(
     rhc->rhc_r = half_w;
     rhc->rhc_c = asymp;
 
-    return wdb_export(wdbp, name, (genptr_t)rhc, ID_RHC, mk_conv2mm);
+    return wdb_export(wdbp, name, (void *)rhc, ID_RHC, mk_conv2mm);
 }
 
 
@@ -521,7 +519,7 @@ mk_epa(
     epa->epa_r1 = r1;
     epa->epa_r2 = r2;
 
-    return wdb_export(wdbp, name, (genptr_t)epa, ID_EPA, mk_conv2mm);
+    return wdb_export(wdbp, name, (void *)epa, ID_EPA, mk_conv2mm);
 }
 
 
@@ -548,7 +546,7 @@ mk_ehy(
     ehy->ehy_r2 = r2;
     ehy->ehy_c = c;
 
-    return wdb_export(wdbp, name, (genptr_t)ehy, ID_EHY, mk_conv2mm);
+    return wdb_export(wdbp, name, (void *)ehy, ID_EHY, mk_conv2mm);
 }
 
 
@@ -565,7 +563,7 @@ int mk_hrt(struct rt_wdb *wdbp, const char *name, const point_t center, const ve
     VMOVE(hrt->zdir, z);
     hrt->d = dist;
 
-    return wdb_export(wdbp, name, (genptr_t)hrt, ID_HRT, mk_conv2mm);
+    return wdb_export(wdbp, name, (void *)hrt, ID_HRT, mk_conv2mm);
 }
 
 
@@ -590,7 +588,7 @@ mk_eto(
     eto->eto_r = rrot;
     eto->eto_rd = sminor;
 
-    return wdb_export(wdbp, name, (genptr_t)eto, ID_ETO, mk_conv2mm);
+    return wdb_export(wdbp, name, (void *)eto, ID_ETO, mk_conv2mm);
 }
 
 
@@ -619,7 +617,7 @@ mk_metaball(
 	}
     }
 
-    return wdb_export(wdbp, name, (genptr_t)mb, ID_METABALL, mk_conv2mm);
+    return wdb_export(wdbp, name, (void *)mb, ID_METABALL, mk_conv2mm);
 }
 
 
@@ -627,10 +625,11 @@ int
 mk_binunif (
     struct rt_wdb *wdbp,
     const char *name,
-    const genptr_t data,
+    const void *data,
     wdb_binunif data_type,
     long count)
 {
+    struct rt_db_internal intern;
     struct rt_binunif_internal *binunif;
     unsigned int minor_type = 0;
     int from_file = 0;
@@ -640,12 +639,14 @@ mk_binunif (
     switch (data_type) {
 	case WDB_BINUNIF_FILE_FLOAT:
 	    from_file = 1;
+	    /* fall through */
 	case WDB_BINUNIF_FLOAT:
 	    bytes = sizeof(float);
 	    minor_type = DB5_MINORTYPE_BINU_FLOAT;
 	    break;
 	case WDB_BINUNIF_FILE_DOUBLE:
 	    from_file = 1;
+	    /* fall through */
 	case WDB_BINUNIF_DOUBLE:
 	    bytes = sizeof(double);
 	    minor_type = DB5_MINORTYPE_BINU_DOUBLE;
@@ -653,44 +654,52 @@ mk_binunif (
 
 	case WDB_BINUNIF_FILE_INT8:
 	    from_file = 1;
+	    /* fall through */
 	case WDB_BINUNIF_INT8:
 	    bytes = 1;
 	    break;
 	case WDB_BINUNIF_FILE_UINT8:
 	    from_file = 1;
+	    /* fall through */
 	case WDB_BINUNIF_UINT8:
 	    nosign = 1;
 	    bytes = 1;
 	    break;
 	case WDB_BINUNIF_FILE_INT16:
 	    from_file = 1;
+	    /* fall through */
 	case WDB_BINUNIF_INT16:
 	    bytes = 2;
 	    break;
 	case WDB_BINUNIF_FILE_UINT16:
 	    from_file = 1;
+	    /* fall through */
 	case WDB_BINUNIF_UINT16:
 	    nosign = 1;
 	    bytes = 2;
 	    break;
 	case WDB_BINUNIF_FILE_INT32:
 	    from_file = 1;
+	    /* fall through */
 	case WDB_BINUNIF_INT32:
 	    bytes = 4;
 	    break;
 	case WDB_BINUNIF_FILE_UINT32:
 	    from_file = 1;
+	    /* fall through */
 	case WDB_BINUNIF_UINT32:
 	    nosign = 1;
 	    bytes = 4;
 	    break;
 	case WDB_BINUNIF_FILE_INT64:
 	    from_file = 1;
+	    /* fall through */
 	case WDB_BINUNIF_INT64:
 	    bytes = 8;
 	    break;
 	case WDB_BINUNIF_FILE_UINT64:
 	    from_file = 1;
+	    /* fall through */
 	case WDB_BINUNIF_UINT64:
 	    nosign = 1;
 	    bytes = 8;
@@ -698,66 +707,68 @@ mk_binunif (
 
 	case WDB_BINUNIF_FILE_CHAR:
 	    from_file = 1;
+	    /* fall through */
 	case WDB_BINUNIF_CHAR:
 	    bytes = sizeof(char);
 	    break;
 	case WDB_BINUNIF_FILE_UCHAR:
 	    from_file = 1;
+	    /* fall through */
 	case WDB_BINUNIF_UCHAR:
 	    nosign = 1;
 	    bytes = sizeof(unsigned char);
 	    break;
 	case WDB_BINUNIF_FILE_SHORT:
 	    from_file = 1;
+	    /* fall through */
 	case WDB_BINUNIF_SHORT:
 	    bytes = sizeof(short);
 	    break;
 	case WDB_BINUNIF_FILE_USHORT:
 	    from_file = 1;
+	    /* fall through */
 	case WDB_BINUNIF_USHORT:
 	    nosign = 1;
 	    bytes = sizeof(unsigned short);
 	    break;
 	case WDB_BINUNIF_FILE_INT:
 	    from_file = 1;
+	    /* fall through */
 	case WDB_BINUNIF_INT:
 	    bytes = sizeof(int);
 	    break;
 	case WDB_BINUNIF_FILE_UINT:
 	    from_file = 1;
+	    /* fall through */
 	case WDB_BINUNIF_UINT:
 	    nosign = 1;
 	    bytes = sizeof(unsigned int);
 	    break;
 	case WDB_BINUNIF_FILE_LONG:
 	    from_file = 1;
+	    /* fall through */
 	case WDB_BINUNIF_LONG:
 	    bytes = sizeof(long);
 	    break;
 	case WDB_BINUNIF_FILE_ULONG:
 	    from_file = 1;
+	    /* fall through */
 	case WDB_BINUNIF_ULONG:
 	    nosign = 1;
 	    bytes = sizeof(unsigned long);
 	    break;
 	case WDB_BINUNIF_FILE_LONGLONG:
 	    from_file = 1;
+	    /* fall through */
 	case WDB_BINUNIF_LONGLONG:
-#if defined(_WIN32) && !defined(__CYGWIN__)
-	    bytes = sizeof(__int64);
-#else
-	    bytes = sizeof(long long);
-#endif
+	    bytes = sizeof(int64_t);
 	    break;
 	case WDB_BINUNIF_FILE_ULONGLONG:
 	    from_file = 1;
+	    /* fall through */
 	case WDB_BINUNIF_ULONGLONG:
 	    nosign = 1;
-#if defined(_WIN32) && !defined(__CYGWIN__)
-	    bytes = sizeof(unsigned __int64);
-#else
-	    bytes = sizeof(unsigned long long);
-#endif
+	    bytes = sizeof(uint64_t);
 	    break;
 	default:
 	    bu_log("Unknown binunif data source type: %d", data_type);
@@ -821,8 +832,15 @@ mk_binunif (
     binunif->magic = RT_BINUNIF_INTERNAL_MAGIC;
     binunif->type = minor_type;
     binunif->count = count;
+    binunif->u.int8 = (char *)bu_malloc(count * bytes, "init binunif container");
+
+    RT_DB_INTERNAL_INIT(&intern);
+    intern.idb_major_type = DB5_MAJORTYPE_BINARY_UNIF;
+    intern.idb_type = minor_type;
+    intern.idb_ptr = (void*)binunif;
+    intern.idb_meth = &OBJ[ID_BINUNIF];
     memcpy(binunif->u.int8, data, count * bytes);
-    return wdb_export(wdbp, name, (genptr_t)binunif, ID_BINUNIF, mk_conv2mm);
+    return wdb_put_internal(wdbp, name, &intern, mk_conv2mm);
 }
 
 
