@@ -1,7 +1,7 @@
 /*                           A D C . C
  * BRL-CAD
  *
- * Copyright (c) 1985-2013 United States Government as represented by
+ * Copyright (c) 1985-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This program is free software; you can redistribute it and/or
@@ -24,12 +24,10 @@
 #include "common.h"
 
 #include <stdlib.h>
-#include <stdio.h>
 #include <math.h>
 #include <string.h>
 
-#include "bio.h"
-#include "bu.h"
+#include "bu/vls.h"
 #include "vmath.h"
 #include "ged.h"
 #include "./mged.h"
@@ -78,30 +76,34 @@ static char adc_syntax4[] = "\
 void
 adc_set_dirty_flag(void)
 {
-    struct dm_list *dmlp;
 
-    FOR_ALL_DISPLAYS(dmlp, &head_dm_list.l)
-	if (dmlp->dml_adc_state == adc_state)
-	    dmlp->dml_dirty = 1;
+    for (size_t i = 0; i < BU_PTBL_LEN(&active_dm_set); i++) {
+	struct mged_dm *m_dmp = (struct mged_dm *)BU_PTBL_GET(&active_dm_set, i);
+	if (m_dmp->dm_adc_state == adc_state) {
+	    m_dmp->dm_dirty = 1;
+	    dm_set_dirty(m_dmp->dm_dmp, 1);
+	}
+    }
 }
 
 
 void
 adc_set_scroll(void)
 {
-    struct dm_list *dmlp;
-    struct dm_list *save_dmlp;
+    struct mged_dm *save_m_dmp;
+    save_m_dmp = mged_curr_dm;
 
-    save_dmlp = curr_dm_list;
-
-    FOR_ALL_DISPLAYS(dmlp, &head_dm_list.l)
-	if (dmlp->dml_adc_state == adc_state) {
-	    curr_dm_list = dmlp;
+    for (size_t i = 0; i < BU_PTBL_LEN(&active_dm_set); i++) {
+	struct mged_dm *m_dmp = (struct mged_dm *)BU_PTBL_GET(&active_dm_set, i);
+	if (m_dmp->dm_adc_state == adc_state) {
+	    set_curr_dm(m_dmp);
 	    set_scroll();
-	    dirty = 1;
+	    DMP_dirty = 1;
+	    dm_set_dirty(DMP, 1);
 	}
+    }
 
-    curr_dm_list = save_dmlp;
+    set_curr_dm(save_m_dmp);
 }
 
 
@@ -117,10 +119,9 @@ adc_model_To_adc_view(void)
 static void
 adc_grid_To_adc_view(void)
 {
-    point_t model_pt;
+    point_t model_pt = VINIT_ZERO;
     point_t view_pt;
 
-    VSETALL(model_pt, 0.0);
     MAT4X3PNT(view_pt, view_state->vs_gvp->gv_model2view, model_pt);
     VADD2(adc_state->adc_pos_view, view_pt, adc_state->adc_pos_grid);
     adc_state->adc_dv_x = adc_state->adc_pos_view[X] * GED_MAX;
@@ -131,10 +132,9 @@ adc_grid_To_adc_view(void)
 static void
 adc_view_To_adc_grid(void)
 {
-    point_t model_pt;
+    point_t model_pt = VINIT_ZERO;
     point_t view_pt;
 
-    VSETALL(model_pt, 0.0);
     MAT4X3PNT(view_pt, view_state->vs_gvp->gv_model2view, model_pt);
     VSUB2(adc_state->adc_pos_grid, adc_state->adc_pos_view, view_pt);
 }
@@ -241,9 +241,9 @@ draw_ticks(fastf_t angle)
     x2 = adc_state->adc_dv_x + d1 -t1;
     y2 = adc_state->adc_dv_y + d2 + t2;
     if (clip(&x1, &Y1, &x2, &y2) == 0) {
-	DM_DRAW_LINE_2D(dmp,
-			GED2PM1(x1), GED2PM1(Y1) * dmp->dm_aspect,
-			GED2PM1(x2), GED2PM1(y2) * dmp->dm_aspect);
+	dm_draw_line_2d(DMP,
+			GED2PM1(x1), GED2PM1(Y1) * dm_get_aspect(DMP),
+			GED2PM1(x2), GED2PM1(y2) * dm_get_aspect(DMP));
     }
 
     /* Quadrant 2 */
@@ -252,9 +252,9 @@ draw_ticks(fastf_t angle)
     x2 = adc_state->adc_dv_x - d2 - t2;
     y2 = adc_state->adc_dv_y + d1 - t1;
     if (clip (&x1, &Y1, &x2, &y2) == 0) {
-	DM_DRAW_LINE_2D(dmp,
-			GED2PM1(x1), GED2PM1(Y1) * dmp->dm_aspect,
-			GED2PM1(x2), GED2PM1(y2) * dmp->dm_aspect);
+	dm_draw_line_2d(DMP,
+			GED2PM1(x1), GED2PM1(Y1) * dm_get_aspect(DMP),
+			GED2PM1(x2), GED2PM1(y2) * dm_get_aspect(DMP));
     }
 
     /* Quadrant 3 */
@@ -263,9 +263,9 @@ draw_ticks(fastf_t angle)
     x2 = adc_state->adc_dv_x - d1 + t1;
     y2 = adc_state->adc_dv_y - d2 - t2;
     if (clip (&x1, &Y1, &x2, &y2) == 0) {
-	DM_DRAW_LINE_2D(dmp,
-			GED2PM1(x1), GED2PM1(Y1) * dmp->dm_aspect,
-			GED2PM1(x2), GED2PM1(y2) * dmp->dm_aspect);
+	dm_draw_line_2d(DMP,
+			GED2PM1(x1), GED2PM1(Y1) * dm_get_aspect(DMP),
+			GED2PM1(x2), GED2PM1(y2) * dm_get_aspect(DMP));
     }
 
     /* Quadrant 4 */
@@ -274,16 +274,14 @@ draw_ticks(fastf_t angle)
     x2 = adc_state->adc_dv_x + d2 + t2;
     y2 = adc_state->adc_dv_y - d1 + t1;
     if (clip (&x1, &Y1, &x2, &y2) == 0) {
-	DM_DRAW_LINE_2D(dmp,
-			GED2PM1(x1), GED2PM1(Y1) * dmp->dm_aspect,
-			GED2PM1(x2), GED2PM1(y2) * dmp->dm_aspect);
+	dm_draw_line_2d(DMP,
+			GED2PM1(x1), GED2PM1(Y1) * dm_get_aspect(DMP),
+			GED2PM1(x2), GED2PM1(y2) * dm_get_aspect(DMP));
     }
 }
 
 
 /**
- * A D C U R S O R
- *
  * Compute and display the angle/distance cursor.
  */
 void
@@ -301,19 +299,19 @@ adcursor(void)
     calc_adc_a2();
     calc_adc_dst();
 
-    DM_SET_FGCOLOR(dmp,
+    dm_set_fg(DMP,
 		   color_scheme->cs_adc_line[0],
 		   color_scheme->cs_adc_line[1],
 		   color_scheme->cs_adc_line[2], 1, 1.0);
-    DM_SET_LINE_ATTR(dmp, mged_variables->mv_linewidth, 0);
+    dm_set_line_attr(DMP, mged_variables->mv_linewidth, 0);
 
     /* Horizontal */
-    DM_DRAW_LINE_2D(dmp,
-		    GED2PM1(GED_MIN), GED2PM1(adc_state->adc_dv_y) * dmp->dm_aspect,
-		    GED2PM1(GED_MAX), GED2PM1(adc_state->adc_dv_y) * dmp->dm_aspect);
+    dm_draw_line_2d(DMP,
+		    GED2PM1(GED_MIN), GED2PM1(adc_state->adc_dv_y) * dm_get_aspect(DMP),
+		    GED2PM1(GED_MAX), GED2PM1(adc_state->adc_dv_y) * dm_get_aspect(DMP));
 
     /* Vertical */
-    DM_DRAW_LINE_2D(dmp,
+    dm_draw_line_2d(DMP,
 		    GED2PM1(adc_state->adc_dv_x), GED2PM1(GED_MAX),
 		    GED2PM1(adc_state->adc_dv_x), GED2PM1(GED_MIN));
 
@@ -333,12 +331,12 @@ adcursor(void)
     x4 = adc_state->adc_dv_x - d2;
     y4 = adc_state->adc_dv_y + d1;
 
-    DM_DRAW_LINE_2D(dmp,
-		    GED2PM1(x1), GED2PM1(Y1) * dmp->dm_aspect,
-		    GED2PM1(x2), GED2PM1(y2) * dmp->dm_aspect);
-    DM_DRAW_LINE_2D(dmp,
-		    GED2PM1(x3), GED2PM1(y3) * dmp->dm_aspect,
-		    GED2PM1(x4), GED2PM1(y4) * dmp->dm_aspect);
+    dm_draw_line_2d(DMP,
+		    GED2PM1(x1), GED2PM1(Y1) * dm_get_aspect(DMP),
+		    GED2PM1(x2), GED2PM1(y2) * dm_get_aspect(DMP));
+    dm_draw_line_2d(DMP,
+		    GED2PM1(x3), GED2PM1(y3) * dm_get_aspect(DMP),
+		    GED2PM1(x4), GED2PM1(y4) * dm_get_aspect(DMP));
 
     d1 = cos(angle2) * 8000.0;
     d2 = sin(angle2) * 8000.0;
@@ -352,16 +350,16 @@ adcursor(void)
     x4 = adc_state->adc_dv_x - d2;
     y4 = adc_state->adc_dv_y + d1;
 
-    DM_SET_LINE_ATTR(dmp, mged_variables->mv_linewidth, 1);
-    DM_DRAW_LINE_2D(dmp,
-		    GED2PM1(x1), GED2PM1(Y1) * dmp->dm_aspect,
-		    GED2PM1(x2), GED2PM1(y2) * dmp->dm_aspect);
-    DM_DRAW_LINE_2D(dmp,
-		    GED2PM1(x3), GED2PM1(y3) * dmp->dm_aspect,
-		    GED2PM1(x4), GED2PM1(y4) * dmp->dm_aspect);
-    DM_SET_LINE_ATTR(dmp, mged_variables->mv_linewidth, 0);
+    dm_set_line_attr(DMP, mged_variables->mv_linewidth, 1);
+    dm_draw_line_2d(DMP,
+		    GED2PM1(x1), GED2PM1(Y1) * dm_get_aspect(DMP),
+		    GED2PM1(x2), GED2PM1(y2) * dm_get_aspect(DMP));
+    dm_draw_line_2d(DMP,
+		    GED2PM1(x3), GED2PM1(y3) * dm_get_aspect(DMP),
+		    GED2PM1(x4), GED2PM1(y4) * dm_get_aspect(DMP));
+    dm_set_line_attr(DMP, mged_variables->mv_linewidth, 0);
 
-    DM_SET_FGCOLOR(dmp,
+    dm_set_fg(DMP,
 		   color_scheme->cs_adc_tick[0],
 		   color_scheme->cs_adc_tick[1],
 		   color_scheme->cs_adc_tick[2], 1, 1.0);
@@ -372,7 +370,7 @@ adcursor(void)
 
 
 static void
-adc_reset(void)
+mged_adc_reset(void)
 {
     adc_state->adc_dv_x = adc_state->adc_dv_y = 0;
     adc_state->adc_dv_a1 = adc_state->adc_dv_a2 = 0;
@@ -435,9 +433,6 @@ adc_print_vars(void)
 }
 
 
-/**
- * F _ A D C
- */
 int
 f_adc (
     ClientData UNUSED(clientData),
@@ -470,7 +465,7 @@ f_adc (
 	    adc_state->adc_draw = 1;
 
 	if (adc_auto) {
-	    adc_reset();
+	    mged_adc_reset();
 	    adc_auto = 0;
 	}
 
@@ -1024,7 +1019,7 @@ f_adc (
 
     if (BU_STR_EQUAL(parameter, "reset")) {
 	if (argc == 0) {
-	    adc_reset();
+	    mged_adc_reset();
 
 	    adc_set_dirty_flag();
 	    return TCL_OK;
