@@ -1,7 +1,7 @@
 /*                       B U T T O N S . C
  * BRL-CAD
  *
- * Copyright (c) 1985-2013 United States Government as represented by
+ * Copyright (c) 1985-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This program is free software; you can redistribute it and/or
@@ -23,25 +23,22 @@
 
 #include "common.h"
 
-#include <stdio.h>
 #include <math.h>
 #include <string.h>
 
-#include "bio.h"
-#include "bu.h"
 #include "vmath.h"
-#include "dg.h"
 
 #include "./mged.h"
 #include "./mged_dm.h"
 #include "./sedit.h"
 
+/* external sp_hook function */
+extern void set_scroll_private(const struct bu_structparse *, const char *, void *, const char *, void *);	/* defined in set.c */
 
 extern int mged_svbase(void);
 extern void set_e_axes_pos(int both);
 extern int mged_zoom(double val);
 extern void set_absolute_tran(void);	/* defined in set.c */
-extern void set_scroll_private(void);	/* defined in set.c */
 extern void adc_set_scroll(void);	/* defined in adc.c */
 
 /* forward declarations for the buttons table */
@@ -150,9 +147,9 @@ struct buttons {
 
 static mat_t sav_viewrot, sav_toviewcenter;
 static fastf_t sav_vscale;
-static int vsaved = 0;	/* set iff view saved */
+static int vsaved = 0;	/* set if view saved */
 
-extern void color_soltab(void);
+extern void mged_color_soltab(void);
 extern void sl_halt_scroll(void);	/* in scroll.c */
 extern void sl_toggle_scroll(void);
 
@@ -209,9 +206,6 @@ struct menu_item oed_menu[] = {
 };
 
 
-/*
- * B U T T O N
- */
 void
 button(int bnum)
 {
@@ -234,8 +228,6 @@ button(int bnum)
 
 
 /*
- * F _ P R E S S
- *
  * Hook for displays with no buttons
  *
  * Given a string description of which button to press, simulate
@@ -292,9 +284,9 @@ f_press(ClientData clientData,
 	    goto next;
 	}
 
-	for (menu=0, m=menu_state->ms_menus; m - menu_state->ms_menus < NMENU; m++, menu++) {
+	for (menu = 0, m = menu_state->ms_menus; m - menu_state->ms_menus < NMENU; m++, menu++) {
 	    if (*m == MENU_NULL) continue;
-	    for (item=0, mptr = *m;
+	    for (item = 0, mptr = *m;
 		 mptr->menu_string[0] != '\0';
 		 mptr++, item++) {
 		if (!BU_STR_EQUAL(str, mptr->menu_string))
@@ -303,7 +295,7 @@ f_press(ClientData clientData,
 		menu_state->ms_cur_item = item;
 		menu_state->ms_cur_menu = menu;
 		menu_state->ms_flag = 1;
-		/* It's up to the menu_func to set menu_state->ms_flag=0
+		/* It's up to the menu_func to set menu_state->ms_flag = 0
 		 * if no arrow is desired */
 		if (mptr->menu_func != ((void (*)())0))
 		    (*(mptr->menu_func))(mptr->menu_arg, menu, item);
@@ -322,8 +314,6 @@ f_press(ClientData clientData,
 
 
 /*
- * L A B E L _ B U T T O N
- *
  * For a given GED button number, return the "press" ID string.
  * Useful for displays with programmable button labels, etc.
  */
@@ -371,7 +361,16 @@ int
 bv_rate_toggle()
 {
     mged_variables->mv_rateknobs = !mged_variables->mv_rateknobs;
-    set_scroll_private();
+
+    {
+	/* need dummy values for func signature--they are unused in the func */
+	const struct bu_structparse *sdp = 0;
+	const char name[] = "name";
+	void *base = 0;
+	const char value[] = "value";
+	set_scroll_private(sdp, name, base, value, NULL);
+    }
+
     return TCL_OK;
 }
 
@@ -460,8 +459,6 @@ bv_vsave()
 
 
 /*
- * B V _ A D C U R S O R
- *
  * Toggle state of angle/distance cursor.
  * "press adc"
  * This command conflicts with existing "adc" command,
@@ -511,16 +508,16 @@ bv_35_25() {
 /* returns 0 if error, !0 if success */
 static int
 ill_common(void) {
-    struct ged_display_list *gdlp;
-    struct ged_display_list *next_gdlp;
+    struct display_list *gdlp;
+    struct display_list *next_gdlp;
     int is_empty = 1;
 
     /* Common part of illumination */
-    gdlp = BU_LIST_NEXT(ged_display_list, gedp->ged_gdp->gd_headDisplay);
-    while (BU_LIST_NOT_HEAD(gdlp, gedp->ged_gdp->gd_headDisplay)) {
-	next_gdlp = BU_LIST_PNEXT(ged_display_list, gdlp);
+    gdlp = BU_LIST_NEXT(display_list, GEDP->ged_gdp->gd_headDisplay);
+    while (BU_LIST_NOT_HEAD(gdlp, GEDP->ged_gdp->gd_headDisplay)) {
+	next_gdlp = BU_LIST_PNEXT(display_list, gdlp);
 
-	if (BU_LIST_NON_EMPTY(&gdlp->gdl_headSolid)) {
+	if (BU_LIST_NON_EMPTY(&gdlp->dl_headSolid)) {
 	    is_empty = 0;
 	    break;
 	}
@@ -534,7 +531,7 @@ ill_common(void) {
     }
 
     illum_gdlp = gdlp;
-    illump = BU_LIST_NEXT(solid, &gdlp->gdl_headSolid);/* any valid solid would do */
+    illump = BU_LIST_NEXT(solid, &gdlp->dl_headSolid);/* any valid solid would do */
     illump->s_iflag = UP;
     edobj = 0;		/* sanity */
     edsol = 0;		/* sanity */
@@ -585,6 +582,7 @@ be_o_scale()
     edobj = BE_O_SCALE;
     movedir = SARROW;
     update_views = 1;
+    dm_set_dirty(DMP, 1);
     set_e_axes_pos(1);
 
     edit_absolute_scale = acc_sc_obj - 1.0;
@@ -603,6 +601,7 @@ be_o_xscale()
     edobj = BE_O_XSCALE;
     movedir = SARROW;
     update_views = 1;
+    dm_set_dirty(DMP, 1);
     set_e_axes_pos(1);
 
     edit_absolute_scale = acc_sc[0] - 1.0;
@@ -621,6 +620,7 @@ be_o_yscale()
     edobj = BE_O_YSCALE;
     movedir = SARROW;
     update_views = 1;
+    dm_set_dirty(DMP, 1);
     set_e_axes_pos(1);
 
     edit_absolute_scale = acc_sc[1] - 1.0;
@@ -639,6 +639,7 @@ be_o_zscale()
     edobj = BE_O_ZSCALE;
     movedir = SARROW;
     update_views = 1;
+    dm_set_dirty(DMP, 1);
     set_e_axes_pos(1);
 
     edit_absolute_scale = acc_sc[2] - 1.0;
@@ -657,6 +658,7 @@ be_o_x()
     edobj = BE_O_X;
     movedir = RARROW;
     update_views = 1;
+    dm_set_dirty(DMP, 1);
     set_e_axes_pos(1);
     return TCL_OK;
 }
@@ -671,6 +673,7 @@ be_o_y()
     edobj = BE_O_Y;
     movedir = UARROW;
     update_views = 1;
+    dm_set_dirty(DMP, 1);
     set_e_axes_pos(1);
     return TCL_OK;
 }
@@ -685,6 +688,7 @@ be_o_xy()
     edobj = BE_O_XY;
     movedir = UARROW | RARROW;
     update_views = 1;
+    dm_set_dirty(DMP, 1);
     set_e_axes_pos(1);
     return TCL_OK;
 }
@@ -699,6 +703,7 @@ be_o_rotate()
     edobj = BE_O_ROTATE;
     movedir = ROTARROW;
     update_views = 1;
+    dm_set_dirty(DMP, 1);
     set_e_axes_pos(1);
     return TCL_OK;
 }
@@ -707,11 +712,6 @@ be_o_rotate()
 int
 be_accept()
 {
-    struct ged_display_list *gdlp;
-    struct ged_display_list *next_gdlp;
-    struct solid *sp;
-    struct dm_list *dmlp;
-
     if (STATE == ST_S_EDIT) {
 	/* Accept a solid edit */
 	edsol = 0;
@@ -721,19 +721,11 @@ be_accept()
 	mmenu_set_all(MENU_L1, MENU_NULL);
 	mmenu_set_all(MENU_L2, MENU_NULL);
 
-	gdlp = BU_LIST_NEXT(ged_display_list, gedp->ged_gdp->gd_headDisplay);
-	while (BU_LIST_NOT_HEAD(gdlp, gedp->ged_gdp->gd_headDisplay)) {
-	    next_gdlp = BU_LIST_PNEXT(ged_display_list, gdlp);
-
-	    FOR_ALL_SOLIDS(sp, &gdlp->gdl_headSolid)
-		sp->s_iflag = DOWN;
-
-	    gdlp = next_gdlp;
-	}
+	dl_set_iflag(GEDP->ged_gdp->gd_headDisplay, DOWN);
 
 	illum_gdlp = GED_DISPLAY_LIST_NULL;
 	illump = SOLID_NULL;
-	color_soltab();
+	mged_color_soltab();
 	(void)chg_state(ST_S_EDIT, ST_VIEW, "Edit Accept");
     }  else if (STATE == ST_O_EDIT) {
 	/* Accept an object edit */
@@ -746,7 +738,7 @@ be_accept()
 
 	illum_gdlp = GED_DISPLAY_LIST_NULL;
 	illump = SOLID_NULL;
-	color_soltab();
+	mged_color_soltab();
 	(void)chg_state(ST_O_EDIT, ST_VIEW, "Edit Accept");
     } else {
 	if (not_state(ST_S_EDIT, "Edit Accept"))
@@ -754,9 +746,11 @@ be_accept()
 	return TCL_OK;
     }
 
-    FOR_ALL_DISPLAYS(dmlp, &head_dm_list.l)
-	if (dmlp->dml_mged_variables->mv_transform == 'e')
-	    dmlp->dml_mged_variables->mv_transform = 'v';
+    for (size_t i = 0; i < BU_PTBL_LEN(&active_dm_set); i++) {
+	struct mged_dm *m_dmp = (struct mged_dm *)BU_PTBL_GET(&active_dm_set, i);
+	if (m_dmp->dm_mged_variables->mv_transform == 'e')
+	    m_dmp->dm_mged_variables->mv_transform = 'v';
+    }
 
     {
 	struct bu_vls vls = BU_VLS_INIT_ZERO;
@@ -772,12 +766,8 @@ be_accept()
 int
 be_reject()
 {
-    struct ged_display_list *gdlp;
-    struct ged_display_list *next_gdlp;
-    struct solid *sp;
-    struct dm_list *dmlp;
-
     update_views = 1;
+    dm_set_dirty(DMP, 1);
 
     /* Reject edit */
 
@@ -816,22 +806,16 @@ be_reject()
     illump = SOLID_NULL;		/* None selected */
 
     /* Clear illumination flags */
-    gdlp = BU_LIST_NEXT(ged_display_list, gedp->ged_gdp->gd_headDisplay);
-    while (BU_LIST_NOT_HEAD(gdlp, gedp->ged_gdp->gd_headDisplay)) {
-	next_gdlp = BU_LIST_PNEXT(ged_display_list, gdlp);
+    dl_set_iflag(GEDP->ged_gdp->gd_headDisplay, DOWN);
 
-	FOR_ALL_SOLIDS(sp, &gdlp->gdl_headSolid)
-	    sp->s_iflag = DOWN;
-
-	gdlp = next_gdlp;
-    }
-
-    color_soltab();
+    mged_color_soltab();
     (void)chg_state(STATE, ST_VIEW, "Edit Reject");
 
-    FOR_ALL_DISPLAYS(dmlp, &head_dm_list.l)
-	if (dmlp->dml_mged_variables->mv_transform == 'e')
-	    dmlp->dml_mged_variables->mv_transform = 'v';
+    for (size_t i = 0; i < BU_PTBL_LEN(&active_dm_set); i++) {
+	struct mged_dm *m_dmp = (struct mged_dm *)BU_PTBL_GET(&active_dm_set, i);
+	if (m_dmp->dm_mged_variables->mv_transform == 'e')
+	    m_dmp->dm_mged_variables->mv_transform = 'v';
+    }
 
     {
 	struct bu_vls vls = BU_VLS_INIT_ZERO;
@@ -908,8 +892,6 @@ be_s_scale()
 
 
 /*
- * N O T _ S T A T E
- *
  * Returns 0 if current state is as desired,
  * Returns !0 and prints error message if state mismatch.
  */
@@ -960,16 +942,13 @@ stateChange(int UNUSED(oldstate), int newstate)
 
 
 /*
- * C H G _ S T A T E
- *
  * Returns 0 if state change is OK,
  * Returns !0 and prints error message if error.
  */
 int
 chg_state(int from, int to, char *str)
 {
-    struct dm_list *p;
-    struct dm_list *save_dm_list;
+    struct mged_dm *save_dm_list;
     struct bu_vls vls = BU_VLS_INIT_ZERO;
 
     if (STATE != from) {
@@ -981,14 +960,15 @@ chg_state(int from, int to, char *str)
 
     stateChange(from, to);
 
-    save_dm_list = curr_dm_list;
-    FOR_ALL_DISPLAYS(p, &head_dm_list.l) {
-	curr_dm_list = p;
+    save_dm_list = mged_curr_dm;
+    for (size_t i = 0; i < BU_PTBL_LEN(&active_dm_set); i++) {
+	struct mged_dm *p = (struct mged_dm *)BU_PTBL_GET(&active_dm_set, i);
+	set_curr_dm(p);
 
 	new_mats();
     }
 
-    curr_dm_list = save_dm_list;
+    set_curr_dm(save_dm_list);
 
     bu_vls_printf(&vls, "%s(state)", MGED_DISPLAY_VAR);
     Tcl_SetVar(INTERP, bu_vls_addr(&vls), state_str[STATE], TCL_GLOBAL_ONLY);
@@ -1007,8 +987,6 @@ state_err(char *str)
 
 
 /*
- * B T N _ I T E M _ H I T
- *
  * Called when a menu item is hit
  */
 void
@@ -1022,8 +1000,6 @@ btn_item_hit(int arg, int menu, int UNUSED(item))
 
 
 /*
- * B T N _ H E A D _ M E N U
- *
  * Called to handle hits on menu heads.
  * Also called from main() with arg 0 in init.
  */
