@@ -1,7 +1,7 @@
 /*                    C A D _ B O U N D P . C
  * BRL-CAD
  *
- * Copyright (c) 2004-2013 United States Government as represented by
+ * Copyright (c) 2004-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This program is free software; you can redistribute it and/or
@@ -41,8 +41,12 @@
 #include <string.h>
 #include <stdarg.h>
 
-#include "bu.h"
 #include "vmath.h"
+#include "bu/app.h"
+#include "bu/getopt.h"
+#include "bu/str.h"
+#include "bu/exit.h"
+#include "bu/log.h"
 
 
 typedef struct
@@ -107,6 +111,8 @@ main(int argc, const char *argv[])			/* "cad_boundp" entry point */
     /* argument count */
     /* argument strings */
 {
+    bu_setprogname(argv[0]);
+
     if (!GetArgs(argc, argv))	/* process command arguments */
 	return 1;
 
@@ -133,11 +139,6 @@ GetArgs(int argc, const char *argv[])	/* process command arguments */
     static int tflag = 0;	/* set if "-t" option found */
     int c;		/* option letter */
 
-if (argc==1) {
-    Usage();
-    fprintf(stderr,"       Program continues running:\n");
-}
-
 #ifdef DEBUG
     fprintf(stderr, "\n\t\tGetArgs\n");
 #endif
