@@ -1,7 +1,7 @@
 /*                       D O E V E N T . C
  * BRL-CAD
  *
- * Copyright (c) 2004-2013 United States Government as represented by
+ * Copyright (c) 2004-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This program is free software; you can redistribute it and/or
@@ -26,7 +26,6 @@
 #include "common.h"
 
 #include <stdlib.h>
-#include <stdio.h>
 #include <math.h>
 
 #ifdef HAVE_GL_DEVICE_H
@@ -40,17 +39,16 @@
 
 #ifdef HAVE_X11_XLIB_H
 #  include <X11/Xutil.h>
-#  include <X11/extensions/XI.h>
-#  include <X11/extensions/XInput.h>
 #  include <X11/keysym.h>
 #endif
+#ifdef HAVE_X11_EXTENSIONS_XINPUT_H
+#  include <X11/extensions/XI.h>
+#  include <X11/extensions/XInput.h>
+#endif
 
-#include "bio.h"
-#include "bu.h"
 #include "vmath.h"
-#include "mater.h"
+#include "raytrace.h"
 #include "ged.h"
-#include "dm_xvars.h"
 
 #include "./mged.h"
 #include "./mged_dm.h"
@@ -62,11 +60,18 @@ extern void rect_view2image();		/* defined in rect.c */
 extern void rect_image2view();
 extern void rb_set_dirty_flag();
 
+#ifdef HAVE_X11_TYPES
 HIDDEN void motion_event_handler();
+#endif
+
 #ifdef IR_KNOBS
 HIDDEN void dials_event_handler();
 #endif
 
+#ifdef IR_KNOBS
+#  define NOISE 16              /* Size of dead spot on knob */
+#endif
+
 #ifdef IR_BUTTONS
 HIDDEN void buttons_event_handler();
 /*
@@ -113,31 +118,28 @@ static int button0 = 0;
 int
 doEvent(ClientData clientData, XEvent *eventPtr)
 {
-    struct dm_list *save_dm_list;
+    struct mged_dm *save_dm_list;
     int status;
 
     if (eventPtr->type == DestroyNotify || (unsigned long)eventPtr->xany.window == 0)
 	return TCL_OK;
 
-    save_dm_list = curr_dm_list;
-    GET_DM_LIST(curr_dm_list, (unsigned long)eventPtr->xany.window);
+    save_dm_list = mged_curr_dm;
+    GET_MGED_DM(mged_curr_dm, (unsigned long)eventPtr->xany.window);
 
     /* it's an event for a window that I'm not handling */
-    if (curr_dm_list == DM_LIST_NULL) {
-	curr_dm_list = save_dm_list;
+    if (mged_curr_dm == MGED_DM_NULL) {
+	set_curr_dm(save_dm_list);
 	return TCL_OK;
     }
 
-    /* XXX - windows fix, for now */
-    if (eventHandler == NULL)
-	return TCL_CONTINUE;
-
     /* calling the display manager specific event handler */
-    status = eventHandler(clientData, eventPtr);
+    status = dm_doevent(DMP, clientData, eventPtr);
+    DMP_dirty = dm_get_dirty(DMP);
 
     /* no further processing of this event */
     if (status != TCL_OK) {
-	curr_dm_list = save_dm_list;
+	set_curr_dm(save_dm_list);
 	return status;
     }
 
@@ -146,63 +148,66 @@ doEvent(ClientData clientData, XEvent *eventPtr)
     if (eventPtr->type == ConfigureNotify) {
 	XConfigureEvent *conf = (XConfigureEvent *)eventPtr;
 
-	DM_CONFIGURE_WIN(dmp, 0);
+	dm_configure_win(DMP, 0);
 	rect_image2view();
-	dirty = 1;
+	DMP_dirty = 1;
+	dm_set_dirty(DMP, 1);
 
 	if (fbp)
-	    fb_configureWindow(fbp, conf->width, conf->height);
+	    (void)fb_configure_window(fbp, conf->width, conf->height);
 
 	/* no further processing of this event */
 	status = TCL_RETURN;
     } else if (eventPtr->type == MapNotify) {
 	mapped = 1;
+	dm_set_dirty(DMP, 1);
 
 	/* no further processing of this event */
 	status = TCL_RETURN;
     } else if (eventPtr->type == UnmapNotify) {
 	mapped = 0;
+	dm_set_dirty(DMP, 1);
 
 	/* no further processing of this event */
 	status = TCL_RETURN;
     } else if (eventPtr->type == MotionNotify) {
 	motion_event_handler((XMotionEvent *)eventPtr);
+	dm_set_dirty(DMP, 1);
 
 	/* no further processing of this event */
 	status = TCL_RETURN;
     }
 #ifdef IR_KNOBS
-    else if (eventPtr->type == ((struct dm_xvars *)dmp->dm_vars.pub_vars)->devmotionnotify) {
+    else if (dm_event_cmp(DMP, DM_MOTION_NOTIFY, eventPtr->type) == 1) {
 	dials_event_handler((XDeviceMotionEvent *)eventPtr);
+	dm_set_dirty(DMP, 1);
 
 	/* no further processing of this event */
 	status = TCL_RETURN;
     }
 #endif
 #ifdef IR_BUTTONS
-    else if (eventPtr->type == ((struct dm_xvars *)dmp->dm_vars.pub_vars)->devbuttonpress) {
+    else if (dm_event_cmp(DMP, DM_BUTTON_PRESS, eventPtr->type) == 1) {
 	buttons_event_handler((XDeviceButtonEvent *)eventPtr, 1);
+	dm_set_dirty(DMP, 1);
 
 	/* no further processing of this event */
 	status = TCL_RETURN;
-    } else if (eventPtr->type == ((struct dm_xvars *)dmp->dm_vars.pub_vars)->devbuttonrelease) {
+    } else if (dm_event_cmp(DMP, DM_BUTTON_RELEASE, eventPtr->type) == 1) {
 	buttons_event_handler((XDeviceButtonEvent *)eventPtr, 0);
+	dm_set_dirty(DMP, 1);
 
 	/* no further processing of this event */
 	status = TCL_RETURN;
     }
 #endif
 
-    curr_dm_list = save_dm_list;
+    set_curr_dm(save_dm_list);
     return status;
 }
 #else
 int
-#if !defined(IF_X) && !defined(IF_WGL) && !defined(IF_OGL) && !defined(IF_TK)
 doEvent(ClientData UNUSED(clientData), void *UNUSED(eventPtr)) {
-#else
-doEvent(ClientData clientData, void *eventPtr) {
-#endif
     return TCL_OK;
 }
 #endif /* HAVE_X11_XLIB_H */
@@ -215,7 +220,7 @@ set_knob_offset()
     int i;
 
     for (i = 0; i < 8; ++i)
-	dml_knobs[i] = 0;
+	dm_knobs[i] = 0;
 }
 #endif
 
@@ -236,29 +241,32 @@ motion_event_handler(XMotionEvent *xmotion)
     int save_edflag = -1;
     int mx, my;
     int dx, dy;
+    int width, height;
     fastf_t f;
     fastf_t fx, fy;
     fastf_t td;
 
-    if (dbip == DBI_NULL)
+    if (DBIP == DBI_NULL)
 	return;
 
+    width = dm_get_width(DMP);
+    height = dm_get_height(DMP);
     mx = xmotion->x;
     my = xmotion->y;
-    dx = mx - dml_omx;
-    dy = my - dml_omy;
+    dx = mx - dm_omx;
+    dy = my - dm_omy;
 
     switch (am_mode) {
 	case AMM_IDLE:
 	    if (scroll_active)
 		bu_vls_printf(&cmd, "M 1 %d %d\n",
-			      (int)(dm_Xx2Normal(dmp, mx) * GED_MAX),
-			      (int)(dm_Xy2Normal(dmp, my, 0) * GED_MAX));
+			      (int)(dm_Xx2Normal(DMP, mx) * GED_MAX),
+			      (int)(dm_Xy2Normal(DMP, my, 0) * GED_MAX));
 	    else if (rubber_band->rb_active) {
-		fastf_t x = dm_Xx2Normal(dmp, mx);
-		fastf_t y = dm_Xy2Normal(dmp, my, 1);
+		fastf_t x = dm_Xx2Normal(DMP, mx);
+		fastf_t y = dm_Xy2Normal(DMP, my, 1);
 
-		if (grid_state->gr_snap)
+		if (grid_state->snap)
 		    snap_to_grid(&x, &y);
 
 		rubber_band->rb_width = x - rubber_band->rb_x;
@@ -270,15 +278,23 @@ motion_event_handler(XMotionEvent *xmotion)
 		 * differences caused by floating point fuzz.
 		 */
 		rect_image2view();
-		rb_set_dirty_flag();
+
+		{
+		    /* need dummy values for func signature--they are unused in the func */
+		    const struct bu_structparse *sdp = 0;
+		    const char name[] = "name";
+		    void *base = 0;
+		    const char value[] = "value";
+		    rb_set_dirty_flag(sdp, name, base, value, NULL);
+		}
 
 		goto handled;
 	    } else if (doMotion)
 		/* do the regular thing */
 		/* Constant tracking (e.g. illuminate mode) bound to M mouse */
 		bu_vls_printf(&cmd, "M 0 %d %d\n",
-			      (int)(dm_Xx2Normal(dmp, mx) * GED_MAX),
-			      (int)(dm_Xy2Normal(dmp, my, 0) * GED_MAX));
+			      (int)(dm_Xx2Normal(DMP, mx) * GED_MAX),
+			      (int)(dm_Xy2Normal(DMP, my, 0) * GED_MAX));
 	    else /* not doing motion */
 		goto handled;
 
@@ -304,16 +320,16 @@ motion_event_handler(XMotionEvent *xmotion)
 
 		    if (mged_variables->mv_rateknobs)
 			bu_vls_printf(&cmd, "knob -i x %lf y %lf\n",
-				      dy / (fastf_t)dmp->dm_height * RATE_ROT_FACTOR * 2.0,
-				      dx / (fastf_t)dmp->dm_width * RATE_ROT_FACTOR * 2.0);
+				      dy / (fastf_t)height * RATE_ROT_FACTOR * 2.0,
+				      dx / (fastf_t)width * RATE_ROT_FACTOR * 2.0);
 		    else
 			bu_vls_printf(&cmd, "knob -i ax %lf ay %lf\n",
 				      dy * 0.25, dx * 0.25);
 		} else {
 		    if (mged_variables->mv_rateknobs)
 			bu_vls_printf(&cmd, "knob -i -v x %lf y %lf\n",
-				      dy / (fastf_t)dmp->dm_height * RATE_ROT_FACTOR * 2.0,
-				      dx / (fastf_t)dmp->dm_width * RATE_ROT_FACTOR * 2.0);
+				      dy / (fastf_t)height * RATE_ROT_FACTOR * 2.0,
+				      dx / (fastf_t)width * RATE_ROT_FACTOR * 2.0);
 		    else
 			bu_vls_printf(&cmd, "knob -i -v ax %lf ay %lf\n",
 				      dy * 0.25, dx * 0.25);
@@ -331,8 +347,8 @@ motion_event_handler(XMotionEvent *xmotion)
 		save_coords = mged_variables->mv_coords;
 		mged_variables->mv_coords = 'v';
 
-		fx = dx / (fastf_t)dmp->dm_width * 2.0;
-		fy = -dy / (fastf_t)dmp->dm_height / dmp->dm_aspect * 2.0;
+		fx = dx / (fastf_t)width * 2.0;
+		fy = -dy / (fastf_t)height / dm_get_aspect(DMP) * 2.0;
 
 		if ((STATE == ST_S_EDIT || STATE == ST_O_EDIT) &&
 		    mged_variables->mv_transform == 'e') {
@@ -348,17 +364,17 @@ motion_event_handler(XMotionEvent *xmotion)
 
 		    if (mged_variables->mv_rateknobs)
 			bu_vls_printf(&cmd, "knob -i X %lf Y %lf\n", fx, fy);
-		    else if (grid_state->gr_snap) {
+		    else if (grid_state->snap) {
 			point_t view_pt;
 			point_t model_pt;
 			point_t vcenter, diff;
 
 			/* accumulate distance mouse moved since starting to translate */
-			dml_mouse_dx += dx;
-			dml_mouse_dy += dy;
+			dm_mouse_dx += dx;
+			dm_mouse_dy += dy;
 
-			view_pt[X] = dml_mouse_dx / (fastf_t)dmp->dm_width * 2.0;
-			view_pt[Y] = -dml_mouse_dy / (fastf_t)dmp->dm_height / dmp->dm_aspect * 2.0;
+			view_pt[X] = dm_mouse_dx / (fastf_t)width * 2.0;
+			view_pt[Y] = -dm_mouse_dy / (fastf_t)height / dm_get_aspect(DMP) * 2.0;
 			view_pt[Z] = 0.0;
 			round_to_grid(&view_pt[X], &view_pt[Y]);
 
@@ -366,7 +382,7 @@ motion_event_handler(XMotionEvent *xmotion)
 			MAT_DELTAS_GET_NEG(vcenter, view_state->vs_gvp->gv_center);
 			VSUB2(diff, model_pt, vcenter);
 			VSCALE(diff, diff, base2local);
-			VADD2(model_pt, dml_work_pt, diff);
+			VADD2(model_pt, dm_work_pt, diff);
 			if (STATE == ST_S_EDIT)
 			    bu_vls_printf(&cmd, "p %lf %lf %lf", model_pt[X], model_pt[Y], model_pt[Z]);
 			else
@@ -378,13 +394,13 @@ motion_event_handler(XMotionEvent *xmotion)
 		    if (mged_variables->mv_rateknobs)      /* otherwise, drag to translate the view */
 			bu_vls_printf(&cmd, "knob -i -v X %lf Y %lf\n", fx, fy);
 		    else {
-			if (grid_state->gr_snap) {
+			if (grid_state->snap) {
 			    /* accumulate distance mouse moved since starting to translate */
-			    dml_mouse_dx += dx;
-			    dml_mouse_dy += dy;
+			    dm_mouse_dx += dx;
+			    dm_mouse_dy += dy;
 
-			    snap_view_to_grid(dml_mouse_dx / (fastf_t)dmp->dm_width * 2.0,
-					      -dml_mouse_dy / (fastf_t)dmp->dm_height / dmp->dm_aspect * 2.0);
+			    snap_view_to_grid(dm_mouse_dx / (fastf_t)width * 2.0,
+					      -dm_mouse_dy / (fastf_t)height / dm_get_aspect(DMP) * 2.0);
 
 			    mged_variables->mv_coords = save_coords;
 			    goto handled;
@@ -417,20 +433,20 @@ motion_event_handler(XMotionEvent *xmotion)
 		f = -dy;
 
 	    if (mged_variables->mv_rateknobs)
-		bu_vls_printf(&cmd, "knob -i S %f\n", f / (fastf_t)dmp->dm_height);
+		bu_vls_printf(&cmd, "knob -i S %f\n", f / height);
 	    else
-		bu_vls_printf(&cmd, "knob -i aS %f\n", f / (fastf_t)dmp->dm_height);
+		bu_vls_printf(&cmd, "knob -i aS %f\n", f / height);
 
 	    break;
 	case AMM_ADC_ANG1:
-	    fx = dm_Xx2Normal(dmp, mx) * GED_MAX - adc_state->adc_dv_x;
-	    fy = dm_Xy2Normal(dmp, my, 1) * GED_MAX - adc_state->adc_dv_y;
+	    fx = dm_Xx2Normal(DMP, mx) * GED_MAX - adc_state->adc_dv_x;
+	    fy = dm_Xy2Normal(DMP, my, 1) * GED_MAX - adc_state->adc_dv_y;
 	    bu_vls_printf(&cmd, "adc a1 %lf\n", RAD2DEG*atan2(fy, fx));
 
 	    break;
 	case AMM_ADC_ANG2:
-	    fx = dm_Xx2Normal(dmp, mx) * GED_MAX - adc_state->adc_dv_x;
-	    fy = dm_Xy2Normal(dmp, my, 1) * GED_MAX - adc_state->adc_dv_y;
+	    fx = dm_Xx2Normal(DMP, mx) * GED_MAX - adc_state->adc_dv_x;
+	    fy = dm_Xy2Normal(DMP, my, 1) * GED_MAX - adc_state->adc_dv_y;
 	    bu_vls_printf(&cmd, "adc a2 %lf\n", RAD2DEG*atan2(fy, fx));
 
 	    break;
@@ -439,9 +455,9 @@ motion_event_handler(XMotionEvent *xmotion)
 		point_t model_pt;
 		point_t view_pt;
 
-		VSET(view_pt, dm_Xx2Normal(dmp, mx), dm_Xy2Normal(dmp, my, 1), 0.0);
+		VSET(view_pt, dm_Xx2Normal(DMP, mx), dm_Xy2Normal(DMP, my, 1), 0.0);
 
-		if (grid_state->gr_snap)
+		if (grid_state->snap)
 		    snap_to_grid(&view_pt[X], &view_pt[Y]);
 
 		MAT4X3PNT(model_pt, view_state->vs_gvp->gv_view2model, view_pt);
@@ -451,8 +467,8 @@ motion_event_handler(XMotionEvent *xmotion)
 
 	    break;
 	case AMM_ADC_DIST:
-	    fx = (dm_Xx2Normal(dmp, mx) * GED_MAX - adc_state->adc_dv_x) * view_state->vs_gvp->gv_scale * base2local * INV_GED;
-	    fy = (dm_Xy2Normal(dmp, my, 1) * GED_MAX - adc_state->adc_dv_y) * view_state->vs_gvp->gv_scale * base2local * INV_GED;
+	    fx = (dm_Xx2Normal(DMP, mx) * GED_MAX - adc_state->adc_dv_x) * view_state->vs_gvp->gv_scale * base2local * INV_GED;
+	    fy = (dm_Xy2Normal(DMP, my, 1) * GED_MAX - adc_state->adc_dv_y) * view_state->vs_gvp->gv_scale * base2local * INV_GED;
 	    td = sqrt(fx * fx + fy * fy);
 	    bu_vls_printf(&cmd, "adc dst %lf\n", td);
 
@@ -477,7 +493,7 @@ motion_event_handler(XMotionEvent *xmotion)
 
 	    if (mged_variables->mv_rateknobs)
 		bu_vls_printf(&cmd, "knob -i x %f\n",
-			      f / (fastf_t)dmp->dm_width * RATE_ROT_FACTOR * 2.0);
+			      f / (fastf_t)width * RATE_ROT_FACTOR * 2.0);
 	    else
 		bu_vls_printf(&cmd, "knob -i ax %f\n", f * 0.25);
 
@@ -502,7 +518,7 @@ motion_event_handler(XMotionEvent *xmotion)
 
 	    if (mged_variables->mv_rateknobs)
 		bu_vls_printf(&cmd, "knob -i y %f\n",
-			      f / (fastf_t)dmp->dm_width * RATE_ROT_FACTOR * 2.0);
+			      f / (fastf_t)width * RATE_ROT_FACTOR * 2.0);
 	    else
 		bu_vls_printf(&cmd, "knob -i ay %f\n", f * 0.25);
 
@@ -527,7 +543,7 @@ motion_event_handler(XMotionEvent *xmotion)
 
 	    if (mged_variables->mv_rateknobs)
 		bu_vls_printf(&cmd, "knob -i z %f\n",
-			      f / (fastf_t)dmp->dm_width * RATE_ROT_FACTOR * 2.0);
+			      f / (fastf_t)width * RATE_ROT_FACTOR * 2.0);
 	    else
 		bu_vls_printf(&cmd, "knob -i az %f\n", f * 0.25);
 
@@ -546,9 +562,9 @@ motion_event_handler(XMotionEvent *xmotion)
 	    }
 
 	    if (abs(dx) >= abs(dy))
-		f = dx / (fastf_t)dmp->dm_width * 2.0;
+		f = dx / (fastf_t)width * 2.0;
 	    else
-		f = -dy / (fastf_t)dmp->dm_height / dmp->dm_aspect * 2.0;
+		f = -dy / (fastf_t)height / dm_get_aspect(DMP) * 2.0;
 
 	    if (mged_variables->mv_rateknobs)
 		bu_vls_printf(&cmd, "knob -i X %f\n", f);
@@ -570,9 +586,9 @@ motion_event_handler(XMotionEvent *xmotion)
 	    }
 
 	    if (abs(dx) >= abs(dy))
-		f = dx / (fastf_t)dmp->dm_width * 2.0;
+		f = dx / (fastf_t)width * 2.0;
 	    else
-		f = -dy / (fastf_t)dmp->dm_height / dmp->dm_aspect * 2.0;
+		f = -dy / (fastf_t)height / dm_get_aspect(DMP) * 2.0;
 
 	    if (mged_variables->mv_rateknobs)
 		bu_vls_printf(&cmd, "knob -i Y %f\n", f);
@@ -594,9 +610,9 @@ motion_event_handler(XMotionEvent *xmotion)
 	    }
 
 	    if (abs(dx) >= abs(dy))
-		f = dx / (fastf_t)dmp->dm_width * 2.0;
+		f = dx / (fastf_t)width * 2.0;
 	    else
-		f = -dy / (fastf_t)dmp->dm_height / dmp->dm_aspect * 2.0;
+		f = -dy / height / dm_get_aspect(DMP) * 2.0;
 
 	    if (mged_variables->mv_rateknobs)
 		bu_vls_printf(&cmd, "knob -i Z %f\n", f);
@@ -623,9 +639,9 @@ motion_event_handler(XMotionEvent *xmotion)
 		f = -dy;
 
 	    if (mged_variables->mv_rateknobs)
-		bu_vls_printf(&cmd, "knob -i S %f\n", f / (fastf_t)dmp->dm_height);
+		bu_vls_printf(&cmd, "knob -i S %f\n", f / height);
 	    else
-		bu_vls_printf(&cmd, "knob -i aS %f\n", f / (fastf_t)dmp->dm_height);
+		bu_vls_printf(&cmd, "knob -i aS %f\n", f / height);
 
 	    break;
 	case AMM_CON_SCALE_Y:
@@ -647,9 +663,9 @@ motion_event_handler(XMotionEvent *xmotion)
 		f = -dy;
 
 	    if (mged_variables->mv_rateknobs)
-		bu_vls_printf(&cmd, "knob -i S %f\n", f / (fastf_t)dmp->dm_height);
+		bu_vls_printf(&cmd, "knob -i S %f\n", f / height);
 	    else
-		bu_vls_printf(&cmd, "knob -i aS %f\n", f / (fastf_t)dmp->dm_height);
+		bu_vls_printf(&cmd, "knob -i aS %f\n", f / height);
 
 	    break;
 	case AMM_CON_SCALE_Z:
@@ -671,9 +687,9 @@ motion_event_handler(XMotionEvent *xmotion)
 		f = -dy;
 
 	    if (mged_variables->mv_rateknobs)
-		bu_vls_printf(&cmd, "knob -i S %f\n", f / (fastf_t)dmp->dm_height);
+		bu_vls_printf(&cmd, "knob -i S %f\n", f / height);
 	    else
-		bu_vls_printf(&cmd, "knob -i aS %f\n", f / (fastf_t)dmp->dm_height);
+		bu_vls_printf(&cmd, "knob -i aS %f\n", f / height);
 
 	    break;
 	case AMM_CON_XADC:
@@ -683,7 +699,7 @@ motion_event_handler(XMotionEvent *xmotion)
 		f = -dy;
 
 	    bu_vls_printf(&cmd, "knob -i xadc %f\n",
-			  f / (fastf_t)dmp->dm_width * GED_RANGE);
+			  f / (fastf_t)width * GED_RANGE);
 	    break;
 	case AMM_CON_YADC:
 	    if (abs(dx) >= abs(dy))
@@ -692,7 +708,7 @@ motion_event_handler(XMotionEvent *xmotion)
 		f = -dy;
 
 	    bu_vls_printf(&cmd, "knob -i yadc %f\n",
-			  f / (fastf_t)dmp->dm_height * GED_RANGE);
+			  f / (fastf_t)height * GED_RANGE);
 	    break;
 	case AMM_CON_ANG1:
 	    if (abs(dx) >= abs(dy))
@@ -701,7 +717,7 @@ motion_event_handler(XMotionEvent *xmotion)
 		f = -dy;
 
 	    bu_vls_printf(&cmd, "knob -i ang1 %f\n",
-			  f / (fastf_t)dmp->dm_width * 90.0);
+			  f / (fastf_t)width * 90.0);
 	    break;
 	case AMM_CON_ANG2:
 	    if (abs(dx) >= abs(dy))
@@ -710,7 +726,7 @@ motion_event_handler(XMotionEvent *xmotion)
 		f = -dy;
 
 	    bu_vls_printf(&cmd, "knob -i ang2 %f\n",
-			  f / (fastf_t)dmp->dm_width * 90.0);
+			  f / (fastf_t)width * 90.0);
 	    break;
 	case AMM_CON_DIST:
 	    if (abs(dx) >= abs(dy))
@@ -719,7 +735,7 @@ motion_event_handler(XMotionEvent *xmotion)
 		f = -dy;
 
 	    bu_vls_printf(&cmd, "knob -i distadc %f\n",
-			  f / (fastf_t)dmp->dm_width * GED_RANGE);
+			  f / (fastf_t)width * GED_RANGE);
 	    break;
     }
 
@@ -735,8 +751,8 @@ motion_event_handler(XMotionEvent *xmotion)
 
  handled:
     bu_vls_free(&cmd);
-    dml_omx = mx;
-    dml_omy = my;
+    dm_omx = mx;
+    dm_omy = my;
 }
 #endif /* HAVE_X11_XLIB_H */
 
@@ -751,7 +767,7 @@ dials_event_handler(XDeviceMotionEvent *dmep)
     int setting;
     fastf_t f;
 
-    if (dbip == DBI_NULL)
+    if (DBIP == DBI_NULL)
 	return;
 
     if (button0) {
@@ -762,46 +778,46 @@ dials_event_handler(XDeviceMotionEvent *dmep)
     switch (DIAL0 + dmep->first_axis) {
 	case DIAL0:
 	    if (adc_state->adc_draw) {
-		if (-NOISE <= dml_knobs[dmep->first_axis] &&
-		    dml_knobs[dmep->first_axis] <= NOISE &&
+		if (-NOISE <= dm_knobs[dmep->first_axis] &&
+		    dm_knobs[dmep->first_axis] <= NOISE &&
 		    !adc_state->adc_dv_a1)
-		    dml_knobs[dmep->first_axis] +=
+		    dm_knobs[dmep->first_axis] +=
 			dmep->axis_data[0] - knob_values[dmep->first_axis];
 		else
-		    dml_knobs[dmep->first_axis] =
+		    dm_knobs[dmep->first_axis] =
 			dm_unlimit(adc_state->adc_dv_a1) + dmep->axis_data[0] - knob_values[dmep->first_axis];
 
-		setting = dm_limit(dml_knobs[dmep->first_axis]);
+		setting = dm_limit(dm_knobs[dmep->first_axis]);
 		bu_vls_printf(&cmd, "knob ang1 %f\n",
 			      45.0 - 45.0*((double)setting) * INV_GED);
 	    } else {
 		if (mged_variables->mv_rateknobs) {
 		    f = view_state->vs_rate_model_rotate[Z];
 
-		    if (-NOISE <= dml_knobs[dmep->first_axis] &&
-			dml_knobs[dmep->first_axis] <= NOISE && !f)
-			dml_knobs[dmep->first_axis] +=
+		    if (-NOISE <= dm_knobs[dmep->first_axis] &&
+			dm_knobs[dmep->first_axis] <= NOISE && !f)
+			dm_knobs[dmep->first_axis] +=
 			    dmep->axis_data[0] - knob_values[dmep->first_axis];
 		    else
-			dml_knobs[dmep->first_axis] =
+			dm_knobs[dmep->first_axis] =
 			    dm_unlimit((int)(512.5 * f)) +
 			    dmep->axis_data[0] - knob_values[dmep->first_axis];
 
-		    setting = dm_limit(dml_knobs[dmep->first_axis]);
+		    setting = dm_limit(dm_knobs[dmep->first_axis]);
 		    bu_vls_printf(&cmd, "knob -m z %f\n", setting / 512.0);
 		} else {
 		    f = view_state->vs_absolute_model_rotate[Z];
 
-		    if (-NOISE <= dml_knobs[dmep->first_axis] &&
-			dml_knobs[dmep->first_axis] <= NOISE && !f)
-			dml_knobs[dmep->first_axis] +=
+		    if (-NOISE <= dm_knobs[dmep->first_axis] &&
+			dm_knobs[dmep->first_axis] <= NOISE && !f)
+			dm_knobs[dmep->first_axis] +=
 			    dmep->axis_data[0] - knob_values[dmep->first_axis];
 		    else
-			dml_knobs[dmep->first_axis] =
+			dm_knobs[dmep->first_axis] =
 			    dm_unlimit((int)(2.847 * f)) +
 			    dmep->axis_data[0] - knob_values[dmep->first_axis];
 
-		    f = dm_limit(dml_knobs[dmep->first_axis]) / 512.0;
+		    f = dm_limit(dm_knobs[dmep->first_axis]) / 512.0;
 		    bu_vls_printf(&cmd, "knob -m az %f\n", dm_wrap(f) * 180.0);
 		}
 	    }
@@ -813,15 +829,15 @@ dials_event_handler(XDeviceMotionEvent *dmep)
 		else
 		    f = view_state->vs_rate_scale;
 
-		if (-NOISE <= dml_knobs[dmep->first_axis] &&
-		    dml_knobs[dmep->first_axis] <= NOISE && !f)
-		    dml_knobs[dmep->first_axis] +=
+		if (-NOISE <= dm_knobs[dmep->first_axis] &&
+		    dm_knobs[dmep->first_axis] <= NOISE && !f)
+		    dm_knobs[dmep->first_axis] +=
 			dmep->axis_data[0] - knob_values[dmep->first_axis];
 		else
-		    dml_knobs[dmep->first_axis] = dm_unlimit((int)(512.5 * f)) +
+		    dm_knobs[dmep->first_axis] = dm_unlimit((int)(512.5 * f)) +
 			dmep->axis_data[0] - knob_values[dmep->first_axis];
 
-		setting = dm_limit(dml_knobs[dmep->first_axis]);
+		setting = dm_limit(dm_knobs[dmep->first_axis]);
 		bu_vls_printf(&cmd, "knob S %f\n", setting / 512.0);
 	    } else {
 		if (EDIT_SCALE && mged_variables->mv_transform == 'e')
@@ -829,31 +845,31 @@ dials_event_handler(XDeviceMotionEvent *dmep)
 		else
 		    f = view_state->vs_absolute_scale;
 
-		if (-NOISE <= dml_knobs[dmep->first_axis] &&
-		    dml_knobs[dmep->first_axis] <= NOISE && !f)
-		    dml_knobs[dmep->first_axis] +=
+		if (-NOISE <= dm_knobs[dmep->first_axis] &&
+		    dm_knobs[dmep->first_axis] <= NOISE && !f)
+		    dm_knobs[dmep->first_axis] +=
 			dmep->axis_data[0] - knob_values[dmep->first_axis];
 		else
-		    dml_knobs[dmep->first_axis] =
+		    dm_knobs[dmep->first_axis] =
 			dm_unlimit((int)(512.5 * f)) +
 			dmep->axis_data[0] - knob_values[dmep->first_axis];
 
-		setting = dm_limit(dml_knobs[dmep->first_axis]);
+		setting = dm_limit(dm_knobs[dmep->first_axis]);
 		bu_vls_printf(&cmd, "knob aS %f\n", setting / 512.0);
 	    }
 	    break;
 	case DIAL2:
 	    if (adc_state->adc_draw) {
-		if (-NOISE <= dml_knobs[dmep->first_axis] &&
-		    dml_knobs[dmep->first_axis] <= NOISE &&
+		if (-NOISE <= dm_knobs[dmep->first_axis] &&
+		    dm_knobs[dmep->first_axis] <= NOISE &&
 		    !adc_state->adc_dv_a2)
-		    dml_knobs[dmep->first_axis] +=
+		    dm_knobs[dmep->first_axis] +=
 			dmep->axis_data[0] - knob_values[dmep->first_axis];
 		else
-		    dml_knobs[dmep->first_axis] =
+		    dm_knobs[dmep->first_axis] =
 			dm_unlimit(adc_state->adc_dv_a2) + dmep->axis_data[0] - knob_values[dmep->first_axis];
 
-		setting = dm_limit(dml_knobs[dmep->first_axis]);
+		setting = dm_limit(dm_knobs[dmep->first_axis]);
 		bu_vls_printf(&cmd, "knob ang2 %f\n",
 			      45.0 - 45.0*((double)setting) * INV_GED);
 	    } else {
@@ -886,16 +902,16 @@ dials_event_handler(XDeviceMotionEvent *dmep)
 		    else
 			f = view_state->vs_rate_rotate[Z];
 
-		    if (-NOISE <= dml_knobs[dmep->first_axis] &&
-			dml_knobs[dmep->first_axis] <= NOISE && !f)
-			dml_knobs[dmep->first_axis] +=
+		    if (-NOISE <= dm_knobs[dmep->first_axis] &&
+			dm_knobs[dmep->first_axis] <= NOISE && !f)
+			dm_knobs[dmep->first_axis] +=
 			    dmep->axis_data[0] - knob_values[dmep->first_axis];
 		    else
-			dml_knobs[dmep->first_axis] =
+			dm_knobs[dmep->first_axis] =
 			    dm_unlimit((int)(512.5 * f)) +
 			    dmep->axis_data[0] - knob_values[dmep->first_axis];
 
-		    setting = dm_limit(dml_knobs[dmep->first_axis]);
+		    setting = dm_limit(dm_knobs[dmep->first_axis]);
 		    bu_vls_printf(&cmd, "knob z %f\n", setting / 512.0);
 		} else {
 		    if ((STATE == ST_S_EDIT || STATE == ST_O_EDIT)
@@ -926,32 +942,32 @@ dials_event_handler(XDeviceMotionEvent *dmep)
 		    else
 			f = view_state->vs_absolute_rotate[Z];
 
-		    if (-NOISE <= dml_knobs[dmep->first_axis] &&
-			dml_knobs[dmep->first_axis] <= NOISE && !f)
-			dml_knobs[dmep->first_axis] +=
+		    if (-NOISE <= dm_knobs[dmep->first_axis] &&
+			dm_knobs[dmep->first_axis] <= NOISE && !f)
+			dm_knobs[dmep->first_axis] +=
 			    dmep->axis_data[0] - knob_values[dmep->first_axis];
 		    else
-			dml_knobs[dmep->first_axis] =
+			dm_knobs[dmep->first_axis] =
 			    dm_unlimit((int)(2.847 * f)) +
 			    dmep->axis_data[0] - knob_values[dmep->first_axis];
 
-		    f = dm_limit(dml_knobs[dmep->first_axis]) / 512.0;
+		    f = dm_limit(dm_knobs[dmep->first_axis]) / 512.0;
 		    bu_vls_printf(&cmd, "knob az %f\n", dm_wrap(f) * 180.0);
 		}
 	    }
 	    break;
 	case DIAL3:
 	    if (adc_state->adc_draw) {
-		if (-NOISE <= dml_knobs[dmep->first_axis] &&
-		    dml_knobs[dmep->first_axis] <= NOISE &&
+		if (-NOISE <= dm_knobs[dmep->first_axis] &&
+		    dm_knobs[dmep->first_axis] <= NOISE &&
 		    !adc_state->adc_dv_dist)
-		    dml_knobs[dmep->first_axis] +=
+		    dm_knobs[dmep->first_axis] +=
 			dmep->axis_data[0] - knob_values[dmep->first_axis];
 		else
-		    dml_knobs[dmep->first_axis] =
+		    dm_knobs[dmep->first_axis] =
 			dm_unlimit(adc_state->adc_dv_dist) + dmep->axis_data[0] - knob_values[dmep->first_axis];
 
-		setting = dm_limit(dml_knobs[dmep->first_axis]);
+		setting = dm_limit(dm_knobs[dmep->first_axis]);
 		bu_vls_printf(&cmd, "knob distadc %d\n", setting);
 	    } else {
 		if (mged_variables->mv_rateknobs) {
@@ -981,16 +997,16 @@ dials_event_handler(XDeviceMotionEvent *dmep)
 		    else
 			f = view_state->vs_rate_tran[Z];
 
-		    if (-NOISE <= dml_knobs[dmep->first_axis] &&
-			dml_knobs[dmep->first_axis] <= NOISE && !f)
-			dml_knobs[dmep->first_axis] +=
+		    if (-NOISE <= dm_knobs[dmep->first_axis] &&
+			dm_knobs[dmep->first_axis] <= NOISE && !f)
+			dm_knobs[dmep->first_axis] +=
 			    dmep->axis_data[0] - knob_values[dmep->first_axis];
 		    else
-			dml_knobs[dmep->first_axis] =
+			dm_knobs[dmep->first_axis] =
 			    dm_unlimit((int)(512.5 * f)) +
 			    dmep->axis_data[0] - knob_values[dmep->first_axis];
 
-		    setting = dm_limit(dml_knobs[dmep->first_axis]);
+		    setting = dm_limit(dm_knobs[dmep->first_axis]);
 		    bu_vls_printf(&cmd, "knob Z %f\n", setting / 512.0);
 		} else {
 		    if ((STATE == ST_S_EDIT || STATE == ST_O_EDIT)
@@ -1019,33 +1035,33 @@ dials_event_handler(XDeviceMotionEvent *dmep)
 		    else
 			f = view_state->vs_absolute_tran[Z];
 
-		    if (-NOISE <= dml_knobs[dmep->first_axis] &&
-			dml_knobs[dmep->first_axis] <= NOISE &&
+		    if (-NOISE <= dm_knobs[dmep->first_axis] &&
+			dm_knobs[dmep->first_axis] <= NOISE &&
 			!f)
-			dml_knobs[dmep->first_axis] +=
+			dm_knobs[dmep->first_axis] +=
 			    dmep->axis_data[0] - knob_values[dmep->first_axis];
 		    else
-			dml_knobs[dmep->first_axis] =
+			dm_knobs[dmep->first_axis] =
 			    dm_unlimit((int)(512.5 * f)) +
 			    dmep->axis_data[0] - knob_values[dmep->first_axis];
 
-		    setting = dm_limit(dml_knobs[dmep->first_axis]);
+		    setting = dm_limit(dm_knobs[dmep->first_axis]);
 		    bu_vls_printf(&cmd, "knob aZ %f\n", setting / 512.0 * view_state->vs_gvp->gv_scale * base2local);
 		}
 	    }
 	    break;
 	case DIAL4:
 	    if (adc_state->adc_draw) {
-		if (-NOISE <= dml_knobs[dmep->first_axis] &&
-		    dml_knobs[dmep->first_axis] <= NOISE &&
+		if (-NOISE <= dm_knobs[dmep->first_axis] &&
+		    dm_knobs[dmep->first_axis] <= NOISE &&
 		    !adc_state->adc_dv_y)
-		    dml_knobs[dmep->first_axis] +=
+		    dm_knobs[dmep->first_axis] +=
 			dmep->axis_data[0] - knob_values[dmep->first_axis];
 		else
-		    dml_knobs[dmep->first_axis] =
+		    dm_knobs[dmep->first_axis] =
 			dm_unlimit(adc_state->adc_dv_y) + dmep->axis_data[0] - knob_values[dmep->first_axis];
 
-		setting = dm_limit(dml_knobs[dmep->first_axis]);
+		setting = dm_limit(dm_knobs[dmep->first_axis]);
 		bu_vls_printf(&cmd, "knob yadc %d\n", setting);
 	    } else {
 		if (mged_variables->mv_rateknobs) {
@@ -1077,16 +1093,16 @@ dials_event_handler(XDeviceMotionEvent *dmep)
 		    else
 			f = view_state->vs_rate_rotate[Y];
 
-		    if (-NOISE <= dml_knobs[dmep->first_axis] &&
-			dml_knobs[dmep->first_axis] <= NOISE && !f)
-			dml_knobs[dmep->first_axis] +=
+		    if (-NOISE <= dm_knobs[dmep->first_axis] &&
+			dm_knobs[dmep->first_axis] <= NOISE && !f)
+			dm_knobs[dmep->first_axis] +=
 			    dmep->axis_data[0] - knob_values[dmep->first_axis];
 		    else
-			dml_knobs[dmep->first_axis] =
+			dm_knobs[dmep->first_axis] =
 			    dm_unlimit((int)(512.5 * f)) +
 			    dmep->axis_data[0] - knob_values[dmep->first_axis];
 
-		    setting = dm_limit(dml_knobs[dmep->first_axis]);
+		    setting = dm_limit(dm_knobs[dmep->first_axis]);
 		    bu_vls_printf(&cmd, "knob y %f\n", setting / 512.0);
 		} else {
 		    if ((STATE == ST_S_EDIT || STATE == ST_O_EDIT)
@@ -1117,17 +1133,17 @@ dials_event_handler(XDeviceMotionEvent *dmep)
 		    else
 			f = view_state->vs_absolute_rotate[Y];
 
-		    if (-NOISE <= dml_knobs[dmep->first_axis] &&
-			dml_knobs[dmep->first_axis] <= NOISE &&
+		    if (-NOISE <= dm_knobs[dmep->first_axis] &&
+			dm_knobs[dmep->first_axis] <= NOISE &&
 			!f)
-			dml_knobs[dmep->first_axis] +=
+			dm_knobs[dmep->first_axis] +=
 			    dmep->axis_data[0] - knob_values[dmep->first_axis];
 		    else
-			dml_knobs[dmep->first_axis] =
+			dm_knobs[dmep->first_axis] =
 			    dm_unlimit((int)(2.847 * f)) +
 			    dmep->axis_data[0] - knob_values[dmep->first_axis];
 
-		    f = dm_limit(dml_knobs[dmep->first_axis]) / 512.0;
+		    f = dm_limit(dm_knobs[dmep->first_axis]) / 512.0;
 		    bu_vls_printf(&cmd, "knob ay %f\n", dm_wrap(f) * 180.0);
 		}
 	    }
@@ -1160,16 +1176,16 @@ dials_event_handler(XDeviceMotionEvent *dmep)
 		else
 		    f = view_state->vs_rate_tran[Y];
 
-		if (-NOISE <= dml_knobs[dmep->first_axis] &&
-		    dml_knobs[dmep->first_axis] <= NOISE && !f)
-		    dml_knobs[dmep->first_axis] +=
+		if (-NOISE <= dm_knobs[dmep->first_axis] &&
+		    dm_knobs[dmep->first_axis] <= NOISE && !f)
+		    dm_knobs[dmep->first_axis] +=
 			dmep->axis_data[0] - knob_values[dmep->first_axis];
 		else
-		    dml_knobs[dmep->first_axis] =
+		    dm_knobs[dmep->first_axis] =
 			dm_unlimit((int)(512.5 * f)) +
 			dmep->axis_data[0] - knob_values[dmep->first_axis];
 
-		setting = dm_limit(dml_knobs[dmep->first_axis]);
+		setting = dm_limit(dm_knobs[dmep->first_axis]);
 		bu_vls_printf(&cmd, "knob Y %f\n", setting / 512.0);
 	    } else {
 		if ((STATE == ST_S_EDIT || STATE == ST_O_EDIT)
@@ -1198,32 +1214,32 @@ dials_event_handler(XDeviceMotionEvent *dmep)
 		else
 		    f = view_state->vs_absolute_tran[Y];
 
-		if (-NOISE <= dml_knobs[dmep->first_axis] &&
-		    dml_knobs[dmep->first_axis] <= NOISE && !f)
-		    dml_knobs[dmep->first_axis] +=
+		if (-NOISE <= dm_knobs[dmep->first_axis] &&
+		    dm_knobs[dmep->first_axis] <= NOISE && !f)
+		    dm_knobs[dmep->first_axis] +=
 			dmep->axis_data[0] -
 			knob_values[dmep->first_axis];
 		else
-		    dml_knobs[dmep->first_axis] =
+		    dm_knobs[dmep->first_axis] =
 			dm_unlimit((int)(512.5 * f)) +
 			dmep->axis_data[0] - knob_values[dmep->first_axis];
 
-		setting = dm_limit(dml_knobs[dmep->first_axis]);
+		setting = dm_limit(dm_knobs[dmep->first_axis]);
 		bu_vls_printf(&cmd, "knob aY %f\n", setting / 512.0 * view_state->vs_gvp->gv_scale * base2local);
 	    }
 	    break;
 	case DIAL6:
 	    if (adc_state->adc_draw) {
-		if (-NOISE <= dml_knobs[dmep->first_axis] &&
-		    dml_knobs[dmep->first_axis] <= NOISE &&
+		if (-NOISE <= dm_knobs[dmep->first_axis] &&
+		    dm_knobs[dmep->first_axis] <= NOISE &&
 		    !adc_state->adc_dv_x)
-		    dml_knobs[dmep->first_axis] +=
+		    dm_knobs[dmep->first_axis] +=
 			dmep->axis_data[0] - knob_values[dmep->first_axis];
 		else
-		    dml_knobs[dmep->first_axis] =
+		    dm_knobs[dmep->first_axis] =
 			dm_unlimit(adc_state->adc_dv_x) + dmep->axis_data[0] - knob_values[dmep->first_axis];
 
-		setting = dm_limit(dml_knobs[dmep->first_axis]);
+		setting = dm_limit(dm_knobs[dmep->first_axis]);
 		bu_vls_printf(&cmd, "knob xadc %d\n", setting);
 	    } else {
 		if (mged_variables->mv_rateknobs) {
@@ -1255,16 +1271,16 @@ dials_event_handler(XDeviceMotionEvent *dmep)
 		    else
 			f = view_state->vs_rate_rotate[X];
 
-		    if (-NOISE <= dml_knobs[dmep->first_axis] &&
-			dml_knobs[dmep->first_axis] <= NOISE && !f)
-			dml_knobs[dmep->first_axis] +=
+		    if (-NOISE <= dm_knobs[dmep->first_axis] &&
+			dm_knobs[dmep->first_axis] <= NOISE && !f)
+			dm_knobs[dmep->first_axis] +=
 			    dmep->axis_data[0] - knob_values[dmep->first_axis];
 		    else
-			dml_knobs[dmep->first_axis] =
+			dm_knobs[dmep->first_axis] =
 			    dm_unlimit((int)(512.5 * f)) +
 			    dmep->axis_data[0] - knob_values[dmep->first_axis];
 
-		    setting = dm_limit(dml_knobs[dmep->first_axis]);
+		    setting = dm_limit(dm_knobs[dmep->first_axis]);
 		    bu_vls_printf(&cmd, "knob x %f\n", setting / 512.0);
 		} else {
 		    if ((STATE == ST_S_EDIT || STATE == ST_O_EDIT)
@@ -1295,16 +1311,16 @@ dials_event_handler(XDeviceMotionEvent *dmep)
 		    else
 			f = view_state->vs_absolute_rotate[X];
 
-		    if (-NOISE <= dml_knobs[dmep->first_axis] &&
-			dml_knobs[dmep->first_axis] <= NOISE && !f)
-			dml_knobs[dmep->first_axis] +=
+		    if (-NOISE <= dm_knobs[dmep->first_axis] &&
+			dm_knobs[dmep->first_axis] <= NOISE && !f)
+			dm_knobs[dmep->first_axis] +=
 			    dmep->axis_data[0] - knob_values[dmep->first_axis];
 		    else
-			dml_knobs[dmep->first_axis] =
+			dm_knobs[dmep->first_axis] =
 			    dm_unlimit((int)(2.847 * f)) +
 			    dmep->axis_data[0] - knob_values[dmep->first_axis];
 
-		    f = dm_limit(dml_knobs[dmep->first_axis]) / 512.0;
+		    f = dm_limit(dm_knobs[dmep->first_axis]) / 512.0;
 		    bu_vls_printf(&cmd, "knob ax %f\n", dm_wrap(f) * 180.0);
 		}
 	    }
@@ -1337,16 +1353,16 @@ dials_event_handler(XDeviceMotionEvent *dmep)
 		else
 		    f = view_state->vs_rate_tran[X];
 
-		if (-NOISE <= dml_knobs[dmep->first_axis] &&
-		    dml_knobs[dmep->first_axis] <= NOISE && !f)
-		    dml_knobs[dmep->first_axis] +=
+		if (-NOISE <= dm_knobs[dmep->first_axis] &&
+		    dm_knobs[dmep->first_axis] <= NOISE && !f)
+		    dm_knobs[dmep->first_axis] +=
 			dmep->axis_data[0] - knob_values[dmep->first_axis];
 		else
-		    dml_knobs[dmep->first_axis] =
+		    dm_knobs[dmep->first_axis] =
 			dm_unlimit((int)(512.5 * f)) +
 			dmep->axis_data[0] - knob_values[dmep->first_axis];
 
-		setting = dm_limit(dml_knobs[dmep->first_axis]);
+		setting = dm_limit(dm_knobs[dmep->first_axis]);
 		bu_vls_printf(&cmd, "knob X %f\n", setting / 512.0);
 	    } else {
 		if ((STATE == ST_S_EDIT || STATE == ST_O_EDIT)
@@ -1375,16 +1391,16 @@ dials_event_handler(XDeviceMotionEvent *dmep)
 		else
 		    f = view_state->vs_absolute_tran[X];
 
-		if (-NOISE <= dml_knobs[dmep->first_axis] &&
-		    dml_knobs[dmep->first_axis] <= NOISE && !f)
-		    dml_knobs[dmep->first_axis] +=
+		if (-NOISE <= dm_knobs[dmep->first_axis] &&
+		    dm_knobs[dmep->first_axis] <= NOISE && !f)
+		    dm_knobs[dmep->first_axis] +=
 			dmep->axis_data[0] - knob_values[dmep->first_axis];
 		else
-		    dml_knobs[dmep->first_axis] =
+		    dm_knobs[dmep->first_axis] =
 			dm_unlimit((int)(512.5 * f)) +
 			dmep->axis_data[0] - knob_values[dmep->first_axis];
 
-		setting = dm_limit(dml_knobs[dmep->first_axis]);
+		setting = dm_limit(dm_knobs[dmep->first_axis]);
 		bu_vls_printf(&cmd, "knob aX %f\n", setting / 512.0 * view_state->vs_gvp->gv_scale * base2local);
 	    }
 	    break;
