@@ -1,7 +1,7 @@
 /*                        F B S E R V . C
  * BRL-CAD
  *
- * Copyright (c) 1995-2013 United States Government as represented by
+ * Copyright (c) 1995-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This program is free software; you can redistribute it and/or
@@ -27,44 +27,34 @@
 #include "common.h"
 
 #include <stdlib.h>
-#include <stdio.h>
 #include <ctype.h>
 
-#ifdef HAVE_WINSOCK_H
-#  include <process.h>
-#  include <winsock.h>
-#else
-#  include <sys/socket.h>
-#  include <netinet/in.h>		/* For htonl(), etc. */
-#endif
 #include "bio.h"
+#include "bnetwork.h"
+#include "bsocket.h"
 
 #include "tcl.h"
-#include "bu.h"
 #include "vmath.h"
 #include "raytrace.h"
 
 #include "./mged.h"
 #include "./mged_dm.h"
 
-#include "./fbserv.h"
-
 #define NET_LONG_LEN 4 /* # bytes to network long */
 
+extern const struct pkg_switch pkg_switch[];
 
 /*
- * C O M M _ E R R O R
- *
  * Communication error.  An error occurred on the PKG link.
  */
-HIDDEN void
-communications_error(char *str)
+static void
+communications_error(const char *str)
 {
-    bu_log(str);
+    bu_log("%s", str);
 }
 
 
-HIDDEN void
+static void
 fbserv_setup_socket(int fd)
 {
     int on = 1;
@@ -98,10 +88,7 @@ fbserv_setup_socket(int fd)
 }
 
 
-/*
- * D R O P _ C L I E N T
- */
-HIDDEN void
+static void
 fbserv_drop_client(int sub)
 {
     if (clients[sub].c_pkg != PKC_NULL) {
@@ -110,7 +97,10 @@ fbserv_drop_client(int sub)
 	Tcl_DeleteChannelHandler(clients[sub].c_chan,
 				 clients[sub].c_handler,
 				 (ClientData)clients[sub].c_fd);
-	Tcl_Close(dmp->dm_interp, clients[sub].c_chan);
+
+	if (dm_interp(DMP) != NULL) {
+	    Tcl_Close((Tcl_Interp *)dm_interp(DMP), clients[sub].c_chan);
+	}
 	clients[sub].c_chan = NULL;
 #else
 	Tcl_DeleteFileHandler(clients[sub].c_fd);
@@ -124,28 +114,31 @@ fbserv_drop_client(int sub)
 /*
  * Process arrivals from existing clients.
  */
-HIDDEN void
+static void
 fbserv_existing_client_handler(ClientData clientData, int UNUSED(mask))
 {
     int i;
     int fd = (int)((long)clientData & 0xFFFF);	/* fd's will be small */
     int npp;			/* number of processed packages */
-    struct dm_list *dlp;
-    struct dm_list *scdlp;  /* save current dm_list pointer */
+    struct mged_dm *dlp = MGED_DM_NULL;
+    struct mged_dm *scdlp;  /* save current dm_list pointer */
 
-    FOR_ALL_DISPLAYS(dlp, &head_dm_list.l) {
+    for (size_t di = 0; di < BU_PTBL_LEN(&active_dm_set); di++) {
+	struct mged_dm *m_dmp = (struct mged_dm *)BU_PTBL_GET(&active_dm_set, di);
 	for (i = MAX_CLIENTS-1; i >= 0; i--)
-	    if (fd == dlp->dml_clients[i].c_fd)
+	    if (fd == m_dmp->dm_clients[i].c_fd) {
+		dlp = m_dmp;
 		goto found;
+	    }
     }
 
     return;
 
  found:
     /* save */
-    scdlp = curr_dm_list;
+    scdlp = mged_curr_dm;
 
-    curr_dm_list = dlp;
+    set_curr_dm(dlp);
     for (i = MAX_CLIENTS-1; i >= 0; i--) {
 	if (clients[i].c_fd == 0)
 	    continue;
@@ -153,8 +146,10 @@ fbserv_existing_client_handler(ClientData clientData, int UNUSED(mask))
 	if ((npp = pkg_process(clients[i].c_pkg)) < 0)
 	    bu_log("pkg_process error encountered (1)\n");
 
-	if (npp > 0)
-	    dirty = 1;
+	if (npp > 0) {
+	    DMP_dirty = 1;
+	    dm_set_dirty(DMP, 1);
+	}
 
 	if (clients[i].c_fd != fd)
 	    continue;
@@ -169,19 +164,21 @@ fbserv_existing_client_handler(ClientData clientData, int UNUSED(mask))
 	if ((npp = pkg_process(clients[i].c_pkg)) < 0)
 	    bu_log("pkg_process error encountered (2)\n");
 
-	if (npp > 0)
-	    dirty = 1;
+	if (npp > 0) {
+	    DMP_dirty = 1;
+	    dm_set_dirty(DMP, 1);
+	}
     }
 
     /* restore */
-    curr_dm_list = scdlp;
+    set_curr_dm(scdlp);
 }
 
 
 #if defined(_WIN32) && !defined(__CYGWIN__)
-HIDDEN struct pkg_conn *fbserv_makeconn(int fd, const struct pkg_switch *switchp);
+static struct pkg_conn *fbserv_makeconn(int fd, const struct pkg_switch *switchp);
 
-HIDDEN void
+static void
 fbserv_new_client(struct pkg_conn *pcp,
 		  Tcl_Channel chan)
 
@@ -214,37 +211,34 @@ fbserv_new_client(struct pkg_conn *pcp,
 }
 
 
-HIDDEN void
+static void
 fbserv_new_client_handler(ClientData clientData,
 			  Tcl_Channel chan,
 			  char *host,
 			  int port)
 {
-    struct dm_list *dlp = (struct dm_list *)clientData;
-    struct dm_list *scdlp;  /* save current dm_list pointer */
+    struct mged_dm *dlp = (struct mged_dm *)clientData;
+    struct mged_dm *scdlp;  /* save current dm_list pointer */
     uintptr_t fd;
 
     if (dlp == NULL)
 	return;
 
     /* save */
-    scdlp = curr_dm_list;
+    scdlp = mged_curr_dm;
 
-    curr_dm_list = dlp;
+    set_curr_dm(dlp);
 
     if (Tcl_GetChannelHandle(chan, TCL_READABLE, (ClientData *)&fd) == TCL_OK)
 	fbserv_new_client(fbserv_makeconn((int)fd, pkg_switch), chan);
 
     /* restore */
-    curr_dm_list = scdlp;
+    set_curr_dm(scdlp);
 }
 
 
-/*
- * S E T _ P O R T
- */
 void
-fbserv_set_port(void)
+fbserv_set_port(const struct bu_structparse *UNUSED(sp), const char *UNUSED(c1), void *UNUSED(v1), const char *UNUSED(c2), void *UNUSED(v2))
 {
     int i;
     int save_port;
@@ -260,9 +254,11 @@ fbserv_set_port(void)
 	    fbserv_drop_client(i);
 
 	fd = (ClientData)netfd;
-	Tcl_DeleteChannelHandler(netchan, fbserv_new_client_handler, fd);
+	Tcl_DeleteChannelHandler(netchan, (Tcl_ChannelProc *)fbserv_new_client_handler, fd);
 
-	Tcl_Close(dmp->dm_interp, netchan);
+	if (dm_interp(DMP) != NULL) {
+	    Tcl_Close((Tcl_Interp *)dm_interp(DMP), netchan);
+	}
 	netchan = NULL;
 
 	closesocket(netfd);
@@ -296,7 +292,10 @@ fbserv_set_port(void)
 	/*
 	 * Hang an unending listen for PKG connections
 	 */
-	netchan = Tcl_OpenTcpServer(dmp->dm_interp, port, hostname, fbserv_new_client_handler, (ClientData)curr_dm_list);
+
+	if (dm_interp(DMP) != NULL) {
+	    netchan = Tcl_OpenTcpServer((Tcl_Interp *)dm_interp(DMP), port, hostname, fbserv_new_client_handler, (ClientData)mged_curr_dm);
+	}
 
 	if (netchan == NULL)
 	    ++port;
@@ -316,7 +315,7 @@ fbserv_set_port(void)
 }
 
 
-HIDDEN struct pkg_conn *
+static struct pkg_conn *
 fbserv_makeconn(int fd,
 		const struct pkg_switch *switchp)
 {
@@ -357,10 +356,7 @@ fbserv_makeconn(int fd,
 #else /* defined(_WIN32) && !defined(__CYGWIN__) */
 
 
-/*
- * N E W _ C L I E N T
- */
-HIDDEN void
+static void
 fbserv_new_client(struct pkg_conn *pcp)
 {
     int i;
@@ -391,37 +387,38 @@ fbserv_new_client(struct pkg_conn *pcp)
 /*
  * Accept any new client connections.
  */
-HIDDEN void
+static void
 fbserv_new_client_handler(ClientData clientData, int UNUSED(mask))
 {
     uintptr_t datafd = (uintptr_t)clientData;
     int fd = (int)((int32_t)datafd & 0xFFFF);	/* fd's will be small */
-    struct dm_list *dlp;
-    struct dm_list *scdlp;  /* save current dm_list pointer */
+    struct mged_dm *dlp;
+    struct mged_dm *scdlp;  /* save current dm_list pointer */
 
-    FOR_ALL_DISPLAYS(dlp, &head_dm_list.l)
-	if (fd == dlp->dml_netfd)
+    for (size_t di = 0; di < BU_PTBL_LEN(&active_dm_set); di++) {
+	struct mged_dm *m_dmp = (struct mged_dm *)BU_PTBL_GET(&active_dm_set, di);
+	if (fd == m_dmp->dm_netfd) {
+	    dlp = m_dmp;
 	    goto found;
+	}
+    }
 
     return;
 
  found:
     /* save */
-    scdlp = curr_dm_list;
+    scdlp = mged_curr_dm;
 
-    curr_dm_list = dlp;
+    set_curr_dm(dlp);
     fbserv_new_client(pkg_getclient(fd, pkg_switch, communications_error, 0));
 
     /* restore */
-    curr_dm_list = scdlp;
+    set_curr_dm(scdlp);
 }
 
 
-/*
- * S E T _ P O R T
- */
 void
-fbserv_set_port(void)
+fbserv_set_port(const struct bu_structparse *UNUSED(sp), const char *UNUSED(c1), void *UNUSED(v1), const char *UNUSED(c2), void *UNUSED(v2))
 {
     int i;
     int save_port;
@@ -484,10 +481,10 @@ fbserv_set_port(void)
  * This is where we go for message types we don't understand.
  */
 void
-rfbunknown(struct pkg_conn *pcp, char *buf)
+fb_server_fb_unknown(struct pkg_conn *pcp, char *buf)
 {
-    if(buf == NULL) {
-	bu_log("rfbunknown: null buffer\n");
+    if (buf == NULL) {
+	bu_log("fb_server_fb_unknown: null buffer\n");
 	return;
     }
 
@@ -498,23 +495,23 @@ rfbunknown(struct pkg_conn *pcp, char *buf)
 
 /******** Here's where the hooks lead *********/
 
-void
-rfbopen(struct pkg_conn *pcp, char *buf)
+static void
+fb_server_fb_open(struct pkg_conn *pcp, char *buf)
 {
     char rbuf[5*NET_LONG_LEN+1];
     int want;
 
-    if(buf == NULL) {
-	bu_log("rfbopen: null buffer\n");
+    if (buf == NULL) {
+	bu_log("fb_server_fb_open: null buffer\n");
 	return;
     }
 
     /* Don't really open a new framebuffer --- use existing one */
     (void)pkg_plong(&rbuf[0*NET_LONG_LEN], 0);	/* ret */
-    (void)pkg_plong(&rbuf[1*NET_LONG_LEN], fbp->if_max_width);
-    (void)pkg_plong(&rbuf[2*NET_LONG_LEN], fbp->if_max_height);
-    (void)pkg_plong(&rbuf[3*NET_LONG_LEN], fbp->if_width);
-    (void)pkg_plong(&rbuf[4*NET_LONG_LEN], fbp->if_height);
+    (void)pkg_plong(&rbuf[1*NET_LONG_LEN], fb_get_max_width(fbp));
+    (void)pkg_plong(&rbuf[2*NET_LONG_LEN], fb_get_max_height(fbp));
+    (void)pkg_plong(&rbuf[3*NET_LONG_LEN], fb_getwidth(fbp));
+    (void)pkg_plong(&rbuf[4*NET_LONG_LEN], fb_getheight(fbp));
 
     want = 5*NET_LONG_LEN;
     if (pkg_send(MSG_RETURN, rbuf, want, pcp) != want)
@@ -524,8 +521,8 @@ rfbopen(struct pkg_conn *pcp, char *buf)
 }
 
 
-void
-rfbclose(struct pkg_conn *pcp, char *buf)
+static void
+fb_server_fb_close(struct pkg_conn *pcp, char *buf)
 {
     char rbuf[NET_LONG_LEN+1];
 
@@ -541,14 +538,13 @@ rfbclose(struct pkg_conn *pcp, char *buf)
      * communication has broken, other end will know we are gone.
      */
     (void)pkg_send(MSG_RETURN, rbuf, NET_LONG_LEN, pcp);
-
     if (buf)
 	(void)free(buf);
 }
 
 
-void
-rfbfree(struct pkg_conn *pcp, char *buf)
+static void
+fb_server_fb_free(struct pkg_conn *pcp, char *buf)
 {
     char rbuf[NET_LONG_LEN+1];
 
@@ -561,14 +557,14 @@ rfbfree(struct pkg_conn *pcp, char *buf)
 }
 
 
-void
-rfbclear(struct pkg_conn *pcp, char *buf)
+static void
+fb_server_fb_clear(struct pkg_conn *pcp, char *buf)
 {
     RGBpixel bg;
     char rbuf[NET_LONG_LEN+1];
 
-    if(buf == NULL) {
-	bu_log("rfbwindow: null buffer\n");
+    if (buf == NULL) {
+	bu_log("fb_server_fb_window: null buffer\n");
 	return;
     }
 
@@ -579,12 +575,13 @@ rfbclear(struct pkg_conn *pcp, char *buf)
     (void)pkg_plong(rbuf, fb_clear(fbp, bg));
     pkg_send(MSG_RETURN, rbuf, NET_LONG_LEN, pcp);
 
-    (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
 
-void
-rfbread(struct pkg_conn *pcp, char *buf)
+static void
+fb_server_fb_read(struct pkg_conn *pcp, char *buf)
 {
     int x, y;
     size_t num;
@@ -592,8 +589,8 @@ rfbread(struct pkg_conn *pcp, char *buf)
     static unsigned char *scanbuf = NULL;
     static size_t buflen = 0;
 
-    if(buf == NULL) {
-	bu_log("rfbreadrect: null buffer\n");
+    if (buf == NULL) {
+	bu_log("fb_server_fb_readrect: null buffer\n");
 	return;
     }
 
@@ -609,7 +606,8 @@ rfbread(struct pkg_conn *pcp, char *buf)
 	    buflen = 1024*sizeof(RGBpixel);
 	if ((scanbuf = (unsigned char *)malloc(buflen)) == NULL) {
 	    fb_log("fb_read: malloc failed!");
-	    (void)free(buf);
+	    if (buf)
+		(void)free(buf);
 	    buflen = 0;
 	    return;
 	}
@@ -619,20 +617,21 @@ rfbread(struct pkg_conn *pcp, char *buf)
     if (ret < 0) ret = 0;		/* map error indications */
     /* sending a 0-length package indicates error */
     pkg_send(MSG_RETURN, (char *)scanbuf, ret*sizeof(RGBpixel), pcp);
-    (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
 
-void
-rfbwrite(struct pkg_conn *pcp, char *buf)
+static void
+fb_server_fb_write(struct pkg_conn *pcp, char *buf)
 {
     int x, y, num;
     char rbuf[NET_LONG_LEN+1];
     int ret;
     int type;
 
-    if(buf == NULL) {
-	bu_log("rfbreadrect: null buffer\n");
+    if (buf == NULL) {
+	bu_log("fb_server_fb_readrect: null buffer\n");
 	return;
     }
 
@@ -646,15 +645,13 @@ rfbwrite(struct pkg_conn *pcp, char *buf)
 	(void)pkg_plong(&rbuf[0*NET_LONG_LEN], ret);
 	pkg_send(MSG_RETURN, rbuf, NET_LONG_LEN, pcp);
     }
-    (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
 
-/*
- * R F B R E A D R E C T
- */
-void
-rfbreadrect(struct pkg_conn *pcp, char *buf)
+static void
+fb_server_fb_readrect(struct pkg_conn *pcp, char *buf)
 {
     int xmin, ymin;
     int width, height;
@@ -663,8 +660,8 @@ rfbreadrect(struct pkg_conn *pcp, char *buf)
     static unsigned char *scanbuf = NULL;
     static size_t buflen = 0;
 
-    if(buf == NULL) {
-	bu_log("rfbreadrect: null buffer\n");
+    if (buf == NULL) {
+	bu_log("fb_server_fb_readrect: null buffer\n");
 	return;
     }
 
@@ -682,7 +679,8 @@ rfbreadrect(struct pkg_conn *pcp, char *buf)
 	    buflen = 1024*sizeof(RGBpixel);
 	if ((scanbuf = (unsigned char *)malloc(buflen)) == NULL) {
 	    fb_log("fb_read: malloc failed!");
-	    (void)free(buf);
+	    if (buf)
+		(void)free(buf);
 	    buflen = 0;
 	    return;
 	}
@@ -692,15 +690,13 @@ rfbreadrect(struct pkg_conn *pcp, char *buf)
     if (ret < 0) ret = 0;		/* map error indications */
     /* sending a 0-length package indicates error */
     pkg_send(MSG_RETURN, (char *)scanbuf, ret*sizeof(RGBpixel), pcp);
-    (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
 
-/*
- * R F B W R I T E R E C T
- */
-void
-rfbwriterect(struct pkg_conn *pcp, char *buf)
+static void
+fb_server_fb_writerect(struct pkg_conn *pcp, char *buf)
 {
     int x, y;
     int width, height;
@@ -708,8 +704,8 @@ rfbwriterect(struct pkg_conn *pcp, char *buf)
     int ret;
     int type;
 
-    if(buf == NULL) {
-	bu_log("rfbreadrect: null buffer\n");
+    if (buf == NULL) {
+	bu_log("fb_server_fb_readrect: null buffer\n");
 	return;
     }
 
@@ -726,15 +722,13 @@ rfbwriterect(struct pkg_conn *pcp, char *buf)
 	(void)pkg_plong(&rbuf[0*NET_LONG_LEN], ret);
 	pkg_send(MSG_RETURN, rbuf, NET_LONG_LEN, pcp);
     }
-    (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
 
-/*
- * R F B B W R E A D R E C T
- */
-void
-rfbbwreadrect(struct pkg_conn *pcp, char *buf)
+static void
+fb_server_fb_bwreadrect(struct pkg_conn *pcp, char *buf)
 {
     int xmin, ymin;
     int width, height;
@@ -743,8 +737,8 @@ rfbbwreadrect(struct pkg_conn *pcp, char *buf)
     static unsigned char *scanbuf = NULL;
     static int buflen = 0;
 
-    if(buf == NULL) {
-	bu_log("rfbbwreadrect: null buffer\n");
+    if (buf == NULL) {
+	bu_log("fb_server_fb_bwreadrect: null buffer\n");
 	return;
     }
 
@@ -761,7 +755,7 @@ rfbbwreadrect(struct pkg_conn *pcp, char *buf)
 	if (buflen < 1024)
 	    buflen = 1024;
 	if ((scanbuf = (unsigned char *)malloc(buflen)) == NULL) {
-	    fb_log("rfbbwreadrect: malloc failed!");
+	    fb_log("fb_server_fb_bwreadrect: malloc failed!");
 	    (void)free(buf);
 	    buflen = 0;
 	    return;
@@ -777,10 +771,10 @@ rfbbwreadrect(struct pkg_conn *pcp, char *buf)
 
 
 /*
- * R F B B W W R I T E R E C T
+ * A whole rectangle of monochrome pixels at once, probably large.
  */
-void
-rfbbwwriterect(struct pkg_conn *pcp, char *buf)
+static void
+fb_server_fb_bwwriterect(struct pkg_conn *pcp, char *buf)
 {
     int x, y;
     int width, height;
@@ -788,7 +782,7 @@ rfbbwwriterect(struct pkg_conn *pcp, char *buf)
     int ret;
     int type;
 
-    if(buf == NULL) {
+    if (buf == NULL) {
 	bu_log("rfbbwwriterect: null buffer\n");
 	return;
     }
@@ -806,18 +800,19 @@ rfbbwwriterect(struct pkg_conn *pcp, char *buf)
 	(void)pkg_plong(&rbuf[0*NET_LONG_LEN], ret);
 	pkg_send(MSG_RETURN, rbuf, NET_LONG_LEN, pcp);
     }
-    (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
 
-void
-rfbcursor(struct pkg_conn *pcp, char *buf)
+static void
+fb_server_fb_cursor(struct pkg_conn *pcp, char *buf)
 {
     int mode, x, y;
     char rbuf[NET_LONG_LEN+1];
 
-    if(buf == NULL) {
-	bu_log("rfbwindow: null buffer\n");
+    if (buf == NULL) {
+	bu_log("fb_server_fb_window: null buffer\n");
 	return;
     }
 
@@ -827,12 +822,13 @@ rfbcursor(struct pkg_conn *pcp, char *buf)
 
     (void)pkg_plong(&rbuf[0], fb_cursor(fbp, mode, x, y));
     pkg_send(MSG_RETURN, rbuf, NET_LONG_LEN, pcp);
-    (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
 
-void
-rfbgetcursor(struct pkg_conn *pcp, char *buf)
+static void
+fb_server_fb_getcursor(struct pkg_conn *pcp, char *buf)
 {
     int ret;
     int mode, x, y;
@@ -844,20 +840,21 @@ rfbgetcursor(struct pkg_conn *pcp, char *buf)
     (void)pkg_plong(&rbuf[2*NET_LONG_LEN], x);
     (void)pkg_plong(&rbuf[3*NET_LONG_LEN], y);
     pkg_send(MSG_RETURN, rbuf, 4*NET_LONG_LEN, pcp);
-    if (buf) (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
 
-void
-rfbsetcursor(struct pkg_conn *pcp, char *buf)
+static void
+fb_server_fb_setcursor(struct pkg_conn *pcp, char *buf)
 {
     char rbuf[NET_LONG_LEN+1];
     int ret;
     int xbits, ybits;
     int xorig, yorig;
 
-    if(buf == NULL) {
-	bu_log("rfbreadrect: null buffer\n");
+    if (buf == NULL) {
+	bu_log("fb_server_fb_readrect: null buffer\n");
 	return;
     }
 
@@ -873,19 +870,22 @@ rfbsetcursor(struct pkg_conn *pcp, char *buf)
 	(void)pkg_plong(&rbuf[0*NET_LONG_LEN], ret);
 	pkg_send(MSG_RETURN, rbuf, NET_LONG_LEN, pcp);
     }
-    (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
 
-/*OLD*/
-void
-rfbscursor(struct pkg_conn *pcp, char *buf)
+/*
+ * An OLD interface.  Retained so old clients can still be served.
+ */
+static void
+fb_server_fb_scursor(struct pkg_conn *pcp, char *buf)
 {
     int mode, x, y;
     char rbuf[NET_LONG_LEN+1];
 
-    if(buf == NULL) {
-	bu_log("rfbopen: null buffer\n");
+    if (buf == NULL) {
+	bu_log("fb_server_fb_open: null buffer\n");
 	return;
     }
 
@@ -899,15 +899,17 @@ rfbscursor(struct pkg_conn *pcp, char *buf)
 }
 
 
-/*OLD*/
-void
-rfbwindow(struct pkg_conn *pcp, char *buf)
+/*
+ * An OLD interface.  Retained so old clients can still be served.
+ */
+static void
+fb_server_fb_window(struct pkg_conn *pcp, char *buf)
 {
     int x, y;
     char rbuf[NET_LONG_LEN+1];
 
-    if(buf == NULL) {
-	bu_log("rfbwindow: null buffer\n");
+    if (buf == NULL) {
+	bu_log("fb_server_fb_window: null buffer\n");
 	return;
     }
 
@@ -916,19 +918,22 @@ rfbwindow(struct pkg_conn *pcp, char *buf)
 
     (void)pkg_plong(&rbuf[0], fb_window(fbp, x, y));
     pkg_send(MSG_RETURN, rbuf, NET_LONG_LEN, pcp);
-    (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
 
-/*OLD*/
-void
-rfbzoom(struct pkg_conn *pcp, char *buf)
+/*
+ * An OLD interface.  Retained so old clients can still be served.
+ */
+static void
+fb_server_fb_zoom(struct pkg_conn *pcp, char *buf)
 {
     int x, y;
     char rbuf[NET_LONG_LEN+1];
 
-    if(buf == NULL) {
-	bu_log("rfbreadrect: null buffer\n");
+    if (buf == NULL) {
+	bu_log("fb_server_fb_readrect: null buffer\n");
 	return;
     }
 
@@ -937,19 +942,20 @@ rfbzoom(struct pkg_conn *pcp, char *buf)
 
     (void)pkg_plong(&rbuf[0], fb_zoom(fbp, x, y));
     pkg_send(MSG_RETURN, rbuf, NET_LONG_LEN, pcp);
-    (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
 
-void
-rfbview(struct pkg_conn *pcp, char *buf)
+static void
+fb_server_fb_view(struct pkg_conn *pcp, char *buf)
 {
     int ret;
     int xcenter, ycenter, xzoom, yzoom;
     char rbuf[NET_LONG_LEN+1];
 
-    if(buf == NULL) {
-	bu_log("rfbreadrect: null buffer\n");
+    if (buf == NULL) {
+	bu_log("fb_server_fb_readrect: null buffer\n");
 	return;
     }
 
@@ -961,12 +967,13 @@ rfbview(struct pkg_conn *pcp, char *buf)
     ret = fb_view(fbp, xcenter, ycenter, xzoom, yzoom);
     (void)pkg_plong(&rbuf[0], ret);
     pkg_send(MSG_RETURN, rbuf, NET_LONG_LEN, pcp);
-    (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
 
-void
-rfbgetview(struct pkg_conn *pcp, char *buf)
+static void
+fb_server_fb_getview(struct pkg_conn *pcp, char *buf)
 {
     int ret;
     int xcenter, ycenter, xzoom, yzoom;
@@ -979,12 +986,13 @@ rfbgetview(struct pkg_conn *pcp, char *buf)
     (void)pkg_plong(&rbuf[3*NET_LONG_LEN], xzoom);
     (void)pkg_plong(&rbuf[4*NET_LONG_LEN], yzoom);
     pkg_send(MSG_RETURN, rbuf, 5*NET_LONG_LEN, pcp);
-    if (buf) (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
 
-void
-rfbrmap(struct pkg_conn *pcp, char *buf)
+static void
+fb_server_fb_rmap(struct pkg_conn *pcp, char *buf)
 {
     int i;
     char rbuf[NET_LONG_LEN+1];
@@ -999,28 +1007,27 @@ rfbrmap(struct pkg_conn *pcp, char *buf)
     }
     pkg_send(MSG_DATA, (char *)cm, sizeof(cm), pcp);
     pkg_send(MSG_RETURN, rbuf, NET_LONG_LEN, pcp);
-    if (buf) (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
 
 /*
- * R F B W M A P
- *
  * Accept a color map sent by the client, and write it to the
  * framebuffer.  Network format is to send each entry as a network
  * (IBM) order 2-byte short, 256 red shorts, followed by 256 green and
  * 256 blue, for a total of 3*256*2 bytes.
  */
-void
-rfbwmap(struct pkg_conn *pcp, char *buf)
+static void
+fb_server_fb_wmap(struct pkg_conn *pcp, char *buf)
 {
     int i;
     char rbuf[NET_LONG_LEN+1];
     long ret;
     ColorMap map;
 
-    if(buf == NULL) {
-	bu_log("rfbwmap: null buffer\n");
+    if (buf == NULL) {
+	bu_log("fb_server_fb_wmap: null buffer\n");
 	return;
     }
 
@@ -1036,12 +1043,13 @@ rfbwmap(struct pkg_conn *pcp, char *buf)
     }
     (void)pkg_plong(&rbuf[0], ret);
     pkg_send(MSG_RETURN, rbuf, NET_LONG_LEN, pcp);
-    (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
 
-void
-rfbflush(struct pkg_conn *pcp, char *buf)
+static void
+fb_server_fb_flush(struct pkg_conn *pcp, char *buf)
 {
     int ret;
     char rbuf[NET_LONG_LEN+1];
@@ -1052,12 +1060,14 @@ rfbflush(struct pkg_conn *pcp, char *buf)
 	(void)pkg_plong(rbuf, ret);
 	pkg_send(MSG_RETURN, rbuf, NET_LONG_LEN, pcp);
     }
-    if (buf) (void)free(buf);
+
+    if (buf)
+	(void)free(buf);
 }
 
 
-void
-rfbpoll(struct pkg_conn *pcp, char *buf)
+static void
+fb_server_fb_poll(struct pkg_conn *pcp, char *buf)
 {
     if (!pcp) return;
     (void)fb_poll(fbp);
@@ -1069,14 +1079,14 @@ rfbpoll(struct pkg_conn *pcp, char *buf)
  * At one time at least we couldn't send a zero length PKG message
  * back and forth, so we receive a dummy long here.
  */
-void
-rfbhelp(struct pkg_conn *pcp, char *buf)
+static void
+fb_server_fb_help(struct pkg_conn *pcp, char *buf)
 {
     long ret;
     char rbuf[NET_LONG_LEN+1];
 
-    if(buf == NULL) {
-	bu_log("rfbwindow: null buffer\n");
+    if (buf == NULL) {
+	bu_log("fb_server_fb_window: null buffer\n");
 	return;
     }
 
@@ -1085,9 +1095,44 @@ rfbhelp(struct pkg_conn *pcp, char *buf)
     ret = fb_help(fbp);
     (void)pkg_plong(&rbuf[0], ret);
     pkg_send(MSG_RETURN, rbuf, NET_LONG_LEN, pcp);
-    (void)free(buf);
+    if (buf)
+	(void)free(buf);
 }
 
+const struct pkg_switch pkg_switch[] = {
+    { MSG_FBOPEN,                       fb_server_fb_open,        "Open Framebuffer", NULL },
+    { MSG_FBCLOSE,                      fb_server_fb_close,       "Close Framebuffer", NULL },
+    { MSG_FBCLEAR,                      fb_server_fb_clear,       "Clear Framebuffer", NULL },
+    { MSG_FBREAD,                       fb_server_fb_read,        "Read Pixels", NULL },
+    { MSG_FBWRITE,                      fb_server_fb_write,       "Write Pixels", NULL },
+    { MSG_FBWRITE + MSG_NORETURN,       fb_server_fb_write,       "Asynch write", NULL },
+    { MSG_FBCURSOR,                     fb_server_fb_cursor,      "Cursor", NULL },
+    { MSG_FBGETCURSOR,                  fb_server_fb_getcursor,   "Get Cursor", NULL },      /*NEW*/
+    { MSG_FBSCURSOR,                    fb_server_fb_scursor,     "Screen Cursor", NULL }, /*OLD*/
+    { MSG_FBWINDOW,                     fb_server_fb_window,      "Window", NULL },          /*OLD*/
+    { MSG_FBZOOM,                       fb_server_fb_zoom,        "Zoom", NULL },    /*OLD*/
+    { MSG_FBVIEW,                       fb_server_fb_view,        "View", NULL },    /*NEW*/
+    { MSG_FBGETVIEW,                    fb_server_fb_getview,     "Get View", NULL },        /*NEW*/
+    { MSG_FBRMAP,                       fb_server_fb_rmap,        "R Map", NULL },
+    { MSG_FBWMAP,                       fb_server_fb_wmap,        "W Map", NULL },
+    { MSG_FBHELP,                       fb_server_fb_help,        "Help Request", NULL },
+    { MSG_ERROR,                        fb_server_fb_unknown,     "Error Message", NULL },
+    { MSG_CLOSE,                        fb_server_fb_unknown,     "Close Connection", NULL },
+    { MSG_FBREADRECT,                   fb_server_fb_readrect,    "Read Rectangle", NULL },
+    { MSG_FBWRITERECT,                  fb_server_fb_writerect,   "Write Rectangle", NULL },
+    { MSG_FBWRITERECT + MSG_NORETURN,   fb_server_fb_writerect,   "Write Rectangle", NULL },
+    { MSG_FBBWREADRECT,                 fb_server_fb_bwreadrect,  "Read BW Rectangle", NULL },
+    { MSG_FBBWWRITERECT,                fb_server_fb_bwwriterect, "Write BW Rectangle", NULL },
+    { MSG_FBBWWRITERECT + MSG_NORETURN, fb_server_fb_bwwriterect, "Write BW Rectangle", NULL },
+    { MSG_FBFLUSH,                      fb_server_fb_flush,       "Flush Output", NULL },
+    { MSG_FBFLUSH + MSG_NORETURN,       fb_server_fb_flush,       "Flush Output", NULL },
+    { MSG_FBFREE,                       fb_server_fb_free,        "Free Resources", NULL },
+    { MSG_FBPOLL,                       fb_server_fb_poll,        "Handle Events", NULL },
+    { MSG_FBSETCURSOR,                  fb_server_fb_setcursor,   "Set Cursor Shape", NULL },
+    { MSG_FBSETCURSOR + MSG_NORETURN,   fb_server_fb_setcursor,   "Set Cursor Shape", NULL },
+    { 0,                                NULL,           NULL, NULL }
+};
+
 
 /*
  * Local Variables:
