@@ -1,7 +1,7 @@
 /*                         S E T U P . C
  * BRL-CAD
  *
- * Copyright (c) 1985-2013 United States Government as represented by
+ * Copyright (c) 1985-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This program is free software; you can redistribute it and/or
@@ -28,14 +28,11 @@
 /* system headers */
 #include <stdlib.h>
 #include <tcl.h>
-#include <itcl.h>
 #include <string.h>
 
 /* common headers */
-#include "bio.h"
-#include "bu.h"
+#include "bu/app.h"
 #include "bn.h"
-#include "dg.h"
 #include "vmath.h"
 #include "tclcad.h"
 #include "ged.h"
@@ -44,7 +41,6 @@
 #include "./mged.h"
 #include "./cmd.h"
 
-
 /* catch auto-formatting errors in this file.  be careful as there are
  * mged_display() vars that use comma too.
  */
@@ -87,6 +83,7 @@ static struct cmdtab mged_cmdtab[] = {
     {"bot", cmd_ged_plain_wrapper, ged_bot},
     {"bot_condense", cmd_ged_plain_wrapper, ged_bot_condense},
     {"bot_decimate", cmd_ged_plain_wrapper, ged_bot_decimate},
+    {"bot_dump", cmd_ged_plain_wrapper, ged_bot_dump},
     {"bot_face_fuse", cmd_ged_plain_wrapper, ged_bot_face_fuse},
     {"bot_face_sort", cmd_ged_plain_wrapper, ged_bot_face_sort},
     {"bot_flip", cmd_ged_plain_wrapper, ged_bot_flip},
@@ -102,6 +99,7 @@ static struct cmdtab mged_cmdtab[] = {
     {"cat", cmd_ged_info_wrapper, ged_cat},
     {"cc", cmd_ged_plain_wrapper, ged_cc},
     {"center", cmd_center, GED_FUNC_PTR_NULL},
+    {"check", cmd_ged_plain_wrapper, ged_check},
     {"clone", cmd_ged_edit_wrapper, ged_clone},
     {"closedb", f_closedb, GED_FUNC_PTR_NULL},
     {"cmd_win", cmd_cmd_win, GED_FUNC_PTR_NULL},
@@ -109,6 +107,7 @@ static struct cmdtab mged_cmdtab[] = {
     {"color", cmd_ged_plain_wrapper, ged_color},
     {"comb", cmd_ged_plain_wrapper, ged_comb},
     {"comb_color", cmd_ged_plain_wrapper, ged_comb_color},
+    {"constraint", cmd_ged_plain_wrapper, ged_constraint},
     {"copyeval", cmd_ged_plain_wrapper, ged_copyeval},
     {"copymat", cmd_ged_plain_wrapper, ged_copymat},
     {"cp", cmd_ged_plain_wrapper, ged_copy},
@@ -120,10 +119,10 @@ static struct cmdtab mged_cmdtab[] = {
     {"dbfind", cmd_ged_info_wrapper, ged_find},
     {"dbip", cmd_ged_plain_wrapper, ged_dbip},
     {"dbversion", cmd_ged_plain_wrapper, ged_version},
+    {"debug", cmd_ged_plain_wrapper, ged_debug},
     {"debugbu", cmd_ged_plain_wrapper, ged_debugbu},
     {"debugdir", cmd_ged_plain_wrapper, ged_debugdir},
     {"debuglib", cmd_ged_plain_wrapper, ged_debuglib},
-    {"debugmem", cmd_ged_plain_wrapper, ged_debugmem},
     {"debugnmg", cmd_ged_plain_wrapper, ged_debugnmg},
     {"decompose", cmd_ged_plain_wrapper, ged_decompose},
     {"delay", cmd_ged_plain_wrapper, ged_delay},
@@ -131,6 +130,7 @@ static struct cmdtab mged_cmdtab[] = {
     {"dump", cmd_ged_plain_wrapper, ged_dump},
     {"dm", f_dm, GED_FUNC_PTR_NULL},
     {"draw", cmd_draw, GED_FUNC_PTR_NULL},
+    {"dsp", cmd_ged_plain_wrapper, ged_dsp},
     {"dup", cmd_ged_plain_wrapper, ged_dup},
     {"E", cmd_E, GED_FUNC_PTR_NULL},
     {"e", cmd_draw, GED_FUNC_PTR_NULL},
@@ -143,10 +143,9 @@ static struct cmdtab mged_cmdtab[] = {
     {"edcomb", cmd_ged_plain_wrapper, ged_edcomb},
     {"edgedir", f_edgedir, GED_FUNC_PTR_NULL},
     {"edmater", f_edmater, GED_FUNC_PTR_NULL},
-    {"em", cmd_emuves, GED_FUNC_PTR_NULL},
+    {"env", cmd_ged_plain_wrapper, ged_env},
     {"erase", cmd_ged_erase_wrapper, ged_erase},
     {"ev", cmd_ev, GED_FUNC_PTR_NULL},
-    {"e_muves", f_e_muves, GED_FUNC_PTR_NULL},
     {"eqn", f_eqn, GED_FUNC_PTR_NULL},
     {"exit", f_quit, GED_FUNC_PTR_NULL},
     {"expand", cmd_ged_plain_wrapper, ged_expand},
@@ -159,6 +158,7 @@ static struct cmdtab mged_cmdtab[] = {
     {"fracture", cmd_ged_plain_wrapper, ged_fracture},
     {"front", f_bv_front, GED_FUNC_PTR_NULL},
     {"g", cmd_ged_plain_wrapper, ged_group},
+    {"gdiff", cmd_ged_plain_wrapper, ged_gdiff},
     {"get", cmd_ged_plain_wrapper, ged_get},
     {"get_autoview", cmd_ged_plain_wrapper, ged_get_autoview},
     {"get_comb", cmd_ged_plain_wrapper, ged_get_comb},
@@ -173,6 +173,7 @@ static struct cmdtab mged_cmdtab[] = {
     {"grid2model_lu", cmd_ged_plain_wrapper, ged_grid2model_lu},
     {"grid2view_lu", cmd_ged_plain_wrapper, ged_grid2view_lu},
     {"has_embedded_fb", cmd_has_embedded_fb, GED_FUNC_PTR_NULL},
+    {"heal", cmd_ged_plain_wrapper, ged_heal},
     {"hide", cmd_ged_plain_wrapper, ged_hide},
     {"hist", cmd_hist, GED_FUNC_PTR_NULL},
     {"history", f_history, GED_FUNC_PTR_NULL},
@@ -183,6 +184,7 @@ static struct cmdtab mged_cmdtab[] = {
     {"inside", cmd_ged_inside, ged_inside},
     {"item", cmd_ged_plain_wrapper, ged_item},
     {"joint", cmd_ged_plain_wrapper, ged_joint},
+    {"joint2", cmd_ged_plain_wrapper, ged_joint2},
     {"journal", f_journal, GED_FUNC_PTR_NULL},
     {"keep", cmd_ged_plain_wrapper, ged_keep},
     {"keypoint", f_keypoint, GED_FUNC_PTR_NULL},
@@ -192,11 +194,12 @@ static struct cmdtab mged_cmdtab[] = {
     {"killtree", cmd_ged_erase_wrapper, ged_killtree},
     {"knob", f_knob, GED_FUNC_PTR_NULL},
     {"l", cmd_ged_info_wrapper, ged_list},
-    {"l_muves", f_l_muves, GED_FUNC_PTR_NULL},
     {"labelvert", f_labelvert, GED_FUNC_PTR_NULL},
+    {"labelface", f_labelface, GED_FUNC_PTR_NULL},
+    {"lc", cmd_ged_plain_wrapper, ged_lc},
     {"left", f_bv_left, GED_FUNC_PTR_NULL},
+    {"lint", cmd_ged_plain_wrapper, ged_lint},
     {"listeval", cmd_ged_plain_wrapper, ged_pathsum},
-    {"lm", cmd_lm, GED_FUNC_PTR_NULL},
     {"loadtk", cmd_tk, GED_FUNC_PTR_NULL},
     {"loadview", cmd_ged_view_wrapper, ged_loadview},
     {"lod", cmd_ged_plain_wrapper, ged_lod},
@@ -206,13 +209,15 @@ static struct cmdtab mged_cmdtab[] = {
     {"M", f_mouse, GED_FUNC_PTR_NULL},
     {"m2v_point", cmd_ged_plain_wrapper, ged_m2v_point},
     {"make", f_make, GED_FUNC_PTR_NULL},
-    {"make_bb", cmd_ged_plain_wrapper, ged_make_bb},
     {"make_name", cmd_ged_plain_wrapper, ged_make_name},
     {"make_pnts", cmd_ged_more_wrapper, ged_make_pnts},
     {"match", cmd_ged_plain_wrapper, ged_match},
     {"mater", cmd_ged_plain_wrapper, ged_mater},
     {"matpick", f_matpick, GED_FUNC_PTR_NULL},
-    {"memprint", f_memprint, GED_FUNC_PTR_NULL},
+    {"mat_ae", cmd_ged_plain_wrapper, ged_mat_ae},
+    {"mat_mul", cmd_ged_plain_wrapper, ged_mat_mul},
+    {"mat4x3pnt", cmd_ged_plain_wrapper, ged_mat4x3pnt},
+    {"mat_scale_about_pnt", cmd_ged_plain_wrapper, ged_mat_scale_about_pnt},
     {"mged_update", f_update, GED_FUNC_PTR_NULL},
     {"mged_wait", f_wait, GED_FUNC_PTR_NULL},
     {"mirface", f_mirface, GED_FUNC_PTR_NULL},
@@ -229,6 +234,8 @@ static struct cmdtab mged_cmdtab[] = {
     {"nmg_collapse", cmd_nmg_collapse, GED_FUNC_PTR_NULL},
     {"nmg_fix_normals", cmd_ged_plain_wrapper, ged_nmg_fix_normals},
     {"nmg_simplify", cmd_ged_plain_wrapper, ged_nmg_simplify},
+    {"nmg", cmd_ged_plain_wrapper, ged_nmg},
+    {"npush", cmd_ged_plain_wrapper, ged_npush},
     {"o_rotate", f_be_o_rotate, GED_FUNC_PTR_NULL},
     {"o_scale", f_be_o_scale, GED_FUNC_PTR_NULL},
     {"oed", cmd_oed, GED_FUNC_PTR_NULL},
@@ -248,20 +255,19 @@ static struct cmdtab mged_cmdtab[] = {
     {"oyscale", f_be_o_yscale, GED_FUNC_PTR_NULL},
     {"ozscale", f_be_o_zscale, GED_FUNC_PTR_NULL},
     {"p", f_param, GED_FUNC_PTR_NULL},
-    {"parse_points", cmd_parse_points, GED_FUNC_PTR_NULL},
     {"pathlist", cmd_ged_plain_wrapper, ged_pathlist},
     {"paths", cmd_ged_plain_wrapper, ged_pathsum},
     {"permute", f_permute, GED_FUNC_PTR_NULL},
-    {"pl", f_pl, GED_FUNC_PTR_NULL},
     {"plot", cmd_ged_plain_wrapper, ged_plot},
     {"png", cmd_ged_plain_wrapper, ged_png},
-    {"polybinout", f_polybinout, GED_FUNC_PTR_NULL},
-    {"pov", cmd_pov, GED_FUNC_PTR_NULL},
+    {"pnts", cmd_ged_plain_wrapper, ged_pnts},
     {"prcolor", cmd_ged_plain_wrapper, ged_prcolor},
     {"prefix", cmd_ged_plain_wrapper, ged_prefix},
     {"press", f_press, GED_FUNC_PTR_NULL},
     {"preview", cmd_ged_dm_wrapper, ged_preview},
-    {"ps", f_ps, GED_FUNC_PTR_NULL},
+    {"process", cmd_ged_plain_wrapper, ged_process},
+    {"postscript", f_postscript, GED_FUNC_PTR_NULL},
+    {"ps", cmd_ps, GED_FUNC_PTR_NULL},
     {"pull", cmd_ged_plain_wrapper, ged_pull},
     {"push", cmd_ged_plain_wrapper, ged_push},
     {"put", cmd_ged_plain_wrapper, ged_put},
@@ -277,7 +283,6 @@ static struct cmdtab mged_cmdtab[] = {
     {"r", cmd_ged_plain_wrapper, ged_region},
     {"rate", f_bv_rate_toggle, GED_FUNC_PTR_NULL},
     {"rcodes", cmd_ged_plain_wrapper, ged_rcodes},
-    {"read_muves", f_read_muves, GED_FUNC_PTR_NULL},
     {"rear", f_bv_rear, GED_FUNC_PTR_NULL},
     {"red", f_red, GED_FUNC_PTR_NULL},
     {"refresh", f_refresh, GED_FUNC_PTR_NULL},
@@ -325,11 +330,12 @@ static struct cmdtab mged_cmdtab[] = {
     {"sill", f_be_s_illuminate, GED_FUNC_PTR_NULL},
     {"size", cmd_size, GED_FUNC_PTR_NULL},
     {"simulate", cmd_ged_simulate_wrapper, ged_simulate},
-    {"solid_report", cmd_ged_plain_wrapper, ged_report},
+    {"solid_report", cmd_ged_plain_wrapper, ged_solid_report},
     {"solids", cmd_ged_plain_wrapper, ged_tables},
     {"solids_on_ray", cmd_ged_plain_wrapper, ged_solids_on_ray},
     {"srot", f_be_s_rotate, GED_FUNC_PTR_NULL},
     {"sscale", f_be_s_scale, GED_FUNC_PTR_NULL},
+    {"stat", cmd_ged_plain_wrapper, ged_stat},
     {"status", f_status, GED_FUNC_PTR_NULL},
     {"stuff_str", cmd_stuff_str, GED_FUNC_PTR_NULL},
     {"summary", cmd_ged_plain_wrapper, ged_summary},
@@ -338,7 +344,6 @@ static struct cmdtab mged_cmdtab[] = {
     {"sxy", f_be_s_trans, GED_FUNC_PTR_NULL},
     {"sync", cmd_ged_plain_wrapper, ged_sync},
     {"t", cmd_ged_plain_wrapper, ged_ls},
-    {"t_muves", f_t_muves, GED_FUNC_PTR_NULL},
     {"ted", f_tedit, GED_FUNC_PTR_NULL},
     {"tie", f_tie, GED_FUNC_PTR_NULL},
     {"tire", cmd_ged_plain_wrapper, ged_tire},
@@ -376,7 +381,7 @@ static struct cmdtab mged_cmdtab[] = {
     {"who", cmd_ged_plain_wrapper, ged_who},
     {"winset", f_winset, GED_FUNC_PTR_NULL},
     {"wmater", cmd_ged_plain_wrapper, ged_wmater},
-    {"x", cmd_ged_plain_wrapper, ged_report},
+    {"x", cmd_ged_plain_wrapper, ged_solid_report},
     {"xpush", cmd_ged_plain_wrapper, ged_xpush},
     {"Z", cmd_zap, GED_FUNC_PTR_NULL},
     {"zoom", cmd_zoom, GED_FUNC_PTR_NULL},
@@ -387,8 +392,6 @@ static struct cmdtab mged_cmdtab[] = {
 
 
 /**
- * C M D _ S E T U P
- *
  * Register all MGED commands.
  */
 HIDDEN void
@@ -396,12 +399,6 @@ cmd_setup(void)
 {
     struct cmdtab *ctp;
     struct bu_vls temp = BU_VLS_INIT_ZERO;
-    const char *pathname;
-    char buffer[1024];
-
-    /* from cmd.c */
-    extern int glob_compat_mode;
-    extern int output_as_return;
 
     for (ctp = mged_cmdtab; ctp->name != NULL; ctp++) {
 	bu_vls_strcpy(&temp, "_mged_");
@@ -413,23 +410,7 @@ cmd_setup(void)
 				(ClientData)ctp, (Tcl_CmdDeleteProc *)NULL);
     }
 
-    /* overrides/wraps the built-in tree command */
-
-    /* Locate the BRL-CAD-specific Tcl scripts */
-    pathname = bu_brlcad_data("tclscripts", 1);
-    snprintf(buffer, sizeof(buffer), "%s", pathname);
-
-    /* link some tcl variables to these corresponding globals */
-    Tcl_LinkVar(INTERP, "glob_compat_mode", (char *)&glob_compat_mode, TCL_LINK_BOOLEAN);
-    Tcl_LinkVar(INTERP, "output_as_return", (char *)&output_as_return, TCL_LINK_BOOLEAN);
-
-    /* Provide Tcl interfaces to the fundamental BRL-CAD libraries */
-    if (Bu_Init(INTERP) == TCL_ERROR) {
-      bu_log("Bu_Init ERROR:\n%s\n", Tcl_GetStringResult(INTERP));
-    }
-    Bn_Init(INTERP);
-    Rt_Init(INTERP);
-    Go_Init(INTERP);
+    /* Init mged's Tcl interface to libwdb */
     Wdb_Init(INTERP);
 
     tkwin = NULL;
@@ -444,12 +425,9 @@ cmd_setup(void)
 void
 mged_setup(Tcl_Interp **interpreter)
 {
-    int try_auto_path = 0;
-
-    int init_tcl = 1;
-    int init_itcl = 1;
     struct bu_vls str = BU_VLS_INIT_ZERO;
-    const char *name = bu_argv0_full_path();
+    struct bu_vls tlog = BU_VLS_INIT_ZERO;
+    const char *name = bu_dir(NULL, 0, BU_DIR_BIN, bu_getprogname(), BU_DIR_EXT, NULL);
 
     /* locate our run-time binary (must be called before Tcl_CreateInterp()) */
     if (name) {
@@ -469,113 +447,29 @@ mged_setup(Tcl_Interp **interpreter)
     /* Create the interpreter */
     *interpreter = Tcl_CreateInterp();
 
-    /* a two-pass init loop.  the first pass just tries default init
-     * routines while the second calls tclcad_auto_path() to help it
-     * find other, potentially uninstalled, resources.
-     */
-    while (1) {
-
-	/* not called first time through, give Tcl_Init() a chance */
-	if (try_auto_path) {
-	    /* Locate the BRL-CAD-specific Tcl scripts, set the auto_path */
-	    tclcad_auto_path(*interpreter);
-	}
-
-	/* Initialize Tcl */
-	Tcl_ResetResult(*interpreter);
-	if (init_tcl && Tcl_Init(*interpreter) == TCL_ERROR) {
-	    if (!try_auto_path) {
-		try_auto_path = 1;
-		continue;
-	    }
-	    bu_log("Tcl_Init ERROR:\n%s\n", Tcl_GetStringResult(*interpreter));
-	    break;
-	}
-	init_tcl = 0;
-
-	/* Initialize [incr Tcl] */
-	Tcl_ResetResult(*interpreter);
-	/* NOTE: Calling "package require Itcl" here is apparently
-	 * insufficient without other changes elsewhere.  The
-	 * Combination Editor in mged fails with an iwidgets class
-	 * already loaded error if we don't perform Itcl_Init() here.
-	 */
-	if (init_itcl && Itcl_Init(*interpreter) == TCL_ERROR) {
-	    if (!try_auto_path) {
-		Tcl_Namespace *nsp;
-
-		try_auto_path = 1;
-		/* Itcl_Init() leaves initialization in a bad state
-		 * and can cause retry failures.  cleanup manually.
-		 */
-		Tcl_DeleteCommand(*interpreter, "::itcl::class");
-		nsp = Tcl_FindNamespace(*interpreter, "::itcl", NULL, 0);
-		if(nsp)
-		    Tcl_DeleteNamespace(nsp);
-		continue;
-	    }
-	    bu_log("Itcl_Init ERROR:\n%s\n", Tcl_GetStringResult(*interpreter));
-	    break;
-	}
-	init_itcl = 0;
-
-	/* don't actually want to loop forever */
-	break;
-
-    } /* end iteration over Init() routines that need auto_path */
-    Tcl_ResetResult(*interpreter);
-
-    /* if we haven't loaded by now, load auto_path so we find our tclscripts */
-    if (!try_auto_path) {
-	/* Locate the BRL-CAD-specific Tcl scripts */
-	tclcad_auto_path(*interpreter);
+    /* Do basic Tcl initialization - note that Tk
+     * is not initialized at this point. */
+    if (tclcad_init(*interpreter, 0, &tlog) == TCL_ERROR) {
+	bu_log("tclcad_init error:\n%s\n", bu_vls_addr(&tlog));
     }
+    bu_vls_free(&tlog);
 
-    /*XXX FIXME: Should not be importing Itcl into the global namespace */
-    /* Import [incr Tcl] commands into the global namespace. */
-    if (Tcl_Import(*interpreter, Tcl_GetGlobalNamespace(*interpreter), "::itcl::*", /* allowOverwrite */ 1) != TCL_OK) {
-	bu_log("Tcl_Import ERROR: %s\n", Tcl_GetStringResult(*interpreter));
-	Tcl_ResetResult(*interpreter);
-    }
-
-    /* Initialize libbu */
-    if (Bu_Init(*interpreter) == TCL_ERROR) {
-	bu_log("Bu_Init ERROR:\n%s\n", Tcl_GetStringResult(*interpreter));
-	Tcl_ResetResult(*interpreter);
-    }
-
-    /* Initialize libbn */
-    if (Bn_Init(*interpreter) == TCL_ERROR) {
-	bu_log("Bn_Init ERROR:\n%s\n", Tcl_GetStringResult(*interpreter));
-	Tcl_ResetResult(*interpreter);
-    }
-
-    /* Initialize librt */
-    if (Rt_Init(*interpreter) == TCL_ERROR) {
-	bu_log("Rt_Init ERROR:\n%s\n", Tcl_GetStringResult(*interpreter));
-	Tcl_ResetResult(*interpreter);
-    }
-
-    /* Initialize libged */
-    if (Go_Init(*interpreter) == TCL_ERROR) {
-	bu_log("Ged_Init ERROR:\n%s\n", Tcl_GetStringResult(*interpreter));
-	Tcl_ResetResult(*interpreter);
-    }
-
-    BU_ALLOC(view_state->vs_gvp, struct ged_view);
+    BU_ALLOC(view_state->vs_gvp, struct bview);
+    BU_GET(view_state->vs_gvp->callbacks, struct bu_ptbl);
+    bu_ptbl_init(view_state->vs_gvp->callbacks, 8, "bview callbacks");
     ged_view_init(view_state->vs_gvp);
 
     view_state->vs_gvp->gv_callback = mged_view_callback;
-    view_state->vs_gvp->gv_clientData = (genptr_t)view_state;
+    view_state->vs_gvp->gv_clientData = (void *)view_state;
     MAT_DELTAS_GET_NEG(view_state->vs_orig_pos, view_state->vs_gvp->gv_center);
 
-    if (gedp) {
+    if (GEDP) {
 	/* release any allocated memory */
-	ged_free(gedp);
+	ged_free(GEDP);
     } else {
-	BU_ALLOC(gedp, struct ged);
+	BU_ALLOC(GEDP, struct ged);
     }
-    GED_INIT(gedp, NULL);
+    GED_INIT(GEDP, NULL);
 
     /* register commands */
     cmd_setup();
@@ -583,6 +477,8 @@ mged_setup(Tcl_Interp **interpreter)
     history_setup();
     mged_global_variable_setup(*interpreter);
     mged_variable_setup(*interpreter);
+    GEDP->ged_interp = (void *)*interpreter;
+    GEDP->ged_interp_eval = &mged_db_search_callback;
 
     /* Tcl needs to write nulls onto subscripted variable names */
     bu_vls_printf(&str, "%s(state)", MGED_DISPLAY_VAR);
