@@ -1,7 +1,7 @@
 /*                          P I P E . C
  * BRL-CAD
  *
- * Copyright (c) 2004-2013 United States Government as represented by
+ * Copyright (c) 2004-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This program is free software; you can redistribute it and/or
@@ -34,6 +34,8 @@
 #include <string.h>
 #include "bio.h"
 
+#include "bu/app.h"
+#include "bu/getopt.h"
 #include "vmath.h"
 #include "raytrace.h"
 #include "wdb.h"
@@ -73,7 +75,7 @@ struct points
 };
 
 
-double radius, wall, pi, k;
+double radius, wall, pi, unit_conversion_factor;
 struct points *root;
 char name[16];
 float delta=10.0;		/* 10mm excess in ARB sizing for cutting tubes */
@@ -134,7 +136,7 @@ Readpoints(void)
 	ptr->next = NULL;
 	ptr->prev = prev;
 	prev = ptr;
-	VSET(ptr->p, k*x, k*y, k*z);
+	VSET(ptr->p, unit_conversion_factor*x, unit_conversion_factor*y, unit_conversion_factor*z);
 	ptr->tube[0] = '\0';
 	ptr->tubflu[0] = '\0';
 	ptr->elbow[0] = '\0';
@@ -613,11 +615,13 @@ Groups(void)
 int
 main(int argc, char **argv)
 {
-    int done;
+    int done = 0;
     char units[16], fname[80];
     int optc;
 
-    while ((optc = bu_getopt(argc, argv, "tsmnc")) != -1) {
+    bu_setprogname(argv[0]);
+
+    while ((optc = bu_getopt(argc, argv, "tsmnch?")) != -1) {
 	/* Set joint type and cable option */
 	switch (optc) {
 	    case 't':
@@ -635,11 +639,9 @@ main(int argc, char **argv)
 	    case 'c':
 		cable = 1;
 		break;
-	    case '?':
-		fprintf(stderr, "Illegal option %c\n", optc);
+	    default:
 		Usage();
 		return 1;
-		break;
 
 	}
     }
@@ -649,8 +651,6 @@ main(int argc, char **argv)
 	Usage();
 	fprintf(stderr, "Options t, s, m, n are mutually exclusive\n");
 	return 1;
-    } else if ((torus + sphere + mitre + nothing) == 0) {
-	torus = 1;		/* default */
     }
 
     if ((argc - bu_optind) != 2) {
@@ -658,6 +658,9 @@ main(int argc, char **argv)
 	return 1;
     }
 
+    if ((torus + sphere + mitre + nothing) == 0)
+	torus = 1;		/* default */
+
     bu_strlcpy(name, argv[bu_optind++], sizeof(name)); /* Base name for objects */
 
     fdout = wdb_fopen(argv[bu_optind]);
@@ -669,42 +672,49 @@ main(int argc, char **argv)
     }
 
     MAT_IDN(identity);	/* Identity matrix for all objects */
-    pi = atan2(0.0, -1.0);	/* PI */
+    pi = M_PI;	/* PI */
 
     printf("FLUID & PIPING V%d.%d 10 Mar 89\n\n", VERSION, RELEASE);
-    printf("append %s to your target description using 'concat' in mged\n", argv[bu_optind]);
+    printf("Append %s to your target description using 'concat' in mged.\n", argv[bu_optind]);
 
-    k = 0.0;
-    while (ZERO(k)) {
+#define MM_TO_CM 10.0
+#define MM_TO_M  1000.0
+#define MM_TO_IN 25.4
+#define MM_TO_FT 304.8
+
+    unit_conversion_factor = 0.0;
+    while (ZERO(unit_conversion_factor)) {
 	printf("UNITS? (ft, in, m, cm, default is millimeters) ");
 	bu_fgets(units, sizeof(units), stdin);
 	switch (units[0]) {
-
 	    case '\0':
-		k = 1.0;
+	    case '\n':
+	    case '\r':
+		unit_conversion_factor=1.0;
 		break;
 
 	    case 'f':
-		k = 12*25.4;
+		unit_conversion_factor=MM_TO_FT;
 		break;
 
 	    case 'i':
-		k=25.4;
+		unit_conversion_factor=MM_TO_IN;
 		break;
 
 	    case 'm':
-		if (units[1] == '\0') k=1000.0;
-		else k=1.0;
+		if (units[1] != 'm')
+		    unit_conversion_factor=MM_TO_M;
+		else
+		    unit_conversion_factor=1.0;
 		break;
 
 	    case 'c':
-		k=10.0;
+		unit_conversion_factor=MM_TO_CM;
 		break;
 
 	    default:
-		k=0.0;
-		printf("\n\t%s is not a legal choice for units\n", units);
-		printf("\tTry again\n");
+		printf("\n%s is not a legal choice for units\n", units);
+		printf("Try again\n");
 		break;
 	}
     }
@@ -719,8 +729,8 @@ main(int argc, char **argv)
 		done = 1;
 	    else {
 		printf(" *** bad input!\n\n");
-		printf("\tradius must be larger than wall thickness\n");
-		printf("\tTry again\n");
+		printf("radius must be larger than wall thickness\n");
+		printf("Try again\n");
 	    }
 	} else {
 	    printf("radius: ");
@@ -732,8 +742,8 @@ main(int argc, char **argv)
     if (radius < SMALL_FASTF)
 	radius = SMALL_FASTF;
 
-    radius=k*radius;
-    wall=k*wall;
+    radius=unit_conversion_factor*radius;
+    wall=unit_conversion_factor*wall;
 
     Readpoints();	/* Read data points */
 
@@ -770,7 +780,7 @@ void
 Usage(void)
 {
     fprintf(stderr, "Usage: pipe [-tsmnc] tag filename\n");
-    fprintf(stderr, "   where 'tag' is the name of the piping run\n");
+    fprintf(stderr, "   where 'tag' is the name of the piping run and is used by mged in object names;\n");
     fprintf(stderr, "   and 'filename' is the .g file (e.g., fuel.g)\n");
     fprintf(stderr, "   -t -> use tori at the bends (default)\n");
     fprintf(stderr, "   -s -> use spheres at the corners\n");
