@@ -1,7 +1,7 @@
 /*                    V E G E T A T I O N . C
  * BRL-CAD
  *
- * Copyright (c) 1998-2013 United States Government as represented by
+ * Copyright (c) 1998-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This program is free software; you can redistribute it and/or
@@ -33,6 +33,7 @@
  */
 
 #include "./vegetation.h"
+#include "bu/app.h"
 
 static void
 ageStructure(structure_t *structure)
@@ -199,7 +200,7 @@ findIntersectors(const growthSegment_t * const segment, const structure_t * cons
     for (i=0; i < structure->subStructureCount; i++) {
 	segList = findIntersectors(segment, structure->subStructure[i], exemptList);
 
-	if(segList == NULL) {
+	if (segList == NULL) {
 	    bu_log("segList is null?\n");
 	    return NULL;
 	}
@@ -302,15 +303,16 @@ branchWithProbability(plant_t *plant, structure_t* structure, unsigned int minAg
 	    if ((structure->segment[i]->age >= minAge) && (structure->segment[i]->age <= maxAge)) {
 
 		/* see if we branch */
-		if (drand48() <= probability) {
+		if (bn_randmt() <= probability) {
 		    double branchPoint, branchPointRadius;
 		    vect_t direction;
 		    growthPoint_t *newGrowthPoint;
 
 		    /* decide whether or not to use an endpoint */
-		    if (drand48() <= plant->characteristic->branchAtEndpointRate) {
+		    if (bn_randmt() <= plant->characteristic->branchAtEndpointRate) {
+
 			/* randomly pick between the two points evenly */
-			if (drand48() < 5.0) {
+			if (bn_randmt() < 0.5) {
 			    branchPoint = 0.0;
 			    branchPointRadius = structure->segment[i]->startRadius;
 			} else {
@@ -323,7 +325,7 @@ branchWithProbability(plant_t *plant, structure_t* structure, unsigned int minAg
 		    } else {
 
 			/* pick a point on the segment to branch from */
-			branchPoint = drand48() * (double)structure->segment[i]->length;
+			branchPoint = bn_randmt() * (double)structure->segment[i]->length;
 			/*
 			  printf("branching between %f and %f at %f\n", 0.0, structure->segment[i]->length,  branchPoint);
 			*/
@@ -347,9 +349,9 @@ branchWithProbability(plant_t *plant, structure_t* structure, unsigned int minAg
 
 		    /* figure out a direction to grow */
 		    VMOVE(direction, structure->segment[i]->direction);
-		    direction[X] += (drand48() * (plant->characteristic->branchMaxVariation[X] - plant->characteristic->branchMinVariation[X])) + plant->characteristic->branchMinVariation[X];
-		    direction[Y] += (drand48() * (plant->characteristic->branchMaxVariation[Y] - plant->characteristic->branchMinVariation[Y])) + plant->characteristic->branchMinVariation[Y];
-		    direction[Z] += (drand48() * (plant->characteristic->branchMaxVariation[Z] - plant->characteristic->branchMinVariation[Z])) + plant->characteristic->branchMinVariation[Z];
+		    direction[X] += (bn_randmt() * (plant->characteristic->branchMaxVariation[X] - plant->characteristic->branchMinVariation[X])) + plant->characteristic->branchMinVariation[X];
+		    direction[Y] += (bn_randmt() * (plant->characteristic->branchMaxVariation[Y] - plant->characteristic->branchMinVariation[Y])) + plant->characteristic->branchMinVariation[Y];
+		    direction[Z] += (bn_randmt() * (plant->characteristic->branchMaxVariation[Z] - plant->characteristic->branchMinVariation[Z])) + plant->characteristic->branchMinVariation[Z];
 		    VUNITIZE(direction);
 		    /*
 		      VPRINT("New Growth Direction: ", direction);
@@ -362,7 +364,7 @@ branchWithProbability(plant_t *plant, structure_t* structure, unsigned int minAg
 
 		    /* length and radius is based off of the segment we grew off of -- random initial length */
 		    /* !!! just use the prior length until working !!! */
-		    newGrowthPoint->length = structure->segment[i]->length - (structure->segment[i]->length * plant->characteristic->lengthDecayRate); /* * drand48(); */
+		    newGrowthPoint->length = structure->segment[i]->length - (structure->segment[i]->length * plant->characteristic->lengthDecayRate); /* * bn_randmt(); */
 
 		    /* starting radius is exactly in line with where on the segment we start from */
 		    newGrowthPoint->radius = branchPointRadius;
@@ -509,7 +511,7 @@ growPlant(plant_t *plant)
 	    point->length -= point->length * point->lengthDecay;
 	    /* jitter next segment length */
 	    if (!NEAR_EQUAL(plant->characteristic->lengthMaxVariation, plant->characteristic->lengthMinVariation, ZERO_TOLERANCE)) {
-		point->length += (drand48() * (plant->characteristic->lengthMaxVariation - plant->characteristic->lengthMinVariation)) + plant->characteristic->lengthMinVariation;
+		point->length += (bn_randmt() * (plant->characteristic->lengthMaxVariation - plant->characteristic->lengthMinVariation)) + plant->characteristic->lengthMinVariation;
 	    }
 	    /* clamp the length to positive values */
 	    if (point->length < ZERO_TOLERANCE) {
@@ -523,7 +525,7 @@ growPlant(plant_t *plant)
 	    point->radius -= point->radius * point->radiusDecay;
 	    /* jitter next radius */
 	    if (!NEAR_EQUAL(plant->characteristic->radiusMaxVariation, plant->characteristic->radiusMinVariation, ZERO_TOLERANCE)) {
-		point->radius += (drand48() * (plant->characteristic->radiusMaxVariation - plant->characteristic->radiusMinVariation)) + plant->characteristic->radiusMinVariation;
+		point->radius += (bn_randmt() * (plant->characteristic->radiusMaxVariation - plant->characteristic->radiusMinVariation)) + plant->characteristic->radiusMinVariation;
 	    }
 	    /* clamp the radius to positive values */
 	    if (point->radius < ZERO_TOLERANCE) {
@@ -537,9 +539,9 @@ growPlant(plant_t *plant)
 	    /* jitter the growth direction */
 	    VSUB2(newGrowthDirection, plant->characteristic->dirMaxVariation, plant->characteristic->dirMinVariation);
 	    if (!VNEAR_ZERO(newGrowthDirection, ZERO_TOLERANCE)) {
-		point->direction[X] += (drand48() * newGrowthDirection[X]) + plant->characteristic->dirMinVariation[X];
-		point->direction[Y] += (drand48() * newGrowthDirection[Y]) + plant->characteristic->dirMinVariation[Y];
-		point->direction[Z] += (drand48() * newGrowthDirection[Z]) + plant->characteristic->dirMinVariation[Z];
+		point->direction[X] += (bn_randmt() * newGrowthDirection[X]) + plant->characteristic->dirMinVariation[X];
+		point->direction[Y] += (bn_randmt() * newGrowthDirection[Y]) + plant->characteristic->dirMinVariation[Y];
+		point->direction[Z] += (bn_randmt() * newGrowthDirection[Z]) + plant->characteristic->dirMinVariation[Z];
 		VUNITIZE(point->direction);
 		/* VPRINT("Point direction: ", point->direction); */
 	    }
@@ -575,9 +577,9 @@ growPlant(plant_t *plant)
 		    /* jitter the growth direction */
 		    VSUB2(newGrowthDirection, plant->characteristic->dirMaxVariation, plant->characteristic->dirMinVariation);
 		    if (!VNEAR_ZERO(newGrowthDirection, ZERO_TOLERANCE)) {
-			point->direction[X] += (drand48() * newGrowthDirection[X]) + plant->characteristic->dirMinVariation[X];
-			point->direction[Y] += (drand48() * newGrowthDirection[Y]) + plant->characteristic->dirMinVariation[Y];
-			point->direction[Z] += (drand48() * newGrowthDirection[Z]) + plant->characteristic->dirMinVariation[Z];
+			point->direction[X] += (bn_randmt() * newGrowthDirection[X]) + plant->characteristic->dirMinVariation[X];
+			point->direction[Y] += (bn_randmt() * newGrowthDirection[Y]) + plant->characteristic->dirMinVariation[Y];
+			point->direction[Z] += (bn_randmt() * newGrowthDirection[Z]) + plant->characteristic->dirMinVariation[Z];
 			VUNITIZE(point->direction);
 			/* VPRINT("Point direction: ", point->direction); */
 		    }
@@ -895,49 +897,46 @@ main(int argc, char *argv[])
     double branchingRate = 0.1;  /* 0->1 probability to branch per iteration */
     long seed;
 
+    bu_setprogname(argv[0]);
+
     printf("Vegetation generator\n");
     printf("====================\n");
 
-    if (argc > 1) {
+    if (argc > 1)
 	age = atoi(argv[1]);
-    }
+
     if (age == 0)
 	age = 1;
-    if (age > UINT32_MAX)
+    else if (age > UINT32_MAX)
 	age = UINT32_MAX;
 
     printf("Growing for %d years\n", age);
-    if (argc > 2) {
+    if (argc > 2)
 	height = atof(argv[2]);
-    }
+
     if (height < (SMALL_FASTF * 1000.0))
 	height = (SMALL_FASTF * 1000.0);
 
     printf("Growing to about %f meters in height\n", height / 1000);
-    if (argc > 3) {
+    if (argc > 3)
 	trunkRadius = atof(argv[3]);
-    }
+
     if (trunkRadius < (SMALL_FASTF * 1000.0))
 	trunkRadius = (SMALL_FASTF * 1000.0);
 
     printf("Growing from a base width of %f meters\n", trunkRadius / 1000);
-    if (argc > 4) {
+    if (argc > 4)
 	branchingRate = atof(argv[4]);
-    }
+
     if (branchingRate < SMALL_FASTF)
 	branchingRate = SMALL_FASTF;
 
-    if (argc > 5) {
+    if (argc > 5)
 	seed = atol(argv[5]);
-    }
 
     /* save the seed just in case we want to know it */
     seed=time(0);
-#ifndef HAVE_SRAND48
-    srand(seed);
-#else
-    srand48(seed);
-#endif
+    bn_randmt_seed(seed);
     printf("Vegetation seed is %ld\n", seed);
 
     fp=wdb_fopen("vegetation.g");
