@@ -1,7 +1,7 @@
 /*                            D O . C
  * BRL-CAD
  *
- * Copyright (c) 1987-2013 United States Government as represented by
+ * Copyright (c) 1987-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This program is free software; you can redistribute it and/or
@@ -27,7 +27,6 @@
 #include "common.h"
 
 #include <stdlib.h>
-#include <stdio.h>
 #include <string.h>
 #include <ctype.h>
 #include <math.h>
@@ -40,15 +39,23 @@
 #endif
 
 #include "bio.h"
+
+#include "bu/app.h"
+#include "bu/getopt.h"
+#include "bu/debug.h"
+#include "bu/mime.h"
+#include "bu/vls.h"
 #include "vmath.h"
 #include "raytrace.h"
-#include "fb.h"
-#include "bu.h"
+#include "dm.h"
 #include "icv.h"
 
 #include "./rtuif.h"
 #include "./ext.h"
 
+#if defined(HAVE_FDOPEN) && !defined(HAVE_DECL_FDOPEN)
+extern FILE *fdopen(int fd, const char *mode);
+#endif
 
 /***** Variables shared with viewing model *** */
 extern FILE *outfp;			/* optional pixel output file */
@@ -57,7 +64,7 @@ extern mat_t model2view;
 /***** end of sharing with viewing model *****/
 
 extern void grid_setup(void);
-extern void worker(int cpu, genptr_t arg);
+extern void worker(int cpu, void *arg);
 
 /***** variables shared with opt.c *****/
 extern int	orientflag;		/* 1 means orientation has been set */
@@ -69,10 +76,6 @@ extern char *string_pix_end;	/* string spec of ending pixel */
 extern int finalframe;		/* frame to halt at */
 /***** end variables shared with rt.c *****/
 
-/***** variables shared with viewg3.c *****/
-struct bu_vls ray_data_file = BU_VLS_INIT_ZERO;  /* file name for ray data output */
-/***** end variables shared with viewg3.c *****/
-
 /***** variables for frame buffer black pixel rendering *****/
 unsigned char *pixmap = NULL; /* Pixel Map for rerendering of black pixels */
 
@@ -86,8 +89,6 @@ extern struct icv_image *bif;
 
 
 /**
- * O L D _ F R A M E
- *
  * Acquire particulars about a frame, in the old format.  Returns -1
  * if unable to acquire info, 0 if successful.
  */
@@ -95,19 +96,20 @@ int
 old_frame(FILE *fp)
 {
     register int i;
-    char number[128];
+#define NUMBER_LEN 128
+    char number[NUMBER_LEN+1];
 
     /* Visible part is from -1 to +1 in view space */
-    if (fscanf(fp, "%128s", number) != 1) return -1;
+    if (fscanf(fp, CPP_SCAN(NUMBER_LEN), number) != 1) return -1;
     viewsize = atof(number);
-    if (fscanf(fp, "%128s", number) != 1) return -1;
+    if (fscanf(fp, CPP_SCAN(NUMBER_LEN), number) != 1) return -1;
     eye_model[X] = atof(number);
-    if (fscanf(fp, "%128s", number) != 1) return -1;
+    if (fscanf(fp, CPP_SCAN(NUMBER_LEN), number) != 1) return -1;
     eye_model[Y] = atof(number);
-    if (fscanf(fp, "%128s", number) != 1) return -1;
+    if (fscanf(fp, CPP_SCAN(NUMBER_LEN), number) != 1) return -1;
     eye_model[Z] = atof(number);
-    for (i=0; i < 16; i++) {
-	if (fscanf(fp, "%128s", number) != 1)
+    for (i = 0; i < 16; i++) {
+	if (fscanf(fp, CPP_SCAN(NUMBER_LEN), number) != 1)
 	    return -1;
 	Viewrotscale[i] = atof(number);
     }
@@ -116,8 +118,6 @@ old_frame(FILE *fp)
 
 
 /**
- * O L D _ W A Y
- *
  * Determine if input file is old or new format, and if old format,
  * handle process.  Returns 0 if new way, 1 if old way (and all done).
  * Note that the rewind() will fail on ttys, pipes, and sockets
@@ -152,6 +152,11 @@ old_way(FILE *fp)
     }
     bu_log("Interpreting command stream in old format\n");
 
+    /* Committing to old way - better have objv ready */
+    if (!objv) {
+	return -1;
+    }
+
     def_tree(APP.a_rt_i);	/* Load the default trees */
 
     curframe = 0;
@@ -167,11 +172,9 @@ old_way(FILE *fp)
 
 
 /**
- * C M _ S T A R T
- *
  * Process "start" command in new format input stream
  */
-int cm_start(int argc, char **argv)
+int cm_start(const int argc, const char **argv)
 {
     char *buf = (char *)NULL;
     int frame;
@@ -210,7 +213,7 @@ int cm_start(int argc, char **argv)
 }
 
 
-int cm_vsize(int argc, char **argv)
+int cm_vsize(const int argc, const char **argv)
 {
     if (argc < 2)
 	return 1;
@@ -220,20 +223,20 @@ int cm_vsize(int argc, char **argv)
 }
 
 
-int cm_eyept(int argc, char **argv)
+int cm_eyept(const int argc, const char **argv)
 {
     register int i;
 
     if (argc < 2)
 	return 1;
 
-    for (i=0; i<3; i++)
+    for (i = 0; i < 3; i++)
 	eye_model[i] = atof(argv[i+1]);
     return 0;
 }
 
 
-int cm_lookat_pt(int argc, char **argv)
+int cm_lookat_pt(const int argc, const char **argv)
 {
     point_t pt;
     vect_t dir;
@@ -266,20 +269,20 @@ int cm_lookat_pt(int argc, char **argv)
 }
 
 
-int cm_vrot(int argc, char **argv)
+int cm_vrot(const int argc, const char **argv)
 {
     register int i;
 
     if (argc < 16)
 	return 1;
 
-    for (i=0; i<16; i++)
+    for (i = 0; i < 16; i++)
 	Viewrotscale[i] = atof(argv[i+1]);
     return 0;
 }
 
 
-int cm_orientation(int argc, char **argv)
+int cm_orientation(const int argc, const char **argv)
 {
     register int i;
     quat_t quat;
@@ -287,7 +290,7 @@ int cm_orientation(int argc, char **argv)
     if (argc < 4)
 	return 1;
 
-    for (i=0; i<4; i++)
+    for (i = 0; i < 4; i++)
 	quat[i] = atof(argv[i+1]);
     quat_quat2mat(Viewrotscale, quat);
     orientflag = 1;
@@ -295,7 +298,7 @@ int cm_orientation(int argc, char **argv)
 }
 
 
-int cm_end(int UNUSED(argc), char **UNUSED(argv))
+int cm_end(const int UNUSED(argc), const char **UNUSED(argv))
 {
     struct rt_i *rtip = APP.a_rt_i;
 
@@ -312,19 +315,70 @@ int cm_end(int UNUSED(argc), char **UNUSED(argv))
 }
 
 
-int cm_tree(int argc, const char **argv)
+
+
+int cm_draw(const int argc, const char **argv)
+{
+    int i = 0;
+    size_t j = 0;
+    if (!cmd_objs || argc < 2) {
+	return 0;
+    }
+    for (i = 1; i < argc; i++) {
+	int is_drawn = 0;
+	for (j = 0; j < BU_PTBL_LEN(cmd_objs); j++) {
+	    const char *dobj = (const char *)BU_PTBL_GET(cmd_objs, j);
+	    if (BU_STR_EQUAL(dobj, argv[1])) {
+		is_drawn = 1;
+		break;
+	    }
+	}
+	if (!is_drawn) {
+	    const char *ndraw = bu_strdup(argv[1]);
+	    bu_ptbl_ins(cmd_objs, (long *)ndraw);
+	}
+    }
+    return 0;
+}
+
+int cm_erase(const int argc, const char **argv)
+{
+    int i = 0;
+    size_t j = 0;
+    if (!cmd_objs) {
+	return 0;
+    }
+    for (i = 1; i < argc; i++) {
+	for (j = 0; j < BU_PTBL_LEN(cmd_objs); j++) {
+	    char *dobj = (char *)BU_PTBL_GET(cmd_objs, j);
+	    if (BU_STR_EQUAL(dobj, argv[1])) {
+		bu_ptbl_rm(cmd_objs, (long *)dobj);
+		bu_free(dobj, "free name");
+	    }
+	}
+    }
+    return 0;
+}
+
+
+int cm_prep(const int UNUSED(argc), const char **UNUSED(argv))
 {
     register struct rt_i *rtip = APP.a_rt_i;
     struct bu_vls times = BU_VLS_INIT_ZERO;
-
-    if (argc <= 1) {
-	def_tree(rtip);		/* Load the default trees */
+    int objcnt = 0;
+    const char **objargv = NULL;
+    if (!cmd_objs) {
 	return 0;
     }
+    objcnt = (int)BU_PTBL_LEN(cmd_objs);
+    if (objcnt <= 0) {
+	return 0;
+    }
+    objargv = (const char **)cmd_objs->buffer;
 
     rt_prep_timer();
-    if (rt_gettrees(rtip, argc-1, &argv[1], npsw) < 0)
-	bu_log("rt_gettrees(%s) FAILED\n", argv[0]);
+    if (rt_gettrees(rtip, objcnt, objargv, npsw) < 0)
+	bu_log("rt_gettrees() FAILED\n");
     (void)rt_get_timer(&times, NULL);
 
     if (rt_verbosity & VERBOSE_STATS)
@@ -333,8 +387,29 @@ int cm_tree(int argc, const char **argv)
     return 0;
 }
 
+int cm_tree(const int argc, const char **argv)
+{
+    int i = 0;
+    size_t j = 0;
 
-int cm_multiview(int UNUSED(argc), char **UNUSED(argv))
+    if (argc <= 1) {
+	return 0;
+    }
+
+    for (j = 0; j < BU_PTBL_LEN(cmd_objs); i++) {
+	char *dobj = (char *)BU_PTBL_GET(cmd_objs, i);
+	bu_free(dobj, "free object name");
+    }
+    bu_ptbl_reset(cmd_objs);
+    for (i = 1; i < argc; i++) {
+	const char *ndraw = bu_strdup(argv[i]);
+	bu_ptbl_ins(cmd_objs, (long *)ndraw);
+    }
+
+    return cm_prep(0,NULL);
+}
+
+int cm_multiview(const int UNUSED(argc), const char **UNUSED(argv))
 {
     register struct rt_i *rtip = APP.a_rt_i;
     size_t i;
@@ -352,7 +427,7 @@ int cm_multiview(int UNUSED(argc), char **UNUSED(argv))
     if (rtip && BU_LIST_IS_EMPTY(&rtip->HeadRegion)) {
 	def_tree(rtip);		/* Load the default trees */
     }
-    for (i=0; i<(sizeof(a)/sizeof(a[0])); i++) {
+    for (i = 0; i < (sizeof(a)/sizeof(a[0])); i++) {
 	do_ae((double)a[i], (double)e[i]);
 	(void)do_frame(curframe++);
     }
@@ -361,13 +436,11 @@ int cm_multiview(int UNUSED(argc), char **UNUSED(argv))
 
 
 /**
- * C M _ A N I M
- *
  * Experimental animation code
  *
  * Usage: anim path type args
  */
-int cm_anim(int argc, const char **argv)
+int cm_anim(const int argc, const char **argv)
 {
 
     if (db_parse_anim(APP.a_rt_i->rti_dbip, argc, argv) < 0) {
@@ -379,37 +452,33 @@ int cm_anim(int argc, const char **argv)
 
 
 /**
- * C M _ C L E A N
- *
  * Clean out results of last rt_prep(), and start anew.
  */
-int cm_clean(int UNUSED(argc), char **UNUSED(argv))
+int cm_clean(const int UNUSED(argc), const char **UNUSED(argv))
 {
     /* Allow lighting model clean up (e.g. lights, materials, etc.) */
     view_cleanup(APP.a_rt_i);
 
     rt_clean(APP.a_rt_i);
 
-    if (R_DEBUG&RDEBUG_RTMEM_END)
+    if (OPTICAL_DEBUG&OPTICAL_DEBUG_RTMEM_END)
 	bu_prmem("After cm_clean");
     return 0;
 }
 
 
 /**
- * C M _ C L O S E D B
- *
  * To be invoked after a "clean" command, to close out the ".g"
  * database.  Intended for memory debugging, to help chase down memory
  * "leaks".  This terminates the program, as there is no longer a
  * database.
  */
-int cm_closedb(int UNUSED(argc), char **UNUSED(argv))
+int cm_closedb(const int UNUSED(argc), const char **UNUSED(argv))
 {
     db_close(APP.a_rt_i->rti_dbip);
     APP.a_rt_i->rti_dbip = DBI_NULL;
 
-    bu_free((genptr_t)APP.a_rt_i, "struct rt_i");
+    bu_free((void *)APP.a_rt_i, "struct rt_i");
     APP.a_rt_i = RTI_NULL;
 
     bu_prmem("After _closedb");
@@ -435,12 +504,10 @@ struct bu_structparse set_parse[] = {
      * not known at compile-time.  they would needed to be added to
      * set_parse() during runtime initialization.
      */
-    {"%d",	1, "rt_bot_mintie",		bu_byteoffset(rt_bot_mintie),		BU_STRUCTPARSE_FUNC_NULL, NULL, NULL },
     {"%d",	1, "rt_bot_minpieces",		bu_byteoffset(rt_bot_minpieces),	BU_STRUCTPARSE_FUNC_NULL, NULL, NULL },
     {"%d",	1, "rt_bot_tri_per_piece",	bu_byteoffset(rt_bot_tri_per_piece),	BU_STRUCTPARSE_FUNC_NULL, NULL, NULL },
     {"%f",	1, "rt_cline_radius",		bu_byteoffset(rt_cline_radius),		BU_STRUCTPARSE_FUNC_NULL, NULL, NULL },
 #endif
-    {"%V",	1, "ray_data_file",		bu_byteoffset(ray_data_file),		BU_STRUCTPARSE_FUNC_NULL, NULL, NULL },
     /* daisy-chain to additional app-specific parameters */
     {"%p",	1, "Application-Specific Parameters", bu_byteoffset(view_parse[0]),	BU_STRUCTPARSE_FUNC_NULL, NULL, NULL },
     {"",	0, (char *)0,		0,						BU_STRUCTPARSE_FUNC_NULL, NULL, NULL }
@@ -448,11 +515,9 @@ struct bu_structparse set_parse[] = {
 
 
 /**
- * C M _ S E T
- *
  * Allow variable values to be set or examined.
  */
-int cm_set(int argc, char **argv)
+int cm_set(const int argc, const char **argv)
 {
     struct bu_vls str = BU_VLS_INIT_ZERO;
 
@@ -463,7 +528,7 @@ int cm_set(int argc, char **argv)
     }
 
     bu_vls_from_argv(&str, argc-1, (const char **)argv+1);
-    if (bu_struct_parse(&str, set_parse, (char *)0) < 0) {
+    if (bu_struct_parse(&str, set_parse, (char *)0, NULL) < 0) {
 	bu_vls_free(&str);
 	return -1;
     }
@@ -472,10 +537,7 @@ int cm_set(int argc, char **argv)
 }
 
 
-/**
- * C M _ A E
- */
-int cm_ae(int argc, char **argv)
+int cm_ae(const int argc, const char **argv)
 {
     if (argc < 3)
 	return 1;
@@ -488,14 +550,11 @@ int cm_ae(int argc, char **argv)
 }
 
 
-/**
- * C M _ O P T
- */
-int cm_opt(int argc, char **argv)
+int cm_opt(const int argc, const char **argv)
 {
     int old_bu_optind=bu_optind;	/* need to restore this value after calling get_args() */
 
-    if (get_args(argc, (const char **)argv) <= 0) {
+    if (get_args(argc, argv) <= 0) {
 	bu_optind = old_bu_optind;
 	return -1;
     }
@@ -505,8 +564,6 @@ int cm_opt(int argc, char **argv)
 
 
 /**
- * D E F _ T R E E
- *
  * Load default tree list, from command line.
  */
 void
@@ -517,8 +574,8 @@ def_tree(register struct rt_i *rtip)
     RT_CK_RTI(rtip);
 
     rt_prep_timer();
-    if (rt_gettrees(rtip, nobjs, (const char **)objtab, npsw) < 0) {
-	bu_log("rt_gettrees(%s) FAILED\n", (objtab && objtab[0]) ? objtab[0] : "ERROR");
+    if (rt_gettrees(rtip, objc, (const char **)objv, npsw) < 0) {
+	bu_log("rt_gettrees(%s) FAILED\n", (objv && objv[0]) ? objv[0] : "ERROR");
     }
     (void)rt_get_timer(&times, NULL);
 
@@ -529,9 +586,148 @@ def_tree(register struct rt_i *rtip)
 }
 
 
+
+/*********************************************************************************/
+#ifdef USE_OPENCL
+/* from opt.c */
+extern double haze[3];
+extern double airdensity;
+
+
+static unsigned int clt_mode;           /* Active render buffers */
+static uint8_t clt_o[2];		/* Sub buffer offsets in bytes: {CLT_COLOR, MAX} */
+
+static struct fb *clt_fbp = FB_NULL;
+
+
+void
+clt_connect_fb(struct fb *fbp)
+{
+    clt_fbp = fbp;
+}
+
+void
+clt_view_init(unsigned int mode)
+{
+    uint8_t o[2];
+    int i;
+
+    clt_mode = mode;
+
+    o[0] = (mode & CLT_COLOR) ? 3 : 0;	/* uchar rgb[3] */
+    o[1] = 0;
+
+    clt_o[0] = 0;
+    for (i=1; i<2; i++) {
+	clt_o[i] = o[i-1] + clt_o[i-1];
+    }
+}
+
+void
+clt_run(int cur_pixel, int last_pixel)
+{
+    int ibackground[3];      /* integer 0..255 version */
+    int inonbackground[3];   /* integer non-background */
+    const double gamma_corr = 0.0;
+
+    int npix, a_x, a_y, i;
+    uint8_t *pixels, *pixelp;
+    size_t cpu = 0;
+    struct application a;
+
+    ssize_t size;
+    ssize_t count;
+
+    npix = last_pixel-cur_pixel+1;
+    size = npix * clt_o[1];
+
+    a_y = (int)(cur_pixel/width);
+    a_x = (int)(cur_pixel - (a_y * width));
+
+    /* Obtain fresh copy of global application struct */
+    a = APP;
+    a.a_resource = &resource[cpu];
+    a.a_level = 0;
+
+
+    if (lightmodel == 2)
+        VSETALL(background, 0);
+
+    /* Create integer version of background color */
+    inonbackground[0] = ibackground[0] = background[0] * 255.0 + 0.5;
+    inonbackground[1] = ibackground[1] = background[1] * 255.0 + 0.5;
+    inonbackground[2] = ibackground[2] = background[2] * 255.0 + 0.5;
+
+    /*
+     * If a non-background pixel comes out the same color as the
+     * background, modify it slightly, to permit compositing.  Perturb
+     * the background color channel with the largest intensity.
+     */
+    if (inonbackground[0] > inonbackground[1]) {
+        if (inonbackground[0] > inonbackground[2]) i = 0;
+        else i = 2;
+    } else {
+        if (inonbackground[1] > inonbackground[2]) i = 1;
+        else i = 2;
+    }
+    if (inonbackground[i] < 127) inonbackground[i]++;
+    else inonbackground[i]--;
+
+
+    pixels = (uint8_t*)bu_calloc(size, sizeof(uint8_t), "image buffer");
+
+    clt_frame(pixels, clt_o, cur_pixel, last_pixel, width,
+              ibackground, inonbackground,
+	      airdensity, haze, gamma_corr, view2model, cell_width,
+              cell_height, aspect, lightmodel, APP.a_no_booleans);
+
+    pixelp = pixels + cur_pixel*clt_o[1];
+
+    if (clt_fbp != FB_NULL) {
+        bu_semaphore_acquire(BU_SEM_SYSCALL);
+        count = fb_write(clt_fbp, a_x, a_y, pixelp, npix);
+        bu_semaphore_release(BU_SEM_SYSCALL);
+        if (count < npix)
+            bu_exit(EXIT_FAILURE, "pixel fb_write error");
+    }
+    if (outfp) {
+        bu_semaphore_acquire(BU_SEM_SYSCALL);
+        if (bu_fseek(outfp, cur_pixel*clt_o[1], 0) != 0)
+            fprintf(stderr, "fseek error\n");
+        if (fwrite(pixelp, size, 1, outfp) != 1)
+            bu_exit(EXIT_FAILURE, "pixel fwrite error");
+        bu_semaphore_release(BU_SEM_SYSCALL);
+    }
+    if (bif) {
+        int span = width*clt_o[1];
+
+        BU_ASSERT(a_x == 0);
+        while (pixelp < pixels+size) {
+            icv_writeline(bif, a_y++, pixelp, ICV_DATA_UCHAR);
+            pixelp += span;
+        }
+    }
+    bu_free(pixels, "image buffer");
+
+    bu_log("sub_grid_mode: %d, fullfloat_mode: %d, hypersample: %d, jitter: %d\n"
+	   "prism: %d, perspective: %e, stereo: %d\n",
+	   sub_grid_mode, fullfloat_mode, hypersample, jitter & JITTER_CELL,
+	   a.a_rt_i->rti_prismtrace, rt_perspective, stereo);
+
+    /* Tally up the statistics */
+    for (cpu=0; cpu < npsw; cpu++) {
+	if (resource[cpu].re_magic != RESOURCE_MAGIC) {
+	    bu_log("ERROR: CPU %d resources corrupted, statistics bad\n", cpu);
+	    continue;
+	}
+	rt_add_res_stats(APP.a_rt_i, &resource[cpu]);
+    }
+    bu_log("SHOT: opencl\n");
+}
+#endif
+
+
 /**
- * D O _ P R E P
- *
  * This is a separate function primarily as a service to REMRT.
  */
 void
@@ -552,13 +748,24 @@ do_prep(struct rt_i *rtip)
 	if (rt_verbosity & VERBOSE_STATS)
 	    bu_log("PREP: %s\n", bu_vls_addr(&times));
 	bu_vls_free(&times);
+
+#ifdef USE_OPENCL
+	if (opencl_mode) {
+	    rt_prep_timer();
+	    clt_prep(rtip);
+
+	    (void)rt_get_timer(&times, NULL);
+	    if (rt_verbosity & VERBOSE_STATS)
+		bu_log("OCLPREP: %s\n", bu_vls_addr(&times));
+	    bu_vls_free(&times);
+	}
+#endif
     }
     memory_summary();
     if (rt_verbosity & VERBOSE_STATS) {
-	bu_log("%s: %d nu, %d cut, %d box (%zu empty)\n",
-	       rtip->rti_space_partition == RT_PART_NUGRID ?
-	       "NUGrid" : "NUBSP",
-	       rtip->rti_ncut_by_type[CUT_NUGRIDNODE],
+	bu_log("%s: %d cut, %d box (%zu empty)\n",
+	       rtip->rti_space_partition == RT_PART_NUBSPT ?
+	       "NUBSP" : "unknown",
 	       rtip->rti_ncut_by_type[CUT_CUTNODE],
 	       rtip->rti_ncut_by_type[CUT_BOXNODE],
 	       rtip->nempty_cells);
@@ -567,8 +774,6 @@ do_prep(struct rt_i *rtip)
 
 
 /**
- * D O _ F R A M E
- *
  * Do all the actual work to run a frame.
  *
  * Returns -1 on error, 0 if OK.
@@ -582,7 +787,6 @@ do_frame(int framenumber)
     double utime = 0.0;			/* CPU time used */
     double nutime = 0.0;		/* CPU time used, normalized by ncpu */
     double wallclock = 0.0;		/* # seconds of wall clock time */
-    int npix = 0;			/* # of pixel values to be done */
     vect_t work, temp;
     quat_t quat;
 
@@ -597,9 +801,9 @@ do_frame(int framenumber)
 	bu_log("Tree: %zu solids in %zu regions\n", rtip->nsolids, rtip->nregions);
 
     if (Query_one_pixel) {
-	query_rdebug = R_DEBUG;
+	query_optical_debug = OPTICAL_DEBUG;
 	query_debug = RT_G_DEBUG;
-	RTG.debug = rdebug = 0;
+	rt_debug = optical_debug = 0;
     }
 
     if (rtip->nsolids <= 0)
@@ -665,7 +869,7 @@ do_frame(int framenumber)
     /* Process -b and ??? options now, for this frame */
     if (pix_start == -1) {
 	pix_start = 0;
-	pix_end = height * width - 1;
+	pix_end = (int)(height * width - 1);
     }
     if (string_pix_start) {
 	int xx, yy;
@@ -677,7 +881,7 @@ do_frame(int framenumber)
 	yy = atoi(cp);
 	bu_log("only pixel %d %d\n", xx, yy);
 	if (xx * yy >= 0) {
-	    pix_start = yy * width + xx;
+	    pix_start = (int)(yy * width + xx);
 	    pix_end = pix_start;
 	}
     }
@@ -691,7 +895,7 @@ do_frame(int framenumber)
 	yy = atoi(cp);
 	bu_log("ending pixel %d %d\n", xx, yy);
 	if (xx * yy >= 0) {
-	    pix_end = yy * width + xx;
+	    pix_end = (int)(yy * width + xx);
 	}
     }
 
@@ -700,17 +904,6 @@ do_frame(int framenumber)
 	pixmap = (unsigned char*)bu_calloc(sizeof(RGBpixel), width*height, "pixmap allocate");
     }
 
-    /*
-     * If this image is unlikely to be for debugging,
-     * be gentle to the machine.
-     */
-    if (!interactive) {
-	if (npix > 512*512)
-	    bu_nice_set(14);
-	else if (npix > 256*256)
-	    bu_nice_set(10);
-    }
-
     /*
      * Determine output file name
      * On UNIX only, check to see if this is a "restart".
@@ -726,7 +919,7 @@ do_frame(int framenumber)
 	/*
 	 * This code allows the computation of a particular frame to a
 	 * disk file to be resumed automatically.  This is worthwhile
-	 * crash protection.  This use of stat() and fseek() is
+	 * crash protection.  This use of stat() and bu_fseek() is
 	 * UNIX-specific.
 	 *
 	 * It is not appropriate for the RT "top part" to assume
@@ -785,6 +978,7 @@ do_frame(int framenumber)
 
 	/* Ordinary case for creating output file */
 	if (outfp == NULL) {
+#ifndef RT_TXT_OUTPUT
 	    /* FIXME: in the case of rtxray, this is wrong.  it writes
 	     * out a bw image so depth should be just 1, not 3.
 	     */
@@ -795,6 +989,14 @@ do_frame(int framenumber)
 		if (matflag) return 0;	/* OK */
 		return -1;			/* Bad */
 	    }
+#else
+	    outfp = fopen(framename, "w");
+	    if (outfp == NULL) {
+		perror(framename);
+		if (matflag) return 0;	/* OK */
+		return -1;			/* Bad */
+	    }
+#endif
 	}
 
 	if (rt_verbosity & VERBOSE_OUTPUTFILE)
@@ -805,9 +1007,16 @@ do_frame(int framenumber)
     /* initialize lighting, may update pix_start */
     view_2init(&APP, framename);
 
-    /* Just while doing the ray-tracing */
-    if (R_DEBUG&RDEBUG_RTMEM)
-	bu_debug |= (BU_DEBUG_MEM_CHECK|BU_DEBUG_MEM_LOG);
+#ifdef USE_OPENCL
+    if (opencl_mode) {
+        unsigned int mode = 0;
+
+                            mode |= CLT_COLOR;
+        if (full_incr_mode) mode |= CLT_ACCUM;
+
+        clt_view_init(mode);
+    }
+#endif
 
     rtip->nshots = 0;
     rtip->nmiss_model = 0;
@@ -828,6 +1037,16 @@ do_frame(int framenumber)
      */
     rt_prep_timer();
 
+#ifdef USE_OPENCL
+    if (opencl_mode) {
+	clt_run(pix_start, pix_end);
+
+	/* Reset values to full size, for next frame (if any) */
+	pix_start = 0;
+	pix_end = (int)(height*width - 1);
+    }
+    else
+#endif
     if (incr_mode) {
 	for (incr_level = 1; incr_level <= incr_nlevel; incr_level++) {
 	    if (incr_level > 1)
@@ -836,11 +1055,11 @@ do_frame(int framenumber)
 	    do_run(0, (1<<incr_level)*(1<<incr_level)-1);
 	}
     }
-    else if (full_incr_mode){
+    else if (full_incr_mode) {
 	/* Multiple frame buffer mode */
-	for(full_incr_sample = 1; full_incr_sample <= full_incr_nsamples;
-	    full_incr_sample++){
-	    if(full_incr_sample > 1) /* first sample was already initialized */
+	for (full_incr_sample = 1; full_incr_sample <= full_incr_nsamples;
+	    full_incr_sample++) {
+	    if (full_incr_sample > 1) /* first sample was already initialized */
 		view_2init(&APP, framename);
 	    do_run(pix_start, pix_end);
 	}
@@ -850,7 +1069,7 @@ do_frame(int framenumber)
 
 	/* Reset values to full size, for next frame (if any) */
 	pix_start = 0;
-	pix_end = height*width - 1;
+	pix_end = (int)(height*width - 1);
     }
     utime = rt_get_timer(&times, &wallclock);
 
@@ -860,17 +1079,13 @@ do_frame(int framenumber)
      */
     view_end(&APP);
 
-    /* Stop memory debug printing until next frame, leave full checking on */
-    if (R_DEBUG&RDEBUG_RTMEM)
-	bu_debug &= ~BU_DEBUG_MEM_LOG;
-
     /* These results need to be normalized.  Otherwise, all we would
      * know is that a given workload takes about the same amount of
      * CPU time, regardless of the number of CPUs.
      */
     if (npsw > 1) {
-	int avail_cpus;
-	int ncpus;
+	size_t avail_cpus;
+	size_t ncpus;
 
 	avail_cpus = bu_avail_cpus();
 	if (npsw > avail_cpus) {
@@ -903,7 +1118,7 @@ do_frame(int framenumber)
 	bu_log("%zu solid/ray intersections: %zu hits + %zu miss\n",
 	       rtip->nshots, rtip->nhits, rtip->nmiss);
 	bu_log("pruned %.1f%%:  %zu model RPP, %zu dups skipped, %zu solid RPP\n",
-	       rtip->nshots>0?((double)rtip->nhits*100.0)/rtip->nshots:100.0,
+	       rtip->nshots > 0 ? ((double)rtip->nhits*100.0)/rtip->nshots : 100.0,
 	       rtip->nmiss_model, rtip->ndup, rtip->nmiss_solid);
 	bu_log("Frame %2d: %10zu pixels in %9.2f sec = %12.2f pixels/sec\n",
 	       framenumber,
@@ -920,21 +1135,17 @@ do_frame(int framenumber)
 	       wallclock, ((double)(rtip->rti_nrays))/wallclock);
     }
     if (bif != NULL) {
-	icv_write(bif, framename, ICV_IMAGE_AUTO);
+	icv_write(bif, framename, BU_MIME_IMAGE_AUTO);
 	icv_destroy(bif);
 	bif = NULL;
     }
 
     if (outfp != NULL) {
-	/* Protect finished product */
-	if (outputfile != (char *)0)
-	    (void)bu_fchmod(fileno(outfp), 0444);
-
 	(void)fclose(outfp);
 	outfp = NULL;
     }
 
-    if (R_DEBUG&RDEBUG_STATS) {
+    if (OPTICAL_DEBUG&OPTICAL_DEBUG_STATS) {
 	/* Print additional statistics */
 	res_pr();
     }
@@ -947,8 +1158,6 @@ do_frame(int framenumber)
 
 
 /**
- * D O _ A E
- *
  * Compute the rotation specified by the azimuth and elevation
  * parameters.  First, note that these are specified relative to the
  * GIFT "front view", i.e., model (X, Y, Z) is view (Z, X, Y): looking
@@ -1034,19 +1243,16 @@ do_ae(double azim, double elev)
 }
 
 
-/**
- * R E S _ P R
- */
 void
 res_pr(void)
 {
     register struct resource *res;
-    register int i;
+    register size_t i;
 
     bu_log("\nResource use summary, by processor:\n");
     res = &resource[0];
-    for (i=0; i<npsw; i++, res++) {
-	bu_log("---CPU %d:\n", i);
+    for (i = 0; i < npsw; i++, res++) {
+	bu_log("---CPU %zu:\n", i);
 	if (res->re_magic != RESOURCE_MAGIC) {
 	    bu_log("Bad magic number!\n");
 	    continue;
@@ -1061,7 +1267,7 @@ res_pr(void)
 /**
  * Command table for RT control script language
  */
-struct command_tab rt_cmdtab[] = {
+struct command_tab rt_do_tab[] = {
     {"start", "frame number", "start a new frame",
      cm_start,	2, 2},
     {"viewsize", "size in mm", "set view size",
@@ -1082,6 +1288,12 @@ struct command_tab rt_cmdtab[] = {
      cm_anim,	4, 999},
     {"tree", 	"treetop(s)", "specify alternate list of tree tops",
      cm_tree,	1, 999},
+    {"draw", 	"obj", "add an object to the active list",
+     cm_draw,	2, 999},
+    {"erase", 	"obj", "remove an object from the active list",
+     cm_erase,	2, 999},
+    {"prep", 	"", "(re)prep for raytrace with the current obj list",
+     cm_prep,	1, 1},
     {"clean", "", "clean articulation from previous frame",
      cm_clean,	1, 1},
     {"_closedb", "", "Close .g database, (for memory debugging)",
