@@ -1,7 +1,7 @@
 /*                          V I E W . C
  * BRL-CAD
  *
- * Copyright (c) 1985-2013 United States Government as represented by
+ * Copyright (c) 1985-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This program is free software; you can redistribute it and/or
@@ -21,9 +21,11 @@
  *
  * Ray Tracing program, lighting model manager.
  *
- * Output is either interactive to a frame buffer, or written in a
- * file.  The output format is a .PIX file (a byte stream of R, G, B
- * as u_char's).
+ * Output is either interactively displayed in a frame buffer, written
+ * out to a file, or both.  The default output format is a .PIX file
+ * (a byte stream of R, G, B as u_char's), but will automatically
+ * write out other file formats based on the provided file extension
+ * (e.g., .PNG).
  *
  * The extern "lightmodel" selects which one is being used:
  * 0 Full lighting model (default)
@@ -49,11 +51,11 @@
 #include <math.h>
 
 #include "vmath.h"
-#include "mater.h"
 #include "icv.h"
 #include "raytrace.h"
-#include "fb.h"
-#include "plot3.h"
+#include "bu/cv.h"
+#include "dm.h"
+#include "bn/plot3.h"
 #include "photonmap.h"
 #include "scanline.h"
 
@@ -61,64 +63,49 @@
 #include "./ext.h"
 
 
-const char title[] = "The BRL-CAD Raytracer RT";
-
-void
-usage(const char *argv0)
-{
-    bu_log("Usage:  %s [options] model.g objects...\n", argv0);
-    bu_log("Options:\n");
-    bu_log(" -r		Report overlaps (default)\n");
-    bu_log(" -R		Do not report overlaps\n");
-    bu_log(" -M		Read matrix+commands on stdin\n");
-    bu_log(" -o model.pix	Output .pix format file (default is window)\n");
-    bu_log(" -s #		Square grid size in pixels (default is 512)\n");
-    bu_log(" -w # -n #	Grid size width (w) and height (n) in pixels\n");
-    bu_log(" -a # -e #	Azimuth (a) and elevation (e) in degrees\n");
-    bu_log(" -V #		View (pixel) aspect ratio (width/height)\n");
-    bu_log(" -p #		Perspective angle, degrees side to side\n");
-    bu_log(" -P #		Set number of processors\n");
-    bu_log(" -T #/#		Tolerance: distance/angular\n");
-    bu_log(" -l #		Set lighting model rendering style\n");
-    bu_log(" -U #		Use air if # is greater than 0\n");
-    bu_log(" -x #		librt debug flags\n");
-    bu_log(" -N #		NMG debug flags\n");
-    bu_log(" -X #		rt debug flags\n");
-}
-
-
-extern FBIO *fbp;			/* Framebuffer handle */
+extern struct fb *fbp;			/* Framebuffer handle */
 
 extern int curframe;		/* from main.c */
-extern fastf_t frame_delta_t;		/* from main.c */
-extern double airdensity;		/* from opt.c */
+extern double airdensity;	/* from opt.c */
 extern double haze[3];		/* from opt.c */
-extern int do_kut_plane;           /* from opt.c */
-extern plane_t kut_plane;              /* from opt.c */
-vect_t kut_norm;
-struct soltab *kut_soltab = NULL;
+extern int do_kut_plane;        /* from opt.c */
+extern plane_t kut_plane;       /* from opt.c */
 extern struct icv_image *bif;
-
 extern struct floatpixel *curr_float_frame;	/* buffer of full frame */
-
-int ambSlow = 0;
-int ambSamples = 0;
-double ambRadius = 0.0;
-double ambOffset = 0.0;
-vect_t ambient_color = { 1, 1, 1 };	/* Ambient white light */
-
-int ibackground[3] = {0};		/* integer 0..255 version */
-int inonbackground[3] = {0};	/* integer non-background */
+extern fastf_t** timeTable_init(int x, int y);  /* from heatgraph.c */
+extern void timeTable_process(fastf_t **timeTable, struct application *UNUSED(app), struct fb *efbp); /* from heatgraph.c */
+extern void free_scanlines(int, struct scanline *);
+extern struct scanline* alloc_scanlines(int);
 
 #ifdef RTSRV
-extern int srv_startpix;		/* offset for view_pixel */
+extern int srv_startpix;	/* offset for view_pixel */
 extern int srv_scanlen;		/* BUFMODE_RTSRV buffer length */
 #endif
 
-void free_scanlines(int, struct scanline *);
-struct scanline* alloc_scanlines(int);
-extern fastf_t** timeTable_init(int x, int y);
-extern void timeTable_process(fastf_t **timeTable, struct application *UNUSED(app), FBIO *efbp);
+
+const char title[] = "The BRL-CAD Raytracer RT";
+
+static struct scanline* scanline = NULL;
+static fastf_t* psum_buffer;            /* Buffer that keeps partial sums for multi-samples modes */
+static size_t pwidth = 0;		/* Width of each pixel (in bytes) */
+struct mfuncs *mfHead = MF_NULL;	/* Head of list of shaders */
+
+/**
+ * Overlay
+ *
+ * If in overlay mode, and writing to a framebuffer, only write
+ * non-background pixels.
+ */
+static int overlay = 0;
+
+/**
+ * Called when the reprojected value lies on the current screen.
+ * Write the reprojected value into the screen, checking *screen* Z
+ * values if the new location is already occupied.
+ *
+ * May be run in parallel.
+ */
+static int scr_lim_dist_sq = 100;	/* dist**2 pixels allowed to move */
 
 static int buf_mode=0;
 #define BUFMODE_UNBUF     1	/* No output buffering */
@@ -131,15 +118,18 @@ static int buf_mode=0;
 				   always have the average of the
 				   colors sampled for each pixel */
 
-static struct scanline* scanline;
-static fastf_t* psum_buffer;              /* Buffer that keeps partial sums for multi-samples modes */
-
-static size_t pwidth;		/* Width of each pixel (in bytes) */
-struct mfuncs *mfHead = MF_NULL;	/* Head of list of shaders */
+vect_t kut_norm = VINIT_ZERO;
+struct soltab *kut_soltab = NULL;
 
+int ambSlow = 0;
+int ambSamples = 0;
+double ambRadius = 0.0;
+double ambOffset = 0.0;
+vect_t ambient_color = { 1, 1, 1 };	/* Ambient white light */
+int ibackground[3] = {0};		/* integer 0..255 version */
+int inonbackground[3] = {0};		/* integer non-background */
 fastf_t gamma_corr = 0.0;		/* gamma correction if !0 */
 
-
 /**
  * The default a_onehit = -1 requires at least one non-air hit, (stop
  * at first surface) and stops ray/geometry intersection after that.
@@ -149,12 +139,9 @@ fastf_t gamma_corr = 0.0;		/* gamma correction if !0 */
 int a_onehit = -1;
 
 /**
- * Overlay
- *
- * If in overlay mode, and writing to a framebuffer, only write
- * non-background pixels.
+ * Set to 1 to turn off boolean evaluation with -c 'set a_no_booleans=1'
  */
-static int overlay = 0;
+int a_no_booleans = -1;
 
 /* Viewing module specific "set" variables:
  *
@@ -169,7 +156,8 @@ struct bu_structparse view_parse[] = {
     {"%d", 1, "bounces", 0, BU_STRUCTPARSE_FUNC_NULL, NULL, NULL},
     {"%d", 1, "ireflect", 0, BU_STRUCTPARSE_FUNC_NULL, NULL, NULL},
     {"%d", 1, "a_onehit", 0, BU_STRUCTPARSE_FUNC_NULL, NULL, NULL},
-    {"%f", ELEMENTS_PER_VECT, "background", 0, BU_STRUCTPARSE_FUNC_NULL, NULL, NULL},
+    {"%d", 1, "a_no_booleans", 0, BU_STRUCTPARSE_FUNC_NULL, NULL, NULL},
+    {"%f", ELEMENTS_PER_VECT, "background", 0, color_hook, NULL, NULL},
     {"%d", 1, "overlay", 0, BU_STRUCTPARSE_FUNC_NULL, NULL, NULL},
     {"%d", 1, "ov", 0, BU_STRUCTPARSE_FUNC_NULL, NULL, NULL},
     {"%d", 1, "ambSamples", 0, BU_STRUCTPARSE_FUNC_NULL, NULL, NULL},
@@ -181,8 +169,6 @@ struct bu_structparse view_parse[] = {
 
 
 /**
- * V I E W _ P I X E L
- *
  * Arrange to have the pixel output.  a_uptr has region pointer, for
  * reference.
  */
@@ -193,8 +179,8 @@ view_pixel(struct application *ap)
     unsigned char *pixelp;
     struct scanline *slp;
     int do_eol = 0;
-    unsigned char dist[8];	/* pixel distance (in IEEE format) */
 
+/* #define DRAW_INDICATOR_LINE 1 */
 #ifdef DRAW_INDICATOR_LINE
     /* this draws a nice indicator line to let you know where you are
      * currently rendering into the frame, but testing demonstrated
@@ -209,10 +195,6 @@ view_pixel(struct application *ap)
     bu_semaphore_release(BU_SEM_SYSCALL);
 #endif
 
-
-    if (rpt_dist)
-	bu_cv_htond(dist, (unsigned char *)&(ap->a_dist), 1);
-
     if (ap->a_user == 0) {
 	/* Shot missed the model, don't dither */
 	r = ibackground[0];
@@ -263,7 +245,7 @@ view_pixel(struct application *ap)
 	    b = 1;
     }
 
-    if (R_DEBUG&RDEBUG_HITS) bu_log("rgb=%3d, %3d, %3d xy=%3d, %3d (%g, %g, %g)\n",
+    if (OPTICAL_DEBUG&OPTICAL_DEBUG_HITS) bu_log("rgb=%3d, %3d, %3d xy=%3d, %3d (%g, %g, %g)\n",
 				    r, g, b, ap->a_x, ap->a_y,
 				    V3ARGS(ap->a_color));
 
@@ -321,26 +303,21 @@ view_pixel(struct application *ap)
 		p[2] = b;
 
 		if (bif != NULL) {
-		    bu_semaphore_acquire(BU_SEM_SYSCALL);
 		    icv_writepixel(bif, ap->a_x, ap->a_y, ap->a_color);
-		    bu_semaphore_release(BU_SEM_SYSCALL);
 		} else if (outfp != NULL) {
 		    bu_semaphore_acquire(BU_SEM_SYSCALL);
 		    if (bu_fseek(outfp, (ap->a_y*width*pwidth) + (ap->a_x*pwidth), 0) != 0)
 			fprintf(stderr, "fseek error\n");
 		    if (fwrite(p, 3, 1, outfp) != 1)
 			bu_exit(EXIT_FAILURE, "pixel fwrite error");
-		    if (rpt_dist &&
-			(fwrite(dist, 8, 1, outfp) != 1))
-			bu_exit(EXIT_FAILURE, "pixel fwrite error");
 		    bu_semaphore_release(BU_SEM_SYSCALL);
 		}
 
-		if (fbp != FBIO_NULL) {
+		if (fbp != FB_NULL) {
 		    /* Framebuffer output */
 		    bu_semaphore_acquire(BU_SEM_SYSCALL);
 		    npix = fb_write(fbp, ap->a_x, ap->a_y,
-				    (unsigned char *)p, 1);
+				    (const unsigned char *)p, 1);
 		    bu_semaphore_release(BU_SEM_SYSCALL);
 		    if (npix < 1)
 			bu_exit(EXIT_FAILURE, "pixel fb_write error");
@@ -356,16 +333,6 @@ view_pixel(struct application *ap)
 	    *pixelp++ = r;
 	    *pixelp++ = g;
 	    *pixelp++ = b;
-	    if (rpt_dist) {
-		*pixelp++ = dist[0];
-		*pixelp++ = dist[1];
-		*pixelp++ = dist[2];
-		*pixelp++ = dist[3];
-		*pixelp++ = dist[4];
-		*pixelp++ = dist[5];
-		*pixelp++ = dist[6];
-		*pixelp++ = dist[7];
-	    }
 	    bu_semaphore_release(RT_SEM_RESULTS);
 	    return;
 #endif
@@ -380,22 +347,12 @@ view_pixel(struct application *ap)
 	    slp = &scanline[ap->a_y];
 	    bu_semaphore_acquire(RT_SEM_RESULTS);
 	    if (slp->sl_buf == (unsigned char *)0) {
-		slp->sl_buf = bu_calloc(width, pwidth, "sl_buf scanline buffer");
+		slp->sl_buf = (unsigned char *)bu_calloc(width, pwidth, "sl_buf scanline buffer");
 	    }
 	    pixelp = slp->sl_buf+(ap->a_x*pwidth);
 	    *pixelp++ = r;
 	    *pixelp++ = g;
 	    *pixelp++ = b;
-	    if (rpt_dist) {
-		*pixelp++ = dist[0];
-		*pixelp++ = dist[1];
-		*pixelp++ = dist[2];
-		*pixelp++ = dist[3];
-		*pixelp++ = dist[4];
-		*pixelp++ = dist[5];
-		*pixelp++ = dist[6];
-		*pixelp++ = dist[7];
-	    }
 	    if (--(slp->sl_left) <= 0)
 		do_eol = 1;
 	    bu_semaphore_release(RT_SEM_RESULTS);
@@ -408,22 +365,12 @@ view_pixel(struct application *ap)
 	case BUFMODE_SCANLINE:
 	    slp = &scanline[ap->a_y];
 	    if (slp->sl_buf == (unsigned char *)0) {
-		slp->sl_buf = bu_calloc(width, pwidth, "sl_buf scanline buffer");
+		slp->sl_buf = (unsigned char *)bu_calloc(width, pwidth, "sl_buf scanline buffer");
 	    }
 	    pixelp = slp->sl_buf+(ap->a_x*pwidth);
 	    *pixelp++ = r;
 	    *pixelp++ = g;
 	    *pixelp++ = b;
-	    if (rpt_dist) {
-		*pixelp++ = dist[0];
-		*pixelp++ = dist[1];
-		*pixelp++ = dist[2];
-		*pixelp++ = dist[3];
-		*pixelp++ = dist[4];
-		*pixelp++ = dist[5];
-		*pixelp++ = dist[6];
-		*pixelp++ = dist[7];
-	    }
 	    if (--(slp->sl_left) <= 0)
 		do_eol = 1;
 	    break;
@@ -440,24 +387,14 @@ view_pixel(struct application *ap)
 		    if ((size_t)ap->a_y+dy >= height) break;
 		    slp = &scanline[ap->a_y+dy];
 		    if (slp->sl_buf == (unsigned char *)0)
-			slp->sl_buf = bu_calloc(width+32,
-						pwidth, "sl_buf scanline buffer");
+			slp->sl_buf = (unsigned char *)bu_calloc(width+32,
+								 pwidth, "sl_buf scanline buffer");
 
 		    pixelp = slp->sl_buf+(ap->a_x*pwidth);
 		    for (dx=0; dx<spread; dx++) {
 			*pixelp++ = r;
 			*pixelp++ = g;
 			*pixelp++ = b;
-			if (rpt_dist) {
-			    *pixelp++ = dist[0];
-			    *pixelp++ = dist[1];
-			    *pixelp++ = dist[2];
-			    *pixelp++ = dist[3];
-			    *pixelp++ = dist[4];
-			    *pixelp++ = dist[5];
-			    *pixelp++ = dist[6];
-			    *pixelp++ = dist[7];
-			}
 		    }
 		}
 		/* First 3 incremental iterations are boring */
@@ -479,17 +416,13 @@ view_pixel(struct application *ap)
 		/* Scanline buffered mode */
 		bu_semaphore_acquire(RT_SEM_RESULTS);
 
-		tmp_pixel = bu_calloc(pwidth, sizeof(fastf_t), "tmp_pixel");
+		tmp_pixel = (fastf_t *)bu_calloc(pwidth, sizeof(fastf_t), "tmp_pixel");
 		VMOVE(tmp_pixel, ap->a_color);
-		if (rpt_dist) {
-		    for (i = 0; i < 8; i++)
-			tmp_pixel[i + 3] = dist[i];
-		}
 
 		psum_p = &psum_buffer[ap->a_y*width*pwidth + ap->a_x*pwidth];
 		slp = &scanline[ap->a_y];
 		if (slp->sl_buf == (unsigned char *)0) {
-		    slp->sl_buf = bu_calloc(width, pwidth, "sl_buf scanline buffer");
+		    slp->sl_buf = (unsigned char *)bu_calloc(width, pwidth, "sl_buf scanline buffer");
 		}
 		pixelp = slp->sl_buf+(ap->a_x*pwidth);
 		/* Update the partial sums and the scanline */
@@ -523,7 +456,7 @@ view_pixel(struct application *ap)
 		long spread;
 		size_t npix = 0;
 
-		if (fbp == FBIO_NULL)
+		if (fbp == FB_NULL)
 		    bu_exit(EXIT_FAILURE, "Incremental rendering with no framebuffer?");
 
 		spread = (1<<(incr_nlevel-incr_level))-1;
@@ -552,7 +485,7 @@ view_pixel(struct application *ap)
 	case BUFMODE_ACC:
 	case BUFMODE_SCANLINE:
 	case BUFMODE_DYNAMIC:
-	    if (fbp != FBIO_NULL) {
+	    if (fbp != FB_NULL) {
 		size_t npix;
 		bu_semaphore_acquire(BU_SEM_SYSCALL);
 		if (sub_grid_mode) {
@@ -574,9 +507,7 @@ view_pixel(struct application *ap)
 	    }
 	    if (bif != NULL) {
 		/* TODO : Add double type data to maintain resolution */
-		bu_semaphore_acquire(BU_SEM_SYSCALL);
 		icv_writeline(bif, ap->a_y, (unsigned char *)scanline[ap->a_y].sl_buf, ICV_DATA_UCHAR);
-		bu_semaphore_release(BU_SEM_SYSCALL);
 	    } else if (outfp != NULL) {
 		size_t count;
 
@@ -596,8 +527,6 @@ view_pixel(struct application *ap)
 
 
 /**
- * V I E W _ E O L
- *
  * This routine is not used; view_pixel() determines when the last
  * pixel of a scanline is really done, for parallel considerations.
  */
@@ -609,27 +538,19 @@ view_eol(struct application *UNUSED(ap))
 
 
 /**
- * V I E W _ E N D
- *
  * Now when lightmodel is 8, heat-graph will be drawn.
  */
 void
 view_end(struct application *ap)
 {
-    /* FIXME: this should work on windows after the bu_timer() is
-     * created to replace the librt timing mechanism.
-     */
-#if !defined(_WIN32) || defined(__CYGWIN__)
     /* If the heat graph is on, render it after all pixels completed */
     if (lightmodel == 8) {
-
 	fastf_t **timeTable;
 	timeTable = timeTable_init(0, 0);
 	bu_log("Building Heat-Graph!\n");
 	bu_log("X:%d Y:%d W:%zu H%zu\n", ap->a_x, ap->a_y, width, height);
 	timeTable_process(timeTable, ap, fbp);
     }
-#endif
 
     if (fullfloat_mode) {
 	struct floatpixel *tmp;
@@ -657,8 +578,6 @@ view_end(struct application *ap)
 
 
 /**
- * V I E W _ S E T U P
- *
  * Called before rt_prep() in do.c
  */
 void
@@ -687,7 +606,7 @@ view_setup(struct rt_i *rtip)
 		{
 		    struct region *r = BU_LIST_NEXT(region, &regp->l);
 
-		    if (R_DEBUG&RDEBUG_MATERIAL)
+		    if (OPTICAL_DEBUG&OPTICAL_DEBUG_MATERIAL)
 			bu_log("mlib_setup: drop region %s\n", regp->reg_name);
 
 		    /* zap reg_udata? beware of light structs */
@@ -697,7 +616,7 @@ view_setup(struct rt_i *rtip)
 		}
 	    case 1:
 		/* Full success */
-		if (R_DEBUG&RDEBUG_MATERIAL &&
+		if (OPTICAL_DEBUG&OPTICAL_DEBUG_MATERIAL &&
 		    ((struct mfuncs *)(regp->reg_mfuncs))->mf_print) {
 		    ((struct mfuncs *)(regp->reg_mfuncs))->
 			mf_print(regp, regp->reg_udata);
@@ -716,8 +635,6 @@ view_setup(struct rt_i *rtip)
 
 
 /**
- * V I E W _ R E _ S E T U P
- *
  * This routine is used to do a "mlib_setup" on reprepped regions.
  * only regions with a NULL reg_mfuncs pointer will be processed.
  */
@@ -752,11 +669,10 @@ view_re_setup(struct rt_i *rtip)
 
 
 /**
- * V I E W _ C L E A N U P
- *
  * Called before rt_clean() in do.c
  */
-void view_cleanup(struct rt_i *rtip)
+void
+view_cleanup(struct rt_i *rtip)
 {
     struct region *regp;
 
@@ -775,15 +691,14 @@ void view_cleanup(struct rt_i *rtip)
 
 
 /**
- * H I T _ N O T H I N G
- *
  * a_miss() routine called when no part of the model is hit.
  * Background texture mapping could be done here.  For now, return a
  * pleasant dark blue.
  */
-static int hit_nothing(struct application *ap)
+static int
+hit_nothing(struct application *ap)
 {
-    if (R_DEBUG&RDEBUG_MISSPLOT) {
+    if (OPTICAL_DEBUG&OPTICAL_DEBUG_MISSPLOT) {
 	vect_t out;
 
 	/* XXX length should be 1 model diameter */
@@ -822,7 +737,7 @@ static int hit_nothing(struct application *ap)
 	VREVERSE(u.sw.sw_hit.hit_normal, ap->a_ray.r_dir);
 	/* U is azimuth, atan() range: -pi to +pi */
 	u.sw.sw_uv.uv_u = bn_atan2(ap->a_ray.r_dir[Y],
-				   ap->a_ray.r_dir[X]) * bn_inv2pi;
+				   ap->a_ray.r_dir[X]) * M_1_2PI;
 	if (u.sw.sw_uv.uv_u < 0)
 	    u.sw.sw_uv.uv_u += 1.0;
 	/*
@@ -832,20 +747,20 @@ static int hit_nothing(struct application *ap)
 	u.sw.sw_uv.uv_v = bn_atan2(ap->a_ray.r_dir[Z],
 				   sqrt(ap->a_ray.r_dir[X] * ap->a_ray.r_dir[X] +
 					ap->a_ray.r_dir[Y] * ap->a_ray.r_dir[Y])) *
-	    bn_invpi + 0.5;
+	    M_1_PI + 0.5;
 	u.sw.sw_uv.uv_du = u.sw.sw_uv.uv_dv = 0;
 
 	VSETALL(u.sw.sw_color, 1);
 	VSETALL(u.sw.sw_basecolor, 1);
 
-	if (R_DEBUG&RDEBUG_SHADE)
+	if (OPTICAL_DEBUG&OPTICAL_DEBUG_SHADE)
 	    bu_log("hit_nothing calling viewshade\n");
 
 	(void)viewshade(ap, &u.part, &u.sw);
 
 	VMOVE(ap->a_color, u.sw.sw_color);
 	ap->a_user = 1;		/* Signal view_pixel:  HIT */
-	ap->a_uptr = (genptr_t)&env_region;
+	ap->a_uptr = (void *)&env_region;
 	return 1;
     }
 
@@ -855,8 +770,7 @@ static int hit_nothing(struct application *ap)
 }
 
 
-/* A O _ R A Y H I T
- *
+/*
  * hit routine for ambient occlusion
  */
 int
@@ -898,8 +812,7 @@ ao_rayhit(register struct application *ap,
 }
 
 
-/* A O _ R A Y H I T
- *
+/*
  * miss routine for ambient occlusion
  */
 int
@@ -911,8 +824,7 @@ ao_raymiss(register struct application *ap)
 }
 
 
-/* a m b i e n t O c c l u s i o n
- *
+/*
  * Compute the ambient term using occlusion rays.
  * Scale the color based upon the occlusion
  */
@@ -927,7 +839,7 @@ ambientOcclusion(struct application *ap, struct partition *pp)
     vect_t uAxis;
     int ao_samp;
     vect_t origin = VINIT_ZERO;
-    float occlusionFactor;
+    double occlusionFactor;
     int hitCount = 0;
 
     stp = pp->pt_inseg->seg_stp;
@@ -1012,8 +924,6 @@ ambientOcclusion(struct application *ap, struct partition *pp)
 
 
 /**
- * C O L O R V I E W
- *
  * Manage the coloring of whatever it was we just hit.  This can be a
  * recursive procedure.
  */
@@ -1034,7 +944,7 @@ colorview(struct application *ap, struct partition *PartHeadp, struct seg *finis
 	 * sliver less than 0.05mm thick will be skipped (0.05 is a
 	 * SWAG).
 	 */
-	if ((genptr_t)pp->pt_regionp == ap->a_uptr &&
+	if ((void *)pp->pt_regionp == ap->a_uptr &&
 	    pp->pt_forw != PartHeadp &&
 	    pp->pt_outhit->hit_dist - pp->pt_inhit->hit_dist < 0.05)
 	    pp = pp->pt_forw;
@@ -1051,7 +961,7 @@ colorview(struct application *ap, struct partition *PartHeadp, struct seg *finis
     if (do_kut_plane) {
 	fastf_t slant_factor;
 	fastf_t dist;
-	fastf_t norm_dist = DIST_PT_PLANE(ap->a_ray.r_pt, kut_plane);
+	fastf_t norm_dist = DIST_PNT_PLANE(ap->a_ray.r_pt, kut_plane);
 
 	if ((slant_factor = -VDOT(kut_plane, ap->a_ray.r_dir)) < -1.0e-10) {
 	    /* exit point, ignore everything before "dist" */
@@ -1096,13 +1006,13 @@ colorview(struct application *ap, struct partition *PartHeadp, struct seg *finis
     hitp = pp->pt_inhit;
     RT_CK_HIT(hitp);
     RT_CK_RAY(hitp->hit_rayp);
-    ap->a_uptr = (genptr_t)pp->pt_regionp;	/* note which region was shaded */
+    ap->a_uptr = (void *)pp->pt_regionp;	/* note which region was shaded */
 
-    if (R_DEBUG&RDEBUG_HITS) {
+    if (OPTICAL_DEBUG&OPTICAL_DEBUG_HITS) {
 	bu_log("colorview: lvl=%d coloring %s\n",
 	       ap->a_level,
 	       pp->pt_regionp->reg_name);
-	rt_pr_pt(ap->a_rt_i, pp);
+	rt_pr_partition(ap->a_rt_i, pp);
     }
     if (hitp->hit_dist >= INFINITY) {
 	bu_log("colorview:  entry beyond infinity\n");
@@ -1127,7 +1037,7 @@ colorview(struct application *ap, struct partition *PartHeadp, struct seg *finis
 
 	if (pp->pt_outhit->hit_dist >= INFINITY ||
 	    ap->a_level > max_bounces) {
-	    if (R_DEBUG&RDEBUG_SHOWERR) {
+	    if (OPTICAL_DEBUG&OPTICAL_DEBUG_SHOWERR) {
 		VSET(ap->a_color, 9, 0, 0);	/* RED */
 		bu_log("colorview:  eye inside %s (x=%d, y=%d, lvl=%d)\n",
 		       pp->pt_regionp->reg_name,
@@ -1160,7 +1070,7 @@ colorview(struct application *ap, struct partition *PartHeadp, struct seg *finis
     }
 
     /* Record the approach path */
-    if (R_DEBUG&RDEBUG_RAYWRITE && (hitp->hit_dist > 0.0001)) {
+    if (OPTICAL_DEBUG&OPTICAL_DEBUG_RAYWRITE && (hitp->hit_dist > 0.0001)) {
 	VJOIN1(hitp->hit_point, ap->a_ray.r_pt,
 	       hitp->hit_dist, ap->a_ray.r_dir);
 	wraypts(ap->a_ray.r_pt,
@@ -1169,7 +1079,7 @@ colorview(struct application *ap, struct partition *PartHeadp, struct seg *finis
 		-1, ap, stdout);	/* -1 = air */
     }
 
-    if ((R_DEBUG&(RDEBUG_RAYPLOT|RDEBUG_RAYWRITE|RDEBUG_REFRACT)) && (hitp->hit_dist > 0.0001)) {
+    if ((OPTICAL_DEBUG&(OPTICAL_DEBUG_RAYPLOT|OPTICAL_DEBUG_RAYWRITE|OPTICAL_DEBUG_REFRACT)) && (hitp->hit_dist > 0.0001)) {
 	/* There are two parts to plot here.  Ray start to inhit
 	 * (purple), and inhit to outhit (grey).
 	 */
@@ -1184,7 +1094,7 @@ colorview(struct application *ap, struct partition *PartHeadp, struct seg *finis
 
 	VJOIN1(inhit, ap->a_ray.r_pt,
 	       hitp->hit_dist, ap->a_ray.r_dir);
-	if (R_DEBUG&RDEBUG_RAYPLOT) {
+	if (OPTICAL_DEBUG&OPTICAL_DEBUG_RAYPLOT) {
 	    bu_semaphore_acquire(BU_SEM_SYSCALL);
 	    pl_color(stdout, i, 0, i);
 	    pdv_3line(stdout, ap->a_ray.r_pt, inhit);
@@ -1201,7 +1111,7 @@ vdraw open oray;vdraw params c %2.2x%2.2x%2.2x;vdraw write n 0 %g %g %g;vdraw wr
 	VJOIN1(outhit,
 	       ap->a_ray.r_pt, out,
 	       ap->a_ray.r_dir);
-	if (R_DEBUG&RDEBUG_RAYPLOT) {
+	if (OPTICAL_DEBUG&OPTICAL_DEBUG_RAYPLOT) {
 	    bu_semaphore_acquire(BU_SEM_SYSCALL);
 	    pl_color(stdout, i, i, i);
 	    pdv_3line(stdout, inhit, outhit);
@@ -1221,12 +1131,11 @@ vdraw open iray;vdraw params c %2.2x%2.2x%2.2x;vdraw write n 0 %g %g %g;vdraw wr
     sw.sw_xmitonly = 0;		/* want full data */
     sw.sw_inputs = 0;		/* no fields filled yet */
     sw.sw_frame = curframe;
-    sw.sw_pixeltime = sw.sw_frametime = curframe * frame_delta_t;
     sw.sw_segs = finished_segs;
     VSETALL(sw.sw_color, 1);
     VSETALL(sw.sw_basecolor, 1);
 
-    if (R_DEBUG&RDEBUG_SHADE)
+    if (OPTICAL_DEBUG&OPTICAL_DEBUG_SHADE)
 	bu_log("colorview calling viewshade\n");
 
     /* individual shaders must handle reflection & refraction */
@@ -1255,7 +1164,7 @@ out:
 	ambientOcclusion(ap, pp);
 
     RT_CK_REGION(ap->a_uptr);
-    if (R_DEBUG&RDEBUG_HITS) {
+    if (OPTICAL_DEBUG&OPTICAL_DEBUG_HITS) {
 	bu_log("colorview: lvl=%d ret a_user=%d %s\n",
 	       ap->a_level,
 	       ap->a_user,
@@ -1267,13 +1176,10 @@ out:
 
 
 /**
- * V I E W I T
- *
  * a_hit() routine for simple lighting model.
  */
-int viewit(struct application *ap,
-	   struct partition *PartHeadp,
-	   struct seg *UNUSED(segHeadp))
+int
+viewit(struct application *ap, struct partition *PartHeadp, struct seg *UNUSED(segHeadp))
 {
     struct partition *pp;
     struct hit *hitp;
@@ -1293,7 +1199,7 @@ int viewit(struct application *ap,
     if (do_kut_plane) {
 	fastf_t slant_factor;
 	fastf_t dist;
-	fastf_t norm_dist = DIST_PT_PLANE(ap->a_ray.r_pt, kut_plane);
+	fastf_t norm_dist = DIST_PNT_PLANE(ap->a_ray.r_pt, kut_plane);
 
 	if ((slant_factor = -VDOT(kut_plane, ap->a_ray.r_dir)) < -1.0e-10) {
 	    /* exit point, ignore everything before "dist" */
@@ -1423,7 +1329,7 @@ int viewit(struct application *ap,
 
     }
 
-    if (R_DEBUG&RDEBUG_HITS) {
+    if (OPTICAL_DEBUG&OPTICAL_DEBUG_HITS) {
 	rt_pr_hit(" In", hitp);
 	bu_log("cosI0=%f, diffuse0=%f   ", cosI0, diffuse0);
 	VPRINT("RGB", ap->a_color);
@@ -1441,8 +1347,6 @@ kut_ft_norm(struct hit *hitp, struct soltab *UNUSED(stp), struct xray *UNUSED(ra
 
 
 /**
- * V I E W _ I N I T
- *
  * Called once, early on in RT setup, before view size is set.
  */
 int
@@ -1453,11 +1357,6 @@ view_init(struct application *UNUSED(ap), char *UNUSED(file), char *UNUSED(obj),
 
     optical_shader_init(&mfHead);	/* in liboptical/init.c */
 
-    if (rpt_dist && !minus_o) {
-	bu_log("Warning: -d ignored.  Writing to frame buffer only\n");
-	rpt_dist = 0;
-    }
-
     if (do_kut_plane) {
 	struct rt_functab *functab;
 	struct directory *dp;
@@ -1484,18 +1383,6 @@ view_init(struct application *UNUSED(ap), char *UNUSED(file), char *UNUSED(obj),
 }
 
 
-/**
- * R E P R O J E C T _ S P L A T
- *
- * Called when the reprojected value lies on the current screen.
- * Write the reprojected value into the screen, checking *screen* Z
- * values if the new location is already occupied.
- *
- * May be run in parallel.
- */
-int rt_scr_lim_dist_sq = 100;	/* dist**2 pixels allowed to move */
-
-
 int
 reproject_splat(int ix, int iy, struct floatpixel *ip, const fastf_t *new_view_pt)
 {
@@ -1529,11 +1416,8 @@ extern int per_processor_chunk;	/* how many pixels to do at once */
 extern int cur_pixel;		/* current pixel number, 0..last_pixel */
 extern int last_pixel;		/* last pixel number */
 
-/**
- * R E P R O J E C T _ W O R K E R
- */
 void
-reproject_worker(int UNUSED(cpu), genptr_t UNUSED(arg))
+reproject_worker(int UNUSED(cpu), void *UNUSED(arg))
 {
     int pixel_start;
     int pixelnum;
@@ -1580,7 +1464,7 @@ reproject_worker(int UNUSED(cpu), genptr_t UNUSED(arg))
 		/* Don't reproject if too pixel moved too far on the screen */
 		dx = ix - ip->ff_x;
 		dy = iy - ip->ff_y;
-		if (dx*dx + dy*dy > rt_scr_lim_dist_sq)
+		if (dx*dx + dy*dy > scr_lim_dist_sq)
 		    continue;	/* moved too far */
 
 				/* Don't reproject for too many frame-times */
@@ -1638,8 +1522,6 @@ collect_soltabs(struct bu_ptbl *stp_list, union tree *tr)
 
 
 /**
- * V I E W 2 _ I N I T
- *
  * Called each time a new image is about to be done.
  */
 void
@@ -1656,11 +1538,11 @@ view_2init(struct application *ap, char *UNUSED(framename))
     } else {
 	ap->a_onehit = a_onehit;
     }
+    if (a_no_booleans >= 0) {
+	ap->a_no_booleans = a_no_booleans;
+    }
 
-    if (rpt_dist)
-	pwidth = 3+8;
-    else
-	pwidth = 3;
+    pwidth = 3;
 
     /* Always allocate the scanline[] array (unless we already have
      * one in incremental mode)
@@ -1673,7 +1555,7 @@ view_2init(struct application *ap, char *UNUSED(framename))
     /* On fully incremental mode, allocate the scanline as the total
        size of the image */
     if (full_incr_mode && !psum_buffer)
-	psum_buffer = bu_calloc(height*width*pwidth, sizeof(fastf_t), "partial sums buffer");
+	psum_buffer = (fastf_t *)bu_calloc(height*width*pwidth, sizeof(fastf_t), "partial sums buffer");
 
 #ifdef RTSRV
     buf_mode = BUFMODE_RTSRV;		/* multi-pixel buffering */
@@ -1684,8 +1566,7 @@ view_2init(struct application *ap, char *UNUSED(framename))
 	buf_mode = BUFMODE_INCR;
     } else if (full_incr_mode) {
 	buf_mode = BUFMODE_ACC;
-    }
-    else if (width <= 96 || random_mode) {
+    } else if (width <= 96 || random_mode) {
 	buf_mode = BUFMODE_UNBUF;
     } else if ((size_t)npsw <= (size_t)height/4) {
 	/* Have each CPU do a whole scanline.  Saves lots of semaphore
@@ -1702,7 +1583,7 @@ view_2init(struct application *ap, char *UNUSED(framename))
 
     switch (buf_mode) {
 	case BUFMODE_UNBUF:
-	    bu_log("Single pixel I/O, unbuffered\n");
+	    bu_log("Mode: Single pixel I/O, unbuffered\n");
 	    break;
 	case BUFMODE_FULLFLOAT:
 	    if (!curr_float_frame) {
@@ -1740,8 +1621,7 @@ view_2init(struct application *ap, char *UNUSED(framename))
 	    break;
 #ifdef RTSRV
 	case BUFMODE_RTSRV:
-	    scanbuf = bu_malloc(srv_scanlen*pwidth + sizeof(long),
-				"scanbuf [multi-line]");
+	    scanbuf = (unsigned char *)bu_malloc(srv_scanlen*pwidth + sizeof(long), "scanbuf [multi-line]");
 	    break;
 #endif
 	case BUFMODE_INCR:
@@ -1772,11 +1652,11 @@ view_2init(struct application *ap, char *UNUSED(framename))
 	    break;
 
 	case BUFMODE_SCANLINE:
-	    bu_log("Low overhead scanline-per-CPU buffering\n");
+	    bu_log("Mode: scanline-per-CPU buffering\n");
 	    /* Fall through... */
 	case BUFMODE_DYNAMIC:
 	    if ((buf_mode == BUFMODE_DYNAMIC) && (rt_verbosity & VERBOSE_OUTPUTFILE)) {
-		bu_log("Dynamic scanline buffering\n");
+		bu_log("Mode: dynamic scanline buffering\n");
 	    }
 
 	    if (sub_grid_mode) {
@@ -1791,10 +1671,10 @@ view_2init(struct application *ap, char *UNUSED(framename))
 	case BUFMODE_ACC:
 	    for (i=0; i<height; i++)
 		scanline[i].sl_left = width;
-	    bu_log("Multiple-sample, average buffering\n");
+	    bu_log("Mode: Multiple-sample, average buffering\n");
 	    break;
 	default:
-	    bu_exit(EXIT_FAILURE, "bad buf_mode: %d", buf_mode);
+	    bu_exit(EXIT_FAILURE, "ERROR: bad buffering mode (%d), try -i", buf_mode);
     }
 
     /* This is where we do Preparations for each Lighting Model if it
@@ -1809,7 +1689,7 @@ view_2init(struct application *ap, char *UNUSED(framename))
 	     */
 	    if (BU_LIST_IS_EMPTY(&(LightHead.l))
 		|| !BU_LIST_IS_INITIALIZED(&(LightHead.l))) {
-		if (R_DEBUG&RDEBUG_SHOWERR)bu_log("No explicit light\n");
+		if (OPTICAL_DEBUG&OPTICAL_DEBUG_SHOWERR)bu_log("No explicit light\n");
 		light_maker(1, view2model);
 	    }
 	    break;
@@ -1833,7 +1713,7 @@ view_2init(struct application *ap, char *UNUSED(framename))
 		 * create one.
 		 */
 		if (BU_LIST_IS_EMPTY(&(LightHead.l)) || !BU_LIST_IS_INITIALIZED(&(LightHead.l))) {
-		    if (rdebug&RDEBUG_SHOWERR)
+		    if (optical_debug&OPTICAL_DEBUG_SHOWERR)
 			bu_log("No explicit light\n");
 		    light_maker(1, view2model);
 		}
@@ -1861,7 +1741,7 @@ view_2init(struct application *ap, char *UNUSED(framename))
 		ap->a_hit = colorview;
 		if (BU_LIST_IS_EMPTY(&(LightHead.l))
 		    || !BU_LIST_IS_INITIALIZED(&(LightHead.l))) {
-		    if (R_DEBUG&RDEBUG_SHOWERR)bu_log("No explicit light\n");
+		    if (OPTICAL_DEBUG&OPTICAL_DEBUG_SHOWERR)bu_log("No explicit light\n");
 		    light_maker(1, view2model);
 		}
 		break;
@@ -1878,7 +1758,7 @@ view_2init(struct application *ap, char *UNUSED(framename))
      * structures in the space partitioning tree
      */
     bu_ptbl_init(&stps, 8, "soltabs to delete");
-    if (R_DEBUG & RDEBUG_LIGHT)
+    if (OPTICAL_DEBUG & OPTICAL_DEBUG_LIGHT)
 	bu_log("deleting %lu invisible light regions\n", BU_PTBL_LEN(&ap->a_rt_i->delete_regs));
 
     for (i=0; i<BU_PTBL_LEN(&ap->a_rt_i->delete_regs); i++) {
@@ -1897,7 +1777,7 @@ view_2init(struct application *ap, char *UNUSED(framename))
 	/* remove the invisible light region pointers from the soltab
 	 * structs.
 	 */
-	if (R_DEBUG & RDEBUG_LIGHT)
+	if (OPTICAL_DEBUG & OPTICAL_DEBUG_LIGHT)
 	    bu_log("Removing invisible light region pointers from %lu soltabs\n",
 		   BU_PTBL_LEN(&stps));
 
@@ -1910,7 +1790,7 @@ view_2init(struct application *ap, char *UNUSED(framename))
 	    for (; k>=0; k--) {
 		rp2 = (struct region *)BU_PTBL_GET(&stp->st_regions, k);
 		if (rp2 == rp) {
-		    if (R_DEBUG & RDEBUG_LIGHT) {
+		    if (OPTICAL_DEBUG & OPTICAL_DEBUG_LIGHT) {
 			bu_log("\tRemoving region %s from soltab for %s\n", rp2->reg_name, stp->st_dp->d_namep);
 		    }
 		    bu_ptbl_rm(&stp->st_regions, (long *)rp2);
@@ -1947,28 +1827,36 @@ view_2init(struct application *ap, char *UNUSED(framename))
 
 
 /**
- * A P P L I C A T I O N _ I N I T
- *
  * Called once, very early on in RT setup, even before command line is
  * processed.
  */
 void
 application_init(void)
 {
-    /* rpt_overlap = 1; */
-
     /* Set the byte offsets at run time */
     view_parse[ 0].sp_offset = bu_byteoffset(gamma_corr);
     view_parse[ 1].sp_offset = bu_byteoffset(max_bounces);
     view_parse[ 2].sp_offset = bu_byteoffset(max_ireflect);
     view_parse[ 3].sp_offset = bu_byteoffset(a_onehit);
-    view_parse[ 4].sp_offset = bu_byteoffset(background[0]);
-    view_parse[ 5].sp_offset = bu_byteoffset(overlay);
+    view_parse[ 4].sp_offset = bu_byteoffset(a_no_booleans);
+    view_parse[ 5].sp_offset = bu_byteoffset(background[0]);
     view_parse[ 6].sp_offset = bu_byteoffset(overlay);
-    view_parse[ 7].sp_offset = bu_byteoffset(ambSamples);
-    view_parse[ 8].sp_offset = bu_byteoffset(ambRadius);
-    view_parse[ 9].sp_offset = bu_byteoffset(ambOffset);
-    view_parse[10].sp_offset = bu_byteoffset(ambSlow);
+    view_parse[ 7].sp_offset = bu_byteoffset(overlay);
+    view_parse[ 8].sp_offset = bu_byteoffset(ambSamples);
+    view_parse[ 9].sp_offset = bu_byteoffset(ambRadius);
+    view_parse[10].sp_offset = bu_byteoffset(ambOffset);
+    view_parse[11].sp_offset = bu_byteoffset(ambSlow);
+
+    option("", "-A #", "Set image brightness, ambient light intensity (default: 0.4)", 0);
+    option("Raytrace", "-i", "Enable incremental (progressive-style) rendering", 1);
+    option("Raytrace", "-t", "Render from top to bottom (default: from bottom up)", 1);
+    option("Advanced", "-O file.dpix", "Render to .dpix format file, double precision image data", 1);
+    option("Advanced", "-m density, r, g, b", "Render hazy air (e.g., 0.0002, 0.8, 0.9, 1 for sky-blue haze)", 1);
+    option("Developer", "-l #", "Select lighting model (default is 0)", 1);
+
+    /* this reassignment hack ensures help is last in the first list */
+    option("dummy", "-? or -h", "Display help", 1);
+    option("", "-? or -h", "Display help", 1);
 }
 
 
