@@ -1,7 +1,7 @@
 /*                        V I E W G 3 . C
  * BRL-CAD
  *
- * Copyright (c) 1989-2013 United States Government as represented by
+ * Copyright (c) 1989-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This program is free software; you can redistribute it and/or
@@ -46,9 +46,11 @@
 #include <math.h>
 #include <string.h>
 
+#include "bu/debug.h"
+#include "bu/parallel.h"
 #include "vmath.h"
 #include "raytrace.h"
-#include "plot3.h"
+#include "bn/plot3.h"
 
 #include "./rtuif.h"
 #include "./ext.h"
@@ -63,55 +65,29 @@ void part_compact(register struct application *ap, register struct partition *Pa
 extern fastf_t gift_grid_rounding;
 extern point_t viewbase_model;
 
-extern int npsw;			/* number of worker PSWs to run */
-
 extern int rpt_overlap;
 
-extern struct bu_vls ray_data_file;	/* file name for ray data output (declared in do.c) */
+struct bu_vls ray_data_file = BU_VLS_INIT_ZERO;  /* file name for ray data output */
 FILE *shot_fp;				/* FILE pointer for ray data output */
 static long line_num;			/* count of lines output to shotline file */
 
 /* Viewing module specific "set" variables */
 struct bu_structparse view_parse[] = {
+    {"%V",	1, "ray_data_file",		bu_byteoffset(ray_data_file),		BU_STRUCTPARSE_FUNC_NULL, NULL, NULL },
     {"",	0, (char *)0,	0,		BU_STRUCTPARSE_FUNC_NULL, NULL, NULL }
 };
 
 
 static mat_t model2hv;			/* model coords to GIFT h, v in inches */
-
 static FILE *plotfp;			/* optional plotting file */
-static long line_num;			/* count of lines output to shotline file */
 
 const char title[] = "RTG3";
 
-void
-usage(const char *argv0)
-{
-    bu_log("Usage:  %s [options] model.g objects... >file.ray\n", argv0);
-    bu_log("Options:\n");
-    bu_log(" -s #		Grid size in pixels, default 512\n");
-    bu_log(" -a Az		Azimuth in degrees	(conflicts with -M)\n");
-    bu_log(" -e Elev	Elevation in degrees	(conflicts with -M)\n");
-    bu_log(" -M		Read model2view matrix on stdin (conflicts with -a, -e)\n");
-    bu_log(" -g #		Grid cell width in millimeters (conflicts with -s)\n");
-    bu_log(" -G #		Grid cell height in millimeters (conflicts with -s)\n");
-    bu_log(" -J #		Jitter.  Default is off.  Any non-zero number is on\n");
-    bu_log(" -o model.g3	Specify output file, GIFT-3 format (default=stdout)\n");
-    bu_log(" -U #		Set use_air boolean to # (default=1)\n");
-    bu_log(" -c \"set ray_data_file=ray_file_name\"         Specify ray data output file (az el x_start y_start z_start x_dir y_dir z_dir line_number_in_shotline_file ray_first_hit_x ray_first_hit_y ray_first_hit_z)\n");
-    bu_log(" -c \"set save_overlaps=1\"     Reproduce FASTGEN behavior for regions flagged as FASTGEN regions\n");
-    bu_log(" -c \"set rt_cline_radius=radius\"      Additional radius to be added to CLINE solids\n");
-    bu_log(" -x #		Set librt debug flags\n");
-}
-
-
 int rayhit(register struct application *ap, struct partition *PartHeadp, struct seg *segp);
 int raymiss(register struct application *ap);
 
 
 /*
- * V I E W _ I N I T
- *
  * This routine is called by main().  It prints the overall shotline
  * header. Furthermore, pointers to rayhit() and raymiss() are set up
  * and are later called from do_run().
@@ -134,11 +110,12 @@ view_init(register struct application *ap, char *file, char *obj, int minus_o, i
     save_file = file;
     save_obj = obj;
 
-    if (ray_data_file.vls_magic == BU_VLS_MAGIC) {
-	if ((shot_fp=fopen(bu_vls_addr(&ray_data_file), "w")) == NULL) {
-	    perror("RTG3");
-	    bu_log("Cannot open ray data output file %s\n", bu_vls_addr(&ray_data_file));
-	    bu_exit(EXIT_FAILURE, "Cannot open ray data output file\n");
+    if (bu_vls_strlen(&ray_data_file)) {
+	shot_fp = fopen(bu_vls_addr(&ray_data_file), "w");
+	if (shot_fp == NULL) {
+	    perror("ERROR - fopen() failed");
+	    bu_log("Cannot open ray data output file [%s]\n", bu_vls_addr(&ray_data_file));
+	    bu_exit(EXIT_FAILURE, "See usage for help setting the ray_data_file variable.\n");
 	}
     }
 
@@ -162,7 +139,7 @@ view_init(register struct application *ap, char *file, char *obj, int minus_o, i
 
     output_is_binary = 0;		/* output is printable ascii */
 
-    if (R_DEBUG & RDEBUG_RAYPLOT) {
+    if (OPTICAL_DEBUG & OPTICAL_DEBUG_RAYPLOT) {
 	plotfp = fopen("rtg3.plot3", "w");
 	if (npsw > 1) {
 	    bu_log("Note: writing rtg3.plot3 file can only be done using only 1 processor\n");
@@ -175,8 +152,6 @@ view_init(register struct application *ap, char *file, char *obj, int minus_o, i
 
 
 /*
- * V I E W _ 2 I N I T
- *
  * View_2init is called by do_frame(), which in turn is called by
  * main() in rt.c.  It writes the view-specific COVART header.
  *
@@ -235,8 +210,6 @@ view_2init(struct application *UNUSED(ap), char *UNUSED(framename))
 
 
 /*
- * R A Y M I S S
- *
  * Null function -- handle a miss
  * This function is called by rt_shootray(), which is called by
  * do_frame().
@@ -249,8 +222,6 @@ raymiss(register struct application *UNUSED(ap))
 
 
 /*
- * V I E W _ P I X E L
- *
  * This routine is called from do_run(), and in this case does
  * nothing.
  */
@@ -262,8 +233,6 @@ view_pixel(struct application *UNUSED(ap))
 
 
 /*
- * R A Y H I T
- *
  * Rayhit() is called by rt_shootray() when the ray hits one or more
  * objects.  A per-shotline header record is written, followed by
  * information about each object hit.
@@ -409,6 +378,9 @@ rayhit(struct application *ap, register struct partition *PartHeadp, struct seg
 	bn_ae_vec(&azimuth, &elevation, ap->a_ray.r_dir);
     }
 
+    if (dfirst > 250000) dfirst = 250000;
+    if (dlast < -250000) dlast = -250000;
+
     bu_vls_printf(&str, SHOT_FMT,
 		  hvcen[0], hvcen[1],
 		  hv[0], hv[1],
@@ -428,7 +400,7 @@ rayhit(struct application *ap, register struct partition *PartHeadp, struct seg
      * The easy way to activate this is with the harmless -!1 option
      * when running RTG3.
      */
-    if (R_DEBUG || bu_debug || RT_G_DEBUG) {
+    if (OPTICAL_DEBUG || bu_debug || RT_G_DEBUG) {
 	bu_vls_printf(&str, "   -b%d, %d -p %26.20e %26.20e %26.20e -d %26.20e %26.20e %26.20e\n",
 		      ap->a_x, ap->a_y,
 		      V3ARGS(ap->a_ray.r_pt),
@@ -521,7 +493,7 @@ rayhit(struct application *ap, register struct partition *PartHeadp, struct seg
 		air_thickness = 0.0;
 	    if (!NEAR_ZERO(air_thickness, 0.1)) {
 		air_id = 1;	/* air gap */
-		if (R_DEBUG & RDEBUG_HITS)
+		if (OPTICAL_DEBUG & OPTICAL_DEBUG_HITS)
 		    bu_log("air gap added\n");
 	    } else {
 		air_thickness = 0.0;
@@ -551,7 +523,7 @@ rayhit(struct application *ap, register struct partition *PartHeadp, struct seg
 	    dot_prod = (-1.0);
 
 	in_obliq = acos(-dot_prod) *
-	    bn_radtodeg;
+	    RAD2DEG;
 	RT_HIT_NORMAL(normal, pp->pt_outhit, pp->pt_outseg->seg_stp, &(ap->a_ray), pp->pt_outflip);
 	dot_prod = VDOT(ap->a_ray.r_dir, normal);
 	if (dot_prod > 1.0)
@@ -560,7 +532,7 @@ rayhit(struct application *ap, register struct partition *PartHeadp, struct seg
 	    dot_prod = (-1.0);
 
 	out_obliq = acos(dot_prod) *
-	    bn_radtodeg;
+	    RAD2DEG;
 
 	/* Check for exit obliquities greater than 90 degrees. */
 
@@ -582,10 +554,14 @@ rayhit(struct application *ap, register struct partition *PartHeadp, struct seg
 	}
 	comp_thickness *= MM2IN;
 	/* Check thickness fields for format overflow */
-	if (comp_thickness > 999.99 || air_thickness*MM2IN > 999.9)
-	    fmt = "%4d%6.1f%5.1f%5.1f%1d%5.0f";
-	else
-	    fmt = "%4d%6.2f%5.1f%5.1f%1d%5.1f";
+
+	/* TODO - Are the old limit checks comp_thickness > 999.99
+	 * or air_thickness*MM2IN > 999.9 not valid?  This clamps their
+	 * values to well below those limits... */
+	if (comp_thickness > 99.99) comp_thickness = 99.99;
+	if (air_thickness*MM2IN > 999.9) air_thickness = 999.9;
+	fmt = "%5d%5.2f%5.1f%5.1f%1d%5.1f";
+
 #ifdef SPRINTF_NOT_PARALLEL
 	bu_semaphore_acquire(BU_SEM_SYSCALL);
 #endif
@@ -612,7 +588,7 @@ rayhit(struct application *ap, register struct partition *PartHeadp, struct seg
 	 * always be done single CPU, to prevent output garbling.
 	 * (See view_init).
 	 */
-	if (R_DEBUG & RDEBUG_RAYPLOT) {
+	if (OPTICAL_DEBUG & OPTICAL_DEBUG_RAYPLOT) {
 	    vect_t inpt;
 	    vect_t outpt;
 	    VJOIN1(inpt, ap->a_ray.r_pt, pp->pt_inhit->hit_dist,
@@ -681,8 +657,6 @@ rayhit(struct application *ap, register struct partition *PartHeadp, struct seg
 
 
 /*
- * V I E W _ E O L
- *
  * View_eol() is called by rt_shootray() in do_run().  In this case,
  * it does nothing.
  */
@@ -693,8 +667,6 @@ view_eol(struct application *UNUSED(ap))
 
 
 /*
- * V I E W _ E N D
- *
  * View_end() is called by rt_shootray in do_run().  It outputs a
  * special 999.9 "end of view" marker, composed of a "999.9" shotline
  * header, with one all-zero component record.  This is the way GIFT
@@ -725,8 +697,6 @@ void view_cleanup(struct rt_i *UNUSED(rtip)) {}
 
 
 /*
- * P A R T _ C O M P A C T
- *
  * This routine takes a partition-head pointer, an application
  * structure pointer, and a tolerance.  It goes through the partition
  * list shot-line by shot-line and checks for regions with identical
@@ -806,7 +776,27 @@ part_compact(register struct application *ap, register struct partition *PartHea
 }
 
 
-void application_init (void) {}
+void
+application_init (void)
+{
+    option("", "-o model.g3", "Specify output file, GIFT-3 format (default=stdout)", 0);
+    option("", "-U #", "Set use_air boolean to # (default=1)", 0);
+    option("Command (-c)", "\"set ray_data_file=ray_file_name\"", "", 0);
+    option("Command (-c)", "", "Specify ray data output file", 0);
+    option("Command (-c)", "", "(az el x_start y_start z_start x_dir y_dir z_dir line_number_in_shotline_file ray_first_hit_x ray_first_hit_y ray_first_hit_z)", 0);
+    option("Command (-c)", "\"set save_overlaps=1\"", "", 0);
+    option("Command (-c)", "", "Reproduce FASTGEN behavior for regions flagged", 0);
+    option("Command (-c)", "", "as FASTGEN regions", 0);
+    option("Command (-c)", "\"set rt_cline_radius=radius\"", "", 0);
+    option("Command (-c)", "", "Additional radius to be added to CLINE solids", 0);
+
+    /* this reassignment hack ensures help is last in the first list */
+    option("dummy", "-? or -h", "Display help", 1);
+    option("", "-? or -h", "Display help", 1);
+
+    option(NULL, "-C", "Disabled, not implemented", -2);
+    option(NULL, "-W", "Disabled, non implemented", -2);
+}
 
 /*
  * Local Variables:
