@@ -1,7 +1,7 @@
 /*                    V I E W W E I G H T . C
  * BRL-CAD
  *
- * Copyright (c) 1988-2013 United States Government as represented by
+ * Copyright (c) 1988-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This program is free software; you can redistribute it and/or
@@ -19,12 +19,12 @@
  */
 /** @file rt/viewweight.c
  *
- *  Ray Tracing program RTWEIGHT bottom half.
+ * Ray Tracing program RTWEIGHT bottom half.
  *
- *  This module outputs the weights and moments of a target model
- *  using density values located in ".density" or "$HOME/.density"
- *  Output is given in metric and english units, although input is
- *  assumed in lbs/cu.in.
+ * This module outputs the weights and moments of a target model
+ * using density values located in ".density" or "$HOME/.density"
+ * Output is given in metric and english units, although input is
+ * assumed in lbs/cu.in.
  *
  */
 
@@ -35,10 +35,17 @@
 #include <string.h>
 #include <time.h>
 
+#include "bu/app.h"
+#include "bu/parallel.h"
+#include "bu/str.h"
+#include "bu/sort.h"
+#include "bu/units.h"
+#include "bu/vls.h"
 #include "vmath.h"
 #include "raytrace.h"
+#include "analyze.h"
 
-#include "db.h"  /* FIXME: Yes, I know I shouldn't be peeking, put I
+#include "rt/db4.h"  /* FIXME: Yes, I know I shouldn't be peeking, put I
 		    am only looking to see what units we prefer... */
 
 #include "./rtuif.h"
@@ -52,71 +59,49 @@ struct bu_structparse view_parse[] = {
     {"",	0, (char *)0,	0,	BU_STRUCTPARSE_FUNC_NULL, NULL, NULL}
 };
 
+
 const char title[] = "RT Weight";
 
-void
-usage(const char *argv0)
-{
-    bu_log("Usage: %s [options] model.g objects...\n", argv0);
-    bu_log("Options:\n");
-    bu_log(" -s #		Grid size in pixels, default 512\n");
-    bu_log(" -g #		Grid cell width [and height] in mm\n");
-    bu_log(" -G #		Grid cell height [and width] in mm\n");
-    bu_log(" -a Az		Azimuth in degrees\n");
-    bu_log(" -e Elev	Elevation in degrees\n");
-    bu_log(" -o file.out	Weights and Moments output file\n");
-    bu_log(" -M		Read matrix, cmds on stdin\n");
-    bu_log(" -r 		Report verbosely mass of each region\n");
-    bu_log(" -x #		Set librt debug flags\n");
-    bu_log("Files:\n");
-    bu_log(" .density, OR\n");
-    bu_log(" $HOME/.density\n");
-}
+int noverlaps = 0;
+FILE *densityfp;
+struct bu_vls densityfile_vls = BU_VLS_INIT_ZERO;
+#define DENSITY_FILE ".density"
 
-
-int	noverlaps = 0;
-FILE	*densityfp;
-char	*densityfile;
-#define	DENSITY_FILE	".density"
-
-/* FIXME: use a bu_avs instead of a hard-coded limit so that materials
- * are looked up by the key in the density file, not assuming it's an
- * integer index into a fixed-size array.  gqa does it better by using
- * dynamic memory.
- */
-#define MAXMATLS	32768
-fastf_t	density[MAXMATLS];
-char	*dens_name[MAXMATLS];
+struct analyze_densities *density = NULL;
 
 struct datapoint {
     struct datapoint *next;
-    vect_t	centroid;
-    fastf_t	weight;
-    fastf_t	volume;
+    vect_t centroid;
+    fastf_t weight;
+    fastf_t volume;
 };
 
-extern int     	rpt_overlap;     	/* report region verbosely */
-extern fastf_t  cell_width;      	/* model space grid cell width */
-extern fastf_t  cell_height;     	/* model space grid cell height */
-extern FILE     *outfp;          	/* optional output file */
-extern char	*outputfile;     	/* name of base of output file */
-extern int	output_is_binary;	/* !0 means output is binary */
+
+extern int rpt_overlap;     	/* report region verbosely */
+extern fastf_t cell_width;      	/* model space grid cell width */
+extern fastf_t cell_height;     	/* model space grid cell height */
+extern FILE *outfp;          	/* optional output file */
+extern char *outputfile;     	/* name of base of output file */
+extern char *densityfile;     	/* name of density file */
+extern int output_is_binary;	/* !0 means output is binary */
 
 
-int
+static int
 hit(struct application *ap, struct partition *PartHeadp, struct seg *UNUSED(segp))
 {
     struct partition *pp;
     register struct xray *rp = &ap->a_ray;
-    genptr_t addp;
+    void *addp;
     int part_count = 0;
 
     for (pp = PartHeadp->pt_forw; pp != PartHeadp; pp = pp->pt_forw) {
-	register struct region	*reg = pp->pt_regionp;
-	register struct hit	*ihitp = pp->pt_inhit;
-	register struct hit	*ohitp = pp->pt_outhit;
-	register fastf_t	depth;
+	register struct region *reg = pp->pt_regionp;
+	register struct hit *ihitp = pp->pt_inhit;
+	register struct hit *ohitp = pp->pt_outhit;
+	register fastf_t depth;
 	register struct datapoint *dp;
+	long int material_id = reg->reg_gmater;
+	fastf_t density_factor = analyze_densities_density(density, material_id);
 
 	if (reg->reg_aircode)
 	    continue;
@@ -132,42 +117,59 @@ hit(struct application *ap, struct partition *PartHeadp, struct seg *UNUSED(segp
 	BU_ALLOC(dp, struct datapoint);
 	bu_semaphore_acquire(BU_SEM_SYSCALL);
 	addp = reg->reg_udata;
-	reg->reg_udata = (genptr_t)dp;
+	reg->reg_udata = (void *)dp;
 	dp->next = (struct datapoint *)addp;
 	bu_semaphore_release(BU_SEM_SYSCALL);
 
-	if (density[reg->reg_gmater] < 0) {
-	    bu_log("Material type %d used, but has no density file entry.\n", reg->reg_gmater);
-	    bu_log("  (region %s)\n", reg->reg_name);
-	    bu_semaphore_acquire(BU_SEM_SYSCALL);
-	    reg->reg_gmater = 0;
-	    bu_semaphore_release(BU_SEM_SYSCALL);
+	/* no density factor means we use a default (zero) */
+	{
+	    /* keep track of the first few only */
+	    #define MAX_MASS_TRACKED 256
+	    static int mass_undef[MAX_MASS_TRACKED] = {0};
+
+	    if (density_factor < 0) {
+		if (material_id > 0 && material_id < MAX_MASS_TRACKED && !mass_undef[material_id]) {
+		    bu_log("WARNING: Material ID %ld has no density file entry.\n"
+			   "         Mass is undefined, only reporting volume.\n"
+			   "       ( Encountered on region %s )\n\n", material_id, reg->reg_name);
+		    mass_undef[material_id] = 1;
+		}
+		density_factor = analyze_densities_density(density, 0);
+		bu_semaphore_acquire(BU_SEM_SYSCALL);
+		reg->reg_gmater = 0;
+		bu_semaphore_release(BU_SEM_SYSCALL);
+	    }
 	}
-	else if (density[reg->reg_gmater] >= 0) {
+
+	{
+	    /* precompute partition size */
+	    const fastf_t partition_volume = depth * cell_height * cell_width / (hypersample + 1);
+
+	    /* convert reg_los percentage to factor */
+	    const fastf_t los_factor = (fastf_t)reg->reg_los * 0.01;
+
+	    dp->volume = partition_volume;
 	    VBLEND2(dp->centroid, 0.5, ihitp->hit_point, 0.5, ohitp->hit_point);
 
-	    /* Compute mass in terms of grams */
-	    dp->weight = depth * density[reg->reg_gmater] *
-		(fastf_t)reg->reg_los *
-		cell_height * cell_height * 0.00001;
-	    dp->volume = depth * cell_height * cell_width;
+	    if (density_factor >= 0) {
+		/* Compute mass in terms of grams */
+		dp->weight = partition_volume * los_factor * density_factor;
+	    }
 	}
     }
     return 1;	/* report hit to main routine */
 }
 
 
-int
+static int
 miss(register struct application *UNUSED(ap))
 {
     return 0;
 }
 
 
-int
-overlap(struct application *UNUSED(ap), struct partition *UNUSED(pp),
-	struct region *UNUSED(reg1), struct region *UNUSED(reg2),
-	struct partition *UNUSED(hp))
+static int
+overlap(struct application *UNUSED(ap), struct partition *UNUSED(pp), struct region *UNUSED(reg1), struct region *UNUSED(reg2), struct partition *UNUSED(hp))
 {
     bu_semaphore_acquire(BU_SEM_SYSCALL);
     noverlaps++;
@@ -177,28 +179,15 @@ overlap(struct application *UNUSED(ap), struct partition *UNUSED(pp),
 
 
 /*
- *  			V I E W _ I N I T
- *
- *  Called at the start of a run.
- *  Returns 1 if framebuffer should be opened, else 0.
+ * Called at the start of a run.
+ * Returns 1 if framebuffer should be opened, else 0.
  */
 int
-view_init(struct application *ap, char *UNUSED(file), char *UNUSED(obj),
-	  int minus_o, int UNUSED(minus_F))
+view_init(struct application *ap, char *UNUSED(file), char *UNUSED(obj), int minus_o, int UNUSED(minus_F))
 {
-    register size_t i;
-    char buf[BUFSIZ+1];
-    char linebuf[BUFSIZ+1];
-    static char null = (char)0;
-    const char *curdir = getenv("PWD");
-    const char *homedir = getenv("HOME");
-    int line;
-
-    /* make sure they're not NULL */
-    if (!curdir)
-	curdir = "."; /* current dir */
-    if (!homedir)
-	homedir = ""; /* drop to root */
+    struct bu_vls pbuff_msgs = BU_VLS_INIT_ZERO;
+    struct bu_mapped_file *dfile = NULL;
+    char *dbuff = NULL;
 
     if (!minus_o) {
 	outfp = stdout;
@@ -208,64 +197,108 @@ view_init(struct application *ap, char *UNUSED(file), char *UNUSED(obj),
 	    outfp = fopen(outputfile, "w");
     }
 
-    for (i = 1; i < MAXMATLS; i++) {
-	density[i]   = -1;
-	dens_name[i] = &null;
+    if (outfp == NULL && outputfile != NULL && strlen(outputfile) > 0) {
+	bu_log("Unable to open output file \"%s\" for writing\n", outputfile);
+	goto view_init_rtweight_fail;
     }
 
-#define maxm(a, b) (a>b?a:b)
 
-    i = maxm(strlen(curdir), strlen(homedir)) + strlen(DENSITY_FILE) + 2;
     /* densityfile is global to this file and will be used later (and then freed) */
-    densityfile = bu_calloc((unsigned int)i, sizeof(char), "densityfile");
-
-    snprintf(densityfile, i, "%s/%s", curdir, DENSITY_FILE);
-
-    if ((densityfp = fopen(densityfile, "r")) == (FILE *)0) {
-	snprintf(densityfile, i, "%s/%s", homedir, DENSITY_FILE);
-	if ((densityfp = fopen(densityfile, "r")) == (FILE *)0) {
-	    bu_log("Unable to load density file \"%s\" for reading\n", densityfile);
-	    perror(densityfile);
-	    if (minus_o) {
-		fclose(outfp);
-	    }
-	    bu_exit(-1, NULL);
-	}
+    bu_vls_init(&densityfile_vls);
+    if (analyze_densities_create(&density)) {
+	bu_log("INTERNAL ERROR: Unable to initialize density table\n");
     }
 
-    /* Read in density in terms of grams/cm^3 */
-
-    /* need to use bu_fgets instead of fscanf because fscanf stops
-       scanning at first failure or error and we get an infinite loop */
-    line = 0;
-    while (bu_fgets(linebuf, BUFSIZ+1, densityfp)) {
-	const int cmt = '#';
-	int idx;
-	float dens;
-	char* c;
-
-	++line;
-
-	/* delete comments before processing */
-	if ((c = strchr(linebuf, cmt)) != NULL) {
-	    /* close linebuf with a newline and a null char */
-	    *c++ = '\n';
-	    *c   = '\0';
-	}
-	i = sscanf(linebuf, "%d %f %[^\n]", &idx, &dens, buf);
-	if (i != 3) {
-	    bu_log("error parsing line %d of density file.\n  %zu args recognized instead of 3\n",
-		   line, i);
-	    bu_log("  line buffer reads : %s\n", linebuf);
-	    continue;
+    if (densityfile) {
+	bu_vls_sprintf(&densityfile_vls, "%s", densityfile);
+	if (!bu_file_exists(bu_vls_cstr(&densityfile_vls), NULL)) {
+	    bu_log("Unable to load density file \"%s\" for reading\n", bu_vls_cstr(&densityfile_vls));
+	    goto view_init_rtweight_fail;
 	}
 
-	if (idx > 0 && idx < MAXMATLS ) {
-	    density[idx] = dens;
-	    dens_name[idx] = bu_strdup(buf);
+	dfile = bu_open_mapped_file(bu_vls_cstr(&densityfile_vls), "densities file");
+	if (!dfile) {
+	    bu_log("Unable to open density file \"%s\" for reading\n", bu_vls_cstr(&densityfile_vls));
+	    goto view_init_rtweight_fail;
+	}
+
+	dbuff = (char *)(dfile->buf);
+
+
+	/* Read in density */
+	if (analyze_densities_load(density, dbuff, &pbuff_msgs, NULL) ==  0) {
+	    bu_log("Unable to parse density file \"%s\":%s\n", bu_vls_cstr(&densityfile_vls), bu_vls_cstr(&pbuff_msgs));
+	    bu_close_mapped_file(dfile);
+	    goto view_init_rtweight_fail;
+	}
+	bu_close_mapped_file(dfile);
+
+
+    } else {
+
+	/* If we don't have a density file, first try the .g itself */
+	struct directory *dp;
+	dp = db_lookup(ap->a_rt_i->rti_dbip, "_DENSITIES", LOOKUP_QUIET);
+	if (dp != (struct directory *)NULL) {
+	    int ret, ecnt;
+	    char *buf;
+	    struct rt_db_internal intern;
+	    struct rt_binunif_internal *bip;
+	    struct bu_vls msgs = BU_VLS_INIT_ZERO;
+	    if (rt_db_get_internal(&intern, dp, ap->a_rt_i->rti_dbip, NULL, &rt_uniresource) < 0) {
+		bu_log("Could not import %s\n", dp->d_namep);
+		goto view_init_rtweight_fail;
+	    }
+	    if ((intern.idb_major_type & DB5_MAJORTYPE_BINARY_MASK) == 0)
+		goto view_init_rtweight_fail;
+
+	    bip = (struct rt_binunif_internal *)intern.idb_ptr;
+	    RT_CHECK_BINUNIF (bip);
+
+	    buf = (char *)bu_calloc(bip->count+1, sizeof(char), "density buffer");
+	    memcpy(buf, bip->u.int8, bip->count);
+	    rt_db_free_internal(&intern);
+
+	    ret = analyze_densities_load(density, buf, &msgs, &ecnt);
+
+	    bu_free((void *)buf, "density buffer");
+
+	    if (ret <= 0) {
+		bu_log("Problem reading densities from .g file:\n%s\n", bu_vls_cstr(&msgs));
+		bu_vls_free(&msgs);
+		goto view_init_rtweight_fail;
+	    }
+
+	    bu_vls_sprintf(&densityfile_vls, "%s", ap->a_rt_i->rti_dbip->dbi_filename);
+
 	} else {
-	    bu_log("Material index %d in '%s' is out of range.\n",
-		   idx, densityfile );
+
+	    /* If we still don't have density information, fall back on the pre-defined defaults */
+	    bu_vls_sprintf(&densityfile_vls, "%s%c%s", bu_dir(NULL, 0, BU_DIR_CURR, NULL), BU_DIR_SEPARATOR, DENSITY_FILE);
+
+	    if (!bu_file_exists(bu_vls_cstr(&densityfile_vls), NULL)) {
+		bu_vls_sprintf(&densityfile_vls, "%s%c%s", bu_dir(NULL, 0, BU_DIR_HOME, NULL), BU_DIR_SEPARATOR, DENSITY_FILE);
+		if (!bu_file_exists(bu_vls_cstr(&densityfile_vls), NULL)) {
+		    bu_log("Unable to load density file \"%s\" for reading\n", bu_vls_cstr(&densityfile_vls));
+		    goto view_init_rtweight_fail;
+		}
+	    }
+	    dfile = bu_open_mapped_file(bu_vls_cstr(&densityfile_vls), "densities file");
+	    if (!dfile) {
+		bu_log("Unable to open density file \"%s\" for reading\n", bu_vls_cstr(&densityfile_vls));
+		goto view_init_rtweight_fail;
+	    }
+
+	    dbuff = (char *)(dfile->buf);
+
+
+	    /* Read in density */
+	    if (analyze_densities_load(density, dbuff, &pbuff_msgs, NULL) ==  0) {
+		bu_log("Unable to parse density file \"%s\":%s\n", bu_vls_cstr(&densityfile_vls), bu_vls_cstr(&pbuff_msgs));
+		bu_close_mapped_file(dfile);
+		goto view_init_rtweight_fail;
+	    }
+	    bu_close_mapped_file(dfile);
 	}
     }
 
@@ -273,13 +306,26 @@ view_init(struct application *ap, char *UNUSED(file), char *UNUSED(obj),
     ap->a_miss = miss;
     ap->a_overlap = overlap;
     ap->a_onehit = 0;
+
+    bu_vls_free(&pbuff_msgs);
     if (minus_o) {
 	fclose(outfp);
     }
 
     return 0;		/* no framebuffer needed */
+
+view_init_rtweight_fail:
+    bu_vls_free(&densityfile_vls);
+    bu_vls_free(&pbuff_msgs);
+    analyze_densities_destroy(density);
+    if (minus_o && outfp) {
+	fclose(outfp);
+    }
+
+    bu_exit(-1, NULL);
 }
 
+
 /* beginning of a frame */
 void
 view_2init(struct application *ap, char *UNUSED(framename))
@@ -288,25 +334,28 @@ view_2init(struct application *ap, char *UNUSED(framename))
     register struct rt_i *rtip = ap->a_rt_i;
 
     for (BU_LIST_FOR(rp, region, &(rtip->HeadRegion))) {
-	rp->reg_udata = (genptr_t)NULL;
+	rp->reg_udata = (void *)NULL;
     }
 }
 
+
 /* end of each pixel */
 void
 view_pixel(struct application *UNUSED(ap))
 {
 }
 
+
 /* end of each line */
 void
 view_eol(struct application *UNUSED(ap))
 {
 }
 
-/* a region ID sort comparison for use with qsort on a region array */
-int region_ID_cmp(const void *p1,
-		  const void *p2)
+
+/* a region ID sort comparison for use with bu_sort on a region array */
+static int
+region_ID_cmp(const void *p1, const void *p2, void *UNUSED(arg))
 {
     /* cast into correct type--note the incoming pointer type is a
        pointer to a pointer which must be dereferenced! */
@@ -314,13 +363,14 @@ int region_ID_cmp(const void *p1,
     struct region *r2 = *(struct region **)p2;
 
     if (r1->reg_regionid < r2->reg_regionid)
-      return -1;
+	return -1;
     else if (r1->reg_regionid > r2->reg_regionid)
-      return +1;
+	return +1;
     else
-      return bu_strcmp(r1->reg_name, r2->reg_name);
+	return bu_strcmp(r1->reg_name, r2->reg_name);
 }
 
+
 /* end of a frame */
 void
 view_end(struct application *ap)
@@ -379,28 +429,34 @@ view_end(struct application *ap)
 	/* cm */
 	bu_strlcpy(units, "grams", sizeof(units));
     } else {
-	bu_log("Warning: base2mm=%g, using default of %s--%s\n",
+	bu_log("WARNING: base2mm=%g, using default of %s--%s\n",
 	       dbp->dbi_base2local, units, unit2);
     }
 
     if (noverlaps)
 	bu_log("%d overlap%c detected.\n\n", noverlaps,
-		noverlaps==1 ? '\0' : 's');
+	       noverlaps==1 ? '\0' : 's');
 
     fprintf(outfp, "RT Weight Program Output:\n");
     fprintf(outfp, "\nDatabase Title: \"%s\"\n", dbp->dbi_title);
-    fprintf(outfp, "Time Stamp: %s\n\nDensity Table Used:%s\n\n", timeptr, densityfile);
-    fprintf(outfp, "Material  Density(g/cm^3)  Name\n");
+    fprintf(outfp, "Time Stamp: %s\n\nDensity Table Used:%s\n\n", timeptr, bu_vls_cstr(&densityfile_vls));
+    fprintf(outfp,
+	    "Material  Density(g/cm^3)  Name\n"
+	    "--------  ---------------  -------------\n");
     {
-	register int i;
-	for (i = 1; i < MAXMATLS; i++) {
-	    if (density[i] >= 0)
-		fprintf(outfp, "%5d     %10.4f       %s\n",
-			i, density[i], dens_name[i]);
-      }
+	long int curr_id = -1;
+	while ((curr_id = analyze_densities_next(density, curr_id)) != -1) {
+	    char *cname = analyze_densities_name(density, curr_id);
+	    fastf_t cdensity = analyze_densities_density(density, curr_id);
+	    fprintf(outfp, "%5ld     %10.4f       %s\n", curr_id, cdensity*1000, cname);
+	    bu_free(cname, "free name copy");
+	}
     }
 
-    /* is this test really necessary? or can we return if !rpt_overlap? */
+    /* NOTE: rtweight repurposes the -r report overlap flag, used here
+     * as a report region information verbosely flag by default.  Thus
+     * -R will output just the summary.
+     */
     if (rpt_overlap) {
 	/* move block scope variables here */
 	fastf_t *item_wt;
@@ -421,9 +477,9 @@ view_end(struct application *ap)
 
 	    for (dp = (struct datapoint *)rp->reg_udata;
 		 dp != (struct datapoint *)NULL; dp = dp->next) {
-		sum_x  += dp->weight * dp->centroid[X];
-		sum_y  += dp->weight * dp->centroid[Y];
-		sum_z  += dp->weight * dp->centroid[Z];
+		sum_x += dp->weight * dp->centroid[X];
+		sum_y += dp->weight * dp->centroid[Y];
+		sum_z += dp->weight * dp->centroid[Z];
 		weight += dp->weight;
 		volume += dp->volume;
 	    }
@@ -435,7 +491,7 @@ view_end(struct application *ap)
 	    *ptr = weight;
 	    /* FIXME: shouldn't the existing reg_udata be bu_free'd first (see previous loop) */
 	    /* FIXME: isn't the region list a "shared resource"? if so, can we use reg_udata so cavalierly? */
-	    rp->reg_udata = (genptr_t)ptr;
+	    rp->reg_udata = (void *)ptr;
 	}
 
 	/* make room for a zero ID number and an "end ID " so we can
@@ -464,7 +520,7 @@ view_end(struct application *ap)
 	}
 
 	/* sort the region array by ID, then by name */
-	qsort(rp_array, nregions, sizeof(struct region *), region_ID_cmp);
+	bu_sort(rp_array, nregions, sizeof(struct region *), region_ID_cmp, NULL);
 
 	/* WEIGHT BY REGION NAME =============== */
 	/* ^L is char code for FormFeed/NewPage */
@@ -474,7 +530,6 @@ view_end(struct application *ap)
 
 	start_ridx = 0; /* region array is indexed from 0 */
 	for (id = 1; id < max_item; ++id) {
-	    const int flen = 37; /* desired size of name field */
 
 	    if (item_wt[id] < 0)
 		continue;
@@ -483,17 +538,19 @@ view_end(struct application *ap)
 	    for (ridx = start_ridx; ridx < nregions; ++ridx) {
 		struct region *r = rp_array[ridx];
 		if (r->reg_regionid == id) {
+		    char *cname = analyze_densities_name(density, r->reg_gmater);
+		    fastf_t cdensity = analyze_densities_density(density, r->reg_gmater);
 		    fastf_t weight = *(fastf_t *)r->reg_udata;
-		    register size_t len = strlen(r->reg_name);
-		    len = len > (size_t)flen ? len - (size_t)flen : 0;
-		    fprintf(outfp, "%8.3f %5d  %3d %-15.15s %7.4f %-*.*s\n",
+		    fprintf(outfp, "%8.3f %5d  %3d %-15.15s %7.4f %s\n",
 			    weight,
 			    r->reg_gmater, r->reg_los,
-			    dens_name[r->reg_gmater],
-			    density[r->reg_gmater],
-			    flen, flen, &r->reg_name[len]);
-		}
-		else if (r->reg_regionid > id) {
+			    cname,
+			    cdensity*1000,
+			    r->reg_name);
+		    if (cname) {
+			bu_free(cname, "free name copy");
+		    }
+		} else if (r->reg_regionid > id) {
 		    /* FIXME: an "else" alone should be good enough
 		       because the test should not be necessary if we
 		       trust the sorted array */
@@ -511,12 +568,11 @@ view_end(struct application *ap)
 
 	start_ridx = 0; /* region array is indexed from 0 */
 	for (id = 1; id < max_item; ++id) {
-	    const int flen = 65; /* desired size of name field */
 	    int CR = 0;
 	    const int ns = 15;
 
 	    if (item_wt[id] < 0)
-	       continue;
+		continue;
 
 	    /* the following format string has 15 spaces before the region name: */
 	    fprintf(outfp, "%5d %8.3f ", id, item_wt[id]);
@@ -525,16 +581,13 @@ view_end(struct application *ap)
 	    for (ridx = start_ridx; ridx < nregions; ++ridx) {
 		struct region *r = rp_array[ridx];
 		if (r->reg_regionid == id) {
-		    register size_t len = strlen(r->reg_name);
-		    len = len > (size_t)flen ? len - (size_t)flen : 0;
 		    if (CR) {
 			/* need leading spaces */
 			fprintf(outfp, "%*.*s", ns, ns, " ");
 		    }
-		    fprintf(outfp, "%-*.*s\n", flen, flen, &r->reg_name[len]);
+		    fprintf(outfp, "%s\n", r->reg_name);
 		    CR = 1;
-		}
-		else if (r->reg_regionid > id) {
+		} else if (r->reg_regionid > id) {
 		    /* FIXME: an "else" alone should be good enough
 		       because the test should not be necessary if we
 		       trust the sorted array */
@@ -552,9 +605,9 @@ view_end(struct application *ap)
     }
 
     volume *= (dbp->dbi_base2local*dbp->dbi_base2local*dbp->dbi_base2local);
-    sum_x  *= (conversion / total_weight) * dbp->dbi_base2local;
-    sum_y  *= (conversion / total_weight) * dbp->dbi_base2local;
-    sum_z  *= (conversion / total_weight) * dbp->dbi_base2local;
+    sum_x *= (conversion / total_weight) * dbp->dbi_base2local;
+    sum_y *= (conversion / total_weight) * dbp->dbi_base2local;
+    sum_z *= (conversion / total_weight) * dbp->dbi_base2local;
 
     fprintf(outfp, "RT Weight Program Output:\n");
     fprintf(outfp, "\nDatabase Title: \"%s\"\n", dbp->dbi_title);
@@ -566,23 +619,39 @@ view_end(struct application *ap)
     fprintf(outfp, "\nTotal mass = %g %s\n\n", total_weight, units);
 
     /* now finished with density file name*/
-    bu_free(densityfile, "density file name");
+    bu_vls_free(&densityfile_vls);
 
+    analyze_densities_destroy(density);
 }
 
-void view_setup(struct rt_i *UNUSED(rtip))
+
+void
+view_setup(struct rt_i *UNUSED(rtip))
 {
 }
 
-/* Associated with "clean" command, before new tree is loaded  */
-void view_cleanup(struct rt_i *UNUSED(rtip))
+
+/* Associated with "clean" command, before new tree is loaded */
+void
+view_cleanup(struct rt_i *UNUSED(rtip))
 {
 }
 
-void application_init (void)
+
+void
+application_init(void)
 {
+    option("", "-o file.out", "Weights and Moments output file", 0);
+    option("", "-d density.txt", "Density definitions input file", 0);
+    option("", "-r", "Report verbosely mass of each region", 0);
+    /* this reassignment hack ensures help is last in the first list */
+    option("dummy", "-? or -h", "Display help", 1);
+    option("", "-? or -h", "Display help", 1);
+    option(NULL, "-C", "Disabled, not implemented", -2);
+    option(NULL, "-W", "Disabled, non implemented", -2);
 }
 
+
 /*
  * Local Variables:
  * mode: C
