@@ -1,7 +1,7 @@
 /*                           U - A . C
  * BRL-CAD
  *
- * Copyright (c) 2004-2013 United States Government as represented by
+ * Copyright (c) 2004-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This program is free software; you can redistribute it and/or
@@ -22,17 +22,25 @@
  */
 
 #include "common.h"
+#include "bio.h"
 
-#include <stdio.h>
+#include "bu/app.h"
+#include "bu/snooze.h"
 
 
-#define MAXBUF 16*1024
-unsigned short ibuf[MAXBUF];
-
-int main(void)
+int
+main(int UNUSED(argc), const char **argv)
 {
+#define MAXBUF 16*1024
+    unsigned short ibuf[MAXBUF];
+
     int n, i;
 
+    bu_setprogname(argv[0]);
+
+    fprintf(stderr,"DEPRECATION WARNING:  This command is scheduled for removal.  Please contact the developers if you use this command.\n\n");
+    bu_snooze(BU_SEC2USEC(1));
+
     while ((n=fread(ibuf, sizeof(*ibuf), MAXBUF, stdin)) > 0) {
 	for (i=0; i < n; ++i)
 	    printf("%hd\n", ibuf[i]);
