@@ -1,7 +1,7 @@
 #                     D I S P L A Y . T C L
 # BRL-CAD
 #
-# Copyright (c) 1998-2013 United States Government as represented by
+# Copyright (c) 1998-2021 United States Government as represented by
 # the U.S. Army Research Laboratory.
 #
 # This library is free software; you can redistribute it and/or
@@ -168,7 +168,7 @@
     public method transparency {args}
     public method zbuffer {args}
     public method zclip {args}
-    if {$tcl_platform(os) != "Windows NT"} {
+    if {$::tcl_platform(os) != "Windows NT"} {
     }
     public method fb_active {args}
 
@@ -750,7 +750,7 @@
 	}
 
 	if {$itk_option(-scaleEnable)} {
-	    Dm::drawScale [View::size] $itk_option(-scaleColor)
+	    Dm::drawScale [View::size] [View::units] $itk_option(-scaleColor)
 	}
 
 	if {$itk_option(-primitiveLabels) != {}} {
@@ -973,10 +973,6 @@
     }
 
     set v_obj [View::get_viewname]
-    #    if {$tcl_platform(os) != "Windows NT"} {
-    #	eval $geo rtedge $v_obj -F $itk_option(-listen) -w $width -n $height -V $aspect $args
-    #    } else {
-    #    }
 
     # If arguments remain, they should specify size, framebuffer etc.
     if {[llength $args] > 0} {
@@ -1140,7 +1136,7 @@
     return $itk_option(-perspective)
 }
 
-if {$tcl_platform(os) != "Windows NT"} {
+if {$::tcl_platform(os) != "Windows NT"} {
 }
 ::itcl::body Display::fb_active {args} {
     if {$args == ""} {
@@ -1675,7 +1671,7 @@ if {$tcl_platform(os) != "Windows NT"} {
 ::itcl::body Display::doBindings {} {
     global tcl_platform
 
-    if {$tcl_platform(os) != "Windows NT"} {
+    if {$::tcl_platform(os) != "Windows NT"} {
 	bind $itk_component(dm) <Enter> "focus $itk_component(dm);"
     }
 
