@@ -1,7 +1,7 @@
 /*                      P L O T 3 - D M . C
  * BRL-CAD
  *
- * Copyright (c) 1999-2013 United States Government as represented by
+ * Copyright (c) 1999-2021 United States Government as represented by
  * the U.S. Army Research Laboratory.
  *
  * This program is free software; you can redistribute it and/or
@@ -37,13 +37,13 @@
 #include "tcl.h"
 #include "tk.h"
 
+#include "bu/app.h"
+#include "bu/getopt.h"
 #include "vmath.h"
-#include "db.h"
+#include "rt/db4.h"
 #include "raytrace.h"
-#include "bu.h"
 #include "bn.h"
 #include "dm.h"
-#include "dm-X.h"
 
 
 struct cmdtab {
@@ -82,23 +82,23 @@ struct plot_list{
 
 struct plot_list HeadPlot;
 
-int dm_type = DM_TYPE_X;
+static const char *Xtype = "X";
+static const char *Otype = "ogl";
+const char *dm_type = "X";
 
 
 /*
- * O U T P U T _ C A T C H
- *
  * Gets the output from bu_log and appends it to clientdata vls.
  */
 static int
-output_catch(genptr_t clientdata, genptr_t str)
+output_catch(void *clientdata, void *str)
 {
     struct bu_vls *vp = (struct bu_vls *)clientdata;
     int len;
 
     BU_CK_VLS(vp);
     len = bu_vls_strlen(vp);
-    bu_vls_strcat(vp, str);
+    bu_vls_strcat(vp, (const char *)str);
     len = bu_vls_strlen(vp) - len;
 
     return len;
@@ -106,27 +106,23 @@ output_catch(genptr_t clientdata, genptr_t str)
 
 
 /*
- * S T A R T _ C A T C H I N G _ O U T P U T
- *
  * Sets up hooks to bu_log so that all output is caught in the given vls.
  *
  */
 void
 start_catching_output(struct bu_vls *vp)
 {
-    bu_log_add_hook(output_catch, (genptr_t)vp);
+    bu_log_add_hook(output_catch, (void *)vp);
 }
 
 
 /*
- * S T O P _ C A T C H I N G _ O U T P U T
- *
  * Turns off the output catch hook.
  */
 void
 stop_catching_output(struct bu_vls *vp)
 {
-    bu_log_delete_hook(output_catch, (genptr_t)vp);
+    bu_log_delete_hook(output_catch, (void *)vp);
 }
 
 
@@ -135,22 +131,22 @@ get_args(int argc, char **argv)
 {
     int c;
 
-    while ((c = bu_getopt(argc, argv, "t:")) != -1) {
+    while ((c = bu_getopt(argc, argv, "t:h?")) != -1) {
 	switch (c) {
 	    case 't':
 		switch (*bu_optarg) {
 		    case 'o':
 		    case 'O':
-			dm_type = DM_TYPE_OGL;
+			dm_type = Otype;
 			break;
 		    case 'x':
 		    case 'X':
 		    default:
-			dm_type = DM_TYPE_X;
+			dm_type = Xtype;
 			break;
 		}
 		break;
-	    default:		/* '?' */
+	    default:		/* 'h' '?' */
 		return 0;
 	}
     }
@@ -173,8 +169,8 @@ refresh() {
     size_t i;
     struct plot_list *plp;
 
-    DM_DRAW_BEGIN(dmp);
-    DM_LOADMATRIX(dmp, model2view, 0);
+    dm_draw_begin(dmp);
+    dm_loadmatrix(dmp, model2view, 0);
 
     for (BU_LIST_FOR(plp, plot_list, &HeadPlot.l)) {
 	if (plp->pl_draw)
@@ -183,20 +179,18 @@ refresh() {
 		    long rgb;
 
 		    rgb = plp->pl_vbp->rgb[i];
-		    DM_SET_FGCOLOR(dmp, (rgb>>16) & 0xFF, (rgb>>8) & 0xFF, rgb & 0xFF, 0, (fastf_t)0.0);
+		    dm_set_fg(dmp, (rgb>>16) & 0xFF, (rgb>>8) & 0xFF, rgb & 0xFF, 0, (fastf_t)0.0);
 		}
-		DM_DRAW_VLIST(dmp, (struct bn_vlist *)&plp->pl_vbp->head[i]);
+		dm_draw_vlist(dmp, (struct bn_vlist *)&plp->pl_vbp->head[i]);
 	    }
     }
 
-    DM_NORMAL(dmp);
-    DM_DRAW_END(dmp);
+    dm_normal(dmp);
+    dm_draw_end(dmp);
 }
 
 
 /*
- * B U I L D H R O T
- *
  * This routine builds a Homogeneous rotation matrix, given
  * alpha, beta, and gamma as angles of rotation.
  *
@@ -256,8 +250,6 @@ buildHrot(matp_t mat, double alpha, double beta, double ggamma)
 
 
 /*
- * S E T V I E W
- *
  * Set the view.  Angles are DOUBLES, in degrees.
  *
  * Given that viewvec = scale . rotate . (xlate to view center) . modelvec,
@@ -333,8 +325,6 @@ vrot(double x, double y, double z)
 
 
 /*
- * N E W _ M A T S
- *
  * Derive the inverse and editing matrices, as required.
  * Centralized here to simplify things.
  */
@@ -376,12 +366,13 @@ X_doEvent(ClientData UNUSED(clientData), XEvent *eventPtr)
 		break;
 	    case MOUSE_MODE_TRANSLATE:
 		{
+		    int width, height;
 		    vect_t vdiff;
 
-		    vdiff[X] = (mx - omx) /
-			(fastf_t)dmp->dm_width * 2.0;
-		    vdiff[Y] = (omy - my) /
-			(fastf_t)dmp->dm_height * 2.0;
+		    width = dm_get_width(dmp);
+		    height = dm_get_height(dmp);
+		    vdiff[X] = (mx - omx) / width * 2.0;
+		    vdiff[Y] = (omy - my) / height * 2.0;
 		    vdiff[Z] = 0.0;
 
 		    (void)islewview(vdiff);
@@ -393,9 +384,9 @@ X_doEvent(ClientData UNUSED(clientData), XEvent *eventPtr)
 	    case MOUSE_MODE_ZOOM:
 		{
 		    double val;
+		    int height = dm_get_height(dmp);
 
-		    val = 1.0 + (omy - my) /
-			(fastf_t)dmp->dm_height;
+		    val = 1.0 + (omy - my) / height;
 
 		    zoom(INTERP, val);
 		    new_mats();
@@ -491,8 +482,6 @@ X_dm(int argc, char *argv[])
 
 
 /*
- * S I Z E _ R E S E T
- *
  * Reset view size and view center so that everything in the vlist
  * is in view.
  * Caller is responsible for calling new_mats().
@@ -600,7 +589,7 @@ cmd_openpl(ClientData UNUSED(clientData), Tcl_Interp *interp, int argc, char **a
 	char *bnp;
 
 	file = argv[i];
-	if ((fp = fopen(file, "r")) == NULL) {
+	if ((fp = fopen(file, "rb")) == NULL) {
 	    bu_log("%s: can't open \"%s\"\n", argv[0], file);
 	    continue;
 	}
@@ -618,7 +607,7 @@ cmd_openpl(ClientData UNUSED(clientData), Tcl_Interp *interp, int argc, char **a
 	for (BU_LIST_FOR(plp, plot_list, &HeadPlot.l)) {
 	    /* found object with same name */
 	    if (BU_STR_EQUAL(bu_vls_addr(&plp->pl_name), bnp)) {
-		rt_vlblock_free(plp->pl_vbp);
+		bn_vlblock_free(plp->pl_vbp);
 		goto up_to_vl;
 	    }
 	}
@@ -745,8 +734,8 @@ cmd_closepl(ClientData UNUSED(clientData), Tcl_Interp *interp, int argc, char **
 	    if (BU_STR_EQUAL(argv[i], bu_vls_addr(&plp->pl_name))) {
 		BU_LIST_DEQUEUE(&plp->l);
 		bu_vls_free(&plp->pl_name);
-		rt_vlblock_free(plp->pl_vbp);
-		bu_free((genptr_t)plp, "cmd_closepl");
+		bn_vlblock_free(plp->pl_vbp);
+		bu_free((void *)plp, "cmd_closepl");
 		break;
 	    }
 	}
@@ -850,9 +839,9 @@ cmd_list(ClientData UNUSED(clientData), Tcl_Interp *interp, int argc, char **UNU
 
 	linelen += len;
 	if (linelen > 80)
-	    bu_log("\n%-*V", len, &plp->pl_name);
+	    bu_log("\n%-*s", len, bu_vls_addr(&plp->pl_name));
 	else
-	    bu_log("%-*V", len, &plp->pl_name);
+	    bu_log("%-*s", len, bu_vls_addr(&plp->pl_name));
     }
 
     bu_log("\n");
@@ -861,8 +850,6 @@ cmd_list(ClientData UNUSED(clientData), Tcl_Interp *interp, int argc, char **UNU
 
 
 /*
- * F _ Z O O M
- *
  * A scale factor of 2 will increase the view size by a factor of 2,
  * (i.e., a zoom out) which is accomplished by reducing Viewscale in half.
  */
@@ -902,8 +889,6 @@ cmd_reset(ClientData UNUSED(clientData), Tcl_Interp *UNUSED(interp), int UNUSED(
 
 
 /*
- * S L E W V I E W
- *
  * Given a position in view space,
  * make that point the new view center.
  */
@@ -1001,7 +986,7 @@ static int
 cmd_exit(ClientData UNUSED(clientData), Tcl_Interp *UNUSED(interp), int UNUSED(argc), char **UNUSED(argv))
 {
     if (dmp != DM_NULL)
-	DM_CLOSE(dmp);
+	dm_close(dmp);
 
     bu_exit (0, NULL);
 
@@ -1019,7 +1004,7 @@ cmd_setup(Tcl_Interp *interp, struct cmdtab commands[])
     struct cmdtab *ctp;
 
     for (ctp = commands; ctp->ct_name; ctp++) {
-	(void)Tcl_CreateCommand(interp, ctp->ct_name, ctp->ct_func,
+	(void)Tcl_CreateCommand(interp, ctp->ct_name, (Tcl_CmdProc *)ctp->ct_func,
 				(ClientData)ctp, (Tcl_CmdDeleteProc *)NULL);
     }
 }
@@ -1064,13 +1049,13 @@ X_dmInit()
     av[2] = "sampler_bind_dm";
     av[3] = (char *)NULL;
 
-    if ((dmp = DM_OPEN(INTERP, DM_TYPE_X, 3, av)) == DM_NULL) {
+    if ((dmp = dm_open(INTERP, "X", 3, av)) == DM_NULL) {
 	Tcl_AppendResult(INTERP, "Failed to open a display manager\n", (char *)NULL);
 	return TCL_ERROR;
     }
 
-    Tk_CreateGenericHandler(X_doEvent, (ClientData)DM_TYPE_X);
-    DM_SET_WIN_BOUNDS(dmp, windowbounds);
+    Tk_CreateGenericHandler(X_doEvent, (ClientData)Xtype);
+    dm_set_win_bounds(dmp, windowbounds);
 
     return TCL_OK;
 }
@@ -1090,13 +1075,13 @@ Ogl_dmInit()
     av[2] = "sampler_bind_dm";
     av[3] = (char *)NULL;
 
-    if ((dmp = DM_OPEN(INTERP, DM_TYPE_OGL, 3, (const char **)av)) == DM_NULL) {
+    if ((dmp = dm_open(INTERP, "ogl", 3, (const char **)av)) == DM_NULL) {
 	Tcl_AppendResult(INTERP, "Failed to open a display manager\n", (char *)NULL);
 	return TCL_ERROR;
     }
 
-    Tk_CreateGenericHandler(X_doEvent, (ClientData)DM_TYPE_OGL);
-    DM_SET_WIN_BOUNDS(dmp, windowbounds);
+    Tk_CreateGenericHandler(X_doEvent, (ClientData)Otype);
+    dm_set_win_bounds(dmp, windowbounds);
 
     return TCL_OK;
 }
@@ -1113,13 +1098,7 @@ appInit(Tcl_Interp *_interp)
     /* libdm uses interp */
     INTERP = _interp;
 
-    switch (dm_type) {
-	case DM_TYPE_OGL:
-	case DM_TYPE_X:
-	default:
-	    cmd_hook = X_dm;
-	    break;
-    }
+    cmd_hook = X_dm;
 
     /* Evaluates init.tcl */
     if (Tcl_Init(_interp) == TCL_ERROR)
@@ -1137,7 +1116,7 @@ appInit(Tcl_Interp *_interp)
 	bu_exit (1, "appInit: Failed to get main window.\n");
 
     /* Locate the BRL-CAD-specific Tcl scripts */
-    filename = bu_brlcad_data("tclscripts", 0);
+    filename = bu_dir(NULL, 0, BU_DIR_DATA, "tclscripts", NULL);
 
     bu_vls_printf(&str2, "%s/plot3-dm", filename);
     bu_vls_printf(&str, "wm withdraw .; set auto_path [linsert $auto_path 0 %s %s]",
@@ -1150,13 +1129,10 @@ appInit(Tcl_Interp *_interp)
     cmd_setup(_interp, cmdtab);
 
     /* open display manager */
-    switch (dm_type) {
-	case DM_TYPE_OGL:
-	    return Ogl_dmInit();
-	case DM_TYPE_X:
-	default:
-	    return X_dmInit();
+    if (BU_STR_EQUIV(dm_type, "ogl")) {
+	return Ogl_dmInit();
     }
+    return X_dmInit();
 }
 
 
@@ -1165,6 +1141,9 @@ main(int argc, char *argv[])
 {
     const char usage[] = "Usage: plot3-dm [-t o|X] plot_file(s)\n";
 
+    /* initialize prognam for run-time reousrce finding */
+    bu_setprogname(argv[0]);
+
     if (!get_args(argc, argv))
 	bu_exit (1, "%s", usage);
 
